@@ -44,6 +44,19 @@ static bool vlistIsInitialized = false;
   if (!vlistIsInitialized) vlist_initialize()
 #endif
 
+void *
+stream_get_pointer(int streamID)
+{
+  return stream_to_pointer(streamID);
+}
+
+void *
+stream_get_vlist_pointer(int streamID)
+{
+  stream_t *streamPtr = stream_to_pointer(streamID);
+  return vlist_to_pointer(streamPtr->vlistID);
+}
+
 static int
 vlist_compare(vlist_t *a, vlist_t *b)
 {
@@ -67,16 +80,12 @@ static int vlistGetSizeP(void *vlistptr, void *context);
 static void vlistPackP(void *vlistptr, void *buff, int size, int *position, void *context);
 static int vlistTxCode(void *vlistptr);
 
-#if !defined(__cplusplus)
-const
-#endif
-    resOps vlistOps
-    = { (valCompareFunc) vlist_compare,
-        (valDestroyFunc) vlist_delete,
-        (valPrintFunc) vlistPrintKernel,
-        vlistGetSizeP,
-        vlistPackP,
-        vlistTxCode };
+const resOps vlistOps = { (valCompareFunc) vlist_compare,
+                          (valDestroyFunc) vlist_delete,
+                          (valPrintFunc) vlistPrintKernel,
+                          vlistGetSizeP,
+                          vlistPackP,
+                          vlistTxCode };
 
 vlist_t *
 vlist_to_pointer(int vlistID)
@@ -299,7 +308,7 @@ var_copy_entries(var_t *var2, var_t *var1)
       if (var1->opt_grib_kvpair[i].keyword != NULL)
         {
           var2->opt_grib_kvpair[i] = var1->opt_grib_kvpair[i];
-          var2->opt_grib_kvpair[i].keyword = strdupx(var1->opt_grib_kvpair[i].keyword);
+          var2->opt_grib_kvpair[i].keyword = strdup(var1->opt_grib_kvpair[i].keyword);
           var2->opt_grib_kvpair[i].update = true;
           if (CDI_Debug) Message("done.");
         }
@@ -316,8 +325,8 @@ var_copy_entries(var_t *var2, var_t *var1)
 
 @Prototype void vlistCopy(int vlistID2, int vlistID1)
 @Parameter
-    @Item  vlistID2  Target variable list ID.
     @Item  vlistID1  Source variable list ID.
+    @Item  vlistID2  Target variable list ID.
 
 @Description
 The function @func{vlistCopy} copies all entries from vlistID1 to vlistID2.
@@ -325,7 +334,7 @@ The function @func{vlistCopy} copies all entries from vlistID1 to vlistID2.
 @EndFunction
 */
 void
-vlistCopy(int vlistID2, int vlistID1)
+vlistCopy(int vlistID1, int vlistID2)
 {
   vlist_t *vlistptr1 = vlist_to_pointer(vlistID1);
   vlist_t *vlistptr2 = vlist_to_pointer(vlistID2);
@@ -391,7 +400,7 @@ vlistDuplicate(int vlistID)
   if (CDI_Debug) Message("call to vlistDuplicate");
 
   int vlistIDnew = vlistCreate();
-  vlistCopy(vlistIDnew, vlistID);
+  vlistCopy(vlistID, vlistIDnew);
   return vlistIDnew;
 }
 
@@ -432,8 +441,8 @@ vgzZAxisSearch(int id, void *res, void *data)
       state->resIDValue = id;
       return CDI_APPLY_STOP;
     }
-  else
-    return CDI_APPLY_GO_ON;
+
+  return CDI_APPLY_GO_ON;
 }
 
 static int
@@ -446,7 +455,7 @@ vlist_generate_zaxis(int vlistID, int zaxistype, int nlevels, const double *leve
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
   int nzaxis = vlistptr->nzaxis;
 
-  const bool hasBounds = (lbounds && ubounds);
+  bool hasBounds = (lbounds && ubounds);
 
   for (int index = 0; index < nzaxis; ++index)
     {
@@ -465,6 +474,8 @@ vlist_generate_zaxis(int vlistID, int zaxistype, int nlevels, const double *leve
       query.zaxistype = zaxistype;
       query.nlevels = nlevels;
       query.levels = levels;
+      query.lbounds = lbounds;
+      query.ubounds = ubounds;
 
       if ((zaxisglobdefined = (cdiResHFilterApply(getZaxisOps(), vgzZAxisSearch, &query) == CDI_APPLY_STOP)))
         zaxisID = query.resIDValue;
@@ -496,6 +507,22 @@ vlist_generate_zaxis(int vlistID, int zaxistype, int nlevels, const double *leve
   return zaxisID;
 }
 
+static void
+delete_chunks(int vlistID, int varID)
+{
+  int chunkKeys[4] = { CDI_KEY_CHUNKSIZE_DIMX, CDI_KEY_CHUNKSIZE_DIMY, CDI_KEY_CHUNKSIZE_DIMZ, CDI_KEY_CHUNKSIZE_DIMT };
+  for (int i = 0; i < 4; ++i)
+    {
+      int chunkSize = 0;
+      cdiInqKeyInt(vlistID, varID, chunkKeys[i], &chunkSize);
+      if (chunkSize != 0) cdiDeleteKey(vlistID, varID, chunkKeys[i]);
+    }
+
+  int chunkSize = 0;
+  cdiInqKeyInt(vlistID, varID, CDI_KEY_CHUNKSIZE, &chunkSize);
+  if (chunkSize > 0) cdiDeleteKey(vlistID, varID, CDI_KEY_CHUNKSIZE);
+}
+
 /*
 @Function  vlistCopyFlag
 @Title     Copy some entries of a variable list
@@ -697,6 +724,7 @@ vlistCopyFlag(int vlistID2, int vlistID1)
 
                 zaxisID = zaxisID2;
                 vars2[varID2].zaxisID = zaxisID2;
+                delete_chunks(vlistID2, varID2);
               }
 
             for (int levID = 0; levID < nlevs2; levID++)
@@ -824,8 +852,8 @@ vlistMerge(int vlistID2, int vlistID1)
       char name1[CDI_MAX_NAME], name2[CDI_MAX_NAME];
       for (varID = 0; varID < nvars2; varID++)
         {
-          size_t ngp1 = gridInqSize(vars1[varID].gridID);
-          size_t ngp2 = gridInqSize(vars2[varID].gridID);
+          SizeType ngp1 = gridInqSize(vars1[varID].gridID);
+          SizeType ngp2 = gridInqSize(vars2[varID].gridID);
           if (ngp1 != ngp2) break;
 
           int length = CDI_MAX_NAME;
@@ -950,7 +978,7 @@ vlistNvars(int vlistID)
 }
 
 int
-vlistNrecs(int vlistID)
+vlistNumFields(int vlistID)
 {
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
 
@@ -963,23 +991,15 @@ vlistNrecs(int vlistID)
 int
 vlistNumber(int vlistID)
 {
-  int number, number2;
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
 
   int datatype = vlistptr->vars[0].datatype;
-  if (datatype == CDI_DATATYPE_CPX32 || datatype == CDI_DATATYPE_CPX64)
-    number = CDI_COMP;
-  else
-    number = CDI_REAL;
+  int number = (datatype == CDI_DATATYPE_CPX32 || datatype == CDI_DATATYPE_CPX64) ? CDI_COMP : CDI_REAL;
 
   for (int varID = 1; varID < vlistptr->nvars; varID++)
     {
       datatype = vlistptr->vars[varID].datatype;
-      if (datatype == CDI_DATATYPE_CPX32 || datatype == CDI_DATATYPE_CPX64)
-        number2 = CDI_COMP;
-      else
-        number2 = CDI_REAL;
-
+      int number2 = (datatype == CDI_DATATYPE_CPX32 || datatype == CDI_DATATYPE_CPX64) ? CDI_COMP : CDI_REAL;
       if (number2 != number)
         {
           number = CDI_BOTH;
@@ -991,58 +1011,67 @@ vlistNumber(int vlistID)
 }
 
 /*
-@Function  vlistNgrids
+@Function  vlistNumGrids
 @Title     Number of grids in a variable list
 
-@Prototype int vlistNgrids(int vlistID)
+@Prototype int vlistNumGrids(int vlistID)
 @Parameter
     @Item  vlistID  Variable list ID, from a previous call to @fref{vlistCreate} or @fref{streamInqVlist}.
 
 @Description
-The function @func{vlistNgrids} returns the number of grids in the variable list vlistID.
+The function @func{vlistNumGrids} returns the number of grids in the variable list vlistID.
 
 @Result
-@func{vlistNgrids} returns the number of grids in a variable list.
+@func{vlistNumGrids} returns the number of grids in a variable list.
 
 @EndFunction
 */
 int
-vlistNgrids(int vlistID)
+vlistNumGrids(int vlistID)
 {
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
-
   return vlistptr->ngrids;
 }
 
+int
+vlistNgrids(int vlistID)
+{
+  return vlistNumGrids(vlistID);
+}
+
 /*
-@Function  vlistNzaxis
+@Function  vlistNumZaxis
 @Title     Number of zaxis in a variable list
 
-@Prototype int vlistNzaxis(int vlistID)
+@Prototype int vlistNumZaxis(int vlistID)
 @Parameter
     @Item  vlistID  Variable list ID, from a previous call to @fref{vlistCreate} or @fref{streamInqVlist}.
 
 @Description
-The function @func{vlistNzaxis} returns the number of zaxis in the variable list vlistID.
+The function @func{vlistNumZaxis} returns the number of zaxis in the variable list vlistID.
 
 @Result
-@func{vlistNzaxis} returns the number of zaxis in a variable list.
+@func{vlistNumZaxis} returns the number of zaxis in a variable list.
 
 @EndFunction
 */
 int
-vlistNzaxis(int vlistID)
+vlistNumZaxis(int vlistID)
 {
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
-
   return vlistptr->nzaxis;
 }
 
+int
+vlistNzaxis(int vlistID)
+{
+  return vlistNumZaxis(vlistID);
+}
+
 int
 vlistNsubtypes(int vlistID)
 {
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
-
   return vlistptr->nsubtypes;
 }
 
@@ -1063,17 +1092,16 @@ int
 vlistNtsteps(int vlistID)
 {
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
-
   return (int) vlistptr->ntsteps;
 }
 
 static void
 vlistPrintKernel(vlist_t *vlistptr, FILE *fp)
 {
-  const int vlistID = vlistptr->self;
+  int vlistID = vlistptr->self;
   fprintf(fp, "#\n# vlistID %d\n#\n", vlistID);
 
-  const int nvars = vlistptr->nvars;
+  int nvars = vlistptr->nvars;
 
   fprintf(fp,
           "nvars    : %d\n"
@@ -1146,7 +1174,7 @@ vlistPrintKernel(vlist_t *vlistptr, FILE *fp)
             fp);
       for (int varID = 0; varID < nvars; varID++)
         fprintf(fp, "%3d %8zu\n", varID,
-                (size_t) zaxisInqSize(vlistptr->vars[varID].zaxisID) * gridInqSize(vlistptr->vars[varID].gridID));
+                (size_t) zaxisInqSize(vlistptr->vars[varID].zaxisID) * (size_t) gridInqSize(vlistptr->vars[varID].gridID));
     }
 }
 
@@ -1351,7 +1379,13 @@ vlistChangeGridIndex(int vlistID, int index, int gridID)
 
       int nvars = vlistptr->nvars;
       for (int varID = 0; varID < nvars; varID++)
-        if (vlistptr->vars[varID].gridID == gridIDold) vlistptr->vars[varID].gridID = gridID;
+        if (vlistptr->vars[varID].gridID == gridIDold)
+          {
+            vlistptr->vars[varID].gridID = gridID;
+            delete_chunks(vlistID, varID);
+            if (gridInqXsize(gridIDold) == 0 && gridInqXsize(gridID) > 0 && vlistInqVarXYZ(vlistID, varID) != 0)
+              vlistDefVarXYZ(vlistID, varID, 0);
+          }
       reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
@@ -1374,7 +1408,11 @@ vlistChangeGrid(int vlistID, int gridID1, int gridID2)
         }
       int nvars = vlistptr->nvars;
       for (int varID = 0; varID < nvars; varID++)
-        if (vlistptr->vars[varID].gridID == gridID1) vlistptr->vars[varID].gridID = gridID2;
+        if (vlistptr->vars[varID].gridID == gridID1)
+          {
+            vlistptr->vars[varID].gridID = gridID2;
+            delete_chunks(vlistID, varID);
+          }
       reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
@@ -1420,6 +1458,7 @@ vlistChangeZaxisIndex(int vlistID, int index, int zaxisID)
         if (vlistptr->vars[varID].zaxisID == zaxisIDold)
           {
             vlistptr->vars[varID].zaxisID = zaxisID;
+            delete_chunks(vlistID, varID);
             if (vlistptr->vars[varID].levinfo && nlevs != nlevsOld)
               {
                 vlistptr->vars[varID].levinfo
@@ -1453,6 +1492,7 @@ vlistChangeZaxis(int vlistID, int zaxisID1, int zaxisID2)
     if (vlistptr->vars[varID].zaxisID == zaxisID1)
       {
         vlistptr->vars[varID].zaxisID = zaxisID2;
+        delete_chunks(vlistID, varID);
 
         if (vlistptr->vars[varID].levinfo && nlevs2 != nlevs1)
           {
@@ -1586,7 +1626,7 @@ vlistUnpack(char *buf, int size, int *position, int originNamespace, void *conte
 void
 vlist_check_contents(int vlistID)
 {
-  int nzaxis = vlistNzaxis(vlistID);
+  int nzaxis = vlistNumZaxis(vlistID);
   for (int index = 0; index < nzaxis; index++)
     {
       int zaxisID = vlistZaxis(vlistID, index);
@@ -1598,31 +1638,26 @@ vlist_check_contents(int vlistID)
 void
 resize_opt_grib_entries(var_t *var, int nentries)
 {
-  if (var->opt_grib_kvpair_size >= nentries)
-    {
-      return; /* nothing to do; array is still large enough */
-    }
-  else
-    {
-      if (CDI_Debug) Message("resize data structure, %d -> %d", var->opt_grib_kvpair_size, nentries);
+  if (var->opt_grib_kvpair_size >= nentries) return;  // nothing to do; array is still large enough
 
-      int new_size = (2 * var->opt_grib_kvpair_size) > nentries ? (2 * var->opt_grib_kvpair_size) : nentries;
-      opt_key_val_pair_t *tmp = (opt_key_val_pair_t *) Malloc((size_t) new_size * sizeof(opt_key_val_pair_t));
-      for (int i = 0; i < var->opt_grib_kvpair_size; ++i)
-        {
-          tmp[i] = var->opt_grib_kvpair[i];
-        }
-      for (int i = var->opt_grib_kvpair_size; i < new_size; ++i)
-        {
-          tmp[i].int_val = 0;
-          tmp[i].dbl_val = 0;
-          tmp[i].update = false;
-          tmp[i].keyword = NULL;
-        }  // for
-      var->opt_grib_kvpair_size = new_size;
-      Free(var->opt_grib_kvpair);
-      var->opt_grib_kvpair = tmp;
+  if (CDI_Debug) Message("resize data structure, %d -> %d", var->opt_grib_kvpair_size, nentries);
+
+  int new_size = (2 * var->opt_grib_kvpair_size) > nentries ? (2 * var->opt_grib_kvpair_size) : nentries;
+  opt_key_val_pair_t *tmp = (opt_key_val_pair_t *) Malloc((size_t) new_size * sizeof(opt_key_val_pair_t));
+  for (int i = 0; i < var->opt_grib_kvpair_size; ++i)
+    {
+      tmp[i] = var->opt_grib_kvpair[i];
     }
+  for (int i = var->opt_grib_kvpair_size; i < new_size; ++i)
+    {
+      tmp[i].int_val = 0;
+      tmp[i].dbl_val = 0;
+      tmp[i].update = false;
+      tmp[i].keyword = NULL;
+    }  // for
+  var->opt_grib_kvpair_size = new_size;
+  Free(var->opt_grib_kvpair);
+  var->opt_grib_kvpair = tmp;
 }
 
 /*
