@@ -76,7 +76,7 @@ adipot_kernel(double t, double s, double p)
 }
 
 static void
-calc_adisit(size_t gridsize, size_t nlevel, const Varray<double> &pressure, const FieldVector &tho, const FieldVector &sao,
+calc_adisit(size_t gridsize, size_t nlevel, Varray<double> const &pressure, FieldVector const &tho, FieldVector const &sao,
             FieldVector &tis)
 {
   // pressure units: hPa
@@ -85,8 +85,8 @@ calc_adisit(size_t gridsize, size_t nlevel, const Varray<double> &pressure, cons
 
   for (size_t levelID = 0; levelID < nlevel; ++levelID)
     {
-      const auto &thovec = tho[levelID].vec_d;
-      const auto &saovec = sao[levelID].vec_d;
+      auto const &thovec = tho[levelID].vec_d;
+      auto const &saovec = sao[levelID].vec_d;
       auto &tisvec = tis[levelID].vec_d;
       auto thoMissval = tho[levelID].missval;
       auto saoMissval = sao[levelID].missval;
@@ -100,7 +100,7 @@ calc_adisit(size_t gridsize, size_t nlevel, const Varray<double> &pressure, cons
 }
 
 static void
-calc_adipot(size_t gridsize, size_t nlevel, const Varray<double> &pressure, const FieldVector &t, const FieldVector &s,
+calc_adipot(size_t gridsize, size_t nlevel, Varray<double> const &pressure, FieldVector const &t, FieldVector const &s,
             FieldVector &tpot)
 {
   // pressure units: hPa
@@ -109,8 +109,8 @@ calc_adipot(size_t gridsize, size_t nlevel, const Varray<double> &pressure, cons
 
   for (size_t levelID = 0; levelID < nlevel; ++levelID)
     {
-      const auto &tvec = t[levelID].vec_d;
-      const auto &svec = s[levelID].vec_d;
+      auto const &tvec = t[levelID].vec_d;
+      auto const &svec = s[levelID].vec_d;
       auto &tpotvec = tpot[levelID].vec_d;
       auto tMissval = t[levelID].missval;
       auto sMissval = s[levelID].missval;
@@ -124,13 +124,13 @@ calc_adipot(size_t gridsize, size_t nlevel, const Varray<double> &pressure, cons
 }
 
 int
-get_code(int vlistID1, int varID, const std::string &cname)
+get_code(const CdoVar &var, std::string const &cname)
 {
-  auto code = vlistInqVarCode(vlistID1, varID);
+  auto code = var.code;
   if (code <= 0)
     {
-      auto varname = string_to_lower(cdo::inq_var_name(vlistID1, varID));
-      auto stdname = string_to_lower(cdo::inq_key_string(vlistID1, varID, CDI_KEY_STDNAME));
+      auto varname = string_to_lower(var.name);
+      auto stdname = string_to_lower(var.stdname);
 
       if (varname == "s" || varname == "so" || stdname == "sea_water_salinity") { code = 5; }
       else if (varname == "t" || varname == "to") { code = 2; }
@@ -144,23 +144,22 @@ get_code(int vlistID1, int varID, const std::string &cname)
 struct IOSettings
 {
   CdoStreamID streamID2;
-  int vlistID2;
+  int vlistID2{ CDI_UNDEFID };
   size_t gridsize;
   int nlevels;
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
   int tisID2;
   int saoID2;
 };
 
 IOSettings
-configureOutput(const std::function<void(int, int)> &outputSettingFunc, int vlistID, int thoID, int saoID, FieldVector &tho,
-                FieldVector &sao, FieldVector &tis, Varray<double> &pressure)
+configureOutput(const std::function<void(int, int)> &outputSettingFunc, const VarList &varList, int vlistID, int thoID, int saoID,
+                FieldVector &tho, FieldVector &sao, FieldVector &tis, Varray<double> &pressure)
 {
   double pin = (cdo_operator_argc() == 1) ? parameter_to_double(cdo_operator_argv(0)) : -1.0;
 
-  VarList varList(vlistID);
-  const auto &vars = varList.vars;
+  auto const &vars = varList.vars;
 
   auto units = vars[thoID].units;
   if (units.empty()) units = "Celcius";
@@ -249,14 +248,16 @@ public:
   inline static RegisterEntry<Adisit> registration = RegisterEntry<Adisit>(module);
 
   int ADISIT, ADIPOT;
-  int thoID = -1, saoID = -1;
+  int thoID = -1;
+  int saoID = -1;
 
   CdoStreamID streamID1;
-  int taxisID1;
-
   CdoStreamID streamID2;
-  int taxisID2;
-  int vlistID2;
+
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
+
+  int vlistID2{ CDI_UNDEFID };
 
   int operatorID;
 
@@ -280,20 +281,19 @@ public:
 
     streamID1 = cdo_open_read(0);
     auto vlistID1 = cdo_stream_inq_vlist(streamID1);
-
-    auto nvars = vlistNvars(vlistID1);
+    VarList varList1(vlistID1);
 
     std::string cname_ADISIT{ "sea_water_potential_temperature" };
     std::string cname_ADIPOT{ "sea_water_temperature" };
     std::string cname = (operatorID == ADISIT) ? cname_ADISIT : cname_ADIPOT;
 
-    for (int varID = 0; varID < nvars; ++varID)
+    for (auto const &var : varList1.vars)
       {
-        auto code = get_code(vlistID1, varID, cname);
+        auto code = get_code(var, cname);
 
-        if (code == 2) { thoID = varID; }
-        else if (code == 20 && operatorID == ADIPOT) { thoID = varID; }
-        else if (code == 5) { saoID = varID; }
+        if (code == 2) { thoID = var.ID; }
+        else if (code == 20 && operatorID == ADIPOT) { thoID = var.ID; }
+        else if (code == 5) { saoID = var.ID; }
       }
 
     if (saoID == -1) cdo_abort("Sea water salinity not found!");
@@ -315,7 +315,7 @@ public:
 
     auto outputSettingFunc = (operatorID == ADISIT) ? outputSetting_ADISIT : outputSetting_ADIPOT;
 
-    const auto &configResults = configureOutput(outputSettingFunc, vlistID1, thoID, saoID, tho, sao, tis, pressure);
+    auto const &configResults = configureOutput(outputSettingFunc, varList1, vlistID1, thoID, saoID, tho, sao, tis, pressure);
 
     streamID2 = configResults.streamID2;
     vlistID2 = configResults.vlistID2;
@@ -333,18 +333,17 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        while (numFields--)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            if (varID == thoID) cdo_read_record(streamID1, tho[levelID].vec_d.data(), &tho[levelID].numMissVals);
-            if (varID == saoID) cdo_read_record(streamID1, sao[levelID].vec_d.data(), &sao[levelID].numMissVals);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            if (varID == thoID) cdo_read_field(streamID1, tho[levelID]);
+            if (varID == saoID) cdo_read_field(streamID1, sao[levelID]);
 
             if (varID == thoID)
               {
@@ -364,14 +363,16 @@ public:
 
         for (int levelID = 0; levelID < nlevels; ++levelID)
           {
-            cdo_def_record(streamID2, tisID2, levelID);
-            cdo_write_record(streamID2, tis[levelID].vec_d.data(), field_num_miss(tis[levelID]));
+            field_num_mv(tis[levelID]);
+            cdo_def_field(streamID2, tisID2, levelID);
+            cdo_write_field(streamID2, tis[levelID]);
           }
 
         for (int levelID = 0; levelID < nlevels; ++levelID)
           {
-            cdo_def_record(streamID2, saoID2, levelID);
-            cdo_write_record(streamID2, sao[levelID].vec_d.data(), field_num_miss(sao[levelID]));
+            field_num_mv(sao[levelID]);
+            cdo_def_field(streamID2, saoID2, levelID);
+            cdo_write_field(streamID2, sao[levelID]);
           }
 
         tsID++;
