@@ -26,11 +26,12 @@
 #include "cdo_vlist.h"
 #include "cdo_options.h"
 #include "cdo_fill.h"
+#include "cdo_cdi_wrapper.h"
 #include "field_functions.h"
 
 class Arith : public Process
 {
-  enum class FillType
+  enum struct FillType
   {
     NONE,
     TS,
@@ -67,12 +68,10 @@ public:
   Varray<double> vardata2;
 
   CdoStreamID streamID1;
-  int vlistID1;
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
 
   CdoStreamID streamID2;
   CdoStreamID streamID2x;
-  int vlistID2;
 
   CdoStreamID streamID3;
   int vlistID3;
@@ -88,11 +87,12 @@ public:
   Field *fieldx1;
   Field *fieldx2;
 
-  VarList varList1, varList2;
+  VarList varList1;
+  VarList varList2;
 
 public:
   void
-  init()
+  init() override
   {
     auto operatorID = cdo_operator_id();
     operfunc = cdo_operator_f1(operatorID);
@@ -108,8 +108,8 @@ public:
     fieldx1 = &field1;
     fieldx2 = &field2;
 
-    vlistID1 = cdo_stream_inq_vlist(streamID1);
-    vlistID2 = cdo_stream_inq_vlist(streamID2);
+    auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+    auto vlistID2 = cdo_stream_inq_vlist(streamID2);
     auto vlistID1x = vlistID1;
     auto vlistID2x = vlistID2;
 
@@ -129,8 +129,8 @@ public:
 
     if (nvars1 == 1 && nvars2 == 1)
       {
-        fillStream2 = (vlistNrecs(vlistID1) != 1 && vlistNrecs(vlistID2) == 1);
-        fillStream1 = (vlistNrecs(vlistID1) == 1 && vlistNrecs(vlistID2) != 1);
+        fillStream2 = (vlistNumFields(vlistID1) != 1 && vlistNumFields(vlistID2) == 1);
+        fillStream1 = (vlistNumFields(vlistID1) == 1 && vlistNumFields(vlistID2) != 1);
         if (fillStream1 && ntsteps1 != 1 && ntsteps2 == 1)
           {
             fillStream1 = false;
@@ -146,14 +146,14 @@ public:
 
     if (fillStream1x)
       {
-        nlevels2 = vlist_compare_x(vlistID2, vlistID1, CmpVlist::Dim);
+        nlevels2 = vlist_compare_x(vlistID2, vlistID1, CmpVarList::Dim);
 
         fillType = FillType::NONE;
         cdo_print("Filling up stream1 >%s< by copying the first variable of each timestep.", cdo_get_stream_name(0));
       }
     else if (fillStream2)
       {
-        nlevels2 = vlist_compare_x(vlistID1, vlistID2, CmpVlist::Dim);
+        nlevels2 = vlist_compare_x(vlistID1, vlistID2, CmpVarList::Dim);
 
         if (ntsteps1 != 1 && ntsteps2 == 1)
           {
@@ -168,7 +168,7 @@ public:
       }
     else if (fillStream1)
       {
-        nlevels2 = vlist_compare_x(vlistID2, vlistID1, CmpVlist::Dim);
+        nlevels2 = vlist_compare_x(vlistID2, vlistID1, CmpVarList::Dim);
 
         if (ntsteps1 == 1 && ntsteps2 != 1)
           {
@@ -189,23 +189,24 @@ public:
         fieldx2 = &field1;
       }
 
-    if (fillStream1x == false && fillType == FillType::NONE) vlist_compare(vlistID1, vlistID2, CmpVlist::All);
+    if (fillStream1x == false && fillType == FillType::NONE) vlist_compare(vlistID1, vlistID2, CmpVarList::All);
 
-    nwpv = (vlistNumber(vlistID1x) == CDI_COMP && vlistNumber(vlistID2x) == CDI_COMP) ? 2 : 1;
+    nwpv = (vlistNumber(vlistID1) == CDI_COMP && vlistNumber(vlistID2) == CDI_COMP) ? 2 : 1;
     if (nwpv == 2 && !opercplx) cdo_abort("Fields with complex numbers are not supported by this operator!");
-    auto gridsizemax = nwpv * vlistGridsizeMax(vlistID1x);
+    auto gridsizeMax = nwpv * cdo_vlist_gridsizemax(vlistID1x);
 
-    field1.resize(gridsizemax);
-    field2.resize(gridsizemax);
+    field1.resize(gridsizeMax);
+    field2.resize(gridsizeMax);
 
     if (fillStream1x || fillType == FillType::VAR || fillType == FillType::VARTS)
       {
-        vardata2.resize(gridsizemax * nlevels2);
+        vardata2.resize(gridsizeMax * nlevels2);
         varnumMissVals2.resize(nlevels2);
       }
 
     if (Options::cdoVerbose) cdo_print("Number of timesteps: file1 %d, file2 %d", ntsteps1, ntsteps2);
 
+    bool streamsSwaped = false;
     if (fillType == FillType::NONE)
       {
         if (ntsteps1 != 1 && ntsteps2 == 1)
@@ -218,6 +219,8 @@ public:
             fillType = FillType::TS;
             cdo_print("Filling up stream1 >%s< by copying the first timestep.", cdo_get_stream_name(0));
 
+            streamsSwaped = true;
+            vlistDefNtsteps(vlistID1, ntsteps2);
             std::swap(streamID1, streamID2);
             vlistID1x = vlistID2;
             vlistID2x = vlistID1;
@@ -229,8 +232,6 @@ public:
         if (fillType == FillType::TS) cdo_fill_ts(vlistID2x, vardata, varnumMissVals);
       }
 
-    auto streamsSwaped = (fillType == FillType::TS && vlistID1x != vlistID1);
-
     vlistID3 = vlistDuplicate(streamsSwaped ? vlistID1 : vlistID1x);
     vlist_unpack(vlistID3);
     if (streamsSwaped)
@@ -252,27 +253,28 @@ public:
     streamID3 = cdo_open_write(2);
     cdo_def_vlist(streamID3, vlistID3);
 
-    varList_init(varList1, vlistID1x);
-    varList_init(varList2, vlistID2x);
+    varList1 = VarList(vlistID1x);
+    varList2 = VarList(vlistID2x);
   }
 
   void
-  run()
+  run() override
   {
-    int nrecs, nrecs2;
+    int numFields1;
+    int numFields2;
     int tsID = 0;
     int tsID2 = 0;
     while (true)
       {
-        nrecs = cdo_stream_inq_timestep(streamID1, tsID);
+        numFields1 = cdo_stream_inq_timestep(streamID1, tsID);
 
-        nrecs2 = 0;
+        numFields2 = 0;
         if (tsID == 0 || fillType == FillType::NONE || fillType == FillType::FULL_FILE || fillType == FillType::VARTS)
           {
-            nrecs2 = cdo_stream_inq_timestep(streamID2, tsID2);
-            if (nrecs2 == 0)
+            numFields2 = cdo_stream_inq_timestep(streamID2, tsID2);
+            if (numFields2 == 0)
               {
-                if (nrecs == 0) break;
+                if (numFields1 == 0) break;
 
                 if (fillType == FillType::NONE && streamID2x == streamID2)
                   {
@@ -289,52 +291,52 @@ public:
                     streamID2 = cdo_open_read(1);
                     streamID2x = streamID2;
 
-                    vlistID2 = cdo_stream_inq_vlist(streamID2);
+                    (void) cdo_stream_inq_vlist(streamID2);
 
                     tsID2 = 0;
-                    nrecs2 = cdo_stream_inq_timestep(streamID2, tsID2);
-                    if (nrecs2 == 0) cdo_abort("Empty input stream %s!", cdo_get_stream_name(1));
+                    numFields2 = cdo_stream_inq_timestep(streamID2, tsID2);
+                    if (numFields2 == 0) cdo_abort("Empty input stream %s!", cdo_get_stream_name(1));
                   }
                 else
                   cdo_abort("Input streams have different number of timesteps!");
               }
           }
 
-        if (nrecs == 0 || (nrecs2 == 0 && fillType != FillType::TS && fillType != FillType::VAR)) break;
+        if (numFields1 == 0 || (numFields2 == 0 && fillType != FillType::TS && fillType != FillType::VAR)) break;
 
         cdo_taxis_copy_timestep(taxisID3, taxisID1);
         cdo_def_timestep(streamID3, tsID);
 
-        auto numrecs = fillStream1x ? nrecs2 : nrecs;
-        for (int recID = 0; recID < numrecs; ++recID)
+        auto numFields = fillStream1x ? numFields2 : numFields1;
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
             auto lread1 = true;
-            if (fillStream1x && recID > 0) lread1 = false;
+            if (fillStream1x && fieldID > 0) lread1 = false;
             int varID = -1, levelID;
             if (lread1)
               {
-                cdo_inq_record(streamID1, &varID, &levelID);
-                cdo_read_record(streamID1, fieldx1->vec_d.data(), &fieldx1->numMissVals);
+                cdo_inq_field(streamID1, &varID, &levelID);
+                cdo_read_field(streamID1, *fieldx1);
 
                 if (fillStream1x)
                   {
-                    auto gridsize = nwpv * varList1[varID].gridsize;
+                    auto gridsize = nwpv * varList1.vars[varID].gridsize;
                     array_copy(gridsize, fieldx1->vec_d.data(), &vardata2[0]);
                     varnumMissVals2[0] = fieldx1->numMissVals;
                   }
               }
 
-            if (fillStream1x) levelID = recID;
+            if (fillStream1x) levelID = fieldID;
 
             auto varID2 = varID;
 
             if (tsID == 0 || fillType == FillType::NONE || fillType == FillType::FULL_FILE || fillType == FillType::VARTS)
               {
-                auto lstatus = (nlevels2 > 1) ? (varID == 0) : (recID == 0);
+                auto lstatus = (nlevels2 > 1) ? (varID == 0) : (fieldID == 0);
                 if (lstatus || (fillType != FillType::VAR && fillType != FillType::VARTS))
                   {
-                    cdo_inq_record(streamID2, &varID2, &levelID2);
-                    cdo_read_record(streamID2, fieldx2->vec_d.data(), &fieldx2->numMissVals);
+                    cdo_inq_field(streamID2, &varID2, &levelID2);
+                    cdo_read_field(streamID2, *fieldx2);
                     if (varID != varID2) cdo_abort("Internal error, varIDs of input streams differ!");
                     if (fillStream1x == false && levelID != levelID2)
                       cdo_abort("Internal error, levelIDs of input streams differ!");
@@ -342,14 +344,14 @@ public:
 
                 if (fillType == FillType::TS)
                   {
-                    auto gridsize = nwpv * varList2[varID].gridsize;
+                    auto gridsize = nwpv * varList2.vars[varID].gridsize;
                     auto offset = gridsize * levelID;
                     array_copy(gridsize, fieldx2->vec_d.data(), &vardata[varID][offset]);
                     varnumMissVals[varID][levelID] = fieldx2->numMissVals;
                   }
                 else if (lstatus && (fillType == FillType::VAR || fillType == FillType::VARTS))
                   {
-                    auto gridsize = nwpv * varList2[0].gridsize;
+                    auto gridsize = nwpv * varList2.vars[0].gridsize;
                     auto offset = gridsize * levelID2;
                     array_copy(gridsize, fieldx2->vec_d.data(), &vardata2[offset]);
                     varnumMissVals2[levelID2] = fieldx2->numMissVals;
@@ -357,7 +359,7 @@ public:
               }
             else if (fillType == FillType::TS)
               {
-                auto gridsize = nwpv * varList2[varID2].gridsize;
+                auto gridsize = nwpv * varList2.vars[varID2].gridsize;
                 auto offset = gridsize * levelID;
                 array_copy(gridsize, &vardata[varID][offset], fieldx2->vec_d.data());
                 fieldx2->numMissVals = varnumMissVals[varID][levelID];
@@ -365,31 +367,31 @@ public:
 
             if (fillStream1x)
               {
-                auto gridsize = nwpv * varList1[0].gridsize;
+                auto gridsize = nwpv * varList1.vars[0].gridsize;
                 array_copy(gridsize, &vardata2[0], fieldx1->vec_d.data());
                 fieldx1->numMissVals = varnumMissVals2[0];
               }
 
-            fieldx1->grid = varList1[varID].gridID;
-            fieldx1->missval = varList1[varID].missval;
-            fieldx1->nwpv = varList1[varID].nwpv;
+            fieldx1->grid = varList1.vars[varID].gridID;
+            fieldx1->missval = varList1.vars[varID].missval;
+            fieldx1->nwpv = varList1.vars[varID].nwpv;
 
             if (fillType == FillType::VAR || fillType == FillType::VARTS)
               {
                 levelID2 = (nlevels2 > 1) ? levelID : 0;
-                auto gridsize = nwpv * varList2[0].gridsize;
+                auto gridsize = nwpv * varList2.vars[0].gridsize;
                 auto offset = gridsize * levelID2;
                 array_copy(gridsize, &vardata2[offset], fieldx2->vec_d.data());
                 fieldx2->numMissVals = varnumMissVals2[levelID2];
-                fieldx2->grid = varList2[0].gridID;
-                fieldx2->missval = varList2[0].missval;
-                fieldx2->nwpv = varList2[0].nwpv;
+                fieldx2->grid = varList2.vars[0].gridID;
+                fieldx2->missval = varList2.vars[0].missval;
+                fieldx2->nwpv = varList2.vars[0].nwpv;
               }
             else
               {
-                fieldx2->grid = varList2[varID2].gridID;
-                fieldx2->missval = varList2[varID2].missval;
-                fieldx2->nwpv = varList2[varID2].nwpv;
+                fieldx2->grid = varList2.vars[varID2].gridID;
+                fieldx2->missval = varList2.vars[varID2].missval;
+                fieldx2->nwpv = varList2.vars[varID2].nwpv;
               }
 
             if (nwpv == 2)
@@ -397,20 +399,20 @@ public:
             else
               field2_function(field1, field2, operfunc);
 
-            cdo_def_record(streamID3, varID, levelID);
-            cdo_write_record(streamID3, field1.vec_d.data(), field1.numMissVals);
+            cdo_def_field(streamID3, varID, levelID);
+            cdo_write_field(streamID3, field1);
           }
 
         tsID++;
         tsID2++;
       }
 
-    if (nrecs == 0 && nrecs2 > 0) cdo_warning("stream2 has more time steps than stream1!");
-    // if (nrecs > 0 && nrecs2 == 0) cdo_warning("stream1 has more time steps than stream2!");
+    if (numFields1 == 0 && numFields2 > 0) cdo_warning("stream2 has more time steps than stream1!");
+    // if (numFields > 0 && numFields2 == 0) cdo_warning("stream1 has more time steps than stream2!");
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID3);
     cdo_stream_close(streamID2);
