@@ -24,7 +24,7 @@
 #include "field_functions.h"
 
 static double
-dayofyear(int calendar, const CdiDateTime &vDateTime)
+dayofyear(int calendar, CdiDateTime const &vDateTime)
 {
   constexpr int month_360[12] = { 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30 };
   constexpr int month_365[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
@@ -79,10 +79,10 @@ public:
   int MULDOY;
 
   CdoStreamID streamID1;
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
 
   CdoStreamID streamID2;
-  int taxisID2;
+  int taxisID2{ CDI_UNDEFID };
 
   int operatorID;
   int operfunc;
@@ -128,8 +128,8 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         auto vDateTime = taxisInqVdatetime(taxisID1);
 
@@ -147,16 +147,16 @@ public:
 
         if (Options::cdoVerbose) cdo_print("calendar %d  year %d  month %d  result %g", calendar, year, month, rconst);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             field.init(varList1.vars[varID]);
-            cdo_read_record(streamID1, field);
+            cdo_read_field(streamID1, field);
 
             fieldc_function(field, rconst, operfunc);
 
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, field);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, field);
           }
 
         tsID++;
