@@ -19,6 +19,28 @@
 #include <mpim_grid.h>
 #include "field_functions.h"
 
+template <typename T>
+static void
+field_scale(Varray<T> &v, size_t n, size_t numMissVals, double missval, Varray<double> const &scale)
+{
+  if (numMissVals)
+    {
+      for (size_t i = 0; i < n; ++i)
+        if (fp_is_not_equal(v[i], missval)) v[i] *= scale[i];
+    }
+  else
+    {
+      for (size_t i = 0; i < n; ++i) v[i] *= scale[i];
+    }
+}
+
+static void
+field_scale(Field &field, Varray<double> const &scale)
+{
+  auto func = [&](auto &v, auto n, auto numMissVals, auto missval) { field_scale(v, n, numMissVals, missval, scale); };
+  field_operation(func, field, field.gridsize, field.numMissVals, field.missval);
+}
+
 class Arithlat : public Process
 {
 public:
@@ -37,16 +59,14 @@ public:
   inline static RegisterEntry<Arithlat> registration = RegisterEntry<Arithlat>(module);
 
   CdoStreamID streamID1;
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
 
   CdoStreamID streamID2;
-  int taxisID2;
+  int taxisID2{ CDI_UNDEFID };
 
   int operfunc;
 
   VarList varList1;
-  Varray<double> array;
-  Varray<double> scale;
 
 public:
   void
@@ -70,36 +90,33 @@ public:
     cdo_def_vlist(streamID2, vlistID2);
 
     varList1 = VarList(vlistID1);
-
-    auto gridsizemax = vlistGridsizeMax(vlistID1);
-
-    array = Varray<double>(gridsizemax);
   }
 
   void
   run() override
   {
+    Varray<double> scale;
+    Field field;
+
     int gridID0 = -1;
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            size_t numMissVals;
-            cdo_read_record(streamID1, array.data(), &numMissVals);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            auto const &var1 = varList1.vars[varID];
+            field.init(var1);
+            cdo_read_field(streamID1, field);
 
-            const auto &var1 = varList1.vars[varID];
             auto gridID = var1.gridID;
             auto gridsize = var1.gridsize;
-            auto missval = var1.missval;
 
             if (gridID != gridID0)
               {
@@ -111,7 +128,7 @@ public:
                 gridInqYvals(gridID, scale.data());
 
                 // Convert lat/lon units if required
-                cdo_grid_to_radian(gridID, CDI_XAXIS, scale, "grid latitudes");
+                cdo_grid_to_radian(gridID, CDI_YAXIS, scale, "grid latitudes");
 
                 if (operfunc == FieldFunc_Mul)
                   for (size_t i = 0; i < gridsize; ++i) scale[i] = std::cos(scale[i]);
@@ -122,18 +139,10 @@ public:
                   for (int i = 0; i < 10; ++i) cdo_print("coslat  %3d  %g", i + 1, scale[i]);
               }
 
-            if (numMissVals)
-              {
-                for (size_t i = 0; i < gridsize; ++i)
-                  if (dbl_is_not_equal(array[i], missval)) array[i] *= scale[i];
-              }
-            else
-              {
-                for (size_t i = 0; i < gridsize; ++i) array[i] *= scale[i];
-              }
+            field_scale(field, scale);
 
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, array.data(), numMissVals);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, field);
           }
 
         tsID++;
