@@ -8,7 +8,6 @@
 #include <cdi.h>
 
 #include "process_int.h"
-#include "cdo_vlist.h"
 #include "cdo_options.h"
 #include "pmlist.h"
 #include "param_conversion.h"
@@ -102,7 +101,7 @@ bitround(int nsb, size_t len, Varray<float> &v, float missval)
 
   constexpr uint32_t BIT_XPL_NBR_SGN_FLT = 23;
 
-  // BitRound interprets nsd as number of significant binary digits (bits)
+  // BitRound interprets nsb as number of significant binary digits (bits)
   uint32_t prc_bnr_xpl_rqr = nsb;
 
   uint32_t bit_xpl_nbr_zro = BIT_XPL_NBR_SGN_FLT - prc_bnr_xpl_rqr;
@@ -134,13 +133,13 @@ bitround(int nsb, size_t len, Varray<float> &v, float missval)
 }
 
 static void
-check_range(double value, double minVal, double maxVal, const std::string &key)
+check_range(double value, double minVal, double maxVal, std::string const &key)
 {
   if (value < minVal || value > maxVal) cdo_abort("Parameter %s=%g out of range (min=%g/max=%g)!", key, value, minVal, maxVal);
 }
 
 static void
-check_range(int value, int minVal, int maxVal, const std::string &key)
+check_range(int value, int minVal, int maxVal, std::string const &key)
 {
   if (value < minVal || value > maxVal) cdo_abort("Parameter %s=%d out of range (min=%d/max=%d)!", key, value, minVal, maxVal);
 }
@@ -153,19 +152,19 @@ get_parameter()
   auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      const auto &pargv = cdo_get_oper_argv();
+      auto const &pargv = cdo_get_oper_argv();
 
       KVList kvlist;
       kvlist.name = cdo_module_name();
-      if (kvlist.parse_arguments(pargc, pargv) != 0) cdo_abort("Parse error!");
+      if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-          const auto &value = kv.values[0];
+          auto const &value = kv.values[0];
 
           // clang-format off
           if      (key == "inflevel")  check_range(params.infLevel = parameter_to_double(value), 0.0, 1.0, key);
@@ -245,9 +244,9 @@ set_global_attributes(int vlistID, const BitroundParams &params, int numVarsHave
 }
 
 static std::vector<int>
-get_vars_numbits(const VarList &varList, const std::string &filename)
+get_vars_numbits(const VarList &varList, std::string const &filename)
 {
-  auto numVars = varList.size();
+  auto numVars = varList.numVars();
   std::vector<int> varsNumbits(numVars, -1);
 
   if (filename.size())
@@ -260,20 +259,20 @@ get_vars_numbits(const VarList &varList, const std::string &filename)
       std::fclose(fp);
       if (Options::cdoVerbose) kvlist.print();
 
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
 
-          for (size_t varID = 0; varID < numVars; ++varID)
+          for (auto const &var : varList.vars)
             {
-              if (key == varList[varID].name)
+              if (key == var.name)
                 {
-                  const auto &value = kv.values[0];
+                  auto const &value = kv.values[0];
                   auto numBits = parameter_to_int(value);
                   check_range(numBits, 1, 23, key);
-                  varsNumbits[varID] = numBits;
+                  varsNumbits[var.ID] = numBits;
                 }
             }
         }
@@ -292,22 +291,30 @@ num_vars_have_numbits(const std::vector<int> &varsNumbits)
   return numVarsHaveNumbits;
 }
 
-class ModuleBitrounding
+class Bitrounding : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Bitrounding",
+    .operators = { { "bitrounding", BitroundingHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Bitrounding> registration = RegisterEntry<Bitrounding>(module);
 
   CdoStreamID streamID1;
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
 
   CdoStreamID streamID2;
-  int taxisID2;
-  int vlistID2;
+  int taxisID2{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
   BitroundParams params;
-  int ntsteps;
-  int nvars;
 
   VarList varList1;
-  Field field;
 
   std::vector<VarStat> varsStatGlob;
   std::vector<bool> varsCheckMiss;
@@ -317,12 +324,8 @@ class ModuleBitrounding
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    cdo_operator_add("bitrounding", 0, 0, nullptr);
-
     params = get_parameter();
     if (Options::cdoVerbose) print_parameter(params);
 
@@ -330,7 +333,7 @@ public:
     auto vlistID1 = cdo_stream_inq_vlist(streamID1);
     taxisID1 = vlistInqTaxis(vlistID1);
 
-    varList_init(varList1, vlistID1);
+    varList1 = VarList(vlistID1);
 
     varsNumbits = get_vars_numbits(varList1, params.filename);
     auto numVarsHaveNumbits = num_vars_have_numbits(varsNumbits);
@@ -342,56 +345,57 @@ public:
     check_attributes(vlistID1);
     set_global_attributes(vlistID2, params, numVarsHaveNumbits);
 
-    nvars = vlistNvars(vlistID1);
-    for (int varID = 0; varID < nvars; ++varID)
+    auto numVars = varList1.numVars();
+    for (auto const &var : varList1.vars)
       {
-        if (varList1[varID].memType == MemType::Float)
+        if (var.memType == MemType::Float)
           {
-            int nsb = (varsNumbits[varID] != -1) ? varsNumbits[varID] : params.numBits;
-            if (nsb >= 1 && nsb <= 23) set_local_attributes(vlistID2, varID, nsb);
+            int nsb = (varsNumbits[var.ID] != -1) ? varsNumbits[var.ID] : params.numBits;
+            if (nsb >= 1 && nsb <= 23) set_local_attributes(vlistID2, var.ID, nsb);
           }
       }
 
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
 
-    varsStatGlob = std::vector<VarStat>(nvars);
-    varsCheckMiss = std::vector<bool>(nvars, true);
-    varsCheckFloat = std::vector<bool>(nvars, true);
+    varsStatGlob = std::vector<VarStat>(numVars);
+    varsCheckMiss = std::vector<bool>(numVars, true);
+    varsCheckFloat = std::vector<bool>(numVars, true);
 
-    nsbVarLevels = std::vector<std::vector<int>>(nvars);
-    for (int varID = 0; varID < nvars; ++varID) nsbVarLevels[varID].resize(varList1[varID].nlevels, 0);
-
-    ntsteps = vlistNtsteps(vlistID1);
-
-    progress::init();
+    nsbVarLevels = std::vector<std::vector<int>>(numVars);
+    for (int varID = 0; varID < numVars; ++varID) nsbVarLevels[varID].resize(varList1.vars[varID].nlevels, 0);
   }
+
   void
-  run()
+  run() override
   {
+    Field field;
+    auto numVars = varList1.numVars();
+    auto numSteps = varList1.numSteps();
+    cdo::Progress progress(get_id());
+
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        std::vector<VarStat> varsStat(nvars);
+        std::vector<VarStat> varsStat(numVars);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto fstatus = (ntsteps > 1) ? (tsID + (recID + 1.0) / nrecs) / ntsteps : 1.0;
-            if (!Options::cdoVerbose) progress::update(0, 1, fstatus);
+            auto fstatus = (tsID + (fieldID + 1.0) / numFields) / numSteps;
+            if (numSteps > 0) progress.update(fstatus);
 
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            cdo_def_record(streamID2, varID, levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            cdo_def_field(streamID2, varID, levelID);
 
-            const auto &var = varList1[varID];
+            auto const &var = varList1.vars[varID];
             field.init(var);
-            cdo_read_record(streamID1, field);
+            cdo_read_field(streamID1, field);
 
             if (field.memType == MemType::Double)
               {
@@ -405,7 +409,7 @@ public:
               {
                 int nsb = (varsNumbits[varID] != -1) ? varsNumbits[varID] : params.numBits;
 
-                if (field.nmiss == 0)
+                if (field.numMissVals == 0)
                   {
                     if (nsb == -1 && (tsID == 0 || params.numSteps == -1))
                       {
@@ -419,37 +423,42 @@ public:
                     else if (params.numSteps == 1) { nsb = nsbVarLevels[varID][levelID]; }
                   }
 
-                varsStat[varID].nsbMin = std::min(varsStat[varID].nsbMin, nsb);
-                varsStat[varID].nsbMax = std::max(varsStat[varID].nsbMax, nsb);
+                auto &varStat = varsStat[varID];
+                varStat.nsbMin = std::min(varStat.nsbMin, nsb);
+                varStat.nsbMax = std::max(varStat.nsbMax, nsb);
 
                 if (nsb >= 1 && nsb <= 23) bitround(nsb, field.size, field.vec_f, var.missval);
 
-                if (nsb == -1 && field.nmiss > 0 && varsCheckMiss[varID])
+                if (nsb == -1 && field.numMissVals > 0 && varsCheckMiss[varID])
                   {
                     varsCheckMiss[varID] = false;
                     cdo_warning("Missing values unsupported, bitrounding disabled for %s!", var.name);
                   }
               }
 
-            cdo_write_record(streamID2, field);
+            cdo_write_field(streamID2, field);
           }
 
         if (Options::cdoVerbose && params.numBits == -1)
           {
             fprintf(stderr, "NSB: step=%d:", tsID + 1);
-            for (int varID = 0; varID < nvars; ++varID)
-              if (varsStat[varID].nsbMin >= 1 && varsStat[varID].nsbMin <= 23)
-                {
-                  fprintf(stderr, " %s=%d", varList1[varID].name.c_str(), varsStat[varID].nsbMin);
-                  if (varList1[varID].nlevels > 1) fprintf(stderr, "-%d ", varsStat[varID].nsbMax);
-                }
+            for (auto const &var1 : varList1.vars)
+              {
+                auto const &varStat = varsStat[var1.ID];
+                if (varStat.nsbMin >= 1 && varStat.nsbMin <= 23)
+                  {
+                    fprintf(stderr, " %s=%d", var1.name.c_str(), varStat.nsbMin);
+                    if (var1.nlevels > 1) fprintf(stderr, "-%d ", varStat.nsbMax);
+                  }
+              }
             fprintf(stderr, "\n");
           }
 
-        for (int varID = 0; varID < nvars; ++varID)
+        for (int varID = 0; varID < numVars; ++varID)
           {
-            varsStatGlob[varID].nsbMin = std::min(varsStatGlob[varID].nsbMin, varsStat[varID].nsbMin);
-            varsStatGlob[varID].nsbMax = std::max(varsStatGlob[varID].nsbMax, varsStat[varID].nsbMax);
+            auto &varStatGlob = varsStatGlob[varID];
+            varStatGlob.nsbMin = std::min(varStatGlob.nsbMin, varsStat[varID].nsbMin);
+            varStatGlob.nsbMax = std::max(varStatGlob.nsbMax, varsStat[varID].nsbMax);
           }
 
         if (params.printBits) break;
@@ -459,39 +468,35 @@ public:
 
     if (params.printBits)
       {
-        for (int varID = 0; varID < nvars; ++varID)
-          if (varsStatGlob[varID].nsbMin >= 1 && varsStatGlob[varID].nsbMin <= 23)
-            fprintf(stdout, "%s=%d\n", varList1[varID].name.c_str(), varsStatGlob[varID].nsbMax);
+        for (auto const &var1 : varList1.vars)
+          {
+            auto const &varStatGlob = varsStatGlob[var1.ID];
+            if (varStatGlob.nsbMin >= 1 && varStatGlob.nsbMin <= 23)
+              fprintf(stdout, "%s=%d\n", var1.name.c_str(), varStatGlob.nsbMax);
+          }
       }
     else if (Options::cdoVerbose && params.numBits == -1)
       {
         fprintf(stderr, "NSB: step=all:");
-        for (int varID = 0; varID < nvars; ++varID)
-          if (varsStatGlob[varID].nsbMin >= 1 && varsStatGlob[varID].nsbMin <= 23)
-            {
-              fprintf(stderr, " %s=%d", varList1[varID].name.c_str(), varsStatGlob[varID].nsbMin);
-              if (varsStatGlob[varID].nsbMin != varsStatGlob[varID].nsbMax) fprintf(stderr, "-%d", varsStatGlob[varID].nsbMax);
-            }
-        fprintf(stderr, "\n");
+        for (auto const &var1 : varList1.vars)
+          {
+            auto const &varStatGlob = varsStatGlob[var1.ID];
+            if (varStatGlob.nsbMin >= 1 && varStatGlob.nsbMin <= 23)
+              {
+                fprintf(stderr, " %s=%d", var1.name.c_str(), varStatGlob.nsbMin);
+                if (varStatGlob.nsbMin != varStatGlob.nsbMax) fprintf(stderr, "-%d", varStatGlob.nsbMax);
+              }
+            fprintf(stderr, "\n");
+          }
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID1);
     cdo_stream_close(streamID2);
 
     vlistDestroy(vlistID2);
-    cdo_finish();
   }
 };
-
-void *
-Bitrounding(void *process)
-{
-  ModuleBitrounding bitrounding;
-  bitrounding.init(process);
-  bitrounding.run();
-  bitrounding.close();
-  return nullptr;
-}
