@@ -15,21 +15,32 @@
 #include "cdo_timer.h"
 #include "process_int.h"
 
-class ModuleCDItest
+class CDItest : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "CDItest",
+    .operators = { { "ncopy" } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<CDItest> registration = RegisterEntry<CDItest>(module);
+
+  int NCOPY;
   bool dataIsUnchanged;
   int max_copy;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
     dataIsUnchanged = false;
     // auto dataIsUnchanged = data_is_unchanged();
 
-    auto NCOPY = cdo_operator_add("ncopy", 0, 0, nullptr);
+    NCOPY = module.get_id("ncopy");
     (void) (NCOPY);  // unused
 
     auto operatorID = cdo_operator_id();
@@ -38,8 +49,9 @@ public:
     //  operator_input_arg("Number of copies");
     max_copy = (cdo_operator_argc() == 1) ? parameter_to_int(cdo_operator_argv(0)) : 3;
   }
+
   void
-  run()
+  run() override
   {
     int n = 0;
     while (true)
@@ -60,31 +72,29 @@ public:
 
         Field field;
 
-        VarList varList1;
-        varList_init(varList1, vlistID1);
+        VarList varList1(vlistID1);
 
         int tsID = 0;
         while (true)
           {
-            auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-            if (nrecs == 0) break;
+            auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+            if (numFields == 0) break;
 
             cdo_taxis_copy_timestep(taxisID2, taxisID1);
 
             cdo_def_timestep(streamID2, tsID);
 
-            for (int recID = 0; recID < nrecs; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                int varID, levelID;
-                cdo_inq_record(streamID1, &varID, &levelID);
-                cdo_def_record(streamID2, varID, levelID);
+                auto [varID, levelID] = cdo_inq_field(streamID1);
+                cdo_def_field(streamID2, varID, levelID);
 
-                if (dataIsUnchanged) { cdo_copy_record(streamID2, streamID1); }
+                if (dataIsUnchanged) { cdo_copy_field(streamID2, streamID1); }
                 else
                   {
-                    field.init(varList1[varID]);
-                    cdo_read_record(streamID1, field);
-                    cdo_write_record(streamID2, field);
+                    field.init(varList1.vars[varID]);
+                    cdo_read_field(streamID1, field);
+                    cdo_write_field(streamID2, field);
                   }
               }
 
@@ -106,20 +116,7 @@ public:
   }
 
   void
-  close()
+  close() override
   {
-    cdo_finish();
   }
 };
-
-void *
-CDItest(void *process)
-{
-  ModuleCDItest cditest;
-
-  cditest.init(process);
-  cditest.run();
-  cditest.close();
-
-  return nullptr;
-}
