@@ -21,7 +21,7 @@
 
 class Cat : public Process
 {
-  enum class StreamMode
+  enum struct StreamMode
   {
     APPEND,
     CREATE
@@ -75,10 +75,10 @@ public:
         auto taxisID1 = vlistInqTaxis(vlistID1);
 
         VarList varList1(vlistID1);
+        auto numSteps = varList1.numSteps();
 
         if (indf == 0)
           {
-            auto numSteps = varList1.numSteps();
             if (numSteps == 1 && varList1.numVaryingVars() == 0) numSteps = 0;
 
             std::string ofilename = cdo_get_stream_name(numFiles);
@@ -103,6 +103,7 @@ public:
                 streamMode = StreamMode::CREATE;
                 streamID2 = cdo_open_write(numFiles);
                 vlistID2 = vlistDuplicate(vlistID1);
+                vlist_unpack(vlistID2);
                 taxisID2 = taxisDuplicate(taxisID1);
                 vlistDefTaxis(vlistID2, taxisID2);
 
@@ -122,12 +123,11 @@ public:
             varList_compare(varList1, varList2);
           }
 
-        auto numSteps = varList1.numSteps();
         int tsID1 = 0;
         while (true)
           {
-            auto numRecords = cdo_stream_inq_timestep(streamID1, tsID1);
-            if (numRecords == 0) break;
+            auto numFields = cdo_stream_inq_timestep(streamID1, tsID1);
+            if (numFields == 0) break;
 
             auto fstatus = (numSteps > 1) ? indf + (tsID1 + 1.0) / numSteps : indf + 1.0;
             progress.update(fstatus / numFiles);
@@ -135,21 +135,21 @@ public:
             cdo_taxis_copy_timestep(taxisID2, taxisID1);
             cdo_def_timestep(streamID2, tsID2);
 
-            for (int recID = 0; recID < numRecords; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                auto [varID, levelID] = cdo_inq_record(streamID1);
+                auto [varID, levelID] = cdo_inq_field(streamID1);
 
-                const auto &var1 = varList1.vars[varID];
+                auto const &var1 = varList1.vars[varID];
                 if (hasConstVars && tsID2 > 0 && tsID1 == 0 && var1.isConstant) continue;
 
-                cdo_def_record(streamID2, varID, levelID);
+                cdo_def_field(streamID2, varID, levelID);
 
-                if (dataIsUnchanged) { cdo_copy_record(streamID2, streamID1); }
+                if (dataIsUnchanged) { cdo_copy_field(streamID2, streamID1); }
                 else
                   {
                     field.init(var1);
-                    cdo_read_record(streamID1, field);
-                    cdo_write_record(streamID2, field);
+                    cdo_read_field(streamID1, field);
+                    cdo_write_field(streamID2, field);
                   }
               }
 
