@@ -27,7 +27,7 @@
 static void
 change_code(const VarList &varList1, int vlistID2, int nch, const std::vector<int> &chints)
 {
-  for (const auto &var1 : varList1.vars)
+  for (auto const &var1 : varList1.vars)
     {
       for (int i = 0; i < nch; i += 2)
         if (var1.code == chints[i]) vlistDefVarCode(vlistID2, var1.ID, chints[i + 1]);
@@ -37,7 +37,7 @@ change_code(const VarList &varList1, int vlistID2, int nch, const std::vector<in
 static void
 change_tabnum(const VarList &varList1, int vlistID2, int nch, const std::vector<int> &chints)
 {
-  for (const auto &var1 : varList1.vars)
+  for (auto const &var1 : varList1.vars)
     {
       auto tabnum = tableInqNum(vlistInqVarTable(vlistID2, var1.ID));
       for (int i = 0; i < nch; i += 2)
@@ -52,7 +52,7 @@ change_tabnum(const VarList &varList1, int vlistID2, int nch, const std::vector<
 static void
 change_param(const VarList &varList1, int vlistID2, int nch, const std::vector<const char *> &chnames)
 {
-  for (const auto &var1 : varList1.vars)
+  for (auto const &var1 : varList1.vars)
     {
       if (Options::cdoVerbose)
         {
@@ -74,7 +74,7 @@ change_name(const VarList &varList1, int vlistID2, int nch, const std::vector<co
   for (int i = 0; i < npairs; ++i) vpairs[i].second = chnames[i * 2 + 1];
 
   std::vector<bool> namefound(npairs, false);
-  for (const auto &var1 : varList1.vars)
+  for (auto const &var1 : varList1.vars)
     {
       for (int i = 0; i < npairs; ++i)
         if (var1.name == vpairs[i].first)
@@ -95,8 +95,8 @@ change_name(const VarList &varList1, int vlistID2, int nch, const std::vector<co
 
   if (searchForGridName)
     {
-      auto ngrids = vlistNumGrids(vlistID2);
-      for (int index = 0; index < ngrids; ++index)
+      auto numGrids = vlistNumGrids(vlistID2);
+      for (int index = 0; index < numGrids; ++index)
         {
           int gridID2 = -1;
           auto gridID1 = vlistGrid(vlistID2, index);
@@ -143,8 +143,8 @@ change_name(const VarList &varList1, int vlistID2, int nch, const std::vector<co
 
   if (searchForZaxisName)
     {
-      auto nzaxis = vlistNumZaxis(vlistID2);
-      for (int index = 0; index < nzaxis; ++index)
+      auto numZaxes = vlistNumZaxis(vlistID2);
+      for (int index = 0; index < numZaxes; ++index)
         {
           auto zaxisID1 = vlistZaxis(vlistID2, index);
           auto varname = cdo::inq_key_string(zaxisID1, CDI_GLOBAL, CDI_KEY_NAME);
@@ -172,7 +172,7 @@ change_name(const VarList &varList1, int vlistID2, int nch, const std::vector<co
 static void
 change_unit(const VarList &varList1, int vlistID2, int nch, const std::vector<const char *> &chnames)
 {
-  for (const auto &var1 : varList1.vars)
+  for (auto const &var1 : varList1.vars)
     {
       for (int i = 0; i < nch; i += 2)
         if (var1.units == chnames[i]) cdiDefKeyString(vlistID2, var1.ID, CDI_KEY_UNITS, chnames[i + 1]);
@@ -182,8 +182,8 @@ change_unit(const VarList &varList1, int vlistID2, int nch, const std::vector<co
 static void
 change_level(int vlistID2, int nch, const std::vector<double> &chlevels)
 {
-  auto nzaxis = vlistNumZaxis(vlistID2);
-  for (int index = 0; index < nzaxis; ++index)
+  auto numZaxes = vlistNumZaxis(vlistID2);
+  for (int index = 0; index < numZaxes; ++index)
     {
       auto zaxisID1 = vlistZaxis(vlistID2, index);
       if (zaxisInqLevels(zaxisID1, nullptr))
@@ -243,7 +243,7 @@ change_varLevel(int varID, int vlistID2, const std::vector<double> &chlevels)
 static void
 change_levelByCode(int chcode, const VarList &varList1, int vlistID2, const std::vector<double> &chlevels)
 {
-  for (const auto &var1 : varList1.vars)
+  for (auto const &var1 : varList1.vars)
     {
       if (var1.code == chcode)
         {
@@ -258,7 +258,7 @@ change_levelByCode(int chcode, const VarList &varList1, int vlistID2, const std:
 static void
 change_levelByName(const char *chname, const VarList &varList1, int vlistID2, const std::vector<double> &chlevels)
 {
-  for (const auto &var1 : varList1.vars)
+  for (auto const &var1 : varList1.vars)
     {
       if (var1.name == chname)
         {
@@ -273,8 +273,8 @@ change_levelByName(const char *chname, const VarList &varList1, int vlistID2, co
 static void
 change_ltype(int vlistID2, int nch, const std::vector<int> &chltypes)
 {
-  auto nzaxis = vlistNumZaxis(vlistID2);
-  for (int index = 0; index < nzaxis; ++index)
+  auto numZaxes = vlistNumZaxis(vlistID2);
+  for (int index = 0; index < numZaxes; ++index)
     {
       auto zaxisID1 = vlistZaxis(vlistID2, index);
       auto zaxisID2 = zaxisDuplicate(zaxisID1);
@@ -320,10 +320,10 @@ public:
   int CHCODE, CHTABNUM, CHPARAM, CHNAME, CHUNIT, CHLEVEL, CHLEVELC, CHLEVELV, CHLTYPE;
 
   CdoStreamID streamID1;
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
 
   CdoStreamID streamID2;
-  int taxisID2;
+  int taxisID2{ CDI_UNDEFID };
 
   VarList varList1;
 
@@ -431,21 +431,20 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            cdo_def_record(streamID2, varID, levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            cdo_def_field(streamID2, varID, levelID);
 
             field.init(varList1.vars[varID]);
-            cdo_read_record(streamID1, field);
-            cdo_write_record(streamID2, field);
+            cdo_read_field(streamID1, field);
+            cdo_write_field(streamID2, field);
           }
 
         tsID++;
