@@ -29,25 +29,25 @@ static VarList varList;
 static CdiStreamID gl_streamID = 0;
 static int gl_vlistID = 0;
 static int gl_varID = 0;
-static int gl_nvars = 0;
+static int gl_numVars = 0;
 static int gl_levelID = 0;
 static int gl_tsID1 = 0;
 static int gl_tsID2 = 0;
-static int gl_ntsteps = -1;
+static int gl_numSteps = -1;
 static Varray<double> gl_data;
 
 static int Done = 0;
 
-static int com_help(const std::string &);
-static int com_list(const std::string &);
-static int com_quit(const std::string &);
-static int com_stat(const std::string &);
-static int com_set(const std::string &);
-static int com_vars(const std::string &);
+static int com_help(std::string const &);
+static int com_list(std::string const &);
+static int com_quit(std::string const &);
+static int com_stat(std::string const &);
+static int com_set(std::string const &);
+static int com_vars(std::string const &);
 
 struct command_t
 {
-  int (*func)(const std::string &);  // Function to call to do the job.
+  int (*func)(std::string const &);  // Function to call to do the job.
   const std::string name;            // User printable name of the function.
   const std::string doc;             // Documentation for this function.
 };
@@ -76,7 +76,7 @@ valid_argument(char *caller, char *arg)
 */
 // Print out help for ARG, or for all of the commands if ARG is not present.
 static int
-com_help(const std::string &arg)
+com_help(std::string const &arg)
 {
   int printed = 0;
 
@@ -111,7 +111,7 @@ com_help(const std::string &arg)
 
 // List the file(s) named in arg
 static int
-com_list(const std::string &arg)
+com_list(std::string const &arg)
 {
   (void) (arg);
 
@@ -120,7 +120,7 @@ com_list(const std::string &arg)
 
 // The user wishes to quit using this program. Just set DONE non-zero.
 static int
-com_quit(const std::string &arg)
+com_quit(std::string const &arg)
 {
   (void) (arg);
 
@@ -130,23 +130,23 @@ com_quit(const std::string &arg)
 }
 
 static int
-com_stat(const std::string &arg)
+com_stat(std::string const &arg)
 {
   (void) (arg);  // unused
 
-  const auto &var = varList.vars[gl_varID];
+  auto const &var = varList.vars[gl_varID];
   auto name = var.name.c_str();
 
   auto tsID2 = gl_tsID2;
-  if (tsID2 == -1) tsID2 = (gl_ntsteps != -1) ? gl_ntsteps - 1 : INT_MAX - 1;
+  if (tsID2 == -1) tsID2 = (gl_numSteps != -1) ? gl_numSteps - 1 : INT_MAX - 1;
 
   for (int tsID = gl_tsID1; tsID <= tsID2; ++tsID)
     {
-      auto nrecs = streamInqTimestep(gl_streamID, tsID);
-      if (nrecs == 0)
+      auto numFields = streamInqTimestep(gl_streamID, tsID);
+      if (numFields == 0)
         {
-          if (gl_ntsteps == -1)
-            gl_ntsteps = tsID;
+          if (gl_numSteps == -1)
+            gl_numSteps = tsID;
           else
             fprintf(stderr, "Timestep %d out of range!\n", tsID + 1);
           break;
@@ -164,7 +164,7 @@ com_stat(const std::string &arg)
           size_t numMissVals;
           streamReadVarSlice(gl_streamID, gl_varID, levelID, gl_data.data(), &numMissVals);
 
-          auto mmm = varray_min_max_mean_mv(gridsize, gl_data, missval);
+          auto mmm = varray_min_max_mean_mv(gl_data, gridsize, missval);
 
           fprintf(stdout, "%s:  z=%d  t=%d  size=%zu numMissVals=%zu  min=%.5g mean=%.5g max=%.5g [%.2fs]\n", name, levelID + 1,
                   tsID + 1, gridsize, numMissVals, mmm.min, mmm.mean, mmm.max, stepTimer.elapsed());
@@ -181,7 +181,7 @@ paramWarning(const char *name, const char *cstring, const char *endptr)
 }
 
 int
-param2int(const std::string &str)
+param2int(std::string const &str)
 {
   char *endptr = nullptr;
   int ival = (int) strtol(str.c_str(), &endptr, 10);
@@ -192,9 +192,9 @@ param2int(const std::string &str)
 static int
 check_tsID(int tsID)
 {
-  if (gl_ntsteps >= 0 && (tsID < -1 || tsID >= gl_ntsteps))
+  if (gl_numSteps >= 0 && (tsID < -1 || tsID >= gl_numSteps))
     {
-      fprintf(stdout, "t=%d out of range (max=%d)!\n", (tsID >= 0) ? tsID + 1 : tsID, gl_ntsteps);
+      fprintf(stdout, "t=%d out of range (max=%d)!\n", (tsID >= 0) ? tsID + 1 : tsID, gl_numSteps);
       return 1;
     }
 
@@ -311,7 +311,7 @@ set_var(const std::vector<std::string> &argv)
 
   auto &name = argv[1];
   auto lfound = false;
-  for (int varID = 0; varID < gl_nvars; ++varID)
+  for (int varID = 0; varID < gl_numVars; ++varID)
     {
       if (name == varList.vars[varID].name)
         {
@@ -329,7 +329,7 @@ set_var(const std::vector<std::string> &argv)
 }
 
 static int
-com_set(const std::string &arg)
+com_set(std::string const &arg)
 {
   printf("com_set: >%s<\n", arg.c_str());
   if (arg.empty())
@@ -353,15 +353,15 @@ com_set(const std::string &arg)
 }
 
 static int
-com_vars(const std::string &arg)
+com_vars(std::string const &arg)
 {
   char paramstr[32];
 
-  printf("com_vars: %s %d\n", arg.c_str(), gl_nvars);
+  printf("com_vars: %s %d\n", arg.c_str(), gl_numVars);
 
-  for (int varID = 0; varID < gl_nvars; ++varID)
+  for (int varID = 0; varID < gl_numVars; ++varID)
     {
-      const auto &var = varList.vars[varID];
+      auto const &var = varList.vars[varID];
       cdiParamToString(var.param, paramstr, sizeof(paramstr));
 
       fprintf(stdout, "varID=%3d, param=%s, name=%s, longname=\"%s\", units=\"%s\"\n", varID + 1, paramstr, var.name.c_str(),
@@ -375,7 +375,7 @@ com_vars(const std::string &arg)
  * name.
  */
 static const command_t *
-find_command(const std::string &name)
+find_command(std::string const &name)
 {
   for (int i = 0; i < ncommands; ++i)
     if (name == commands[i].name) return &commands[i];
@@ -385,7 +385,7 @@ find_command(const std::string &name)
 
 // Execute a command line.
 static int
-execute_line(const std::string &line)
+execute_line(std::string const &line)
 {
   if (line.empty()) return 0;
 
@@ -415,7 +415,7 @@ execute_line(const std::string &line)
 }
 
 std::string
-trim(const std::string &str, const std::string &chars = "\t\n\v\f\r ")
+trim(std::string const &str, std::string const &chars = "\t\n\v\f\r ")
 {
   auto first = str.find_first_not_of(chars);
   if (std::string::npos == first) return str;
@@ -448,7 +448,7 @@ peakRSS_string()
 }
 
 static void
-read_line(const std::string &p_prompt, std::string &line)
+read_line(std::string const &p_prompt, std::string &line)
 {
   fputs(p_prompt.c_str(), stdout);
   if (Options::cdoVerbose)
@@ -471,15 +471,14 @@ command_init()
   auto taxisID = vlistInqTaxis(gl_vlistID);
   (void) (taxisID);  // unused
 
-  auto ntsteps = vlistNtsteps(gl_vlistID);
-  if (ntsteps == 0) ntsteps = 1;
-  gl_ntsteps = ntsteps;
-
-  auto gridsizemax = vlistGridsizeMax(gl_vlistID);
-  gl_data.resize(gridsizemax);
-
-  gl_nvars = vlistNvars(gl_vlistID);
   varList = VarList(gl_vlistID);
+  gl_numVars = varList.numVars();
+
+  auto numSteps = varList.numSteps();
+  if (numSteps == 0) numSteps = 1;
+  gl_numSteps = numSteps;
+
+  gl_data.resize(varList.gridsizeMax());
 
   set_varID(0);
 }
@@ -489,9 +488,9 @@ run_demo()
 {
   gl_tsID1 = 0;
   gl_tsID2 = -1;
-  for (int varID = 0; varID < gl_nvars; ++varID)
+  for (int varID = 0; varID < gl_numVars; ++varID)
     {
-      const auto &var = varList.vars[varID];
+      auto const &var = varList.vars[varID];
       auto nlevels = var.nlevels;
       for (int levelID = 0; levelID < nlevels; ++levelID)
         {
