@@ -27,12 +27,12 @@
 #include "field_functions.h"
 
 static auto func_comp
-    = [](auto hasMissvals, auto n, auto mv1, auto mv2, const auto &vIn1, const auto &vIn2, auto &vOut, auto binary_operator) {
+    = [](auto hasMissvals, auto n, auto mv1, auto mv2, auto const &vIn1, auto const &vIn2, auto &vOut, auto binary_operator) {
         if (hasMissvals)
           {
             if (std::isnan(mv1) || std::isnan(mv2))
               for (size_t i = 0; i < n; ++i)
-                vOut[i] = (dbl_is_equal(vIn1[i], mv1) || dbl_is_equal(vIn2[i], mv2)) ? mv1 : binary_operator(vIn1[i], vIn2[i]);
+                vOut[i] = (fp_is_equal(vIn1[i], mv1) || fp_is_equal(vIn2[i], mv2)) ? mv1 : binary_operator(vIn1[i], vIn2[i]);
             else
               for (size_t i = 0; i < n; ++i)
                 vOut[i] = (is_equal(vIn1[i], mv1) || is_equal(vIn2[i], mv2)) ? mv1 : binary_operator(vIn1[i], vIn2[i]);
@@ -45,7 +45,7 @@ static auto func_comp
 
 class Comp : public Process
 {
-  enum class FillType
+  enum struct FillType
   {
     NONE,
     TS,
@@ -73,7 +73,7 @@ public:
   Varray2D<double> vardata;
 
   CdoStreamID streamID1;
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
   CdoStreamID streamID2;
   CdoStreamID streamID3;
   int taxisID3;
@@ -113,16 +113,16 @@ public:
 
     auto fillstream1 = false;
 
-    if (vlistNumRecords(vlistID1) != 1 && vlistNumRecords(vlistID2) == 1)
+    if (vlistNumFields(vlistID1) != 1 && vlistNumFields(vlistID2) == 1)
       {
         fillType = FillType::REC;
-        cdo_print("Filling up stream2 >%s< by copying the first record.", cdo_get_stream_name(1));
+        cdo_print("Filling up stream2 >%s< by copying the first field.", cdo_get_stream_name(1));
         if (ntsteps2 != 1) cdo_abort("stream2 has more than 1 timestep!");
       }
-    else if (vlistNumRecords(vlistID1) == 1 && vlistNumRecords(vlistID2) != 1)
+    else if (vlistNumFields(vlistID1) == 1 && vlistNumFields(vlistID2) != 1)
       {
         fillType = FillType::REC;
-        cdo_print("Filling up stream1 >%s< by copying the first record.", cdo_get_stream_name(0));
+        cdo_print("Filling up stream1 >%s< by copying the first field.", cdo_get_stream_name(0));
         if (ntsteps1 != 1) cdo_abort("stream1 has more than 1 timestep!");
         fillstream1 = true;
         std::swap(streamID1, streamID2);
@@ -132,14 +132,6 @@ public:
 
     if (fillType == FillType::NONE) vlist_compare(vlistID1, vlistID2, CmpVarList::All);
 
-    auto gridsizemax = vlistGridsizeMax(vlistID1);
-    vaIn1 = Varray<double>(gridsizemax);
-    vaIn2 = Varray<double>(gridsizemax);
-    vaOut = Varray<double>(gridsizemax);
-
-    arrayx1 = vaIn1.data();
-    arrayx2 = vaIn2.data();
-
     if (Options::cdoVerbose) cdo_print("Number of timesteps: file1 %d, file2 %d", ntsteps1, ntsteps2);
 
     if (fillType == FillType::NONE)
@@ -162,15 +154,17 @@ public:
         if (fillType == FillType::TS) cdo_fill_ts(vlistID2, vardata);
       }
 
-    if (fillstream1)
-      {
-        arrayx1 = vaIn2.data();
-        arrayx2 = vaIn1.data();
-      }
-
     varList1 = VarList(vlistID1);
     varList2 = VarList(vlistID2);
 
+    vaIn1.resize(varList1.gridsizeMax());
+    vaIn2.resize(varList1.gridsizeMax());
+    vaOut.resize(varList1.gridsizeMax());
+
+    arrayx1 = vaIn1.data();
+    arrayx2 = vaIn2.data();
+    if (fillstream1) { std::swap(arrayx1, arrayx2); }
+
     auto vlistID3 = vlistDuplicate(vlistID1);
 
     vlist_unpack(vlistID3);
@@ -188,31 +182,32 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         if (tsID == 0 || fillType == FillType::NONE)
           {
-            auto nrecs2 = cdo_stream_inq_timestep(streamID2, tsID);
-            if (nrecs2 == 0) cdo_abort("Input streams have different number of timesteps!");
+            auto numFields2 = cdo_stream_inq_timestep(streamID2, tsID);
+            if (numFields2 == 0) cdo_abort("Input streams have different number of timesteps!");
           }
 
         cdo_taxis_copy_timestep(taxisID3, taxisID1);
         cdo_def_timestep(streamID3, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
             size_t numMissVals1 = 0, numMissVals2 = 0;
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            cdo_read_record(streamID1, arrayx1, &numMissVals1);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            cdo_read_field(streamID1, arrayx1, &numMissVals1);
 
             if (tsID == 0 || fillType == FillType::NONE)
               {
-                if (recID == 0 || fillType != FillType::REC)
+                if (fieldID == 0 || fillType != FillType::REC)
                   {
-                    cdo_inq_record(streamID2, &varID, &levelID);
-                    cdo_read_record(streamID2, arrayx2, &numMissVals2);
+                    auto [varIDx, levelIDx] = cdo_inq_field(streamID2);
+                    cdo_read_field(streamID2, arrayx2, &numMissVals2);
+                    varID = varIDx;
+                    levelID = levelIDx;
                   }
 
                 if (fillType == FillType::TS)
@@ -229,13 +224,13 @@ public:
                 array_copy(gridsize, &vardata[varID][offset], arrayx2);
               }
 
-            const auto &var1 = varList1.vars[varID];
+            auto const &var1 = varList1.vars[varID];
             auto datatype1 = var1.dataType;
             auto gridsize1 = var1.gridsize;
             auto missval1 = var1.missval;
 
             auto xvarID = (fillType == FillType::REC) ? 0 : varID;
-            const auto &var2 = varList2.vars[xvarID];
+            auto const &var2 = varList2.vars[xvarID];
             auto datatype2 = var2.dataType;
             auto gridsize2 = var2.gridsize;
             auto missval2 = var2.missval;
@@ -274,8 +269,8 @@ public:
             // clang-format on
 
             auto numMissValsOut = varray_num_mv(ngp, vaOut, missval1);
-            cdo_def_record(streamID3, varID, levelID);
-            cdo_write_record(streamID3, vaOut.data(), numMissValsOut);
+            cdo_def_field(streamID3, varID, levelID);
+            cdo_write_field(streamID3, vaOut.data(), numMissValsOut);
           }
 
         tsID++;
