@@ -27,21 +27,17 @@ public:
   int COMPLEXTORECT, COMPLEXTOPOL;
 
   CdoStreamID streamID1;
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
 
   CdoStreamID streamID2;
-  int taxisID2;
-  int vlistID2;
+  int taxisID2{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
   CdoStreamID streamID3;
   int taxisID3;
   int vlistID3;
 
   int operatorID;
-
-  Varray<double> array1;
-  Varray<double> array2;
-  Varray<double> array3;
   VarList varList1;
 
 public:
@@ -82,21 +78,21 @@ public:
     cdo_def_vlist(streamID2, vlistID2);
     cdo_def_vlist(streamID3, vlistID3);
     varList1 = VarList(vlistID1);
-
-    auto gridsizemax = vlistGridsizeMax(vlistID1);
-    array1 = Varray<double>(2 * gridsizemax);
-    array2 = Varray<double>(gridsizemax);
-    array3 = Varray<double>(gridsizemax);
   }
 
   void
   run() override
   {
+    auto gridsizeMax = varList1.gridsizeMax();
+    Varray<double> array1(2 * gridsizeMax);
+    Varray<double> array2(gridsizeMax);
+    Varray<double> array3(gridsizeMax);
+
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_taxis_copy_timestep(taxisID3, taxisID1);
@@ -104,20 +100,19 @@ public:
         cdo_def_timestep(streamID2, tsID);
         cdo_def_timestep(streamID3, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_def_record(streamID3, varID, levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_def_field(streamID3, varID, levelID);
 
-            const auto &var1 = varList1.vars[varID];
+            auto const &var1 = varList1.vars[varID];
             auto gridsize = var1.gridsize;
 
             size_t numMissVals;
-            cdo_read_record(streamID1, array1.data(), &numMissVals);
+            cdo_read_field(streamID1, array1.data(), &numMissVals);
 
-            auto is_EQ = dbl_is_equal;
+            auto is_EQ = fp_is_equal;
             auto missval1 = var1.missval;
             auto missval2 = missval1;
 
@@ -140,8 +135,8 @@ public:
                   }
               }
 
-            cdo_write_record(streamID2, array2.data(), numMissVals);
-            cdo_write_record(streamID3, array3.data(), numMissVals);
+            cdo_write_field(streamID2, array2.data(), numMissVals);
+            cdo_write_field(streamID3, array3.data(), numMissVals);
           }
 
         tsID++;
