@@ -20,22 +20,26 @@
 
 template <typename T>
 static void
-operator_IFTHENC(size_t n, T mv, Varray<T> &v, T cVal)
+operator_IFTHENC(size_t n, double missval, Varray<T> &v, double value)
 {
+  T mv = missval;
+  T cVal = value;
   constexpr T nullVal{ 0 };
   if (std::isnan(mv))
-    for (size_t i = 0; i < n; ++i) v[i] = (dbl_is_not_equal(v[i], mv) && dbl_is_not_equal(v[i], nullVal)) ? cVal : mv;
+    for (size_t i = 0; i < n; ++i) v[i] = (fp_is_not_equal(v[i], mv) && fp_is_not_equal(v[i], nullVal)) ? cVal : mv;
   else
     for (size_t i = 0; i < n; ++i) v[i] = (!is_equal(v[i], mv) && !is_equal(v[i], nullVal)) ? cVal : mv;
 }
 
 template <typename T>
 static void
-operator_IFNOTTHENC(size_t n, T mv, Varray<T> &v, T cVal)
+operator_IFNOTTHENC(size_t n, double missval, Varray<T> &v, double value)
 {
+  T mv = missval;
+  T cVal = value;
   constexpr T nullVal{ 0 };
   if (std::isnan(mv))
-    for (size_t i = 0; i < n; ++i) v[i] = (dbl_is_not_equal(v[i], mv) && dbl_is_equal(v[i], nullVal)) ? cVal : mv;
+    for (size_t i = 0; i < n; ++i) v[i] = (fp_is_not_equal(v[i], mv) && fp_is_equal(v[i], nullVal)) ? cVal : mv;
   else
     for (size_t i = 0; i < n; ++i) v[i] = (!is_equal(v[i], mv) && is_equal(v[i], nullVal)) ? cVal : mv;
 }
@@ -56,10 +60,10 @@ public:
 
   int IFTHENC, IFNOTTHENC;
   CdoStreamID streamID1;
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
 
   CdoStreamID streamID2;
-  int taxisID2;
+  int taxisID2{ CDI_UNDEFID };
 
   int operatorID;
   double rconst;
@@ -103,45 +107,35 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            const auto &var = varList1.vars[varID];
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            auto const &var = varList1.vars[varID];
             field.init(var);
-            cdo_read_record(streamID1, field);
-
-            auto missval = field.missval;
-            auto ngp = field.size;
+            cdo_read_field(streamID1, field);
 
             auto numMissVals = field.numMissVals;
-            if (numMissVals > 0) cdo_check_missval(missval, var.name);
+            if (numMissVals > 0) cdo_check_missval(field.missval, var.name);
+
+            auto func_ifthenc = [&](auto &v, auto n, double mv) { operator_IFTHENC(n, mv, v, rconst); };
+            auto func_ifnotthenc = [&](auto &v, auto n, double mv) { operator_IFNOTTHENC(n, mv, v, rconst); };
 
             // clang-format off
-            if (field.memType == MemType::Float)
-              {
-                if      (operatorID == IFTHENC)    operator_IFTHENC(ngp, (float) missval, field.vec_f, (float) rconst);
-                else if (operatorID == IFNOTTHENC) operator_IFNOTTHENC(ngp, (float) missval, field.vec_f, (float) rconst);
-                else cdo_abort("Operator not implemented!");
-              }
-            else
-              {
-                if      (operatorID == IFTHENC)    operator_IFTHENC(ngp, missval, field.vec_d, rconst);
-                else if (operatorID == IFNOTTHENC) operator_IFNOTTHENC(ngp, missval, field.vec_d, rconst);
-                else cdo_abort("Operator not implemented!");
-              }
+            if      (operatorID == IFTHENC)    field_operation(func_ifthenc, field, field.size, field.missval);
+            else if (operatorID == IFNOTTHENC) field_operation(func_ifnotthenc, field, field.size, field.missval);
+            else cdo_abort("Operator not implemented!");
             // clang-format on
 
             if (numMissVals > 0) field_num_mv(field);
 
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, field);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, field);
           }
 
         tsID++;
