@@ -19,7 +19,7 @@
 #include "cdo_options.h"
 
 bool
-is_fdb_stream(const std::string &filename)
+is_fdb_stream(std::string const &filename)
 {
   return (filename.size() >= 4 && filename.starts_with("fdb:"));
 }
@@ -144,33 +144,33 @@ public:
         int tsID1 = 0;
         while (true)
           {
-            auto numRecords = cdo_stream_inq_timestep(streamID1, tsID1);
-            if (numRecords == 0) break;
+            auto numFields = cdo_stream_inq_timestep(streamID1, tsID1);
+            if (numFields == 0) break;
 
             cdo_taxis_copy_timestep(taxisID2, taxisID1);
             cdo_def_timestep(streamID2, tsID2);
 
-            for (int recID = 0; recID < numRecords; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                double fstatus = indf + ((numSteps >= 0) ? (tsID1 + (recID + 1.0) / numRecords) / numSteps : 1.0);
+                double fstatus = indf + ((numSteps >= 0) ? (tsID1 + (fieldID + 1.0) / numFields) / numSteps : 1.0);
                 progress.update(fstatus / numFiles);
 
-                auto [varID, levelID] = cdo_inq_record(streamID1);
+                auto [varID, levelID] = cdo_inq_field(streamID1);
 
-                const auto &var1 = varList1.vars[varID];
+                auto const &var1 = varList1.vars[varID];
                 if (hasConstantFields && tsID2 > 0 && tsID1 == 0 && var1.isConstant) continue;
 
-                cdo_def_record(streamID2, varID, levelID);
+                cdo_def_field(streamID2, varID, levelID);
 
                 if (dataIsUnchanged && (isFdbCopy || operatorID == CLONE || operatorID == SZIP))
                   {
-                    cdo_copy_record(streamID2, streamID1);
+                    cdo_copy_field(streamID2, streamID1);
                   }
                 else
                   {
                     field.init(var1);
-                    cdo_read_record(streamID1, field);
-                    cdo_write_record(streamID2, field);
+                    cdo_read_field(streamID1, field);
+                    cdo_write_field(streamID2, field);
                   }
               }
 
