@@ -10,7 +10,7 @@
 #include "dcw_reader.h"
 
 static void
-print_polygons(DCW_Lists &dcw_lists, const std::string &codeNames)
+print_polygons(DCW_Lists &dcw_lists, std::string const &codeNames)
 {
   auto codeList = split_string(codeNames, "\\+");
 
@@ -18,7 +18,7 @@ print_polygons(DCW_Lists &dcw_lists, const std::string &codeNames)
 
   printf("# Digital Chart of the World\n");
   printf("# Region for country:");
-  for (auto &code : codeList) printf(" %s", code.c_str());
+  for (auto const &code : codeList) printf(" %s", code.c_str());
   printf("\n");
 
   codeList = dcw_expand_code_list(dcw_lists, codeList);
@@ -32,16 +32,26 @@ print_polygons(DCW_Lists &dcw_lists, const std::string &codeNames)
   dcw_print_polygons(dcw_lists, codeList);
 }
 
-class ModuleDCW_util
+class DCW_util : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "DCW_util",
+    .operators = { { "dcw" } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 0, 0, NoRestriction },
+  };
+  inline static RegisterEntry<DCW_util> registration = RegisterEntry<DCW_util>(module);
   int argc = 0;
   DCW_Lists dcw_lists;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
 
     if (dcw_load_lists(dcw_lists)) cdo_abort("dcw_load_lists() failed!");
 
@@ -49,7 +59,7 @@ public:
   }
 
   void
-  run()
+  run() override
   {
 
     if (argc == 0) { cdo_abort("Parameter missing (available keywords are path/countries)"); }
@@ -59,35 +69,20 @@ public:
         const std::string paramCountry = "country=";
         const std::string paramDCW = "dcw:";
         auto &param = cdo_operator_argv(0);
-        if (param == "path" || param == "dir")
-          dcw_print_path();
-        else if (param == "countries")
-          dcw_print_countries(dcw_lists);
-        else if (param == "states")
-          dcw_print_states(dcw_lists);
-        else if (string_starts_with(param, paramCountry))
-          print_polygons(dcw_lists, param.substr(paramCountry.size()));
-        else if (string_starts_with(param, paramDCW))
-          print_polygons(dcw_lists, param.substr(paramDCW.size()));
+        // clang-format off
+        if (param == "path" || param == "dir")    dcw_print_path();
+        else if (param == "countries")            dcw_print_countries(dcw_lists);
+        else if (param == "states")               dcw_print_states(dcw_lists);
+        else if (param.starts_with(paramCountry)) print_polygons(dcw_lists, param.substr(paramCountry.size()));
+        else if (param.starts_with(paramDCW))     print_polygons(dcw_lists, param.substr(paramDCW.size()));
         else
           cdo_abort("Unsupported parameter: %s", param);
+        // clang-format on
       }
   }
 
   void
-  close()
+  close() override
   {
-    cdo_finish();
   }
 };
-
-void *
-DCW_util(void *process)
-{
-  ModuleDCW_util dcw_util;
-  dcw_util.init(process);
-  dcw_util.run();
-  dcw_util.close();
-
-  return nullptr;
-}
