@@ -18,7 +18,7 @@
 
 template <typename T>
 static void
-varray_deltat(const size_t len, const Varray<T> &v0, const Varray<T> &v1, Varray<T> &v2, double factor, double mv)
+varray_deltat(size_t len, Varray<T> const &v0, Varray<T> const &v1, Varray<T> &v2, double factor, double mv)
 {
   assert(len > 0);
   assert(v0.size() > 0);
@@ -30,7 +30,7 @@ varray_deltat(const size_t len, const Varray<T> &v0, const Varray<T> &v1, Varray
 
   for (size_t i = 0; i < len; ++i)
     {
-      if (DBL_IS_EQUAL(v0[i], mv) || DBL_IS_EQUAL(v1[i], mv))
+      if (fp_is_equal(v0[i], mv) || fp_is_equal(v1[i], mv))
         v2[i] = mv;
       else
         v2[i] = (v1[i] - v0[i]) * factor;
@@ -39,7 +39,7 @@ varray_deltat(const size_t len, const Varray<T> &v0, const Varray<T> &v1, Varray
 
 template <typename T>
 static void
-varray_deltat(const size_t len, const Varray<T> &v0, const Varray<T> &v1, Varray<T> &v2, double factor)
+varray_deltat(size_t len, Varray<T> const &v0, Varray<T> const &v1, Varray<T> &v2, double factor)
 {
   assert(len > 0);
 
@@ -47,7 +47,7 @@ varray_deltat(const size_t len, const Varray<T> &v0, const Varray<T> &v1, Varray
 }
 
 void
-field_deltat(const Field &field0, const Field &field1, Field &field2, double factor)
+field_deltat(Field const &field0, Field const &field1, Field &field2, double factor)
 {
   if (field1.memType != field2.memType) cdo_abort("Interal error, memType of field1 and field2 differ!");
 
@@ -87,14 +87,11 @@ private:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
-  int calendar;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
+  int calendar{ CDI_UNDEFID };
   VarList varList1;
-  FieldVector2D varsData;
-
-  Field field1, field2;
-  bool ldivdt;
+  bool ldivdt{ false };
 
 public:
   void
@@ -120,30 +117,32 @@ public:
 
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
-
-    field2D_init(varsData, vlistID1, FIELD_VEC | FIELD_NAT);
   }
+
   void
   run() override
   {
+    Field field1, field2;
+    FieldVector2D varsData;
+    field2D_init(varsData, varList1, FIELD_VEC | FIELD_NAT);
+
     int tsID = 0;
-    auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
+    auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
 
     auto julianDate0 = julianDate_encode(calendar, taxisInqVdatetime(taxisID1));
 
-    for (int recID = 0; recID < nrecs; ++recID)
+    for (int fieldID = 0; fieldID < numFields; ++fieldID)
       {
-        int varID, levelID;
-        cdo_inq_record(streamID1, &varID, &levelID);
-        cdo_read_record(streamID1, varsData[varID][levelID]);
+        auto [varID, levelID] = cdo_inq_field(streamID1);
+        cdo_read_field(streamID1, varsData[varID][levelID]);
       }
 
     tsID++;
     int tsID2 = 0;
     while (true)
       {
-        nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         auto julianDate1 = julianDate_encode(calendar, taxisInqVdatetime(taxisID1));
         auto idtInSec = ldivdt ? 1.0 / julianDate_to_seconds(julianDate_sub(julianDate1, julianDate0)) : 1.0;
@@ -152,13 +151,12 @@ public:
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID2);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            const auto &var1 = varList1.vars[varID];
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            auto const &var1 = varList1.vars[varID];
             field1.init(var1);
-            cdo_read_record(streamID1, field1);
+            cdo_read_field(streamID1, field1);
 
             auto &field0 = varsData[varID][levelID];
 
@@ -167,8 +165,8 @@ public:
 
             field_copy(field1, field0);
 
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, field2);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, field2);
           }
 
         tsID++;
