@@ -31,11 +31,11 @@ public:
   int dday, dmon;
 
   CdoStreamID streamID1;
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
 
   CdoStreamID streamID2;
-  int taxisID2;
-  int vlistID2;
+  int taxisID2{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
   int nfound = 0;
   bool dataIsUnchanged;
@@ -111,8 +111,8 @@ public:
     int tsID2 = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         auto vDateTime = taxisInqVdatetime(taxisID1);
 
@@ -132,17 +132,16 @@ public:
             cdo_taxis_copy_timestep(taxisID2, taxisID1);
             cdo_def_timestep(streamID2, tsID2++);
 
-            for (int recID = 0; recID < nrecs; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                int varID, levelID;
-                cdo_inq_record(streamID1, &varID, &levelID);
-                cdo_def_record(streamID2, varID, levelID);
-                if (dataIsUnchanged) { cdo_copy_record(streamID2, streamID1); }
+                auto [varID, levelID] = cdo_inq_field(streamID1);
+                cdo_def_field(streamID2, varID, levelID);
+                if (dataIsUnchanged) { cdo_copy_field(streamID2, streamID1); }
                 else
                   {
                     field.init(varList1.vars[varID]);
-                    cdo_read_record(streamID1, field);
-                    cdo_write_record(streamID2, field);
+                    cdo_read_field(streamID1, field);
+                    cdo_write_field(streamID2, field);
                   }
               }
           }
