@@ -14,10 +14,10 @@
 
 template <typename T>
 static void
-calc_full_depth(size_t gridsize, size_t nlevels, const Varray<T> &thick_c, const Varray<T> &stretch_c, const Varray<T> &zos,
+calc_full_depth(size_t gridsize, size_t nlevels, Varray<T> const &thick_c, Varray<T> const &stretch_c, Varray<T> const &zos,
                 Varray<T> &fullDepth)
 {
-  ranges::fill_n(fullDepth.begin(), gridsize, 0.0);
+  std::ranges::fill_n(fullDepth.begin(), gridsize, 0.0);
 
   for (size_t k = 1; k < nlevels; ++k)
     {
@@ -62,10 +62,10 @@ public:
   inline static RegisterEntry<Depth> registration = RegisterEntry<Depth>(module);
 
   CdoStreamID streamID1;
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
 
   CdoStreamID streamID2;
-  int taxisID2;
+  int taxisID2{ CDI_UNDEFID };
 
   int nlevels;
   int depthID;
@@ -121,8 +121,8 @@ public:
     auto zaxisID = varList1.vars[thickID].zaxisID;
     nlevels = varList1.vars[thickID].nlevels;
 
-    auto ngrids = vlistNumGrids(vlistID1);
-    if (ngrids > 1) cdo_abort("Too many different grids!");
+    auto numGrids = vlistNumGrids(vlistID1);
+    if (numGrids > 1) cdo_abort("Too many different grids!");
 
     int index = 0;
     auto gridID = vlistGrid(vlistID1, index);
@@ -153,18 +153,17 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             size_t numMissVals;
-            cdo_read_record(streamID1, vardata1[varID], levelID, &numMissVals);
+            cdo_read_field(streamID1, vardata1[varID], levelID, &numMissVals);
             if (numMissVals) cdo_abort("Missing values unsupported!");
           }
 
@@ -173,8 +172,8 @@ public:
 
         for (int levelID = 0; levelID < nlevels; ++levelID)
           {
-            cdo_def_record(streamID2, depthID, levelID);
-            cdo_write_record(streamID2, fullDepth, levelID, 0);
+            cdo_def_field(streamID2, depthID, levelID);
+            cdo_write_field(streamID2, fullDepth, levelID, 0);
           }
 
         tsID++;
