@@ -17,15 +17,17 @@
 
 #include <cdi.h>
 
-#include "cdo_task.h"
+#include "workerthread.h"
 #include "process_int.h"
 #include "mpmo_color.h"
+#include "cdo_math.h"
 #include "cdo_options.h"
 #include "printinfo.h"
 #include "cdo_zaxis.h"
 #include "param_conversion.h"
 #include "field_functions.h"
 #include "pmlist.h"
+#include "progress.h"
 
 struct DiffResult
 {
@@ -75,20 +77,20 @@ diff_kernel_mv(double v1, double v2, double missval1, double missval2, DiffResul
 }
 
 static DiffResult
-diff(size_t n, const Field &field1, const Field &field2)
+diff(size_t n, Field const &field1, Field const &field2)
 {
   DiffResult diffParam;
   auto hasMissvals = (field1.numMissVals || field2.numMissVals);
   if (hasMissvals)
     {
-      auto func = [&](const auto &v1, const auto &v2, double mv1, double mv2) {
+      auto func = [&](auto const &v1, auto const &v2, double mv1, double mv2) {
         for (size_t i = 0; i < n; ++i) { diff_kernel_mv(v1[i], v2[i], mv1, mv2, diffParam); }
       };
       field_operation2(func, field1, field2, field1.missval, field2.missval);
     }
   else
     {
-      auto func = [&](const auto &v1, const auto &v2) {
+      auto func = [&](auto const &v1, auto const &v2) {
         for (size_t i = 0; i < n; ++i) { diff_kernel(v1[i], v2[i], diffParam); }
       };
       field_operation2(func, field1, field2);
@@ -118,7 +120,7 @@ diff_kernel2(double v1, double v2, DiffResult &result)
 }
 
 static DiffResult
-diff2(size_t n, const Field &field1, const Field &field2)
+diff2(size_t n, Field const &field1, Field const &field2)
 {
   DiffResult diffParam;
   auto hasMissvals = (field1.numMissVals || field2.numMissVals);
@@ -133,7 +135,7 @@ diff2(size_t n, const Field &field1, const Field &field2)
     }
   else
     {
-      auto func = [&](const auto &v1, const auto &v2) {
+      auto func = [&](auto const &v1, auto const &v2) {
         for (size_t i = 0; i < n; ++i) { diff_kernel2(v1[i], v2[i], diffParam); }
       };
       field_operation2(func, field1, field2);
@@ -160,9 +162,9 @@ struct DiffParam
   double absLimit2{ 1.e-3 };
   double relLimit{ 1.0 };
   MapFlag mapFlag{ MapFlag::Undefined };
-  int maxDiffRecords{ INT_MAX };
-  int numDiffRecords{ 0 };
-  int numDiffRecords2{ 0 };
+  int maxDiffFields{ INT_MAX };
+  int numDiffFields{ 0 };
+  int numDiffFields2{ 0 };
   bool printHeader{ true };
 };
 
@@ -174,25 +176,25 @@ get_parameter(void)
   auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      const auto &pargv = cdo_get_oper_argv();
+      auto const &pargv = cdo_get_oper_argv();
 
       KVList kvlist;
       kvlist.name = cdo_module_name();
       if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-          const auto &value = kv.values[0];
+          auto const &value = kv.values[0];
 
           // clang-format off
           if      (key == "abslim")   params.absLimit = parameter_to_double(value);
           else if (key == "abslim2")  params.absLimit2 = parameter_to_double(value);
           else if (key == "rellim")   params.relLimit = parameter_to_double(value);
-          else if (key == "maxcount") params.maxDiffRecords = parameter_to_int(value);
+          else if (key == "maxcount") params.maxDiffFields = parameter_to_int(value);
           else if (key == "names")
             {
               if      (value == "left")      params.mapFlag = MapFlag::Left;
@@ -240,58 +242,81 @@ print_header2(int operfunc, DiffParam &params)
   fprintf(stdout, "\n");
 }
 
-static void
-compare_fields(const Field &field1, const Field &field2, int recordNumber, const CdoVar &var, int levelID, int operfunc,
-               CdiDateTime vDateTime, DiffParam &params, bool operfunc2)
+void
+print_diff(Field const &field1, Field const &field2, int fieldNumber, CdoVar const &var1, int levelID, int operfunc,
+           CdiDateTime const &vDateTime, DiffParam &params, int operfunc2, const DiffResult &dr)
 {
-  auto dr = (operfunc2 == 0) ? diff(var.gridsize, field1, field2) : diff2(var.gridsize, field1, field2);
+  if (params.printHeader) (operfunc2 == 0) ? print_header(operfunc, params) : print_header2(operfunc, params);
+
+  fprintf(stdout, "%6d ", fieldNumber);
+  fprintf(stdout, ":");
+
+  auto vdateString = date_to_string(vDateTime.date);
+  auto vtimeString = time_to_string(vDateTime.time);
+
+  set_text_color(stdout, MAGENTA);
+  fprintf(stdout, "%s %s ", vdateString.c_str(), vtimeString.c_str());
+  reset_text_color(stdout);
+  set_text_color(stdout, GREEN);
+  fprintf(stdout, "%7g ", cdo_zaxis_inq_level(var1.zaxisID, levelID));
+  fprintf(stdout, "%8zu %7zu ", var1.gridsize, std::max(field1.numMissVals, field2.numMissVals));
+  fprintf(stdout, "%7zu ", dr.ndiff);
+  reset_text_color(stdout);
+
+  fprintf(stdout, ":");
+  fprintf(stdout, " %c %c ", dr.dsgn ? 'T' : 'F', dr.zero ? 'T' : 'F');
+  set_text_color(stdout, BLUE);
+  fprintf(stdout, "%#12.5g%#12.5g", dr.absm, dr.relm);
+  reset_text_color(stdout);
+  fprintf(stdout, " : ");
+
+  char paramstr[32];
+  if (operfunc == Func_Param) cdiParamToString(var1.param, paramstr, sizeof(paramstr));
+
+  set_text_color(stdout, BRIGHT, GREEN);
+  // clang-format off
+          if      (operfunc == Func_Name)  fprintf(stdout, "%-11s", var1.name.c_str());
+          else if (operfunc == Func_Param) fprintf(stdout, "%-11s", paramstr);
+          else if (operfunc == Func_Code)  fprintf(stdout, "%4d", var1.code);
+  // clang-format on
+  reset_text_color(stdout);
+
+  fprintf(stdout, "\n");
+}
+
+static void
+compare_fields(Field &field1, Field &field2, int fieldNumber, CdoVar &var1, int levelID, int operfunc, CdiDateTime vDateTime,
+               DiffParam &params, int operfunc2, cdo::Progress &progress)
+{
+  size_t numNANs1 = (Options::fast || std::isnan(field1.missval)) ? 0 : field_num_NANs(field1);
+  var1.counter += numNANs1;
+  if (numNANs1 && field1.numMissVals == 0)
+    {
+      field1.missval = cdo::NaN();
+      field1.numMissVals = numNANs1;
+    }
+  size_t numNANs2 = (Options::fast || std::isnan(field2.missval)) ? 0 : field_num_NANs(field2);
+  var1.counter += numNANs2;
+  if (numNANs2 && field2.numMissVals == 0)
+    {
+      field2.missval = cdo::NaN();
+      field2.numMissVals = numNANs2;
+    }
+
+  auto dr = (operfunc2 == 0) ? diff(var1.gridsize, field1, field2) : diff2(var1.gridsize, field1, field2);
 
   auto checkRelativeLimit = true;
   if (!Options::silentMode || Options::cdoVerbose)
     {
       if (dr.absm > params.absLimit || (checkRelativeLimit && dr.relm >= params.relLimit) || Options::cdoVerbose)
         {
-          if (params.printHeader) (operfunc2 == 0) ? print_header(operfunc, params) : print_header2(operfunc, params);
-
-          fprintf(stdout, "%6d ", recordNumber);
-          fprintf(stdout, ":");
-
-          auto vdateString = date_to_string(vDateTime.date);
-          auto vtimeString = time_to_string(vDateTime.time);
-
-          set_text_color(stdout, MAGENTA);
-          fprintf(stdout, "%s %s ", vdateString.c_str(), vtimeString.c_str());
-          reset_text_color(stdout);
-          set_text_color(stdout, GREEN);
-          fprintf(stdout, "%7g ", cdo_zaxis_inq_level(var.zaxisID, levelID));
-          fprintf(stdout, "%8zu %7zu ", var.gridsize, std::max(field1.numMissVals, field2.numMissVals));
-          fprintf(stdout, "%7zu ", dr.ndiff);
-          reset_text_color(stdout);
-
-          fprintf(stdout, ":");
-          fprintf(stdout, " %c %c ", dr.dsgn ? 'T' : 'F', dr.zero ? 'T' : 'F');
-          set_text_color(stdout, BLUE);
-          fprintf(stdout, "%#12.5g%#12.5g", dr.absm, dr.relm);
-          reset_text_color(stdout);
-          fprintf(stdout, " : ");
-
-          char paramstr[32];
-          if (operfunc == Func_Param) cdiParamToString(var.param, paramstr, sizeof(paramstr));
-
-          set_text_color(stdout, BRIGHT, GREEN);
-          // clang-format off
-          if      (operfunc == Func_Name)  fprintf(stdout, "%-11s", var.name.c_str());
-          else if (operfunc == Func_Param) fprintf(stdout, "%-11s", paramstr);
-          else if (operfunc == Func_Code)  fprintf(stdout, "%4d", var.code);
-          // clang-format on
-          reset_text_color(stdout);
-
-          fprintf(stdout, "\n");
+          progress.update(1);
+          print_diff(field1, field2, fieldNumber, var1, levelID, operfunc, vDateTime, params, operfunc2, dr);
         }
     }
 
-  if (dr.absm > params.absLimit || (checkRelativeLimit && dr.relm >= params.relLimit)) params.numDiffRecords++;
-  if (dr.absm > params.absLimit2 || (checkRelativeLimit && dr.relm >= params.relLimit)) params.numDiffRecords2++;
+  if (dr.absm > params.absLimit || (checkRelativeLimit && dr.relm >= params.relLimit)) params.numDiffFields++;
+  if (dr.absm > params.absLimit2 || (checkRelativeLimit && dr.relm >= params.relLimit)) params.numDiffFields2++;
 }
 
 class Diff : public Process
@@ -301,11 +326,11 @@ public:
   inline static CdoModule module = {
     .name = "Diff",
     // clang-format off
-    .operators = { { "diff",  Func_Param, 0, DiffHelp },
-                   { "diffp", Func_Param, 0, DiffHelp },
-                   { "diffn", Func_Name,  0, DiffHelp },
-                   { "diffc", Func_Code,  0, DiffHelp },
-                   { "difftest", Func_Name, 1, DiffHelp } },
+    .operators = { { "diff",     Func_Param, 0, DiffHelp },
+                   { "diffp",    Func_Param, 0, DiffHelp },
+                   { "diffn",    Func_Name,  0, DiffHelp },
+                   { "diffc",    Func_Code,  0, DiffHelp },
+                   { "difftest", Func_Name,  1, DiffHelp } },
     // clang-format on
     .aliases = { { "diffv", "diffn" } },
     .mode = EXPOSED,     // Module mode: 0:intern 1:extern
@@ -333,6 +358,9 @@ public:
   void
   init() override
   {
+    if (Options::lazyGridLoad && this_is_the_only_process()) { cdiDefGlobal("NETCDF_LAZY_GRID_LOAD", true); }
+    if (this_is_the_only_process()) { cdiDefGlobal("READ_CELL_CORNERS", false); }
+
     auto operatorID = cdo_operator_id();
     operfunc = cdo_operator_f1(operatorID);
     operfunc2 = cdo_operator_f2(operatorID);
@@ -357,7 +385,7 @@ public:
     if (params.mapFlag == MapFlag::Undefined)
       {
         varList_compare(varList1, varList2);
-        for (const auto &var : varList1.vars) mapOfVarIDs[var.ID] = var.ID;
+        for (auto const &var : varList1.vars) mapOfVarIDs[var.ID] = var.ID;
       }
     else { varList_map(varList1, varList2, params.mapFlag, mapOfVarIDs); }
 
@@ -367,87 +395,93 @@ public:
   void
   run() override
   {
-    auto runAsync = (Options::CDO_Parallel_Read > 0);
-    auto task = runAsync ? std::make_unique<cdo::Task>() : nullptr;
+    auto runAsync = (Options::CDO_Async_Read > 0);
+    auto workerThread = runAsync ? std::make_unique<WorkerThread>() : nullptr;
     auto numTasks = runAsync ? 2 : 1;
 
     FieldVector fieldVector1(numTasks);
     FieldVector fieldVector2(numTasks);
 
+    auto numSteps1 = varList1.numSteps();
+    cdo::Progress progress(get_id());
+
     int numSets = 0;
-    int nrecs, nrecs2;
+    int numFields, numFields2;
     int tsID = 0;
     while (true)
       {
         auto stopRead = false;
 
-        nrecs = cdo_stream_inq_timestep(streamID1, tsID);
+        numFields = cdo_stream_inq_timestep(streamID1, tsID);
         auto vDateTime = taxisInqVdatetime(taxisID);
 
-        nrecs2 = cdo_stream_inq_timestep(streamID2, tsID);
+        numFields2 = cdo_stream_inq_timestep(streamID2, tsID);
 
-        if (nrecs == 0 || nrecs2 == 0) break;
+        if (numFields == 0 || numFields2 == 0) break;
 
-        int recID2next = 0;
+        if (numSteps1 > 1 && params.numDiffFields == 0) progress.update((tsID + 1.0) / numSteps1);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        int fieldID2next = 0;
+
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID1, levelID] = cdo_inq_record(streamID1);
+            auto [varID1, levelID] = cdo_inq_field(streamID1);
 
             auto it = mapOfVarIDs.find(varID1);
             if (it == mapOfVarIDs.end())
               {
                 if (params.mapFlag == MapFlag::Right || params.mapFlag == MapFlag::Intersect) continue;
-                cdo_abort("Internal problem (tsID=%d recID=%d): varID1=%d not found!", tsID + 1, recID + 1, varID1);
+                cdo_abort("Internal problem (tsID=%d fieldID=%d): varID1=%d not found!", tsID + 1, fieldID + 1, varID1);
               }
 
             int varID2 = 0;
-            for (; recID2next < nrecs2; ++recID2next)
+            for (; fieldID2next < numFields2; ++fieldID2next)
               {
-                cdo_inq_record(streamID2, &varID2, &levelID);
+                auto [varID2x, levelID2x] = cdo_inq_field(streamID2);
+                varID2 = varID2x;
                 if (it->second == varID2)
                   {
-                    ++recID2next;
+                    ++fieldID2next;
                     break;
                   }
               }
 
-            if (it->second != varID2 && recID2next == nrecs2)
-              cdo_abort("Internal problem (tsID=%d recID=%d): varID2=%d not found in second stream!", tsID + 1, recID + 1,
+            if (it->second != varID2 && fieldID2next == numFields2)
+              cdo_abort("Internal problem (tsID=%d fieldID=%d): varID2=%d not found in second stream!", tsID + 1, fieldID + 1,
                         it->second);
 
-            const auto &var1 = varList1.vars[varID1];
-            const auto &var2 = varList2.vars[varID2];
+            auto &var1 = varList1.vars[varID1];
+            auto &var2 = varList2.vars[varID2];
 
             auto taskNum = numSets % numTasks;
             auto &field1 = fieldVector1[taskNum];
             auto &field2 = fieldVector2[taskNum];
 
             field1.init(var1);
-            cdo_read_record(streamID1, field1);
+            cdo_read_field(streamID1, field1);
             if (var1.nwpv == CDI_COMP) use_real_part(field1);
 
             field2.init(var2);
-            cdo_read_record(streamID2, field2);
+            cdo_read_field(streamID2, field2);
             if (var2.nwpv == CDI_COMP) use_real_part(field2);
 
             if (runAsync && numSets > 0)
               {
-                task->wait();
+                workerThread->wait();
                 // clang-format off
-                if (params.numDiffRecords >= params.maxDiffRecords) { stopRead = true; break; }
+                if (params.numDiffFields >= params.maxDiffFields) { stopRead = true; break; }
                 // clang-format on
               }
 
-            std::function<void()> compare_fields_func
-                = std::bind(compare_fields, std::cref(field1), std::cref(field2), numSets + 1, std::cref(var1), levelID, operfunc,
-                            vDateTime, std::ref(params), operfunc2);
-            runAsync ? task->doAsync(compare_fields_func) : compare_fields_func();
+            std::function<void()> compare_fields_task
+                = std::bind(compare_fields, std::ref(field1), std::ref(field2), numSets + 1, std::ref(var1), levelID, operfunc,
+                            vDateTime, std::ref(params), operfunc2, std::ref(progress));
+            runAsync ? workerThread->doAsync(compare_fields_task) : compare_fields_task();
 
             if (not runAsync)
               {
                 // clang-format off
-                if (params.numDiffRecords >= params.maxDiffRecords) { stopRead = true; break; }
+                if (params.numDiffFields >= params.maxDiffFields) { stopRead = true; break; }
                 // clang-format on
               }
 
@@ -459,24 +493,34 @@ public:
         tsID++;
       }
 
-    if (runAsync) task->wait();
+    if (runAsync) workerThread->wait();
 
-    if (params.numDiffRecords > 0)
+    if (params.numDiffFields > 0)
       {
         Options::cdoExitStatus = 1;
 
         set_text_color(stdout, BRIGHT, RED);
-        fprintf(stdout, "  %d of %d records differ", params.numDiffRecords, numSets);
+        fprintf(stdout, "  %d of %d fields differ", params.numDiffFields, numSets);
         reset_text_color(stdout);
         fprintf(stdout, "\n");
 
-        if (params.numDiffRecords != params.numDiffRecords2 && params.absLimit < params.absLimit2)
-          fprintf(stdout, "  %d of %d records differ more than %g\n", params.numDiffRecords2, numSets, params.absLimit2);
-        //  fprintf(stdout, "  %d of %d records differ more then one thousandth\n", nprec, ngrec);
+        if (params.numDiffFields != params.numDiffFields2 && params.absLimit < params.absLimit2)
+          fprintf(stdout, "  %d of %d fields differ more than %g\n", params.numDiffFields2, numSets, params.absLimit2);
+        //  fprintf(stdout, "  %d of %d fields differ more then one thousandth\n", nprec, ngrec);
       }
 
-    if (nrecs == 0 && nrecs2 > 0) cdo_warning("stream2 has more time steps than stream1!");
-    if (nrecs > 0 && nrecs2 == 0) cdo_warning("stream1 has more time steps than stream2!");
+    if (numFields == 0 && numFields2 > 0) cdo_warning("stream2 has more time steps than stream1!");
+    if (numFields > 0 && numFields2 == 0) cdo_warning("stream1 has more time steps than stream2!");
+
+    for (auto const &var : varList1.vars)
+      {
+        if (var.counter > 0)
+          {
+            cdo_warning("%s contains %zu NaNs which are not treated as missing values. "
+                        "This can lead to incorrect CDO results in all other arithmetic functions!",
+                        var.name, var.counter);
+          }
+      }
   }
 
   void
