@@ -9,32 +9,42 @@
 
 #include "cdo_options.h"
 #include "process_int.h"
-#include "cdo_vlist.h"
 #include "param_conversion.h"
 #include "field_functions.h"
 
-class ModuleDuplicate
+class Duplicate : public Process
 {
-  FieldVector3D vars;
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Duplicate",
+    .operators = { { "duplicate", DuplicateHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Duplicate> registration = RegisterEntry<Duplicate>(module);
+
+  FieldVector3D varsData;
   std::vector<CdiDateTime> vDateTimes;
 
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
-  int vlistID1;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
+  int vlistID1{ CDI_UNDEFID };
 
   int nvars;
   int ndup;
 
-  VarList varList;
+  VarList varList1;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
 
     if (cdo_operator_argc() > 1) cdo_abort("Too many arguments!");
 
@@ -50,12 +60,12 @@ public:
     taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
 
-    varList_init(varList, vlistID1);
+    varList1 = VarList(vlistID1);
 
     nvars = vlistNvars(vlistID1);
     auto ntsteps = vlistNtsteps(vlistID1);
 
-    if (ntsteps == 1 && varList_numVaryingVars(varList) == 0) ntsteps = 0;
+    if (ntsteps == 1 && varList1.numVaryingVars() == 0) ntsteps = 0;
 
     if (ntsteps == 0)
       {
@@ -71,29 +81,28 @@ public:
   }
 
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         constexpr size_t NALLOC_INC = 1024;
         if ((size_t) tsID >= vDateTimes.size()) vDateTimes.resize(vDateTimes.size() + NALLOC_INC);
-        if ((size_t) tsID >= vars.size()) vars.resize(vars.size() + NALLOC_INC);
+        if ((size_t) tsID >= varsData.size()) varsData.resize(varsData.size() + NALLOC_INC);
 
         vDateTimes[tsID] = taxisInqVdatetime(taxisID1);
 
-        fields_from_vlist(vlistID1, vars[tsID]);
+        field2D_init(varsData[tsID], varList1);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            auto &field = vars[tsID][varID][levelID];
-            field.init(varList[varID]);
-            cdo_read_record(streamID1, field);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            auto &field = varsData[tsID][varID][levelID];
+            field.init(varList1.vars[varID]);
+            cdo_read_field(streamID1, field);
           }
 
         tsID++;
@@ -111,38 +120,27 @@ public:
 
             for (int varID = 0; varID < nvars; ++varID)
               {
-                fieldOut.init(varList[varID]);
-                for (int levelID = 0; levelID < varList[varID].nlevels; ++levelID)
+                auto const &var1 = varList1.vars[varID];
+                fieldOut.init(var1);
+                for (int levelID = 0; levelID < var1.nlevels; ++levelID)
                   {
-                    const auto &field = vars[tsID][varID][levelID];
+                    auto const &field = varsData[tsID][varID][levelID];
                     if (field.hasData())
                       {
                         field_copy(field, fieldOut);
-                        cdo_def_record(streamID2, varID, levelID);
-                        cdo_write_record(streamID2, fieldOut);
+                        cdo_def_field(streamID2, varID, levelID);
+                        cdo_write_field(streamID2, fieldOut);
                       }
                   }
               }
           }
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-    cdo_finish();
   }
 };
-
-void *
-Duplicate(void *process)
-{
-  ModuleDuplicate dup;
-
-  dup.init(process);
-  dup.run();
-  dup.close();
-
-  return nullptr;
-}
