@@ -35,29 +35,28 @@
 // No missing value support added so far!
 
 static void
-scale_eigvec_grid(Varray<double> &out, int tsID, size_t npack, const std::vector<size_t> &pack, const Varray<double> &weight,
-                  const Varray2D<double> &covar, double sum_w)
+scale_eigvec_grid(Varray<double> &out, int tsID, size_t npack, const std::vector<size_t> &pack, Varray<double> const &weight,
+                  const Varray2D<double> &covar, double sumWeights)
 {
-  for (size_t i = 0; i < npack; ++i) out[pack[i]] = covar[tsID][i] / std::sqrt(weight[pack[i]] / sum_w);
+  for (size_t i = 0; i < npack; ++i) out[pack[i]] = covar[tsID][i] / std::sqrt(weight[pack[i]] / sumWeights);
 }
 
 static void
-scale_eigvec_time(Varray<double> &out, int tsID, int nts, size_t npack, const std::vector<size_t> &pack,
-                  const Varray<double> &weight, const Varray2D<double> &covar, const Varray2D<double> &data, double missval,
-                  double sum_w)
+scale_eigvec_time(Varray<double> &out, int tsID, int numSteps, size_t npack, const std::vector<size_t> &pack,
+                  Varray<double> const &weight, const Varray2D<double> &covar, const Varray2D<double> &data, double missval,
+                  double sumWeights)
 {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(npack, nts, tsID, pack, data, covar, out)
+#pragma omp parallel for default(none) shared(npack, numSteps, tsID, pack, data, covar, out)
 #endif
   for (size_t i = 0; i < npack; ++i)
     {
       double sum = 0.0;
-      for (int j = 0; j < nts; ++j) sum += data[j][i] * covar[tsID][j];
-
+      for (int j = 0; j < numSteps; ++j) sum += data[j][i] * covar[tsID][j];
       out[pack[i]] = sum;
     }
   /*
-  for ( size_t j = 0; j < nts; ++j )
+  for ( size_t j = 0; j < numSteps; ++j )
     {
       for ( size_t i = 0; i < npack; ++i )
         out[pack[i]] += data[j][i] * covar[tsID][j];
@@ -78,7 +77,7 @@ scale_eigvec_time(Varray<double> &out, int tsID, int nts, size_t npack, const st
 
   if (sum > 0.0)
     {
-      sum = std::sqrt(sum / sum_w);
+      sum = std::sqrt(sum / sumWeights);
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(npack, pack, sum, out)
 #endif
@@ -118,15 +117,13 @@ public:
   {
     bool init = false;
     bool first_call = true;
-    Varray<double> eig_val;
+    Varray<double> eigenValues;
     Varray2D<double> covar;
     Varray2D<double> data;
   };
 
   size_t numMissVals = 0;
-  int varID, levelID;
-  int nts = 1;
-  int grid_space = 0, time_space = 0;
+  int gridSpace = 0, timeSpace = 0;
   double sum_w = 1.0;
 
   CdoStreamID streamID1;
@@ -137,27 +134,27 @@ public:
   int vlistID2 = -1;
   int vlistID3 = -1;
 
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
 
   int gridID2;
 
   int calendar = CALENDAR_STANDARD;
 
-  T_WEIGHT_MODE weight_mode;
-  T_EIGEN_MODE eigen_mode;
+  T_WEIGHT_MODE weightMode;
+  T_EIGEN_MODE eigenMode;
 
-  size_t num_eigen_functions = 0;
+  size_t numEigenFunctions = 0;
 
   int numVars;
-  int n_eig;
-  int ngrids;
+  int numEigen;
+  int numGrids;
+  int numSteps = 1;
 
   size_t npack = SIZE_MAX;
   VarList varList1;
   std::vector<size_t> pack;
   Varray<double> in;
-  std::vector<std::vector<eofdata_t>> eofdata;
-
+  std::vector<std::vector<eofdata_t>> eofData2D;
   Varray<double> weights;
 
 public:
@@ -168,10 +165,10 @@ public:
     auto operfunc = cdo_operator_f1(operatorID);
 
     operator_input_arg("Number of eigen functions to write out");
-    n_eig = parameter_to_int(cdo_operator_argv(0));
+    numEigen = parameter_to_int(cdo_operator_argv(0));
 
-    eigen_mode = get_eigenmode();
-    weight_mode = get_weightmode();
+    eigenMode = get_eigenmode();
+    weightMode = get_weightmode();
 
     streamID1 = cdo_open_read(0);
     vlistID1 = cdo_stream_inq_vlist(streamID1);
@@ -180,11 +177,11 @@ public:
     varList1 = VarList(vlistID1);
 
     auto gridID1 = varList1.vars[0].gridID;
-    auto gridsizemax = vlistGridsizeMax(vlistID1);
+    auto gridsizeMax = varList1.gridsizeMax();
     numVars = varList1.numVars();
 
-    ngrids = vlistNumGrids(vlistID1);
-    for (int index = 1; index < ngrids; ++index)
+    numGrids = vlistNumGrids(vlistID1);
+    for (int index = 1; index < numGrids; ++index)
       if (vlistGrid(vlistID1, 0) != vlistGrid(vlistID1, index)) cdo_abort("Too many different grids!");
 
     // eigenvalues
@@ -192,16 +189,14 @@ public:
     // Count number of timesteps if EOF_ or EOF_TIME
     if (operfunc == EOF_ || operfunc == EOF_TIME)
       {
-        if (Options::cdoVerbose) cdo_print("Counting timesteps in ifile");
-
-        nts = vlistNtsteps(vlistID1);
-        if (nts == 0) nts = 1;
-        if (nts == -1)
+        numSteps = vlistNtsteps(vlistID1);
+        if (numSteps == 0) numSteps = 1;
+        if (numSteps == -1)
           {
-            nts = 0;
-            while (cdo_stream_inq_timestep(streamID1, nts)) nts++;
+            numSteps = 0;
+            while (cdo_stream_inq_timestep(streamID1, numSteps)) numSteps++;
 
-            if (Options::cdoVerbose) cdo_print("Counted %d timeSteps", nts);
+            if (Options::cdoVerbose) cdo_print("Counted %d timeSteps", numSteps);
 
             cdo_stream_close(streamID1);
 
@@ -209,86 +204,87 @@ public:
             vlistID1 = cdo_stream_inq_vlist(streamID1);
             taxisID1 = vlistInqTaxis(vlistID1);
           }
-        else if (Options::cdoVerbose)
-          cdo_print("Found %d timeSteps", nts);
+        else if (Options::cdoVerbose) { cdo_print("Found %d timeSteps", numSteps); }
 
-        if ((size_t) nts < gridsizemax || operfunc == EOF_TIME)
+        if ((size_t) numSteps < gridsizeMax || operfunc == EOF_TIME)
           {
-            time_space = 1;
-            grid_space = 0;
+            timeSpace = 1;
+            gridSpace = 0;
           }
         else
           {
-            time_space = 0;
-            grid_space = 1;
+            timeSpace = 0;
+            gridSpace = 1;
           }
       }
     else if (operfunc == EOF_SPATIAL)
       {
-        time_space = 0;
-        grid_space = 1;
+        timeSpace = 0;
+        gridSpace = 1;
       }
 
     // reset the requested number of eigen-function to the maximum if neccessary
-    if (time_space)
+    if (timeSpace)
       {
-        if (n_eig > nts)
+        if (numEigen > numSteps)
           {
             cdo_warning("Solving in time-space:");
             cdo_warning("Number of eigen-functions to write out is bigger than number of time-steps.");
-            cdo_warning("Setting n_eig to %d.", nts);
+            cdo_warning("Setting numEigen to %d.", numSteps);
             cdo_warning("If You want to force a solution in grid-space use operator eofspatial");
-            n_eig = nts;
+            numEigen = numSteps;
           }
-        num_eigen_functions = nts;
+        numEigenFunctions = numSteps;
       }
-    else if (grid_space)
+    else if (gridSpace)
       {
-        if (((double) gridsizemax) * gridsizemax > (double) SIZE_MAX) cdo_abort("Grid space too large!");
+        if (((double) gridsizeMax) * gridsizeMax > (double) SIZE_MAX) cdo_abort("Grid space too large!");
 
-        if ((size_t) n_eig > gridsizemax)
+        if ((size_t) numEigen > gridsizeMax)
           {
             cdo_warning("Solving in spatial space");
             cdo_warning("Number of eigen-functions to write out is bigger than grid size");
-            cdo_warning("Setting n_eig to %zu", gridsizemax);
+            cdo_warning("Setting numEigen to %zu", gridsizeMax);
             cdo_warning("If You want to force a solution in time-space use operator eoftime");
-            n_eig = gridsizemax;
+            numEigen = gridsizeMax;
           }
-        num_eigen_functions = gridsizemax;
+        numEigenFunctions = gridsizeMax;
       }
 
     if (Options::cdoVerbose)
-      cdo_print("Calculating %d eigenvectors and %zu eigenvalues in %s", n_eig, num_eigen_functions,
-                (grid_space == 1) ? "grid_space" : "time_space");
+      cdo_print("Calculating %d eigenvectors and %zu eigenvalues in %s", numEigen, numEigenFunctions,
+                (gridSpace == 1) ? "grid_space" : "time_space");
 
-    weights = Varray<double>(gridsizemax, 1.0);
+    weights = Varray<double>(gridsizeMax, 1.0);
 
-    if (weight_mode == WEIGHT_ON)
+    if (weightMode == WEIGHT_ON)
       {
         auto wstatus = gridcell_weights(gridID1, weights);
         if (wstatus != 0)
           {
-            weight_mode = WEIGHT_OFF;
+            weightMode = WEIGHT_OFF;
             cdo_warning("Using constant grid cell area weights!");
           }
       }
 
     // allocation of temporary fields and output structures
 
-    pack = std::vector<size_t>(gridsizemax);
-    in = Varray<double>(gridsizemax);
-    eofdata = std::vector<std::vector<eofdata_t>>(numVars);
+    pack = std::vector<size_t>(gridsizeMax);
+    in = Varray<double>(gridsizeMax);
+    eofData2D.resize(numVars);
 
-    for (varID = 0; varID < numVars; ++varID)
+    if (Options::cdoVerbose)
+      cdo_print("Allocate eigenvalue/eigenvector object with numSteps=%d and gridSize=%zu (%zu Bytes)", numSteps, gridsizeMax,
+                numSteps * gridsizeMax * 8);
+
+    for (int varID = 0; varID < numVars; ++varID)
       {
-        auto nlevs = varList1.vars[varID].nlevels;
-        eofdata[varID].resize(nlevs);
+        auto numLevels = varList1.vars[varID].nlevels;
+        eofData2D[varID].resize(numLevels);
 
-        if (time_space)
-          for (levelID = 0; levelID < nlevs; ++levelID) eofdata[varID][levelID].data.resize(nts);
+        if (timeSpace)
+          for (int levelID = 0; levelID < numLevels; ++levelID) eofData2D[varID][levelID].data.resize(numSteps);
       }
-
-    if (Options::cdoVerbose) cdo_print("Allocated eigenvalue/eigenvector structures with nts=%d gridsize=%zu", nts, gridsizemax);
   }
 
   void
@@ -299,30 +295,30 @@ public:
     // read the data and create covariance matrices for each var & level
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            cdo_inq_record(streamID1, &varID, &levelID);
-            cdo_read_record(streamID1, in.data(), &numMissVals);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            cdo_read_field(streamID1, in.data(), &numMissVals);
 
-            auto gridsize = varList1.vars[varID].gridsize;
-            auto missval = varList1.vars[varID].missval;
+            auto const &var1 = varList1.vars[varID];
 
             if (npack == SIZE_MAX)
               {
                 npack = 0;
-                for (size_t i = 0; i < gridsize; ++i)
+                for (size_t i = 0; i < var1.gridsize; ++i)
                   {
-                    if (!fp_is_equal(weights[i], 0.0) && !fp_is_equal(weights[i], missval) && !fp_is_equal(in[i], missval))
+                    if (!fp_is_equal(weights[i], 0.0) && !fp_is_equal(weights[i], var1.missval)
+                        && !fp_is_equal(in[i], var1.missval))
                       {
                         pack[npack] = i;
                         npack++;
                       }
                   }
 
-                if (weight_mode == WEIGHT_ON)
+                if (weightMode == WEIGHT_ON)
                   {
                     sum_w = 0.0;
                     for (size_t i = 0; i < npack; ++i) sum_w += weights[pack[i]];
@@ -331,9 +327,9 @@ public:
 
             {
               size_t ipack = 0;
-              for (size_t i = 0; i < gridsize; ++i)
+              for (size_t i = 0; i < var1.gridsize; ++i)
                 {
-                  if (!fp_is_equal(weights[i], 0.0) && !fp_is_equal(weights[i], missval) && !fp_is_equal(in[i], missval))
+                  if (!fp_is_equal(weights[i], 0.0) && !fp_is_equal(weights[i], var1.missval) && !fp_is_equal(in[i], var1.missval))
                     {
                       if (pack[ipack] != i) cdo_abort("Missing values unsupported!");
                       ipack++;
@@ -342,15 +338,19 @@ public:
               if (ipack != npack) cdo_abort("Missing values unsupported!");
             }
 
-            if (grid_space)
+            auto &eofData = eofData2D[varID][levelID];
+            if (gridSpace)
               {
-                if (!eofdata[varID][levelID].init)
+                if (!eofData.init)
                   {
-                    eofdata[varID][levelID].covar.resize(npack);
-                    for (size_t i = 0; i < npack; ++i) eofdata[varID][levelID].covar[i].resize(npack, 0.0);
+                    if (Options::cdoVerbose)
+                      cdo_print("Allocate covar-matrix with %zux%zu elements for %s [layer: %d] (%zu Bytes)", npack, npack,
+                                var1.name, levelID + 1, npack * npack * 8);
+                    eofData.covar.resize(npack);
+                    for (size_t i = 0; i < npack; ++i) eofData.covar[i].resize(npack, 0.0);
                   }
 
-                auto &covar = eofdata[varID][levelID].covar;
+                auto &covar = eofData.covar;
 #ifdef _OPENMP
 #pragma omp parallel for default(none) shared(npack, covar, in, pack)
 #endif
@@ -361,21 +361,21 @@ public:
                     for (size_t jpack = ipack; jpack < npack; ++jpack) covar_i[jpack] += in_i * in[pack[jpack]];
                   }
               }
-            else if (time_space)
+            else if (timeSpace)
               {
-                eofdata[varID][levelID].data[tsID].resize(npack);
-                auto &data = eofdata[varID][levelID].data[tsID];
+                eofData.data[tsID].resize(npack);
+                auto &data = eofData.data[tsID];
 
                 for (size_t ipack = 0; ipack < npack; ipack++) data[ipack] = in[pack[ipack]];
               }
 
-            eofdata[varID][levelID].init = true;
+            eofData.init = true;
           }
 
         tsID++;
       }
 
-    if (grid_space) nts = tsID;
+    if (gridSpace) numSteps = tsID;
 
     if (tsID == 1) cdo_abort("File consists of only one timestep!");
 
@@ -384,10 +384,12 @@ public:
     // eigenvalues
     streamID2 = cdo_open_write(1);
 
+    auto vDateTime0 = cdiDateTime_set(10101, 0);
+
     vlistID2 = vlistDuplicate(vlistID1);
     auto taxisID2 = taxisDuplicate(taxisID1);
-    taxisDefRdate(taxisID2, 0);
-    taxisDefRtime(taxisID2, 0);
+    cdiDefKeyInt(taxisID2, CDI_GLOBAL, CDI_KEY_DATATYPE, CDI_DATATYPE_FLT64);
+    taxisDefRdatetime(taxisID2, vDateTime0);
     vlistDefTaxis(vlistID2, taxisID2);
 
     gridID2 = gridCreate(GRID_LONLAT, 1);
@@ -396,72 +398,63 @@ public:
     double xvals = 0.0, yvals = 0.0;
     gridDefXvals(gridID2, &xvals);
     gridDefYvals(gridID2, &yvals);
-    for (int i = 0; i < ngrids; ++i) vlistChangeGridIndex(vlistID2, i, gridID2);
+    for (int i = 0; i < numGrids; ++i) vlistChangeGridIndex(vlistID2, i, gridID2);
 
     // eigenvectors
     streamID3 = cdo_open_write(2);
 
     vlistID3 = vlistDuplicate(vlistID1);
     auto taxisID3 = taxisDuplicate(taxisID1);
-    taxisDefRdate(taxisID3, 0);
-    taxisDefRtime(taxisID3, 0);
+    cdiDefKeyInt(taxisID3, CDI_GLOBAL, CDI_KEY_DATATYPE, CDI_DATATYPE_FLT64);
+    taxisDefRdatetime(taxisID3, vDateTime0);
     vlistDefTaxis(vlistID3, taxisID3);
 
     cdo_def_vlist(streamID2, vlistID2);
     cdo_def_vlist(streamID3, vlistID3);
 
-    int64_t vdate = 10101;
-    int vtime = 0;
-    CdiDateTime vDateTime = cdiDateTime_set(vdate, vtime);
-    auto julianDate = julianDate_encode(calendar, vDateTime);
+    auto julianDate = julianDate_encode(calendar, vDateTime0);
 
     Varray<double> &out = in;
 
-    int nts_out = (npack < (size_t) nts) ? npack : nts;
+    int numStepsOut = (npack < (size_t) numSteps) ? npack : numSteps;
 
-    for (tsID = 0; tsID < nts_out; ++tsID)
+    for (tsID = 0; tsID < numStepsOut; ++tsID)
       {
         julianDate = julianDate_add_seconds(julianDate, 60);
-        vDateTime = julianDate_decode(calendar, julianDate);
+        vDateTime0 = julianDate_decode(calendar, julianDate);
 
-        taxisDefVdatetime(taxisID2, vDateTime);
+        taxisDefVdatetime(taxisID2, vDateTime0);
         cdo_def_timestep(streamID2, tsID);
 
-        if (tsID < n_eig)
+        if (tsID < numEigen)
           {
-            taxisDefVdatetime(taxisID3, vDateTime);
+            taxisDefVdatetime(taxisID3, vDateTime0);
             cdo_def_timestep(streamID3, tsID);
           }
 
-        for (varID = 0; varID < numVars; ++varID)
+        for (int varID = 0; varID < numVars; ++varID)
           {
-            const auto &var1 = varList1.vars[varID];
-            auto &vname = var1.name;
-            auto gridsize = var1.gridsize;
-            auto nlevs = var1.nlevels;
-            auto missval = var1.missval;
+            auto const &var1 = varList1.vars[varID];
 
-            for (levelID = 0; levelID < nlevs; ++levelID)
+            for (int levelID = 0; levelID < var1.nlevels; ++levelID)
               {
-                const auto &data = eofdata[varID][levelID].data;
-                auto &covar = eofdata[varID][levelID].covar;
+                auto &eofData = eofData2D[varID][levelID];
+                auto const &data = eofData.data;
+                auto &covar = eofData.covar;
 
-                if (eofdata[varID][levelID].first_call)
+                if (eofData.first_call)
                   {
-                    eofdata[varID][levelID].first_call = false;
+                    eofData.first_call = false;
 
-                    if (Options::cdoVerbose)
-                      cdo_print("Calculating covar matrices for %d levels of var%i (%s)", nlevs, varID + 1, vname);
-
-                    if (Options::cdoVerbose) cdo_print("processing level %d", levelID + 1);
-
-                    if (grid_space)
+                    if (gridSpace)
                       {
-                        if (npack) eofdata[varID][levelID].eig_val.resize(npack);
+                        if (Options::cdoVerbose) cdo_print("Processing level %d of %s", levelID + 1, var1.name);
+
+                        eofData.eigenValues.resize(npack);
 
                         for (size_t ipack = 0; ipack < npack; ++ipack)
                           {
-                            size_t i = pack[ipack];
+                            auto i = pack[ipack];
                             for (size_t jpack = 0; jpack < npack; ++jpack)
                               {
                                 if (jpack < ipack) { covar[ipack][jpack] = covar[jpack][ipack]; }
@@ -470,73 +463,73 @@ public:
                                     auto j = pack[jpack];
                                     covar[ipack][jpack] = covar[ipack][jpack] *                                    // covariance
                                                           std::sqrt(weights[i]) * std::sqrt(weights[j]) / sum_w /  // weights
-                                                          nts;  // number of data contributing
+                                                          numSteps;  // number of data contributing
                                   }
                               }
                           }
                       }
-                    else if (time_space)
+                    else if (timeSpace)
                       {
-                        if (Options::cdoVerbose) cdo_print("allocating covar with %dx%d elements | npack=%zu", nts, nts, npack);
+                        if (Options::cdoVerbose)
+                          cdo_print("Allocate covar-matrix with %dx%d elements (npack=%zu) for %s [layer: %d] (%zu Bytes)",
+                                    numSteps, numSteps, npack, var1.name, levelID + 1, numSteps * numSteps * sizeof(double));
 
-                        covar.resize(nts);
-                        for (int i = 0; i < nts; ++i) covar[i].resize(nts);
+                        covar.resize(numSteps);
+                        for (int i = 0; i < numSteps; ++i) covar[i].resize(numSteps);
 
-                        eofdata[varID][levelID].eig_val.resize(nts);
+                        eofData.eigenValues.resize(numSteps);
 
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(nts, data, covar, weights, npack, pack, sum_w) schedule(static)
+#pragma omp parallel for default(none) shared(numSteps, data, covar, weights, npack, pack, sum_w) schedule(static)
 #endif
-                        for (int j1 = 0; j1 < nts; ++j1)
+                        for (int j1 = 0; j1 < numSteps; ++j1)
                           {
-                            const auto &df1p = data[j1];
-                            for (int j2 = j1; j2 < nts; ++j2)
+                            auto const &df1p = data[j1];
+                            for (int j2 = j1; j2 < numSteps; ++j2)
                               {
-                                const auto &df2p = data[j2];
+                                auto const &df2p = data[j2];
                                 double sum = 0.0;
                                 for (size_t i = 0; i < npack; ++i) sum += weights[pack[i]] * df1p[i] * df2p[i];
-                                covar[j2][j1] = covar[j1][j2] = sum / sum_w / nts;
+                                covar[j2][j1] = covar[j1][j2] = sum / sum_w / numSteps;
                               }
                           }
-
-                        if (Options::cdoVerbose) cdo_print("finished calculation of covar-matrix for var %s", vname);
                       }
 
                     // Solve the eigen problem
-                    auto &eig_val = eofdata[varID][levelID].eig_val;
-                    if (eigen_mode == JACOBI)
+                    auto &eigenValues = eofData.eigenValues;
+                    if (eigenMode == JACOBI)
                       // TODO: use return status (>0 okay, -1 did not converge at all)
-                      parallel_eigen_solution_of_symmetric_matrix(covar, eig_val, num_eigen_functions, __func__);
+                      parallel_eigen_solution_of_symmetric_matrix(covar, eigenValues, numEigenFunctions, __func__);
                     else
-                      eigen_solution_of_symmetric_matrix(covar, eig_val, num_eigen_functions, __func__);
+                      eigen_solution_of_symmetric_matrix(covar, eigenValues, numEigenFunctions, __func__);
 
                     // NOW: covar contains the eigenvectors, eig_val the eigenvalues
 
-                    for (size_t i = 0; i < gridsize; ++i) out[i] = missval;
+                    for (size_t i = 0; i < var1.gridsize; ++i) out[i] = var1.missval;
 
                     // for ( int i = 0; i < n; i++ ) eig_val[i] *= sum_w;
                   }  // first_call
 
-                if (tsID < n_eig)
+                if (tsID < numEigen)
                   {
-                    if (grid_space)
+                    if (gridSpace)
                       scale_eigvec_grid(out, tsID, npack, pack, weights, covar, sum_w);
-                    else if (time_space)
-                      scale_eigvec_time(out, tsID, nts, npack, pack, weights, covar, data, missval, sum_w);
+                    else if (timeSpace)
+                      scale_eigvec_time(out, tsID, numSteps, npack, pack, weights, covar, data, var1.missval, sum_w);
 
-                    numMissVals = varray_num_mv(gridsize, out, missval);
-                    cdo_def_record(streamID3, varID, levelID);
-                    cdo_write_record(streamID3, out.data(), numMissVals);
-                  }  // loop n_eig
+                    numMissVals = varray_num_mv(var1.gridsize, out, var1.missval);
+                    cdo_def_field(streamID3, varID, levelID);
+                    cdo_write_field(streamID3, out.data(), numMissVals);
+                  }  // loop numEigen
 
-                auto eig_val = eofdata[varID][levelID].eig_val.data();
+                auto eigenValues = eofData.eigenValues.data();
 
-                numMissVals = (fp_is_equal(eig_val[tsID], missval)) ? 1 : 0;
+                numMissVals = (fp_is_equal(eigenValues[tsID], var1.missval)) ? 1 : 0;
 
-                cdo_def_record(streamID2, varID, levelID);
-                cdo_write_record(streamID2, &eig_val[tsID], numMissVals);
+                cdo_def_field(streamID2, varID, levelID);
+                cdo_write_field(streamID2, &eigenValues[tsID], numMissVals);
               }  // loop nlevs
-          }      // loop nvars
+          }  // loop nvars
       }
   }
 
