@@ -93,26 +93,25 @@ windowDays(int dayOfYear, std::vector<int> &wdays, const std::vector<bool> &wday
 
 static void
 writeTimesteps(int MaxMonths, int recentYear, FieldVector3D &cei, int frequency, int taxisID4, const CdoStreamID &streamID4,
-               int *otsID, const VarList &varList1, const std::vector<RecordInfo> &recList, const std::vector<int> &tempdpm,
+               int *otsID, const VarList &varList1, const std::vector<FieldInfo> &fieldInfoList, const std::vector<int> &tempdpm,
                int tempdpy, int func2)
 {
-  const int maxRecs = recList.size();
+  const int maxRecs = fieldInfoList.size();
 
   if (frequency == 8)
     {
-      for (int loopmonth = 2; loopmonth < MaxMonths + 2; loopmonth++)
+      for (int loopmonth = 1; loopmonth < MaxMonths + 1; loopmonth++)
         {
           define_mid_of_time(frequency, taxisID4, recentYear, loopmonth, MaxMonths);
           cdo_def_timestep(streamID4, *otsID);
-          for (int recID = 0; recID < maxRecs; ++recID)
+          for (int fieldID = 0; fieldID < maxRecs; ++fieldID)
             {
-              auto [varIDo, levelIDo] = recList[recID].get();
+              auto [varIDo, levelIDo] = fieldInfoList[fieldID].get();
               if (*otsID && varList1.vars[varIDo].isConstant) continue;
 
-              fieldc_div(cei[loopmonth - 2][0][levelIDo], (double) (tempdpm[loopmonth - 2] / 100.0));
-              cdo_def_record(streamID4, varIDo, levelIDo);
-              cdo_write_record(streamID4, cei[loopmonth - 2][0][levelIDo].vec_d.data(),
-                               cei[loopmonth - 2][0][levelIDo].numMissVals);
+              fieldc_div(cei[loopmonth - 1][0][levelIDo], (double) (tempdpm[loopmonth - 1] / 100.0));
+              cdo_def_field(streamID4, varIDo, levelIDo);
+              cdo_write_field(streamID4, cei[loopmonth - 1][0][levelIDo]);
             }
           (*otsID)++;
         }
@@ -121,14 +120,14 @@ writeTimesteps(int MaxMonths, int recentYear, FieldVector3D &cei, int frequency,
     {
       define_mid_of_time(frequency, taxisID4, recentYear, 0, MaxMonths);
       cdo_def_timestep(streamID4, *otsID);
-      for (int recID = 0; recID < maxRecs; ++recID)
+      for (int fieldID = 0; fieldID < maxRecs; ++fieldID)
         {
-          auto [varIDo, levelIDo] = recList[recID].get();
+          auto [varIDo, levelIDo] = fieldInfoList[fieldID].get();
           if (*otsID && varList1.vars[varIDo].isConstant) continue;
 
           if (func2 == FieldFunc_Avg) fieldc_div(cei[0][0][levelIDo], (double) (tempdpy / 100.0));
-          cdo_def_record(streamID4, varIDo, levelIDo);
-          cdo_write_record(streamID4, cei[0][0][levelIDo].vec_d.data(), cei[0][0][levelIDo].numMissVals);
+          cdo_def_field(streamID4, varIDo, levelIDo);
+          cdo_write_field(streamID4, cei[0][0][levelIDo]);
         }
       (*otsID)++;
     }
@@ -137,7 +136,7 @@ writeTimesteps(int MaxMonths, int recentYear, FieldVector3D &cei, int frequency,
 static void
 calculateOuterPeriod(Field &field, int MaxMonths, int recentYear, int endOfCalc, FieldVector3D &cei, FieldVector3D &varsPtemp,
                      int frequency, int taxisID4, const CdoStreamID &streamID4, int *otsID, const VarList &varList1,
-                     const std::vector<RecordInfo> &recList, int selection, int func2)
+                     const std::vector<FieldInfo> &fieldInfoList, int selection, int func2)
 {
   if (cdo_assert_files_only() == false) cdo_abort("This operator can't be combined with other operators!");
 
@@ -155,8 +154,8 @@ calculateOuterPeriod(Field &field, int MaxMonths, int recentYear, int endOfCalc,
 
   while (true)
     {
-      auto nrecs = streamInqTimestep(cdiStream, tsID++);
-      if (nrecs == 0) break;
+      auto numFields = streamInqTimestep(cdiStream, tsID++);
+      if (numFields == 0) break;
 
       auto vDateTime = taxisInqVdatetime(cdiTaxisID);
       cdiDate_decode(vDateTime.date, &year, &month, &day);
@@ -167,8 +166,8 @@ calculateOuterPeriod(Field &field, int MaxMonths, int recentYear, int endOfCalc,
 
       if (year != recentYear)
         {
-          writeTimesteps(MaxMonths, recentYear, cei, frequency, taxisID4, streamID4, otsID, varList1, recList, tempdpm, tempdpy,
-                         func2);
+          writeTimesteps(MaxMonths, recentYear, cei, frequency, taxisID4, streamID4, otsID, varList1, fieldInfoList, tempdpm,
+                         tempdpy, func2);
           recentYear = year;
           tempdpy = 0;
           tempdpm[0] = 0;
@@ -187,10 +186,10 @@ calculateOuterPeriod(Field &field, int MaxMonths, int recentYear, int endOfCalc,
 
       if (func2 == FieldFunc_Sum) dayOfYear = 1;
 
-      for (int recID = 0; recID < nrecs; ++recID)
+      for (int fieldID = 0; fieldID < numFields; ++fieldID)
         {
-          streamInqRecord(cdiStream, &varID, &levelID);
-          streamReadRecord(cdiStream, field.vec_d.data(), &field.numMissVals);
+          streamInqField(cdiStream, &varID, &levelID);
+          streamReadField(cdiStream, field.vec_d.data(), &field.numMissVals);
 
           Field &pctls = varsPtemp[dayOfYear][0][levelID];
           if (selection == func_selle)
@@ -200,9 +199,9 @@ calculateOuterPeriod(Field &field, int MaxMonths, int recentYear, int endOfCalc,
 
           auto &array = field.vec_d;
           if (func2 == FieldFunc_Avg)
-            for (size_t i = 0; i < field.size; ++i) array[i] = (DBL_IS_EQUAL(array[i], field.missval)) ? 0.0 : 1.0;
+            for (size_t i = 0; i < field.size; ++i) array[i] = (fp_is_equal(array[i], field.missval)) ? 0.0 : 1.0;
           else
-            for (size_t i = 0; i < field.size; ++i) array[i] = (DBL_IS_EQUAL(array[i], field.missval)) ? 0.0 : array[i];
+            for (size_t i = 0; i < field.size; ++i) array[i] = (fp_is_equal(array[i], field.missval)) ? 0.0 : array[i];
           if (frequency == 8)
             field2_add(cei[(int) ((dayOfYear - 1) / 31.)][0][levelID], field);
           else
@@ -211,7 +210,7 @@ calculateOuterPeriod(Field &field, int MaxMonths, int recentYear, int endOfCalc,
     }
   if (Options::cdoVerbose) cdo_print("Finished Processing variables");
   if (year != endOfCalc)
-    writeTimesteps(MaxMonths, year, cei, frequency, taxisID4, streamID4, otsID, varList1, recList, tempdpm, tempdpy, func2);
+    writeTimesteps(MaxMonths, year, cei, frequency, taxisID4, streamID4, otsID, varList1, fieldInfoList, tempdpm, tempdpy, func2);
 
   field_fill(cei[0][0][0], 0.);
 
@@ -235,7 +234,7 @@ etccdi_op(ETCCDI_REQUEST &request)
 
   const int operatorID = cdo_operator_id();
   auto selection = cdo_operator_f1(operatorID);
-  auto frequency = cdo_operator_f2(operatorID);
+  auto frequency = request.compare_type;
 
   percentile_set_method("rtype8");
 
@@ -297,7 +296,7 @@ etccdi_op(ETCCDI_REQUEST &request)
       cdiDefKeyString(vlistID4, 0, CDI_KEY_NAME, request.name);
       cdiDefKeyString(vlistID4, 0, CDI_KEY_LONGNAME, request.longname);
       cdiDefKeyString(vlistID4, 0, CDI_KEY_UNITS, request.units);
-      cdiDefAttTxt(vlistID4, 0, "cell_methods", (int) strlen("time: maximum"), "time: maximum");
+      cdiDefAttTxt(vlistID4, 0, "cell_methods", (int) std::strlen("time: maximum"), "time: maximum");
     }
   else
     {
@@ -307,10 +306,10 @@ etccdi_op(ETCCDI_REQUEST &request)
 
   cdo_def_vlist(streamID4, vlistID4);
 
-  auto maxRecords = varList1.numRecords();
-  std::vector<RecordInfo> recordList(maxRecords);
+  auto maxFields = varList1.maxFields();
+  std::vector<FieldInfo> fieldInfoList(maxFields);
 
-  auto gridsizemax = vlistGridsizeMax(vlistID1);
+  auto gridsizeMax = varList1.gridsizeMax();
 
   Field field;
 
@@ -332,11 +331,11 @@ etccdi_op(ETCCDI_REQUEST &request)
 
   while (true)
     {
-      auto nrecs = cdo_stream_inq_timestep(streamID2, tsID);
-      if (nrecs == 0) break;
+      auto numFields = cdo_stream_inq_timestep(streamID2, tsID);
+      if (numFields == 0) break;
 
-      if (nrecs != cdo_stream_inq_timestep(streamID3, tsID))
-        cdo_abort("Number of records at time step %d of %s and %s differ!", tsID + 1, cdo_get_stream_name(1),
+      if (numFields != cdo_stream_inq_timestep(streamID3, tsID))
+        cdo_abort("Number of fields at time step %d of %s and %s differ!", tsID + 1, cdo_get_stream_name(1),
                   cdo_get_stream_name(2));
 
       auto vDateTime2 = taxisInqVdatetime(taxisID2);
@@ -359,23 +358,21 @@ etccdi_op(ETCCDI_REQUEST &request)
           field2D_init(varsPtemp[dayOfYear], varList2, FIELD_VEC);
           hsets[dayOfYear].create(numVars);
 
-          for (const auto &var : varList1.vars) hsets[dayOfYear].createVarLevels(var.ID, var.nlevels, var.gridsize);
+          for (auto const &var : varList1.vars) hsets[dayOfYear].createVarLevels(var.ID, var.nlevels, var.gridsize);
         }
 
-      for (int recID = 0; recID < nrecs; ++recID)
+      for (int fieldID = 0; fieldID < numFields; ++fieldID)
         {
-          int varID, levelID;
-          cdo_inq_record(streamID2, &varID, &levelID);
-          cdo_read_record(streamID2, varsData2[dayOfYear][varID][levelID]);
+          auto [varID, levelID] = cdo_inq_field(streamID2);
+          cdo_read_field(streamID2, varsData2[dayOfYear][varID][levelID]);
           varsPtemp[dayOfYear][varID][levelID].numMissVals = varsData2[dayOfYear][varID][levelID].numMissVals;
         }
 
-      for (int recID = 0; recID < nrecs; ++recID)
+      for (int fieldID = 0; fieldID < numFields; ++fieldID)
         {
-          int varID, levelID;
-          cdo_inq_record(streamID3, &varID, &levelID);
+          auto [varID, levelID] = cdo_inq_field(streamID3);
           field.init(varList1.vars[varID]);
-          cdo_read_record(streamID3, field);
+          cdo_read_field(streamID3, field);
 
           hsets[dayOfYear].defVarLevelBounds(varID, levelID, varsData2[dayOfYear][varID][levelID], field);
         }
@@ -392,8 +389,8 @@ etccdi_op(ETCCDI_REQUEST &request)
   int year;
   while (true)
     {
-      auto nrecs = cdo_stream_inq_timestep(streamID1, tsID++);
-      if (nrecs == 0) break;
+      auto numFields = cdo_stream_inq_timestep(streamID1, tsID++);
+      if (numFields == 0) break;
 
       auto vDateTime = taxisInqVdatetime(taxisID1);
 
@@ -418,13 +415,12 @@ etccdi_op(ETCCDI_REQUEST &request)
                   = dayOfYear + (year - request.startboot) * (MaxDays - 1);
               dpy[year - request.startboot]++;
               dpm[(year - request.startboot) * MaxMonths + (int) ((dayOfYear - 1) / 31.)]++;
-              for (int recID = 0; recID < nrecs; ++recID)
+              for (int fieldID = 0; fieldID < numFields; ++fieldID)
                 {
-                  int varID, levelID;
-                  cdo_inq_record(streamID1, &varID, &levelID);
-                  cdo_read_record(streamID1, varsData1[dayOfYear + (year - request.startboot) * (MaxDays - 1)][varID][levelID]);
+                  auto [varID, levelID] = cdo_inq_field(streamID1);
+                  cdo_read_field(streamID1, varsData1[dayOfYear + (year - request.startboot) * (MaxDays - 1)][varID][levelID]);
 
-                  if (tsID == 0) recordList[recID].set(varID, levelID);
+                  if (tsID == 0) fieldInfoList[fieldID].set(varID, levelID);
                 }
             }
           else
@@ -463,7 +459,7 @@ etccdi_op(ETCCDI_REQUEST &request)
 
   for (int varID = 0; varID < numVars; ++varID)
     {
-      const auto &var1 = varList1.vars[varID];
+      auto const &var1 = varList1.vars[varID];
       if (var1.isConstant) continue;
       for (int levelID = 0; levelID < var1.nlevels; ++levelID)
         {
@@ -513,7 +509,7 @@ etccdi_op(ETCCDI_REQUEST &request)
     {
       for (int varID = 0; varID < numVars; ++varID)
         {
-          const auto &var1 = varList1.vars[varID];
+          auto const &var1 = varList1.vars[varID];
           if (var1.isConstant) continue;
           for (int levelID = 0; levelID < var1.nlevels; ++levelID)
             {
@@ -538,9 +534,9 @@ etccdi_op(ETCCDI_REQUEST &request)
                 }
             }
         }
-      field.resize(gridsizemax);
+      field.resize(gridsizeMax);
       calculateOuterPeriod(field, MaxMonths, firstYear, request.startboot, cei, varsPtemp, frequency, taxisID4, streamID4, &otsID,
-                           varList1, recordList, selection, request.func2);
+                           varList1, fieldInfoList, selection, request.func2);
     }
   else if (!lOnlyRefPeriod && firstYear != request.startboot)
     cdo_warning("Since you have more than one variable in the input file, only the bootstrapping period can be calculated");
@@ -592,7 +588,7 @@ etccdi_op(ETCCDI_REQUEST &request)
                         {
 #ifdef _OPENMP
 #pragma omp parallel for shared(sumboot, request, varsData1, varID, levelID, hsets, wdaysRead, varsPtemp, varsData2, cei, subyear, \
-                                bootsyear, wdays, frequency) schedule(dynamic)
+                                    bootsyear, wdays, frequency) schedule(dynamic)
 #endif
                           for (int loopdoy = 1; loopdoy < MaxDays; loopdoy++)
                             {
@@ -634,13 +630,13 @@ etccdi_op(ETCCDI_REQUEST &request)
                                     {
                                       auto &array = toCompare.vec_d;
                                       for (size_t i = 0; i < toCompare.size; ++i)
-                                        array[i] = (DBL_IS_EQUAL(array[i], toCompare.missval)) ? 0.0 : 1.0;
+                                        array[i] = (fp_is_equal(array[i], toCompare.missval)) ? 0.0 : 1.0;
                                     }
                                   else
                                     {
                                       auto &array = toCompare.vec_d;
                                       for (size_t i = 0; i < toCompare.size; ++i)
-                                        array[i] = (DBL_IS_EQUAL(array[i], toCompare.missval)) ? 0.0 : array[i];
+                                        array[i] = (fp_is_equal(array[i], toCompare.missval)) ? 0.0 : array[i];
                                     }
                                   // printf("Haben ein Percentil berechnet.\n");
                                   // Year sum
@@ -697,20 +693,18 @@ etccdi_op(ETCCDI_REQUEST &request)
             }
           if (frequency == 8)
             {
-              for (int month = 2; month < MaxMonths + 2; ++month)
+              for (int month = 1; month < MaxMonths + 1; ++month)
                 {
                   define_mid_of_time(frequency, taxisID4, bootsyear, month, MaxMonths);
                   cdo_def_timestep(streamID4, otsID);
 
-                  for (int recID = 0; recID < maxRecords; ++recID)
+                  for (int fieldID = 0; fieldID < maxFields; ++fieldID)
                     {
-                      auto [varIDo, levelIDo] = recordList[recID].get();
+                      auto [varIDo, levelIDo] = fieldInfoList[fieldID].get();
                       if (otsID && varList1.vars[varIDo].isConstant) continue;
 
-                      cdo_def_record(streamID4, varIDo, levelIDo);
-                      cdo_write_record(
-                          streamID4, cei[(bootsyear - request.startboot) * MaxMonths + (month - 2)][varIDo][levelIDo].vec_d.data(),
-                          cei[(bootsyear - request.startboot) * MaxMonths + (month - 2)][varIDo][levelIDo].numMissVals);
+                      cdo_def_field(streamID4, varIDo, levelIDo);
+                      cdo_write_field(streamID4, cei[(bootsyear - request.startboot) * MaxMonths + (month - 1)][varIDo][levelIDo]);
                     }
                   otsID++;
                 }
@@ -720,14 +714,13 @@ etccdi_op(ETCCDI_REQUEST &request)
               define_mid_of_time(frequency, taxisID4, bootsyear, 0, MaxMonths);
               cdo_def_timestep(streamID4, otsID);
 
-              for (int recID = 0; recID < maxRecords; ++recID)
+              for (int fieldID = 0; fieldID < maxFields; ++fieldID)
                 {
-                  auto [varIDo, levelIDo] = recordList[recID].get();
+                  auto [varIDo, levelIDo] = fieldInfoList[fieldID].get();
                   if (otsID && varList1.vars[varIDo].isConstant) continue;
 
-                  cdo_def_record(streamID4, varIDo, levelIDo);
-                  cdo_write_record(streamID4, cei[(bootsyear - request.startboot) * MaxMonths][varIDo][levelIDo].vec_d.data(),
-                                   cei[(bootsyear - request.startboot) * MaxMonths][varIDo][levelIDo].numMissVals);
+                  cdo_def_field(streamID4, varIDo, levelIDo);
+                  cdo_write_field(streamID4, cei[(bootsyear - request.startboot) * MaxMonths][varIDo][levelIDo]);
                 }
               otsID++;
             }
@@ -775,12 +768,12 @@ etccdi_op(ETCCDI_REQUEST &request)
                 }
             }
         }
-      field.resize(gridsizemax);
+      field.resize(gridsizeMax);
       field.missval = varsData1[1][0][0].missval;
       field.size = varsData1[1][0][0].size;
       field.grid = varsData1[1][0][0].grid;
       calculateOuterPeriod(field, MaxMonths, request.endboot + 1, lastYear + 1, cei, varsPtemp, frequency, taxisID4, streamID4,
-                           &otsID, varList1, recordList, selection, request.func2);
+                           &otsID, varList1, fieldInfoList, selection, request.func2);
     }
 
   cdo_stream_close(streamID4);
