@@ -346,7 +346,7 @@ class EcaIndices : public Process
 {
 
 protected:
-  EcaIndices(int p_ID, const std::string &p_operatorName, const std::vector<std::string> &p_arguments, const CdoModule &p_module,
+  EcaIndices(int p_ID, std::string const &p_operatorName, const std::vector<std::string> &p_arguments, const CdoModule &p_module,
              std::function<void(Request p_request)> p_eca_func)
       : Process(p_ID, p_operatorName, p_arguments, p_module), eca_func(p_eca_func)
   {
@@ -372,7 +372,7 @@ public:
 class EcaIndices1 : public EcaIndices<ECA_REQUEST_1>
 {
 public:
-  EcaIndices1(int p_ID, const std::string &p_operatorName, const std::vector<std::string> &p_arguments, const CdoModule &p_module)
+  EcaIndices1(int p_ID, std::string const &p_operatorName, const std::vector<std::string> &p_arguments, const CdoModule &p_module)
       : EcaIndices<ECA_REQUEST_1>(p_ID, p_operatorName, p_arguments, p_module, eca1)
   {
   }
@@ -381,7 +381,7 @@ public:
 class EcaIndices2 : public EcaIndices<ECA_REQUEST_2>
 {
 public:
-  EcaIndices2(int p_ID, const std::string &p_operatorName, const std::vector<std::string> &p_arguments, const CdoModule &p_module)
+  EcaIndices2(int p_ID, std::string const &p_operatorName, const std::vector<std::string> &p_arguments, const CdoModule &p_module)
       : EcaIndices<ECA_REQUEST_2>(p_ID, p_operatorName, p_arguments, p_module, eca2)
   {
   }
@@ -390,7 +390,7 @@ public:
 class EcaIndices3 : public EcaIndices<ECA_REQUEST_3>
 {
 public:
-  EcaIndices3(int p_ID, const std::string &p_operatorName, const std::vector<std::string> &p_arguments, const CdoModule &p_module)
+  EcaIndices3(int p_ID, std::string const &p_operatorName, const std::vector<std::string> &p_arguments, const CdoModule &p_module)
       : EcaIndices<ECA_REQUEST_3>(p_ID, p_operatorName, p_arguments, p_module, eca3)
   {
   }
@@ -399,7 +399,7 @@ public:
 class EcaIndices4 : public EcaIndices<ECA_REQUEST_4>
 {
 public:
-  EcaIndices4(int p_ID, const std::string &p_operatorName, const std::vector<std::string> &p_arguments, const CdoModule &p_module)
+  EcaIndices4(int p_ID, std::string const &p_operatorName, const std::vector<std::string> &p_arguments, const CdoModule &p_module)
       : EcaIndices<ECA_REQUEST_4>(p_ID, p_operatorName, p_arguments, p_module, eca4)
   {
   }
@@ -696,7 +696,7 @@ public:
 
     if (cdo_operator_argc() > 0)
       {
-        const auto &params = cdo_get_oper_argv();
+        auto const &params = cdo_get_oper_argv();
         set_compare_type_from_params(request.compare_type, params);
       }
     else {}
@@ -968,7 +968,7 @@ public:
 
     if (cdo_operator_argc() > 0)
       {
-        const auto &params = cdo_get_oper_argv();
+        auto const &params = cdo_get_oper_argv();
         set_compare_type_from_params(request.compare_type, params);
       }
     else {}
@@ -978,12 +978,14 @@ public:
         request.var1.name = ID_NAME;
         request.var1.longname = ID_LONGNAME;
         request.var1.units = ID_UNITS;
+        request.var1.refdate = ECA_refdate;
       }
     else if (ETCCDI_ID == cdo_operator_id())
       {
         request.var1.name = ID_NAME_ET;
         request.var1.longname = ID_LONGNAME_ET;
         request.var1.units = ID_UNITS_ET;
+        request.var1.refdate = ETC_refdate;
       }
 
     request.var1.f1 = vfarselltc;
@@ -1933,7 +1935,7 @@ public:
 
     if (cdo_operator_argc() > 0)
       {
-        const auto &params = cdo_get_oper_argv();
+        auto const &params = cdo_get_oper_argv();
         set_compare_type_from_params(request.compare_type, params);
       }
 
@@ -2061,12 +2063,15 @@ public:
         request.var1.name = SDII_NAME;
         request.var1.longname = longname;
         request.var1.units = SDII_UNITS;
+        request.var1.refdate = ECA_refdate;
+
       }
     else if (ETCCDI_SDII == cdo_operator_id())
       {
         request.var1.name = SDII_NAME_ET;
         request.var1.longname = SDII_LONGNAME_ET;
         request.var1.units = SDII_UNITS_ET;
+        request.var1.refdate = ETC_refdate;
       }
 
     request.var1.f1 = vfarselgec;
@@ -2186,7 +2191,7 @@ public:
 
     if (cdo_operator_argc() > 0)
       {
-        const auto &params = cdo_get_oper_argv();
+        auto const &params = cdo_get_oper_argv();
         set_compare_type_from_params(request.compare_type, params);
       }
 
@@ -2230,7 +2235,7 @@ public:
 
     if (cdo_operator_argc() > 0)
       {
-        const auto &params = cdo_get_oper_argv();
+        auto const &params = cdo_get_oper_argv();
         set_compare_type_from_params(request.compare_type, params);
       }
 
@@ -2274,7 +2279,7 @@ public:
 
     if (cdo_operator_argc() > 0)
       {
-        const auto &params = cdo_get_oper_argv();
+        auto const &params = cdo_get_oper_argv();
         set_compare_type_from_params(request.compare_type, params);
       }
 
