@@ -29,7 +29,7 @@
 #include "cdo_default_values.h"
 
 #ifdef HAVE_LIBNETCDF
-#include "netcdf.h"
+#include <netcdf.h>
 #endif
 
 static constexpr int nvars_ml = 4;
@@ -70,8 +70,8 @@ iniatts(ATTS *atts)
 }
 
 static void
-inivar(VAR &var, int gridtype, int zaxistype, int code, const std::string &name, const std::string &longname,
-       const std::string &units)
+inivar(VAR &var, int gridtype, int zaxistype, int code, std::string const &name, std::string const &longname,
+       std::string const &units)
 {
   var.gridtype = gridtype;
   var.zaxistype = zaxistype;
@@ -334,10 +334,10 @@ export_e5ml(const char *filename, const std::vector<VAR> &vars, int nvars, int v
   if (nsp == 0) cdo_abort("Spectral data not found!");
   if (nlev == 0) cdo_abort("Hybrid level not found!");
 
-  const size_t nlon = lon;
-  const size_t nlat = lat;
+  size_t nlon = lon;
+  size_t nlat = lat;
 
-  const size_t data_size = nlon + nlat + 2 * nvclev + 2 * nsp * 2 * nlev + nsp * 2 * nlevp1 + nlon * nlat * nlev;
+  size_t data_size = nlon + nlat + 2 * nvclev + 2 * nsp * 2 * nlev + nsp * 2 * nlevp1 + nlon * nlat * nlev;
 
   int writemode = NC_CLOBBER;
   if (data_size * 8 > 2147000000)
@@ -355,51 +355,51 @@ export_e5ml(const char *filename, const std::vector<VAR> &vars, int nvars, int v
 
   char atttext[1024];
   std::strcpy(atttext, "IEEE");
-  size_t attlen = strlen(atttext);
+  size_t attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "source_type", attlen, atttext));
 
-  std::strcpy(atttext, command_line());
-  attlen = strlen(atttext);
+  std::strcpy(atttext, cdo::command_line());
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "history", attlen, atttext));
 
   std::strcpy(atttext, username);
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "user", attlen, atttext));
 
   std::strcpy(atttext, timestr);
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "created", attlen, atttext));
 
   std::strcpy(atttext, "");
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "label_1", attlen, atttext));
 
   std::strcpy(atttext, "");
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "label_2", attlen, atttext));
 
   std::strcpy(atttext, "");
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "label_3", attlen, atttext));
 
   std::strcpy(atttext, "");
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "label_4", attlen, atttext));
 
   std::strcpy(atttext, "");
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "label_5", attlen, atttext));
 
   std::strcpy(atttext, "");
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "label_6", attlen, atttext));
 
   std::strcpy(atttext, "");
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "label_7", attlen, atttext));
 
   std::strcpy(atttext, "");
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "label_8", attlen, atttext));
 
   nce(nc_put_att_int(nc_file_id, NC_GLOBAL, "fdate", NC_INT, 1, &vdate));
@@ -413,7 +413,7 @@ export_e5ml(const char *filename, const std::vector<VAR> &vars, int nvars, int v
   nce(nc_put_att_int(nc_file_id, NC_GLOBAL, "spherical_truncation_m", NC_INT, 1, &ntr));
   nce(nc_put_att_int(nc_file_id, NC_GLOBAL, "spherical_truncation_k", NC_INT, 1, &ntr));
 
-  nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "file_type", strlen(strfiletype_ml), strfiletype_ml));
+  nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "file_type", std::strlen(strfiletype_ml), strfiletype_ml));
 
   int lat_dimid;
   nce(nc_def_dim(nc_file_id, "lat", lat, &lat_dimid));
@@ -448,10 +448,10 @@ export_e5ml(const char *filename, const std::vector<VAR> &vars, int nvars, int v
   int nc_var_id;
   nce(nc_def_var(nc_file_id, "lat", NC_DOUBLE, 1, &lat_dimid, &nc_var_id));
   std::strcpy(atttext, "Gaussian latitude");
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, nc_var_id, "long_name", attlen, atttext));
   std::strcpy(atttext, "degrees_N");
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, nc_var_id, "units", attlen, atttext));
   nce(nc_enddef(nc_file_id));
   nce(nc_put_var_double(nc_file_id, nc_var_id, yvals));
@@ -459,10 +459,10 @@ export_e5ml(const char *filename, const std::vector<VAR> &vars, int nvars, int v
   nce(nc_redef(nc_file_id));
   nce(nc_def_var(nc_file_id, "lon", NC_DOUBLE, 1, &lon_dimid, &nc_var_id));
   std::strcpy(atttext, "longitude");
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, nc_var_id, "long_name", attlen, atttext));
   std::strcpy(atttext, "degrees_E");
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, nc_var_id, "units", attlen, atttext));
   nce(nc_enddef(nc_file_id));
   nce(nc_put_var_double(nc_file_id, nc_var_id, xvals));
@@ -479,10 +479,10 @@ export_e5ml(const char *filename, const std::vector<VAR> &vars, int nvars, int v
   nce(nc_redef(nc_file_id));
   nce(nc_def_var(nc_file_id, "vct_a", NC_DOUBLE, 1, &nvclev_dimid, &nc_var_id));
   std::strcpy(atttext, "vertical-coordinate parameter set A");
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, nc_var_id, "long_name", attlen, atttext));
   std::strcpy(atttext, "");
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, nc_var_id, "units", attlen, atttext));
   nce(nc_enddef(nc_file_id));
   nce(nc_put_var_double(nc_file_id, nc_var_id, vct));
@@ -490,10 +490,10 @@ export_e5ml(const char *filename, const std::vector<VAR> &vars, int nvars, int v
   nce(nc_redef(nc_file_id));
   nce(nc_def_var(nc_file_id, "vct_b", NC_DOUBLE, 1, &nvclev_dimid, &nc_var_id));
   std::strcpy(atttext, "vertical-coordinate parameter set B");
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, nc_var_id, "long_name", attlen, atttext));
   std::strcpy(atttext, "");
-  attlen = strlen(atttext);
+  attlen = std::strlen(atttext);
   nce(nc_put_att_text(nc_file_id, nc_var_id, "units", attlen, atttext));
   nce(nc_enddef(nc_file_id));
   nce(nc_put_var_double(nc_file_id, nc_var_id, vct + nlevp1));
@@ -626,7 +626,7 @@ public:
     int ntr = 0;
     for (int varID = 0; varID < numVars; ++varID)
       {
-        const auto &var1 = varList1.vars[varID];
+        auto const &var1 = varList1.vars[varID];
         auto code = var1.code;
         auto name = var1.name;
         auto longname = var1.longname;
@@ -685,7 +685,7 @@ public:
         vars[varID].ptr = new double[var1.nlevels * var1.gridsize];
       }
 
-    auto nrecs = cdo_stream_inq_timestep(streamID1, 0);
+    auto numFields = cdo_stream_inq_timestep(streamID1, 0);
     auto vDateTime = taxisInqVdatetime(taxisID);
 
     auto vdate = cdiDate_get(vDateTime.date);
@@ -696,14 +696,13 @@ public:
         vtime = 120000;
       }
 
-    for (int recID = 0; recID < nrecs; ++recID)
+    for (int fieldID = 0; fieldID < numFields; ++fieldID)
       {
-        int varID, levelID;
-        cdo_inq_record(streamID1, &varID, &levelID);
+        auto [varID, levelID] = cdo_inq_field(streamID1);
 
         auto gridsize = vars[varID].gridsize;
         size_t numMissVals;
-        cdo_read_record(streamID1, vars[varID].ptr + levelID * gridsize, &numMissVals);
+        cdo_read_field(streamID1, vars[varID].ptr + levelID * gridsize, &numMissVals);
       }
 
     cdo_stream_close(streamID1);
@@ -737,7 +736,7 @@ public:
     for (int iatt = 0; iatt < atts.natxt; ++iatt)
       {
         // printf("%s: %s\n", atts.atxtname[iatt], atts.atxtentry[iatt]);
-        cdiDefAttTxt(vlistID2, CDI_GLOBAL, atts.atxtname[iatt], (int) strlen(atts.atxtentry[iatt]) + 1, atts.atxtentry[iatt]);
+        cdiDefAttTxt(vlistID2, CDI_GLOBAL, atts.atxtname[iatt], (int) std::strlen(atts.atxtentry[iatt]) + 1, atts.atxtentry[iatt]);
       }
 
     auto taxisID = cdo_taxis_create(TAXIS_ABSOLUTE);
@@ -759,8 +758,8 @@ public:
 
         for (int levelID = 0; levelID < nlev; ++levelID)
           {
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, vars[varID].ptr + levelID * gridsize, 0);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, vars[varID].ptr + levelID * gridsize, 0);
           }
       }
 
