@@ -122,10 +122,10 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  int vlistID1;
+  int vlistID1{ CDI_UNDEFID };
 
   size_t gridsize1;
   size_t gridsize2;
@@ -151,11 +151,11 @@ public:
     taxisID1 = vlistInqTaxis(vlistID1);
     taxisID2 = taxisDuplicate(taxisID1);
 
-    int ndiffgrids = 0;
+    int numDiffGrids = 0;
     for (int index = 1; index < vlistNumGrids(vlistID1); ++index)
-      if (vlistGrid(vlistID1, 0) != vlistGrid(vlistID1, index)) ndiffgrids++;
+      if (vlistGrid(vlistID1, 0) != vlistGrid(vlistID1, index)) numDiffGrids++;
 
-    if (ndiffgrids > 0) cdo_abort("Too many different grids in %s!", cdo_get_stream_name(0));
+    if (numDiffGrids > 0) cdo_abort("Too many different grids in %s!", cdo_get_stream_name(0));
 
     auto gridID1 = vlistGrid(vlistID1, 0);
 
@@ -185,18 +185,17 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        while (numFields--)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             size_t numMissVals;
-            cdo_read_record(streamID1, array1.data(), &numMissVals);
+            cdo_read_field(streamID1, array1.data(), &numMissVals);
 
             auto missval1 = vlistInqVarMissval(vlistID1, varID);
 
@@ -205,8 +204,8 @@ public:
               if (gindex[i] >= 0) array2[gindex[i]] = array1[i];
 
             numMissVals = varray_num_mv(gridsize2, array2, missval1);
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, array2.data(), numMissVals);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, array2.data(), numMissVals);
           }
 
         tsID++;
