@@ -15,11 +15,11 @@
 #include <cdi.h>
 
 #include "process_int.h"
-#include "cdo_vlist.h"
 #include "param_conversion.h"
 #include "cdo_options.h"
+#include "cdo_cdi_wrapper.h"
 #include "util_files.h"
-#include "cimdOmp.h"
+#include "cdo_omp.h"
 #include "field_functions.h"
 
 // Defines for rank histogram
@@ -69,14 +69,32 @@ roc_curve_integrate(const Varray2D<double> &roc, int n)
   return area - 0.5;
 }
 
-class ModuleEnsstat3
+class Ensstat3 : public Process
 {
+  enum
+  {
+    func_roc,
+    func_rank
+  };
+
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Ensstat3",
+    .operators = { { "ensrkhistspace", func_rank, space_data, Ensstat2Help },
+                   { "ensrkhisttime", func_rank, time_data, Ensstat2Help },
+                   { "ensroc", func_roc, 0, Ensstat2Help } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { -1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Ensstat3> registration = RegisterEntry<Ensstat3>(module);
+
   int j;
-  int nrecs = 0, nrecs0;
-  size_t nmiss = 0;
+  size_t numMissVals = 0;
   int chksum = 0;  // for check of histogram population
-  int levelID, varID, binID = 0;
-  int gridID, gridID2;
+  int binID = 0;
   int have_miss = 0;
   CdoStreamID streamID2 = 0;
 
@@ -85,30 +103,23 @@ class ModuleEnsstat3
     Varray<double> array;
     CdoStreamID streamID;
     int vlistID;
-  };
-
-  enum
-  {
-    func_roc,
-    func_rank
+    VarList varList;
   };
 
   std::vector<ens_file_t> ef;
-  int nfiles;
+  int numFiles;
   int operfunc;
   int datafunc;
 
-  int taxisID1;
-  int taxisID2;
-  int vlistID1;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  int nbins = 0;
-
-  int nens;
+  int numBins = 0;
+  int numEns;
 
   std::vector<int> varIDs2;
 
-  size_t gridsize;
+  size_t gridsizeMax;
   std::vector<std::vector<int>> array2;
   FieldVector field;
 
@@ -119,16 +130,8 @@ class ModuleEnsstat3
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    // clang-format off
-  cdo_operator_add("ensroc",         func_roc,  0,          nullptr);
-  cdo_operator_add("ensrkhistspace", func_rank, space_data, nullptr);
-  cdo_operator_add("ensrkhisttime",  func_rank, time_data,  nullptr);
-    // clang-format on
-
     auto operatorID = cdo_operator_id();
     operfunc = cdo_operator_f1(operatorID);
     datafunc = cdo_operator_f2(operatorID);
@@ -136,67 +139,70 @@ public:
     if (operfunc == func_roc)
       {
         operator_input_arg("Number of eigen functions to write out");
-        nbins = parameter_to_int(cdo_operator_argv(0));
+        numBins = parameter_to_int(cdo_operator_argv(0));
       }
 
-    nfiles = cdo_stream_cnt() - 1;
-    nens = nfiles - 1;
+    numFiles = cdo_stream_cnt() - 1;
+    numEns = numFiles - 1;
 
-    if (Options::cdoVerbose) cdo_print("Ensemble over %d files.", nfiles);
+    if (Options::cdoVerbose) cdo_print("Ensemble over %d files.", numFiles);
 
-    std::string ofilename = cdo_get_stream_name(nfiles);
+    std::string ofilename = cdo_get_stream_name(numFiles);
 
     if (!Options::cdoOverwriteMode && FileUtils::file_exists(ofilename) && !FileUtils::user_file_overwrite(ofilename))
       cdo_abort("Outputfile %s already exists!", ofilename);
 
-    ef = std::vector<ens_file_t>(nfiles);
+    ef = std::vector<ens_file_t>(numFiles);
 
     /* *************************************************** */
     /* should each thread be allocating memory locally???? */
     /* ("first touch strategy")                            */
     /* --> #pragma omp parallel for ...                    */
     /* *************************************************** */
-    field = FieldVector(Threading::ompNumThreads);
-    for (int i = 0; i < Threading::ompNumThreads; ++i)
+    field = FieldVector(Threading::ompNumMaxThreads);
+    for (int i = 0; i < Threading::ompNumMaxThreads; ++i)
       {
-        field[i].resize(nfiles);
-        field[i].weightv.resize(nfiles);
-        for (int fileID = 0; fileID < nfiles; ++fileID) field[i].weightv[fileID] = 1;
+        field[i].resize(numFiles);
+        field[i].weightv.resize(numFiles);
+        for (int fileID = 0; fileID < numFiles; ++fileID) field[i].weightv[fileID] = 1;
       }
 
-    for (int fileID = 0; fileID < nfiles; ++fileID)
+    for (int fileID = 0; fileID < numFiles; ++fileID)
       {
         auto streamID = cdo_open_read(fileID);
         ef[fileID].streamID = streamID;
         ef[fileID].vlistID = cdo_stream_inq_vlist(streamID);
+        ef[fileID].varList = VarList(ef[fileID].vlistID);
       }
 
     // check for identical contents of all ensemble members
-    for (int fileID = 1; fileID < nfiles; ++fileID) vlist_compare(ef[0].vlistID, ef[fileID].vlistID, CmpVlist::All);
+    for (int fileID = 1; fileID < numFiles; ++fileID) varList_compare(ef[0].varList, ef[fileID].varList);
 
-    vlistID1 = ef[0].vlistID;
+    auto vlistID1 = ef[0].vlistID;
+    auto const &varList1 = ef[0].varList;
     auto vlistID2 = vlistCreate();
-    vlistDefNtsteps(vlistID2, vlistNtsteps(vlistID1));
+    vlistDefNtsteps(vlistID2, varList1.numSteps());
 
-    auto nvars = vlistNvars(vlistID1);
-    varIDs2 = std::vector<int>(nvars);
+    auto numVars = varList1.numVars();
+    varIDs2 = std::vector<int>(numVars);
 
-    auto zaxisID2 = zaxisCreate(ZAXIS_GENERIC, nfiles);
+    auto zaxisID2 = zaxisCreate(ZAXIS_GENERIC, numFiles);
     {
-      Varray<double> levs(nfiles, 0);
-      for (int i = 0; i < nfiles; ++i) levs[i] = i;
+      Varray<double> levs(numFiles, 0);
+      for (int i = 0; i < numFiles; ++i) levs[i] = i;
       zaxisDefLevels(zaxisID2, levs.data());
       cdiDefKeyString(zaxisID2, CDI_GLOBAL, CDI_KEY_NAME, "histogram_binID");
     }
 
     int time_mode = (datafunc == TIME) ? TIME_VARYING : TIME_CONSTANT;
 
-    for (varID = 0; varID < nvars; ++varID)
+    for (int varID = 0; varID < numVars; ++varID)
       {
-        /* **************************************************************** */
-        /* nfiles includes the observation, so there are nfiles-1 ensembles */
-        /* and exactly nfiles bins, in which the observation could fall     */
-        /* **************************************************************** */
+        int gridID2;
+        /* ******************************************************************** */
+        /* numFiles includes the observation, so there are numFiles-1 ensembles */
+        /* and exactly numFiles bins, in which the observation could fall       */
+        /* ******************************************************************** */
         if (datafunc == TIME)
           {
             double val = 0.0;
@@ -207,7 +213,7 @@ public:
             gridDefYvals(gridID2, &val);
           }
         else  // datafunc == SPACE
-          gridID2 = vlistInqVarGrid(vlistID1, varID);
+          gridID2 = varList1.vars[varID].gridID;
 
         varIDs2[varID] = vlistDefVar(vlistID2, gridID2, zaxisID2, time_mode);
       }
@@ -216,9 +222,9 @@ public:
     taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
 
-    for (varID = 0; varID < nvars; ++varID)
+    for (int varID = 0; varID < numVars; ++varID)
       {
-        if (zaxisInqSize(vlistInqVarZaxis(vlistID1, varID)) > 1)
+        if (varList1.vars[varID].nlevels > 1)
           {
             cdo_warning("More than one level not supported when processing ranked histograms.");
             cdo_warning("Try to use `cdo splitlevel` to split the dataset into levels and apply");
@@ -229,61 +235,66 @@ public:
 
     if (operfunc != func_roc)
       {
-        streamID2 = cdo_open_write(nfiles);
+        streamID2 = cdo_open_write(numFiles);
         cdo_def_vlist(streamID2, vlistID2);
       }
 
-    gridsize = vlistGridsizeMax(vlistID1);
+    gridsizeMax = cdo_vlist_gridsizemax(vlistID1);
 
-    for (int fileID = 0; fileID < nfiles; ++fileID) ef[fileID].array.resize(gridsize);
+    for (int fileID = 0; fileID < numFiles; ++fileID) ef[fileID].array.resize(gridsizeMax);
 
-    array2 = std::vector<std::vector<int>>(nfiles + 1);
+    array2 = std::vector<std::vector<int>>(numFiles + 1);
     if (operfunc == func_rank && datafunc == SPACE)
       {  // need to memorize data for entire grid before writing
-        for (binID = 0; binID < nfiles; binID++) array2[binID].resize(gridsize, 0);
+        for (binID = 0; binID < numFiles; binID++) array2[binID].resize(gridsizeMax, 0);
       }
     else if (operfunc == func_rank)
       {  // can process data separately for each timestep and only need to cumulate values over the grid
-        for (binID = 0; binID < nfiles; binID++) array2[binID].resize(1, 0);
+        for (binID = 0; binID < numFiles; binID++) array2[binID].resize(1, 0);
       }
 
     if (operfunc == func_roc)
       {
-        hist.resize(nbins);
+        hist.resize(numBins);
 
-        roc.resize(nbins + 1);
-        ctg_tab.resize(nbins + 1);
-        for (int i = 0; i <= nbins; ++i)
+        roc.resize(numBins + 1);
+        ctg_tab.resize(numBins + 1);
+        for (int i = 0; i <= numBins; ++i)
           {
             roc[i].resize(2, 0);
             ctg_tab[i].resize(4, 0);
           }
 
-        uThresh.resize(nbins);
-        lThresh.resize(nbins);
-        for (int i = 0; i < nbins; ++i)
+        uThresh.resize(numBins);
+        lThresh.resize(numBins);
+        for (int i = 0; i < numBins; ++i)
           {
-            uThresh[i] = ((double) i + 1) / nbins;
-            lThresh[i] = (double) i / nbins;
+            uThresh[i] = ((double) i + 1) / numBins;
+            lThresh[i] = (double) i / numBins;
           }
       }
   }
+
   void
-  run()
+  run() override
   {
+    auto const &varList1 = ef[0].varList;
+
+    int varID = 0, levelID;
+    int numFields0;
     int tsID = 0;
     do {
-        nrecs0 = cdo_stream_inq_timestep(ef[0].streamID, tsID);
-        for (int fileID = 1; fileID < nfiles; ++fileID)
+        numFields0 = cdo_stream_inq_timestep(ef[0].streamID, tsID);
+        for (int fileID = 1; fileID < numFiles; ++fileID)
           {
             auto streamID = ef[fileID].streamID;
-            nrecs = cdo_stream_inq_timestep(streamID, tsID);
-            if (nrecs != nrecs0)
+            int numFields = cdo_stream_inq_timestep(streamID, tsID);
+            if (numFields != numFields0)
               {
-                if (nrecs == 0)
+                if (numFields == 0)
                   cdo_abort("Inconsistent ensemble file, too few time steps in %s!", cdo_get_stream_name(fileID));
                 else
-                  cdo_abort("Inconsistent ensemble file, number of records at time step %d of %s and %s differ!", tsID + 1,
+                  cdo_abort("Inconsistent ensemble file, number of fields at time step %d of %s and %s differ!", tsID + 1,
                             cdo_get_stream_name(0), cdo_get_stream_name(fileID));
               }
           }
@@ -291,45 +302,42 @@ public:
         if (operfunc == func_rank && (datafunc == TIME || tsID == 0))
           {
             cdo_taxis_copy_timestep(taxisID2, taxisID1);
-            if (nrecs0 > 0) cdo_def_timestep(streamID2, tsID);
+            if (numFields0 > 0) cdo_def_timestep(streamID2, tsID);
           }
 
-        //      fprintf(stderr,"TIMESTEP %i varID %i rec %i\n",tsID,varID,recID);
-
-        for (int recID = 0; recID < nrecs0; ++recID)
+        for (int fieldID = 0; fieldID < numFields0; ++fieldID)
           {
 #ifdef _OPENMP
-#pragma omp parallel for default(none) shared(ef, nfiles) private(nmiss) lastprivate(varID, levelID)
+#pragma omp parallel for default(none) shared(ef, numFiles) private(numMissVals) lastprivate(varID, levelID)
 #endif
-            for (int fileID = 0; fileID < nfiles; ++fileID)
+            for (int fileID = 0; fileID < numFiles; ++fileID)
               {
                 auto streamID = ef[fileID].streamID;
-                cdo_inq_record(streamID, &varID, &levelID);
-                cdo_read_record(streamID, ef[fileID].array.data(), &nmiss);
+                cdo_inq_field(streamID, &varID, &levelID);
+                cdo_read_field(streamID, ef[fileID].array.data(), &numMissVals);
               }
 
-            gridID = vlistInqVarGrid(vlistID1, varID);
-            gridsize = gridInqSize(gridID);
-            auto missval = vlistInqVarMissval(vlistID1, varID);
+            gridsizeMax = varList1.vars[varID].gridsize;
+            auto missval = varList1.vars[varID].missval;
 
-            nmiss = 0;
+            numMissVals = 0;
             if (datafunc == TIME && operfunc == func_rank)
-              for (binID = 0; binID < nfiles; binID++) array2[binID][0] = 0;
+              for (binID = 0; binID < numFiles; binID++) array2[binID][0] = 0;
 
 #ifdef _OPENMP
 #pragma omp parallel for default(shared) private(binID)
 #endif
-            for (size_t i = 0; i < gridsize; ++i)
+            for (size_t i = 0; i < gridsizeMax; ++i)
               {
                 auto ompthID = cdo_omp_get_thread_num();
 
                 field[ompthID].missval = missval;
-                field[ompthID].nmiss = 0;
+                field[ompthID].numMissVals = 0;
                 have_miss = 0;
-                for (int fileID = 0; fileID < nfiles; ++fileID)
+                for (int fileID = 0; fileID < numFiles; ++fileID)
                   {
                     field[ompthID].vec_d[fileID] = ef[fileID].array[i];
-                    if (DBL_IS_EQUAL(field[ompthID].vec_d[fileID], missval))
+                    if (fp_is_equal(field[ompthID].vec_d[fileID], missval))
                       {
                         have_miss = 1;
                         break;
@@ -346,7 +354,7 @@ public:
                         /* ****************/
                         /* RANK HISTOGRAM */
                         /* ************** */
-                        // for ( j=0; j<nfiles; j++ )
+                        // for ( j=0; j<numFiles; j++ )
                         //   fprintf(stderr,"%5.2g ",field[ompthID].vec_d[j]);
 #ifdef _OPENMP
 #pragma omp critical
@@ -368,61 +376,61 @@ public:
                         auto dat = &field[ompthID].vec_d[1];
                         auto ival = (field[ompthID].vec_d[0] > 0.5) ? 1 : 0;
 
-                        for (binID = 0; binID < nbins; binID++) hist[binID] = 0;
+                        for (binID = 0; binID < numBins; binID++) hist[binID] = 0;
 
-                        for (j = 0; j < nens; ++j)
-                          for (binID = 0; binID < nbins; binID++)
+                        for (j = 0; j < numEns; ++j)
+                          for (binID = 0; binID < numBins; binID++)
                             if (dat[j] >= lThresh[binID] && dat[j] < uThresh[binID]) hist[binID]++;
 
                         chksum = 0;
-                        for (binID = 0; binID < nbins; binID++) chksum += hist[binID];
+                        for (binID = 0; binID < numBins; binID++) chksum += hist[binID];
 
-                        if (chksum != nens) exit(1);
+                        if (chksum != numEns) exit(1);
 
                         int cum = 0;
                         if (ival == 1)
                           {
                             // all true positives in first bin
-                            ctg_tab[0][TP] += nens;
+                            ctg_tab[0][TP] += numEns;
 
                             cum += hist[0];
-                            for (binID = 1; binID < nbins; binID++)
+                            for (binID = 1; binID < numBins; binID++)
                               {
-                                ctg_tab[binID][TP] += nens - cum;
+                                ctg_tab[binID][TP] += numEns - cum;
                                 ctg_tab[binID][FN] += cum;
                                 cum += hist[binID];
                               }
-                            ctg_tab[binID][TP] += nens - cum;
+                            ctg_tab[binID][TP] += numEns - cum;
                             ctg_tab[binID][FN] += cum;
                           }
                         else if (ival == 0)
                           {
                             // all false positives in first bin
-                            ctg_tab[0][FP] += nens;
+                            ctg_tab[0][FP] += numEns;
                             cum += hist[0];
-                            for (binID = 1; binID < nbins; binID++)
+                            for (binID = 1; binID < numBins; binID++)
                               {
-                                ctg_tab[binID][FP] += nens - cum;
+                                ctg_tab[binID][FP] += numEns - cum;
                                 ctg_tab[binID][TN] += cum;
                                 cum += hist[binID];
                               }
-                            ctg_tab[binID][FP] += nens - cum;
+                            ctg_tab[binID][FP] += numEns - cum;
                             ctg_tab[binID][TN] += cum;
                           }
                         break;
 
                       }  // switch ( operfunc )
-                  }      // if ( ! have_miss )
-              }          // for ( i=0; i<gridsize; i++ )
+                  }  // if ( ! have_miss )
+              }  // for ( i=0; i<gridsize; i++ )
 
             if (datafunc == TIME && operfunc == func_rank)
               {
-                for (binID = 0; binID < nfiles; binID++)
+                for (binID = 0; binID < numFiles; binID++)
                   {
                     double val = (double) array2[binID][0];
                     //		fprintf(stderr,"%i ",(int)val);
-                    cdo_def_record(streamID2, varIDs2[varID], binID);
-                    cdo_write_record(streamID2, &val, nmiss);
+                    cdo_def_field(streamID2, varIDs2[varID], binID);
+                    cdo_write_field(streamID2, &val, numMissVals);
                   }
                 // fprintf(stderr,"\n");
               }
@@ -432,7 +440,7 @@ public:
                   {
                     fprintf(stderr, "#             :     TP     FP     FN     TN         TPR        FPR\n");
 
-                    for (binID = 0; binID <= nbins; binID++)
+                    for (binID = 0; binID <= numBins; binID++)
                       {
                         int p = ctg_tab[binID][TP] + ctg_tab[binID][FN];
                         int n = ctg_tab[binID][FP] + ctg_tab[binID][TN];
@@ -443,36 +451,37 @@ public:
                         roc[binID][TPR] = tpr;
                         roc[binID][FPR] = fpr;
 
-                        fprintf(stderr, "%3i %10.4g: %6i %6i %6i %6i: %10.4g %10.4g\n", binID, (binID < nbins) ? lThresh[binID] : 1,
-                                ctg_tab[binID][0], ctg_tab[binID][1], ctg_tab[binID][2], ctg_tab[binID][3], tpr, fpr);
+                        fprintf(stderr, "%3i %10.4g: %6i %6i %6i %6i: %10.4g %10.4g\n", binID,
+                                (binID < numBins) ? lThresh[binID] : 1, ctg_tab[binID][0], ctg_tab[binID][1], ctg_tab[binID][2],
+                                ctg_tab[binID][3], tpr, fpr);
                       }
-                    fprintf(stderr, "nbins %10i\n", nbins);
-                    fprintf(stderr, "#ROC CurveArea: %10.6f\n", roc_curve_integrate(roc, nbins));
+                    fprintf(stderr, "nbins %10i\n", numBins);
+                    fprintf(stderr, "#ROC CurveArea: %10.6f\n", roc_curve_integrate(roc, numBins));
                   }  // if ( DEBUG_ROC )
-              }      // else if (operfunc == func_roc )
-          }          // for ( recID=0; recID<nrecs; recID++ )
+              }  // else if (operfunc == func_roc )
+          }  // for ( fieldID=0; fieldID<numFields0; fieldID++ )
         tsID++;
       }  // do [...]
-    while (nrecs0 > 0);
+    while (numFields0 > 0);
 
     if (operfunc == func_rank)
       {
-        int osize = (datafunc == TIME) ? 1 : gridsize;
+        int osize = (datafunc == TIME) ? 1 : gridsizeMax;
         Varray<double> tmpdoub(osize);
 
-        for (binID = 0; binID < nfiles; binID++)
+        for (binID = 0; binID < numFiles; binID++)
           {
             for (int i = 0; i < osize; ++i) tmpdoub[i] = (double) array2[binID][i];
 
-            cdo_def_record(streamID2, varIDs2[varID], binID);
-            cdo_write_record(streamID2, tmpdoub.data(), nmiss);
+            cdo_def_field(streamID2, varIDs2[varID], binID);
+            cdo_write_field(streamID2, tmpdoub.data(), numMissVals);
           }
       }
     else if (operfunc == func_roc)
       {
         fprintf(stdout, "#             :     TP     FP     FN     TN         TPR        FPR\n");
 
-        for (int i = 0; i <= nbins; ++i)
+        for (int i = 0; i <= numBins; ++i)
           {
             int p = ctg_tab[i][TP] + ctg_tab[i][FN];
             int n = ctg_tab[i][FP] + ctg_tab[i][TN];
@@ -485,31 +494,18 @@ public:
 
             int sum = ctg_tab[i][TP] + ctg_tab[i][TN] + ctg_tab[i][FP] + ctg_tab[i][FN];
 
-            fprintf(stdout, "%3i %10.4g: %6i %6i %6i %6i (%6i): %10.4g %10.4g\n", i, (i < nbins) ? lThresh[i] : 1, ctg_tab[i][0],
+            fprintf(stdout, "%3i %10.4g: %6i %6i %6i %6i (%6i): %10.4g %10.4g\n", i, (i < numBins) ? lThresh[i] : 1, ctg_tab[i][0],
                     ctg_tab[i][1], ctg_tab[i][2], ctg_tab[i][3], sum, tpr, fpr);
           }
 
-        fprintf(stdout, "#ROC CurveArea: %10.6f\n", roc_curve_integrate(roc, nbins));
+        fprintf(stdout, "#ROC CurveArea: %10.6f\n", roc_curve_integrate(roc, numBins));
       }
   }
   void
-  close()
+  close() override
   {
-    for (int fileID = 0; fileID < nfiles; ++fileID) cdo_stream_close(ef[fileID].streamID);
+    for (int fileID = 0; fileID < numFiles; ++fileID) cdo_stream_close(ef[fileID].streamID);
 
     if (operfunc != func_roc) cdo_stream_close(streamID2);
-
-    cdo_finish();
   }
 };
-void *
-Ensstat3(void *process)
-{
-  ModuleEnsstat3 ensstat3;
-
-  ensstat3.init(process);
-  ensstat3.run();
-  ensstat3.close();
-
-  return nullptr;
-}
