@@ -15,12 +15,9 @@
      Score for Ensemble Prediction Systems, in: Weather and Forecasting (15) pp. 559-570
 */
 
-#include <algorithm>  // sort
-
 #include <cdi.h>
 
 #include "process_int.h"
-#include "cdo_vlist.h"
 #include "cdo_options.h"
 #include "param_conversion.h"
 #include "util_files.h"
@@ -47,12 +44,22 @@ enum RESTYPE_CRPS
   CRPS_POT
 };
 
-class ModuleEnsval
+class Ensval : public Process
 {
-  int k;
-  int nrecs = 0, nostreams = 0, ngrids;
-  size_t nmiss;
-  int levelID = -1, varID = -1;
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Ensval",
+    .operators = { { "enscrps", CRPS, 0, EnsvalHelp }, { "ensbrs", BRS, 0, EnsvalHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { -1, OBASE, NoRestriction },
+  };
+  inline static RegisterEntry<Ensval> registration = RegisterEntry<Ensval>(module);
+
+  int nostreams = 0, numGrids;
+  size_t numMissVals;
   size_t gridsize = 0;
   int vlistID;
   int have_miss = 0;
@@ -70,13 +77,14 @@ class ModuleEnsval
   {
     CdoStreamID streamID;
     int vlistID;
+    VarList varList;
     Varray<double> array;
   };
 
   int nfiles;
 
   int operfunc;
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
   int gridID2;
 
   int nens;
@@ -98,13 +106,8 @@ class ModuleEnsval
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    cdo_operator_add("enscrps", CRPS, 0, nullptr);
-    cdo_operator_add("ensbrs", BRS, 0, nullptr);
-
     auto operatorID = cdo_operator_id();
     operfunc = cdo_operator_f1(operatorID);
 
@@ -156,6 +159,7 @@ public:
 
         ef[fileID].streamID = streamID;
         ef[fileID].vlistID = vlistID;
+        ef[fileID].varList = VarList(ef[fileID].vlistID);
       }
 
     auto streamID1 = ef[0].streamID;
@@ -166,7 +170,7 @@ public:
     auto nvars = vlistNvars(ef[0].vlistID);
     if (Options::cdoVerbose) cdo_print("nvars %i", nvars);
 
-    for (fileID = 1; fileID < nfiles; ++fileID) vlist_compare(ef[0].vlistID, ef[fileID].vlistID, CmpVlist::All);
+    for (fileID = 1; fileID < nfiles; ++fileID) varList_compare(ef[0].varList, ef[fileID].varList);
 
     auto vlistID1 = ef[0].vlistID;
     taxisID1 = vlistInqTaxis(vlistID1);
@@ -205,20 +209,19 @@ public:
             break;
           }
 
-        auto ofilename = ofilebase + string_format(".%s", typeSuffix);
-        if (ofilename.size() > 0) ofilename += fileSuffix;
+        auto fileName = ofilebase + string_format(".%s", typeSuffix);
+        if (fileName.size() > 0) fileName += fileSuffix;
 
-        if (!Options::cdoOverwriteMode && FileUtils::file_exists(ofilename)
-            && !FileUtils::user_file_overwrite(ofilename))
-          cdo_abort("Outputfile %s already exists!", ofilename);
+        if (!Options::cdoOverwriteMode && FileUtils::file_exists(fileName) && !FileUtils::user_file_overwrite(fileName))
+          cdo_abort("Outputfile %s already exists!", fileName);
 
-        streamID2[stream] = cdo_open_write(ofilename.c_str());
+        streamID2[stream] = open_write(fileName);
 
         taxisID2[stream] = taxisDuplicate(taxisID1);
         vlistID2[stream] = vlistDuplicate(vlistID1);
 
-        ngrids = vlistNgrids(vlistID2[stream]);
-        for (int i = 0; i < ngrids; ++i) vlistChangeGridIndex(vlistID2[stream], i, gridID2);
+        numGrids = vlistNumGrids(vlistID2[stream]);
+        for (int i = 0; i < numGrids; ++i) vlistChangeGridIndex(vlistID2[stream], i, gridID2);
 
         vlistDefTaxis(vlistID2[stream], taxisID2[stream]);
         cdo_def_vlist(streamID2[stream], vlistID2[stream]);
@@ -226,46 +229,49 @@ public:
 
     if (Options::cdoVerbose) cdo_print(" sum_weights %10.6f", sum_weights);
 
-    varList_init(varList1, vlistID1);
+    varList1 = VarList(vlistID1);
   }
+
   void
-  run()
+  run() override
   {
+    int numFields = 0;
     int tsID = 0;
     do {
-        auto nrecs0 = cdo_stream_inq_timestep(ef[0].streamID, tsID);
+        auto numFields0 = cdo_stream_inq_timestep(ef[0].streamID, tsID);
         for (fileID = 1; fileID < nfiles; ++fileID)
           {
             streamID = ef[fileID].streamID;
-            nrecs = cdo_stream_inq_timestep(streamID, tsID);
-            if (nrecs != nrecs0)
-              cdo_abort("Number of records at time step %d of %s and %s differ!", tsID + 1, cdo_get_stream_name(0),
+            numFields = cdo_stream_inq_timestep(streamID, tsID);
+            if (numFields != numFields0)
+              cdo_abort("Number of fields at time step %d of %s and %s differ!", tsID + 1, cdo_get_stream_name(0),
                         cdo_get_stream_name(fileID));
           }
 
         for (stream = 0; stream < nostreams; stream++)
           {
             cdo_taxis_copy_timestep(taxisID2[stream], taxisID1);
-            if (nrecs0 > 0) cdo_def_timestep(streamID2[stream], tsID);
+            if (numFields0 > 0) cdo_def_timestep(streamID2[stream], tsID);
           }
 
-        for (int recID = 0; recID < nrecs0; ++recID)
+        while (numFields0-- > 0)
           {
+            int varID = 0, levelID = 0;
             for (fileID = 0; fileID < nfiles; ++fileID)
               {
                 streamID = ef[fileID].streamID;
-                cdo_inq_record(streamID, &varID, &levelID);
+                cdo_inq_field(streamID, &varID, &levelID);
 
                 if (fileID == 0)
                   {
-                    gridsize = varList1[varID].gridsize;
-                    missval = varList1[varID].missval;
+                    gridsize = varList1.vars[varID].gridsize;
+                    missval = varList1.vars[varID].missval;
                     weights.resize(gridsize);
                   }
 
                 ef[fileID].array.resize(gridsize);
 
-                cdo_read_record(streamID, ef[fileID].array.data(), &nmiss);
+                cdo_read_field(streamID, ef[fileID].array.data(), &numMissVals);
               }
 
             // xsize = gridInqXsize(gridID);
@@ -280,11 +286,11 @@ public:
             else
             */
             {
-              varray_fill(weights, 1.0 / gridsize);
+              std::ranges::fill(weights, 1.0 / gridsize);
               sum_weights = 1.0;
             }
 
-            nmiss = 0;
+            numMissVals = 0;
             heavyside0 = 0;
             heavysideN = 0;
 
@@ -300,7 +306,7 @@ public:
                     else
                       val[fileID - 1] = value;  // Ensembles start at 2nd file
 
-                    if (dbl_is_equal(value, missval))
+                    if (fp_is_equal(value, missval))
                       {
                         have_miss = 1;
                         break;
@@ -308,7 +314,7 @@ public:
                   }
 
                 auto &x = val;
-                std::sort(x.begin(), x.end());  // Sort The Ensemble Array to ascending order
+                std::ranges::sort(x);  // Sort The Ensemble Array to ascending order
 
                 // only process if no missing value in ensemble
                 if (!have_miss && operfunc == CRPS)
@@ -326,7 +332,7 @@ public:
                       }
 
                     // Loop start at zero ==> 1st ensemble (c-indexing)
-                    for (k = 0; k < nens - 1; ++k)
+                    for (int k = 0; k < nens - 1; ++k)
                       {                     // Cumulate alpha and beta
                         if (xa > x[k + 1])  // left of heavyside
                           alpha[k + 1] += (x[k + 1] - x[k]) * weights[i];
@@ -349,7 +355,7 @@ public:
                     else if (x[nens - 1] < brs_thresh)
                       brs_g[nens] += weights[i];
                     else
-                      for (k = 0; k < nens - 1; ++k)
+                      for (int k = 0; k < nens - 1; ++k)
                         {
                           if (x[k + 1] >= brs_thresh && brs_thresh >= x[k])
                             {
@@ -366,7 +372,7 @@ public:
                         else if (x[nens - 1] < xa)
                           brs_o[nens] += weights[i];
                         else
-                          for (k = 0; k < nens - 1; ++k)
+                          for (int k = 0; k < nens - 1; ++k)
                             {
                               if (x[k + 1] >= xa && xa >= x[k])
                                 {
@@ -389,11 +395,11 @@ public:
                 crps = g * ((1. - o) * p * p + o * (1. - p) * (1. - p));
 
                 // Middle Bins
-                for (k = 1; k < nens; ++k)
+                for (int k = 1; k < nens; ++k)
                   {
                     p = (double) k / (double) nens;
 
-                    if (!dbl_is_equal(sum_weights, 1.0))
+                    if (fp_is_not_equal(sum_weights, 1.0))
                       {
                         alpha[k] /= sum_weights;
                         beta[k] /= sum_weights;
@@ -430,7 +436,7 @@ public:
                 brs_uncty = 0;
 
                 double gsum = 0, obar = 0, osum = 0;
-                for (k = 0; k <= nens; ++k)
+                for (int k = 0; k <= nens; ++k)
                   {
                     obar += brs_g[k] * brs_o[k];
                     gsum += brs_g[k];
@@ -443,15 +449,13 @@ public:
                     cdo_abort("This is likely due to missing values");
                   }
 
-                double o = 0, p = 0, g = 0;
                 brs_uncty = obar * (1 - obar);
 
-                for (k = 0; k <= nens; ++k)
+                for (int k = 0; k <= nens; ++k)
                   {
-
-                    g = brs_g[k];
-                    o = brs_o[k];
-                    p = 1.0 - k / (float) nens;
+                    auto g = brs_g[k];
+                    auto o = brs_o[k];
+                    auto p = 1.0 - k / (float) nens;
                     // need p = 1 - k/nens here as k=0 if all members forecast
                     // event and k=nens if none does so.
 
@@ -466,7 +470,6 @@ public:
 
                 if (Options::cdoVerbose)
                   {
-                    // cdo_print("Brier score for var %i level %i calculated",varID, levelID);
                     cdo_print("BRS: obar %12.6g brs  %12.6g reli %12.6g resol %12.6g u %12.6g", obar,
                               brs_reli - brs_resol + brs_uncty, brs_reli, brs_resol, brs_uncty);
                   }
@@ -477,35 +480,36 @@ public:
 
             for (stream = 0; stream < nostreams; stream++)
               {
-                cdo_def_record(streamID2[stream], varID, levelID);
+                cdo_def_field(streamID2[stream], varID, levelID);
                 if (std::isnan(results[stream]))
                   {
                     results[stream] = missval;
                     have_miss = 1;
                   }
-                cdo_write_record(streamID2[stream], &results[stream], have_miss);
+                cdo_write_field(streamID2[stream], &results[stream], have_miss);
               }
 
             switch (operfunc)
               {
               case (CRPS):
-                varray_fill(alpha, 0.0);
-                varray_fill(beta, 0.0);
+                std::ranges::fill(alpha, 0.0);
+                std::ranges::fill(beta, 0.0);
                 heavyside0 = 0;
                 heavysideN = 0;
                 break;
               case (BRS):
-                varray_fill(brs_o, 0.0);
-                varray_fill(brs_g, 0.0);
+                std::ranges::fill(brs_o, 0.0);
+                std::ranges::fill(brs_g, 0.0);
                 break;
               }
-          }  // for ( int recID = 0; recID < nrecs; recID++ )
+          }  // while (numFields0-- > 0)
         tsID++;
       }
-    while (nrecs);
+    while (numFields);
   }
+
   void
-  close()
+  close() override
   {
     for (fileID = 0; fileID < nfiles; ++fileID)
       {
@@ -520,21 +524,5 @@ public:
         vlistDestroy(vlistID2[stream]);
         taxisDestroy(taxisID2[stream]);
       }
-
-    gridDestroy(gridID2);
-
-    cdo_finish();
   }
 };
-
-void *
-Ensval(void *process)
-{
-  ModuleEnsval ensval;
-
-  ensval.init(process);
-  ensval.run();
-  ensval.close();
-
-  return nullptr;
-}
