@@ -59,41 +59,35 @@ public:
   inline static RegisterEntry<EOF3d> registration = RegisterEntry<EOF3d>(module);
 
   size_t temp_size = 0, npack = 0;
-  int varID, levelID;
   bool missval_warning = false;
-  size_t numMissVals;
-  int ngrids;
 
   int calendar = CALENDAR_STANDARD;
 
-  double sum_w;
+  double sumWeights;
 
   CdoStreamID streamID1;
   CdoStreamID streamID2;
   CdoStreamID streamID3;
 
-  int vlistID1;
-  int taxisID1;
+  int vlistID1{ CDI_UNDEFID };
+  int taxisID1{ CDI_UNDEFID };
 
-  int nrecs;
-  int numVars;
+  VarList varList1;
 
   T_WEIGHT_MODE weight_mode;
   T_EIGEN_MODE eigen_mode;
 
-  VarList varList1;
-
-  size_t n;
-  size_t gridsizemax;
-  int nts;
-  int n_eig;
+  size_t gridsizeMax;
+  int numSteps;
+  int numEigenFunctions;
+  int numFields;
+  int numVars;
 
   Varray<double> in;
   Varray2D<int> datacounts;
   Varray3D<double> datafields;
   Varray3D<double> eigenvectors;
   Varray3D<double> eigenvalues;
-
   Varray<double> weights;
 
 public:
@@ -104,7 +98,7 @@ public:
     auto operfunc = cdo_operator_f1(operatorID);
 
     operator_input_arg("Number of eigen functions to write out");
-    n_eig = parameter_to_int(cdo_operator_argv(0));
+    numEigenFunctions = parameter_to_int(cdo_operator_argv(0));
 
     eigen_mode = get_eigenmode();
     weight_mode = get_weightmode();
@@ -119,81 +113,74 @@ public:
     varList1 = VarList(vlistID1);
 
     // COUNT NUMBER OF TIMESTEPS if EOF3D_ or EOF3D_TIME
-    nts = varList1.numSteps();
-    if (nts == -1)
+    numSteps = varList1.numSteps();
+    if (numSteps == -1)
       {
-        nts = 0;
-        while (cdo_stream_inq_timestep(streamID1, nts)) nts++;
+        numSteps = 0;
+        while (cdo_stream_inq_timestep(streamID1, numSteps)) numSteps++;
 
-        if (Options::cdoVerbose) cdo_print("Counted %d timeSteps", nts);
+        if (Options::cdoVerbose) cdo_print("Counted %d timeSteps", numSteps);
 
         cdo_stream_close(streamID1);
 
         streamID1 = cdo_open_read(0);
         vlistID1 = cdo_stream_inq_vlist(streamID1);
       }
-    else if (Options::cdoVerbose)
-      cdo_print("Found %d timeSteps", nts);
+    else if (Options::cdoVerbose) { cdo_print("Found %d timeSteps", numSteps); }
 
     taxisID1 = vlistInqTaxis(vlistID1);
 
     // reset the requested number of eigen-function to the maximum if neccessary
-    if (n_eig > nts)
+    if (numEigenFunctions > numSteps)
       {
         cdo_warning("Solving in time-space:");
         cdo_warning("Number of eigen-functions to write out is bigger than number of time-steps.");
-        cdo_warning("Setting n_eig to %d.", nts);
-        n_eig = nts;
+        cdo_warning("Setting n_eig to %d.", numSteps);
+        numEigenFunctions = numSteps;
       }
 
-    n = nts;
-
-    if (Options::cdoVerbose) cdo_print("counted %d timesteps", n);
-
     numVars = varList1.numVars();
 
     auto gridID1 = varList1.vars[0].gridID;
-    gridsizemax = vlistGridsizeMax(vlistID1);
+    gridsizeMax = varList1.gridsizeMax();
 
     // allocation of temporary fields and output structures
 
-    in = Varray<double>(gridsizemax);
+    in = Varray<double>(gridsizeMax);
     datacounts = Varray2D<int>(numVars);
     datafields = Varray3D<double>(numVars);
     eigenvectors = Varray3D<double>(numVars);
     eigenvalues = Varray3D<double>(numVars);
 
-    size_t maxlevs = 0;
-    for (varID = 0; varID < numVars; ++varID)
+    int maxLevels = 0;
+    for (int varID = 0; varID < numVars; ++varID)
       {
-        auto gridsize = vlistGridsizeMax(vlistID1);
-        size_t nlevs = varList1.vars[varID].nlevels;
-        temp_size = gridsize * nlevs;
-        auto missval = varList1.vars[varID].missval;
+        auto const &var1 = varList1.vars[varID];
+        auto gridsize = varList1.gridsizeMax();
+        temp_size = gridsize * var1.nlevels;
 
-        if (nlevs > maxlevs) maxlevs = nlevs;
+        maxLevels = std::max(maxLevels, var1.nlevels);
 
-        datafields[varID].resize(nts);
-        for (int tsID = 0; tsID < nts; ++tsID) datafields[varID][tsID].resize(temp_size, 0.0);
+        datafields[varID].resize(numSteps);
+        for (int tsID = 0; tsID < numSteps; ++tsID) datafields[varID][tsID].resize(temp_size, 0.0);
 
         datacounts[varID].resize(temp_size, 0);
 
-        eigenvectors[varID].resize(n_eig);
-        eigenvalues[varID].resize(nts);
+        eigenvectors[varID].resize(numEigenFunctions);
+        eigenvalues[varID].resize(numSteps);
 
-        for (size_t i = 0; i < n; ++i)
+        for (int i = 0; i < numSteps; ++i)
           {
-            if (i < (size_t) n_eig) eigenvectors[varID][i].resize(temp_size, missval);
-
-            eigenvalues[varID][i].resize(1, missval);
+            if (i < numEigenFunctions) eigenvectors[varID][i].resize(temp_size, var1.missval);
+            eigenvalues[varID][i].resize(1, var1.missval);
           }
       }
 
     if (Options::cdoVerbose)
-      cdo_print("Allocated eigenvalue/eigenvector with nts=%d, n=%d, gridsize=%zu for processing in %s", nts, n, gridsizemax,
-                "time_space");
+      cdo_print("Allocate eigenvalue/eigenvector object with numSteps=%d and gridSize=%zu for processing in %s (%zu Bytes)",
+                numSteps, gridsizeMax, "time_space", numSteps * gridsizeMax * 8);
 
-    weights = Varray<double>(maxlevs * gridsizemax, 1.0);
+    weights = Varray<double>(maxLevels * gridsizeMax, 1.0);
 
     if (weight_mode == WEIGHT_ON)
       {
@@ -205,8 +192,8 @@ public:
           }
         else
           {
-            for (size_t k = 1; k < maxlevs; ++k)
-              for (size_t i = 0; i < gridsizemax; ++i) weights[k * gridsizemax + i] = weights[i];
+            for (int k = 1; k < maxLevels; ++k)
+              for (size_t i = 0; i < gridsizeMax; ++i) weights[k * gridsizeMax + i] = weights[i];
           }
       }
   }
@@ -219,22 +206,20 @@ public:
     // read the data and create covariance matrices for each var & level
     while (true)
       {
-        nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            auto const &var1 = varList1.vars[varID];
+            size_t numMissVals;
+            cdo_read_field(streamID1, in.data(), &numMissVals);
 
-            auto gridsize = varList1.vars[varID].gridsize;
-            auto missval = varList1.vars[varID].missval;
-
-            cdo_read_record(streamID1, in.data(), &numMissVals);
-
-            auto offset = gridsize * levelID;
-            for (size_t i = 0; i < gridsize; ++i)
+            auto offset = var1.gridsize * levelID;
+            for (size_t i = 0; i < var1.gridsize; ++i)
               {
-                if (dbl_is_not_equal(in[i], missval))
+                if (fp_is_not_equal(in[i], var1.missval))
                   {
                     datafields[varID][tsID][offset + i] = in[i];
                     datacounts[varID][offset + i]++;
@@ -259,13 +244,10 @@ public:
 
     Varray<size_t> pack(temp_size);  // TODO
 
-    for (varID = 0; varID < numVars; ++varID)
+    for (int varID = 0; varID < numVars; ++varID)
       {
-        const auto &var1 = varList1.vars[varID];
-        auto gridsize = var1.gridsize;
-        auto nlevs = var1.nlevels;
-        auto missval = var1.missval;
-        temp_size = gridsize * nlevs;
+        auto const &var1 = varList1.vars[varID];
+        temp_size = var1.gridsize * var1.nlevels;
 
         if (Options::cdoVerbose)
           {
@@ -284,11 +266,11 @@ public:
               }
           }
 
-        sum_w = 1;
+        sumWeights = 1;
         if (weight_mode == WEIGHT_ON)
           {
-            sum_w = 0;
-            for (size_t i = 0; i < npack; ++i) sum_w += weights[pack[i]];
+            sumWeights = 0;
+            for (size_t i = 0; i < npack; ++i) sumWeights += weights[pack[i]];
           }
 
         if (npack < 1)
@@ -297,48 +279,53 @@ public:
             continue;
           }
 
-        Varray<double> eigv(n);
-        Varray2D<double> covar(nts);
-        for (int j1 = 0; j1 < nts; ++j1) covar[j1].resize(nts);
+        if (Options::cdoVerbose)
+          cdo_print("Allocate covar-matrix with %dx%d elements (npack=%zu) for %s (%zu Bytes)", numSteps, numSteps, npack,
+                    var1.name, numSteps * numSteps * sizeof(double));
+
+        Varray<double> eigv(numSteps);
+        Varray2D<double> covar(numSteps);
+        for (int j1 = 0; j1 < numSteps; ++j1) covar[j1].resize(numSteps);
 
         if (Options::cdoVerbose)
           {
-            cdo_print("varID %d allocated eigv and cov with nts=%d and n=%d", varID + 1, nts, n);
-            cdo_print("   npack=%zu, nts=%d temp_size=%zu", npack, nts, temp_size);
+            cdo_print("varID %d allocated eigv and cov with %dx%d elements", varID + 1, numSteps, numSteps);
+            cdo_print("   npack=%zu, nts=%d temp_size=%zu", npack, numSteps, temp_size);
           }
 
-        const auto &data = datafields[varID];
+        auto const &data = datafields[varID];
 #ifdef _OPENMP
 #pragma omp parallel for default(shared) schedule(static)
 #endif
-        for (int j1 = 0; j1 < nts; ++j1)
+        for (int j1 = 0; j1 < numSteps; ++j1)
           {
-            const auto &df1p = data[j1];
-            for (int j2 = j1; j2 < nts; ++j2)
+            auto const &df1p = data[j1];
+            for (int j2 = j1; j2 < numSteps; ++j2)
               {
-                const auto &df2p = data[j2];
+                auto const &df2p = data[j2];
                 double sum = 0.0;
-                for (size_t i = 0; i < npack; ++i) sum += weights[pack[i] % gridsizemax] * df1p[pack[i]] * df2p[pack[i]];
-                covar[j2][j1] = covar[j1][j2] = sum / sum_w / nts;
+                for (size_t i = 0; i < npack; ++i) sum += weights[pack[i] % gridsizeMax] * df1p[pack[i]] * df2p[pack[i]];
+                covar[j2][j1] = covar[j1][j2] = sum / sumWeights / numSteps;
               }
           }
 
         if (Options::cdoVerbose) cdo_print("calculated cov-matrix");
 
         // SOLVE THE EIGEN PROBLEM
-        if (Options::cdoVerbose) cdo_print("Processed correlation matrix for var %d | npack: %zu", varID + 1, n);
+        if (Options::cdoVerbose) cdo_print("Processed correlation matrix for var %d | npack: %zu", varID + 1, numSteps);
 
         if (eigen_mode == JACOBI)
-          parallel_eigen_solution_of_symmetric_matrix(covar, eigv, n, __func__);
+          parallel_eigen_solution_of_symmetric_matrix(covar, eigv, numSteps, __func__);
         else
-          eigen_solution_of_symmetric_matrix(covar, eigv, n, __func__);
+          eigen_solution_of_symmetric_matrix(covar, eigv, numSteps, __func__);
         // NOW: covar contains the eigenvectors, eigv the eigenvalues
 
-        if (Options::cdoVerbose) cdo_print("Processed SVD decomposition for var %d from %dx%d matrix", varID + 1, n, n);
+        if (Options::cdoVerbose)
+          cdo_print("Processed SVD decomposition for var %d from %dx%d matrix", varID + 1, numSteps, numSteps);
 
-        for (size_t eofID = 0; eofID < n; eofID++) eigenvalues[varID][eofID][0] = eigv[eofID];
+        for (int eofID = 0; eofID < numSteps; eofID++) eigenvalues[varID][eofID][0] = eigv[eofID];
 
-        for (int eofID = 0; eofID < n_eig; eofID++)
+        for (int eofID = 0; eofID < numEigenFunctions; eofID++)
           {
             double *eigenvec = eigenvectors[varID][eofID].data();
 
@@ -348,8 +335,7 @@ public:
             for (size_t i = 0; i < npack; ++i)
               {
                 double sum = 0.0;
-                for (int j = 0; j < nts; ++j) sum += datafields[varID][j][pack[i]] * covar[eofID][j];
-
+                for (int j = 0; j < numSteps; ++j) { sum += datafields[varID][j][pack[i]] * covar[eofID][j]; }
                 eigenvec[pack[i]] = sum;
               }
 
@@ -359,7 +345,7 @@ public:
 #ifdef _OPENMP
 #pragma omp parallel for default(shared) schedule(static) reduction(+ : sum)
 #endif
-            for (size_t i = 0; i < npack; ++i) sum += weights[pack[i] % gridsizemax] * eigenvec[pack[i]] * eigenvec[pack[i]];
+            for (size_t i = 0; i < npack; ++i) sum += weights[pack[i] % gridsizeMax] * eigenvec[pack[i]] * eigenvec[pack[i]];
 
             if (sum > 0)
               {
@@ -374,20 +360,22 @@ public:
 #ifdef _OPENMP
 #pragma omp parallel for default(shared) schedule(static)
 #endif
-                for (size_t i = 0; i < npack; ++i) eigenvec[pack[i]] = missval;
+                for (size_t i = 0; i < npack; ++i) eigenvec[pack[i]] = var1.missval;
               }
           }  // for ( eofID = 0; eofID < n_eig; eofID++ )
-      }      // for ( varID = 0; varID < numVars; varID++ )
+      }  // for ( varID = 0; varID < numVars; varID++ )
 
     // write files with eigenvalues (ID3) and eigenvectors (ID2)
 
     // eigenvalues
     streamID2 = cdo_open_write(1);
 
+    auto vDateTime0 = cdiDateTime_set(10101, 0);
+
     auto vlistID2 = vlistDuplicate(vlistID1);
     auto taxisID2 = taxisDuplicate(taxisID1);
-    taxisDefRdate(taxisID2, 0);
-    taxisDefRtime(taxisID2, 0);
+    cdiDefKeyInt(taxisID2, CDI_GLOBAL, CDI_KEY_DATATYPE, CDI_DATATYPE_FLT64);
+    taxisDefRdatetime(taxisID2, vDateTime0);
     vlistDefTaxis(vlistID2, taxisID2);
 
     auto gridID2 = gridCreate(GRID_LONLAT, 1);
@@ -397,32 +385,32 @@ public:
     gridDefXvals(gridID2, &xvals);
     gridDefYvals(gridID2, &yvals);
 
-    ngrids = vlistNumGrids(vlistID2);
-    for (int i = 0; i < ngrids; ++i) vlistChangeGridIndex(vlistID2, i, gridID2);
+    auto numGrids = vlistNumGrids(vlistID2);
+    for (int i = 0; i < numGrids; ++i) vlistChangeGridIndex(vlistID2, i, gridID2);
 
     auto zaxisID2 = zaxisCreate(ZAXIS_GENERIC, 1);
-    double zvals = 0;
-    zaxisDefLevels(zaxisID2, &zvals);
+    double zValue = 0;
+    zaxisDefLevels(zaxisID2, &zValue);
     cdiDefKeyString(zaxisID2, CDI_GLOBAL, CDI_KEY_NAME, "zaxis_Reduced");
     cdiDefKeyString(zaxisID2, CDI_GLOBAL, CDI_KEY_LONGNAME, "Reduced zaxis from EOF3D - only one eigen value per 3D eigen vector");
 
-    auto nzaxis = vlistNumZaxis(vlistID2);
-    for (int i = 0; i < nzaxis; ++i) vlistChangeZaxisIndex(vlistID2, i, zaxisID2);
+    auto numZaxes = vlistNumZaxis(vlistID2);
+    for (int i = 0; i < numZaxes; ++i) vlistChangeZaxisIndex(vlistID2, i, zaxisID2);
 
     // eigenvectors
     streamID3 = cdo_open_write(2);
 
     auto vlistID3 = vlistDuplicate(vlistID1);
     auto taxisID3 = taxisDuplicate(taxisID1);
-    taxisDefRdate(taxisID3, 0);
-    taxisDefRtime(taxisID3, 0);
+    cdiDefKeyInt(taxisID3, CDI_GLOBAL, CDI_KEY_DATATYPE, CDI_DATATYPE_FLT64);
+    taxisDefRdatetime(taxisID3, vDateTime0);
     vlistDefTaxis(vlistID3, taxisID3);
 
     cdo_def_vlist(streamID2, vlistID2);
     cdo_def_vlist(streamID3, vlistID3);
 
-    auto julianDate = julianDate_encode(calendar, cdiDateTime_set(10101, 0));
-    for (tsID = 0; tsID < (int) n; ++tsID)
+    auto julianDate = julianDate_encode(calendar, vDateTime0);
+    for (tsID = 0; tsID < numSteps; ++tsID)
       {
         julianDate = julianDate_add_seconds(julianDate, 60);
         auto vDateTime = julianDate_decode(calendar, julianDate);
@@ -430,33 +418,32 @@ public:
         taxisDefVdatetime(taxisID2, vDateTime);
         cdo_def_timestep(streamID2, tsID);
 
-        if (tsID < n_eig)
+        if (tsID < numEigenFunctions)
           {
             taxisDefVdatetime(taxisID3, vDateTime);
             cdo_def_timestep(streamID3, tsID);
           }
 
-        for (varID = 0; varID < numVars; ++varID)
+        for (int varID = 0; varID < numVars; ++varID)
           {
-            auto missval = varList1.vars[varID].missval;
-            auto nlevs = varList1.vars[varID].nlevels;
-            for (levelID = 0; levelID < (int) nlevs; ++levelID)
+            auto const &var1 = varList1.vars[varID];
+            for (int levelID = 0; levelID < var1.nlevels; ++levelID)
               {
-                auto offset = levelID * gridsizemax;
-                if (tsID < n_eig)
+                auto offset = levelID * gridsizeMax;
+                if (tsID < numEigenFunctions)
                   {
-                    numMissVals = array_num_mv(gridsizemax, &eigenvectors[varID][tsID][offset], missval);
-                    cdo_def_record(streamID3, varID, levelID);
-                    cdo_write_record(streamID3, &eigenvectors[varID][tsID][offset], numMissVals);
+                    auto numMissVals = array_num_mv(gridsizeMax, &eigenvectors[varID][tsID][offset], var1.missval);
+                    cdo_def_field(streamID3, varID, levelID);
+                    cdo_write_field(streamID3, &eigenvectors[varID][tsID][offset], numMissVals);
                   }
               }
 
-            numMissVals = (DBL_IS_EQUAL(eigenvalues[varID][tsID][0], missval)) ? 1 : 0;
+            auto numMissVals = (fp_is_equal(eigenvalues[varID][tsID][0], var1.missval)) ? 1 : 0;
 
-            cdo_def_record(streamID2, varID, 0);
-            cdo_write_record(streamID2, eigenvalues[varID][tsID].data(), numMissVals);
+            cdo_def_field(streamID2, varID, 0);
+            cdo_write_field(streamID2, eigenvalues[varID][tsID].data(), numMissVals);
           }  // for ( varID = 0; ... )
-      }      // for ( tsID = 0; ... )
+      }  // for ( tsID = 0; ... )
   }
 
   void
