@@ -36,26 +36,23 @@ public:
   inline static RegisterEntry<Eofcoeff3d> registration = RegisterEntry<Eofcoeff3d>(module);
 
   double missval1 = -999, missval2 = -999;
-  int varID, levelID;
-  int nrecs;
-  size_t numMissVals;
+  int numFields;
 
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int vlistID1;
-  int vlistID2;
+  int vlistID1{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
   VarList varList1;
   VarList varList2;
 
   int gridID1;
 
-  int taxisID2;
+  int taxisID2{ CDI_UNDEFID };
 
-  size_t gridsize;
+  size_t gridsizeMax;
   int numVars;
-  int neof;
   int numLevels;
   int taxisID3;
 
@@ -64,8 +61,6 @@ public:
 
   std::vector<CdoStreamID> streamIDs;
 
-  int eofID = 0;
-
 public:
   void
   init() override
@@ -86,50 +81,54 @@ public:
     gridID1 = varList1.vars[0].gridID;
     int gridID2 = varList2.vars[0].gridID;
 
-    gridsize = vlistGridsizeMax(vlistID1);
-    if (gridsize != vlistGridsizeMax(vlistID2)) cdo_abort("Gridsize of input files does not match!");
+    gridsizeMax = varList1.gridsizeMax();
+    if (gridsizeMax != varList2.gridsizeMax()) cdo_abort("Gridsize of input files does not match!");
 
     if (vlistNumGrids(vlistID2) > 1 || vlistNumGrids(vlistID1) > 1) cdo_abort("Too many different grids in input");
 
     numVars = (varList1.numVars() == varList2.numVars()) ? varList1.numVars() : -1;
     numLevels = varList1.vars[0].nlevels;
 
-    if (gridID1 != gridID2) cdo_compare_grids(gridID1, gridID2);
+    cdo_compare_grids(gridID1, gridID2);
 
     fileSuffix = FileUtils::gen_suffix(cdo_inq_filetype(streamID1), vlistID1, cdo_get_stream_name(0));
 
     eof = FieldVector3D(numVars);
-    for (varID = 0; varID < numVars; ++varID) eof[varID].resize(numLevels);
+    for (int varID = 0; varID < numVars; ++varID) eof[varID].resize(numLevels);
   }
 
   void
   run() override
   {
-    while (1)
-      {
-        nrecs = cdo_stream_inq_timestep(streamID1, eofID);
-        if (nrecs == 0) break;
+    int neof = 0;
 
-        for (int recID = 0; recID < nrecs; ++recID)
-          {
-            cdo_inq_record(streamID1, &varID, &levelID);
-            missval1 = varList1.vars[varID].missval;
-            eof[varID][levelID].resize((eofID == 0) ? 1 : eofID + 1);
-            eof[varID][levelID][eofID].grid = gridID1;
-            eof[varID][levelID][eofID].missval = missval1;
-            eof[varID][levelID][eofID].resize(gridsize);
-            ranges::fill(eof[varID][levelID][eofID].vec_d, missval1);
+    {
+      int eofID = 0;
+      while (1)
+        {
+          numFields = cdo_stream_inq_timestep(streamID1, eofID);
+          if (numFields == 0) break;
 
-            if (varID >= numVars) cdo_abort("Internal error - varID >= nvars");
-            if (levelID >= numLevels) cdo_abort("Internal error - levelID >= nlevs");
+          for (int fieldID = 0; fieldID < numFields; ++fieldID)
+            {
+              auto [varID, levelID] = cdo_inq_field(streamID1);
+              missval1 = varList1.vars[varID].missval;
+              eof[varID][levelID].resize((eofID == 0) ? 1 : eofID + 1);
+              eof[varID][levelID][eofID].grid = gridID1;
+              eof[varID][levelID][eofID].missval = missval1;
+              eof[varID][levelID][eofID].resize(gridsizeMax);
+              std::ranges::fill(eof[varID][levelID][eofID].vec_d, missval1);
 
-            cdo_read_record(streamID1, eof[varID][levelID][eofID].vec_d.data(), &numMissVals);
-            eof[varID][levelID][eofID].numMissVals = numMissVals;
-          }
-        eofID++;
-      }
+              if (varID >= numVars) cdo_abort("Internal error - varID >= nvars");
+              if (levelID >= numLevels) cdo_abort("Internal error - levelID >= nlevs");
 
-    neof = eofID;
+              cdo_read_field(streamID1, eof[varID][levelID][eofID]);
+            }
+          eofID++;
+        }
+
+      neof = eofID;
+    }
 
     if (Options::cdoVerbose) cdo_print("%s contains %i eof's", cdo_get_stream_name(0), neof);
     // Create 1x1 Grid for output
@@ -151,40 +150,36 @@ public:
 
     auto vlistID3 = vlistDuplicate(vlistID2);
 
-    auto ngrids = vlistNumGrids(vlistID3);
-    for (int i = 0; i < ngrids; ++i) vlistChangeGridIndex(vlistID3, i, gridID3);
+    auto numGrids = vlistNumGrids(vlistID3);
+    for (int i = 0; i < numGrids; ++i) vlistChangeGridIndex(vlistID3, i, gridID3);
 
-    auto nzaxis = vlistNumZaxis(vlistID3);
-    for (int i = 0; i < nzaxis; ++i) vlistChangeZaxisIndex(vlistID3, i, zaxisID3);
+    auto numZaxes = vlistNumZaxis(vlistID3);
+    for (int i = 0; i < numZaxes; ++i) vlistChangeZaxisIndex(vlistID3, i, zaxisID3);
 
     vlistDefTaxis(vlistID3, taxisID3);
-    for (varID = 0; varID < numVars; ++varID) vlistDefVarTimetype(vlistID3, varID, TIME_VARYING);
+    for (int varID = 0; varID < numVars; ++varID) vlistDefVarTimetype(vlistID3, varID, TIME_VARYING);
 
     // open streams for eofcoeff output
 
     streamIDs = std::vector<CdoStreamID>(neof);
-    for (eofID = 0; eofID < neof; eofID++)
+    for (int eofID = 0; eofID < neof; eofID++)
       {
         auto fileName = cdo_get_obase() + string_format("%5.5i", eofID);
         if (fileSuffix.size() > 0) fileName += fileSuffix;
 
-        streamIDs[eofID] = cdo_open_write(fileName.c_str());
-
-        if (Options::cdoVerbose)
-          cdo_print("opened %s ('w')  as stream%i for %i. eof", fileName, streamIDs[eofID]->get_id(), eofID + 1);
-
+        streamIDs[eofID] = open_write(fileName);
         cdo_def_vlist(streamIDs[eofID], vlistID3);
       }
 
     // ALLOCATE temporary fields for data read and write
     Field in;
-    in.resize(gridsize);
+    in.resize(gridsizeMax);
     in.grid = gridID1;
     FieldVector2D out(numVars);
-    for (varID = 0; varID < numVars; ++varID)
+    for (int varID = 0; varID < numVars; ++varID)
       {
         out[varID].resize(neof);
-        for (eofID = 0; eofID < neof; eofID++)
+        for (int eofID = 0; eofID < neof; eofID++)
           {
             out[varID][eofID].missval = missval1;
             out[varID][eofID].numMissVals = 0;
@@ -195,11 +190,11 @@ public:
     int tsID = 0;
     while (1)
       {
-        nrecs = cdo_stream_inq_timestep(streamID2, tsID);
-        if (nrecs == 0) break;
+        numFields = cdo_stream_inq_timestep(streamID2, tsID);
+        if (numFields == 0) break;
 
-        for (varID = 0; varID < numVars; ++varID)
-          for (eofID = 0; eofID < neof; eofID++)
+        for (int varID = 0; varID < numVars; ++varID)
+          for (int eofID = 0; eofID < neof; eofID++)
             {
               out[varID][eofID].vec_d[0] = 0;
               out[varID][eofID].grid = gridID3;
@@ -208,26 +203,25 @@ public:
 
         cdo_taxis_copy_timestep(taxisID3, taxisID2);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            cdo_inq_record(streamID2, &varID, &levelID);
-            cdo_read_record(streamID2, in.vec_d.data(), &in.numMissVals);
+            auto [varID, levelID] = cdo_inq_field(streamID2);
+            cdo_read_field(streamID2, in);
             missval2 = varList2.vars[varID].missval;
 
-            for (eofID = 0; eofID < neof; eofID++)
+            for (int eofID = 0; eofID < neof; eofID++)
               {
-                if (recID == 0) cdo_def_timestep(streamIDs[eofID], tsID);
+                if (fieldID == 0) cdo_def_timestep(streamIDs[eofID], tsID);
 
-                numMissVals = 0;
-                for (size_t i = 0; i < gridsize; ++i)
+                // size_t numMissVals = 0;
+                for (size_t i = 0; i < gridsizeMax; ++i)
                   {
-                    if (!DBL_IS_EQUAL(in.vec_d[i], missval2) && !DBL_IS_EQUAL(eof[varID][levelID][eofID].vec_d[i], missval1))
+                    if (!fp_is_equal(in.vec_d[i], missval2) && !fp_is_equal(eof[varID][levelID][eofID].vec_d[i], missval1))
                       {
                         // out[varID][eofID].vec_d[0] += w[i]*in.vec_d[i]*eof[varID][levelID][eofID].vec_d[i];
                         out[varID][eofID].vec_d[0] += in.vec_d[i] * eof[varID][levelID][eofID].vec_d[i];
                       }
-                    else
-                      numMissVals += 1;
+                    // else { numMissVals += 1; }
                   }
                 /*
                 if ( numMissVals ) {
@@ -241,12 +235,12 @@ public:
             if (levelID >= numLevels) cdo_abort("Internal error - levelID >= nlevs");
           }
 
-        for (eofID = 0; eofID < neof; eofID++)
+        for (int eofID = 0; eofID < neof; eofID++)
           {
-            for (varID = 0; varID < numVars; ++varID)
+            for (int varID = 0; varID < numVars; ++varID)
               {
-                cdo_def_record(streamIDs[eofID], varID, 0);
-                cdo_write_record(streamIDs[eofID], out[varID][eofID].vec_d.data(), out[varID][eofID].numMissVals);
+                cdo_def_field(streamIDs[eofID], varID, 0);
+                cdo_write_field(streamIDs[eofID], out[varID][eofID]);
               }
           }
 
@@ -257,7 +251,7 @@ public:
   void
   close() override
   {
-    for (eofID = 0; eofID < neof; eofID++) cdo_stream_close(streamIDs[eofID]);
+    for (auto &streamID : streamIDs) cdo_stream_close(streamID);
 
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
