@@ -10,20 +10,31 @@
 #include "process_int.h"
 #include "cdo_options.h"
 
-class ModuleEstFreq
+class EstFreq : public Process
 {
-  int tsID = 0;
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "EstFreq",
+    .operators = { { "estfreq" } },
+    .aliases = {},
+    .mode = INTERNAL,    // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<EstFreq> registration = RegisterEntry<EstFreq>(module);
+
   int fyear = 0, lyear, fmonth = 0, lmonth, dummy;
   int step_per_year = 0, currentyear, currentmon;
 
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int vlistID1;
-  int vlistID2;
+  int vlistID1{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   Field field;
   VarList varList1;
@@ -34,11 +45,8 @@ class ModuleEstFreq
 
 public:
   void
-  init(void *process)
+  init() override
   {
-
-    cdo_initialize(process);
-
     operator_check_argc(0);
 
     dataIsUnchanged = data_is_unchanged();
@@ -58,16 +66,17 @@ public:
 
     ntsteps = vlistNtsteps(vlistID1);
 
-    varList_init(varList1, vlistID1);
+    varList1 = VarList(vlistID1);
   }
-  void
-  run()
-  {
 
+  void
+  run() override
+  {
+    int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdiDate_decode(taxisInqVdatetime(taxisID1).date, &currentyear, &currentmon, &dummy);
 
@@ -86,18 +95,17 @@ public:
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            cdo_def_record(streamID2, varID, levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            cdo_def_field(streamID2, varID, levelID);
 
-            if (dataIsUnchanged) { cdo_copy_record(streamID2, streamID1); }
+            if (dataIsUnchanged) { cdo_copy_field(streamID2, streamID1); }
             else
               {
-                field.init(varList1[varID]);
-                cdo_read_record(streamID1, field);
-                cdo_write_record(streamID2, field);
+                field.init(varList1.vars[varID]);
+                cdo_read_field(streamID1, field);
+                cdo_write_field(streamID2, field);
               }
           }
 
@@ -118,15 +126,15 @@ public:
                  ntsteps, lyear, fyear);
         double covered_years = lyear - fyear + 1.0;
         double freq = ntsteps / covered_years;
-        if (DBL_IS_EQUAL(freq, 1.))
+        if (fp_is_equal(freq, 1.))
           frequency = "yr";
-        else if (DBL_IS_EQUAL(freq, 12.))
+        else if (fp_is_equal(freq, 12.))
           frequency = "mon";
-        else if (IS_EQUAL(freq, 365.) || IS_EQUAL(freq, 365.25) || IS_EQUAL(freq, 366.))
+        else if (is_equal(freq, 365.) || is_equal(freq, 365.25) || is_equal(freq, 366.))
           frequency = "day";
-        else if (IS_EQUAL(freq, 365. * 4) || IS_EQUAL(freq, 365.25 * 4) || IS_EQUAL(freq, 366. * 4))
+        else if (is_equal(freq, 365. * 4) || is_equal(freq, 365.25 * 4) || is_equal(freq, 366. * 4))
           frequency = "6hr";
-        else if (IS_EQUAL(freq, 365. * 8) || IS_EQUAL(freq, 365.25 * 8) || IS_EQUAL(freq, 366. * 8))
+        else if (is_equal(freq, 365. * 8) || is_equal(freq, 365.25 * 8) || is_equal(freq, 366. * 8))
           frequency = "3hr";
         else
           {
@@ -159,24 +167,13 @@ public:
     if (Options::cdoVerbose) printf("Your file indicates a frequency of '%s'.\n", frequency);
     cdiDefAttTxt(vlistID2, CDI_GLOBAL, "frequency", 3, frequency);
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID1);
     cdo_stream_close(streamID2);
 
     vlistDestroy(vlistID2);
-
-    cdo_finish();
   }
 };
-
-void *
-EstFreq(void *process)
-{
-  ModuleEstFreq estFreq;
-  estFreq.init(process);
-  estFreq.run();
-  estFreq.close();
-  return nullptr;
-}
