@@ -75,7 +75,7 @@ exprs_from_file(const std::vector<std::string> &exprArgv)
 constexpr int MaxParams = 4096;
 
 static std::size_t
-replace_all(std::string &inout, const std::string &what, const std::string &with)
+replace_all(std::string &inout, std::string const &what, std::string const &with)
 {
   std::size_t count{};
   for (std::string::size_type pos{}; inout.npos != (pos = inout.find(what.data(), pos, what.length()));
@@ -87,7 +87,7 @@ replace_all(std::string &inout, const std::string &what, const std::string &with
 }
 
 static std::string
-exprs_expand(std::string &exprString, const VarList &varList)
+exprs_expand(std::string const &exprString, const VarList &varList)
 {
   auto replaceTemplate = false;
   std::string templateName = "_ALL_";
@@ -95,7 +95,7 @@ exprs_expand(std::string &exprString, const VarList &varList)
   if (exprString.find(templateName) != std::string::npos)
     {
       replaceTemplate = true;
-      for (const auto &var : varList.vars)
+      for (auto const &var : varList.vars)
         {
           if (templateName == var.name)
             {
@@ -108,13 +108,13 @@ exprs_expand(std::string &exprString, const VarList &varList)
   if (replaceTemplate)
     {
       std::string exprStringNew{};
-      auto exprStringArgv = split_with_seperator(exprString, ';');
-      for (auto &string : exprStringArgv)
+      auto exprStringArgv = split_string(exprString, ";");
+      for (auto const &string : exprStringArgv)
         {
           if (string.find(templateName) == std::string::npos) { exprStringNew += string + ";"; }
           else
             {
-              for (const auto &var : varList.vars)
+              for (auto const &var : varList.vars)
                 {
                   auto tmpString = string;
                   replace_all(tmpString, templateName, var.name);
@@ -132,10 +132,9 @@ exprs_expand(std::string &exprString, const VarList &varList)
 static void
 params_init(std::vector<ParamEntry> &params, const VarList &varList)
 {
-  for (const auto &var : varList.vars)
+  for (auto const &var : varList.vars)
     {
       auto &param = params[var.ID];
-      auto stdname = cdo::inq_key_string(varList.vlistID, var.ID, CDI_KEY_STDNAME);
 
       param.type = ParamType::VAR;
       param.isValid = true;
@@ -151,21 +150,21 @@ params_init(std::vector<ParamEntry> &params, const VarList &varList)
       param.name = var.name;
       if (var.longname.size()) param.longname = var.longname;
       if (var.units.size()) param.units = var.units;
-      if (stdname.size()) param.stdname = stdname;
+      if (var.stdname.size()) param.stdname = var.stdname;
     }
 }
 
 static void
 params_delete(const std::vector<ParamEntry> &params)
 {
-  for (auto &param : params)
+  for (auto const &param : params)
     {
       if (param.data) delete[] param.data;
     }
 }
 
 static void
-params_add_coord(ParseParamType &parseArg, int coord, int cdiID, size_t size, const std::string &units, const std::string &longname)
+params_add_coord(ParseParamType &parseArg, int coord, int cdiID, size_t size, std::string const &units, std::string const &longname)
 {
   auto ncoords = parseArg.numCoords;
   if (ncoords >= parseArg.maxCoords) cdo_abort("Too many coordinates (limit=%d)", parseArg.maxCoords);
@@ -184,8 +183,8 @@ params_add_coord(ParseParamType &parseArg, int coord, int cdiID, size_t size, co
 static void
 params_add_coordinates(int vlistID, ParseParamType &parseArg)
 {
-  auto ngrids = vlistNumGrids(vlistID);
-  for (int index = 0; index < ngrids; ++index)
+  auto numGrids = vlistNumGrids(vlistID);
+  for (int index = 0; index < numGrids; ++index)
     {
       auto gridID = vlistGrid(vlistID, index);
       auto size = gridInqSize(gridID);
@@ -199,8 +198,8 @@ params_add_coordinates(int vlistID, ParseParamType &parseArg)
       params_add_coord(parseArg, 'g', gridID, size, "", "grid cell indices");
     }
 
-  auto nzaxis = vlistNumZaxis(vlistID);
-  for (int index = 0; index < nzaxis; ++index)
+  auto numZaxes = vlistNumZaxis(vlistID);
+  for (int index = 0; index < numZaxes; ++index)
     {
       auto zaxisID = vlistZaxis(vlistID, index);
       auto size = zaxisInqSize(zaxisID);
@@ -217,8 +216,8 @@ params_add_ts(ParseParamType &parseArg)
 {
   auto &params = parseArg.params;
 
-  auto varID = parseArg.nparams;
-  if (varID >= parseArg.maxparams) cdo_abort("Too many parameter (limit=%d)", parseArg.maxparams);
+  auto varID = parseArg.numParams;
+  if (varID >= parseArg.maxParams) cdo_abort("Too many parameter (limit=%d)", parseArg.maxParams);
 
   auto &param = params[varID];
   param.name = "_timestep_info";
@@ -228,8 +227,7 @@ params_add_ts(ParseParamType &parseArg)
   param.ngp = CoordIndex::LEN;
   param.nlev = 1;
 
-  parseArg.nparams++;
-  parseArg.cnparams++;
+  parseArg.numParams++;
 
   return varID;
 }
@@ -238,14 +236,13 @@ static void
 parse_param_init(ParseParamType &parseArg, int vlistID, int pointID, int zonalID, int surfaceID)
 {
   auto numVars = vlistNvars(vlistID);
-  auto ngrids = vlistNumGrids(vlistID);
-  auto nzaxis = vlistNumZaxis(vlistID);
-  auto maxCoords = ngrids * 5 + nzaxis * 3;
+  auto numGrids = vlistNumGrids(vlistID);
+  auto numZaxes = vlistNumZaxis(vlistID);
+  auto maxCoords = numGrids * 5 + numZaxes * 3;
 
-  parseArg.maxparams = MaxParams;
+  parseArg.maxParams = MaxParams;
   parseArg.params.resize(MaxParams);
-  parseArg.nparams = numVars;
-  parseArg.cnparams = numVars;
+  parseArg.numParams = numVars;
   parseArg.numVars1 = numVars;
   parseArg.init = true;
   parseArg.debug = (Options::cdoVerbose != 0);
@@ -263,8 +260,8 @@ genZonalID(int vlistID)
 {
   int zonalID = -1;
 
-  auto ngrids = vlistNumGrids(vlistID);
-  for (int index = 0; index < ngrids; ++index)
+  auto numGrids = vlistNumGrids(vlistID);
+  for (int index = 0; index < numGrids; ++index)
     {
       auto gridID = vlistGrid(vlistID, index);
       auto gridtype = gridInqType(gridID);
@@ -280,7 +277,7 @@ genZonalID(int vlistID)
 }
 
 static void
-set_date_and_time(ParamEntry &varts, int calendar, int tsID, const CdiDateTime &vDateTime0, const CdiDateTime &vDateTime)
+set_date_and_time(ParamEntry &varts, int calendar, int tsID, CdiDateTime const &vDateTime0, CdiDateTime const &vDateTime)
 {
   double jdelta = 0.0;
 
@@ -330,13 +327,14 @@ public:
   };
   inline static RegisterEntry<Exprf> registration = RegisterEntry<Exprf>(module);
 
+private:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int vlistID2;
+  int vlistID2{ CDI_UNDEFID };
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   int vartsID;
 
@@ -354,6 +352,207 @@ public:
   int surfaceID;
   int calendar;
 
+  void
+  parse_expressions()
+  {
+    void *scanner = nullptr;
+    yylex_init(&scanner);
+    yyset_extra(&parseArg, scanner);
+    yy_scan_string(exprString.c_str(), scanner);
+    yyparse(parseArg, scanner);
+    yylex_destroy(scanner);
+  }
+
+  void
+  allocate_params()
+  {
+    auto &params = parseArg.params;
+    for (int varID = 0; varID < numVars1; ++varID)
+      {
+        if (parseArg.needed[varID])
+          {
+            auto nItems = std::max((size_t) 4, params[varID].ngp * params[varID].nlev);
+            params[varID].data = new double[nItems];
+          }
+      }
+
+    for (int varID = parseArg.numVars1, n = parseArg.numParams; varID < n; ++varID)
+      {
+        auto nItems = std::max((size_t) 4, params[varID].ngp * params[varID].nlev);
+        params[varID].data = new double[nItems];
+      }
+  }
+
+  void
+  read_coordinates_hgrid(std::vector<double> &cdata, size_t csize, int cdiID, int coord)
+  {
+    {
+      auto gridID = cdiID;
+      auto ngp = csize;
+      cdata.resize(ngp);
+      if (coord == 'x' || coord == 'y')
+        {
+          gridID = generate_full_point_grid(gridID);
+          if (!gridHasCoordinates(gridID)) cdo_abort("Cell center coordinates missing!");
+
+          if (coord == 'x') gridInqXvals(gridID, cdata.data());
+          if (coord == 'y') gridInqYvals(gridID, cdata.data());
+
+          if (gridID != cdiID) gridDestroy(gridID);
+        }
+      else if (coord == 'a') { gridcell_areas(gridID, cdata); }
+      else if (coord == 'w')
+        {
+          cdata[0] = 1;
+          if (ngp > 1)
+            {
+              auto wstatus = gridcell_weights(gridID, cdata);
+              if (wstatus) cdo_warning("Grid cell bounds not available, using constant grid cell area weights!");
+            }
+        }
+      else if (coord == 'g')
+        {
+          for (size_t k = 0; k < ngp; ++k) cdata[k] = k + 1;
+        }
+    }
+  }
+
+  void
+  read_coordinates_vgrid(std::vector<double> &cdata, size_t csize, int cdiID, int coord)
+  {
+    {
+      auto zaxisID = cdiID;
+      auto nlev = csize;
+      cdata.resize(nlev);
+      if (coord == 'z') { cdo_zaxis_inq_levels(zaxisID, cdata.data()); }
+      else if (coord == 'i')
+        {
+          for (size_t k = 0; k < nlev; ++k) cdata[k] = k + 1;
+          cdo_zaxis_inq_levels(zaxisID, cdata.data());
+        }
+      else if (coord == 'd')
+        {
+          std::ranges::fill(cdata, 1.0);
+          if (zaxisInqLbounds(zaxisID, nullptr) && zaxisInqUbounds(zaxisID, nullptr))
+            {
+              std::vector<double> lbounds(nlev), ubounds(nlev);
+              zaxisInqLbounds(zaxisID, lbounds.data());
+              zaxisInqUbounds(zaxisID, ubounds.data());
+              for (size_t k = 0; k < nlev; ++k) cdata[k] = ubounds[k] - lbounds[k];
+            }
+        }
+    }
+  }
+
+  void
+  read_coordinates()
+  {
+    for (int i = 0, n = parseArg.numCoords; i < n; ++i)
+      {
+        if (parseArg.coords[i].needed)
+          {
+            auto &cdata = parseArg.coords[i].data;
+            auto csize = parseArg.coords[i].size;
+            auto cdiID = parseArg.coords[i].cdiID;
+            auto coord = parseArg.coords[i].coord;
+            if (coord == 'x' || coord == 'y' || coord == 'a' || coord == 'w' || coord == 'g')
+              {
+                read_coordinates_hgrid(cdata, csize, cdiID, coord);
+              }
+            else if (coord == 'z' || coord == 'i' || coord == 'd') { read_coordinates_vgrid(cdata, csize, cdiID, coord); }
+            else { cdo_abort("Computation of coordinate %c not implemented!", coord); }
+          }
+      }
+  }
+
+  void
+  copy_coordinates()
+  {
+    auto &params = parseArg.params;
+    for (int varID = parseArg.numVars1, n = parseArg.numParams; varID < n; ++varID)
+      {
+        auto coord = params[varID].coord;
+        if (coord)
+          {
+            if (coord == 'x' || coord == 'y' || coord == 'a' || coord == 'w' || coord == 'g')
+              {
+                auto coordID = params_get_coord_ID(parseArg, coord, params[varID].gridID);
+                auto gridID = parseArg.coords[coordID].cdiID;
+                auto ngp = parseArg.coords[coordID].size;
+                auto const &cdata = parseArg.coords[coordID].data;
+                assert(gridID == params[varID].gridID);
+                assert(!cdata.empty());
+
+                array_copy(ngp, cdata.data(), params[varID].data);
+              }
+            else if (coord == 'z' || coord == 'i' || coord == 'd')
+              {
+                auto coordID = params_get_coord_ID(parseArg, coord, params[varID].zaxisID);
+                auto zaxisID = parseArg.coords[coordID].cdiID;
+                auto nlev = parseArg.coords[coordID].size;
+                auto const &cdata = parseArg.coords[coordID].data;
+                assert(zaxisID == params[varID].zaxisID);
+                assert(!cdata.empty());
+
+                array_copy(nlev, cdata.data(), params[varID].data);
+              }
+            else
+              cdo_abort("Computation of coordinate %c not implemented!", coord);
+          }
+      }
+  }
+
+  void
+  init_output_params(std::vector<ParamEntry> &params)
+  {
+    for (int varID = 0; varID < numVars2; ++varID)
+      {
+        auto pidx = varIDmap[varID];
+        if (pidx < numVars1) continue;
+
+        auto &param = params[pidx];
+        param.numMissVals = 0;
+        std::ranges::fill_n(param.data, param.ngp * param.nlev, 0.0);
+      }
+  }
+
+  void
+  add_new_output_params()
+  {
+    auto const &params = parseArg.params;
+    //  printf("parseArg.nparams %d\n", parseArg.nparams);
+    for (int pidx = 0, n = parseArg.numParams; pidx < n; pidx++)
+      {
+        auto const &param = params[pidx];
+        if (pidx < numVars1 && !param.select) continue;
+        if (pidx >= numVars1)
+          {
+            if (param.type == ParamType::CONST) continue;
+            if (param.name[0] == '_') continue;
+            if (param.remove) continue;
+            if (param.coord) continue;
+          }
+
+        // printf("gridID %d zaxisID %d\n",  param.gridID, param.zaxisID);
+        auto varID = vlistDefVar(vlistID2, param.gridID, param.zaxisID, param.steptype);
+        cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, param.name.c_str());
+        // printf("add: %d %s %d levs %d\n", pidx,  param.name.c_str(), varID, zaxisInqSize(param.zaxisID));
+        if (param.hasMV) vlistDefVarMissval(vlistID2, varID, param.missval);
+        if (param.units.size()) cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, param.units.c_str());
+        if (param.longname.size()) cdiDefKeyString(vlistID2, varID, CDI_KEY_LONGNAME, param.longname.c_str());
+        if (param.stdname.size()) cdiDefKeyString(vlistID2, varID, CDI_KEY_STDNAME, param.stdname.c_str());
+        if (param.name.size() > 3 && param.name.rfind("var", 0) == 0)
+          {
+            if (std::isdigit(param.name[3]))
+              {
+                auto code = std::atoi(param.name.c_str() + 3);
+                vlistDefVarCode(vlistID2, varID, code);
+              }
+          }
+        varIDmap[varID] = pidx;
+      }
+  }
+
 public:
   void
   init() override
@@ -364,7 +563,7 @@ public:
 
     operator_input_arg(cdo_operator_enter(operatorID));
 
-    const auto &exprArgv = cdo_get_oper_argv();
+    auto const &exprArgv = cdo_get_oper_argv();
 
     exprString = readsCommandLine ? exprs_from_argument(exprArgv) : exprs_from_file(exprArgv);
 
@@ -397,16 +596,9 @@ public:
     parseArg.tsID = vartsID;
     params_add_coordinates(vlistID1, parseArg);
 
-    {
-      CDO_parser_errorno = 0;
-      void *scanner = nullptr;
-      yylex_init(&scanner);
-      yyset_extra(&parseArg, scanner);
-      yy_scan_string(exprString.c_str(), scanner);
-      yyparse(parseArg, scanner);
-      yylex_destroy(scanner);
-      if (CDO_parser_errorno != 0) cdo_abort("Syntax error!");
-    }
+    CDO_parser_errorno = 0;
+    parse_expressions();
+    if (CDO_parser_errorno != 0) cdo_abort("Syntax error!");
 
     parseArg.init = false;
 
@@ -415,11 +607,11 @@ public:
         if (parseArg.needed[varID]) cdo_print("Needed var: %d %s", varID, params[varID].name);
 
     if (Options::cdoVerbose)
-      for (int varID = 0, n = parseArg.nparams; varID < n; ++varID)
+      for (int varID = 0, n = parseArg.numParams; varID < n; ++varID)
         cdo_print("var: %d %s ngp=%zu nlev=%zu coord=%c", varID, params[varID].name, params[varID].ngp, params[varID].nlev,
                   (params[varID].coord == 0) ? ' ' : params[varID].coord);
 
-    varIDmap = std::vector<int>(parseArg.nparams);
+    varIDmap.resize(parseArg.numParams);
 
     vlistID2 = vlistCreate();
     vlistDefNtsteps(vlistID2, vlistNtsteps(vlistID1));
@@ -441,37 +633,7 @@ public:
       }
     cdo_vlist_copy_flag(vlistID2, vlistID1);  // Copy global attributes
 
-    //  printf("parseArg.nparams %d\n", parseArg.nparams);
-    for (int pidx = 0, n = parseArg.nparams; pidx < n; pidx++)
-      {
-        const auto &param = params[pidx];
-        if (pidx < numVars1 && !param.select) continue;
-        if (pidx >= numVars1)
-          {
-            if (param.type == ParamType::CONST) continue;
-            if (param.name[0] == '_') continue;
-            if (param.remove) continue;
-            if (param.coord) continue;
-          }
-
-        // printf("gridID %d zaxisID %d\n",  param.gridID, param.zaxisID);
-        auto varID = vlistDefVar(vlistID2, param.gridID, param.zaxisID, param.steptype);
-        cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, param.name.c_str());
-        // printf("add: %d %s %d levs %d\n", pidx,  param.name.c_str(), varID, zaxisInqSize(param.zaxisID));
-        if (param.hasMV) vlistDefVarMissval(vlistID2, varID, param.missval);
-        if (param.units.size()) cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, param.units.c_str());
-        if (param.longname.size()) cdiDefKeyString(vlistID2, varID, CDI_KEY_LONGNAME, param.longname.c_str());
-        if (param.stdname.size()) cdiDefKeyString(vlistID2, varID, CDI_KEY_STDNAME, param.stdname.c_str());
-        if (param.name.size() > 3 && param.name.rfind("var", 0) == 0)
-          {
-            if (std::isdigit(param.name[3]))
-              {
-                auto code = atoi(param.name.c_str() + 3);
-                vlistDefVarCode(vlistID2, varID, code);
-              }
-          }
-        varIDmap[varID] = pidx;
-      }
+    add_new_output_params();
 
     if (Options::cdoVerbose)
       {
@@ -483,118 +645,10 @@ public:
     numVars2 = vlistNvars(vlistID2);
     if (numVars2 == 0) cdo_abort("No output variable found!");
 
-    for (int varID = 0; varID < numVars1; ++varID)
-      {
-        if (parseArg.needed[varID])
-          {
-            auto nItems = std::max((size_t) 4, params[varID].ngp * params[varID].nlev);
-            params[varID].data = new double[nItems];
-          }
-      }
+    allocate_params();
 
-    for (int varID = parseArg.numVars1, n = parseArg.nparams; varID < n; ++varID)
-      {
-        auto nItems = std::max((size_t) 4, params[varID].ngp * params[varID].nlev);
-        params[varID].data = new double[nItems];
-      }
-
-    for (int i = 0, n = parseArg.numCoords; i < n; ++i)
-      {
-        if (parseArg.coords[i].needed)
-          {
-            auto &cdata = parseArg.coords[i].data;
-            auto csize = parseArg.coords[i].size;
-            auto cdiID = parseArg.coords[i].cdiID;
-            auto coord = parseArg.coords[i].coord;
-            if (coord == 'x' || coord == 'y' || coord == 'a' || coord == 'w' || coord == 'g')
-              {
-                auto gridID = cdiID;
-                auto ngp = csize;
-                cdata.resize(ngp);
-                if (coord == 'x' || coord == 'y')
-                  {
-                    gridID = generate_full_point_grid(gridID);
-                    if (!gridHasCoordinates(gridID)) cdo_abort("Cell center coordinates missing!");
-
-                    if (coord == 'x') gridInqXvals(gridID, cdata.data());
-                    if (coord == 'y') gridInqYvals(gridID, cdata.data());
-
-                    if (gridID != parseArg.coords[i].cdiID) gridDestroy(gridID);
-                  }
-                else if (coord == 'a') { gridcell_areas(gridID, cdata); }
-                else if (coord == 'w')
-                  {
-                    cdata[0] = 1;
-                    if (ngp > 1)
-                      {
-                        auto wstatus = gridcell_weights(gridID, cdata);
-                        if (wstatus) cdo_warning("Grid cell bounds not available, using constant grid cell area weights!");
-                      }
-                  }
-                else if (coord == 'g')
-                  {
-                    for (size_t k = 0; k < ngp; ++k) cdata[k] = k + 1;
-                  }
-              }
-            else if (coord == 'z' || coord == 'i' || coord == 'd')
-              {
-                auto zaxisID = cdiID;
-                auto nlev = csize;
-                cdata.resize(nlev);
-                if (coord == 'z') { cdo_zaxis_inq_levels(zaxisID, cdata.data()); }
-                else if (coord == 'i')
-                  {
-                    for (size_t k = 0; k < nlev; ++k) cdata[k] = k + 1;
-                    cdo_zaxis_inq_levels(zaxisID, cdata.data());
-                  }
-                else if (coord == 'd')
-                  {
-                    ranges::fill(cdata, 1.0);
-                    if (zaxisInqLbounds(zaxisID, nullptr) && zaxisInqUbounds(zaxisID, nullptr))
-                      {
-                        std::vector<double> lbounds(nlev), ubounds(nlev);
-                        zaxisInqLbounds(zaxisID, lbounds.data());
-                        zaxisInqUbounds(zaxisID, ubounds.data());
-                        for (size_t k = 0; k < nlev; ++k) cdata[k] = ubounds[k] - lbounds[k];
-                      }
-                  }
-              }
-            else
-              cdo_abort("Computation of coordinate %c not implemented!", coord);
-          }
-      }
-
-    for (int varID = parseArg.numVars1, n = parseArg.nparams; varID < n; ++varID)
-      {
-        auto coord = params[varID].coord;
-        if (coord)
-          {
-            if (coord == 'x' || coord == 'y' || coord == 'a' || coord == 'w' || coord == 'g')
-              {
-                auto coordID = params_get_coord_ID(parseArg, coord, params[varID].gridID);
-                auto gridID = parseArg.coords[coordID].cdiID;
-                auto ngp = parseArg.coords[coordID].size;
-                auto &cdata = parseArg.coords[coordID].data;
-                assert(gridID == params[varID].gridID);
-                assert(!cdata.empty());
-
-                array_copy(ngp, cdata.data(), params[varID].data);
-              }
-            else if (coord == 'z' || coord == 'i' || coord == 'd')
-              {
-                auto coordID = params_get_coord_ID(parseArg, coord, params[varID].zaxisID);
-                auto zaxisID = parseArg.coords[coordID].cdiID;
-                auto nlev = parseArg.coords[coordID].size;
-                auto &cdata = parseArg.coords[coordID].data;
-                assert(zaxisID == params[varID].zaxisID);
-                assert(!cdata.empty());
-
-                array_copy(nlev, cdata.data(), params[varID].data);
-              }
-            else
-              cdo_abort("Computation of coordinate %c not implemented!", coord);
-          }
-      }
+    read_coordinates();
+    copy_coordinates();
 
     if (Options::cdoVerbose) vlistPrint(vlistID2);
 
@@ -615,8 +669,8 @@ public:
     while (true)
       {
         auto &params = parseArg.params;
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         auto vDateTime = taxisInqVdatetime(taxisID1);
 
@@ -633,51 +687,31 @@ public:
         for (int varID = 0; varID < numVars1; ++varID)
           if (tsID == 0 || params[varID].steptype != TIME_CONSTANT) params[varID].numMissVals = 0;
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             if (parseArg.needed[varID])
               {
                 auto offset = params[varID].ngp * levelID;
                 auto vardata = params[varID].data + offset;
                 size_t numMissVals;
-                cdo_read_record(streamID1, vardata, &numMissVals);
+                cdo_read_field(streamID1, vardata, &numMissVals);
                 params[varID].numMissVals += numMissVals;
 
                 if (numMissVals > 0) cdo_check_missval(params[varID].missval, params[varID].name);
               }
           }
 
-        for (int varID = 0; varID < numVars2; ++varID)
-          {
-            auto pidx = varIDmap[varID];
-            if (pidx < numVars1) continue;
+        init_output_params(params);
 
-            auto &param = params[pidx];
-            param.numMissVals = 0;
-            ranges::fill_n(param.data, param.ngp * param.nlev, 0.0);
-          }
-
-        parseArg.cnparams = vartsID + 1;
-
-        {
-          void *scanner = nullptr;
-          yylex_init(&scanner);
-          yyset_extra(&parseArg, scanner);
-          yy_scan_string(exprString.c_str(), scanner);
-          yyparse(parseArg, scanner);
-          yylex_destroy(scanner);
-        }
+        parse_expressions();
 
         for (int varID = 0; varID < numVars2; ++varID)
           {
             auto pidx = varIDmap[varID];
-
             if (tsID > 0 && params[pidx].steptype == TIME_CONSTANT) continue;
 
             auto missval = vlistInqVarMissval(vlistID2, varID);
-
             auto ngp = params[pidx].ngp;
             auto nlev = (int) params[pidx].nlev;
             for (int levelID = 0; levelID < nlev; ++levelID)
@@ -685,8 +719,8 @@ public:
                 auto offset = ngp * levelID;
                 double *vardata = params[pidx].data + offset;
                 auto numMissVals = array_num_mv(ngp, vardata, missval);
-                cdo_def_record(streamID2, varID, levelID);
-                cdo_write_record(streamID2, vardata, numMissVals);
+                cdo_def_field(streamID2, varID, levelID);
+                cdo_write_field(streamID2, vardata, numMissVals);
               }
           }
 
