@@ -30,14 +30,14 @@
 #include <mpim_grid.h>
 #include "specspace.h"
 #include "griddes.h"
-#include "cimdOmp.h"
+#include "cdo_omp.h"
 
 static int
 vlistGetFirstReg2DGrid(int vlistID)
 {
   // find first gaussian grid
-  auto ngrids = vlistNumGrids(vlistID);
-  for (int index = 0; index < ngrids; ++index)
+  auto numGrids = vlistNumGrids(vlistID);
+  for (int index = 0; index < numGrids; ++index)
     {
       auto gridID = vlistGrid(vlistID, index);
       if (gridInqType(gridID) == GRID_GAUSSIAN || gridInqType(gridID) == GRID_LONLAT || gridInqType(gridID) == GRID_CURVILINEAR)
@@ -72,11 +72,11 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int vlistID1;
+  int vlistID1{ CDI_UNDEFID };
   VarList varList1;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   bool dataIsUnchanged;
 
@@ -278,7 +278,7 @@ public:
 
     auto numVars = varList1.numVars();
     vars = std::vector<bool>(numVars);
-    for (const auto &var : varList1.vars) vars[var.ID] = (gridID1 == var.gridID);
+    for (auto const &var : varList1.vars) vars[var.ID] = (gridID1 == var.gridID);
 
     if (gridID1 != -1) vlistChangeGrid(vlistID2, gridID1, gridID2);
     if (operatorID == GRID2FOURIER)
@@ -296,9 +296,9 @@ public:
 
     cdo_def_vlist(streamID2, vlistID2);
 
-    auto gridsizemax = vlistGridsizeMax(vlistID1);
-    if (operatorID == FOURIER2GRID) gridsizemax *= 2;
-    array1 = Varray<double>(gridsizemax);
+    auto gridsizeMax = varList1.gridsizeMax();
+    if (operatorID == FOURIER2GRID) gridsizeMax *= 2;
+    array1 = Varray<double>(gridsizeMax);
 
     if (gridID2 != -1)
       {
@@ -314,21 +314,20 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
 
             if (vars[varID])
               {
                 size_t numMissVals;
-                cdo_read_record(streamID1, array1.data(), &numMissVals);
+                cdo_read_field(streamID1, array1.data(), &numMissVals);
                 if (numMissVals) cdo_abort("Missing values unsupported for spectral/fourier data!");
 
                 gridID1 = varList1.vars[varID].gridID;
@@ -344,18 +343,18 @@ public:
                   fourier2grid(gridID1, array1, array2);
                 else if (operatorID == GRID2FOURIER)
                   grid2fourier(gridID1, array1, gridID2, array2);
-                cdo_def_record(streamID2, varID, levelID);
-                cdo_write_record(streamID2, array2.data(), numMissVals);
+                cdo_def_field(streamID2, varID, levelID);
+                cdo_write_field(streamID2, array2.data(), numMissVals);
               }
             else
               {
-                cdo_def_record(streamID2, varID, levelID);
-                if (dataIsUnchanged) { cdo_copy_record(streamID2, streamID1); }
+                cdo_def_field(streamID2, varID, levelID);
+                if (dataIsUnchanged) { cdo_copy_field(streamID2, streamID1); }
                 else
                   {
                     size_t numMissVals;
-                    cdo_read_record(streamID1, array1.data(), &numMissVals);
-                    cdo_write_record(streamID2, array1.data(), numMissVals);
+                    cdo_read_field(streamID1, array1.data(), &numMissVals);
+                    cdo_write_field(streamID2, array1.data(), numMissVals);
                   }
               }
           }
