@@ -19,7 +19,7 @@
 #include "param_conversion.h"
 #include "cdo_timer.h"
 #include <mpim_grid.h>
-#include "grid_point_search.h"
+#include "grid_pointsearch.h"
 #include "cdo_options.h"
 #include "progress.h"
 #include "cdo_omp.h"
@@ -268,7 +268,8 @@ setmisstodis(Varray<T1> &vIn, Varray<T2> &vOut, int gridID, size_t numMissVals,
 
   if (!gridHasCoordinates(gridID)) cdo_abort("Cell center coordinates missing!");
 
-  Varray<double> xvals(gridsize), yvals(gridsize);
+  Varray<double> xvals(gridsize);
+  Varray<double> yvals(gridsize);
   gridInqXvals(gridID, xvals.data());
   gridInqYvals(gridID, yvals.data());
 
@@ -310,19 +311,17 @@ setmisstodis(Varray<T1> &vIn, Varray<T2> &vOut, int gridID, size_t numMissVals,
 
   if (nv != nvals) cdo_abort("Internal problem, number of valid values differ!");
 
-  std::vector<knnWeightsType> knnWeightsMem;
-  for (int i = 0; i < Threading::ompNumThreads; ++i) knnWeightsMem.push_back(knnWeightsType(numNeighbors));
+  std::vector<KnnData> knnDataList;
+  for (int i = 0; i < Threading::ompNumMaxThreads; ++i) knnDataList.emplace_back(numNeighbors);
 
   cdo::timer timer;
 
-  GridPointSearch gps;
+  GridPointsearch gps;
 
   if (numMissVals)
     {
-      auto xIsCyclic = false;
-      size_t dims[2] = { nvals, 0 };
-      grid_point_search_create(gps, xIsCyclic, dims, nvals, lons, lats);
-      grid_point_search_extrapolate(gps);
+      gps.enable_extrapolation();
+      grid_pointsearch_create_unstruct(gps, lons, lats, true);
     }
 
   if (Options::cdoVerbose) cdo_print("Point search created: %.2f seconds", timer.elapsed());
@@ -339,27 +338,25 @@ setmisstodis(Varray<T1> &vIn, Varray<T2> &vOut, int gridID, size_t numMissVals,
   for (size_t i = 0; i < numMissVals; ++i)
     {
       atomicCount++;
-      if (cdo_omp_get_thread_num() == 0) progress.update((double) atomicCount / numMissVals);
-
       auto ompthID = cdo_omp_get_thread_num();
-      auto &knnWeights = knnWeightsMem[ompthID];
+      if (ompthID == 0 && numMissVals > progressMinSize) progress.update((double) atomicCount / numMissVals);
 
-      grid_search_point(gps, xvals[mindex[i]], yvals[mindex[i]], knnWeights);
+      auto &knnData = knnDataList[ompthID];
 
-      // Compute weights based on inverse distance if mask is false, eliminate those points
-      auto nadds = knnWeights.computeWeights();
-      if (nadds)
+      grid_search_point_unstruct(gps, PointLonLat{ xvals[mindex[i]], yvals[mindex[i]] }, knnData);
+
+      // Compute weights if mask is false, eliminate those points
+      auto numWeights = knnData.compute_weights();
+      if (numWeights)
         {
           double result = 0.0;
-          for (size_t n = 0; n < nadds; ++n) result += vIn[vindex[knnWeights.m_indices[n]]] * knnWeights.m_dist[n];
+          for (size_t n = 0; n < numWeights; ++n) result += vIn[vindex[knnData.m_indices[n]]] * knnData.m_dist[n];
           vOut[mindex[i]] = result;
         }
     }
 
   if (Options::cdoVerbose) cdo_print("Point search nearest: %.2f seconds", timer.elapsed());
 
-  grid_point_search_delete(gps);
-
   if (gridID0 != gridID) gridDestroy(gridID);
 }
 
@@ -393,8 +390,8 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   int operatorID;
   int nfill;
@@ -455,24 +452,23 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
 
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        while (numFields--)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            const auto &var1 = varList1.vars[varID];
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            auto const &var1 = varList1.vars[varID];
             field1.init(var1);
-            cdo_read_record(streamID1, field1);
+            cdo_read_field(streamID1, field1);
 
-            cdo_def_record(streamID2, varID, levelID);
+            cdo_def_field(streamID2, varID, levelID);
 
-            if (field1.numMissVals == 0 || field1.numMissVals == var1.gridsize) { cdo_write_record(streamID2, field1); }
+            if (field1.numMissVals == 0 || field1.numMissVals == var1.gridsize) { cdo_write_field(streamID2, field1); }
             else
               {
                 auto gridtype = var1.gridType;
@@ -485,7 +481,7 @@ public:
 
                 field2.numMissVals = field_num_mv(field2);
 
-                cdo_write_record(streamID2, field2);
+                cdo_write_field(streamID2, field2);
               }
           }
 
