@@ -16,28 +16,40 @@
 #include <mpim_grid.h>
 #include "field_functions.h"
 
-class ModuleFldrms
+class Fldrms : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Fldrms",
+    .operators = { { "fldrms" } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 2, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Fldrms> registration = RegisterEntry<Fldrms>(module);
+
   int lastgrid = -1;
 
   CdoStreamID streamID1;
   CdoStreamID streamID2;
   CdoStreamID streamID3;
 
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
   int taxisID3;
 
-  VarList varList1, varList2;
-  Field field1, field2, field3;
+  int gridID3 = -1;
+
+  VarList varList1;
+  VarList varList2;
 
   bool needWeights;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
     operator_check_argc(0);
 
     needWeights = true;
@@ -49,21 +61,24 @@ public:
     auto vlistID2 = cdo_stream_inq_vlist(streamID2);
     auto vlistID3 = vlistDuplicate(vlistID1);
 
+    varList1 = VarList(vlistID1);
+    varList2 = VarList(vlistID2);
+
     taxisID1 = vlistInqTaxis(vlistID1);
     taxisID3 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID3, taxisID3);
 
     double slon = 0.0, slat = 0.0;
-    auto gridID3 = gridCreate(GRID_LONLAT, 1);
+    gridID3 = gridCreate(GRID_LONLAT, 1);
     gridDefXsize(gridID3, 1);
     gridDefYsize(gridID3, 1);
     gridDefXvals(gridID3, &slon);
     gridDefYvals(gridID3, &slat);
 
-    auto ngrids = vlistNgrids(vlistID1);
-    int ndiffgrids = 0;
-    for (int index = 1; index < ngrids; ++index)
-      if (vlistGrid(vlistID1, 0) != vlistGrid(vlistID1, index)) ndiffgrids++;
+    auto numGrids = varList1.numGrids();
+    int numDiffGrids = 0;
+    for (int index = 1; index < numGrids; ++index)
+      if (vlistGrid(vlistID1, 0) != vlistGrid(vlistID1, index)) numDiffGrids++;
 
     auto gridID1 = vlistGrid(vlistID1, 0);
     auto gridID2 = vlistGrid(vlistID2, 0);
@@ -73,52 +88,48 @@ public:
     if (needWeights && gridInqType(gridID1) != GRID_LONLAT && gridInqType(gridID1) != GRID_GAUSSIAN)
       cdo_abort("Unsupported gridtype: %s", gridNamePtr(gridInqType(gridID1)));
 
-    for (int index = 0; index < ngrids; ++index) vlistChangeGridIndex(vlistID3, index, gridID3);
+    for (int index = 0; index < numGrids; ++index) vlistChangeGridIndex(vlistID3, index, gridID3);
 
-    if (ndiffgrids > 0) cdo_abort("Too many different grids!");
+    if (numDiffGrids > 0) cdo_abort("Too many different grids!");
 
     streamID3 = cdo_open_write(2);
     cdo_def_vlist(streamID3, vlistID3);
+  }
 
-    auto gridsizemax = vlistGridsizeMax(vlistID1);
-
-    varListInit(varList1, vlistID1);
-    varListInit(varList2, vlistID2);
-
-    field1.resize(gridsizemax);
-    if (needWeights) field1.weightv.resize(gridsizemax);
-
-    field2.resize(gridsizemax);
-
+  void
+  run() override
+  {
+    Field field1, field2, field3;
+    field1.resize(varList1.gridsizeMax());
+    if (needWeights) field1.weightv.resize(varList1.gridsizeMax());
+    field2.resize(varList1.gridsizeMax());
     field3.resize(1);
     field3.grid = gridID3;
-  }
-  void
-  run()
-  {
+
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
-        auto nrecs2 = cdo_stream_inq_timestep(streamID2, tsID);
-        if (nrecs2 == 0) cdo_abort("Input streams have different number of timesteps!");
+        auto numFields2 = cdo_stream_inq_timestep(streamID2, tsID);
+        if (numFields2 == 0) cdo_abort("Input streams have different number of timesteps!");
 
         cdo_taxis_copy_timestep(taxisID3, taxisID1);
 
         cdo_def_timestep(streamID3, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        while (numFields--)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            cdo_read_record(streamID1, field1.vec_d.data(), &field1.nmiss);
-            cdo_inq_record(streamID2, &varID, &levelID);
-            cdo_read_record(streamID2, field2.vec_d.data(), &field2.nmiss);
+            (void) cdo_inq_field(streamID1);
+            cdo_read_field(streamID1, field1);
+            auto [varID, levelID] = cdo_inq_field(streamID2);
+            cdo_read_field(streamID2, field2);
 
-            field1.grid = varList1[varID].gridID;
-            field2.grid = varList2[varID].gridID;
+            auto const &var1 = varList1.vars[varID];
+            auto const &var2 = varList2.vars[varID];
+            field1.grid = var1.gridID;
+            field2.grid = var2.gridID;
 
             if (needWeights && field1.grid != lastgrid)
               {
@@ -129,40 +140,29 @@ public:
                     auto wstatus = gridcell_weights(field1.grid, field1.weightv);
                     if (wstatus != 0 && tsID == 0 && levelID == 0)
                       cdo_warning("Grid cell bounds not available, using constant grid cell area weights for variable %s!",
-                                  varList1[varID].name);
+                                  var1.name);
                   }
               }
 
-            field1.missval = varList1[varID].missval;
-            field2.missval = varList1[varID].missval;
-            field3.missval = varList1[varID].missval;
+            field1.missval = var1.missval;
+            field2.missval = var1.missval;
+            field3.missval = var1.missval;
 
             field_rms(field1, field2, field3);
 
-            cdo_def_record(streamID3, varID, levelID);
-            cdo_write_record(streamID3, field3.vec_d.data(), field3.nmiss);
+            cdo_def_field(streamID3, varID, levelID);
+            cdo_write_field(streamID3, field3);
           }
 
         tsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID3);
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Fldrms(void *process)
-{
-  ModuleFldrms fldrms;
-  fldrms.init(process);
-  fldrms.run();
-  fldrms.close();
-  return nullptr;
-}
