@@ -37,7 +37,7 @@ void gridcell_areas(int gridID, Varray<double> &array);
 
 template <typename T>
 static void
-print_location_LL(int operfunc, const CdoVar &var, int levelID, double sglval, const Varray<T> &v, CdiDateTime vDateTime)
+print_location_LL(int operfunc, const CdoVar &var, int levelID, double sglval, Varray<T> const &v, CdiDateTime vDateTime)
 {
   static auto printHeader = true;
   const char *funcName = (operfunc == FieldFunc_Min) ? "Minval" : "Maxval";
@@ -72,18 +72,17 @@ print_location_LL(int operfunc, const CdoVar &var, int levelID, double sglval, c
 }
 
 static void
-print_location_LL(int operfunc, const CdoVar &var, int levelID, double sglval, const Field &field, CdiDateTime vDateTime)
+print_location_LL(int operfunc, const CdoVar &var, int levelID, double sglval, Field const &field, CdiDateTime vDateTime)
 {
-  if (field.memType == MemType::Float)
-    print_location_LL(operfunc, var, levelID, sglval, field.vec_f, vDateTime);
-  else
-    print_location_LL(operfunc, var, levelID, sglval, field.vec_d, vDateTime);
+  auto func = [&](auto const &v) { print_location_LL(operfunc, var, levelID, sglval, v, vDateTime); };
+  field_operation(func, field);
 }
 
 template <typename T>
 static void
-field_mul_weights(Varray<T> &v1, const Varray<double> &v2, size_t numMissVals, T missval)
+field_mul_weights(Varray<T> &v1, Varray<double> const &v2, size_t numMissVals, double mv)
 {
+  T missval = mv;
   assert(v1.size() > 0);
   assert(v2.size() == v1.size());
 
@@ -102,10 +101,8 @@ field_mul_weights(Varray<T> &v1, const Varray<double> &v2, size_t numMissVals, T
 static void
 field_mul_weights(Field &field)
 {
-  if (field.memType == MemType::Float)
-    field_mul_weights(field.vec_f, field.weightv, field.numMissVals, (float) field.missval);
-  else
-    field_mul_weights(field.vec_d, field.weightv, field.numMissVals, field.missval);
+  auto func = [&](auto &v, auto const &w, auto numMissVals, double missval) { field_mul_weights(v, w, numMissVals, missval); };
+  field_operation(func, field, field.weightv, field.numMissVals, field.missval);
 }
 
 static void
@@ -149,33 +146,33 @@ gen_target_gridpoint(int gridID1)
 }
 
 static void
-printWeightsWarning(int ngrids, const std::string &varname)
+print_weights_warning(int numGrids, std::string const &varname)
 {
-  if (ngrids == 1)
+  if (numGrids == 1)
     cdo_warning("Grid cell bounds not available, using constant grid cell area weights!");
   else
     cdo_warning("Grid cell bounds not available, using constant grid cell area weights for variable %s!", varname);
 }
 
 static void
-fldstatGetParameter(bool &useWeights)
+get_parameter(bool &useWeights)
 {
   auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      auto pargv = cdo_get_oper_argv();
+      auto const &pargv = cdo_get_oper_argv();
 
       KVList kvlist;
       kvlist.name = cdo_module_name();
       if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-          const auto &value = kv.values[0];
+          auto const &value = kv.values[0];
 
           if (key == "weights")
             useWeights = parameter_to_bool(value);
@@ -186,21 +183,21 @@ fldstatGetParameter(bool &useWeights)
 }
 
 static int
-get_gridcell_weights(Field &field, bool useWeights, bool doPrintWarning, int ngrids, std::string &varName)
+get_gridcell_weights(Field &field, bool useWeights, bool doPrintWarning, int numGrids, std::string const &varName)
 {
   auto gridSize = field.size;
   field.weightv.resize(gridSize);
   if (!useWeights)
     {
       cdo_print("Using constant grid cell area weights!");
-      ranges::fill(field.weightv, 1.0);
+      std::ranges::fill(field.weightv, 1.0);
     }
 
   field.weightv[0] = 1;
   if (useWeights && field.size > 1)
     {
       auto wstatus = (gridcell_weights(field.grid, field.weightv) != 0);
-      if (wstatus && doPrintWarning) printWeightsWarning(ngrids, varName);
+      if (wstatus && doPrintWarning) print_weights_warning(numGrids, varName);
     }
 
   return field.grid;
@@ -223,6 +220,7 @@ public:
   using Process::Process;
   inline static CdoModule module = {
     .name = "Fldstat",
+    // clang-format off
     .operators = { { "fldrange", FieldFunc_Range, 0, FldstatHelp },
                    { "fldmin", FieldFunc_Min, 0, FldstatHelp },
                    { "fldmax", FieldFunc_Max, 0, FldstatHelp },
@@ -239,6 +237,7 @@ public:
                    { "fldmedian", FieldFunc_Median, 0, FldstatHelp },
                    { "fldcount", FieldFunc_Count, 0, FldstatHelp },
                    { "fldpctl", FieldFunc_Pctl, 0, FldstatHelp } },
+    // clang-format on
     .aliases = { { "globavg", "fldavg" } },
     .mode = EXPOSED,     // Module mode: 0:intern 1:extern
     .number = CDI_REAL,  // Allowed number type
@@ -250,10 +249,10 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  int vlistID2;
+  int vlistID2{ CDI_UNDEFID };
 
   bool isMinMaxFunc;
   bool needWeights;
@@ -261,7 +260,7 @@ public:
   bool useWeights;
 
   int operfunc;
-  int ngrids;
+  int numGrids;
 
   double pn = 0.0;
 
@@ -281,12 +280,18 @@ public:
     needCellarea = (operatorID == FLDINT);
     useWeights = true;
 
+    auto loadGrid = (needWeights || needCellarea);
+    if (Options::lazyGridLoad && this_is_the_only_process()) { cdiDefGlobal("NETCDF_LAZY_GRID_LOAD", true); }
+    if (not loadGrid && this_is_the_only_process()) { cdiDefGlobal("READ_CELL_CORNERS", false); }
+    if (not loadGrid && this_is_the_only_process()) { cdiDefGlobal("READ_CELL_CENTER", false); }
+
     if (operfunc == FieldFunc_Pctl)
       {
+        operator_check_argc(1);
         operator_input_arg("percentile number");
         pn = parameter_to_double(cdo_operator_argv(0));
       }
-    else if (needWeights) { fldstatGetParameter(useWeights); }
+    else if (needWeights) { get_parameter(useWeights); }
     else { operator_check_argc(0); }
 
     streamID1 = cdo_open_read(0);
@@ -301,9 +306,9 @@ public:
     taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
 
-    ngrids = vlistNumGrids(vlistID1);
+    numGrids = vlistNumGrids(vlistID1);
 
-    for (int index = 0; index < ngrids; ++index)
+    for (int index = 0; index < numGrids; ++index)
       {
         auto gridID1 = vlistGrid(vlistID1, index);
         auto gridID2 = gen_target_gridpoint(gridID1);
@@ -322,34 +327,33 @@ public:
   {
     Field field;
     auto numSteps = varList1.numSteps();
-    cdo::Progress progress;
+    cdo::Progress progress(get_id());
 
     int lastgrid = -1;
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
         auto vDateTime = taxisInqVdatetime(taxisID1);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto fstatus = ((tsID + (recID + 1.0) / nrecs) / numSteps);
+            auto fstatus = ((tsID + (fieldID + 1.0) / numFields) / numSteps);
             if (numSteps > 1) progress.update(fstatus);
 
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             auto &var = varList1.vars[varID];
             field.init(var);
-            cdo_read_record(streamID1, field);
+            cdo_read_field(streamID1, field);
 
             auto doPrintWarning = (tsID == 0 && levelID == 0);
             if (needWeights && field.grid != lastgrid)
-              lastgrid = get_gridcell_weights(field, useWeights, doPrintWarning, ngrids, var.name);
+              lastgrid = get_gridcell_weights(field, useWeights, doPrintWarning, numGrids, var.name);
             else if (needCellarea && field.grid != lastgrid)
               lastgrid = get_gridcell_areas(field);
 
@@ -361,8 +365,8 @@ public:
 
             size_t numMissVals = fp_is_equal(singleValue, field.missval);
 
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, &singleValue, numMissVals);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, &singleValue, numMissVals);
           }
 
         tsID++;
