@@ -21,7 +21,7 @@ static std::mutex fftwMutex;
 #include "param_conversion.h"
 #include "cdo_fft.h"
 #include "cdo_options.h"
-#include "cimdOmp.h"
+#include "cdo_omp.h"
 #include "field_functions.h"
 
 struct FourierMemory
@@ -60,7 +60,7 @@ fourier_fftw(int sign, int varID, int levelID, int nts, size_t gridsize, double
           auto imag = vars[tsID][varID][levelID].vec_d[2 * i + 1];
           fm.in_fft[tsID][0] = real;
           fm.in_fft[tsID][1] = imag;
-          if (DBL_IS_EQUAL(real, missval) || DBL_IS_EQUAL(imag, missval)) hasMissvals = true;
+          if (fp_is_equal(real, missval) || fp_is_equal(imag, missval)) hasMissvals = true;
         }
 
       if (hasMissvals)
@@ -106,7 +106,7 @@ fourier_intrinsic(int sign, int varID, int levelID, int nts, size_t gridsize, do
           auto imag = vars[tsID][varID][levelID].vec_d[2 * i + 1];
           fm.real[tsID] = real;
           fm.imag[tsID] = imag;
-          if (DBL_IS_EQUAL(real, missval) || DBL_IS_EQUAL(imag, missval)) hasMissvals = true;
+          if (fp_is_equal(real, missval) || fp_is_equal(imag, missval)) hasMissvals = true;
         }
 
       if (hasMissvals)
@@ -152,18 +152,15 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  int vlistID1;
+  int vlistID1{ CDI_UNDEFID };
 
   bool use_fftw = false;
-  int numVars;
   int sign;
 
   VarList varList;
-  FieldVector3D varsData;
-  std::vector<CdiDateTime> vDateTimes;
 
 public:
   void
@@ -197,18 +194,20 @@ public:
     cdo_def_vlist(streamID2, vlistID2);
 
     varList = VarList(vlistID1);
-
-    numVars = vlistNvars(vlistID1);
   }
 
   void
   run() override
   {
+    FieldVector3D varsData;
+    std::vector<CdiDateTime> vDateTimes;
+
+    auto numVars = varList.numVars();
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         if ((size_t) tsID >= nalloc)
           {
@@ -222,15 +221,12 @@ public:
 
         field2D_init(varsData[tsID], varList);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             auto gridsize = varList.vars[varID].gridsize;
             varsData[tsID][varID][levelID].resize(2 * gridsize);
-            size_t numMissVals;
-            cdo_read_record(streamID1, varsData[tsID][varID][levelID].vec_d.data(), &numMissVals);
-            varsData[tsID][varID][levelID].numMissVals = numMissVals;
+            cdo_read_field(streamID1, varsData[tsID][varID][levelID]);
           }
 
         tsID++;
@@ -238,7 +234,7 @@ public:
 
     int nts = tsID;
 
-    std::vector<FourierMemory> fourierMemory(Threading::ompNumThreads);
+    std::vector<FourierMemory> fourierMemory(Threading::ompNumMaxThreads);
 
     if (use_fftw)
       {
@@ -267,16 +263,13 @@ public:
 
     for (int varID = 0; varID < numVars; ++varID)
       {
-        const auto &var = varList.vars[varID];
-        auto missval = var.missval;
-        auto gridsize = var.gridsize;
-        auto nlevels = var.nlevels;
-        for (int levelID = 0; levelID < nlevels; ++levelID)
+        auto const &var = varList.vars[varID];
+        for (int levelID = 0; levelID < var.nlevels; ++levelID)
           {
             if (use_fftw)
-              fourier_fftw(sign, varID, levelID, nts, gridsize, missval, varsData, fourierMemory);
+              fourier_fftw(sign, varID, levelID, nts, var.gridsize, var.missval, varsData, fourierMemory);
             else
-              fourier_intrinsic(sign, varID, levelID, nts, gridsize, missval, varsData, fourierMemory);
+              fourier_intrinsic(sign, varID, levelID, nts, var.gridsize, var.missval, varsData, fourierMemory);
           }
       }
 
@@ -301,14 +294,13 @@ public:
 
         for (int varID = 0; varID < numVars; ++varID)
           {
-            auto nlevels = varList.vars[varID].nlevels;
-            for (int levelID = 0; levelID < nlevels; ++levelID)
+            auto numLevels = varList.vars[varID].nlevels;
+            for (int levelID = 0; levelID < numLevels; ++levelID)
               {
                 if (!varsData[tsID][varID][levelID].empty())
                   {
-                    auto numMissVals = varsData[tsID][varID][levelID].numMissVals;
-                    cdo_def_record(streamID2, varID, levelID);
-                    cdo_write_record(streamID2, varsData[tsID][varID][levelID].vec_d.data(), numMissVals);
+                    cdo_def_field(streamID2, varID, levelID);
+                    cdo_write_field(streamID2, varsData[tsID][varID][levelID]);
                   }
               }
           }
