@@ -551,10 +551,10 @@ ctl_zdef(FILE *gdp, int vlistID, bool *zrev)
   double levinc = 0;
 
   *zrev = false;
-  int nzaxis = vlistNumZaxis(vlistID);
+  auto numZaxes = vlistNumZaxis(vlistID);
 
   int nlevmax = 0;
-  for (int index = 0; index < nzaxis; ++index)
+  for (int index = 0; index < numZaxes; ++index)
     {
       zaxisID = vlistZaxis(vlistID, index);
       nlev = zaxisInqSize(zaxisID);
@@ -668,14 +668,14 @@ ctl_vars(FILE *gdp, int filetype, int vlistID, const VarList &varList, int nvars
     {
       if (vars[varID] == true)
         {
-          const auto &var = varList.vars[varID];
+          auto const &var = varList.vars[varID];
           int zaxisID = var.zaxisID;
           int ltype = 0;
           cdiInqKeyInt(zaxisID, CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &ltype);
           int nlev = zaxisInqSize(zaxisID);
 
           std::strcpy(varname, var.name.c_str());
-          int len = (int) strlen(varname);
+          int len = (int) std::strlen(varname);
           int i;
           for (i = 0; i < len; ++i)
             if (varname[i] == '-') break;
@@ -827,9 +827,7 @@ write_map_grib1(const char *ctlfile, int map_version, int nrecords, int *intnum,
       if (indx.hinum)
         for (int i = 0; i < indx.hinum; ++i) Put4Byte(map, bcnt, hinum[i]);
 
-      if (indx.hfnum)
-        { /* blank for now */
-        }
+      if (indx.hfnum) { /* blank for now */ }
 
       for (int i = 0; i < nrecords; ++i)
         {
@@ -928,7 +926,6 @@ public:
   int imn0 = 0, ihh0 = 0, iyy0 = 0, imm0 = 0, idd0 = 0;
   int idmn, idhh, idmm, idyy, iddd;
   int dt = 1, iik = 0, mdt = 0;
-  size_t gridsize = 0;
   int prec;
   int map_version = 2;
   int maxrecs = 0;
@@ -946,7 +943,7 @@ public:
   int vlistID;
 
   int filetype;
-  int nvarsout = 0, nrecsout = 0;
+  int nvarsout = 0, numFieldsout = 0;
 
   VarList varList;
 
@@ -964,8 +961,8 @@ public:
   {
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID, tsID);
+        if (numFields == 0) break;
 
         auto vDateTime = taxisInqVdatetime(taxisID);
 
@@ -1053,7 +1050,7 @@ public:
 
         if (filetype == CDI_FILETYPE_GRB)
           {
-            nrecords += nrecsout;
+            nrecords += numFieldsout;
             if (nrecords >= maxrecs)
               {
                 maxrecs = nrecords;
@@ -1062,25 +1059,24 @@ public:
                 bignum.resize(2 * maxrecs);
               }
 
-            for (int recID = 0; recID < nrecs; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                int varID, levelID;
-                cdo_inq_record(streamID, &varID, &levelID);
+                auto [varID, levelID] = cdo_inq_field(streamID);
                 if (vars[varID] == true)
                   {
                     size_t numMissVals;
-                    cdo_read_record(streamID, array.data(), &numMissVals);
+                    cdo_read_field(streamID, array.data(), &numMissVals);
 
-                    index = (tsID * nrecsout + recoffset[varID] + levelID);
+                    index = (tsID * numFieldsout + recoffset[varID] + levelID);
 
                     cdo_inq_grib_info(streamID, &intnum[index], &fltnum[index * 3], &bignum[index * 2]);
 
                     if (map_version != 4)
                       {
-                        long checksize = (long) bignum[index * 2] + (long) gridsize * intnum[index] / 8;
+                        long checksize = (long) bignum[index * 2] + (long) varList.gridsizeMax() * intnum[index] / 8;
                         if (checksize < 0L || checksize > 2147483647L)
                           {
-                            nrecords -= nrecsout;
+                            nrecords -= numFieldsout;
                             cdo_warning("File size limit reached for GrADS GRIB map_version=%d! Only the first %d time "
                                         "steps (2GB) are processed.",
                                         map_version, tsID);
@@ -1107,7 +1103,7 @@ public:
     operatorID = cdo_operator_id();
 
     datfile = cdo_get_stream_name(0);
-    auto len = strlen(datfile);
+    auto len = std::strlen(datfile);
     ctlfile = new char[len + 10];
     std::strcpy(ctlfile, datfile);
 
@@ -1137,10 +1133,11 @@ public:
     streamID = cdo_open_read(0);
 
     vlistID = cdo_stream_inq_vlist(streamID);
+    varList = VarList(vlistID);
 
-    auto nvars = vlistNvars(vlistID);
-    auto ntsteps = vlistNtsteps(vlistID);
-    auto ngrids = vlistNumGrids(vlistID);
+    auto numVars = varList.numVars();
+    auto numSteps = varList.numSteps();
+    auto numGrids = varList.numGrids();
 
     filetype = cdo_inq_filetype(streamID);
     auto byteorder = cdo_inq_byteorder(streamID);
@@ -1163,7 +1160,7 @@ public:
       }
 
     // find the first lonlat or Gaussian grid
-    for (index = 0; index < ngrids; ++index)
+    for (index = 0; index < numGrids; ++index)
       {
         gridID = vlistGrid(vlistID, index);
         gridtype = gridInqType(gridID);
@@ -1173,16 +1170,14 @@ public:
           break;
       }
 
-    if (index == ngrids) cdo_abort("No Lon/Lat, Gaussian or Lambert grid found (%s data unsupported)!", gridNamePtr(gridtype));
-
-    varList = VarList(vlistID);
+    if (index == numGrids) cdo_abort("No Lon/Lat, Gaussian or Lambert grid found (%s data unsupported)!", gridNamePtr(gridtype));
 
     // select all variables with used gridID
-    vars = std::vector<int>(nvars);
-    recoffset = std::vector<int>(nvars);
-    for (int varID = 0; varID < nvars; ++varID)
+    vars = std::vector<int>(numVars);
+    recoffset = std::vector<int>(numVars);
+    for (int varID = 0; varID < numVars; ++varID)
       {
-        const auto &var = varList.vars[varID];
+        auto const &var = varList.vars[varID];
         if (var.gridID == gridID)
           {
             if (filetype == CDI_FILETYPE_SRV || filetype == CDI_FILETYPE_EXT || filetype == CDI_FILETYPE_IEG)
@@ -1191,10 +1186,10 @@ public:
                 if (prec == CDI_DATATYPE_FLT64) flt64 = true;
               }
             vars[varID] = true;
-            recoffset[varID] = nrecsout;
+            recoffset[varID] = numFieldsout;
             nvarsout++;
-            nrecsout += zaxisInqSize(var.zaxisID);
-            if (ntsteps != 1 && ntsteps != 0 && var.isConstant)
+            numFieldsout += zaxisInqSize(var.zaxisID);
+            if (numSteps != 1 && numSteps != 0 && var.isConstant)
               cdo_abort("Unsupported GrADS record structure! Variable %d has only 1 time step.", var.code);
           }
         else
@@ -1204,7 +1199,7 @@ public:
           }
       }
 
-    if (filetype != CDI_FILETYPE_GRB && nvars != nvarsout) cdo_abort("Too many different grids!");
+    if (filetype != CDI_FILETYPE_GRB && numVars != nvarsout) cdo_abort("Too many different grids!");
 
     if (filetype == CDI_FILETYPE_SRV)
       {
@@ -1292,8 +1287,7 @@ public:
             fprintf(gdp, "INDEX  ^%s\n", pidxfile);
           }
 
-        gridsize = vlistGridsizeMax(vlistID);
-        array.resize(gridsize);
+        array.resize(varList.gridsizeMax());
       }
     else if (filetype == CDI_FILETYPE_NC) { fprintf(gdp, "DTYPE  NetCDF\n"); }
 
