@@ -182,7 +182,7 @@ gen_boxgrid_curv2D(int gridID1, size_t xinc, size_t yinc, int gridID2)
                   yvals2[g2_add] += yval1 / area_norm;
                 }
             }  // x1
-        }      // y1
+        }  // y1
 
       //  while ( xvals2[g2_add] >  180. ) xvals2[g2_add] -= 360.;
       //  while ( xvals2[g2_add] < -180. ) xvals2[g2_add] += 360.;
@@ -226,13 +226,13 @@ gen_boxgrid(int gridID1, size_t xinc, size_t yinc)
 }
 
 static void
-gridbox_stat(const Field &field1, Field &field2, size_t xinc, size_t yinc, int statfunc)
+gridbox_stat(Field const &field1, Field &field2, size_t xinc, size_t yinc, int statfunc)
 {
   auto useWeight = !field1.weightv.empty();
 
   auto boxsize = xinc * yinc;
-  FieldVector fields(Threading::ompNumThreads);
-  for (int i = 0; i < Threading::ompNumThreads; ++i)
+  FieldVector fields(Threading::ompNumMaxThreads);
+  for (int i = 0; i < Threading::ompNumMaxThreads; ++i)
     {
       fields[i].resize(boxsize);
       if (useWeight) fields[i].weightv.resize(boxsize);
@@ -320,8 +320,8 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   Field field1, field2;
   VarList varList1;
@@ -358,13 +358,13 @@ public:
     taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
 
-    auto ngrids = vlistNumGrids(vlistID1);
-    if (ngrids > 1) cdo_abort("Too many different grids!");
+    auto numGrids = vlistNumGrids(vlistID1);
+    if (numGrids > 1) cdo_abort("Too many different grids!");
 
     auto gridID1 = vlistGrid(vlistID1, 0);
 
     auto gridID2 = gen_boxgrid(gridID1, xinc, yinc);
-    for (int index = 0; index < ngrids; ++index) vlistChangeGridIndex(vlistID2, index, gridID2);
+    for (int index = 0; index < numGrids; ++index) vlistChangeGridIndex(vlistID2, index, gridID2);
 
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
@@ -383,18 +383,17 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             field1.init(varList1.vars[varID]);
-            cdo_read_record(streamID1, field1);
+            cdo_read_field(streamID1, field1);
 
             if (needWeights && field1.grid != lastgrid)
               {
@@ -408,8 +407,8 @@ public:
             field2.init(varList2.vars[varID]);
             gridbox_stat(field1, field2, xinc, yinc, operfunc);
 
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, field2);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, field2);
           }
         tsID++;
       }
