@@ -22,6 +22,7 @@
 #include "constants.h"
 #include "param_conversion.h"
 #include "pmlist.h"
+#include "util_string.h"
 
 double gridGetPlanetRadius(int gridID);
 
@@ -79,7 +80,7 @@ gridcell_areas(int gridID, Varray<double> &array, double planetRadiusInMeter)
         }
       else
         {
-          auto status = gridGenArea(gridID, array.data());
+          auto status = gridGenArea(gridID, array);
           // clang-format off
           if      (status == 1) cdo_abort("%s: Cell corner coordinates missing!", __func__);
           else if (status == 2) cdo_abort("%s: Can't compute grid cell area for this grid!", __func__);
@@ -199,19 +200,19 @@ get_parameter(double &radiusInMeter)
   auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      const auto &pargv = cdo_get_oper_argv();
+      auto const &pargv = cdo_get_oper_argv();
 
       KVList kvlist;
       kvlist.name = cdo_module_name();
       if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-          const auto &value = kv.values[0];
+          auto const &value = kv.values[0];
 
           // clang-format off
           if (key == "radius") radiusInMeter = radius_str_to_meter(value);
@@ -246,7 +247,7 @@ public:
 
   int operatorID;
   int gridID;
-  int vlistID2;
+  int vlistID2{ CDI_UNDEFID };
 
   size_t gridsize;
 
@@ -286,9 +287,9 @@ public:
     streamID1 = cdo_open_read(0);
     auto vlistID1 = cdo_stream_inq_vlist(streamID1);
 
-    auto ngrids = vlistNumGrids(vlistID1);
+    auto numGrids = vlistNumGrids(vlistID1);
 
-    if (ngrids > 1) cdo_warning("Found more than 1 grid, using the first one!");
+    if (numGrids > 1) cdo_warning("Found more than 1 grid, using the first one!");
 
     gridID = vlistGrid(vlistID1, 0);
     auto zaxisID = zaxis_from_name("surface");
@@ -376,8 +377,8 @@ public:
     auto streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
     cdo_def_timestep(streamID2, 0);
-    cdo_def_record(streamID2, 0, 0);
-    cdo_write_record(streamID2, &array[0], 0);
+    cdo_def_field(streamID2, 0, 0);
+    cdo_write_field(streamID2, &array[0], 0);
     cdo_stream_close(streamID2);
   }
 
