@@ -16,7 +16,7 @@ struct grid_type
 {
   int gridID;
   long size;
-  long num_cell_corners;
+  long numCorners;
   Varray<double> cell_corner_lon;
   Varray<double> cell_corner_lat;
 };
@@ -54,11 +54,11 @@ grid_new(int gridID, const char *txt)
 
   grid->gridID = gridID;
   grid->size = gridInqSize(grid->gridID);
-  grid->num_cell_corners = (gridInqType(grid->gridID) == GRID_UNSTRUCTURED) ? gridInqNvertex(grid->gridID) : 4;
+  grid->numCorners = (gridInqType(grid->gridID) == GRID_UNSTRUCTURED) ? gridInqNvertex(grid->gridID) : 4;
 
-  // printf("%s grid size %ld nv %ld\n", txt, grid->size, grid->num_cell_corners);
-  grid->cell_corner_lon.resize(grid->num_cell_corners * grid->size);
-  grid->cell_corner_lat.resize(grid->num_cell_corners * grid->size);
+  // printf("%s grid size %ld nv %ld\n", txt, grid->size, grid->numCorners);
+  grid->cell_corner_lon.resize(grid->numCorners * grid->size);
+  grid->cell_corner_lat.resize(grid->numCorners * grid->size);
   gridInqXbounds(grid->gridID, grid->cell_corner_lon.data());
   gridInqYbounds(grid->gridID, grid->cell_corner_lat.data());
 
@@ -77,7 +77,7 @@ grid_delete(grid_type *grid)
 }
 
 static void
-boundbox_from_corners1r(long ic, long nc, const Varray<double> &cornerLon, const Varray<double> &cornerLat, float *bound_box)
+boundbox_from_corners1r(long ic, long nc, Varray<double> const &cornerLon, Varray<double> const &cornerLat, float *bound_box)
 {
   auto inc = ic * nc;
 
@@ -110,7 +110,7 @@ boundbox_from_corners1r(long ic, long nc, const Varray<double> &cornerLon, const
 }
 
 static void
-boundbox_from_corners(long size, long nc, const Varray<double> &cornerLon, const Varray<double> &cornerLat,
+boundbox_from_corners(long size, long nc, Varray<double> const &cornerLon, Varray<double> const &cornerLat,
                       Varray<float> &bound_box)
 {
   for (long i = 0; i < size; ++i)
@@ -145,7 +145,7 @@ cellsearch_new(grid_type *srcGrid, grid_type *tgtGrid)
 
   cellsearch->src_cell_bound_box.resize(4 * srcGrid->size);
 
-  boundbox_from_corners(srcGrid->size, srcGrid->num_cell_corners, srcGrid->cell_corner_lon, srcGrid->cell_corner_lat,
+  boundbox_from_corners(srcGrid->size, srcGrid->numCorners, srcGrid->cell_corner_lon, srcGrid->cell_corner_lat,
                         cellsearch->src_cell_bound_box);
 
   return cellsearch;
@@ -162,10 +162,10 @@ search_cells(cellsearch_type *cellsearch, long tgtCellIndex, long *srchIndices)
 {
   const grid_type *srcGrid = cellsearch->srcGrid;
   const grid_type *tgtGrid = cellsearch->tgtGrid;
-  const auto &src_cell_bound_box = cellsearch->src_cell_bound_box;
+  auto const &src_cell_bound_box = cellsearch->src_cell_bound_box;
 
   float tgt_cell_bound_box[4];
-  boundbox_from_corners1r(tgtCellIndex, tgtGrid->num_cell_corners, tgtGrid->cell_corner_lon, tgtGrid->cell_corner_lat,
+  boundbox_from_corners1r(tgtCellIndex, tgtGrid->numCorners, tgtGrid->cell_corner_lon, tgtGrid->cell_corner_lat,
                           tgt_cell_bound_box);
 
   auto bound_box_lat1 = tgt_cell_bound_box[0];
