@@ -20,7 +20,7 @@
 #include "pmlist.h"
 #include "mpim_grid.h"
 
-enum class Stat
+enum struct Stat
 {
   Mean = 1,
   Avg = 2
@@ -107,10 +107,10 @@ degrade(const Varray<T1> &v1, size_t gridsize2, Varray<T2> &v2, bool hasMissvals
 }
 
 static void
-hp_degrade(const Field &field1, Field &field2, const HealpixParams &params)
+hp_degrade(Field const &field1, Field &field2, const HealpixParams &params)
 {
   auto hasMissvals = (field1.numMissVals > 0);
-  auto func = [&](auto &v1, auto &v2, auto size, double mv) { degrade(v1, size, v2, hasMissvals, mv, params); };
+  auto func = [&](auto const &v1, auto &v2, auto size, double mv) { degrade(v1, size, v2, hasMissvals, mv, params); };
   field_operation2(func, field1, field2, field2.gridsize, field1.missval);
   if (hasMissvals) field_num_mv(field2);
 }
@@ -135,10 +135,10 @@ upgrade(size_t gridsize1, const Varray<T1> &v1, Varray<T2> &v2, bool hasMissvals
 }
 
 static void
-hp_upgrade(const Field &field1, Field &field2, const HealpixParams &params)
+hp_upgrade(Field const &field1, Field &field2, const HealpixParams &params)
 {
   auto hasMissvals = (field1.numMissVals > 0);
-  auto func = [&](auto &v1, auto &v2, auto size, double mv) { upgrade(size, v1, v2, hasMissvals, mv, params); };
+  auto func = [&](auto const &v1, auto &v2, auto size, double mv) { upgrade(size, v1, v2, hasMissvals, mv, params); };
   field_operation2(func, field1, field2, field1.gridsize, field1.missval);
   if (hasMissvals) field_num_mv(field2);
 }
@@ -174,7 +174,7 @@ nested_to_ring(Field &field, int nside)
 }
 
 static Stat
-set_stat(const std::string &statString)
+set_stat(std::string const &statString)
 {
   if (statString == "mean")
     return Stat::Mean;
@@ -194,19 +194,19 @@ get_parameter(void)
   auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      auto &pargv = cdo_get_oper_argv();
+      auto const &pargv = cdo_get_oper_argv();
 
       KVList kvlist;
       kvlist.name = cdo_module_name();
       if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-          const auto &value = kv.values[0];
+          auto const &value = kv.values[0];
 
           // clang-format off
           if      (key == "nside") params.nsideOut = parameter_to_int(value);
@@ -237,9 +237,9 @@ define_healpix_grid(size_t gridsize, int nside, HpOrder order)
   cdiDefKeyString(gridID, CDI_GLOBAL, CDI_KEY_DIMNAME, "cells");
   cdiDefKeyString(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_VARNAME, projection);
   cdiDefKeyString(gridID, CDI_GLOBAL, CDI_KEY_GRIDMAP_NAME, projection);
-  cdiDefAttTxt(gridID, CDI_GLOBAL, "grid_mapping_name", (int) strlen(projection), projection);
+  cdiDefAttTxt(gridID, CDI_GLOBAL, "grid_mapping_name", (int) std::strlen(projection), projection);
   cdiDefAttInt(gridID, CDI_GLOBAL, "healpix_nside", CDI_DATATYPE_INT32, 1, &nside);
-  cdiDefAttTxt(gridID, CDI_GLOBAL, "healpix_order", (int) strlen(orderString), orderString);
+  cdiDefAttTxt(gridID, CDI_GLOBAL, "healpix_order", (int) std::strlen(orderString), orderString);
 
   return gridID;
 }
@@ -249,7 +249,9 @@ hp_define_grid(int gridID1, HealpixParams &params)
 {
   int gridIDout = -1;
 
-  auto [nside, order] = cdo::get_healpix_params(gridID1);
+  auto hpParams = cdo::get_healpix_params(gridID1);
+  auto nside = hpParams.get_nside();
+  auto order = hpParams.get_order();
   params.nsideIn = nside;
   params.orderIn = order;
 
@@ -301,9 +303,9 @@ public:
   int HPDEGRADE;
   CdoStreamID streamID1;
   CdoStreamID streamID2;
-  int taxisID1;
-  int taxisID2;
-  int vlistID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
   bool doDegrade;
 
@@ -332,8 +334,8 @@ public:
     auto vlistID1 = cdo_stream_inq_vlist(streamID1);
     taxisID1 = vlistInqTaxis(vlistID1);
 
-    auto ngrids = vlistNumGrids(vlistID1);
-    if (ngrids > 1) cdo_abort("Too many different grids!");
+    auto numGrids = vlistNumGrids(vlistID1);
+    if (numGrids > 1) cdo_abort("Too many different grids!");
 
     auto gridID = vlistGrid(vlistID1, 0);
     if (!is_healpix_grid(gridID)) cdo_abort("Input grid is not healpix!");
@@ -343,7 +345,7 @@ public:
     vlistDefTaxis(vlistID2, taxisID2);
 
     auto gridID2 = hp_define_grid(gridID, params);
-    for (int index = 0; index < ngrids; ++index) vlistChangeGridIndex(vlistID2, index, gridID2);
+    for (int index = 0; index < numGrids; ++index) vlistChangeGridIndex(vlistID2, index, gridID2);
 
     varList1 = VarList(vlistID1);
     varList2 = VarList(vlistID2);
@@ -358,17 +360,17 @@ public:
     int tsID1 = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID1);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID1);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID1);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             field1.init(varList1.vars[varID]);
-            cdo_read_record(streamID1, field1);
+            cdo_read_field(streamID1, field1);
 
             field2.init(varList2.vars[varID]);
 
@@ -378,8 +380,8 @@ public:
 
             if (params.orderOut == HpOrder::Ring) nested_to_ring(field2, params.nsideOut);
 
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, field2);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, field2);
           }
 
         tsID1++;
