@@ -34,7 +34,7 @@ humidityIndex(double t, double e, double r, double missval)
 }
 
 static void
-farexpr(Field &field1, const Field &field2, const Field &field3, double (*expression)(double, double, double, double))
+farexpr(Field &field1, Field const &field2, Field const &field3, double (*expression)(double, double, double, double))
 {
   auto grid1 = field1.grid;
   auto grid2 = field2.grid;
@@ -82,12 +82,12 @@ public:
   CdoStreamID streamID3;
   CdoStreamID streamID4;
 
-  int vlistID1;
-  int vlistID2;
+  int vlistID1{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
   int vlistID3;
   int vlistID4;
 
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
   // int taxisID2
   // int taxisID3
   int taxisID4;
@@ -137,8 +137,7 @@ public:
     taxisID4 = cdo_taxis_create(TAXIS_RELATIVE);
     taxisDefTunit(taxisID4, TUNIT_MINUTE);
     taxisDefCalendar(taxisID4, CALENDAR_STANDARD);
-    taxisDefRdate(taxisID4, 19550101);
-    taxisDefRtime(taxisID4, 0);
+    taxisDefRdatetime(taxisID4, cdiDateTime_set(19550101, 0));
     vlistDefTaxis(vlistID4, taxisID4);
 
     constexpr char HI_NAME[] = "hum_index";
@@ -167,29 +166,29 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
-        auto nrecs2 = cdo_stream_inq_timestep(streamID2, tsID);
-        auto nrecs3 = cdo_stream_inq_timestep(streamID3, tsID);
-        if (nrecs2 == 0 || nrecs3 == 0) cdo_abort("Input streams have different number of timesteps!");
+        auto numFields2 = cdo_stream_inq_timestep(streamID2, tsID);
+        auto numFields3 = cdo_stream_inq_timestep(streamID3, tsID);
+        if (numFields2 == 0 || numFields3 == 0) cdo_abort("Input streams have different number of timesteps!");
 
         cdo_taxis_copy_timestep(taxisID4, taxisID1);
         cdo_def_timestep(streamID4, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID1, levelID1] = cdo_inq_record(streamID1);
+            auto [varID1, levelID1] = cdo_inq_field(streamID1);
             field1.init(varList1.vars[varID1]);
-            cdo_read_record(streamID1, field1);
+            cdo_read_field(streamID1, field1);
 
-            auto [varID2, levelID2] = cdo_inq_record(streamID2);
+            auto [varID2, levelID2] = cdo_inq_field(streamID2);
             field2.init(varList2.vars[varID2]);
-            cdo_read_record(streamID2, field2);
+            cdo_read_field(streamID2, field2);
 
-            auto [varID3, levelID3] = cdo_inq_record(streamID3);
+            auto [varID3, levelID3] = cdo_inq_field(streamID3);
             field3.init(varList3.vars[varID3]);
-            cdo_read_record(streamID3, field3);
+            cdo_read_field(streamID3, field3);
 
             if (varID1 != varID2 || varID1 != varID3 || levelID1 != levelID2 || levelID1 != levelID3)
               cdo_abort("Input streams have different structure!");
@@ -198,8 +197,8 @@ public:
 
             farexpr(field1, field2, field3, humidityIndex);
 
-            cdo_def_record(streamID4, varID4, levelID1);
-            cdo_write_record(streamID4, field1);
+            cdo_def_field(streamID4, varID4, levelID1);
+            cdo_write_field(streamID4, field1);
           }
 
         tsID++;
