@@ -39,21 +39,15 @@ private:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  Varray2D<double> vardata;
-  Varray2D<double> varcount;
-  Varray2D<double> vartcount;
-
-  Varray<double> array;
   VarList varList1;
   VarList varList2;
 
-  int nbins;
+  int numBins;
 
   std::vector<double> fltarr;
-  int nvars;
   int operatorID;
 
 public:
@@ -71,25 +65,26 @@ public:
 
     operator_input_arg("bins");
 
-    fltarr = cdo_argv_to_flt(cdo_get_oper_argv());
-    nbins = fltarr.size() - 1;
-    if (nbins < 1) cdo_abort("Too few arguments!");
+    fltarr = cdo_argv_to_fltarr(cdo_get_oper_argv());
+    numBins = fltarr.size() - 1;
+    if (numBins < 1) cdo_abort("Too few arguments!");
 
     if (Options::cdoVerbose)
       {
-        printf("nbins = %d\n", nbins);
-        for (int i = 0; i < nbins; ++i) printf("flt %d = %g\n", i + 1, fltarr[i]);
+        printf("nbins = %d\n", numBins);
+        for (int i = 0; i < numBins; ++i) printf("flt %d = %g\n", i + 1, fltarr[i]);
       }
 
     streamID1 = cdo_open_read(0);
     auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+    varList1 = VarList(vlistID1);
 
     taxisID1 = vlistInqTaxis(vlistID1);
 
     auto vlistID2 = vlistDuplicate(vlistID1);
 
     /* create zaxis for output bins */
-    auto zaxisID2 = zaxisCreate(ZAXIS_GENERIC, nbins);
+    auto zaxisID2 = zaxisCreate(ZAXIS_GENERIC, numBins);
 
     zaxisDefLevels(zaxisID2, &fltarr[0]);
     zaxisDefLbounds(zaxisID2, &fltarr[0]);
@@ -99,13 +94,13 @@ public:
     cdiDefKeyString(zaxisID2, CDI_GLOBAL, CDI_KEY_LONGNAME, "histogram bins");
     cdiDefKeyString(zaxisID2, CDI_GLOBAL, CDI_KEY_UNITS, "level");
 
-    /* check zaxis: only 2D fields allowed */
-    auto nzaxis = vlistNumZaxis(vlistID1);
-    for (int index = 0; index < nzaxis; ++index)
+    // check zaxis: only 2D fields allowed
+    auto numZaxes = varList1.numZaxes();
+    for (int index = 0; index < numZaxes; ++index)
       {
         auto zaxisID = vlistZaxis(vlistID1, index);
-        auto nlevel = zaxisInqSize(zaxisID);
-        if (nlevel > 1) cdo_abort("Found 3D field with %d levels. Only 2D fields allowed!", nlevel);
+        auto numLevels = zaxisInqSize(zaxisID);
+        if (numLevels > 1) cdo_abort("Found 3D field with %d levels. Only 2D fields allowed!", numLevels);
         vlistChangeZaxisIndex(vlistID2, index, zaxisID2);
       }
 
@@ -116,44 +111,39 @@ public:
 
     cdo_def_vlist(streamID2, vlistID2);
 
-    varList1 = VarList(vlistID1);
     varList2 = VarList(vlistID2);
-
-    nvars = vlistNvars(vlistID2);
-
-    vardata = Varray2D<double>(nvars);
-    varcount = Varray2D<double>(nvars);
-    vartcount = Varray2D<double>(nvars);
-
-    for (int varID = 0; varID < nvars; ++varID)
-      {
-        auto gridsize = varList1.vars[varID].gridsize;
-        vardata[varID].resize(nbins * gridsize, 0);
-        varcount[varID].resize(nbins * gridsize, 0);
-        vartcount[varID].resize(gridsize, 0);
-      }
-
-    auto gridsizemax = vlistGridsizeMax(vlistID1);
-    array = Varray<double>(gridsizemax);
   }
 
   void
   run() override
   {
+    auto numVars = varList2.numVars();
+    Varray2D<double> vardata(numVars);
+    Varray2D<double> varcount(numVars);
+    Varray2D<double> vartcount(numVars);
+    for (int varID = 0; varID < numVars; ++varID)
+      {
+        auto gridsize = varList1.vars[varID].gridsize;
+        vardata[varID].resize(numBins * gridsize, 0);
+        varcount[varID].resize(numBins * gridsize, 0);
+        vartcount[varID].resize(gridsize, 0);
+      }
+
+    Varray<double> array(varList1.gridsizeMax());
+
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID++);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID++);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             size_t numMissVals;
-            cdo_read_record(streamID1, array.data(), &numMissVals);
+            cdo_read_field(streamID1, array.data(), &numMissVals);
 
             auto gridsize = varList1.vars[varID].gridsize;
             auto missval = varList1.vars[varID].missval;
@@ -165,7 +155,7 @@ public:
                   {
                     vartcount[varID][i] += 1;
                     int index = 0;
-                    while (index < nbins)
+                    while (index < numBins)
                       {
                         auto offset = gridsize * index;
                         if (!fp_is_equal(vardata[varID][offset + i], missval) && array[i] >= fltarr[index]
@@ -188,14 +178,14 @@ public:
 
     cdo_def_timestep(streamID2, 0);
 
-    for (int varID = 0; varID < nvars; ++varID)
+    for (int varID = 0; varID < numVars; ++varID)
       {
         auto gridsize = varList2.vars[varID].gridsize;
         auto missval = varList2.vars[varID].missval;
 
         // fix missing values
 
-        for (int index = 0; index < nbins; ++index)
+        for (int index = 0; index < numBins; ++index)
           {
             size_t numMissVals = 0;
             auto offset = gridsize * index;
@@ -223,12 +213,12 @@ public:
                   }
               }
 
-            cdo_def_record(streamID2, varID, index);
+            cdo_def_field(streamID2, varID, index);
 
             if (operatorID == HISTCOUNT)
-              cdo_write_record(streamID2, &varcount[varID][offset], numMissVals);
+              cdo_write_field(streamID2, &varcount[varID][offset], numMissVals);
             else
-              cdo_write_record(streamID2, &vardata[varID][offset], numMissVals);
+              cdo_write_field(streamID2, &vardata[varID][offset], numMissVals);
           }
       }
   }
