@@ -8,7 +8,6 @@
 /*
    This module contains the following operators:
 
-      Intgrid    interpolate     PINGO grid interpolation
       Intgrid    intgridbil      Bilinear grid interpolation
 */
 
@@ -20,10 +19,11 @@
 #include <mpim_grid.h>
 #include "griddes.h"
 #include "matrix_view.h"
+#include "remapknn.h"
 
-template <typename T>
+template <typename T1, typename T2>
 static void
-thinout(const Varray<T> &varray1, Varray<T> &varray2, int gridID1, int gridID2, size_t xinc, size_t yinc)
+thinout(const Varray<T1> &varray1, Varray<T2> &varray2, int gridID1, int gridID2, size_t xinc, size_t yinc)
 {
   auto nlon1 = gridInqXsize(gridID1);
   auto nlat1 = gridInqYsize(gridID1);
@@ -31,8 +31,8 @@ thinout(const Varray<T> &varray1, Varray<T> &varray2, int gridID1, int gridID2,
   auto nlon2 = gridInqXsize(gridID2);
   auto nlat2 = gridInqYsize(gridID2);
 
-  MatrixView<const T> xfield1(varray1.data(), nlat1, nlon1);
-  MatrixView<T> xfield2(varray2.data(), nlat2, nlon2);
+  MatrixView<const T1> xfield1(varray1.data(), nlat1, nlon1);
+  MatrixView<T2> xfield2(varray2.data(), nlat2, nlon2);
 
   size_t olat = 0;
   for (size_t ilat = 0; ilat < nlat1; ilat += yinc)
@@ -48,15 +48,10 @@ thinout(const Varray<T> &varray1, Varray<T> &varray2, int gridID1, int gridID2,
 }
 
 static void
-thinout(const Field &field1, Field &field2, size_t xinc, size_t yinc)
+thinout(Field const &field1, Field &field2, size_t xinc, size_t yinc)
 {
-  if (field1.memType != field2.memType) cdo_abort("Interal error, memType of field1 and field2 differ!");
-
-  if (field1.memType == MemType::Float)
-    thinout(field1.vec_f, field2.vec_f, field1.grid, field2.grid, xinc, yinc);
-  else
-    thinout(field1.vec_d, field2.vec_d, field1.grid, field2.grid, xinc, yinc);
-
+  auto func = [&](auto const &v1, auto &v2, auto grid1, auto grid2) { thinout(v1, v2, grid1, grid2, xinc, yinc); };
+  field_operation2(func, field1, field2, field1.grid, field2.grid);
   field_num_mv(field2);
 }
 
@@ -137,43 +132,48 @@ gen_boxavg_grid(int gridID1, size_t xinc, size_t yinc)
       gridInqXvals(gridID1, &xvals1[0]);
       gridInqYvals(gridID1, &yvals1[0]);
 
-      Varray<double> grid1_corner_lon, grid1_corner_lat;
+      for (size_t i = 0, j = 0; i < nlon1; i += xinc)
+        {
+          auto i1 = i + (xinc - 1);
+          if (i1 >= nlon1 - 1) i1 = nlon1 - 1;
+          xvals2[j] = xvals1[i] + (xvals1[i1] - xvals1[i]) / 2;
+          j++;
+        }
+      for (size_t i = 0, j = 0; i < nlat1; i += yinc)
+        {
+          auto i1 = i + (yinc - 1);
+          if (i1 >= nlat1 - 1) i1 = nlat1 - 1;
+          yvals2[j] = yvals1[i] + (yvals1[i1] - yvals1[i]) / 2;
+          j++;
+        }
+
       Varray<double> grid2_corner_lon, grid2_corner_lat;
       if (gridHasBounds(gridID1))
         {
+          Varray<double> grid1_corner_lon, grid1_corner_lat;
           grid1_corner_lon.resize(2 * nlon1);
           grid1_corner_lat.resize(2 * nlat1);
           grid2_corner_lon.resize(2 * nlon2);
           grid2_corner_lat.resize(2 * nlat2);
           gridInqXbounds(gridID1, &grid1_corner_lon[0]);
           gridInqYbounds(gridID1, &grid1_corner_lat[0]);
-        }
 
-      size_t j = 0;
-      for (size_t i = 0; i < nlon1; i += xinc)
-        {
-          auto i1 = i + (xinc - 1);
-          if (i1 >= nlon1 - 1) i1 = nlon1 - 1;
-          xvals2[j] = xvals1[i] + (xvals1[i1] - xvals1[i]) / 2;
-          if (!grid2_corner_lon.empty())
+          for (size_t i = 0, j = 0; i < nlon1; i += xinc)
             {
+              auto i1 = i + (xinc - 1);
+              if (i1 >= nlon1 - 1) i1 = nlon1 - 1;
               grid2_corner_lon[2 * j] = grid1_corner_lon[2 * i];
               grid2_corner_lon[2 * j + 1] = grid1_corner_lon[2 * i1 + 1];
+              j++;
             }
-          j++;
-        }
-      j = 0;
-      for (size_t i = 0; i < nlat1; i += yinc)
-        {
-          auto i1 = i + (yinc - 1);
-          if (i1 >= nlat1 - 1) i1 = nlat1 - 1;
-          yvals2[j] = yvals1[i] + (yvals1[i1] - yvals1[i]) / 2;
-          if (!grid2_corner_lat.empty())
+          for (size_t i = 0, j = 0; i < nlat1; i += yinc)
             {
+              auto i1 = i + (yinc - 1);
+              if (i1 >= nlat1 - 1) i1 = nlat1 - 1;
               grid2_corner_lat[2 * j] = grid1_corner_lat[2 * i];
               grid2_corner_lat[2 * j + 1] = grid1_corner_lat[2 * i1 + 1];
+              j++;
             }
-          j++;
         }
 
       gridDefXvals(gridID2, &xvals2[0]);
@@ -191,9 +191,9 @@ gen_boxavg_grid(int gridID1, size_t xinc, size_t yinc)
   return gridID2;
 }
 
-template <typename T>
+template <typename T1, typename T2>
 static void
-boxavg(const Varray<T> &varray1, Varray<T> &varray2, int gridID1, int gridID2, size_t xinc, size_t yinc)
+boxavg(const Varray<T1> &varray1, Varray<T2> &varray2, int gridID1, int gridID2, size_t xinc, size_t yinc)
 {
   auto nlon1 = gridInqXsize(gridID1);
   auto nlat1 = gridInqYsize(gridID1);
@@ -201,14 +201,13 @@ boxavg(const Varray<T> &varray1, Varray<T> &varray2, int gridID1, int gridID2, s
   auto nlon2 = gridInqXsize(gridID2);
   auto nlat2 = gridInqYsize(gridID2);
 
-  MatrixView<const T> xfield1(varray1.data(), nlat1, nlon1);
-  MatrixView<T> xfield2(varray2.data(), nlat2, nlon2);
+  MatrixView<const T1> xfield1(varray1.data(), nlat1, nlon1);
+  MatrixView<T2> xfield2(varray2.data(), nlat2, nlon2);
 
   for (size_t ilat = 0; ilat < nlat2; ilat++)
     for (size_t ilon = 0; ilon < nlon2; ilon++)
       {
         double xsum = 0.0;
-
         size_t in = 0;
         for (size_t j = 0; j < yinc; ++j)
           {
@@ -228,20 +227,28 @@ boxavg(const Varray<T> &varray1, Varray<T> &varray2, int gridID1, int gridID2, s
 }
 
 static void
-boxavg(const Field &field1, Field &field2, size_t xinc, size_t yinc)
+boxavg(Field const &field1, Field &field2, size_t xinc, size_t yinc)
 {
-  if (field1.memType != field2.memType) cdo_abort("Interal error, memType of field1 and field2 differ!");
-
-  if (field1.memType == MemType::Float)
-    boxavg(field1.vec_f, field2.vec_f, field1.grid, field2.grid, xinc, yinc);
-  else
-    boxavg(field1.vec_d, field2.vec_d, field1.grid, field2.grid, xinc, yinc);
-
+  auto func = [&](auto const &v1, auto &v2, auto grid1, auto grid2) { boxavg(v1, v2, grid1, grid2, xinc, yinc); };
+  field_operation2(func, field1, field2, field1.grid, field2.grid);
   field_num_mv(field2);
 }
 
-class ModuleIntgrid
+class Intgrid : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Intgrid",
+    .operators = { { "intgridbil" }, { "intgriddis" }, { "intgridnn" }, { "intgridknn" }, { "boxavg" }, { "thinout" } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Intgrid> registration = RegisterEntry<Intgrid>(module);
+
+  int INTGRID_BIL, INTGRID_DIS, INTGRID_NN, INTGRID_KNN, BOXAVG, THINOUT;
   int gridID2 = -1;
   int xinc = 1, yinc = 1;
 
@@ -250,37 +257,65 @@ class ModuleIntgrid
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  bool lfieldmem;
+  bool useFeldMem;
 
-  VarList varList1, varList2;
-  Field field1, field2;
+  VarList varList1;
+  VarList varList2;
 
-  int INTGRIDBIL, INTGRIDDIS, INTGRIDNN, INTERPOLATE, BOXAVG, THINOUT;
+  KnnParams knnParams;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
+    if (this_is_the_only_process()) { cdiDefGlobal("READ_CELL_CORNERS", false); }
 
-    INTGRIDBIL = cdo_operator_add("intgridbil", 0, 0, nullptr);
-    INTGRIDDIS = cdo_operator_add("intgriddis", 0, 0, nullptr);
-    INTGRIDNN = cdo_operator_add("intgridnn", 0, 0, nullptr);
-    INTERPOLATE = cdo_operator_add("interpolate", 0, 0, nullptr);
-    BOXAVG = cdo_operator_add("boxavg", 0, 0, nullptr);
-    THINOUT = cdo_operator_add("thinout", 0, 0, nullptr);
+    INTGRID_BIL = module.get_id("intgridbil");
+    INTGRID_DIS = module.get_id("intgriddis");
+    INTGRID_NN = module.get_id("intgridnn");
+    INTGRID_KNN = module.get_id("intgridknn");
+    BOXAVG = module.get_id("boxavg");
+    THINOUT = module.get_id("thinout");
 
     operatorID = cdo_operator_id();
 
-    lfieldmem = (operatorID == INTGRIDBIL || operatorID == THINOUT || operatorID == BOXAVG);
+    useFeldMem = (operatorID == INTGRID_BIL || operatorID == THINOUT || operatorID == BOXAVG);
+    useFeldMem = true;
 
-    if (operatorID == INTGRIDBIL || operatorID == INTERPOLATE || operatorID == INTGRIDDIS || operatorID == INTGRIDNN)
+    if (operatorID == INTGRID_BIL || operatorID == INTGRID_DIS || operatorID == INTGRID_NN)
       {
         operator_input_arg("grid description file or name");
         gridID2 = cdo_define_grid(cdo_operator_argv(0));
+        if (operatorID == INTGRID_NN)
+          {
+            INTGRID_KNN = INTGRID_NN;
+            knnParams.k = 1;
+            knnParams.kMin = 1;
+            knnParams.extrapolate = true;
+            knnParams.weighted = WeightingMethod::distanceWeighted;
+          }
+        if (operatorID == INTGRID_DIS)
+          {
+            INTGRID_KNN = INTGRID_DIS;
+            knnParams.k = 4;
+            knnParams.kMin = 1;
+            knnParams.extrapolate = true;
+            knnParams.weighted = WeightingMethod::distanceWeighted;
+          }
+      }
+    else if (operatorID == INTGRID_KNN)
+      {
+        auto remapParams = remapknn_get_parameter();
+        if (Options::cdoVerbose) remapknn_print_parameter(remapParams);
+        if (remapParams.gridString.empty()) cdo_abort("target grid parameter missing!");
+        gridID2 = cdo_define_grid(remapParams.gridString);
+        knnParams = remapParams.knnParams;
+        if (knnParams.kMin == 0) knnParams.kMin = knnParams.k;
+        remapknn_verify_parameter(knnParams);
+        if (Options::cdoVerbose) print_knn_parameter(knnParams, "KNN parameter: ");
       }
     else if (operatorID == THINOUT || operatorID == BOXAVG)
       {
@@ -299,8 +334,8 @@ public:
     taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
 
-    auto ngrids = vlistNgrids(vlistID1);
-    for (int index = 0; index < ngrids; ++index)
+    auto numGrids = vlistNumGrids(vlistID1);
+    for (int index = 0; index < numGrids; ++index)
       {
         auto gridID1 = vlistGrid(vlistID1, index);
         auto gridtype = gridInqType(gridID1);
@@ -309,8 +344,7 @@ public:
           {
             if (index == 0)
               {
-                if (gridtype != GRID_LONLAT && gridtype != GRID_GAUSSIAN)
-                  cdo_abort("%s data unsupported!", gridNamePtr(gridtype));
+                if (gridtype != GRID_LONLAT && gridtype != GRID_GAUSSIAN) cdo_abort("%s data unsupported!", gridNamePtr(gridtype));
 
                 gridID2 = gen_boxavg_grid(gridID1, xinc, yinc);
               }
@@ -329,13 +363,13 @@ public:
             else
               cdo_abort("Too many different grids!");
           }
-        else if (operatorID == INTGRIDBIL || operatorID == INTERPOLATE)
+        else if (operatorID == INTGRID_BIL)
           {
             auto ldistgen = (grid_is_distance_generic(gridID1) && grid_is_distance_generic(gridID2));
             if (!ldistgen && gridtype != GRID_LONLAT && gridtype != GRID_GAUSSIAN)
               cdo_abort("Interpolation of %s data unsupported!", gridNamePtr(gridtype));
           }
-        else if (operatorID == INTGRIDNN || operatorID == INTGRIDDIS)
+        else if (operatorID == INTGRID_KNN)
           {
             auto hasProjParams = ((gridtype == GRID_PROJECTION) && grid_has_proj_params(gridID1));
             if (!gridProjIsSupported(gridID1) && !hasProjParams && gridtype != GRID_LONLAT && gridtype != GRID_GAUSSIAN
@@ -346,90 +380,71 @@ public:
         vlistChangeGridIndex(vlistID2, index, gridID2);
       }
 
-    varList_init(varList1, vlistID1);
-    varList_init(varList2, vlistID2);
-
-    auto gridsizemax = vlistGridsizeMax(vlistID1);
-    if (!lfieldmem) field1.resize(gridsizemax);
-
-    auto gridsize = gridInqSize(gridID2);
-    if (!lfieldmem) field2.resize(gridsize);
+    varList1 = VarList(vlistID1);
+    varList2 = VarList(vlistID2);
 
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
   }
 
   void
-  run()
+  run() override
   {
+    Field field1;
+    if (!useFeldMem) field1.resize(varList1.gridsizeMax());
+
+    auto gridsize = gridInqSize(gridID2);
+    Field field2;
+    if (!useFeldMem) field2.resize(gridsize);
+
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            if (lfieldmem)
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            if (useFeldMem)
               {
-                field1.init(varList1[varID]);
-                cdo_read_record(streamID1, field1);
+                field1.init(varList1.vars[varID]);
+                cdo_read_field(streamID1, field1);
 
-                field2.init(varList2[varID]);
+                field2.init(varList2.vars[varID]);
               }
             else
               {
-                cdo_read_record(streamID1, field1.vec_d.data(), &field1.nmiss);
+                cdo_read_field(streamID1, field1);
 
-                field1.grid = varList1[varID].gridID;
-                field1.missval = varList1[varID].missval;
+                field1.grid = varList1.vars[varID].gridID;
+                field1.missval = varList1.vars[varID].missval;
                 field2.grid = gridID2;
                 field2.missval = field1.missval;
-                field2.nmiss = 0;
+                field2.numMissVals = 0;
               }
 
             // clang-format off
-  	    if      (operatorID == INTGRIDBIL)  intgridbil(field1, field2);
-	    else if (operatorID == INTGRIDNN)   intgridnn(field1, field2);
-	    else if (operatorID == INTGRIDDIS)  intgriddis(field1, field2, 4);
-	    else if (operatorID == INTERPOLATE) interpolate(field1, field2);
-	    else if (operatorID == BOXAVG)      boxavg(field1, field2, xinc, yinc);
-	    else if (operatorID == THINOUT)     thinout(field1, field2, xinc, yinc);
+       	    if      (operatorID == INTGRID_BIL)  intgrid_bil(field1, field2);
+            else if (operatorID == INTGRID_KNN)  intgrid_knn(knnParams, field1, field2);
+            else if (operatorID == BOXAVG)       boxavg(field1, field2, xinc, yinc);
+            else if (operatorID == THINOUT)      thinout(field1, field2, xinc, yinc);
             // clang-format on
 
-            cdo_def_record(streamID2, varID, levelID);
-            if (lfieldmem)
-              cdo_write_record(streamID2, field2);
-            else
-              cdo_write_record(streamID2, field2.vec_d.data(), field2.nmiss);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, field2);
           }
         tsID++;
       }
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Intgrid(void *process)
-{
-
-  ModuleIntgrid intgrid;
-  intgrid.init(process);
-  intgrid.run();
-  intgrid.close();
-
-  return nullptr;
-}
