@@ -13,21 +13,19 @@
 
 #include <cdi.h>
 
-#include "cimdOmp.h"
+#include "cdo_omp.h"
 #include "cdo_options.h"
 #include "process_int.h"
-#include "cdo_vlist.h"
 #include "cdo_zaxis.h"
-#include "parse_literals.h"
 #include "pmlist.h"
 #include "param_conversion.h"
 
 template <typename T>
 static double
-vert_interp_lev_kernel(float w1, float w2, const T var1L1, const T var1L2, const T missval)
+vert_interp_lev_kernel(float w1, float w2, T var1L1, T var1L2, T missval)
 {
-  if (dbl_is_equal(var1L1, missval)) w1 = 0.0f;
-  if (dbl_is_equal(var1L2, missval)) w2 = 0.0f;
+  if (fp_is_equal(var1L1, missval)) w1 = 0.0f;
+  if (fp_is_equal(var1L2, missval)) w2 = 0.0f;
 
   // clang-format off
   if      (is_equal(w1, 0.0f) && is_equal(w2, 0.0f)) return missval;
@@ -68,11 +66,13 @@ restore_index_and_weights(int nlev1, int idx, float wgt, int &idx1, int &idx2, f
 }
 
 //  1D vertical interpolation
-template <typename T>
+template <typename T1, typename T2>
 static void
-vert_interp_lev(size_t gridsize, int nlev1, T missval, const Varray<T> &vardata1, Varray<T> &vardata2, int nlev2,
-                const Varray<int> &lev_idx, const Varray<float> &lev_wgt)
+vert_interp_lev(size_t gridsize, int nlev1, double mv, const Varray<T1> &vardata1, Varray<T2> &vardata2, int nlev2,
+                const Varray<int> &lev_idx, Varray<float> const &lev_wgt)
 {
+  T1 missval = mv;
+
   for (int ilev = 0; ilev < nlev2; ++ilev)
     {
       auto idx = lev_idx[ilev];
@@ -96,22 +96,20 @@ vert_interp_lev(size_t gridsize, int nlev1, T missval, const Varray<T> &vardata1
 
 static void
 vert_interp_lev(size_t gridsize, int nlev1, double missval, const Field3D &field1, Field3D &field2, int nlev2,
-                const Varray<int> &lev_idx, const Varray<float> &lev_wgt)
+                const Varray<int> &lev_idx, Varray<float> const &lev_wgt)
 {
-  if (field1.memType != field2.memType) cdo_abort("Interal error, memType of field1 and field2 differ!");
-
-  if (field1.memType == MemType::Float)
-    vert_interp_lev(gridsize, nlev1, (float) missval, field1.vec_f, field2.vec_f, nlev2, lev_idx, lev_wgt);
-  else
-    vert_interp_lev(gridsize, nlev1, missval, field1.vec_d, field2.vec_d, nlev2, lev_idx, lev_wgt);
+  auto func = [&](auto &v1, auto &v2) { vert_interp_lev(gridsize, nlev1, missval, v1, v2, nlev2, lev_idx, lev_wgt); };
+  field_operation2(func, field1, field2);
 }
 
 //  3D vertical interpolation
-template <typename T>
+template <typename T1, typename T2>
 void
-vert_interp_lev3d(size_t gridsize, int nlev1, T missval, const Varray<T> &vardata1, Varray<T> &vardata2, int nlev2,
-                  const Varray<int> &lev_idx, const Varray<float> &lev_wgt)
+vert_interp_lev3d(size_t gridsize, int nlev1, double mv, const Varray<T1> &vardata1, Varray<T2> &vardata2, int nlev2,
+                  const Varray<int> &lev_idx, Varray<float> const &lev_wgt)
 {
+  T1 missval = mv;
+
   for (int ilev = 0; ilev < nlev2; ilev++)
     {
       auto offset = ilev * gridsize;
@@ -137,32 +135,22 @@ vert_interp_lev3d(size_t gridsize, int nlev1, T missval, const Varray<T> &vardat
     }
 }
 
-// Explicit instantiation
-template void vert_interp_lev3d(size_t gridsize, int nlev1, float missval, const Varray<float> &vardata1, Varray<float> &vardata2,
-                                int nlev2, const Varray<int> &lev_idx, const Varray<float> &lev_wgt);
-template void vert_interp_lev3d(size_t gridsize, int nlev1, double missval, const Varray<double> &vardata1,
-                                Varray<double> &vardata2, int nlev2, const Varray<int> &lev_idx, const Varray<float> &lev_wgt);
-
 void
 vert_interp_lev3d(size_t gridsize, int nlev1, double missval, const Field3D &field1, Field3D &field2, int nlev2,
-                  const Varray<int> &lev_idx, const Varray<float> &lev_wgt)
+                  const Varray<int> &lev_idx, Varray<float> const &lev_wgt)
 {
-  if (field1.memType != field2.memType) cdo_abort("Interal error, memType of field1 and field2 differ!");
-
-  if (field1.memType == MemType::Float)
-    vert_interp_lev3d(gridsize, nlev1, (float) missval, field1.vec_f, field2.vec_f, nlev2, lev_idx, lev_wgt);
-  else
-    vert_interp_lev3d(gridsize, nlev1, missval, field1.vec_d, field2.vec_d, nlev2, lev_idx, lev_wgt);
+  auto func = [&](auto &v1, auto &v2) { vert_interp_lev3d(gridsize, nlev1, missval, v1, v2, nlev2, lev_idx, lev_wgt); };
+  field_operation2(func, field1, field2);
 }
 
 void
-vert_gen_weights(int expol, int nlev1, const Varray<double> &lev1, int nlev2, const Varray<double> &lev2, Varray<int> &lev_idx,
-                 Varray<float> &lev_wgt)
+vert_gen_weights(int extrapolate, int nlev1, Varray<double> const &lev1, int nlev2, Varray<double> const &lev2,
+                 Varray<int> &lev_idx, Varray<float> &lev_wgt)
 {
   for (int i2 = 0; i2 < nlev2; ++i2)
     {
       int idx1 = 0, idx2 = 0;
-      double val1, val2 = 0.0;
+      double val2 = 0.0;
 
       // Because 2 levels were added to the source vertical coordinate (one on top, one at the bottom), its loop starts at 1
       int i1;
@@ -171,7 +159,7 @@ vert_gen_weights(int expol, int nlev1, const Varray<double> &lev1, int nlev2, co
           auto lev1_isUp = (lev1[i1 - 1] < lev1[i1]);
           idx1 = lev1_isUp ? i1 - 1 : i1;
           idx2 = lev1_isUp ? i1 : i1 - 1;
-          val1 = lev1[idx1];
+          auto val1 = lev1[idx1];
           val2 = lev1[idx2];
           if (lev2[i2] > val1 && lev2[i2] <= val2) break;
         }
@@ -181,12 +169,12 @@ vert_gen_weights(int expol, int nlev1, const Varray<double> &lev1, int nlev2, co
       if (i1 - 1 == 0)  // destination levels is not covert by the first two input z levels
         {
           lev_idx[i2] = BottomLevel;
-          lev_wgt[i2] = static_cast<float>(expol || is_equal(lev2[i2], val2));
+          lev_wgt[i2] = static_cast<float>(extrapolate || is_equal(lev2[i2], val2));
         }
       else if (i1 == nlev1 - 1)  // destination level is beyond the last value of the input z field
         {
           lev_idx[i2] = TopLevel;
-          lev_wgt[i2] = static_cast<float>(expol || is_equal(lev2[i2], val2));
+          lev_wgt[i2] = static_cast<float>(extrapolate || is_equal(lev2[i2], val2));
         }
       else  // target z values has two bounday values in input z field
         {
@@ -199,7 +187,7 @@ vert_gen_weights(int expol, int nlev1, const Varray<double> &lev1, int nlev2, co
 }
 
 bool
-levelDirUp(const int nlev, const double *const lev)
+levelDirUp(int nlev, const double *const lev)
 {
   auto lup = (nlev > 1 && lev[1] > lev[0]);
   for (int k = 1; k < nlev - 1; ++k)
@@ -209,7 +197,7 @@ levelDirUp(const int nlev, const double *const lev)
 }
 
 bool
-levelDirDown(const int nlev, const double *const lev)
+levelDirDown(int nlev, const double *const lev)
 {
   auto ldown = (nlev > 1 && lev[1] < lev[0]);
   for (int k = 1; k < nlev - 1; ++k)
@@ -220,10 +208,10 @@ levelDirDown(const int nlev, const double *const lev)
 
 template <typename T>
 static void
-vert_gen_weights3d1d(bool expol, size_t gridsize, int nlev1, const Varray<T> &xlev1, int nlev2, const std::vector<double> &lev2,
-                     Varray<int> &xlev_idx, Varray<float> &xlev_wgt)
+vert_gen_weights3d1d(bool extrapolate, size_t gridsize, int nlev1, Varray<T> const &xlev1, int nlev2,
+                     const std::vector<double> &lev2, Varray<int> &xlev_idx, Varray<float> &xlev_wgt)
 {
-  auto nthreads = Threading::ompNumThreads;
+  auto nthreads = Threading::ompNumMaxThreads;
   Varray2D<double> lev1p2(nthreads, Varray<double>(nlev1 + 2));
   Varray2D<float> lev_wgt(nthreads, Varray<float>(nlev2));
   Varray2D<int> lev_idx(nthreads, Varray<int>(nlev2));
@@ -247,7 +235,7 @@ vert_gen_weights3d1d(bool expol, size_t gridsize, int nlev1, const Varray<T> &xl
       lev1p2[ompthID][nlev1 + 1] = level_N;
       for (int k = 0; k < nlev1; ++k) lev1p2[ompthID][k + 1] = xlev1[k * gridsize + i];
 
-      vert_gen_weights(expol, nlev1 + 2, lev1p2[ompthID], nlev2, lev2, lev_idx[ompthID], lev_wgt[ompthID]);
+      vert_gen_weights(extrapolate, nlev1 + 2, lev1p2[ompthID], nlev2, lev2, lev_idx[ompthID], lev_wgt[ompthID]);
 
       for (int k = 0; k < nlev2; ++k) xlev_idx[k * gridsize + i] = lev_idx[ompthID][k];
       for (int k = 0; k < nlev2; ++k) xlev_wgt[k * gridsize + i] = lev_wgt[ompthID][k];
@@ -255,13 +243,11 @@ vert_gen_weights3d1d(bool expol, size_t gridsize, int nlev1, const Varray<T> &xl
 }
 
 static void
-vert_gen_weights3d1d(bool expol, size_t gridsize, int nlev1, Field3D &field1, int nlev2, const std::vector<double> &lev2,
+vert_gen_weights3d1d(bool extrapolate, size_t gridsize, int nlev1, Field3D &field1, int nlev2, const std::vector<double> &lev2,
                      Varray<int> &lev_idx, Varray<float> &lev_wgt)
 {
-  if (field1.memType == MemType::Float)
-    vert_gen_weights3d1d(expol, gridsize, nlev1, field1.vec_f, nlev2, lev2, lev_idx, lev_wgt);
-  else
-    vert_gen_weights3d1d(expol, gridsize, nlev1, field1.vec_d, nlev2, lev2, lev_idx, lev_wgt);
+  auto func = [&](auto &v) { vert_gen_weights3d1d(extrapolate, gridsize, nlev1, v, nlev2, lev2, lev_idx, lev_wgt); };
+  field_operation(func, field1);
 }
 
 static int
@@ -298,119 +284,128 @@ create_zaxis_from_zaxis(const std::vector<double> &levels, int zaxisID1)
   return zaxisID2;
 }
 
-static std::vector<double>
-intlevel_get_parameter(std::string &zdescription, std::string &zvarname)
+struct IntlevelParams
 {
   std::vector<double> levels;
+  std::string zdescription;
+  std::string zvarname;
+  bool extrapolate{ false };
+};
+
+static IntlevelParams
+get_parameter()
+{
+  IntlevelParams params;
 
   auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      const auto &pargv = cdo_get_oper_argv();
+      auto const &pargv = cdo_get_oper_argv();
 
       KVList kvlist;
       kvlist.name = cdo_module_name();
-      if (kvlist.parse_arguments(pargc, pargv) != 0) cdo_abort("Parse error!");
+      if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      auto def_level = false;
-      auto def_zdescription = false;
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           // if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-          const auto &values = kv.values;
-          const auto &value = kv.values[0];
-          int nvalues = kv.nvalues;
-          if (nvalues == 1 && value.empty()) nvalues = 0;
+          auto const &values = kv.values;
+          auto const &value = kv.values[0];
+          int numValues = kv.nvalues;
+          if (numValues == 1 && value.empty()) numValues = 0;
 
           // clang-format off
-         if (key == "level")
+          if (key == "level")
             {
-              if (def_zdescription) cdo_abort("Parameter level and zdescription can't be mixed!");
-              levels.resize(nvalues);
-              for (int i = 0; i < nvalues; ++i) levels[i] = parameter_to_double(values[i]);
-              def_level = true;
+              if (!params.zdescription.empty()) cdo_abort("Parameter level and zdescription can't be mixed!");
+              params.levels = cdo_argv_to_fltarr(values);
             }
-         else if (key == "zdescription")
+          else if (key == "zdescription")
             {
-              if (def_level) cdo_abort("Parameter file and level can't be mixed!");
-              zdescription = value;
-              def_zdescription = true;
+              if (!params.levels.empty()) cdo_abort("Parameter zdescription and level can't be mixed!");
+              params.zdescription = value;
             }
-         else if (key == "zvarname")
+          else if (key == "zvarname")
             {
-              zvarname = value;
+              params.zvarname = value;
             }
-         else cdo_abort("Invalid parameter key >%s<!", key);
+          else if (key == "extrapolate")
+            {
+              params.extrapolate = parameter_to_bool(value);
+            }
+          else cdo_abort("Invalid parameter key >%s<!", key);
           // clang-format on
         }
     }
 
-  return levels;
+  return params;
 }
 
 static void
-handle_zvar(size_t &zvarGridsize, size_t &wisize, int nvars, const std::string &zvarname, const VarList &varList1, int &zvarID,
-            bool &zvarIsVarying, int &nlevel, const std::vector<double> &lev2, int &nlev1, const int &nlev2, const int &vlistID1,
-            int &zaxisID2, int &zaxisID1)
+handle_zvar(size_t &zvarGridsize, size_t &wisize, std::string const &zvarname, const VarList &varList1, int &zvarID,
+            bool &zvarIsVarying, const std::vector<double> &lev2, int &nlev1, int const &nlev2, int const &vlistID1, int &zaxisID2,
+            int &zaxisID1)
 {
-  for (int varID = 0; varID < nvars; ++varID)
+  for (auto const &var : varList1.vars)
     {
-      if (zvarname == varList1[varID].name)
+      if (zvarname == var.name)
         {
-          zvarID = varID;
+          zvarID = var.ID;
           break;
         }
     }
 
+  auto const &zvar = varList1.vars[zvarID];
   if (zvarID == CDI_UNDEFID) cdo_abort("Variable %s not found!", zvarname);
-  zvarIsVarying = (varList1[zvarID].timetype == TIME_VARYING);
-  zvarGridsize = varList1[zvarID].gridsize;
-  nlev1 = varList1[zvarID].nlevels;
+  zvarIsVarying = (zvar.timeType == TIME_VARYING);
+  zvarGridsize = zvar.gridsize;
+  nlev1 = zvar.nlevels;
 
   if (zaxisID2 == CDI_UNDEFID) zaxisID2 = create_zaxis_from_zvar(lev2, vlistID1, zvarID);
 
   wisize = zvarGridsize * nlev2;
 
   int i;
-  auto nzaxis = vlistNzaxis(vlistID1);
-  for (i = 0; i < nzaxis; ++i)
+  auto numZaxes = vlistNumZaxis(vlistID1);
+  for (i = 0; i < numZaxes; ++i)
     {
       auto zaxisID = vlistZaxis(vlistID1, i);
-      nlevel = zaxisInqSize(zaxisID);
-      if (nlevel == nlev1)
+      auto nlevels = zaxisInqSize(zaxisID);
+      if (nlevels == nlev1)
         {
           zaxisID1 = zaxisID;
           break;
         }
     }
-  if (i == nzaxis) cdo_abort("No processable variable found!");
+  if (i == numZaxes) cdo_abort("No processable variable found!");
 }
 
 static void
-handle_empty_zvar(size_t &wisize, int &nlevel, std::vector<double> &lev1, const std::vector<double> &lev2, int &nlev1,
-                  const int &nlev2, const int &vlistID1, int &zaxisID2, int &zaxisID1)
+handle_empty_zvar(size_t &wisize, std::vector<double> &lev1, const std::vector<double> &lev2, int &nlev1, int const &nlev2,
+                  int const &vlistID1, int &zaxisID2, int &zaxisID1)
 {
+  int numLevels = 0;
   int i;
-  auto nzaxis = vlistNzaxis(vlistID1);
-  for (i = 0; i < nzaxis; ++i)
+  auto numZaxes = vlistNumZaxis(vlistID1);
+  for (i = 0; i < numZaxes; ++i)
     {
       auto zaxisID = vlistZaxis(vlistID1, i);
-      nlevel = zaxisInqSize(zaxisID);
+      numLevels = zaxisInqSize(zaxisID);
       // if (zaxisInqType(zaxisID) != ZAXIS_HYBRID && zaxisInqType(zaxisID) != ZAXIS_HYBRID_HALF)
-      if (nlevel > 1)
+      if (numLevels > 1)
         {
           zaxisID1 = zaxisID;
           break;
         }
     }
-  if (i == nzaxis) cdo_abort("No processable variable found!");
+  if (i == numZaxes) cdo_abort("No processable variable found!");
 
   if (zaxisID2 == CDI_UNDEFID) zaxisID2 = create_zaxis_from_zaxis(lev2, zaxisID1);
 
-  nlev1 = nlevel;
+  nlev1 = numLevels;
   lev1.resize(nlev1 + 2);
   cdo_zaxis_inq_levels(zaxisID1, &lev1[1]);
 
@@ -426,34 +421,43 @@ handle_empty_zvar(size_t &wisize, int &nlevel, std::vector<double> &lev1, const
   wisize = nlev2;
 }
 
-class ModuleIntlevel
+class Intlevel : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Intlevel",
+    // clang-format off
+    .operators = { { "intlevel", IntlevelHelp },
+                   { "intlevelx", IntlevelHelp } },
+    // clang-format on
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Intlevel> registration = RegisterEntry<Intlevel>(module);
+
   int zaxisID1 = -1;
-  int nlevel = 0;
 
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   std::vector<bool> processVars;
   std::vector<bool> interpVars;
-  std::vector<std::vector<size_t>> varnmiss;
+  std::vector<std::vector<size_t>> varnumMissVals;
   Field3DVector vardata1;
   Field3DVector vardata2;
 
   std::vector<double> lev2;
   int zaxisID2 = CDI_UNDEFID;
-  std::string zvarname;
 
   int nlev1 = 0;
   bool zvarIsVarying = false;
 
-  int nvars;
-
-  bool expol;
-
   Varray<double> lev1;
   int nlev2;
 
@@ -468,49 +472,37 @@ class ModuleIntlevel
   VarList varList1;
   VarList varList2;
 
-  int INTLEVEL, INTLEVELX;
+  IntlevelParams params;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-
-    cdo_initialize(process);
-
-    INTLEVEL = cdo_operator_add("intlevel", 0, 0, nullptr);
-    INTLEVELX = cdo_operator_add("intlevelx", 0, 0, nullptr);
-
-    (void) (INTLEVEL);  // unused
-
-    auto operatorID = cdo_operator_id();
-
-    expol = (operatorID == INTLEVELX);
-
-    operator_input_arg("level|zdescription, [zvarname]");
+    operator_input_arg("level|zdescription[, zvarname, extrapolate]");
 
     zaxisID2 = CDI_UNDEFID;
-    auto argv = cdo_get_oper_argv();
+    auto const &pargv = cdo_get_oper_argv();
 
-    if (std::isdigit((int) argv[0][0])) { lev2 = cdo_argv_to_flt(argv); }
+    if (std::isdigit((int) pargv[0][0])) { params.levels = cdo_argv_to_fltarr(pargv); }
     else
       {
-        std::string zdescription;
-        lev2 = intlevel_get_parameter(zdescription, zvarname);
-        if (!zdescription.empty())
+        params = get_parameter();
+        if (!params.zdescription.empty())
           {
-            auto zfilename = zdescription.c_str();
+            auto zfilename = params.zdescription.c_str();
             auto zfp = std::fopen(zfilename, "r");
             if (!zfp) cdo_abort("Open failed on %s", zfilename);
             zaxisID2 = zaxis_from_file(zfp, zfilename);
             std::fclose(zfp);
             if (zaxisID2 == CDI_UNDEFID) cdo_abort("Invalid zaxis description file %s!", zfilename);
             auto nlevels = zaxisInqSize(zaxisID2);
-            lev2.resize(nlevels);
-            zaxisInqLevels(zaxisID2, lev2.data());
+            params.levels.resize(nlevels);
+            zaxisInqLevels(zaxisID2, params.levels.data());
           }
       }
 
-     nlev2 = lev2.size();
+    lev2 = params.levels;
+    nlev2 = lev2.size();
 
     if (Options::cdoVerbose)
       for (int i = 0; i < nlev2; ++i) cdo_print("level out %d: %g", i, lev2[i]);
@@ -524,32 +516,32 @@ public:
     taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
 
-    varList_init(varList1, vlistID1);
-    varListSetUniqueMemtype(varList1);
-    memType = varList1[0].memType;
+    varList1 = VarList(vlistID1);
+    varList_set_unique_memtype(varList1);
+    memType = varList1.vars[0].memType;
 
-    nvars = vlistNvars(vlistID1);
+    auto numVars = varList1.numVars();
 
     // Find z-variable
     size_t wisize = 0;
 
-    if (zvarname.empty()) { handle_empty_zvar(wisize, nlevel, lev1, lev2, nlev1, nlev2, vlistID1, zaxisID2, zaxisID1); }
+    if (params.zvarname.empty()) { handle_empty_zvar(wisize, lev1, lev2, nlev1, nlev2, vlistID1, zaxisID2, zaxisID1); }
     else
       {
-        handle_zvar(zvarGridsize, wisize, nvars, zvarname, varList1, zvarID, zvarIsVarying, nlevel, lev2, nlev1, nlev2, vlistID1,
-                    zaxisID2, zaxisID1);
+        handle_zvar(zvarGridsize, wisize, params.zvarname, varList1, zvarID, zvarIsVarying, lev2, nlev1, nlev2, vlistID1, zaxisID2,
+                    zaxisID1);
       }
 
-    auto nzaxis = vlistNzaxis(vlistID1);
-    for (int index = 0; index < nzaxis; ++index)
+    auto numZaxes = vlistNumZaxis(vlistID1);
+    for (int index = 0; index < numZaxes; ++index)
       {
         auto zaxisID = vlistZaxis(vlistID1, index);
-        auto nlevels = zaxisInqSize(zaxisID);
-        if (zaxisID == zaxisID1 || nlevels == nlev1) vlistChangeZaxisIndex(vlistID2, index, zaxisID2);
+        auto numLevels = zaxisInqSize(zaxisID);
+        if (zaxisID == zaxisID1 || numLevels == nlev1) vlistChangeZaxisIndex(vlistID2, index, zaxisID2);
       }
 
-    varList_init(varList2, vlistID2);
-    varListSetMemtype(varList2, memType);
+    varList2 = VarList(vlistID2);
+    varList_set_memtype(varList2, memType);
 
     lev_idx = Varray<int>(wisize);
     lev_wgt = Varray<float>(wisize);
@@ -557,68 +549,72 @@ public:
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
 
-    processVars = std::vector<bool>(nvars);
-    interpVars = std::vector<bool>(nvars, false);
-    varnmiss = std::vector<std::vector<size_t>>(nvars);
-    vardata1 = Field3DVector(nvars);
-    vardata2 = Field3DVector(nvars);
+    processVars = std::vector<bool>(numVars);
+    interpVars = std::vector<bool>(numVars, false);
+    varnumMissVals = std::vector<std::vector<size_t>>(numVars);
+    vardata1 = Field3DVector(numVars);
+    vardata2 = Field3DVector(numVars);
 
-    const int maxlev = (nlev1 > nlev2) ? nlev1 : nlev2;
+    int maxlev = std::max(nlev1, nlev2);
 
-    for (int varID = 0; varID < nvars; ++varID)
+    for (int varID = 0; varID < numVars; ++varID)
       {
-        auto nlevels = varList1[varID].nlevels;
+        auto const &var1 = varList1.vars[varID];
 
-        vardata1[varID].init(varList1[varID]);
+        vardata1[varID].init(var1);
 
-        interpVars[varID] = (varList1[varID].zaxisID == zaxisID1 || nlevels == nlev1);
+        interpVars[varID] = (var1.zaxisID == zaxisID1 || var1.nlevels == nlev1);
 
         if (interpVars[varID])
           {
-            varnmiss[varID].resize(maxlev, 0);
-            vardata2[varID].init(varList2[varID]);
+            varnumMissVals[varID].resize(maxlev, 0);
+            vardata2[varID].init(varList2.vars[varID]);
           }
-        else { varnmiss[varID].resize(nlevels); }
+        else { varnumMissVals[varID].resize(var1.nlevels); }
       }
   }
+
   void
-  run()
+  run() override
   {
+    auto numVars = varList1.numVars();
+
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
-        for (int varID = 0; varID < nvars; ++varID) processVars[varID] = false;
+        for (int varID = 0; varID < numVars; ++varID) processVars[varID] = false;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            cdo_read_record(streamID1, vardata1[varID], levelID, &varnmiss[varID][levelID]);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            cdo_read_field(streamID1, vardata1[varID], levelID, &varnumMissVals[varID][levelID]);
             processVars[varID] = true;
           }
 
         if (tsID == 0 || zvarIsVarying)
           {
-            if (!zvarname.empty())
-              vert_gen_weights3d1d(expol, zvarGridsize, nlev1, vardata1[zvarID], nlev2, lev2, lev_idx, lev_wgt);
+            if (!params.zvarname.empty())
+              vert_gen_weights3d1d(params.extrapolate, zvarGridsize, nlev1, vardata1[zvarID], nlev2, lev2, lev_idx, lev_wgt);
             else
-              vert_gen_weights(expol, nlev1 + 2, lev1, nlev2, lev2, lev_idx, lev_wgt);
+              vert_gen_weights(params.extrapolate, nlev1 + 2, lev1, nlev2, lev2, lev_idx, lev_wgt);
           }
 
-        for (int varID = 0; varID < nvars; ++varID)
+        for (int varID = 0; varID < numVars; ++varID)
           {
             if (processVars[varID] && interpVars[varID])
               {
-                auto missval = varList1[varID].missval;
-                auto gridsize = varList1[varID].gridsize;
+                auto const &var1 = varList1.vars[varID];
 
-                if (!zvarname.empty())
+                auto missval = var1.missval;
+                auto gridsize = var1.gridsize;
+
+                if (!params.zvarname.empty())
                   vert_interp_lev3d(gridsize, nlev1, missval, vardata1[varID], vardata2[varID], nlev2, lev_idx, lev_wgt);
                 else
                   vert_interp_lev(gridsize, nlev1, missval, vardata1[varID], vardata2[varID], nlev2, lev_idx, lev_wgt);
@@ -626,23 +622,22 @@ public:
                 for (int levelID = 0; levelID < nlev2; ++levelID)
                   {
                     auto offset = gridsize * levelID;
-                    if (memType == MemType::Float)
-                      varnmiss[varID][levelID] = array_num_mv(gridsize, &vardata2[varID].vec_f[offset], (float) missval);
-                    else
-                      varnmiss[varID][levelID] = array_num_mv(gridsize, &vardata2[varID].vec_d[offset], missval);
+                    auto func
+                        = [&](auto const &v) { varnumMissVals[varID][levelID] = array_num_mv(gridsize, &v[offset], missval); };
+                    field_operation(func, vardata2[varID]);
                   }
               }
           }
 
-        for (int varID = 0; varID < nvars; ++varID)
+        for (int varID = 0; varID < numVars; ++varID)
           {
             if (processVars[varID])
               {
-                for (int levelID = 0; levelID < varList2[varID].nlevels; ++levelID)
+                for (int levelID = 0; levelID < varList2.vars[varID].nlevels; ++levelID)
                   {
-                    cdo_def_record(streamID2, varID, levelID);
-                    cdo_write_record(streamID2, interpVars[varID] ? vardata2[varID] : vardata1[varID], levelID,
-                                     varnmiss[varID][levelID]);
+                    cdo_def_field(streamID2, varID, levelID);
+                    cdo_write_field(streamID2, interpVars[varID] ? vardata2[varID] : vardata1[varID], levelID,
+                                    varnumMissVals[varID][levelID]);
                   }
               }
           }
@@ -650,24 +645,11 @@ public:
         tsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Intlevel(void *process)
-{
-  ModuleIntlevel intlevel;
-
-  intlevel.init(process);
-  intlevel.run();
-  intlevel.close();
-
-  return nullptr;
-}
