@@ -20,7 +20,7 @@
 #include "printinfo.h"
 #include "field_functions.h"
 
-void interp_time(double fac1, double fac2, const Field &field1, const Field &field2, Field &field3, bool withMissval);
+void interp_time(double fac1, double fac2, Field const &field1, Field const &field2, Field &field3, bool withMissval);
 
 class Intntime : public Process
 {
@@ -41,8 +41,8 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   int numts;
   VarList varList1;
@@ -82,43 +82,43 @@ public:
     field2D_init(varsData[0], varList1, FIELD_VEC | FIELD_NAT);
     field2D_init(varsData[1], varList1, FIELD_VEC | FIELD_NAT);
 
-    auto maxRecords = varList1.numRecords();
-    std::vector<RecordInfo> recordList(maxRecords);
+    auto maxFields = varList1.maxFields();
+    std::vector<FieldInfo> fieldInfoList(maxFields);
 
     auto calendar = taxisInqCalendar(taxisID1);
 
     int tsID = 0;
     int tsIDo = 0;
 
-    auto nrecs = cdo_stream_inq_timestep(streamID1, tsID++);
+    auto numFields = cdo_stream_inq_timestep(streamID1, tsID++);
     auto julianDate1 = julianDate_encode(calendar, taxisInqVdatetime(taxisID1));
 
     cdo_taxis_copy_timestep(taxisID2, taxisID1);
     cdo_def_timestep(streamID2, tsIDo++);
-    for (int recID = 0; recID < nrecs; ++recID)
+    for (int fieldID = 0; fieldID < numFields; ++fieldID)
       {
-        auto [varID, levelID] = cdo_inq_record(streamID1);
+        auto [varID, levelID] = cdo_inq_field(streamID1);
         auto &field = varsData[curFirst][varID][levelID];
-        cdo_read_record(streamID1, field);
+        cdo_read_field(streamID1, field);
 
-        cdo_def_record(streamID2, varID, levelID);
-        cdo_write_record(streamID2, field);
+        cdo_def_field(streamID2, varID, levelID);
+        cdo_write_field(streamID2, field);
       }
 
     while (true)
       {
-        nrecs = cdo_stream_inq_timestep(streamID1, tsID++);
-        if (nrecs == 0) break;
+        numFields = cdo_stream_inq_timestep(streamID1, tsID++);
+        if (numFields == 0) break;
 
         auto vDateTime2 = taxisInqVdatetime(taxisID1);
         auto julianDate2 = julianDate_encode(calendar, vDateTime2);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
-            recordList[recID].set(varID, levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            fieldInfoList[fieldID].set(varID, levelID);
             auto &field = varsData[curSecond][varID][levelID];
-            cdo_read_record(streamID1, field);
+            cdo_read_field(streamID1, field);
           }
 
         for (int it = 1; it < numts; it++)
@@ -136,31 +136,31 @@ public:
             auto fac1 = julianDate_to_seconds(julianDate_sub(julianDate2, julianDate)) / diff;
             auto fac2 = julianDate_to_seconds(julianDate_sub(julianDate, julianDate1)) / diff;
 
-            for (int recID = 0; recID < nrecs; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                auto [varID, levelID] = recordList[recID].get();
+                auto [varID, levelID] = fieldInfoList[fieldID].get();
 
-                const auto &field1 = varsData[curFirst][varID][levelID];
-                const auto &field2 = varsData[curSecond][varID][levelID];
+                auto const &field1 = varsData[curFirst][varID][levelID];
+                auto const &field2 = varsData[curSecond][varID][levelID];
 
                 field3.init(varList1.vars[varID]);
 
                 auto withMissval = (field1.numMissVals || field2.numMissVals);
                 interp_time(fac1, fac2, field1, field2, field3, withMissval);
 
-                cdo_def_record(streamID2, varID, levelID);
-                cdo_write_record(streamID2, field3);
+                cdo_def_field(streamID2, varID, levelID);
+                cdo_write_field(streamID2, field3);
               }
           }
 
         taxisDefVdatetime(taxisID2, vDateTime2);
         cdo_def_timestep(streamID2, tsIDo++);
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = recordList[recID].get();
+            auto [varID, levelID] = fieldInfoList[fieldID].get();
             auto &field = varsData[curSecond][varID][levelID];
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, field);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, field);
           }
 
         julianDate1 = julianDate2;
