@@ -17,6 +17,7 @@
 #include <utility>
 
 #include "cdo_options.h"
+#include "cdo_omp.h"
 #include "process_int.h"
 #include "datetime.h"
 #include "printinfo.h"
@@ -24,14 +25,14 @@
 
 template <typename T>
 size_t
-interp_time(double fac1, double fac2, size_t gridsize, const Varray<T> &v1, const Varray<T> &v2, Varray<T> &v3, bool withMissval,
+interp_time(double fac1, double fac2, size_t n, Varray<T> const &v1, Varray<T> const &v2, Varray<T> &v3, bool withMissval,
             T missval)
 {
   size_t numMissVals3 = 0;
 
   if (withMissval)
     {
-      for (size_t i = 0; i < gridsize; ++i)
+      for (size_t i = 0; i < n; ++i)
         {
           auto v1IsMissval = fp_is_equal(v1[i], missval);
           auto v2IsMissval = fp_is_equal(v2[i], missval);
@@ -51,17 +52,18 @@ interp_time(double fac1, double fac2, size_t gridsize, const Varray<T> &v1, cons
   else
     {
 #ifdef _OPENMP
-#pragma omp parallel for default(shared)
+#pragma omp parallel for if (n > cdoMinLoopSize) default(shared)
 #endif
-      for (size_t i = 0; i < gridsize; ++i) v3[i] = v1[i] * fac1 + v2[i] * fac2;
+      for (size_t i = 0; i < n; ++i) { v3[i] = v1[i] * fac1 + v2[i] * fac2; }
     }
 
   return numMissVals3;
 }
 
 void
-interp_time(double fac1, double fac2, const Field &field1, const Field &field2, Field &field3, bool withMissval)
+interp_time(double fac1, double fac2, Field const &field1, Field const &field2, Field &field3, bool withMissval)
 {
+  if (field1.memType != field3.memType) cdo_abort("Interal error, memType of field1 and field3 differ!");
   if (field3.memType == MemType::Float)
     field3.numMissVals
         = interp_time(fac1, fac2, field3.gridsize, field1.vec_f, field2.vec_f, field3.vec_f, withMissval, (float) field3.missval);
@@ -119,11 +121,11 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2 = CDO_STREAM_UNDEF;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  int vlistID1;
-  int vlistID2;
+  int vlistID1{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
   int curFirst = 0, curSecond = 1;
 
@@ -168,8 +170,8 @@ public:
     field2D_init(varsData[0], varList1, FIELD_VEC | FIELD_NAT);
     field2D_init(varsData[1], varList1, FIELD_VEC | FIELD_NAT);
 
-    auto maxRecords = varList1.numRecords();
-    std::vector<RecordInfo> recordList(maxRecords);
+    auto maxFields = varList1.maxFields();
+    std::vector<FieldInfo> fieldInfoList(maxFields);
 
     auto calendar = taxisInqCalendar(taxisID1);
     auto julianDate = julianDate_encode(calendar, sDateTime);
@@ -184,14 +186,14 @@ public:
     int tsID = 0;
     int tsIDo = 0;
 
-    auto nrecs = cdo_stream_inq_timestep(streamID1, tsID++);
+    auto numFields = cdo_stream_inq_timestep(streamID1, tsID++);
     auto vDateTime1 = taxisInqVdatetime(taxisID1);
     auto julianDate1 = julianDate_encode(calendar, vDateTime1);
-    for (int recID = 0; recID < nrecs; ++recID)
+    for (int fieldID = 0; fieldID < numFields; ++fieldID)
       {
-        auto [varID, levelID] = cdo_inq_record(streamID1);
+        auto [varID, levelID] = cdo_inq_field(streamID1);
         auto &field = varsData[curFirst][varID][levelID];
-        cdo_read_record(streamID1, field);
+        cdo_read_field(streamID1, field);
       }
 
     if (Options::cdoVerbose)
@@ -208,8 +210,8 @@ public:
 
     while (julianDate_to_seconds(julianDate1) <= julianDate_to_seconds(julianDate))
       {
-        nrecs = cdo_stream_inq_timestep(streamID1, tsID++);
-        if (nrecs == 0) break;
+        numFields = cdo_stream_inq_timestep(streamID1, tsID++);
+        if (numFields == 0) break;
 
         auto vDateTime = taxisInqVdatetime(taxisID1);
         auto julianDate2 = julianDate_encode(calendar, vDateTime);
@@ -219,12 +221,12 @@ public:
             cdo_print("julianDate2 = %f", julianDate_to_seconds(julianDate2));
           }
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
-            recordList[recID].set(varID, levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            fieldInfoList[fieldID].set(varID, levelID);
             auto &field = varsData[curSecond][varID][levelID];
-            cdo_read_record(streamID1, field);
+            cdo_read_field(streamID1, field);
           }
 
         while (julianDate_to_seconds(julianDate) <= julianDate_to_seconds(julianDate2))
@@ -251,20 +253,20 @@ public:
                 auto fac1 = julianDate_to_seconds(julianDate_sub(julianDate2, julianDate)) / diff;
                 auto fac2 = julianDate_to_seconds(julianDate_sub(julianDate, julianDate1)) / diff;
 
-                for (int recID = 0; recID < nrecs; ++recID)
+                for (int fieldID = 0; fieldID < numFields; ++fieldID)
                   {
-                    auto [varID, levelID] = recordList[recID].get();
+                    auto [varID, levelID] = fieldInfoList[fieldID].get();
 
-                    const auto &field1 = varsData[curFirst][varID][levelID];
-                    const auto &field2 = varsData[curSecond][varID][levelID];
+                    auto const &field1 = varsData[curFirst][varID][levelID];
+                    auto const &field2 = varsData[curSecond][varID][levelID];
 
                     field3.init(varList1.vars[varID]);
 
                     auto withMissval = (field1.numMissVals || field2.numMissVals);
                     interp_time(fac1, fac2, field1, field2, field3, withMissval);
 
-                    cdo_def_record(streamID2, varID, levelID);
-                    cdo_write_record(streamID2, field3);
+                    cdo_def_field(streamID2, varID, levelID);
+                    cdo_write_field(streamID2, field3);
                   }
               }
 
