@@ -20,7 +20,7 @@
 #include "util_string.h"
 
 static size_t
-intlin_year(double fac1, double fac2, size_t gridsize, const Varray<double> &array1, const Varray<double> &array2,
+intlin_year(double fac1, double fac2, size_t gridsize, Varray<double> const &array1, Varray<double> const &array2,
             Varray<double> &array3, bool withMissval, double missval1, double missval2)
 {
   size_t numMissVals3 = 0;
@@ -64,22 +64,18 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
   int taxisID3;
 
   std::vector<int> iyears;
-  int nyears;
+  int numYears;
 
   VarList varList1;
   VarList varList2;
 
   std::vector<CdoStreamID> streamIDs;
 
-  Varray<double> array1;
-  Varray<double> array2;
-  Varray<double> array3;
-
 public:
   void
   init() override
@@ -88,11 +84,11 @@ public:
     operator_input_arg("years");
 
     iyears = cdo_argv_to_intarr(cdo_get_oper_argv());
-    nyears = iyears.size();
+    numYears = iyears.size();
 
-    cdo::set_numfiles(nyears + 8);
+    cdo::set_numfiles(numYears + 8);
 
-    streamIDs = std::vector<CdoStreamID>(nyears);
+    streamIDs.resize(numYears);
 
     streamID1 = cdo_open_read(0);
     streamID2 = cdo_open_read(1);
@@ -105,12 +101,6 @@ public:
     varList2 = VarList(vlistID2);
     varList_compare(varList1, varList2);
 
-    auto gridsizemax = vlistGridsizeMax(vlistID1);
-
-    array1 = Varray<double>(gridsizemax);
-    array2 = Varray<double>(gridsizemax);
-    array3 = Varray<double>(gridsizemax);
-
     taxisID1 = vlistInqTaxis(vlistID1);
     taxisID2 = vlistInqTaxis(vlistID2);
     taxisID3 = taxisDuplicate(taxisID1);
@@ -119,12 +109,12 @@ public:
 
     auto fileSuffix = FileUtils::gen_suffix(cdo_inq_filetype(streamID1), vlistID1, cdo_get_stream_name(0));
 
-    for (int iy = 0; iy < nyears; iy++)
+    for (int iy = 0; iy < numYears; iy++)
       {
         auto fileName = cdo_get_obase() + string_format("%04d", iyears[iy]);
         if (fileSuffix.size() > 0) fileName += fileSuffix;
 
-        streamIDs[iy] = cdo_open_write(fileName.c_str());
+        streamIDs[iy] = open_write(fileName);
         cdo_def_vlist(streamIDs[iy], vlistID3);
       }
   }
@@ -132,20 +122,24 @@ public:
   void
   run() override
   {
+    Varray<double> array1(varList1.gridsizeMax());
+    Varray<double> array2(varList1.gridsizeMax());
+    Varray<double> array3(varList1.gridsizeMax());
+
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
-        nrecs = cdo_stream_inq_timestep(streamID2, tsID);
-        if (nrecs == 0) cdo_abort("Too few timesteps in second inputfile!");
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
+        numFields = cdo_stream_inq_timestep(streamID2, tsID);
+        if (numFields == 0) cdo_abort("Too few timesteps in second inputfile!");
 
         auto vDateTime1 = taxisInqVdatetime(taxisID1);
         auto vDateTime2 = taxisInqVdatetime(taxisID2);
         auto year1 = vDateTime1.date.year;
         auto year2 = vDateTime2.date.year;
 
-        for (int iy = 0; iy < nyears; iy++)
+        for (int iy = 0; iy < numYears; iy++)
           {
             if (iyears[iy] < year1 || iyears[iy] > year2)
               cdo_abort("Year %d out of bounds (first year %d; last year %d)!", iyears[iy], year1, year2);
@@ -156,30 +150,29 @@ public:
             cdo_def_timestep(streamIDs[iy], tsID);
           }
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            cdo_inq_record(streamID2, &varID, &levelID);
+            (void) cdo_inq_field(streamID1);
+            auto [varID, levelID] = cdo_inq_field(streamID2);
 
             size_t numMissVals1, numMissVals2;
-            cdo_read_record(streamID1, array1.data(), &numMissVals1);
-            cdo_read_record(streamID2, array2.data(), &numMissVals2);
+            cdo_read_field(streamID1, array1.data(), &numMissVals1);
+            cdo_read_field(streamID2, array2.data(), &numMissVals2);
 
             auto gridsize = varList1.vars[varID].gridsize;
             auto missval1 = varList1.vars[varID].missval;
             auto missval2 = varList2.vars[varID].missval;
             auto withMissval = (numMissVals1 || numMissVals2);
 
-            for (int iy = 0; iy < nyears; iy++)
+            for (int iy = 0; iy < numYears; iy++)
               {
                 auto fac1 = ((double) year2 - iyears[iy]) / (year2 - year1);
                 auto fac2 = ((double) iyears[iy] - year1) / (year2 - year1);
 
                 auto numMissVals3 = intlin_year(fac1, fac2, gridsize, array1, array2, array3, withMissval, missval1, missval2);
 
-                cdo_def_record(streamIDs[iy], varID, levelID);
-                cdo_write_record(streamIDs[iy], array3.data(), numMissVals3);
+                cdo_def_field(streamIDs[iy], varID, levelID);
+                cdo_write_field(streamIDs[iy], array3.data(), numMissVals3);
               }
           }
 
@@ -190,7 +183,7 @@ public:
   void
   close() override
   {
-    for (int iy = 0; iy < nyears; iy++) cdo_stream_close(streamIDs[iy]);
+    for (auto &streamID : streamIDs) cdo_stream_close(streamID);
 
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
