@@ -44,13 +44,10 @@ invert_lon_data(Varray<T> &v, size_t nlon, size_t nlat)
 static void
 invert_lon_data(Field &field)
 {
-  const auto nlon = gridInqXsize(field.grid);
-  const auto nlat = gridInqYsize(field.grid);
-
-  if (field.memType == MemType::Float)
-    invert_lon_data(field.vec_f, nlon, nlat);
-  else
-    invert_lon_data(field.vec_d, nlon, nlat);
+  auto nlon = gridInqXsize(field.grid);
+  auto nlat = gridInqYsize(field.grid);
+  auto func = [&](auto &v) { invert_lon_data(v, nlon, nlat); };
+  field_operation(func, field);
 }
 
 template <typename T>
@@ -74,25 +71,22 @@ invert_lat_data(Varray<T> &v, size_t nlon, size_t nlat)
 static void
 invert_lat_data(Field &field)
 {
-  const auto nlon = gridInqXsize(field.grid);
-  const auto nlat = gridInqYsize(field.grid);
-
-  if (field.memType == MemType::Float)
-    invert_lat_data(field.vec_f, nlon, nlat);
-  else
-    invert_lat_data(field.vec_d, nlon, nlat);
+  auto nlon = gridInqXsize(field.grid);
+  auto nlat = gridInqYsize(field.grid);
+  auto func = [&](auto &v) { invert_lat_data(v, nlon, nlat); };
+  field_operation(func, field);
 }
 
 static void
 invert_lon_des(int vlistID)
 {
-  const auto ngrids = vlistNgrids(vlistID);
-  for (int index = 0; index < ngrids; ++index)
+  auto numGrids = vlistNumGrids(vlistID);
+  for (int index = 0; index < numGrids; ++index)
     {
-      const auto gridID1 = vlistGrid(vlistID, index);
-      const auto gridID2 = gridDuplicate(gridID1);
+      auto gridID1 = vlistGrid(vlistID, index);
+      auto gridID2 = gridDuplicate(gridID1);
 
-      const auto gridtype = gridInqType(gridID1);
+      auto gridtype = gridInqType(gridID1);
 
       if (!(gridtype == GRID_GENERIC || gridtype == GRID_GAUSSIAN || gridtype == GRID_PROJECTION || gridtype == GRID_LONLAT
             || gridtype == GRID_CURVILINEAR))
@@ -100,9 +94,9 @@ invert_lon_des(int vlistID)
 
       if (gridInqXvals(gridID1, nullptr))
         {
-          const auto nlon = gridInqXsize(gridID1);
-          const auto nlat = gridInqYsize(gridID1);
-          const auto size = (gridtype == GRID_CURVILINEAR) ? nlon * nlat : nlon;
+          auto nlon = gridInqXsize(gridID1);
+          auto nlat = gridInqYsize(gridID1);
+          auto size = (gridtype == GRID_CURVILINEAR) ? nlon * nlat : nlon;
 
           Varray<double> coords(size);
 
@@ -129,10 +123,10 @@ invert_lon_des(int vlistID)
 
       if (gridInqXbounds(gridID1, nullptr))
         {
-          const auto nlon = gridInqXsize(gridID1);
-          const auto nlat = gridInqYsize(gridID1);
-          const auto nv = gridInqNvertex(gridID1);
-          const auto size = (gridtype == GRID_CURVILINEAR) ? nv * nlon * nlat : nv * nlon;
+          auto nlon = gridInqXsize(gridID1);
+          auto nlat = gridInqYsize(gridID1);
+          auto nv = gridInqNvertex(gridID1);
+          auto size = (gridtype == GRID_CURVILINEAR) ? nv * nlon * nlat : nv * nlon;
 
           Varray<double> bounds(size);
 
@@ -168,13 +162,13 @@ invert_lon_des(int vlistID)
 static void
 invert_lat_coord(int gridID)
 {
-  const auto gridtype = gridInqType(gridID);
+  auto gridtype = gridInqType(gridID);
 
   if (gridInqYvals(gridID, nullptr))
     {
-      const auto nlon = gridInqXsize(gridID);
-      const auto nlat = gridInqYsize(gridID);
-      const auto size = (gridtype == GRID_CURVILINEAR) ? nlon * nlat : nlat;
+      auto nlon = gridInqXsize(gridID);
+      auto nlat = gridInqYsize(gridID);
+      auto size = (gridtype == GRID_CURVILINEAR) ? nlon * nlat : nlat;
 
       Varray<double> coords(size);
 
@@ -201,10 +195,10 @@ invert_lat_coord(int gridID)
 
   if (gridInqYbounds(gridID, nullptr))
     {
-      const auto nlon = gridInqXsize(gridID);
-      const auto nlat = gridInqYsize(gridID);
-      const auto nv = gridInqNvertex(gridID);
-      const auto size = (gridtype == GRID_CURVILINEAR) ? nv * nlon * nlat : nv * nlat;
+      auto nlon = gridInqXsize(gridID);
+      auto nlat = gridInqYsize(gridID);
+      auto nv = gridInqNvertex(gridID);
+      auto size = (gridtype == GRID_CURVILINEAR) ? nv * nlon * nlat : nv * nlat;
 
       Varray<double> bounds(size);
 
@@ -237,13 +231,13 @@ invert_lat_coord(int gridID)
 static void
 invert_lat_des(int vlistID)
 {
-  const auto ngrids = vlistNgrids(vlistID);
-  for (int index = 0; index < ngrids; ++index)
+  auto numGrids = vlistNumGrids(vlistID);
+  for (int index = 0; index < numGrids; ++index)
     {
-      const auto gridID1 = vlistGrid(vlistID, index);
-      const auto gridID2 = gridDuplicate(gridID1);
+      auto gridID1 = vlistGrid(vlistID, index);
+      auto gridID2 = gridDuplicate(gridID1);
 
-      const auto gridtype = gridInqType(gridID1);
+      auto gridtype = gridInqType(gridID1);
 
       if (!(gridtype == GRID_GENERIC || gridtype == GRID_GAUSSIAN || gridtype == GRID_PROJECTION || gridtype == GRID_LONLAT
             || gridtype == GRID_CURVILINEAR))
@@ -251,14 +245,14 @@ invert_lat_des(int vlistID)
 
       invert_lat_coord(gridID2);
 
-      const auto projID = gridInqProj(gridID2);
+      auto projID = gridInqProj(gridID2);
       if (projID != CDI_UNDEFID) invert_lat_coord(projID);
 
       vlistChangeGrid(vlistID, gridID1, gridID2);
     }
 }
 
-class ModuleInvert
+class Invert : public Process
 {
   enum
   {
@@ -269,34 +263,39 @@ class ModuleInvert
     func_lat
   };
 
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Invert",
+    .operators = { { "invertlat", func_all, func_lat, InvertHelp },
+                   { "invertlon", func_all, func_lon, InvertHelp },
+                   { "invertlatdes", func_hrd, func_lat, InvertHelp },
+                   { "invertlondes", func_hrd, func_lon, InvertHelp },
+                   { "invertlatdata", func_fld, func_lat, InvertHelp },
+                   { "invertlondata", func_fld, func_lon, InvertHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Invert> registration = RegisterEntry<Invert>(module);
+
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   int operfunc1;
   int operfunc2;
 
   VarList varList1;
-  Field field;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    // clang-format off
-  cdo_operator_add("invertlat",     func_all, func_lat, nullptr);
-  cdo_operator_add("invertlon",     func_all, func_lon, nullptr);
-  cdo_operator_add("invertlatdes",  func_hrd, func_lat, nullptr);
-  cdo_operator_add("invertlondes",  func_hrd, func_lon, nullptr);
-  cdo_operator_add("invertlatdata", func_fld, func_lat, nullptr);
-  cdo_operator_add("invertlondata", func_fld, func_lon, nullptr);
-    // clang-format on
-
-    const auto operatorID = cdo_operator_id();
+    auto operatorID = cdo_operator_id();
     operfunc1 = cdo_operator_f1(operatorID);
     operfunc2 = cdo_operator_f2(operatorID);
 
@@ -304,8 +303,8 @@ public:
 
     streamID1 = cdo_open_read(0);
 
-    const auto vlistID1 = cdo_stream_inq_vlist(streamID1);
-    const auto vlistID2 = vlistDuplicate(vlistID1);
+    auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+    auto vlistID2 = vlistDuplicate(vlistID1);
 
     taxisID1 = vlistInqTaxis(vlistID1);
     taxisID2 = taxisDuplicate(taxisID1);
@@ -323,29 +322,31 @@ public:
 
     cdo_def_vlist(streamID2, vlistID2);
 
-    varList_init(varList1, vlistID1);
+    varList1 = VarList(vlistID1);
   }
+
   void
-  run()
+  run() override
   {
+    Field field;
+
     int tsID = 0;
     while (true)
       {
-        const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
 
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            field.init(varList1[varID]);
-            cdo_read_record(streamID1, field);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            field.init(varList1.vars[varID]);
+            cdo_read_field(streamID1, field);
 
-            cdo_def_record(streamID2, varID, levelID);
+            cdo_def_field(streamID2, varID, levelID);
 
             if (operfunc1 == func_all || operfunc1 == func_fld)
               {
@@ -354,9 +355,9 @@ public:
                 else
                   invert_lon_data(field);
 
-                cdo_write_record(streamID2, field);
+                cdo_write_field(streamID2, field);
               }
-            else { cdo_write_record(streamID2, field); }
+            else { cdo_write_field(streamID2, field); }
           }
 
         tsID++;
@@ -364,22 +365,9 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-void *
-Invert(void *process)
-{
-  ModuleInvert invert;
-
-  invert.init(process);
-  invert.run();
-  invert.close();
-
-  return nullptr;
-}
