@@ -128,7 +128,7 @@ compareTime(CdiTime time1, CdiTime time2)
 }
 
 static void
-maggraph(const char *plotfile, const std::string &varname, const std::string &varunits, long nfiles, std::vector<long> nts,
+maggraph(const char *plotfile, std::string const &varname, std::string const &varunits, long nfiles, std::vector<long> nts,
          const std::vector<std::vector<CdiDateTime>> &vDateTimes, std::vector<std::vector<double>> datatab, int nparam,
          std::vector<std::string> &params)
 {
@@ -156,9 +156,9 @@ maggraph(const char *plotfile, const std::string &varname, const std::string &va
   std::string temp_str;
   for (i = 0; i < nparam; ++i)
     {
-      auto splitStrings = split_with_seperator(params[i], '=');
-      const auto &key = splitStrings[0];
-      const auto &value = splitStrings[1];
+      auto splitStrings = split_string(params[i], "=");
+      auto const &key = splitStrings[0];
+      auto const &value = splitStrings[1];
 
       if (key == "obsv")
         {
@@ -269,7 +269,7 @@ maggraph(const char *plotfile, const std::string &varname, const std::string &va
         }
     }
 
-  char ***date_time_str = (char ***) malloc(nfiles * sizeof(char **));
+  char ***date_time_str = (char ***) std::malloc(nfiles * sizeof(char **));
 
   std::vector<double> date_time;
   std::vector<double> mean_val, std_dev_val;
@@ -278,7 +278,7 @@ maggraph(const char *plotfile, const std::string &varname, const std::string &va
   if (stat)
     {
       // if all files are of same number of steps, only one date_time_str array is being used
-      date_time_str[0] = (char **) malloc(ntime_steps * sizeof(char *));
+      date_time_str[0] = (char **) std::malloc(ntime_steps * sizeof(char *));
 
       date_time.resize(ntime_steps);
       mean_val.resize(ntime_steps);
@@ -289,7 +289,7 @@ maggraph(const char *plotfile, const std::string &varname, const std::string &va
       for (tsID = 0; tsID < ntime_steps; ++tsID)
         {
           date_time[tsID] = tsID + 1;
-          date_time_str[0][tsID] = (char *) malloc(256);
+          date_time_str[0][tsID] = (char *) std::malloc(256);
           std::snprintf(date_time_str[0][tsID], 256, "%s", datetime_to_string(vDateTimes[0][tsID]).c_str());
           mean_val[tsID] = 0.;
           std_dev_val[tsID] = 0.;
@@ -366,13 +366,13 @@ maggraph(const char *plotfile, const std::string &varname, const std::string &va
         {
           if (DBG) fprintf(stderr, "FILE  %ld\n", fileID);
           date_time.resize(nts[fileID]);
-          date_time_str[fileID] = (char **) malloc(nts[fileID] * sizeof(char *));
+          date_time_str[fileID] = (char **) std::malloc(nts[fileID] * sizeof(char *));
 
           for (tsID = 0; tsID < nts[fileID]; ++tsID)
             {
               date_time[tsID] = tsID + 1;
 
-              date_time_str[fileID][tsID] = (char *) malloc(256);
+              date_time_str[fileID][tsID] = (char *) std::malloc(256);
               std::snprintf(date_time_str[fileID][tsID], 256, "%s", datetime_to_string(vDateTimes[fileID][tsID]).c_str());
               if (DBG && (tsID == 0 || tsID == nts[fileID] - 1)) fprintf(stderr, "%s\n", date_time_str[fileID][tsID]);
 
@@ -426,13 +426,13 @@ maggraph(const char *plotfile, const std::string &varname, const std::string &va
 
   if (DBG) fprintf(stderr, "%s %s\n", min_date_time_str, max_date_time_str);
 
-  auto splitStrings = split_with_seperator(max_date_time_str, '-');
+  auto splitStrings = split_string(max_date_time_str, "-");
 
   auto num_years = std::stoi(splitStrings[0]);
   auto num_months = std::stoi(splitStrings[1]);
   auto num_days = std::stoi(splitStrings[2]);
 
-  splitStrings = split_with_seperator(min_date_time_str, '-');
+  splitStrings = split_string(min_date_time_str, "-");
   num_years -= std::stoi(splitStrings[0]);
 
   if (num_years <= 1)
@@ -602,7 +602,7 @@ maggraph(const char *plotfile, const std::string &varname, const std::string &va
     }
 
   char *lines[1];
-  lines[0] = (char *) malloc(1024);
+  lines[0] = (char *) std::malloc(1024);
   // To be obtained from Meta Data
   // std::snprintf(lines[0], 1024, "%s","ExpID : " );
   // std::snprintf(lines[0], 1024, "%sxxxx  Variable : %s[%s]",lines[0], varname.c_str(), varunits.c_str() );
@@ -624,11 +624,11 @@ maggraph(const char *plotfile, const std::string &varname, const std::string &va
   mag_setc("text_justification", "left");
   mag_text();
 
-  free(date_time_str);
+  std::free(date_time_str);
 
   if (DBG) fprintf(stderr, "lines=%s\n", lines[0]);
 
-  free(lines[0]);
+  std::free(lines[0]);
 }
 
 static void
@@ -660,12 +660,12 @@ verifyGraphParameters(int num_param, std::vector<std::string> &param_names)
     {
       found = false;
       syntax = true;
-      auto splitStrings = split_with_seperator(param_names[i], '=');
+      auto splitStrings = split_string(param_names[i], "=");
 
       if (splitStrings.size() > 1)
         {
-          const auto &key = splitStrings[0];
-          const auto &value = splitStrings[1];
+          auto const &key = splitStrings[0];
+          auto const &value = splitStrings[1];
 
           for (j = 0; j < graph_param_count; ++j)
             {
@@ -777,10 +777,16 @@ public:
   std::vector<std::string> pnames;
 
 public:
+#ifndef HAVE_LIBMAGICS
+  /* clang-format off */
+  void init() override { cdo_abort("MAGICS support not compiled in!"); }
+  void run() override { cdo_abort("MAGICS support not compiled in!"); }
+  void close() override { cdo_abort("MAGICS support not compiled in!"); }
+  /* clang-format on */
+#else
   void
   init() override
   {
-#ifdef HAVE_LIBMAGICS
     nparam = cdo_operator_argc();
     pnames = cdo_get_oper_argv();
 
@@ -794,13 +800,11 @@ public:
         fprintf(stderr, " Num of files %d\n", nfiles);
         fprintf(stderr, " files %s\n", ofilename);
       }
-#endif
   }
 
   void
   run() override
   {
-#ifdef HAVE_LIBMAGICS
     std::vector<std::vector<double>> datatab(nfiles);
     std::vector<std::vector<CdiDateTime>> vDateTimes(nfiles);
     std::vector<long> nts(nfiles, 0);
@@ -835,10 +839,10 @@ public:
         size_t numTsAlloc = 0;
         while (true)
           {
-            auto nrecs = cdo_stream_inq_timestep(streamID, tsID);
-            if (nrecs == 0) break;
+            auto numFields = cdo_stream_inq_timestep(streamID, tsID);
+            if (numFields == 0) break;
 
-            if (nrecs != 1) cdo_abort("Input stream has more than one point in time!");
+            if (numFields != 1) cdo_abort("Input stream has more than one point in time!");
 
             if ((size_t) tsID >= numTsAlloc)
               {
@@ -851,10 +855,10 @@ public:
             nts[fileID]++;
 
             int varID, levelID;
-            cdo_inq_record(streamID, &varID, &levelID);
+            cdo_inq_field(streamID, &varID, &levelID);
             size_t numMissVals;
             double val;
-            cdo_read_record(streamID, &val, &numMissVals);
+            cdo_read_field(streamID, &val, &numMissVals);
             datatab[fileID][tsID] = val;
             vDateTimes[fileID][tsID] = taxisInqVdatetime(taxisID);
 
@@ -875,19 +879,13 @@ public:
     maggraph(ofilename, varname, units, nfiles, nts, vDateTimes, datatab, nparam, pnames);
 
     // quit_XML_template_parser( );
-
-#else
-
-    cdo_abort("MAGICS support not compiled in!");
-#endif
   }
 
   void
   close() override
   {
-#ifdef HAVE_LIBMAGICS
     quit_MAGICS();
     if (vlistID0 != -1) vlistDestroy(vlistID0);
-#endif
   }
+#endif
 };
