@@ -202,7 +202,7 @@ checkcolour(char *colour_in)
           return 1;
         }
 
-      int n = strlen(colour_in);
+      int n = std::strlen(colour_in);
 
       if (DBG) fprintf(stdout, "  count %d  original colour %s RGB %d\n", n, colour_in, isRGB);
 
@@ -214,9 +214,9 @@ checkcolour(char *colour_in)
         }
       temp[i - 4] = '\0';
 
-      if (DBG) fprintf(stdout, "  count %d  modified color %s \n", (int) strlen(temp), temp);
+      if (DBG) fprintf(stdout, "  count %d  modified color %s \n", (int) std::strlen(temp), temp);
 
-      const auto splitStrings = split_with_seperator(temp, ';');
+      const auto splitStrings = split_string(temp, ";");
 
       if (splitStrings.size() != 3)
         {
@@ -277,12 +277,12 @@ ReadColourTable(char *filepath)
     }
 
   USR_COLOUR_COUNT = 0;
-  USR_COLOUR_TABLE = (char **) malloc(num_colors * sizeof(char *));
-  char **temp_table = (char **) malloc(num_colors * sizeof(char *));
+  USR_COLOUR_TABLE = (char **) std::malloc(num_colors * sizeof(char *));
+  char **temp_table = (char **) std::malloc(num_colors * sizeof(char *));
 
   for (int i = 0; i < num_colors; ++i)
     {
-      temp_table[i] = (char *) malloc(256 * sizeof(char));
+      temp_table[i] = (char *) std::malloc(256 * sizeof(char));
       std::fscanf(fp, "%s", temp_table[i]);
       if (DBG) fprintf(stdout, "%s\n", temp_table[i]);
     }
@@ -311,8 +311,8 @@ ReadColourTable(char *filepath)
 
   if (USR_COLOUR_COUNT < num_colors) cdo_warning(" Discarding improper format colours and continuing!");
 
-  for (int i = 0; i < num_colors; ++i) free(temp_table[i]);
-  free(temp_table);
+  for (int i = 0; i < num_colors; ++i) std::free(temp_table[i]);
+  std::free(temp_table);
 
   return 0;
 }
@@ -458,13 +458,13 @@ public:
       {
         auto found = false;
         auto syntax = true;
-        const auto splitStrings = split_with_seperator(param_names[i], '=');
+        const auto splitStrings = split_string(param_names[i], "=");
 
         if (DBG) fprintf(stderr, "Verifying params!\n");
 
         if (splitStrings.size() > 1)
           {
-            const auto &key = splitStrings[0];
+            auto const &key = splitStrings[0];
             auto value = strdup(splitStrings[1].c_str());
 
             if (opID == CONTOUR)
@@ -615,7 +615,7 @@ public:
 
                     if (key == "list")
                       {
-                        const auto splitStrings2 = split_with_seperator(value, ';');
+                        const auto splitStrings2 = split_string(value, ";");
                         if (!splitStrings2.size()) { syntax = false; }
                         else
                           {
@@ -626,7 +626,7 @@ public:
                             if (syntax == true)
                               {
                                 NUM_LEVELS = (int) splitStrings2.size();
-                                LEV_LIST = (double *) malloc(NUM_LEVELS * sizeof(double));
+                                LEV_LIST = (double *) std::malloc(NUM_LEVELS * sizeof(double));
                                 for (int k = 0; k < NUM_LEVELS; ++k) LEV_LIST[k] = std::stof(splitStrings2[k]);
                               }
                           }
@@ -634,7 +634,7 @@ public:
                   } /*** if(key == params[j])  ***/
               }     /*** Loop over param count ***/
 
-            // if (value) free(value); // value is use e.g. for DEVICE
+            // if (value) std::free(value); // value is use e.g. for DEVICE
           } /*** (splitStrings.size() > 1) ***/
         else { syntax = false; }
 
@@ -656,9 +656,9 @@ public:
 
 #ifdef HAVE_LIBMAGICS
   void
-  magplot(const char *plotfile, int p_operatorID, const std::string &varname, const std::string &units, long p_nlon, long p_nlat,
+  magplot(const char *plotfile, int p_operatorID, std::string const &varname, std::string const &units, long p_nlon, long p_nlat,
           Varray<double> &p_grid_center_lon, Varray<double> &p_grid_center_lat, Varray<double> &p_array, size_t numMissVals,
-          double missval, int p_nparam, const std::vector<std::string> &params, const std::string &datetimeStr, bool p_lregular)
+          double missval, int p_nparam, const std::vector<std::string> &params, std::string const &datetimeStr, bool p_lregular)
 
   {
     double dlon = 0, dlat = 0;
@@ -675,8 +675,8 @@ public:
 
         for (int i = 0; i < p_nparam; ++i)
           {
-            const auto splitStrings = split_with_seperator(params[i], '=');
-            const auto &key = splitStrings[0];
+            const auto splitStrings = split_string(params[i], "=");
+            auto const &key = splitStrings[0];
 
             if (key == "min") fprintf(stderr, "Min Val %g\n", YMIN);
             if (key == "max") fprintf(stderr, "Max Val %g\n", YMAX);
@@ -994,7 +994,7 @@ public:
     mag_setc("text_justification", "left");
     mag_text();
 
-    if (LEV_LIST) free(LEV_LIST);
+    if (LEV_LIST) std::free(LEV_LIST);
   }
 
 #endif
@@ -1073,12 +1073,12 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID, tsID);
+        if (numFields == 0) break;
 
         if (ANIM_FLAG)
           {
-            if (nrecs > 1)
+            if (numFields > 1)
               {
                 cdo_warning("File has more than one variable! Animation creation not possible!");
                 break;
@@ -1117,12 +1117,12 @@ public:
         const auto datetimeStr = datetime_to_string(taxisInqVdatetime(taxisID));
         if (DBG) fprintf(stderr, "Date/Time %s\n", datetimeStr.c_str());
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
             int varID, levelID;
-            cdo_inq_record(streamID, &varID, &levelID);
+            cdo_inq_field(streamID, &varID, &levelID);
             size_t numMissVals;
-            cdo_read_record(streamID, array.data(), &numMissVals);
+            cdo_read_field(streamID, array.data(), &numMissVals);
 
             const auto missval = vlistInqVarMissval(vlistID, varID);
             if (numMissVals) cdo_set_nan(missval, gridsize, array.data());
