@@ -14,14 +14,14 @@
 
 // read only the first data variable from input filename into a given double pointer
 static void
-read_first_record(const std::string &filename, double *field)
+read_first_field(std::string const &filename, double *field)
 {
   size_t numMissVals;
   int varID, levelID;
   auto streamID = stream_open_read_locked(filename.c_str());
   streamInqTimestep(streamID, 0);
-  streamInqRecord(streamID, &varID, &levelID);
-  streamReadRecord(streamID, field, &numMissVals);
+  streamInqField(streamID, &varID, &levelID);
+  streamReadField(streamID, field, &numMissVals);
   streamClose(streamID);
 }
 
@@ -33,7 +33,7 @@ countMask(const double *maskField, size_t gridSize, double falseVal)
 
   for (size_t i = 0; i < gridSize; ++i)
     {
-      if (!DBL_IS_EQUAL(maskField[i], falseVal)) counter++;
+      if (!fp_is_equal(maskField[i], falseVal)) counter++;
     }
 
   return counter;
@@ -55,10 +55,10 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  int vlistID2;
+  int vlistID2{ CDI_UNDEFID };
 
   std::vector<bool> vars;
   size_t maskSize;
@@ -76,7 +76,7 @@ public:
     if (cdo_operator_argc() < 1) cdo_abort("Too few arguments!");
 
     // check input grid type and size - this will be used for selecting relevant variables from the input file
-    const auto &maskfilename = cdo_operator_argv(0);
+    auto const &maskfilename = cdo_operator_argv(0);
     auto inputGridID = cdo_define_grid(maskfilename);
     auto inputGridSize = gridInqSize(inputGridID);
     auto inputGridType = gridInqType(inputGridID);
@@ -85,7 +85,7 @@ public:
     // create an index list of the relevant locations
     {
       Varray<double> inputMaskField(inputGridSize);
-      read_first_record(maskfilename, inputMaskField.data());
+      read_first_field(maskfilename, inputMaskField.data());
 
       // non-zero values mark the relevant points
       maskSize = countMask(inputMaskField.data(), inputGridSize, 0.0);
@@ -96,7 +96,7 @@ public:
       size_t k = 0;
       for (size_t i = 0; i < inputGridSize; ++i)
         {
-          if (!DBL_IS_EQUAL(inputMaskField[i], 0.0)) maskIndexList[k++] = i;
+          if (!fp_is_equal(inputMaskField[i], 0.0)) maskIndexList[k++] = i;
         }
 
       if (k == inputGridSize) cdo_warning("Number of mask values and gridsize is the same, no reduction!");
@@ -106,7 +106,7 @@ public:
     bool nobounds = false, nocoords = false;
     if (2 <= cdo_operator_argc())
       {
-        const auto &coordinatesLimitation = cdo_operator_argv(1);
+        auto const &coordinatesLimitation = cdo_operator_argv(1);
         if (coordinatesLimitation == "nobounds") nobounds = true;
         if (coordinatesLimitation == "nocoords") nocoords = true;
       }
@@ -128,7 +128,7 @@ public:
     vlistClearFlag(vlistID1);
     for (int varID = 0; varID < nvars; ++varID)
       {
-        const auto &var = varList1.vars[varID];
+        auto const &var = varList1.vars[varID];
         if (inputGridType == var.gridType && inputGridSize == var.gridsize)
           {
             vars[varID] = true;
@@ -148,8 +148,8 @@ public:
     vlistDefTaxis(vlistID2, taxisID2);
 
     // use the new selection grid for all output variables
-    auto ngrids = vlistNumGrids(vlistID2);
-    for (int index = 0; index < ngrids; ++index) vlistChangeGridIndex(vlistID2, index, outputGridID);
+    auto numGrids = vlistNumGrids(vlistID2);
+    for (int index = 0; index < numGrids; ++index) vlistChangeGridIndex(vlistID2, index, outputGridID);
 
     // loop over input fields and mask the data values {{{
     streamID2 = cdo_open_write(1);
@@ -165,28 +165,27 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             if (vars[varID])
               {
                 auto varID2 = vlistFindVar(vlistID2, varID);
                 auto levelID2 = vlistFindLevel(vlistID2, varID, levelID);
 
                 size_t numMissVals;
-                cdo_read_record(streamID1, arrayIn.data(), &numMissVals);
+                cdo_read_field(streamID1, arrayIn.data(), &numMissVals);
 
                 for (size_t i = 0; i < maskSize; ++i) arrayOut[i] = arrayIn[maskIndexList[i]];
 
-                cdo_def_record(streamID2, varID2, levelID2);
-                cdo_write_record(streamID2, arrayOut.data(), 0);
+                cdo_def_field(streamID2, varID2, levelID2);
+                cdo_write_field(streamID2, arrayOut.data(), 0);
               }
           }
         tsID++;
