@@ -24,14 +24,14 @@
 #include "region.h"
 
 static void
-maskbox(std::vector<int8_t> &mask, int gridID, const SelboxInfo &selboxInfo)
+maskbox(Vmask &mask, int gridID, const SelboxInfo &selboxInfo)
 {
-  const auto &lat1 = selboxInfo.lat1;
-  const auto &lat2 = selboxInfo.lat2;
-  const auto &lon11 = selboxInfo.lon11;
-  const auto &lon12 = selboxInfo.lon12;
-  const auto &lon21 = selboxInfo.lon21;
-  const auto &lon22 = selboxInfo.lon22;
+  auto const &lat1 = selboxInfo.lat1;
+  auto const &lat2 = selboxInfo.lat2;
+  auto const &lon11 = selboxInfo.lon11;
+  auto const &lon12 = selboxInfo.lon12;
+  auto const &lon21 = selboxInfo.lon21;
+  auto const &lon22 = selboxInfo.lon22;
   long nlon = gridInqXsize(gridID);
   long nlat = gridInqYsize(gridID);
 
@@ -44,7 +44,7 @@ maskbox(std::vector<int8_t> &mask, int gridID, const SelboxInfo &selboxInfo)
 void getlonlatparams(int argc_offset, double &xlon1, double &xlon2, double &xlat1, double &xlat2);
 
 static void
-maskbox_cell(std::vector<int8_t> &mask, int gridID)
+maskbox_cell(Vmask &mask, int gridID)
 {
   double xlon1 = 0, xlon2 = 0, xlat1 = 0, xlat2 = 0;
   getlonlatparams(0, xlon1, xlon2, xlat1, xlat2);
@@ -123,7 +123,7 @@ point_is_inside(double xval, double yval, double xmin, double xmax, const double
 }
 
 static void
-mask_region_regular(std::vector<int8_t> &mask, size_t nlon, size_t nlat, const Varray<double> &xvals, const Varray<double> &yvals,
+mask_region_regular(Vmask &mask, size_t nlon, size_t nlat, Varray<double> const &xvals, Varray<double> const &yvals,
                     const double *xcoords, const double *ycoords, size_t segmentSize)
 {
   auto xmm = varray_min_max(segmentSize, xcoords);
@@ -145,8 +145,8 @@ mask_region_regular(std::vector<int8_t> &mask, size_t nlon, size_t nlat, const V
 }
 
 static void
-mask_region_cell(std::vector<int8_t> &mask, size_t gridsize, const Varray<double> &xvals, const Varray<double> &yvals,
-                 const double *xcoords, const double *ycoords, size_t segmentSize)
+mask_region_cell(Vmask &mask, size_t gridsize, Varray<double> const &xvals, Varray<double> const &yvals, const double *xcoords,
+                 const double *ycoords, size_t segmentSize)
 {
   auto xmm = varray_min_max(segmentSize, xcoords);
   auto ymm = varray_min_max(segmentSize, ycoords);
@@ -169,8 +169,8 @@ get_gridID(int vlistID1, bool operIndexBox)
 {
   std::vector<int> gridsFound;
 
-  auto ngrids = vlistNumGrids(vlistID1);
-  for (int index = 0; index < ngrids; ++index)
+  auto numGrids = vlistNumGrids(vlistID1);
+  for (int index = 0; index < numGrids; ++index)
     {
       auto gridID1 = vlistGrid(vlistID1, index);
       if (gridInqSize(gridID1) == 1) continue;
@@ -240,10 +240,10 @@ public:
 private:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
   std::vector<bool> processVars;
-  std::vector<int8_t> mask;
+  Vmask mask;
   size_t gridsize;
 
   VarList varList1;
@@ -347,8 +347,7 @@ public:
         selcircle_get_parameter(cpoint);
         if (cpoint.radius < 0.0 || cpoint.radius > 180.0) cdo_abort("radius=%g out of bounds (0-180 deg)!", cpoint.radius);
 
-        auto gridsizemax = vlistGridsizeMax(vlistID1);
-        if (gridsizemax < cpoint.maxpoints) cpoint.maxpoints = gridsizemax;
+        if (varList1.gridsizeMax() < cpoint.maxpoints) cpoint.maxpoints = varList1.gridsizeMax();
         if (Options::cdoVerbose)
           cdo_print("lon = %g, lat = %g, radius = %gdeg(%gkm)", cpoint.lon, cpoint.lat, cpoint.radius,
                     radiusDegToKm(cpoint.radius));
@@ -371,22 +370,21 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
 
             if (processVars[varID])
               {
-                const auto &var = varList1.vars[varID];
+                auto const &var = varList1.vars[varID];
                 field.init(var);
-                cdo_read_record(streamID1, field);
+                cdo_read_field(streamID1, field);
 
                 auto func = [&](auto &v) {
                   for (size_t i = 0; i < gridsize; ++i)
@@ -395,8 +393,8 @@ public:
                 field_operation(func, field);
 
                 field_num_mv(field);
-                cdo_def_record(streamID2, varID, levelID);
-                cdo_write_record(streamID2, field);
+                cdo_def_field(streamID2, varID, levelID);
+                cdo_write_field(streamID2, field);
               }
           }
 
