@@ -62,7 +62,7 @@ mastrfu(int gridID, int zaxisID, const Varray2D<double> &field1, Varray2D<double
         for (int ilev = nlev - 1; ilev >= 0; ilev--)
           for (int n = ilev; n < nlev - 1; ++n)
             {
-              if (dbl_is_equal(field1[n][ilat], missval) || dbl_is_equal(field1[n + 1][ilat], missval))
+              if (fp_is_equal(field1[n][ilat], missval) || fp_is_equal(field1[n + 1][ilat], missval))
                 {
                   field2[ilev][ilat] = missval;
                   break;
@@ -90,8 +90,8 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   int zaxisID;
 
@@ -117,7 +117,7 @@ public:
     auto numVars = varList1.numVars();
     if (numVars != 1) cdo_abort("This operator works only with one variable!");
 
-    const auto &var0 = varList1.vars[0];
+    auto const &var0 = varList1.vars[0];
     if (var0.code > 0 && var0.code != 132) cdo_warning("Unexpected code %d!", var0.code);
 
     missval = var0.missval;
@@ -159,31 +159,30 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
         size_t numMissVals = 0;
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             size_t numMissVals1;
-            cdo_read_record(streamID1, array1[levelID].data(), &numMissVals1);
+            cdo_read_field(streamID1, array1[levelID].data(), &numMissVals1);
             numMissVals += numMissVals1;
           }
 
         mastrfu(gridID, zaxisID, array1, array2, numMissVals, missval);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
             int varID = 0;
-            int levelID = recID;
-            cdo_def_record(streamID2, varID, levelID);
+            int levelID = fieldID;
+            cdo_def_field(streamID2, varID, levelID);
             numMissVals = array_num_mv(nlat, array2[levelID].data(), missval);
-            cdo_write_record(streamID2, array2[levelID].data(), numMissVals);
+            cdo_write_field(streamID2, array2[levelID].data(), numMissVals);
           }
 
         tsID++;
