@@ -16,19 +16,21 @@
 #include "cdo_options.h"
 #include "cdo_rlimit.h"
 #include "process_int.h"
+#include "util_string.h"
 #include "util_files.h"
 #include "printinfo.h"
 #include "param_conversion.h"
 #include "pmlist.h"
+#include "progress.h"
 
 struct StreamInfo
 {
   CdoStreamID streamID;
   CdiDateTime vDateTime{};
-  int vlistID;
-  int taxisID;
-  int tsID;
-  int numRecords;
+  int vlistID{ -1 };
+  int taxisID{ -1 };
+  int tsID{ -1 };
+  int numFields{ 0 };
   VarList varList;
   std::map<int, int> mapOfVarIDs;
 };
@@ -50,9 +52,10 @@ getenv_skip_same_time()
   return false;
 }
 
-static void
+static int
 open_all_files(int numFiles, std::vector<StreamInfo> &streamInfoList)
 {
+  int numSteps = 0;
   for (int fileID = 0; fileID < numFiles; ++fileID)
     {
       if (Options::cdoVerbose) cdo_print("process: %s", cdo_get_stream_name(fileID));
@@ -62,7 +65,10 @@ open_all_files(int numFiles, std::vector<StreamInfo> &streamInfoList)
       si.vlistID = cdo_stream_inq_vlist(si.streamID);
       si.taxisID = vlistInqTaxis(si.vlistID);
       si.varList = VarList(si.vlistID);
+      if (si.varList.numSteps() > 0) numSteps += si.varList.numSteps();
     }
+
+  return numSteps;
 }
 
 static void
@@ -72,8 +78,8 @@ read_first_timestep(int numFiles, std::vector<StreamInfo> &streamInfoList)
     {
       auto &si = streamInfoList[fileID];
       si.tsID = 0;
-      si.numRecords = cdo_stream_inq_timestep(si.streamID, si.tsID);
-      if (si.numRecords == 0)
+      si.numFields = cdo_stream_inq_timestep(si.streamID, si.tsID);
+      if (si.numFields == 0)
         {
           cdo_stream_close(si.streamID);
           si.streamID = CDO_STREAM_UNDEF;
@@ -88,19 +94,19 @@ get_parameter(bool &skipSameTime, MapFlag &mapFlag)
   auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      const auto &pargv = cdo_get_oper_argv();
+      auto const &pargv = cdo_get_oper_argv();
 
       KVList kvlist;
       kvlist.name = cdo_module_name();
       if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-          const auto &value = kv.values[0];
+          auto const &value = kv.values[0];
 
           // clang-format off
           if      (key == "skip_same_time")  skipSameTime = parameter_to_bool(value);
@@ -130,9 +136,9 @@ public:
   };
   inline static RegisterEntry<Mergetime> registration = RegisterEntry<Mergetime>(module);
 
-  int tsID2 = 0;
-  int vlistID2 = CDI_UNDEFID;
-  int taxisID2 = CDI_UNDEFID;
+  int tsID2{ 0 };
+  int vlistID2{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
   CdiDateTime lastDateTime{};
   Field field;
 
@@ -145,6 +151,8 @@ public:
   int vlistFileIDmin{ -1 };
   int vlistFileIDmax{ -1 };
 
+  int numSteps{ 0 };
+
   MapFlag mapFlag{ MapFlag::Undefined };
 
 public:
@@ -162,7 +170,7 @@ public:
 
     cdo::set_numfiles(numFiles + 8);
 
-    open_all_files(numFiles, streamInfoList);
+    numSteps = open_all_files(numFiles, streamInfoList);
 
     // check that the contents is always the same
     if (mapFlag == MapFlag::Undefined)
@@ -171,7 +179,7 @@ public:
           {
             auto &si = streamInfoList[fileID];
             varList_compare(streamInfoList[0].varList, si.varList);
-            for (const auto &var : si.varList.vars) si.mapOfVarIDs[var.ID] = var.ID;
+            for (auto const &var : si.varList.vars) si.mapOfVarIDs[var.ID] = var.ID;
           }
       }
     else
@@ -185,7 +193,7 @@ public:
             if (numVars > streamInfoList[vlistFileIDmax].varList.numVars()) vlistFileIDmax = fileID;
           }
 
-        const auto &varList2 = streamInfoList[(mapFlag == MapFlag::Intersect) ? vlistFileIDmin : vlistFileIDmax].varList;
+        auto const &varList2 = streamInfoList[(mapFlag == MapFlag::Intersect) ? vlistFileIDmin : vlistFileIDmax].varList;
         for (int fileID = 0; fileID < numFiles; ++fileID)
           {
             auto &si = streamInfoList[fileID];
@@ -204,7 +212,7 @@ public:
   }
 
   void
-  fill_missing_fields(const StreamInfo &si1, StreamInfo &si2)
+  fill_missing_fields(StreamInfo const &si1, StreamInfo &si2)
   {
     auto maxVars = si1.varList.numVars();
     auto numVars = si2.varList.numVars();
@@ -216,14 +224,14 @@ public:
           {
             if (missingIDs[varID] != 1)
               {
-                const auto &var1 = si1.varList.vars[varID];
+                auto const &var1 = si1.varList.vars[varID];
                 field.init(var1);
                 field_fill(field, var1.missval);
                 field.numMissVals = field.gridsize;
                 for (int levelID = 0; levelID < var1.nlevels; levelID++)
                   {
-                    cdo_def_record(streamID2, varID, levelID);
-                    cdo_write_record(streamID2, field);
+                    cdo_def_field(streamID2, varID, levelID);
+                    cdo_write_field(streamID2, field);
                   }
               }
           }
@@ -233,6 +241,8 @@ public:
   void
   run() override
   {
+    cdo::Progress progress(get_id());
+
     while (true)
       {
         auto processTimestep = true;
@@ -269,6 +279,8 @@ public:
 
         if (processTimestep)
           {
+            if (numSteps > 1) progress.update((tsID2 + 1.0) / numSteps);
+
             if (tsID2 == 0)
               {
                 auto vlistID1 = (mapFlag == MapFlag::Undefined)
@@ -276,6 +288,7 @@ public:
                                     : ((mapFlag == MapFlag::Intersect) ? streamInfoList[vlistFileIDmin].vlistID
                                                                        : streamInfoList[vlistFileIDmax].vlistID);
                 vlistID2 = vlistDuplicate(vlistID1);
+                vlist_unpack(vlistID2);
                 auto taxisID1 = vlistInqTaxis(si.vlistID);
                 taxisID2 = taxisDuplicate(taxisID1);
                 vlistDefTaxis(vlistID2, taxisID2);
@@ -287,11 +300,11 @@ public:
             cdo_taxis_copy_timestep(taxisID2, si.taxisID);
             cdo_def_timestep(streamID2, tsID2);
 
-            for (int recID = 0; recID < si.numRecords; ++recID)
+            for (int fieldID = 0; fieldID < si.numFields; ++fieldID)
               {
-                auto [varID, levelID] = cdo_inq_record(si.streamID);
+                auto [varID, levelID] = cdo_inq_field(si.streamID);
 
-                const auto &var = si.varList.vars[varID];
+                auto const &var = si.varList.vars[varID];
                 if (tsID2 > 0 && si.tsID == 0 && var.isConstant) continue;
 
                 auto varID2 = varID;
@@ -302,14 +315,14 @@ public:
                     varID2 = it->second;
                   }
 
-                cdo_def_record(streamID2, varID2, levelID);
+                cdo_def_field(streamID2, varID2, levelID);
 
-                if (dataIsUnchanged) { cdo_copy_record(streamID2, si.streamID); }
+                if (dataIsUnchanged) { cdo_copy_field(streamID2, si.streamID); }
                 else
                   {
                     field.init(var);
-                    cdo_read_record(si.streamID, field);
-                    cdo_write_record(streamID2, field);
+                    cdo_read_field(si.streamID, field);
+                    cdo_write_field(streamID2, field);
                   }
               }
 
@@ -318,8 +331,8 @@ public:
             tsID2++;
           }
 
-        si.numRecords = cdo_stream_inq_timestep(si.streamID, ++si.tsID);
-        if (si.numRecords == 0)
+        si.numFields = cdo_stream_inq_timestep(si.streamID, ++si.tsID);
+        if (si.numFields == 0)
           {
             cdo_stream_close(si.streamID);
             si.streamID = CDO_STREAM_UNDEF;
