@@ -20,39 +20,40 @@
 #include "cdo_vlist.h"
 #include "field_functions.h"
 
-static void
-add_operators(void)
+class Monarith : public Process
 {
-  cdo_operator_add("monadd", FieldFunc_Add, 0, nullptr);
-  cdo_operator_add("monsub", FieldFunc_Sub, 0, nullptr);
-  cdo_operator_add("monmul", FieldFunc_Mul, 0, nullptr);
-  cdo_operator_add("mondiv", FieldFunc_Div, 0, nullptr);
-}
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Monarith",
+    .operators = { { "monadd", FieldFunc_Add, 0, MonarithHelp },
+                   { "monsub", FieldFunc_Sub, 0, MonarithHelp },
+                   { "monmul", FieldFunc_Mul, 0, MonarithHelp },
+                   { "mondiv", FieldFunc_Div, 0, MonarithHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 2, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Monarith> registration = RegisterEntry<Monarith>(module);
 
-class ModuleMonarith
-{
   CdoStreamID streamID1;
   CdoStreamID streamID2;
   CdoStreamID streamID3;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
   int taxisID3;
 
-  FieldVector2D vars2;
+  FieldVector2D varsData2;
   VarList varList1;
-  Field field;
 
   int operfunc;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    add_operators();
-
     auto operatorID = cdo_operator_id();
     operfunc = cdo_operator_f1(operatorID);
 
@@ -67,9 +68,9 @@ public:
 
     vlist_unpack(vlistID3);
 
-    vlist_compare(vlistID1, vlistID2, CmpVlist::All);
-
-    varList_init(varList1, vlistID1);
+    varList1 = VarList(vlistID1);
+    VarList varList2(vlistID2);
+    varList_compare(varList1, varList2);
 
     taxisID1 = vlistInqTaxis(vlistID1);
     taxisID2 = vlistInqTaxis(vlistID2);
@@ -79,19 +80,21 @@ public:
     streamID3 = cdo_open_write(2);
     cdo_def_vlist(streamID3, vlistID3);
 
-    fields_from_vlist(vlistID2, vars2, FIELD_VEC | FIELD_NAT);
+    field2D_init(varsData2, varList2, FIELD_VEC | FIELD_NAT);
   }
 
   void
-  run()
+  run() override
   {
+    Field field;
+
     int yearmon2 = -1;
     int tsID = 0;
     int tsID2 = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         auto vDate1 = taxisInqVdatetime(taxisID1).date;
         int year1, month1, day1;
@@ -102,8 +105,8 @@ public:
           {
             if (Options::cdoVerbose) cdo_print("Process: Year = %4d  Month = %2d", year1, month1);
 
-            auto nrecs2 = cdo_stream_inq_timestep(streamID2, tsID2);
-            if (nrecs2 == 0) cdo_abort("Missing year=%4d mon=%2d in %s!", year1, month1, cdo_get_stream_name(1));
+            auto numFields2 = cdo_stream_inq_timestep(streamID2, tsID2);
+            if (numFields2 == 0) cdo_abort("Missing year=%4d mon=%2d in %s!", year1, month1, cdo_get_stream_name(1));
 
             auto vDate2 = taxisInqVdatetime(taxisID2).date;
             int year2, month2, day2;
@@ -115,11 +118,10 @@ public:
                           cdo_get_stream_name(1), year2, month2, year1, month1);
               }
 
-            for (int recID = 0; recID < nrecs2; ++recID)
+            for (int fieldID = 0; fieldID < numFields2; ++fieldID)
               {
-                int varID, levelID;
-                cdo_inq_record(streamID2, &varID, &levelID);
-                cdo_read_record(streamID2, vars2[varID][levelID]);
+                auto [varID, levelID] = cdo_inq_field(streamID2);
+                cdo_read_field(streamID2, varsData2[varID][levelID]);
               }
 
             tsID2++;
@@ -128,17 +130,16 @@ public:
         cdo_taxis_copy_timestep(taxisID3, taxisID1);
         cdo_def_timestep(streamID3, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            field.init(varList1[varID]);
-            cdo_read_record(streamID1, field);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            field.init(varList1.vars[varID]);
+            cdo_read_field(streamID1, field);
 
-            field2_function(field, vars2[varID][levelID], operfunc);
+            field2_function(field, varsData2[varID][levelID], operfunc);
 
-            cdo_def_record(streamID3, varID, levelID);
-            cdo_write_record(streamID3, field);
+            cdo_def_field(streamID3, varID, levelID);
+            cdo_write_field(streamID3, field);
           }
 
         tsID++;
@@ -146,23 +147,10 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID3);
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Monarith(void *process)
-{
-  ModuleMonarith monarith;
-  monarith.init(process);
-  monarith.run();
-  monarith.close();
-
-  return nullptr;
-}
