@@ -197,8 +197,8 @@ public:
   CdoStreamID streamID2;
   CdoStreamID streamID3;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
   int taxisID3;
 
   Varray<double> ufield_v;
@@ -252,8 +252,8 @@ public:
           cdo_abort("U and V not found in %s", cdo_get_stream_name(0));
       }
 
-    const auto &varU = varList1.vars[uid];
-    const auto &varV = varList1.vars[vid];
+    auto const &varU = varList1.vars[uid];
+    auto const &varV = varList1.vars[vid];
     if (varU.nlevels != varV.nlevels) cdo_abort("U and V have different number of levels!");
 
     auto gridID1 = varU.gridID;
@@ -356,21 +356,19 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
         cdo_taxis_copy_timestep(taxisID3, taxisID1);
         cdo_def_timestep(streamID3, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-
-            if (varID == uid) cdo_read_record(streamID1, urfield[levelID].data(), &numMissVals1);
-            if (varID == vid) cdo_read_record(streamID1, vrfield[levelID].data(), &numMissVals2);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            if (varID == uid) cdo_read_field(streamID1, urfield[levelID].data(), &numMissVals1);
+            if (varID == vid) cdo_read_field(streamID1, vrfield[levelID].data(), &numMissVals2);
           }
 
         for (int levelID = 0; levelID < nlevs; ++levelID)
@@ -380,8 +378,8 @@ public:
               {
                 for (size_t i = 0; i < gridsize; ++i)
                   {
-                    if (DBL_IS_EQUAL(urfield[levelID][i], missval1)) urfield[levelID][i] = 0.0;
-                    if (DBL_IS_EQUAL(vrfield[levelID][i], missval2)) vrfield[levelID][i] = 0.0;
+                    if (fp_is_equal(urfield[levelID][i], missval1)) urfield[levelID][i] = 0.0;
+                    if (fp_is_equal(vrfield[levelID][i], missval2)) vrfield[levelID][i] = 0.0;
                   }
               }
 
@@ -414,10 +412,10 @@ public:
 
             for (size_t i = 0; i < nlon; ++i) { vfield[nlat - 1][i] = vhelp[nlat - 1][i + 1]; }
 
-            cdo_def_record(streamID2, 0, levelID);
-            cdo_write_record(streamID2, ufield_v.data(), numMissVals1);
-            cdo_def_record(streamID3, 0, levelID);
-            cdo_write_record(streamID3, vfield_v.data(), numMissVals2);
+            cdo_def_field(streamID2, 0, levelID);
+            cdo_write_field(streamID2, ufield_v.data(), numMissVals1);
+            cdo_def_field(streamID3, 0, levelID);
+            cdo_write_field(streamID3, vfield_v.data(), numMissVals2);
           }
 
         tsID++;
