@@ -36,7 +36,7 @@ uv2dv_cfd_W(double missval, double *u, double *v, double *lon, double *lat, size
       double *tmp_v = v + k * gridsize_uv;
       double *tmp_div = div + k * gridsize_uv;
       // Init output array.
-      ranges::fill_n(tmp_div, gridsize_uv, 0.0);
+      std::ranges::fill_n(tmp_div, gridsize_uv, 0.0);
       // Call the Fortran routine.
 #ifdef HAVE_CF_INTERFACE
       DDVFIDF(tmp_u, tmp_v, lat, lon, inlon, inlat, missval, boundOpt, tmp_div, ierror);
@@ -66,7 +66,7 @@ uv2vr_cfd_W(double missval, double *u, double *v, double *lon, double *lat, size
       double *tmp_v = v + k * gridsize_uv;
       double *tmp_vort = vort + k * gridsize_uv;
       // Init output array.
-      ranges::fill_n(tmp_vort, gridsize_uv, 0.0);
+      std::ranges::fill_n(tmp_vort, gridsize_uv, 0.0);
       // Call the Fortran routine.
 #ifdef HAVE_CF_INTERFACE
       DVRFIDF(tmp_u, tmp_v, lat, lon, inlon, inlat, missval, boundOpt, tmp_vort, ierror);
@@ -77,7 +77,7 @@ uv2vr_cfd_W(double missval, double *u, double *v, double *lon, double *lat, size
 }
 
 static int
-find_name(const VarList &varList, const std::string &name)
+find_name(const VarList &varList, std::string const &name)
 {
   auto numVars = varList.numVars();
   for (int varID = 0; varID < numVars; ++varID)
@@ -119,19 +119,19 @@ set_parameter(void)
   auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      const auto &pargv = cdo_get_oper_argv();
+      auto const &pargv = cdo_get_oper_argv();
 
       KVList kvlist;
       kvlist.name = "PARAMETER";
       if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-          const auto &value = kv.values[0];
+          auto const &value = kv.values[0];
 
           // clang-format off
           if      (key == "u") uName = value;
@@ -176,10 +176,10 @@ public:
 
   int operatorID;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  int vlistID2;
+  int vlistID2{ CDI_UNDEFID };
 
   int varIDu;
   int varIDv;
@@ -190,11 +190,6 @@ public:
   size_t nlon;
   size_t nlat;
 
-  Varray<double> array;
-  Varray<double> arrayu;
-  Varray<double> arrayv;
-  Varray<double> arrayo;
-
   Varray<double> lon;
   Varray<double> lat;
 
@@ -230,8 +225,8 @@ public:
     if (varIDu == CDI_UNDEFID) cdo_abort("%s not found!", uName);
     if (varIDv == CDI_UNDEFID) cdo_abort("%s not found!", vName);
 
-    const auto &varU = varList1.vars[varIDu];
-    const auto &varV = varList1.vars[varIDv];
+    auto const &varU = varList1.vars[varIDu];
+    auto const &varV = varList1.vars[varIDv];
 
     auto gridIDu = varU.gridID;
     auto gridIDv = varV.gridID;
@@ -283,36 +278,34 @@ public:
     streamID2 = cdo_open_write(1);
 
     cdo_def_vlist(streamID2, vlistID2);
-
-    auto gridsizemax = vlistGridsizeMax(vlistID1);
-    array = Varray<double>(gridsizemax);
-    arrayu = Varray<double>(nlev * gridsizeuv);
-    arrayv = Varray<double>(nlev * gridsizeuv);
-    arrayo = Varray<double>(nlev * gridsizeuv);
   }
 
   void
   run() override
   {
-    const auto &varU = varList1.vars[varIDu];
-    const auto &varV = varList1.vars[varIDv];
+    Varray<double> array(varList1.gridsizeMax());
+    Varray<double> arrayu(nlev * gridsizeuv);
+    Varray<double> arrayv(nlev * gridsizeuv);
+    Varray<double> arrayo(nlev * gridsizeuv);
+
+    auto const &varU = varList1.vars[varIDu];
+    auto const &varV = varList1.vars[varIDv];
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         size_t numMissValsu = 0, numMissValsv = 0;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             size_t numMissVals;
-            cdo_read_record(streamID1, &array[0], &numMissVals);
+            cdo_read_field(streamID1, &array[0], &numMissVals);
 
             if (varID == varIDu || varID == varIDv)
               {
@@ -330,8 +323,8 @@ public:
 
             if (outMode == OutMode::APPEND)
               {
-                cdo_def_record(streamID2, varID, levelID);
-                cdo_write_record(streamID2, &array[0], numMissVals);
+                cdo_def_field(streamID2, varID, levelID);
+                cdo_write_field(streamID2, &array[0], numMissVals);
               }
           }
 
@@ -358,8 +351,8 @@ public:
           {
             auto parray = &arrayo[levelID * gridsizeuv];
             auto numMissVals = array_num_mv(gridsizeuv, parray, varU.missval);
-            cdo_def_record(streamID2, varIDo, levelID);
-            cdo_write_record(streamID2, parray, numMissVals);
+            cdo_def_field(streamID2, varIDo, levelID);
+            cdo_write_field(streamID2, parray, numMissVals);
           }
 
         tsID++;
