@@ -26,13 +26,13 @@
 #include "cdo_zaxis.h"
 
 static void
-outputarr(int dig, size_t gridsize, const Varray<double> &array)
+outputarr(int dig, size_t gridsize, Varray<double> const &array)
 {
   for (size_t i = 0; i < gridsize; ++i) { fprintf(stdout, "  arr[%zu] = %.*g;\n", i, dig, array[i]); }
 }
 
 static void
-outputsp(size_t gridsize, const Varray<double> &array, long ntr)
+outputsp(size_t gridsize, Varray<double> const &array, long ntr)
 {
   auto mm = varray_min_max(gridsize, array);
   if (/* T11 */ mm.min >= -1 && mm.max <= 12)
@@ -51,7 +51,7 @@ outputsp(size_t gridsize, const Varray<double> &array, long ntr)
 }
 
 static void
-output(size_t gridsize, const Varray<double> &array)
+output(size_t gridsize, Varray<double> const &array)
 {
   int nout = 0;
   for (size_t i = 0; i < gridsize; ++i)
@@ -68,11 +68,16 @@ output(size_t gridsize, const Varray<double> &array)
 }
 
 static void
-outputxyz(size_t gridsize, const Varray<double> &array, double missval, size_t nlon, size_t nlat, const Varray<double> &lon,
-          const Varray<double> &lat)
+print_xyz(FILE *fp, double x, double y, double z)
+{
+  fprintf(fp, "%g %g %g\n", x, y, z);
+}
+
+static void
+outputxyz(size_t gridsize, Varray<double> const &array, double missval, size_t nlon, size_t nlat, Varray<double> const &lon,
+          Varray<double> const &lat)
 {
   double fmin = 0.0;
-  double x, y, z;
   for (size_t i = 0; i < gridsize; ++i)
     if (fp_is_not_equal(array[i], missval))
       {
@@ -90,26 +95,16 @@ outputxyz(size_t gridsize, const Varray<double> &array, double missval, size_t n
   fprintf(fp, ">\n");
   for (size_t i = 0; i < nlon; ++i)
     {
-      x = x0;
-      y = y0;
-      z = z0;
-      fprintf(fp, "%g %g %g\n", x, y, z);
-      x = x0;
-      y = y0;
-      z = array[i];
-      fprintf(fp, "%g %g %g\n", x, y, z);
-      x = x0 + dx;
-      y = y0;
-      fprintf(fp, "%g %g %g\n", x, y, z);
-      x0 = x; /*y0 = y0;*/
-      z0 = z;
+      print_xyz(fp, x0, y0, z0);
+      z0 = array[i];
+      print_xyz(fp, x0, y0, z0);
+      x0 = x0 + dx;
+      print_xyz(fp, x0, y0, z0);
     }
-  x = x0;
-  y = y0;
-  z = fmin;
-  fprintf(fp, "%g %g %g\n", x, y, z);
-  x = lon[0] - dx / 2;
-  fprintf(fp, "%g %g %g\n", x, y, z);
+  z0 = fmin;
+  print_xyz(fp, x0, y0, z0);
+  x0 = lon[0] - dx / 2;
+  print_xyz(fp, x0, y0, z0);
 
   // second front plane
   x0 = lon[0] - dx / 2;
@@ -118,26 +113,16 @@ outputxyz(size_t gridsize, const Varray<double> &array, double missval, size_t n
   fprintf(fp, ">\n");
   for (size_t i = 0; i < nlat; ++i)
     {
-      x = x0;
-      y = y0;
-      z = z0;
-      fprintf(fp, "%g %g %g\n", x, y, z);
-      x = x0;
-      y = y0;
-      z = array[i * nlon];
-      fprintf(fp, "%g %g %g\n", x, y, z);
-      x = x0;
-      y = y0 + dx;
-      fprintf(fp, "%g %g %g\n", x, y, z);
-      /*x0 = x0;*/ y0 = y;
-      z0 = z;
+      print_xyz(fp, x0, y0, z0);
+      z0 = array[i * nlon];
+      print_xyz(fp, x0, y0, z0);
+      y0 += dx;
+      print_xyz(fp, x0, y0, z0);
     }
-  x = x0;
-  y = y0;
-  z = fmin;
-  fprintf(fp, "%g %g %g\n", x, y, z);
-  y = lat[0] - dx / 2;
-  fprintf(fp, "%g %g %g\n", x, y, z);
+  z0 = fmin;
+  print_xyz(fp, x0, y0, z0);
+  y0 = lat[0] - dx / 2;
+  print_xyz(fp, x0, y0, z0);
 
   std::fclose(fp);
 }
@@ -190,7 +175,7 @@ read_lonlat_coordinates(int gridID0, Varray<double> &grid_center_lon, Varray<dou
 }
 
 static void
-outputint(size_t gridsize, const Varray<double> &array)
+outputint(size_t gridsize, Varray<double> const &array)
 {
   int nout = 0;
   for (size_t i = 0; i < gridsize; ++i)
@@ -207,7 +192,7 @@ outputint(size_t gridsize, const Varray<double> &array)
 }
 
 static void
-outputf(int nelem, const std::string &format, size_t gridsize, const Varray<double> &array)
+outputf(int nelem, std::string const &format, size_t gridsize, Varray<double> const &array)
 {
   int nout = 0;
   for (size_t i = 0; i < gridsize; ++i)
@@ -314,6 +299,8 @@ public:
   void
   init() override
   {
+    if (this_is_the_only_process()) { cdiDefGlobal("READ_CELL_CORNERS", false); }
+
     OUTPUT = module.get_id("output");
     OUTPUTINT = module.get_id("outputint");
     OUTPUTSRV = module.get_id("outputsrv");
@@ -346,7 +333,7 @@ public:
         operator_input_arg("keys to print");
 
         auto npar = cdo_operator_argc();
-        const auto &parnames = cdo_get_oper_argv();
+        auto const &parnames = cdo_get_oper_argv();
 
         if (Options::cdoVerbose)
           for (int i = 0; i < npar; ++i) cdo_print("key%d=%s", i + 1, parnames[i]);
@@ -360,7 +347,7 @@ public:
             size_t k;
             for (k = 0; k < keyMap.size(); ++k)
               {
-                const auto &key = keyMap[k].key;
+                auto const &key = keyMap[k].key;
                 if (key == currentName)
                   {
                     if (keyMap[k].idx == knohead)
@@ -400,12 +387,12 @@ public:
 
         VarList varList(vlistID);
 
-        auto ngrids = vlistNumGrids(vlistID);
-        int ndiffgrids = 0;
-        for (index = 1; index < ngrids; ++index)
-          if (vlistGrid(vlistID, 0) != vlistGrid(vlistID, index)) ndiffgrids++;
+        auto numGrids = vlistNumGrids(vlistID);
+        int numDiffGrids = 0;
+        for (index = 1; index < numGrids; ++index)
+          if (vlistGrid(vlistID, 0) != vlistGrid(vlistID, index)) numDiffGrids++;
 
-        if (ndiffgrids > 0) cdo_abort("Too many different grids!");
+        if (numDiffGrids > 0) cdo_abort("Too many different grids!");
 
         auto gridID0 = vlistGrid(vlistID, 0);
         auto gridtype = gridInqType(gridID0);
@@ -428,8 +415,8 @@ public:
         auto taxisID = vlistInqTaxis(vlistID);
         while (true)
           {
-            auto nrecs = cdo_stream_inq_timestep(streamID, tsID);
-            if (nrecs == 0) break;
+            auto numFields = cdo_stream_inq_timestep(streamID, tsID);
+            if (numFields == 0) break;
 
             auto vDateTime = taxisInqVdatetime(taxisID);
             auto vDateStr = date_to_string(vDateTime.date);
@@ -437,11 +424,10 @@ public:
 
             cdiDate_decode(vDateTime.date, &year, &month, &day);
 
-            for (int recID = 0; recID < nrecs; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                int varID, levelID;
-                cdo_inq_record(streamID, &varID, &levelID);
-                const auto &var = varList.vars[varID];
+                auto [varID, levelID] = cdo_inq_field(streamID);
+                auto const &var = varList.vars[varID];
 
                 auto code = var.code;
                 auto gridID = var.gridID;
@@ -460,7 +446,7 @@ public:
                     nlat = 1;
                   }
 
-                cdo_read_record(streamID, array.data(), &numMissVals);
+                cdo_read_field(streamID, array.data(), &numMissVals);
 
                 auto vdate = cdiDate_get(vDateTime.date);
                 auto vtime = cdiTime_get(vDateTime.time);
@@ -537,7 +523,8 @@ public:
                   }
                 else if (operatorID == OUTPUTXYZ)
                   {
-                    if (tsID == 0 && recID == 0) outputxyz(gridsize, array, missval, nlon, nlat, grid_center_lon, grid_center_lat);
+                    if (tsID == 0 && fieldID == 0)
+                      outputxyz(gridsize, array, missval, nlon, nlat, grid_center_lon, grid_center_lat);
                   }
                 else if (operatorID == OUTPUTARR) { outputarr(dig, gridsize, array); }
                 else
