@@ -62,12 +62,12 @@ check_lonbounds(int ncorner, double *lon_bounds)
       for (int k = 0; k < ncorner; ++k)
         if (lon_bounds[k] < -90.0) lon_bounds[k] += 360.0;
 
-      // printf("XXXX %g %g %g\n", lon_bounds[0], lon_bounds[1], lon_bounds[2]);
+      // printf("OUTPUTCENTER2X %g %g %g\n", lon_bounds[0], lon_bounds[1], lon_bounds[2]);
     }
 }
 
 static void
-make_cyclic(double *array1, double *array2, long nlon, long nlat)
+make_cyclic(const double *array1, double *array2, long nlon, long nlat)
 {
   for (long j = 0; j < nlat; ++j)
     for (long i = 0; i < nlon; ++i)
@@ -113,7 +113,7 @@ output_mer(double levmin, double levmax, const double *cell_corner_lon)
 static const int *
 get_rgb(double value, double missval, const CPT &cpt)
 {
-  if (!DBL_IS_EQUAL(value, missval))
+  if (fp_is_not_equal(value, missval))
     {
       int n;
       for (n = 0; n < cpt.ncolors; ++n)
@@ -125,7 +125,7 @@ get_rgb(double value, double missval, const CPT &cpt)
 }
 
 static void
-output_vrml(long nlon, long nlat, long ngp, const Varray<double> &array, double missval, const CPT &cpt)
+output_vrml(long nlon, long nlat, long ngp, Varray<double> const &array, double missval, const CPT &cpt)
 {
   auto mm = varray_min_max_mv(ngp, array, missval);
   auto dx = 10.0 / nlon;
@@ -173,7 +173,7 @@ output_vrml(long nlon, long nlat, long ngp, const Varray<double> &array, double
         int r = 0, g = 0, b = 0;
         auto val = array[j * nlon + i];
 
-        if (!DBL_IS_EQUAL(val, missval))
+        if (!fp_is_equal(val, missval))
           {
             int n;
             for (n = 0; n < cpt.ncolors; ++n)
@@ -219,7 +219,7 @@ output_vrml(long nlon, long nlat, long ngp, const Varray<double> &array, double
 }
 
 static void
-output_kml(size_t ngp, const Varray<double> &array, int ncorner, const Varray<double> &grid_corner_lat,
+output_kml(size_t ngp, Varray<double> const &array, int ncorner, Varray<double> const &grid_corner_lat,
            Varray<double> &grid_corner_lon, const std::vector<int> &grid_mask, double missval, const CPT &cpt)
 {
   fprintf(stdout, "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n");
@@ -330,8 +330,8 @@ output_kml(size_t ngp, const Varray<double> &array, int ncorner, const Varray<do
 }
 
 static void
-output_vector(long nlon, long nlat, int ninc, const Varray<double> &lon, const Varray<double> &lat, const Varray<double> &uf,
-              const Varray<double> &vf)
+output_vector(long nlon, long nlat, int ninc, Varray<double> const &lon, Varray<double> const &lat, Varray<double> const &uf,
+              Varray<double> const &vf)
 {
   for (long j = 0; j < nlat; j += ninc)
     for (long i = 0; i < nlon; i += ninc)
@@ -353,11 +353,31 @@ output_vector(long nlon, long nlat, int ninc, const Varray<double> &lon, const V
   fprintf(stdout, "#\n");
 }
 
-class ModuleOutputgmt
+class Outputgmt : public Process
 {
-  int varID0;
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Outputgmt",
+    .operators = { { "gmtxyz", OutputgmtHelp },
+                   { "gmtcells", OutputgmtHelp },
+                   { "outputcenter2", OutputgmtHelp },
+                   { "outputcentercpt", OutputgmtHelp },
+                   { "outputboundscpt", OutputgmtHelp },
+                   { "outputvector", OutputgmtHelp },
+                   { "outputtri", OutputgmtHelp },
+                   { "outputvrml", OutputgmtHelp },
+                   { "outputkml", OutputgmtHelp } },
+    .aliases = { { "outputcenter", "gmtxyz" }, { "outputbounds", "gmtcells" } },
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 0, NoRestriction },
+  };
+  inline static RegisterEntry<Outputgmt> registration = RegisterEntry<Outputgmt>(module);
+
+  int GMTXYZ, OUTPUTCENTER2, OUTPUTCENTERCPT, GMTCELLS, OUTPUTBOUNDSCPT, OUTPUTVECTOR, OUTPUTTRI, OUTPUTVRML, OUTPUTKML;
   size_t gridsize2 = 0;
-  size_t nmiss;
+  size_t numMissVals;
   int ninc = 1;
   bool lzon = false, lmer = false, lhov = false;
   Varray<double> grid_center_lat2, grid_center_lon2;
@@ -368,8 +388,6 @@ class ModuleOutputgmt
   CdoStreamID streamID;
   int taxisID;
 
-  int varID = 0;
-
   int operatorID;
 
   int ncorner;
@@ -386,8 +404,6 @@ class ModuleOutputgmt
   bool printHeader;
   bool grid_is_circular;
 
-  int OUTPUTCENTER, OUTPUTCENTER2, OUTPUTCENTERCPT, OUTPUTBOUNDS, OUTPUTBOUNDSCPT, OUTPUTVECTOR, OUTPUTTRI, OUTPUTVRML, OUTPUTKML;
-
   VarList varList;
   Varray<double> array;
   Varray<double> array2;
@@ -404,28 +420,19 @@ class ModuleOutputgmt
   double *plat;
   double *parray;
 
-  void
-  add_operators()
-  {
-    // clang-format off
-    OUTPUTCENTER    = cdo_operator_add("gmtxyz",          0, 0, nullptr);
-    OUTPUTCENTER2   = cdo_operator_add("outputcenter2",   0, 0, nullptr);
-    OUTPUTCENTERCPT = cdo_operator_add("outputcentercpt", 0, 0, nullptr);
-    OUTPUTBOUNDS    = cdo_operator_add("gmtcells",        0, 0, nullptr);
-    OUTPUTBOUNDSCPT = cdo_operator_add("outputboundscpt", 0, 0, nullptr);
-    OUTPUTVECTOR    = cdo_operator_add("outputvector",    0, 0, nullptr);
-    OUTPUTTRI       = cdo_operator_add("outputtri",       0, 0, nullptr);
-    OUTPUTVRML      = cdo_operator_add("outputvrml",      0, 0, nullptr);
-    OUTPUTKML       = cdo_operator_add("outputkml",       0, 0, nullptr);
-    // clang-format on
-  }
-
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-    add_operators();
+    GMTXYZ = module.get_id("gmtxyz");
+    OUTPUTCENTER2 = module.get_id("outputcenter2");
+    OUTPUTCENTERCPT = module.get_id("outputcentercpt");
+    GMTCELLS = module.get_id("gmtcells");
+    OUTPUTBOUNDSCPT = module.get_id("outputboundscpt");
+    OUTPUTVECTOR = module.get_id("outputvector");
+    OUTPUTTRI = module.get_id("outputtri");
+    OUTPUTVRML = module.get_id("outputvrml");
+    OUTPUTKML = module.get_id("outputkml");
 
     operatorID = cdo_operator_id();
 
@@ -439,7 +446,7 @@ public:
         if (ninc < 1) cdo_abort("Increment must be greater than 0!");
       }
 
-    auto needCellCorners = (operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT || operatorID == OUTPUTKML);
+    auto needCellCorners = (operatorID == GMTCELLS || operatorID == OUTPUTBOUNDSCPT || operatorID == OUTPUTKML);
 
     if (operatorID == OUTPUTCENTERCPT || operatorID == OUTPUTBOUNDSCPT || operatorID == OUTPUTKML || operatorID == OUTPUTVRML)
       {
@@ -460,11 +467,12 @@ public:
     auto vlistID = cdo_stream_inq_vlist(streamID);
     taxisID = vlistInqTaxis(vlistID);
 
-    varList_init(varList, vlistID);
+    varList = VarList(vlistID);
 
-    gridID = varList[0].gridID;
-    auto zaxisID = varList[0].zaxisID;
-    missval = varList[0].missval;
+    auto const &var0 = varList.vars[0];
+    gridID = var0.gridID;
+    auto zaxisID = var0.zaxisID;
+    missval = var0.missval;
 
     gridID = generate_full_cell_grid(gridID);
 
@@ -496,13 +504,13 @@ public:
     /*
     if ( lzon || lmer )
       {
-        if ( operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT )
+        if ( operatorID == GMTCELLS || operatorID == OUTPUTBOUNDSCPT )
           cdo_abort("Bounds not available for zonal/meridional data!");
       }
     */
     if (lhov)
       {
-        if (operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT) cdo_abort("Bounds not available hovmoeller data!");
+        if (operatorID == GMTCELLS || operatorID == OUTPUTBOUNDSCPT) cdo_abort("Bounds not available hovmoeller data!");
       }
 
     ncorner = (gridInqType(gridID) == GRID_UNSTRUCTURED) ? gridInqNvertex(gridID) : 4;
@@ -535,7 +543,7 @@ public:
 
         for (long j = 0; j < nlat; ++j)
           {
-            const long ij2 = j * (nlon + 1);
+            long ij2 = j * (nlon + 1);
             grid_center_lon2[ij2 + nlon] += 360;
           }
 
@@ -553,7 +561,7 @@ public:
     if (needCellCorners)
       {
         if (ncorner == 0) cdo_abort("Number of cell corners undefined!");
-        const size_t nalloc = ncorner * gridsize;
+        size_t nalloc = ncorner * gridsize;
         grid_corner_lat.resize(nalloc);
         grid_corner_lon.resize(nalloc);
 
@@ -596,14 +604,15 @@ public:
     if (operatorID == OUTPUTVECTOR) uf.resize(gridsize);
     if (operatorID == OUTPUTVECTOR) vf.resize(gridsize);
   }
+
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID, tsID);
+        if (numFields == 0) break;
 
         auto vDateTime = taxisInqVdatetime(taxisID);
         auto vdateString = date_to_string(vDateTime.date);
@@ -611,6 +620,7 @@ public:
 
         if (tsID == 0 && printHeader)
           {
+            auto const &var0 = varList.vars[0];
             if (operatorID == OUTPUTVRML) printf("#VRML V2.0 utf8\n\n");
 #ifdef VERSION
             fprintf(stdout, "# Generated by CDO version %s\n", VERSION);
@@ -618,10 +628,10 @@ public:
 #endif
             fprintf(stdout, "# Operator = %s\n", cdo_operator_name(operatorID));
             // clang-format off
-          if      (lhov) fprintf(stdout, "# Mode     = hovmoeller\n");
-          else if (lzon) fprintf(stdout, "# Mode     = zonal\n");
-          else if (lmer) fprintf(stdout, "# Mode     = meridional\n");
-          else           fprintf(stdout, "# Mode     = horizonal\n");
+            if      (lhov) fprintf(stdout, "# Mode     = hovmoeller\n");
+            else if (lzon) fprintf(stdout, "# Mode     = zonal\n");
+            else if (lmer) fprintf(stdout, "# Mode     = meridional\n");
+            else           fprintf(stdout, "# Mode     = horizonal\n");
             // clang-format on
 
             if (operatorID == OUTPUTVECTOR) fprintf(stdout, "# Increment = %d\n", ninc);
@@ -629,21 +639,20 @@ public:
             fprintf(stdout, "# Stream = %s\n", cdo_get_stream_name(0));
             fprintf(stdout, "# Date   = %s\n", vdateString.c_str());
             fprintf(stdout, "# Time   = %s\n", vtimeString.c_str());
-            fprintf(stdout, "# Name   = %s\n", varList[0].name.c_str());
-            fprintf(stdout, "# Code   = %d\n", varList[0].code);
+            fprintf(stdout, "# Name   = %s\n", var0.name.c_str());
+            fprintf(stdout, "# Code   = %d\n", var0.code);
           }
 
-        varID0 = varID;
+        int varID0 = 0;
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int levelID;
-            cdo_inq_record(streamID, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID);
 
             if (varID != varID0) continue;
-            if (recID > 0 && !lzon && !lmer) continue;
+            if (fieldID > 0 && !lzon && !lmer) continue;
 
-            cdo_read_record(streamID, array.data(), &nmiss);
+            cdo_read_field(streamID, array.data(), &numMissVals);
 
             if (operatorID == OUTPUTCENTER2 && grid_is_circular) make_cyclic(array.data(), array2.data(), nlon, nlat);
 
@@ -654,7 +663,7 @@ public:
 
             if (printHeader) fprintf(stdout, "#\n");
 
-            if (operatorID == OUTPUTCENTER || operatorID == OUTPUTCENTER2 || operatorID == OUTPUTCENTERCPT)
+            if (operatorID == GMTXYZ || operatorID == OUTPUTCENTER2 || operatorID == OUTPUTCENTERCPT)
               {
                 if (Options::cdoVerbose)
                   {
@@ -669,26 +678,28 @@ public:
                   {
                     if (grid_mask.size() && grid_mask[i] == 0) continue;
 
-                    if (operatorID == OUTPUTCENTER)
+                    auto lon = grid_center_lon[i];
+                    auto lat = grid_center_lat[i];
+                    if (operatorID == GMTXYZ)
                       {
                         if (lzon)
-                          fprintf(stdout, " %g  %g  %g\n", grid_center_lat[i], level, array[i]);
+                          fprintf(stdout, " %g  %g  %g\n", lat, level, array[i]);
                         else if (lmer)
-                          fprintf(stdout, " %g  %g  %g\n", grid_center_lon[i], level, array[i]);
+                          fprintf(stdout, " %g  %g  %g\n", lon, level, array[i]);
                         else if (lhov)
-                          fprintf(stdout, " %d  %g  %g\n", tsID + 1, grid_center_lat[i], array[i]);
+                          fprintf(stdout, " %d  %g  %g\n", tsID + 1, lat, array[i]);
                         else
-                          fprintf(stdout, " %g  %g  %g\n", grid_center_lon[i], grid_center_lat[i], array[i]);
+                          fprintf(stdout, " %g  %g  %g\n", lon, lat, array[i]);
                       }
                     else if (operatorID == OUTPUTCENTER2) { fprintf(stdout, " %g  %g  %g\n", plon[i], plat[i], parray[i]); }
                     else
                       {
                         if (lzon)
-                          fprintf(stdout, " %g  %g  %g\n", grid_center_lat[i], level, array[i]);
+                          fprintf(stdout, " %g  %g  %g\n", lat, level, array[i]);
                         else if (lmer)
-                          fprintf(stdout, " %g  %g  %g\n", grid_center_lon[i], level, array[i]);
+                          fprintf(stdout, " %g  %g  %g\n", lon, level, array[i]);
                         else
-                          fprintf(stdout, " %g  %g  %g  %g\n", grid_center_lon[i], grid_center_lat[i], array[i], array[i]);
+                          fprintf(stdout, " %g  %g  %g  %g\n", lon, lat, array[i], array[i]);
                       }
                   }
                 fprintf(stdout, "#\n");
@@ -697,13 +708,12 @@ public:
               {
                 if (gridInqType(gridID) != GRID_CURVILINEAR) cdo_abort("Unsupported grid!");
 
-                const long mlon = nlon - 1;
+                long mlon = nlon - 1;
                 // if ( gridIsCircular(gridID) ) mlon = nlon;
                 for (long j = 0; j < nlat - 1; ++j)
                   for (long i = 0; i < mlon; ++i)
                     {
-                      int ip1 = i + 1;
-                      if (i == nlon - 1) ip1 = 0;
+                      int ip1 = (i == nlon - 1) ? 0 : i + 1;
                       int c1 = (j) *nlon + ip1;
                       int c2 = (j) *nlon + i;
                       int c3 = (j + 1) * nlon + i;
@@ -720,18 +730,18 @@ public:
               }
             else if (operatorID == OUTPUTVECTOR)
               {
-                if (nrecs < 2) cdo_abort("Too few fields!");
+                if (numFields < 2) cdo_abort("Too few fields!");
 
                 varray_copy(gridsize, array, uf);
-                cdo_inq_record(streamID, &varID, &levelID);
-                cdo_read_record(streamID, vf.data(), &nmiss);
+                (void) cdo_inq_field(streamID);
+                cdo_read_field(streamID, vf.data(), &numMissVals);
 
                 output_vector(nlon, nlat, ninc, grid_center_lon, grid_center_lat, uf, vf);
 
                 break;
               }
             else if (operatorID == OUTPUTVRML) { output_vrml(nlon, nlat, gridsize, array, missval, cpt); }
-            else if (operatorID == OUTPUTBOUNDS || operatorID == OUTPUTBOUNDSCPT)
+            else if (operatorID == GMTCELLS || operatorID == OUTPUTBOUNDSCPT)
               {
                 if (Options::cdoVerbose)
                   {
@@ -750,7 +760,7 @@ public:
                   {
                     if (grid_mask.size() && grid_mask[i] == 0) continue;
 
-                    if (!DBL_IS_EQUAL(array[i], missval))
+                    if (!fp_is_equal(array[i], missval))
                       fprintf(stdout, "> -Z%g", array[i]);
                     else
                       fprintf(stdout, "> -ZNaN");
@@ -770,7 +780,7 @@ public:
                       {
                         const double *lon_bounds = grid_corner_lon.data() + i * ncorner;
                         const double *lat_bounds = grid_corner_lat.data() + i * ncorner;
-                        const int ncorner_new = check_ncorner(ncorner, lon_bounds, lat_bounds);
+                        int ncorner_new = check_ncorner(ncorner, lon_bounds, lat_bounds);
 
                         for (int ic = 0; ic < ncorner_new; ++ic) fprintf(stdout, "   %g  %g\n", lon_bounds[ic], lat_bounds[ic]);
                         fprintf(stdout, "   %g  %g\n", lon_bounds[0], lat_bounds[0]);
@@ -785,21 +795,10 @@ public:
         tsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID);
-
-    cdo_finish();
   }
 };
-
-void *
-Outputgmt(void *process)
-{
-  ModuleOutputgmt outputgmt;
-  outputgmt.init(process);
-  outputgmt.run();
-  outputgmt.close();
-  return nullptr;
-}
