@@ -34,14 +34,13 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   int operatorID;
 
   size_t imiss = 0;
   double arrmin, arrmax, arrmean;
-  size_t gridsizemax;
 
   VarList varList1;
 
@@ -71,21 +70,19 @@ public:
     cdo_def_vlist(streamID2, vlistID2);
 
     varList1 = VarList(vlistID1);
-
-    gridsizemax = vlistGridsizeMax(vlistID1);
   }
 
   void
   run() override
   {
-    Varray<double> array1(gridsizemax), array2(gridsizemax);
+    Varray<double> array1(varList1.gridsizeMax()), array2(varList1.gridsizeMax());
 
     int indg = 0;
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         auto vDateTime = taxisInqVdatetime(taxisID1);
         auto vdateString = date_to_string(vDateTime.date);
@@ -94,9 +91,9 @@ public:
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            if (tsID == 0 && recID == 0)
+            if (tsID == 0 && fieldID == 0)
               {
                 if (operatorID == PINFOV)
                   fprintf(stdout,
@@ -106,12 +103,11 @@ public:
                           "   Rec :       Date  Time    Code  Level    Size    Miss :     Minimum        Mean     Maximum\n");
               }
 
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             size_t numMissVals;
-            cdo_read_record(streamID1, array1.data(), &numMissVals);
+            cdo_read_field(streamID1, array1.data(), &numMissVals);
 
-            const auto &var = varList1.vars[varID];
+            auto const &var = varList1.vars[varID];
             indg += 1;
             auto gridsize = var.gridsize;
 
@@ -158,8 +154,8 @@ public:
 
             varray_copy(gridsize, array1, array2);
 
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, array2.data(), numMissVals);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, array2.data(), numMissVals);
           }
 
         tsID++;
