@@ -14,26 +14,33 @@
 #include "pmlist.h"
 #include "cdo_default_values.h"
 
-class ModuleQuery
+class Query : public Process
 {
-  int streamID1; //QueryStream
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Query",
+    .operators = { { "query", 0, 0, "queryentries" } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, FilesOnly },
+  };
+  inline static RegisterEntry<Query> registration = RegisterEntry<Query>(module);
+
+  int streamID1;  // QueryStream
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  Field field;
   VarList varList1;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    cdo_operator_add("query", 0, 0, "query entries");
-
-    //auto dataIsUnchanged = data_is_unchanged();
+    // auto dataIsUnchanged = data_is_unchanged();
 
     auto operatorID = cdo_operator_id();
 
@@ -47,7 +54,7 @@ public:
     PMList pmlist;
     KVList kvlist;
     kvlist.name = cdo_module_name();
-    if (kvlist.parse_arguments(natts, cdo_get_oper_argv()) != 0) cdo_abort("Parse error!");
+    if (kvlist.parse_arguments(cdo_get_oper_argv()) != 0) cdo_abort("Parse error!");
     if (Options::cdoVerbose) kvlist.print();
 
     auto pkvlist = &kvlist;
@@ -71,7 +78,7 @@ public:
     set_query_parameter(*pkvlist, query);
     if (Options::cdoVerbose) cdiQueryPrint(query);
 
-     streamID1 = streamOpenReadQuery(cdo_get_stream_name(0), query);
+    streamID1 = streamOpenReadQuery(cdo_get_stream_name(0), query);
     if (streamID1 < 0) cdi_open_error(streamID1, "Open failed on >%s<", cdo_get_stream_name(0));
 
     cdiQueryPrintEntriesNotFound(query);
@@ -83,47 +90,49 @@ public:
     auto vlistID1 = streamInqVlist(streamID1);
     auto vlistID2 = vlistDuplicate(vlistID1);
 
-     taxisID1 = vlistInqTaxis(vlistID1);
-     taxisID2 = taxisDuplicate(taxisID1);
+    taxisID1 = vlistInqTaxis(vlistID1);
+    taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
 
-     streamID2 = cdo_open_write(1);
+    streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
 
-    varList_init(varList1, vlistID1);
+    varList1 = VarList(vlistID1);
   }
 
   void
-  run()
+  run() override
   {
+    Field field;
+
     int tsID = 0;
     while (true)
       {
-        auto nrecs = streamInqTimestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = streamInqTimestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
             int varID, levelID;
-            streamInqRecord(streamID1, &varID, &levelID);
-            cdo_def_record(streamID2, varID, levelID);
+            streamInqField(streamID1, &varID, &levelID);
+            cdo_def_field(streamID2, varID, levelID);
             /*
             if (dataIsUnchanged)
               {
-                streamCopyRecord(streamID2, streamID1);
+                streamCopyField(streamID2, streamID1);
               }
             else
             */
             {
-              field.init(varList1[varID]);
+              field.init(varList1.vars[varID]);
               if (field.memType == MemType::Float)
-                streamReadRecordF(streamID1, field.vec_f.data(), &field.nmiss);
+                streamReadFieldF(streamID1, field.vec_f.data(), &field.numMissVals);
               else
-                streamReadRecord(streamID1, field.vec_d.data(), &field.nmiss);
-              cdo_write_record(streamID2, field);
+                streamReadField(streamID1, field.vec_d.data(), &field.numMissVals);
+              cdo_write_field(streamID2, field);
             }
           }
 
@@ -132,22 +141,9 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     streamClose(streamID1);
     cdo_stream_close(streamID2);
-
-    cdo_finish();
   }
 };
-
-void *
-Query(void *process)
-{
-  ModuleQuery query;
-  query.init(process);
-  query.run();
-  query.close();
-
-  return nullptr;
-}
