@@ -26,7 +26,7 @@
 #include "cdi_lockedIO.h"
 
 static void
-set_missval(const Varray<int8_t> &imiss, double missval, auto &sarray)
+set_missval(Vmask const &imiss, double missval, auto &sarray)
 {
   auto n = imiss.size();
   if (n > 0)
@@ -90,7 +90,7 @@ ncctop(double cptop, long nlev, long nlevp1, const double *vct_a, const double *
 }
 
 static Varray<double>
-vct_from_file(const std::string &filename)
+vct_from_file(std::string const &filename)
 {
   constexpr int maxvct = 8192;
   Varray<double> vct;
@@ -118,7 +118,7 @@ vct_from_file(const std::string &filename)
 
       vct[i + maxvct / 2] = strtod(pline, &pline);
 
-      free(lineCpy);
+      std::free(lineCpy);
 
       i++;
     }
@@ -137,7 +137,7 @@ vct_from_file(const std::string &filename)
 
 template <typename T>
 static void
-vertSum(Varray<double> &sum, const Varray<T> &var3d, size_t gridsize, size_t nlevels)
+vertSum(Varray<double> &sum, Varray<T> const &var3d, size_t gridsize, size_t nlevels)
 {
   for (size_t i = 0; i < gridsize; ++i) sum[i] = 0;
 
@@ -147,7 +147,7 @@ vertSum(Varray<double> &sum, const Varray<T> &var3d, size_t gridsize, size_t nle
 
 template <typename T>
 static void
-vertSumw(Varray<double> &sum, const Varray<T> &var3d, size_t gridsize, size_t nlevels, const Varray<double> &deltap)
+vertSumw(Varray<double> &sum, Varray<T> const &var3d, size_t gridsize, size_t nlevels, Varray<double> const &deltap)
 {
   for (size_t i = 0; i < gridsize; ++i) sum[i] = 0;
 
@@ -157,9 +157,9 @@ vertSumw(Varray<double> &sum, const Varray<T> &var3d, size_t gridsize, size_t nl
 
 template <typename T>
 void
-field_copy_array(size_t len, const Field &field, T *array)
+field_copy_array(size_t len, Field const &field, T *array)
 {
-  auto func = [&](const auto &v) {
+  auto func = [&](auto const &v) {
     for (size_t i = 0; i < len; ++i) array[i] = v[i];
   };
   field_operation(func, field);
@@ -198,15 +198,15 @@ public:
   size_t numMissValsout = 0;
   bool lfis2 = false;
   int varids[MaxVars3D];
-  Varray<int8_t> imiss;
+  Vmask imiss;
   double missval = 0.0;
   double cptop = 0.0;  // min. pressure level for cond.
 
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   int operatorID;
   int presID;
@@ -258,7 +258,7 @@ public:
   {
     if (ltq)
       {
-        int varID = varIDs.tempID;
+        int varID = varIDs.taID;
         for (int levelID = 0; levelID < varList2.vars[varID].nlevels; ++levelID)
           {
             std::span sarray(&t2[gridsize * levelID], gridsize);
@@ -268,15 +268,15 @@ public:
               cdo_warning("Output temperature at level %d out of range (min=%g max=%g)!", levelID + 1, mm.min, mm.max);
 
             set_missval(imiss, missval, sarray);
-            cdo_def_record(streamID2, varID, levelID);
+            cdo_def_field(streamID2, varID, levelID);
 
             if (memType == MemType::Float)
-              cdo_write_record_f(streamID2, (float *) sarray.data(), numMissValsout);
+              cdo_write_field_f(streamID2, (float *) sarray.data(), numMissValsout);
             else
-              cdo_write_record(streamID2, (double *) sarray.data(), numMissValsout);
+              cdo_write_field(streamID2, (double *) sarray.data(), numMissValsout);
           }
 
-        varID = varIDs.humID;
+        varID = varIDs.husID;
         for (int levelID = 0; levelID < varList2.vars[varID].nlevels; ++levelID)
           {
             std::span sarray(&q2[gridsize * levelID], gridsize);
@@ -291,12 +291,12 @@ public:
               cdo_warning("Output humidity at level %d out of range (min=%g max=%g)!", levelID + 1, mm.min, mm.max);
 
             set_missval(imiss, missval, sarray);
-            cdo_def_record(streamID2, varID, levelID);
+            cdo_def_field(streamID2, varID, levelID);
 
             if (memType == MemType::Float)
-              cdo_write_record_f(streamID2, (float *) sarray.data(), numMissValsout);
+              cdo_write_field_f(streamID2, (float *) sarray.data(), numMissValsout);
             else
-              cdo_write_record(streamID2, (double *) sarray.data(), numMissValsout);
+              cdo_write_field(streamID2, (double *) sarray.data(), numMissValsout);
           }
       }
 
@@ -340,33 +340,33 @@ public:
               for (size_t i = 0; i < gridsize; ++i) sarray[i] *= sum1[i] / sum2[i];
 
             set_missval(imiss, missval, sarray);
-            cdo_def_record(streamID2, varID, levelID);
+            cdo_def_field(streamID2, varID, levelID);
 
             if (memType == MemType::Float)
-              cdo_write_record_f(streamID2, (float *) sarray.data(), numMissValsout);
+              cdo_write_field_f(streamID2, (float *) sarray.data(), numMissValsout);
             else
-              cdo_write_record(streamID2, (double *) sarray.data(), numMissValsout);
+              cdo_write_field(streamID2, (double *) sarray.data(), numMissValsout);
           }
       }
   }
 
   void
-  read_fis(const std::string &fileName)
+  read_fis(std::string const &fileName)
   {
     auto streamID = stream_open_read_locked(fileName.c_str());
     auto vlistID = streamInqVlist(streamID);
     VarList varList(vlistID);
 
     int varID, levelID;
-    streamInqRecord(streamID, &varID, &levelID);
+    streamInqField(streamID, &varID, &levelID);
 
-    const auto &var = varList.vars[0];
+    auto const &var = varList.vars[0];
 
     nfis2gp = var.gridsize;
     fis2.resize(nfis2gp);
 
     size_t numMissVals;
-    streamReadRecord(streamID, fis2.data(), &numMissVals);
+    streamReadField(streamID, fis2.data(), &numMissVals);
 
     if (numMissVals)
       {
@@ -458,12 +458,12 @@ public:
 
     int numHybridLevels = 0, numHalfLevels1 = 0;
     vct1 = vlist_read_vct(vlistID1, zaxisID_ML, numHybridLevels, numFullLevels1, numHalfLevels1);
-    const int nvct1 = vct1.size();
+    int nvct1 = vct1.size();
 
     vlist_change_hybrid_zaxis(vlistID1, vlistID2, zaxisID_ML, zaxisID2);
 
-    auto nzaxis = vlistNumZaxis(vlistID1);
-    for (int i = 0; i < nzaxis; ++i)
+    auto numZaxes = varList1.numZaxes();
+    for (int i = 0; i < numZaxes; ++i)
       {
         auto zaxisID = vlistZaxis(vlistID1, i);
         auto nlevels = zaxisInqSize(zaxisID);
@@ -490,17 +490,17 @@ public:
       {
         cdo_print("Found:");
         // clang-format off
-        if (-1 != varIDs.tempID)    cdo_print("  %s", var_stdname(air_temperature));
+        if (-1 != varIDs.taID)    cdo_print("  %s", var_stdname(air_temperature));
         if (-1 != varIDs.psID)      cdo_print("  %s", var_stdname(surface_air_pressure));
         if (-1 != varIDs.lnpsID)    cdo_print("  LOG(%s)", var_stdname(surface_air_pressure));
         if (-1 != varIDs.sgeopotID) cdo_print("  %s", var_stdname(surface_geopotential));
-        if (-1 != varIDs.humID)     cdo_print("  %s", var_stdname(specific_humidity));
+        if (-1 != varIDs.husID)     cdo_print("  %s", var_stdname(specific_humidity));
         // clang-format on
       }
 
     for (int varID = 0; varID < numVars; ++varID)
       {
-        const auto &var = varList1.vars[varID];
+        auto const &var = varList1.vars[varID];
 
         if (var.gridType == GRID_SPECTRAL && var.zaxisType == ZAXIS_HYBRID) cdo_abort("Spectral data on model level unsupported!");
 
@@ -508,21 +508,21 @@ public:
 
         if (var.zaxisType == ZAXIS_HYBRID && zaxisID_ML != -1 && var.nlevels == numFullLevels1)
           {
-            if (!(varID == varIDs.tempID || varID == varIDs.humID)) varids[nvars3D++] = varID;
+            if (!(varID == varIDs.taID || varID == varIDs.husID)) varids[nvars3D++] = varID;
           }
         else
           {
-            if (varID == varIDs.tempID) varIDs.tempID = -1;
-            if (varID == varIDs.humID) varIDs.humID = -1;
+            if (varID == varIDs.taID) varIDs.taID = -1;
+            if (varID == varIDs.husID) varIDs.husID = -1;
           }
       }
 
-    ltq = (varIDs.tempID != -1 && varIDs.humID != -1);
+    ltq = (varIDs.taID != -1 && varIDs.husID != -1);
 
     if (!ltq)
       {
-        if (varIDs.tempID != -1) cdo_abort("Temperature without humidity unsupported!");
-        if (varIDs.humID != -1) cdo_abort("Humidity without temperature unsupported!");
+        if (varIDs.taID != -1) cdo_abort("Temperature without humidity unsupported!");
+        if (varIDs.husID != -1) cdo_abort("Humidity without temperature unsupported!");
       }
 
     if (operatorID == REMAPETA_S || operatorID == REMAPETA_Z)
@@ -564,7 +564,7 @@ public:
 
     if (zaxisID_ML != -1 && varIDs.sgeopotID == -1)
       {
-        ranges::fill(fis1, 0.0);
+        std::ranges::fill(fis1, 0.0);
         if (ltq) cdo_warning("%s not found - set to zero!", var_stdname(surface_geopotential));
       }
 
@@ -615,19 +615,18 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            const auto &var = varList1.vars[varID];
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            auto const &var = varList1.vars[varID];
             field.init(var);
-            cdo_read_record(streamID1, field);
+            cdo_read_field(streamID1, field);
 
             if (zaxisID_ML != -1)
               {
@@ -647,12 +646,12 @@ public:
                     else if (varIDs.psID != -1)
                       field_copy_array(gridsize, field, ps1.data());
                   }
-                else if (ltq && varID == varIDs.tempID)
+                else if (ltq && varID == varIDs.taID)
                   {
                     if (memType == MemType::Float) { field_copy_array(gridsize, field, &f_t1[offset]); }
                     else { field_copy_array(gridsize, field, &d_t1[offset]); }
                   }
-                else if (ltq && varID == varIDs.humID)
+                else if (ltq && varID == varIDs.husID)
                   {
                     if (memType == MemType::Float) { field_copy_array(gridsize, field, &f_q1[offset]); }
                     else { field_copy_array(gridsize, field, &d_q1[offset]); }
@@ -671,14 +670,14 @@ public:
                   }
                 else
                   {
-                    cdo_def_record(streamID2, varID, levelID);
-                    cdo_write_record(streamID2, field);
+                    cdo_def_field(streamID2, varID, levelID);
+                    cdo_write_field(streamID2, field);
                   }
               }
             else
               {
-                cdo_def_record(streamID2, varID, levelID);
-                cdo_write_record(streamID2, field);
+                cdo_def_field(streamID2, varID, levelID);
+                cdo_write_field(streamID2, field);
               }
           }
 
@@ -698,8 +697,8 @@ public:
 
         if (ltq)
           {
-            int varID = varIDs.tempID;
-            const auto &var1 = varList1.vars[varID];
+            int varID = varIDs.taID;
+            auto const &var1 = varList1.vars[varID];
             for (int levelID = 0; levelID < var1.nlevels; ++levelID)
               {
                 auto offset = gridsize * levelID;
@@ -712,7 +711,7 @@ public:
                   cdo_warning("Input temperature at level %d out of range (min=%g max=%g)!", levelID + 1, mm.min, mm.max);
               }
 
-            varID = varIDs.humID;
+            varID = varIDs.husID;
             for (int levelID = 0; levelID < var1.nlevels; ++levelID)
               {
                 auto offset = gridsize * levelID;
@@ -748,8 +747,8 @@ public:
             int varID = varIDs.sgeopotID;
             int levelID = 0;
             set_missval(imiss, missval, fis2);
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, fis2.data(), numMissValsout);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, fis2.data(), numMissValsout);
           }
 
         if (zaxisID_ML != -1 && varIDs.lnpsID != -1)
@@ -760,8 +759,8 @@ public:
             int varID = presID;
             int levelID = 0;
             set_missval(imiss, missval, ps2);
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, ps2.data(), numMissValsout);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, ps2.data(), numMissValsout);
           }
 
         if (memType == MemType::Float) { do_work(nctop, f_t2, f_q2, f_vars1, f_vars2); }
