@@ -12,10 +12,11 @@
 
 #include <cdi.h>
 
+#include "cdo_math.h"
 #include "cdo_timer.h"
 #include "process_int.h"
 #include "griddes.h"
-#include "grid_point_search.h"
+#include "grid_pointsearch.h"
 #include <mpim_grid.h>
 #include "cdo_omp.h"
 #include "verifygrid.h"
@@ -27,7 +28,7 @@
 #ifdef USE_YAC
 extern "C"
 {
-#include "lib/yac/geometry.h"
+#include "lib/yac/src/geometry.h"
 }
 #endif
 #endif
@@ -158,7 +159,7 @@ read_ybounds_reg2d(int gridID, std::vector<double> &ybounds2d)
       std::vector<double> yvals(nlat);
       gridInqYvals(gridID, yvals.data());
       grid_gen_bounds(nlat, yvals, ybounds2d);
-      grid_check_lat_borders(2 * nlat, ybounds2d.data());
+      grid_check_lat_borders(2 * nlat, ybounds2d);
     }
 
   for (size_t i = 0; i < 2 * nlat; ++i) ybounds2d[i] *= DEG2RAD;
@@ -189,10 +190,10 @@ is_zonal_grid(int gridID, const std::vector<double> &xbounds2d, const std::vecto
 }
 
 static double
-calc_maxdist(size_t i, size_t nv, double plon, double plat, const Varray<double> &xbounds, const Varray<double> &ybounds)
+calc_maxdist(size_t i, size_t nv, PointLonLat const &pointLL, Varray<double> const &xbounds, Varray<double> const &ybounds)
 {
   double p1[3], p2[3];
-  gcLLtoXYZ(plon, plat, p1);
+  gcLLtoXYZ(pointLL.get_lon(), pointLL.get_lat(), p1);
 
   double maxdist = 0.0;
   for (size_t k = 0; k < nv; ++k)
@@ -200,7 +201,7 @@ calc_maxdist(size_t i, size_t nv, double plon, double plat, const Varray<double>
       auto lons = &xbounds[nv * i];
       auto lats = &ybounds[nv * i];
       gcLLtoXYZ(lons[k], lats[k], p2);
-      auto sdist = squareDistance(p1, p2);
+      auto sdist = cdo::sqr_distance(p1, p2);
       maxdist = std::max(maxdist, sdist);
     }
 
@@ -208,11 +209,11 @@ calc_maxdist(size_t i, size_t nv, double plon, double plat, const Varray<double>
 }
 
 static double
-calc_maxdist_rec2d(size_t i, size_t nlon, double plon, double plat, const std::vector<double> &xbounds,
+calc_maxdist_rec2d(size_t i, size_t nlon, PointLonLat const &pointLL, const std::vector<double> &xbounds,
                    const std::vector<double> &ybounds)
 {
   double p1[3], p2[3];
-  gcLLtoXYZ(plon, plat, p1);
+  gcLLtoXYZ(pointLL.get_lon(), pointLL.get_lat(), p1);
 
   constexpr int nv = 4;
   double lons[nv], lats[nv];
@@ -231,7 +232,7 @@ calc_maxdist_rec2d(size_t i, size_t nlon, double plon, double plat, const std::v
   for (size_t k = 0; k < nv; ++k)
     {
       gcLLtoXYZ(lons[k], lats[k], p2);
-      auto sdist = squareDistance(p1, p2);
+      auto sdist = cdo::sqr_distance(p1, p2);
       maxdist = std::max(maxdist, sdist);
     }
 
@@ -240,9 +241,9 @@ calc_maxdist_rec2d(size_t i, size_t nlon, double plon, double plat, const std::v
 
 #ifdef USE_YAC
 static size_t
-find_points_yac(std::vector<int8_t> &vmask, size_t cell_no, size_t ncorner, size_t numIndices, Varray<size_t> &indices,
-                const Varray<double> &xvals1, const Varray<double> &yvals1, const Varray<double> &xbounds,
-                const Varray<double> &ybounds)
+find_points_yac(Vmask &vmask, size_t cell_no, size_t ncorner, size_t numIndices, Varray<size_t> &indices,
+                Varray<double> const &xvals1, Varray<double> const &yvals1, Varray<double> const &xbounds,
+                Varray<double> const &ybounds)
 {
 #ifndef TESTIMPL
   cdo_abort("Internal error: find_points() not implemented!");
@@ -296,9 +297,8 @@ find_points_yac(std::vector<int8_t> &vmask, size_t cell_no, size_t ncorner, size
 #endif
 
 static size_t
-find_points(std::vector<int8_t> &vmask, size_t cell_no, size_t ncorner, size_t numIndices, Varray<size_t> &indices,
-            const Varray<double> &xvals1, const Varray<double> &yvals1, const Varray<double> &xbounds,
-            const Varray<double> &ybounds)
+find_points(Vmask &vmask, size_t cell_no, size_t ncorner, size_t numIndices, Varray<size_t> &indices, Varray<double> const &xvals1,
+            Varray<double> const &yvals1, Varray<double> const &xbounds, Varray<double> const &ybounds)
 {
 #ifndef TESTIMPL
   cdo_abort("Internal error: find_points() not implemented!");
@@ -369,9 +369,8 @@ find_points(std::vector<int8_t> &vmask, size_t cell_no, size_t ncorner, size_t n
 }
 
 static size_t
-find_points_rec2d(std::vector<int8_t> &vmask, size_t i, size_t nlon2, size_t numIndices, Varray<size_t> &indices,
-                  const Varray<double> &xvals1, const Varray<double> &yvals1, const std::vector<double> &xbounds2d,
-                  const std::vector<double> &ybounds2d)
+find_points_rec2d(Vmask &vmask, size_t i, size_t nlon2, size_t numIndices, Varray<size_t> &indices, Varray<double> const &xvals1,
+                  Varray<double> const &yvals1, const std::vector<double> &xbounds2d, const std::vector<double> &ybounds2d)
 {
   auto iy = i / nlon2;
   auto ix = i - iy * nlon2;
@@ -426,7 +425,7 @@ get_bin_num(double lon1, double lat1, size_t numBins, const std::vector<double>
 }
 
 static void
-check_vmask(const std::vector<int8_t> &vmask)
+check_vmask(Vmask const &vmask)
 {
   constexpr int maxVals = 128;
   size_t vm[maxVals] = { 0 };
@@ -510,18 +509,18 @@ gen_mapdata(int gridID1, int gridID2)
     {
       cdo::timer timer;
 
-      GridPointSearch gps;
-      grid_point_search_create(gps, xvals1, yvals1, true);
+      GridPointsearch gps;
+      grid_pointsearch_create_unstruct(gps, xvals1, yvals1, true);
 
       if (Options::cdoVerbose) cdo_print("Point search created: %.2f seconds", timer.elapsed());
 
       timer.reset();
 
-      auto ndistMax = gridsize1;
-      if (gridsize1 > 1000000) ndistMax /= 4;
-      std::vector<int8_t> vmask(gridsize1, 0);
-      Varray2D<size_t> indices_2D(Threading::ompNumThreads, Varray<size_t>(ndistMax));
-      Varray2D<double> dist_2D(Threading::ompNumThreads, Varray<double>(ndistMax));
+      auto maxIndex = gridsize1;
+      if (gridsize1 > 1000000) maxIndex /= 4;
+      Vmask vmask(gridsize1, 0);
+      Varray2D<size_t> indices_2D(Threading::ompNumMaxThreads, Varray<size_t>(maxIndex));
+      Varray2D<double> dist_2D(Threading::ompNumMaxThreads, Varray<double>(maxIndex));
 
       StatInfo statInfo;
 
@@ -534,16 +533,16 @@ gen_mapdata(int gridID1, int gridID2)
           auto &indices = indices_2D[ompthID];
           auto &dist = dist_2D[ompthID];
 
-          auto lon2 = xvals2[i];
-          auto lat2 = yvals2[i];
+          PointLonLat pointLL2{ xvals2[i], yvals2[i] };
 
-          auto maxdist = grid2IsReg2d ? calc_maxdist_rec2d(i, nlon2, lon2, lat2, xbounds2d, ybounds2d)
-                                      : calc_maxdist(i, nv, lon2, lat2, xbounds, ybounds);
+          auto searchRadius = grid2IsReg2d ? calc_maxdist_rec2d(i, nlon2, pointLL2, xbounds2d, ybounds2d)
+                                           : calc_maxdist(i, nv, pointLL2, xbounds, ybounds);
 
-          if (maxdist < 2.0) { maxdist = 1.01 * std::sqrt(maxdist); }
+          if (searchRadius < 2.0) { searchRadius = 1.01 * std::sqrt(searchRadius); }
           else { cdo_abort("Search radius of target grid cell[%zu] > 90 degrees!", i + 1); }
 
-          auto numIndices = grid_point_search_distance_qnearest(gps, maxdist, lon2, lat2, ndistMax, indices, dist);
+          gps.set_radius(searchRadius);
+          auto numIndices = grid_pointsearch_qnearest(gps, pointLL2, maxIndex, indices.data(), dist.data());
           // printf("%zu numIndices %zu\n", i+1, numIndices);
 
           auto nvalues = grid2IsReg2d
@@ -561,17 +560,13 @@ gen_mapdata(int gridID1, int gridID2)
             }
 
           // if (Options::cdoVerbose) printf("%zu numIndices %zu nvalues %zu  maxdist %g\n", i+1, numIndices, nvalues, maxdist);
-          if (Options::cdoVerbose && Threading::ompNumThreads == 1) statInfo.add(numIndices, nvalues, maxdist);
+          if (Options::cdoVerbose && Threading::ompNumMaxThreads == 1) statInfo.add(numIndices, nvalues, searchRadius);
         }
 
-      if (Options::cdoVerbose && Threading::ompNumThreads == 1) statInfo.print();
-
+      if (Options::cdoVerbose && Threading::ompNumMaxThreads == 1) statInfo.print();
       if (Options::cdoVerbose) check_vmask(vmask);
-
       if (Options::cdoVerbose) cdo_print("Point search qnearest: %.2f seconds", timer.elapsed());
 
-      grid_point_search_delete(gps);
-
       if (gridIDdestroy != -1) gridDestroy(gridIDdestroy);
     }
 
@@ -581,7 +576,7 @@ gen_mapdata(int gridID1, int gridID2)
 template <typename T>
 static T
 remap_kernel(int operfunc, const Varray<size_t> &indices, size_t &numMissVals2, Field &field, Varray<T> &fieldvec,
-             const Varray<T> &vec1, double mv)
+             Varray<T> const &vec1, double mv)
 {
   T missval = mv;
   T value;
@@ -611,9 +606,9 @@ remap_kernel(int operfunc, const Varray<size_t> &indices, size_t &numMissVals2,
 }
 
 static void
-remap_field(const Varray2D<size_t> &mapdata, const Field &field1, Field &field2, int operfunc)
+remap_field(const Varray2D<size_t> &mapdata, Field const &field1, Field &field2, int operfunc)
 {
-  std::vector<Field> fields(Threading::ompNumThreads);
+  std::vector<Field> fields(Threading::ompNumMaxThreads);
 
   auto gridsize2 = gridInqSize(field2.grid);
   cdo::timer timer;
@@ -624,7 +619,7 @@ remap_field(const Varray2D<size_t> &mapdata, const Field &field1, Field &field2,
 #endif
   for (size_t i = 0; i < gridsize2; ++i)
     {
-      const auto &indices = mapdata[i];
+      auto const &indices = mapdata[i];
       auto nvalues = indices.size();
       auto missval = field1.missval;
 
@@ -680,8 +675,8 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   VarList varList1;
   VarList varList2;
@@ -724,9 +719,9 @@ public:
     taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
 
-    auto ngrids = vlistNumGrids(vlistID1);
+    auto numGrids = vlistNumGrids(vlistID1);
     auto gridID1 = vlistGrid(vlistID1, 0);
-    for (int index = 0; index < ngrids; ++index)
+    for (int index = 0; index < numGrids; ++index)
       {
         if (index > 0) cdo_abort("Too many different grids!");
 
@@ -757,24 +752,24 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             field1.init(varList1.vars[varID]);
-            cdo_read_record(streamID1, field1);
+            cdo_read_field(streamID1, field1);
 
             field2.init(varList2.vars[varID]);
 
             remap_field(mapdata, field1, field2, operfunc);
 
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, field2);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, field2);
           }
         tsID++;
       }
