@@ -8,12 +8,12 @@
 /*
    This module contains the following operators:
 
-      Remapweights  gencon          Generate YAC first order conservative remap weights
-      Remapweights  genycon2        Generate YAC second order conservative remap weights
       Remapweights  genbil          Generate bilinear interpolation weights
       Remapweights  genbic          Generate bicubic interpolation weights
-      Remapweights  gendis          Generate distance-weighted averaging weights
+      Remapweights  genknn          Generate k-nearest neighbor weights
       Remapweights  gennn           Generate nearest neighbor weights
+      Remapweights  gendis          Generate distance-weighted averaging weights
+      Remapweights  gencon          Generate YAC first order conservative remap weights
       Remapweights  genlaf          Generate largest area fraction weights
 */
 
@@ -31,8 +31,69 @@
 #include "util_string.h"
 #include "pmlist.h"
 
+int get_remapIndex(int numRemaps, std::vector<RemapType> &remapList, int gridID, size_t numMissVals, bool useMask,
+                   Vmask const &imask);
+void pack_gme_vgpm(const Varray<int> &vgpm, Vmask &imask);
+
+struct RemapweightsParams
+{
+  std::string gridString;
+  KnnParams knnParams;
+};
+
+static RemapweightsParams
+get_parameter_knn()
+{
+  RemapweightsParams params;
+
+  auto pargc = cdo_operator_argc();
+  if (pargc)
+    {
+      auto const &pargv = cdo_get_oper_argv();
+
+      KVList kvlist;
+      kvlist.name = cdo_module_name();
+      if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
+      if (Options::cdoVerbose) kvlist.print();
+
+      for (auto const &kv : kvlist)
+        {
+          auto const &key = kv.key;
+          if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
+          if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
+          auto const &value = kv.values[0];
+
+          // clang-format off
+          if      (key == "k")           params.knnParams.k = parameter_to_int(value);
+          else if (key == "kmin")        params.knnParams.kMin = parameter_to_int(value);
+          else if (key == "weighted")    params.knnParams.weighted = string_to_weightingMethod(parameter_to_word(value));
+          else if (key == "gauss_scale") params.knnParams.gaussScale = parameter_to_double(value);
+          else if (key == "extrapolate") params.knnParams.extrapolate = parameter_to_bool(value);
+          else if (key == "grid")        params.gridString = parameter_to_word(value);
+          else cdo_abort("Invalid parameter key >%s<!", key);
+          // clang-format on
+        }
+    }
+
+  return params;
+}
+
 static void
-get_parameter(int offset, int &neighbors, bool &map3D, std::string &grid)
+print_parameter(const RemapweightsParams &params)
+{
+  std::stringstream outbuffer;
+  outbuffer << "grid=" << params.gridString;
+  outbuffer << ", k=" << params.knnParams.k;
+  outbuffer << ", kmin=" << params.knnParams.kMin;
+  outbuffer << ", weighted=" << weightingMethod_to_string(params.knnParams.weighted);
+  outbuffer << ", gauss_scale=" << params.knnParams.gaussScale;
+  outbuffer << ", extrapolate=" << params.knnParams.extrapolate;
+
+  cdo_verbose("%s", outbuffer.str());
+}
+
+static void
+get_parameter_map3d(int offset, int &neighbors, bool &map3D, std::string &grid)
 {
   auto pargc = cdo_operator_argc() - offset;
   if (pargc)
@@ -45,12 +106,12 @@ get_parameter(int offset, int &neighbors, bool &map3D, std::string &grid)
       if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-          const auto &value = kv.values[0];
+          auto const &value = kv.values[0];
 
           // clang-format off
           if      (key == "grid")                    grid = parameter_to_word(value);
@@ -63,15 +124,15 @@ get_parameter(int offset, int &neighbors, bool &map3D, std::string &grid)
 }
 
 static void
-remap_write_weights(const std::string &remapWeightsFile, const RemapSwitches &remapSwitches, RemapType &remap)
+remap_write_weights(std::string const &remapWeightsFile, KnnParams const &knnParams, const RemapSwitches &remapSwitches,
+                    RemapType &remap)
 {
-  remap_write_data_scrip(remapWeightsFile, remapSwitches, remap.srcGrid, remap.tgtGrid, remap.vars);
+  remap_write_data_scrip(remapWeightsFile, knnParams, remapSwitches, remap.srcGrid, remap.tgtGrid, remap.vars);
 
   constexpr auto removeMask{ false };
   remap_vars_free(remap.vars);
   remap_grid_free(remap.srcGrid, removeMask);
   remap_grid_free(remap.tgtGrid);
-  remap_search_free(remap.search);
 }
 
 class Remapweights : public Process
@@ -82,6 +143,7 @@ public:
     .name = "Remapweights",
     .operators = { { "genbil", GENBIL, 0, RemapbilHelp },
                    { "genbic", GENBIC, 0, RemapbicHelp },
+                   { "genknn", GENKNN, 0, {} },
                    { "gennn", GENNN, 0, RemapnnHelp },
                    { "gendis", GENDIS, 0, RemapdisHelp },
                    { "gencon", GENCON, 0, RemapconHelp },
@@ -94,13 +156,14 @@ public:
   };
   inline static RegisterEntry<Remapweights> registration = RegisterEntry<Remapweights>(module);
 
+  KnnParams knnParams;
   RemapSwitches remapSwitches;
   int numRemaps{ 0 };
   int numNeighbors{ 0 };
   bool map3D{ false };
 
   CdoStreamID streamID1;
-  int vlistID1;
+  int vlistID1{ CDI_UNDEFID };
 
   int gridID2;
 
@@ -114,9 +177,9 @@ public:
 
   VarList varList1;
   std::vector<bool> remapGrids;
-  std::vector<RemapType> remaps;
+  std::vector<RemapType> remapList;
 
-  RemapParams remapParams;
+  RemapDefaults remapDefaults;
   RemapMethod mapType;
   int remapOrder;
 
@@ -127,6 +190,42 @@ public:
   Field field1;
 
 public:
+  std::string
+  get_parameter()
+  {
+    std::string targetGridName;
+
+    if (operfunc == GENKNN)
+      {
+        auto remapParams = get_parameter_knn();
+        if (Options::cdoVerbose) print_parameter(remapParams);
+        if (remapParams.gridString.empty()) cdo_abort("grid parameter missing!");
+        targetGridName = remapParams.gridString;
+        knnParams = remapParams.knnParams;
+        if (knnParams.kMin == 0) knnParams.kMin = knnParams.k;
+      }
+    else
+      {
+        operator_input_arg("grid description file or name");
+        targetGridName = cdo_operator_argv(0);
+        int offset = targetGridName.starts_with("grid=") ? 0 : 1;
+        if (cdo_operator_argc() > offset)
+          {
+            int numNeighborsParam = 0;
+            get_parameter_map3d(offset, numNeighborsParam, map3D, targetGridName);
+            if (map3D) remapDefaults.genMultiWeights = 1;
+            if (operfunc == GENDIS)
+              {
+                if (numNeighborsParam < 0) cdo_abort("Number of nearest neighbors out of range (>0)!");
+                if (numNeighborsParam > 0) numNeighbors = numNeighborsParam;
+              }
+          }
+        else { operator_check_argc(1); }
+      }
+
+    return targetGridName;
+  }
+
   void
   init() override
   {
@@ -136,55 +235,50 @@ public:
 
     remap_set_int(REMAP_WRITE_REMAP, writeRemapWeightsOnly);
 
-    remapParams = remap_get_params();
-    extrapolateIsSet = (remapParams.extrapolate != -1);
-    remapExtrapolate = extrapolateIsSet ? (bool) remapParams.extrapolate : remap_func_is_dist(operfunc);
+    remapDefaults = remap_get_params();
+    extrapolateIsSet = (remapDefaults.extrapolate != -1);
+    remapExtrapolate = extrapolateIsSet ? (bool) remapDefaults.extrapolate : remap_func_is_dist(operfunc);
 
     if (Options::cdoVerbose) cdo_print("Extrapolation %s!", remapExtrapolate ? "enabled" : "disabled");
 
-    operator_input_arg("grid description file or name");
-    auto targetGrid = cdo_operator_argv(0);
-    int offset = targetGrid.starts_with("grid=") ? 0 : 1;
-    if (cdo_operator_argc() > offset)
-      {
-        int numNeighborsParam = 0;
-        get_parameter(offset, numNeighborsParam, map3D, targetGrid);
-        if (map3D) remapParams.genMultiWeights = 1;
-        if (operfunc == GENDIS)
-          {
-            if (numNeighborsParam < 0) cdo_abort("Number of nearest neighbors out of range (>0)!");
-            if (numNeighborsParam > 0) numNeighbors = numNeighborsParam;
-          }
-      }
-    else { operator_check_argc(1); }
+    auto targetGridName = get_parameter();
 
-    gridID2 = cdo_define_grid(targetGrid);
+    gridID2 = cdo_define_grid(targetGridName);
     if (gridInqType(gridID2) == GRID_GENERIC) cdo_abort("Unsupported target grid type (generic)!");
 
     streamID1 = cdo_open_read(0);
 
     vlistID1 = cdo_stream_inq_vlist(streamID1);
-
     varList1 = VarList(vlistID1);
 
-    remapGrids = remap_set_grids(vlistID1, varList1);
-
-    auto numRemapGrids = ranges::count_if(remapGrids, [](auto flag) { return (flag == true); });
+    auto findOnlyFirst = true;
+    remapGrids = remap_set_grids(vlistID1, varList1, findOnlyFirst);
+    auto numRemapGrids = std::ranges::count_if(remapGrids, [](auto flag) { return (flag == true); });
     if (numRemapGrids == 0) cdo_abort("No remappable grid found!");
 
-    maxRemaps = remapParams.maxRemaps;
+    maxRemaps = remapDefaults.maxRemaps;
     if (maxRemaps == -1) maxRemaps = remap_get_max_maps(vlistID1);
     if (maxRemaps < 1) cdo_abort("maxRemaps out of range (>0)!");
 
-    remaps.resize(maxRemaps);
+    remapList.resize(maxRemaps);
 
     remapSwitches = remap_operfunc_to_maptype(operfunc);
-    if (numNeighbors) remapSwitches.numNeighbors = numNeighbors;
+    if (remapSwitches.mapType == RemapMethod::KNN)
+      {
+        if (numNeighbors) remapSwitches.numNeighbors = numNeighbors;
+        if (remapSwitches.numNeighbors != -1)
+          {
+            knnParams.k = remapSwitches.numNeighbors;
+            knnParams.kMin = 1;
+            knnParams.extrapolate = remapExtrapolate;
+            knnParams.weighted = WeightingMethod::distanceWeighted;
+          }
+      }
 
     mapType = remapSwitches.mapType;
     remapOrder = remapSwitches.remapOrder;
     useMask = !(!remap_genweights
-                && (mapType == RemapMethod::BILINEAR || mapType == RemapMethod::BICUBIC || mapType == RemapMethod::DISTWGT
+                && (mapType == RemapMethod::BILINEAR || mapType == RemapMethod::BICUBIC || mapType == RemapMethod::KNN
                     || mapType == RemapMethod::CONSERV));
 
     remap_set_int(REMAP_GENWEIGHTS, (int) remap_genweights);
@@ -204,23 +298,21 @@ public:
   run() override
   {
     std::thread writeWorker;
-    Varray<int8_t> imask;
+    Vmask imask;
 
     int tsID = 0;
-    auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
+    auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
 
     int gridIDout = -1;
-    for (int recID = 0; recID < nrecs; ++recID)
+    for (int fieldID = 0; fieldID < numFields; ++fieldID)
       {
-        int varID, levelID;
-        cdo_inq_record(streamID1, &varID, &levelID);
+        auto [varID, levelID] = cdo_inq_field(streamID1);
         auto &var = varList1.vars[varID];
         field1.init(var);
-        cdo_read_record(streamID1, field1);
+        cdo_read_field(streamID1, field1);
         auto numMissVals1 = useMask ? field1.numMissVals : 0;
 
         auto gridIndex = vlistGridIndex(vlistID1, var.gridID);
-
         if (remapGrids[gridIndex])
           {
             if (numRemaps == 0) { gridIDout = var.gridID; }
@@ -233,20 +325,7 @@ public:
 
             remap_set_mask(field1, var.gridsize, numMissVals1, var.missval, imask);
 
-            int remapIndex = -1;
-            for (remapIndex = numRemaps - 1; remapIndex >= 0; remapIndex--)
-              {
-                auto &remap = remaps[remapIndex];
-                if (var.gridID == remap.gridID)
-                  {
-                    if ((useMask && numMissVals1 == remap.numMissVals && imask == remap.srcGrid.mask) || !useMask)
-                      {
-                        remap.nused++;
-                        break;
-                      }
-                  }
-              }
-
+            int remapIndex = get_remapIndex(numRemaps, remapList, var.gridID, numMissVals1, useMask, imask);
             if (remapIndex >= 0) continue;
 
             if (numRemaps >= maxRemaps) break;
@@ -254,20 +333,11 @@ public:
             remapIndex = numRemaps;
             numRemaps++;
 
-            auto &remap = remaps[remapIndex];
+            auto &remap = remapList[remapIndex];
 
             //  remap.srcGrid.luse_cell_area = false;
             //  remap.tgtGrid.luse_cell_area = false;
 
-            auto numSearchBins = remapParams.numSearchBins;
-            if (var.gridType != GRID_UNSTRUCTURED && !remapParams.numSearchBinsIsSet)
-              {
-                numSearchBins
-                    = (!remapExtrapolate && mapType == RemapMethod::DISTWGT) ? 1 : remap_gen_numbins(gridInqYsize(var.gridID));
-              }
-
-            remap_set_int(REMAP_NUM_SRCH_BINS, numSearchBins);
-
             remap.vars.normOpt = normOpt;
 
             if ((mapType == RemapMethod::BILINEAR || mapType == RemapMethod::BICUBIC)
@@ -281,48 +351,40 @@ public:
             remap.gridID = var.gridID;
             remap.numMissVals = numMissVals1;
 
-            if (var.gridType == GRID_GME)
-              {
-                for (size_t i = 0, j = 0; i < var.gridsize; ++i)
-                  if (remap.srcGrid.vgpm[i]) imask[j++] = imask[i];
-              }
+            if (var.gridType == GRID_GME) { pack_gme_vgpm(remap.srcGrid.vgpm, imask); }
 
             varray_copy(remap.srcGrid.size, imask, remap.srcGrid.mask);
 
             if (mapType == RemapMethod::CONSERV)
               {
-                ranges::fill(remap.srcGrid.cell_area, 0.0);
-                ranges::fill(remap.srcGrid.cell_frac, 0.0);
-                ranges::fill(remap.tgtGrid.cell_area, 0.0);
+                std::ranges::fill(remap.srcGrid.cellArea, 0.0);
+                std::ranges::fill(remap.srcGrid.cellFrac, 0.0);
+                std::ranges::fill(remap.tgtGrid.cellArea, 0.0);
               }
-            ranges::fill(remap.tgtGrid.cell_frac, 0.0);
+            std::ranges::fill(remap.tgtGrid.cellFrac, 0.0);
 
             // initialize some remapping variables
             remap_vars_init(mapType, remapOrder, remap.vars);
 
-            remap_print_info(operfunc, remap_genweights, remap.srcGrid, remap.tgtGrid, numMissVals1, remapSwitches.numNeighbors);
+            remap_print_info(operfunc, remap_genweights, remap.srcGrid, remap.tgtGrid, numMissVals1, knnParams);
 
             if (needGradients && remap.srcGrid.rank != 2 && remapOrder == 2)
               {
                 cdo_abort("Second order remapping is not available for unstructured grids!");
               }
 
-            remap_gen_weights(remapSwitches, remap);
+            remap_gen_weights(remapSwitches.mapType, knnParams, remap);
 
             std::string outFile = cdo_get_stream_name(1);
-            if (remapParams.genMultiWeights)
-              {
-                outFile += string_format("%05d", numRemaps);
-                outFile += ".nc";
-              }
+            if (remapDefaults.genMultiWeights) { outFile += string_format("%05d", numRemaps) + ".nc"; }
 
             // remap_write_weights(outFile, remapSwitches, remap);
 
             if (numRemaps > 1) writeWorker.join();
 
-            writeWorker = std::thread(remap_write_weights, outFile, remapSwitches, std::ref(remap));
+            writeWorker = std::thread(remap_write_weights, outFile, knnParams, remapSwitches, std::ref(remap));
 
-            if (!remapParams.genMultiWeights) break;
+            if (!remapDefaults.genMultiWeights) break;
           }
       }
 
@@ -330,7 +392,7 @@ public:
 
     for (int remapIndex = 0; remapIndex < numRemaps; remapIndex++)
       {
-        auto &remap = remaps[remapIndex];
+        auto &remap = remapList[remapIndex];
         remap_grid_free(remap.srcGrid);
       }
   }
