@@ -116,7 +116,7 @@ r[i]-x[i]);
 */
 
 static void
-calc_rhopot(size_t gridsize, size_t nlevel, const Varray<double> &pressure, const FieldVector &to, const FieldVector &sao,
+calc_rhopot(size_t gridsize, size_t nlevel, Varray<double> const &pressure, FieldVector const &to, FieldVector const &sao,
             FieldVector &rho)
 {
   // pressure units: hPa
@@ -125,15 +125,15 @@ calc_rhopot(size_t gridsize, size_t nlevel, const Varray<double> &pressure, cons
 
   for (size_t levelID = 0; levelID < nlevel; ++levelID)
     {
-      const auto &tovec = to[levelID].vec_d;
-      const auto &saovec = sao[levelID].vec_d;
+      auto const &tovec = to[levelID].vec_d;
+      auto const &saovec = sao[levelID].vec_d;
       auto &rhovec = rho[levelID].vec_d;
       auto to_missval = to[levelID].missval;
       auto sao_missval = sao[levelID].missval;
       auto rho_missval = rho[levelID].missval;
       for (size_t i = 0; i < gridsize; ++i)
         {
-          if (DBL_IS_EQUAL(tovec[i], to_missval) || DBL_IS_EQUAL(saovec[i], sao_missval))
+          if (fp_is_equal(tovec[i], to_missval) || fp_is_equal(saovec[i], sao_missval))
             rhovec[i] = rho_missval;
           else
             rhovec[i] = potrho_1(tovec[i], saovec[i], pressure[levelID]);
@@ -162,10 +162,8 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int vlistID2;
-
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   int numLevels;
   int gridsize;
@@ -190,11 +188,11 @@ public:
 
     for (int varID = 0; varID < numVars; ++varID)
       {
-        const auto &var = varList1.vars[varID];
+        auto const &var = varList1.vars[varID];
         auto code = var.code;
         if (code <= 0)
           {
-            auto stdname = string_to_lower(cdo::inq_key_string(vlistID1, varID, CDI_KEY_STDNAME));
+            auto stdname = string_to_lower(var.stdname);
 
             // clang-format off
             if      (var.name == "to") code = 20;
@@ -226,8 +224,8 @@ public:
     auto gridID = vlistGrid(vlistID1, 0);
     gridsize = vlist_check_gridsize(vlistID1);
 
-    const auto varSAO = varList1.vars[saoID];
-    const auto varTO = varList1.vars[toID];
+    auto const &varSAO = varList1.vars[saoID];
+    auto const &varTO = varList1.vars[toID];
     zaxisID = varSAO.zaxisID;
 
     if (varSAO.nlevels != varTO.nlevels) cdo_abort("temperature and salinity have different number of levels!");
@@ -264,7 +262,7 @@ public:
     int datatype = CDI_DATATYPE_FLT32;
     if (varTO.dataType == CDI_DATATYPE_FLT64 && varSAO.dataType == CDI_DATATYPE_FLT64) datatype = CDI_DATATYPE_FLT64;
 
-    vlistID2 = vlistCreate();
+    auto vlistID2 = vlistCreate();
     vlistDefNtsteps(vlistID2, vlistNtsteps(vlistID1));
 
     auto varID2 = vlistDefVar(vlistID2, gridID, zaxisID, TIME_VARYING);
@@ -282,6 +280,7 @@ public:
 
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
+    vlistDestroy(vlistID2);
   }
 
   void
@@ -290,26 +289,26 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            if (varID == toID) cdo_read_record(streamID1, to[levelID].vec_d.data(), &to[levelID].numMissVals);
-            if (varID == saoID) cdo_read_record(streamID1, sao[levelID].vec_d.data(), &sao[levelID].numMissVals);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            if (varID == toID) cdo_read_field(streamID1, to[levelID]);
+            if (varID == saoID) cdo_read_field(streamID1, sao[levelID]);
           }
 
         calc_rhopot(gridsize, numLevels, pressure, to, sao, rho);
 
         for (int levelID = 0; levelID < numLevels; ++levelID)
           {
-            cdo_def_record(streamID2, 0, levelID);
-            cdo_write_record(streamID2, rho[levelID].vec_d.data(), field_num_miss(rho[levelID]));
+            field_num_mv(rho[levelID]);
+            cdo_def_field(streamID2, 0, levelID);
+            cdo_write_field(streamID2, rho[levelID]);
           }
 
         tsID++;
@@ -321,7 +320,5 @@ public:
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    vlistDestroy(vlistID2);
   }
 };
