@@ -19,15 +19,16 @@
 #include "percentiles.h"
 #include "datetime.h"
 #include "field_functions.h"
-#include "cimdOmp.h"
+#include "cdo_omp.h"
 
 template <typename T>
 static size_t
-runpctl(double pn, int ndates, size_t gridsize, Varray<T> &v2, T missval, const FieldVector3D &vars1, int varID, int levelID,
+runpctl(double pn, int ndates, size_t gridsize, Varray<T> &v2, double mv, const FieldVector3D &vars1, int varID, int levelID,
         MemType memType)
 {
+  T missval = mv;
   size_t numMissVals = 0;
-  Varray2D<T> array_2D(Threading::ompNumThreads, Varray<T>(ndates));
+  Varray2D<T> array_2D(Threading::ompNumMaxThreads, Varray<T>(ndates));
 
 #ifdef _OPENMP
 #pragma omp parallel for default(shared) schedule(dynamic)
@@ -38,12 +39,13 @@ runpctl(double pn, int ndates, size_t gridsize, Varray<T> &v2, T missval, const
       auto &array = array_2D[ompthID];
 
       int j = 0;
+
       if (memType == MemType::Float)
         {
           for (int inp = 0; inp < ndates; ++inp)
             {
               auto val = vars1[inp][varID][levelID].vec_f[i];
-              if (dbl_is_not_equal(val, missval)) array[j++] = val;
+              if (fp_is_not_equal(val, missval)) array[j++] = val;
             }
         }
       else
@@ -51,10 +53,19 @@ runpctl(double pn, int ndates, size_t gridsize, Varray<T> &v2, T missval, const
           for (int inp = 0; inp < ndates; ++inp)
             {
               auto val = vars1[inp][varID][levelID].vec_d[i];
-              if (dbl_is_not_equal(val, missval)) array[j++] = val;
+              if (fp_is_not_equal(val, missval)) array[j++] = val;
             }
         }
-
+      /*
+      for (int inp = 0; inp < ndates; ++inp)
+        {
+          auto func = [&](auto &v) {
+            auto val = v[i];
+            if (fp_is_not_equal(val, missval)) array[j++] = val;
+          };
+          field_operation(func, vars1[inp][varID][levelID]);
+        }
+      */
       if (j > 0) { v2[i] = percentile(array.data(), j, pn); }
       else
         {
@@ -69,11 +80,10 @@ runpctl(double pn, int ndates, size_t gridsize, Varray<T> &v2, T missval, const
 static void
 runpctl(double pn, int ndates, Field &field1, const FieldVector3D &vars1, int varID, int levelID)
 {
-  if (field1.memType == MemType::Float)
-    field1.numMissVals
-        = runpctl(pn, ndates, field1.gridsize, field1.vec_f, (float) field1.missval, vars1, varID, levelID, field1.memType);
-  else
-    field1.numMissVals = runpctl(pn, ndates, field1.gridsize, field1.vec_d, field1.missval, vars1, varID, levelID, field1.memType);
+  auto func = [&](auto &v) {
+    field1.numMissVals = runpctl(pn, ndates, field1.gridsize, v, field1.missval, vars1, varID, levelID, field1.memType);
+  };
+  field_operation(func, field1);
 }
 
 class Runpctl : public Process
@@ -92,18 +102,18 @@ public:
 
   CdoStreamID streamID1;
   CdoStreamID streamID2;
-  int vlistID1;
-  int vlistID2;
+  int vlistID1{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
   VarList varList1;
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
   FieldVector3D varsData1;
 
   DateTimeList dtlist;
   double pn;
   int ndates;
-  int maxRecords;
-  std::vector<RecordInfo> recordList;
+  int maxFields;
+  std::vector<FieldInfo> fieldInfoList;
   int tsID;
 
 public:
@@ -134,8 +144,8 @@ public:
 
     varList1 = VarList(vlistID1);
 
-    maxRecords = varList1.numRecords();
-    recordList = std::vector<RecordInfo>(maxRecords);
+    maxFields = varList1.maxFields();
+    fieldInfoList = std::vector<FieldInfo>(maxFields);
 
     dtlist.set_stat(timestatDate);
     dtlist.set_calendar(taxisInqCalendar(taxisID1));
@@ -145,18 +155,18 @@ public:
   }
 
   void
-  write_records(const int otsID)
+  write_fields(int otsID)
   {
     dtlist.stat_taxis_def_timestep(taxisID2, ndates);
     cdo_def_timestep(streamID2, otsID);
-    for (int recID = 0; recID < maxRecords; ++recID)
+    for (int fieldID = 0; fieldID < maxFields; ++fieldID)
       {
-        auto [varID, levelID] = recordList[recID].get();
+        auto [varID, levelID] = fieldInfoList[fieldID].get();
         if (otsID && varList1.vars[varID].isConstant) continue;
 
-        cdo_def_record(streamID2, varID, levelID);
+        cdo_def_field(streamID2, varID, levelID);
         auto &field1 = varsData1[0][varID][levelID];
-        cdo_write_record(streamID2, field1);
+        cdo_write_field(streamID2, field1);
       }
   }
 
@@ -165,21 +175,20 @@ public:
   {
     for (tsID = 0; tsID < ndates; ++tsID)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) cdo_abort("File has less than %d timesteps!", ndates);
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) cdo_abort("File has less than %d timesteps!", ndates);
 
         dtlist.taxis_inq_timestep(taxisID1, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
 
-            if (tsID == 0) recordList[recID].set(varID, levelID);
+            if (tsID == 0) fieldInfoList[fieldID].set(varID, levelID);
 
             auto &field = varsData1[tsID][varID][levelID];
             field.init(varList1.vars[varID]);
-            cdo_read_record(streamID1, field);
+            cdo_read_field(streamID1, field);
           }
       }
     int otsID = 0;
@@ -198,7 +207,7 @@ public:
               }
           }
 
-        write_records(otsID);
+        write_fields(otsID);
         otsID++;
 
         dtlist.shift();
@@ -206,17 +215,16 @@ public:
         varsData1[ndates] = varsData1[0];
         for (int inp = 0; inp < ndates; ++inp) varsData1[inp] = varsData1[inp + 1];
 
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         dtlist.taxis_inq_timestep(taxisID1, ndates - 1);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             auto &fieldN = varsData1[ndates - 1][varID][levelID];
-            cdo_read_record(streamID1, fieldN);
+            cdo_read_field(streamID1, fieldN);
           }
 
         tsID++;
