@@ -52,19 +52,14 @@ public:
   inline static RegisterEntry<Runstat> registration = RegisterEntry<Runstat>(module);
 
 private:
-  TimeStat timestatDate{ TimeStat::MEAN };
   bool runstat_nomiss = false;
 
   CdoStreamID streamID1;
   CdoStreamID streamID2;
-  int taxisID1;
-  int taxisID2;
-
-  DateTimeList dtlist;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   VarList varList1;
-  int maxRecords;
-  std::vector<RecordInfo> recordList;
 
   int ndates;
 
@@ -76,7 +71,7 @@ public:
   void
   init() override
   {
-    auto envstr = getenv("RUNSTAT_NOMISS");
+    const auto envstr = getenv("RUNSTAT_NOMISS");
     if (envstr)
       {
         char *endptr;
@@ -106,7 +101,7 @@ public:
     taxisID2 = taxisDuplicate(taxisID1);
     taxisWithBounds(taxisID2);
     vlistDefTaxis(vlistID2, taxisID2);
-    // Number of timestep will be reduced compared to the input error handling in case of not enough timesteps is done per record
+    // Number of timestep will be reduced compared to the input error handling in case of not enough timesteps is done per field
     auto nsteps = vlistNtsteps(vlistID1);
     if (nsteps != -1)
       {
@@ -119,12 +114,6 @@ public:
 
     varList1 = VarList(vlistID1);
 
-    maxRecords = varList1.numRecords();
-    recordList = std::vector<RecordInfo>(maxRecords);
-
-    dtlist.set_stat(timestatDate);
-    dtlist.set_calendar(taxisInqCalendar(taxisID1));
-
     stepStat.set_dimlen0(ndates + 1);
 
     int VARS_MEMTYPE = stepStat.lminmax ? FIELD_NAT : 0;
@@ -139,16 +128,23 @@ public:
   void
   run() override
   {
-    std::vector<short> imask;
+    TimeStat timestatDate{ TimeStat::MEAN };
+    DateTimeList dtlist;
+    dtlist.set_stat(timestatDate);
+    dtlist.set_calendar(taxisInqCalendar(taxisID1));
+
+    auto maxFields = varList1.maxFields();
+    std::vector<FieldInfo> fieldInfoList(maxFields);
+
+    Vmask imask;
 
     int tsID = 0;
     int otsID = 0;
-    int numSteps = 0;
     while (true)
       {
       FILL_FIRST_NDATES:
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0)
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0)
           {
             if (tsID < ndates)
               cdo_abort("File has less then %d timesteps!", ndates);
@@ -156,16 +152,17 @@ public:
               break;
           }
 
-        numSteps = (tsID < ndates) ? tsID : ndates - 1;
+        auto numSteps = (tsID < ndates) ? tsID : ndates - 1;
 
         dtlist.taxis_inq_timestep(taxisID1, numSteps);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varIDx, levelIDx] = cdo_inq_field(streamID1);
+            int varID = varIDx;  // needed for omp loop with intel icpx 2022.0.0
+            int levelID = levelIDx;
 
-            if (tsID == 0) recordList[recID].set(varID, levelID);
+            if (tsID == 0) fieldInfoList[fieldID].set(varID, levelID);
 
             auto &rsamp = stepStat.samp(numSteps, varID, levelID);
             auto &rvar1 = stepStat.var1(numSteps, varID, levelID);
@@ -173,7 +170,7 @@ public:
 
             auto fieldsize = rvar1.size;  // used in omp loop
 
-            cdo_read_record(streamID1, rvar1);
+            cdo_read_field(streamID1, rvar1);
 
             if (runstat_nomiss && rvar1.numMissVals) cdo_abort("Missing values supported was swichted off by env. RUNSTAT_NOMISS!");
 
@@ -185,9 +182,9 @@ public:
                 auto missval = rvar1.missval;
 
                 if (rvar1.memType == MemType::Float)
-                  for (size_t i = 0; i < fieldsize; ++i) imask[i] = dbl_is_not_equal(rvar1.vec_f[i], (float) missval);
+                  for (size_t i = 0; i < fieldsize; ++i) imask[i] = fp_is_not_equal(rvar1.vec_f[i], (float) missval);
                 else
-                  for (size_t i = 0; i < fieldsize; ++i) imask[i] = dbl_is_not_equal(rvar1.vec_d[i], missval);
+                  for (size_t i = 0; i < fieldsize; ++i) imask[i] = fp_is_not_equal(rvar1.vec_d[i], missval);
 
                 for (size_t i = 0; i < fieldsize; ++i) rsamp.vec_d[i] = (double) imask[i];
 
@@ -240,18 +237,18 @@ public:
         if (tsID < ndates) goto FILL_FIRST_NDATES;
 
         auto numSets = ndates;
-        cdo::records_process_3D(0, recordList, varList1, stepStat, numSets);
+        cdo::fields_process_3D(0, fieldInfoList, varList1, stepStat, numSets);
 
         dtlist.stat_taxis_def_timestep(taxisID2, ndates);
         cdo_def_timestep(streamID2, otsID);
 
-        for (int recID = 0; recID < maxRecords; ++recID)
+        for (int fieldID = 0; fieldID < maxFields; ++fieldID)
           {
-            auto [varID, levelID] = recordList[recID].get();
+            auto [varID, levelID] = fieldInfoList[fieldID].get();
             if (otsID && varList1.vars[varID].isConstant) continue;
 
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, stepStat.var1(0, varID, levelID));
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, stepStat.var1(0, varID, levelID));
           }
 
         otsID++;
