@@ -16,7 +16,7 @@
 #include "process_int.h"
 #include "cdi_lockedIO.h"
 #include <mpim_grid.h>
-#include "grid_point_search.h"
+#include "grid_pointsearch.h"
 #include "verifygrid.h"
 #include "field_functions.h"
 
@@ -31,19 +31,19 @@ struct CellIndex
 };
 
 static void
-copy_data_to_index(long ncells, const Varray<double> &data, long *cellindex)
+copy_data_to_index(long ncells, Varray<double> const &data, long *cellindex)
 {
   for (long i = 0; i < ncells; ++i) cellindex[i] = std::lround(data[i]);
 }
 
 static void
-read_cellindex(const std::string &filename, CellIndex &cellindex)
+read_cellindex(std::string const &filename, CellIndex &cellindex)
 {
   auto streamID = stream_open_read_locked(filename.c_str());
   auto vlistID = streamInqVlist(streamID);
-  auto ngrids = vlistNumGrids(vlistID);
+  auto numGrids = vlistNumGrids(vlistID);
   int gridID = -1;
-  for (int index = 0; index < ngrids; ++index)
+  for (int index = 0; index < numGrids; ++index)
     {
       gridID = vlistGrid(vlistID, index);
       if (gridInqType(gridID) == GRID_UNSTRUCTURED && gridInqNvertex(gridID) == 3) break;
@@ -93,15 +93,15 @@ read_cellindex(const std::string &filename, CellIndex &cellindex)
 
   for (long i = 0; i < ncells; ++i) cellindex.parent[i] = 0;
 
-  auto nrecs = streamInqTimestep(streamID, 0);
-  for (int recID = 0; recID < nrecs; ++recID)
+  auto numFields = streamInqTimestep(streamID, 0);
+  for (int fieldID = 0; fieldID < numFields; ++fieldID)
     {
       int varID, levelID;
       size_t numMissVals;
-      streamInqRecord(streamID, &varID, &levelID);
+      streamInqField(streamID, &varID, &levelID);
       if (varID == pid /* || varID == nid || varID == cid */)
         {
-          streamReadRecord(streamID, data.data(), &numMissVals);
+          streamReadField(streamID, data.data(), &numMissVals);
           // if (varID == pid)
           {
             if (Options::cdoVerbose) cdo_print("Read parent_cell_index");
@@ -120,13 +120,13 @@ read_cellindex(const std::string &filename, CellIndex &cellindex)
 }
 
 static int
-read_grid(const std::string &filename)
+read_grid(std::string const &filename)
 {
   auto streamID = stream_open_read_locked(filename.c_str());
   auto vlistID = streamInqVlist(streamID);
-  auto ngrids = vlistNumGrids(vlistID);
+  auto numGrids = vlistNumGrids(vlistID);
   int gridID = -1;
-  for (int index = 0; index < ngrids; ++index)
+  for (int index = 0; index < numGrids; ++index)
     {
       gridID = vlistGrid(vlistID, index);
       if (gridInqType(gridID) == GRID_UNSTRUCTURED && gridInqNvertex(gridID) == 3) break;
@@ -207,7 +207,7 @@ compute_child_from_parent(CellIndex &cellindex1, CellIndex &cellindex2)
               sinfo[j].p = parent1[j];
               sinfo[j].i = idx1[j];
             }
-          ranges::sort(sinfo, {}, &SortInfo::p);
+          std::ranges::sort(sinfo, {}, &SortInfo::p);
           for (long j = 0; j < ncells1; ++j)
             {
               parent1[j] = sinfo[j].p;
@@ -236,13 +236,13 @@ compute_child_from_parent(CellIndex &cellindex1, CellIndex &cellindex2)
 }
 
 static void
-read_coordinates(const std::string &filename, long n, Varray<double> &lon, Varray<double> &lat)
+read_coordinates(std::string const &filename, long n, Varray<double> &lon, Varray<double> &lat)
 {
   auto streamID = streamOpenRead(filename.c_str());
   auto vlistID = streamInqVlist(streamID);
-  auto ngrids = vlistNumGrids(vlistID);
+  auto numGrids = vlistNumGrids(vlistID);
   int gridID = -1;
-  for (int index = 0; index < ngrids; ++index)
+  for (int index = 0; index < numGrids; ++index)
     {
       gridID = vlistGrid(vlistID, index);
       if (gridInqType(gridID) == GRID_UNSTRUCTURED && (long) gridInqSize(gridID) == n && gridInqNvertex(gridID) == 3) break;
@@ -261,14 +261,14 @@ read_coordinates(const std::string &filename, long n, Varray<double> &lon, Varra
 }
 
 static void
-read_coordinates(const std::string &filename, long n, Varray<double> &lon, Varray<double> &lat, int nv, Varray<double> &lon_bnds,
+read_coordinates(std::string const &filename, long n, Varray<double> &lon, Varray<double> &lat, int nv, Varray<double> &lon_bnds,
                  Varray<double> &lat_bnds)
 {
   auto streamID = streamOpenRead(filename.c_str());
   auto vlistID = streamInqVlist(streamID);
-  auto ngrids = vlistNumGrids(vlistID);
+  auto numGrids = vlistNumGrids(vlistID);
   int gridID = -1;
-  for (int index = 0; index < ngrids; ++index)
+  for (int index = 0; index < numGrids; ++index)
     {
       gridID = vlistGrid(vlistID, index);
       if (gridInqType(gridID) == GRID_UNSTRUCTURED && (long) gridInqSize(gridID) == n && gridInqNvertex(gridID) == 3) break;
@@ -296,24 +296,22 @@ read_coordinates(const std::string &filename, long n, Varray<double> &lon, Varra
 }
 
 static void
-compute_child_from_bounds(CellIndex &cellindex2, long ncells2, Varray<double> &grid_center_lon2, Varray<double> &grid_center_lat2,
-                          Varray<double> &grid_corner_lon2, Varray<double> &grid_corner_lat2, long ncells1,
-                          const Varray<double> &grid_center_lon1, const Varray<double> &grid_center_lat1)
+compute_child_from_bounds(CellIndex &cellindex2, Varray<double> &grid_center_lon2, Varray<double> &grid_center_lat2,
+                          Varray<double> &grid_corner_lon2, Varray<double> &grid_corner_lat2,
+                          Varray<double> const &grid_center_lon1, Varray<double> const &grid_center_lat1)
 {
   if (Options::cdoVerbose) cdo_print("%s", __func__);
 
-  bool xIsCyclic = false;
-  size_t dims[2];
-  dims[0] = ncells1;
-  dims[1] = 0;
-  GridPointSearch gps;
-  grid_point_search_create(gps, xIsCyclic, dims, ncells1, grid_center_lon1, grid_center_lat1);
+  long ncells2 = (long) grid_center_lon2.size();
+
+  GridPointsearch gps;
+  grid_pointsearch_create_unstruct(gps, grid_center_lon1, grid_center_lat1, true);
 
   int ncorner = 3;
 
   constexpr int MaxSearch = 128;
-  std::vector<knnWeightsType> knnWeightsMem;
-  for (int i = 0; i < Threading::ompNumThreads; ++i) knnWeightsMem.push_back(knnWeightsType(MaxSearch));
+  std::vector<KnnData> knnDataList;
+  for (int i = 0; i < Threading::ompNumMaxThreads; ++i) knnDataList.emplace_back(MaxSearch);
 
   cellindex2.child.resize(MAX_CHILDS * ncells2);
   auto &child2 = cellindex2.child;
@@ -344,16 +342,16 @@ compute_child_from_bounds(CellIndex &cellindex2, long ncells2, Varray<double> &g
       if (isClockwise) continue;
 
       auto ompthID = cdo_omp_get_thread_num();
-      auto &knnWeights = knnWeightsMem[ompthID];
+      auto &knnData = knnDataList[ompthID];
 
-      grid_search_point(gps, grid_center_lon2[cellNo2], grid_center_lat2[cellNo2], knnWeights);
+      grid_search_point_unstruct(gps, PointLonLat{ grid_center_lon2[cellNo2], grid_center_lat2[cellNo2] }, knnData);
 
       int k = 0;
       double centerCoordinates[3];
       Point3D centerPoint3D;
       for (int i = 0; i < MaxSearch; ++i)
         {
-          auto cellNo1 = knnWeights.m_indices[i];
+          auto cellNo1 = knnData.m_indices[i];
           if (cellNo1 < SIZE_MAX)
             {
               gcLLtoXYZ(grid_center_lon1[cellNo1], grid_center_lat1[cellNo1], centerCoordinates);
@@ -372,8 +370,6 @@ compute_child_from_bounds(CellIndex &cellindex2, long ncells2, Varray<double> &g
             }
         }
     }
-
-  grid_point_search_delete(gps);
 }
 
 static void
@@ -390,7 +386,7 @@ compute_child_from_coordinates(const CellIndex &cellindex1, CellIndex &cellindex
   read_coordinates(cellindex1.filename, ncells1, lon1, lat1);
   read_coordinates(cellindex2.filename, ncells2, lon2, lat2, 3, lon2_bnds, lat2_bnds);
 
-  compute_child_from_bounds(cellindex2, ncells2, lon2, lat2, lon2_bnds, lat2_bnds, ncells1, lon1, lat1);
+  compute_child_from_bounds(cellindex2, lon2, lat2, lon2_bnds, lat2_bnds, lon1, lat1);
 }
 
 static void
@@ -398,7 +394,7 @@ compute_child(CellIndex &cellindex1, CellIndex &cellindex2)
 {
   bool lparent = true;
   auto ncells1 = cellindex1.ncells;
-  const auto &parent1 = cellindex1.parent;
+  auto const &parent1 = cellindex1.parent;
 
   long i;
   for (i = 0; i < ncells1; ++i)
@@ -414,7 +410,7 @@ compute_child(CellIndex &cellindex1, CellIndex &cellindex2)
 }
 
 static void
-compute_sum(long i, long &n, double &sum, double &sumq, long kci, Varray<CellIndex> &cellindex, const Varray<double> &array)
+compute_sum(long i, long &n, double &sum, double &sumq, long kci, Varray<CellIndex> &cellindex, Varray<double> const &array)
 {
   // printf("compute: i, kci %d %d\n", i, kci);
   auto ncells2 = cellindex[kci].ncells;
@@ -436,7 +432,7 @@ compute_sum(long i, long &n, double &sum, double &sumq, long kci, Varray<CellInd
 }
 
 static void
-samplegrid(double missval, long nci, Varray<CellIndex> &cellindex, const Varray<double> &array1, Varray<double> &array2,
+samplegrid(double missval, long nci, Varray<CellIndex> &cellindex, Varray<double> const &array1, Varray<double> &array2,
            Varray<double> &array3)
 {
   static bool lstat = true;
@@ -487,13 +483,13 @@ public:
   CdoStreamID streamID2;
   CdoStreamID streamID3;
 
-  int vlistID2;
+  int vlistID2{ CDI_UNDEFID };
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
   int taxisID3;
 
-  int vlistID1;
+  int vlistID1{ CDI_UNDEFID };
   int gridID2;
 
   Varray<double> array1;
@@ -527,12 +523,13 @@ public:
     streamID1 = cdo_open_read(0);
     vlistID1 = cdo_stream_inq_vlist(streamID1);
 
-    long gridsize = vlistGridsizeMax(vlistID1);
-    if (Options::cdoVerbose) cdo_print("Source gridsize = %zu", gridsize);
-    if (gridsize != cellindex[0].ncells)
-      cdo_abort("Gridsize (%ld) of input stream and first grid (%ld) differ!", gridsize, cellindex[0].ncells);
-    if (vlistNumber(vlistID1) != CDI_REAL) gridsize *= 2;
-    array1 = Varray<double>(gridsize);
+    VarList varList1(vlistID1);
+    long gridsizeMax = varList1.gridsizeMax();
+    if (Options::cdoVerbose) cdo_print("Source gridsize = %zu", gridsizeMax);
+    if (gridsizeMax != cellindex[0].ncells)
+      cdo_abort("Gridsize (%ld) of input stream and first grid (%ld) differ!", gridsizeMax, cellindex[0].ncells);
+    if (vlistNumber(vlistID1) != CDI_REAL) gridsizeMax *= 2;
+    array1 = Varray<double>(gridsizeMax);
 
     vlistID2 = vlistDuplicate(vlistID1);
     auto vlistID3 = vlistDuplicate(vlistID1);
@@ -543,8 +540,8 @@ public:
     vlistDefTaxis(vlistID2, taxisID2);
     vlistDefTaxis(vlistID3, taxisID3);
 
-    auto ngrids = vlistNumGrids(vlistID1);
-    for (int index = 0; index < ngrids; ++index)
+    auto numGrids = vlistNumGrids(vlistID1);
+    for (int index = 0; index < numGrids; ++index)
       {
         auto gridID = vlistGrid(vlistID1, index);
         auto gridtype = gridInqType(gridID);
@@ -574,32 +571,31 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_taxis_copy_timestep(taxisID3, taxisID1);
         cdo_def_timestep(streamID2, tsID);
         cdo_def_timestep(streamID3, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             size_t numMissVals;
-            cdo_read_record(streamID1, array1.data(), &numMissVals);
+            cdo_read_field(streamID1, array1.data(), &numMissVals);
 
             auto missval = vlistInqVarMissval(vlistID1, varID);
 
             samplegrid(missval, nsamplegrids, cellindex, array1, array2, array3);
 
             numMissVals = varray_num_mv(gridsize2, array2, missval);
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, array2.data(), numMissVals);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, array2.data(), numMissVals);
 
             numMissVals = varray_num_mv(gridsize2, array3, missval);
-            cdo_def_record(streamID3, varID, levelID);
-            cdo_write_record(streamID3, array3.data(), numMissVals);
+            cdo_def_field(streamID3, varID, levelID);
+            cdo_write_field(streamID3, array3.data(), numMissVals);
           }
 
         tsID++;
