@@ -21,8 +21,20 @@
 #include "process_int.h"
 #include "field_functions.h"
 
-class ModuleSeascount
+class Seascount : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Seascount",
+    .operators = { { "seascount" } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_BOTH,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Seascount> registration = RegisterEntry<Seascount>(module);
+
   CdiDateTime vDateTime0{};
   int seas0 = 0;
   int oldmon = 0;
@@ -30,24 +42,15 @@ class ModuleSeascount
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
-
-  int maxrecs;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   VarList varList1;
-  Field field;
-  std::vector<RecordInfo> recList;
-  FieldVector2D vars1;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    cdo_operator_add("seascount", 0, 0, nullptr);
-
     operator_check_argc(0);
 
     streamID1 = cdo_open_read(0);
@@ -62,29 +65,32 @@ public:
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
 
-    maxrecs = vlistNrecs(vlistID1);
-    recList = std::vector<RecordInfo>(maxrecs);
-
-    varList_init(varList1, vlistID1);
-
-    fields_from_vlist(vlistID1, vars1, FIELD_VEC);
+    varList1 = VarList(vlistID1);
   }
+
   void
-  run()
+  run() override
   {
+    FieldVector2D varsData1;
+    field2D_init(varsData1, varList1, FIELD_VEC);
+
+    auto maxFields = varList1.maxFields();
+    std::vector<FieldInfo> fieldInfoList(maxFields);
+    Field field;
+
     auto seasonStart = get_season_start();
 
     int tsID = 0;
     int otsID = 0;
     while (true)
       {
-        int nrecs = 0;
+        int numFields = 0;
         int numSets = 0;
         auto newseas = false;
         while (true)
           {
-            nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-            if (nrecs == 0) break;
+            numFields = cdo_stream_inq_timestep(streamID1, tsID);
+            if (numFields == 0) break;
 
             auto vDateTime = taxisInqVdatetime(taxisID1);
 
@@ -110,26 +116,25 @@ public:
 
             oldmon = newmon;
 
-            for (int recID = 0; recID < nrecs; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                int varID, levelID;
-                cdo_inq_record(streamID1, &varID, &levelID);
-                const auto &var = varList1[varID];
+                auto [varID, levelID] = cdo_inq_field(streamID1);
+                auto const &var = varList1.vars[varID];
 
-                if (tsID == 0) recList[recID].set(varID, levelID);
+                if (tsID == 0) fieldInfoList[fieldID].set(varID, levelID);
 
-                auto fieldsize = vars1[varID][levelID].size;
+                auto fieldsize = varsData1[varID][levelID].size;
 
                 if (numSets == 0)
                   {
-                    for (size_t i = 0; i < fieldsize; ++i) vars1[varID][levelID].vec_d[i] = vars1[varID][levelID].missval;
-                    vars1[varID][levelID].nmiss = fieldsize;
+                    for (size_t i = 0; i < fieldsize; ++i) varsData1[varID][levelID].vec_d[i] = varsData1[varID][levelID].missval;
+                    varsData1[varID][levelID].numMissVals = fieldsize;
                   }
 
                 field.init(var);
-                cdo_read_record(streamID1, field);
+                cdo_read_field(streamID1, field);
 
-                field2_count(vars1[varID][levelID], field);
+                field2_count(varsData1[varID][levelID], field);
               }
 
             vDateTime0 = vDateTime;
@@ -137,40 +142,29 @@ public:
             tsID++;
           }
 
-        if (nrecs == 0 && numSets == 0) break;
+        if (numFields == 0 && numSets == 0) break;
 
         taxisDefVdatetime(taxisID2, vDateTime0);
         cdo_def_timestep(streamID2, otsID);
 
-        for (int recID = 0; recID < maxrecs; ++recID)
+        for (int fieldID = 0; fieldID < maxFields; ++fieldID)
           {
-            auto [varID, levelID] = recList[recID].get();
-            if (otsID && varList1[varID].isConstant) continue;
+            auto [varID, levelID] = fieldInfoList[fieldID].get();
+            if (otsID && varList1.vars[varID].isConstant) continue;
 
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, vars1[varID][levelID].vec_d.data(), vars1[varID][levelID].nmiss);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, varsData1[varID][levelID]);
           }
 
-        if (nrecs == 0) break;
+        if (numFields == 0) break;
         otsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Seascount(void *process)
-{
-  ModuleSeascount seascount;
-  seascount.init(process);
-  seascount.run();
-  seascount.close();
-  return nullptr;
-}
