@@ -39,7 +39,7 @@ public:
   TimeStat timestatDate{ TimeStat::MEAN };
   CdiDateTime vDateTime0{};
   CdiDateTime vDateTime1{};
-  int nrecs;
+  int numFields;
   int seas0 = 0;
   int oldmon = 0;
   int nseason = 0;
@@ -52,8 +52,8 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   VarList varList1;
   FieldVector2D samp1, varsData1;
@@ -100,8 +100,8 @@ public:
   void
   run() override
   {
-    auto maxRecords = varList1.numRecords();
-    std::vector<RecordInfo> recordList(maxRecords);
+    auto maxFields = varList1.maxFields();
+    std::vector<FieldInfo> fieldInfoList(maxFields);
     Field field;
 
     auto seasonStart = get_season_start();
@@ -116,8 +116,8 @@ public:
         auto newseas = false;
         while (true)
           {
-            nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-            if (nrecs <= 0) break;
+            numFields = cdo_stream_inq_timestep(streamID1, tsID);
+            if (numFields <= 0) break;
 
             dtlist.taxis_inq_timestep(taxisID1, numSets);
             auto vDateTime = dtlist.get_vDateTime(numSets);
@@ -155,13 +155,12 @@ public:
 
             oldmon = newmon;
 
-            for (int recID = 0; recID < nrecs; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                int varID, levelID;
-                cdo_inq_record(streamID1, &varID, &levelID);
-                const auto &var = varList1.vars[varID];
+                auto [varID, levelID] = cdo_inq_field(streamID1);
+                auto const &var = varList1.vars[varID];
 
-                if (tsID == 0) recordList[recID].set(varID, levelID);
+                if (tsID == 0) fieldInfoList[fieldID].set(varID, levelID);
 
                 auto &rsamp1 = samp1[varID][levelID];
                 auto &rvars1 = varsData1[varID][levelID];
@@ -170,7 +169,7 @@ public:
 
                 if (numSets == 0)
                   {
-                    cdo_read_record(streamID1, rvars1);
+                    cdo_read_field(streamID1, rvars1);
 
                     fieldc_mul(rvars1, dpm);
 
@@ -183,7 +182,7 @@ public:
                 else
                   {
                     field.init(var);
-                    cdo_read_record(streamID1, field);
+                    cdo_read_field(streamID1, field);
 
                     fieldc_mul(field, dpm);
 
@@ -204,16 +203,16 @@ public:
             tsID++;
           }
 
-        if (nrecs == 0 && numSets == 0) break;
+        if (numFields == 0 && numSets == 0) break;
 
         auto numVars = varList1.numVars();
         for (int varID = 0; varID < numVars; ++varID)
           {
-            const auto &var = varList1.vars[varID];
+            auto const &var = varList1.vars[varID];
             if (var.isConstant) continue;
             for (int levelID = 0; levelID < var.nlevels; ++levelID)
               {
-                const auto &rsamp1 = samp1[varID][levelID];
+                auto const &rsamp1 = samp1[varID][levelID];
                 auto &rvars1 = varsData1[varID][levelID];
                 if (!rsamp1.empty())
                   field2_div(rvars1, rsamp1);
@@ -233,17 +232,17 @@ public:
           cdo_warning("Season %3d (%s) has only %d input time step%s!", otsID + 1, date_to_string(vDateTime0.date), numSets,
                       numSets == 1 ? "" : "s");
 
-        for (int recID = 0; recID < maxRecords; ++recID)
+        for (int fieldID = 0; fieldID < maxFields; ++fieldID)
           {
-            auto [varID, levelID] = recordList[recID].get();
+            auto [varID, levelID] = fieldInfoList[fieldID].get();
             if (otsID && varList1.vars[varID].isConstant) continue;
 
             auto &rvars1 = varsData1[varID][levelID];
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, rvars1);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, rvars1);
           }
 
-        if (nrecs == 0) break;
+        if (numFields == 0) break;
         otsID++;
       }
   }
