@@ -24,8 +24,20 @@
 #include "cdo_season.h"
 #include "field_functions.h"
 
-class ModuleSeaspctl
+class Seaspctl : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Seaspctl",
+    .operators = { { "seaspctl", FieldFunc_Pctl, 0, SeaspctlHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 3, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Seaspctl> registration = RegisterEntry<Seaspctl>(module);
+
   TimeStat timestatDate{ TimeStat::MEAN };
   int seas0 = 0;
   int oldmon = 0;
@@ -35,29 +47,21 @@ class ModuleSeaspctl
   CdoStreamID streamID3;
   CdoStreamID streamID4;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
   int taxisID3;
   int taxisID4;
 
-  Field field1, field2;
   DateTimeList dtlist;
   VarList varList1;
-  int maxrecs;
   double pn;
 
-  std::vector<RecordInfo> recList;
   HistogramSet hset;
-  FieldVector constFields;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    cdo_operator_add("seaspctl", FieldFunc_Pctl, 0, nullptr);
-
     operator_input_arg("percentile number");
     pn = parameter_to_double(cdo_operator_argv(0));
 
@@ -72,8 +76,12 @@ public:
 
     vlist_unpack(vlistID4);
 
-    vlist_compare(vlistID1, vlistID2, CmpVlist::All);
-    vlist_compare(vlistID1, vlistID3, CmpVlist::All);
+    varList1 = VarList(vlistID1);
+    VarList varList2(vlistID2);
+    VarList varList3(vlistID3);
+
+    varList_compare(varList1, varList2);
+    varList_compare(varList1, varList3);
 
     taxisID1 = vlistInqTaxis(vlistID1);
     taxisID2 = vlistInqTaxis(vlistID2);
@@ -87,39 +95,35 @@ public:
     streamID4 = cdo_open_write(3);
     cdo_def_vlist(streamID4, vlistID4);
 
-    auto ntsteps = vlistNtsteps(vlistID1);
-    auto nvars = vlistNvars(vlistID1);
-
-    maxrecs = vlistNrecs(vlistID1);
-    recList = std::vector<RecordInfo>(maxrecs);
-    constFields = FieldVector(maxrecs);
+    auto numVars = varList1.numVars();
+    auto numSteps = varList1.numSteps();
 
     dtlist.set_stat(timestatDate);
     dtlist.set_calendar(taxisInqCalendar(taxisID1));
 
-    varList_init(varList1, vlistID1);
+    hset = HistogramSet(numVars, numSteps);
 
-    hset = HistogramSet(nvars, ntsteps);
-
-    for (int varID = 0; varID < nvars; ++varID)
-      {
-        const auto &var = varList1[varID];
-        hset.createVarLevels(varID, var.nlevels, var.gridsize);
-      }
+    for (auto const &var : varList1.vars) hset.createVarLevels(var.ID, var.nlevels, var.gridsize);
   }
 
   void
-  run()
+  run() override
   {
+    auto maxFields = varList1.maxFields();
+    std::vector<FieldInfo> fieldInfoList(maxFields);
+    FieldVector constFields(maxFields);
+
+    Field field1, field2;
+
     auto seasonStart = get_season_start();
 
     int tsID = 0;
     int otsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID2, otsID);
-        if (nrecs != cdo_stream_inq_timestep(streamID3, otsID))
-          cdo_abort("Number of records at time step %d of %s and %s differ!", otsID + 1, cdo_get_stream_name(1),
+        auto numFields = cdo_stream_inq_timestep(streamID2, otsID);
+        if (numFields != cdo_stream_inq_timestep(streamID3, otsID))
+          cdo_abort("Number of fields at time step %d of %s and %s differ!", otsID + 1, cdo_get_stream_name(1),
                     cdo_get_stream_name(2));
 
         auto vDateTime2 = taxisInqVdatetime(taxisID2);
@@ -128,24 +132,23 @@ public:
           cdo_abort("Verification dates at time step %d of %s and %s differ!", otsID + 1, cdo_get_stream_name(1),
                     cdo_get_stream_name(2));
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID2, &varID, &levelID);
-            const auto &var = varList1[varID];
+            auto [varID, levelID] = cdo_inq_field(streamID2);
+            auto const &var = varList1.vars[varID];
             field1.init(var);
-            cdo_read_record(streamID2, field1);
+            cdo_read_field(streamID2, field1);
 
-            cdo_inq_record(streamID3, &varID, &levelID);
+            auto [varID2, levelID2] = cdo_inq_field(streamID3);
             field2.init(var);
-            cdo_read_record(streamID3, field2);
+            cdo_read_field(streamID3, field2);
 
-            hset.defVarLevelBounds(varID, levelID, field1, field2);
+            hset.defVarLevelBounds(varID2, levelID2, field1, field2);
           }
 
         int numSets = 0;
         auto newseas = false;
-        while (nrecs && (nrecs = cdo_stream_inq_timestep(streamID1, tsID)))
+        while (numFields && (numFields = cdo_stream_inq_timestep(streamID1, tsID)))
           {
             dtlist.taxis_inq_timestep(taxisID1, numSets);
             auto vDateTime1 = dtlist.get_vDateTime(numSets);
@@ -172,23 +175,22 @@ public:
 
             oldmon = newmon;
 
-            for (int recID = 0; recID < nrecs; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                int varID, levelID;
-                cdo_inq_record(streamID1, &varID, &levelID);
-                const auto &var = varList1[varID];
+                auto [varID, levelID] = cdo_inq_field(streamID1);
+                auto const &var = varList1.vars[varID];
 
-                if (tsID == 0) recList[recID].set(varID, levelID);
+                if (tsID == 0) fieldInfoList[fieldID].set(varID, levelID);
 
                 if (tsID == 0 && var.isConstant)
                   {
-                    constFields[recID].init(var);
-                    cdo_read_record(streamID1, constFields[recID]);
+                    constFields[fieldID].init(var);
+                    cdo_read_field(streamID1, constFields[fieldID]);
                   }
                 else
                   {
                     field1.init(var);
-                    cdo_read_record(streamID1, field1);
+                    cdo_read_field(streamID1, field1);
 
                     hset.addVarLevelValues(varID, levelID, field1);
                   }
@@ -198,52 +200,39 @@ public:
             tsID++;
           }
 
-        if (nrecs == 0 && numSets == 0) break;
+        if (numFields == 0 && numSets == 0) break;
 
         dtlist.stat_taxis_def_timestep(taxisID4, numSets);
         cdo_def_timestep(streamID4, otsID);
 
-        for (int recID = 0; recID < maxrecs; ++recID)
+        for (int fieldID = 0; fieldID < maxFields; ++fieldID)
           {
-            auto [varID, levelID] = recList[recID].get();
-            if (otsID && varList1[varID].isConstant) continue;
+            auto [varID, levelID] = fieldInfoList[fieldID].get();
+            auto const &var = varList1.vars[varID];
+            if (otsID && var.isConstant) continue;
 
-            cdo_def_record(streamID4, varID, levelID);
+            cdo_def_field(streamID4, varID, levelID);
 
-            if (varList1[varID].isConstant) { cdo_write_record(streamID4, constFields[recID]); }
+            if (var.isConstant) { cdo_write_field(streamID4, constFields[fieldID]); }
             else
               {
-                const auto &var = varList1[varID];
                 field1.init(var);
                 hset.getVarLevelPercentiles(field1, varID, levelID, pn);
-                cdo_write_record(streamID4, field1);
+                cdo_write_field(streamID4, field1);
               }
           }
 
-        if (nrecs == 0) break;
+        if (numFields == 0) break;
         otsID++;
       }
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID4);
     cdo_stream_close(streamID3);
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Seaspctl(void *process)
-{
-  ModuleSeaspctl seaspctl;
-  seaspctl.init(process);
-  seaspctl.run();
-  seaspctl.close();
-
-  return nullptr;
-}
