@@ -39,15 +39,15 @@ public:
     .name = "Seasstat",
     // clang-format off
     .operators = { { "seasrange", FieldFunc_Range, 0, SeasstatHelp },
-                   { "seasmin", FieldFunc_Min, 0, SeasstatHelp },
-                   { "seasmax", FieldFunc_Max, 0, SeasstatHelp },
-                   { "seassum", FieldFunc_Sum, 0, SeasstatHelp },
-                   { "seasmean", FieldFunc_Mean, 0, SeasstatHelp },
-                   { "seasavg", FieldFunc_Avg, 0, SeasstatHelp },
-                   { "seasstd", FieldFunc_Std, 0, SeasstatHelp },
-                   { "seasstd1", FieldFunc_Std1, 0, SeasstatHelp },
-                   { "seasvar", FieldFunc_Var, 0, SeasstatHelp },
-                   { "seasvar1", FieldFunc_Var1, 0, SeasstatHelp } },
+                   { "seasmin",   FieldFunc_Min,   0, SeasstatHelp },
+                   { "seasmax",   FieldFunc_Max,   0, SeasstatHelp },
+                   { "seassum",   FieldFunc_Sum,   0, SeasstatHelp },
+                   { "seasmean",  FieldFunc_Mean,  0, SeasstatHelp },
+                   { "seasavg",   FieldFunc_Avg,   0, SeasstatHelp },
+                   { "seasstd",   FieldFunc_Std,   0, SeasstatHelp },
+                   { "seasstd1",  FieldFunc_Std1,  0, SeasstatHelp },
+                   { "seasvar",   FieldFunc_Var,   0, SeasstatHelp },
+                   { "seasvar1",  FieldFunc_Var1,  0, SeasstatHelp } },
     // clang-format on
     .aliases = {},
     .mode = EXPOSED,     // Module mode: 0:intern 1:extern
@@ -59,8 +59,8 @@ public:
 private:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   TimeStat timestatDate{ TimeStat::MEAN };
   CdiDateTime vDateTime0{};
@@ -72,8 +72,8 @@ private:
 
   cdo::StepStat2D stepStat;
 
-  int maxRecords;
-  std::vector<RecordInfo> recordList;
+  int maxFields;
+  std::vector<FieldInfo> fieldInfoList;
 
   DateTimeList dtlist;
   VarList varList1;
@@ -107,8 +107,8 @@ public:
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
 
-    maxRecords = varList1.numRecords();
-    recordList = std::vector<RecordInfo>(maxRecords);
+    maxFields = varList1.maxFields();
+    fieldInfoList = std::vector<FieldInfo>(maxFields);
 
     dtlist.set_stat(timestatDate);
     dtlist.set_calendar(taxisInqCalendar(taxisID1));
@@ -122,7 +122,7 @@ public:
   {
     Field field;
 
-    auto seasonStart = get_season_start();
+    auto seasonStartIsDecember = (get_season_start() == SeasonStart::DEC);
     auto seasonNames = get_season_name();
 
     auto numSteps = varList1.numSteps();
@@ -134,11 +134,11 @@ public:
       {
         int numSets = 0;
         bool newseas = false;
-        int nrecs = 0;
+        int numFields = 0;
         while (true)
           {
-            nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-            if (nrecs == 0) break;
+            numFields = cdo_stream_inq_timestep(streamID1, tsID);
+            if (numFields == 0) break;
 
             if (numSteps > 1) progress.update((tsID + 1.0) / numSteps);
 
@@ -147,7 +147,7 @@ public:
 
             auto month = decode_month(vDateTime.date);
             auto newmon = month;
-            if (seasonStart == SeasonStart::DEC && newmon == 12) newmon = 0;
+            if (seasonStartIsDecember && newmon == 12) newmon = 0;
 
             auto seas = month_to_season(month);
 
@@ -169,12 +169,12 @@ public:
 
             oldmon = newmon;
 
-            for (int recID = 0; recID < nrecs; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                auto [varID, levelID] = cdo_inq_record(streamID1);
-                if (tsID == 0) recordList[recID].set(varID, levelID);
+                auto [varID, levelID] = cdo_inq_field(streamID1);
+                if (tsID == 0) fieldInfoList[fieldID].set(varID, levelID);
                 field.init(varList1.vars[varID]);
-                cdo_read_record(streamID1, field);
+                cdo_read_field(streamID1, field);
                 stepStat.add_field(field, varID, levelID, numSets);
               }
 
@@ -183,9 +183,9 @@ public:
             tsID++;
           }
 
-        if (nrecs == 0 && numSets == 0) break;
+        if (numFields == 0 && numSets == 0) break;
 
-        cdo::records_process(recordList, varList1, stepStat, numSets);
+        cdo::fields_process(fieldInfoList, varList1, stepStat, numSets);
 
         if (Options::cdoVerbose)
           cdo_print("season: %3d %3s  start: %s  end: %s ntimesteps: %ld", nseason, seasonNames[seas0],
@@ -196,9 +196,9 @@ public:
                       (numSets == 1) ? "" : "s");
 
         dtlist.stat_taxis_def_timestep(taxisID2, numSets);
-        cdo::write_out_stream(streamID2, recordList, varList1, stepStat, otsID);
+        cdo::write_out_stream(streamID2, fieldInfoList, varList1, stepStat, otsID);
 
-        if (nrecs == 0) break;
+        if (numFields == 0) break;
         otsID++;
       }
   }
