@@ -74,12 +74,12 @@ gengridxyvals(int gridtype, int gridID1, int gridID2, long nlon, long nlat, long
   if (lxvals) gridInqXvals(gridID1, xvals1.data());
   if (lyvals) gridInqYvals(gridID1, yvals1.data());
 
-  const auto &lat1 = selboxInfo.lat1;
-  const auto &lat2 = selboxInfo.lat2;
-  const auto &lon11 = selboxInfo.lon11;
-  const auto &lon12 = selboxInfo.lon12;
-  const auto &lon21 = selboxInfo.lon21;
-  const auto &lon22 = selboxInfo.lon22;
+  auto const &lat1 = selboxInfo.lat1;
+  auto const &lat2 = selboxInfo.lat2;
+  auto const &lon11 = selboxInfo.lon11;
+  auto const &lon12 = selboxInfo.lon12;
+  auto const &lon21 = selboxInfo.lon21;
+  auto const &lon22 = selboxInfo.lon22;
 
   if (gridtype == GRID_CURVILINEAR)
     {
@@ -127,12 +127,12 @@ gengridXboundsCurvi(int gridID1, long nlon, long nlat, long nlon2, long nlat2, c
 
   gridInqXbounds(gridID1, xbounds1.data());
 
-  const auto &lat1 = selboxInfo.lat1;
-  const auto &lat2 = selboxInfo.lat2;
-  const auto &lon11 = selboxInfo.lon11;
-  const auto &lon12 = selboxInfo.lon12;
-  const auto &lon21 = selboxInfo.lon21;
-  const auto &lon22 = selboxInfo.lon22;
+  auto const &lat1 = selboxInfo.lat1;
+  auto const &lat2 = selboxInfo.lat2;
+  auto const &lon11 = selboxInfo.lon11;
+  auto const &lon12 = selboxInfo.lon12;
+  auto const &lon21 = selboxInfo.lon21;
+  auto const &lon22 = selboxInfo.lon22;
   for (long ilat = lat1; ilat <= lat2; ilat++)
     {
       for (long ilon = 4 * lon21; ilon < 4 * (lon22 + 1); ilon++) *pxbounds2++ = xbounds1[4 * ilat * nlon + ilon];
@@ -151,12 +151,12 @@ gengridYboundsCurvi(int gridID1, long nlon, long nlat, long nlon2, long nlat2, c
 
   gridInqYbounds(gridID1, ybounds1.data());
 
-  const auto &lat1 = selboxInfo.lat1;
-  const auto &lat2 = selboxInfo.lat2;
-  const auto &lon11 = selboxInfo.lon11;
-  const auto &lon12 = selboxInfo.lon12;
-  const auto &lon21 = selboxInfo.lon21;
-  const auto &lon22 = selboxInfo.lon22;
+  auto const &lat1 = selboxInfo.lat1;
+  auto const &lat2 = selboxInfo.lat2;
+  auto const &lon11 = selboxInfo.lon11;
+  auto const &lon12 = selboxInfo.lon12;
+  auto const &lon21 = selboxInfo.lon21;
+  auto const &lon22 = selboxInfo.lon22;
   for (long ilat = lat1; ilat <= lat2; ilat++)
     {
       for (long ilon = 4 * lon21; ilon < 4 * (lon22 + 1); ilon++) *pybounds2++ = ybounds1[4 * ilat * nlon + ilon];
@@ -175,10 +175,10 @@ gengridXboundsRect2D(int gridID1, long nlon, long nlon2, const SelboxInfo &selbo
 
   gridInqXbounds(gridID1, xbounds1.data());
 
-  const auto &lon11 = selboxInfo.lon11;
-  const auto &lon12 = selboxInfo.lon12;
-  const auto &lon21 = selboxInfo.lon21;
-  const auto &lon22 = selboxInfo.lon22;
+  auto const &lon11 = selboxInfo.lon11;
+  auto const &lon12 = selboxInfo.lon12;
+  auto const &lon21 = selboxInfo.lon21;
+  auto const &lon22 = selboxInfo.lon22;
   for (long i = 2 * lon21; i < 2 * (lon22 + 1); ++i) *pxbounds2++ = xbounds1[i];
   for (long i = 2 * lon11; i < 2 * (lon12 + 1); ++i) *pxbounds2++ = xbounds1[i];
 
@@ -206,8 +206,8 @@ gengridYboundsRect2D(int gridID1, long nlat, long nlat2, const SelboxInfo &selbo
 
   gridInqYbounds(gridID1, ybounds1.data());
 
-  const auto &lat1 = selboxInfo.lat1;
-  const auto &lat2 = selboxInfo.lat2;
+  auto const &lat1 = selboxInfo.lat1;
+  auto const &lat2 = selboxInfo.lat2;
   for (long i = 2 * lat1; i < 2 * (lat2 + 1); ++i) *pybounds2++ = ybounds1[i];
 }
 
@@ -255,12 +255,12 @@ gengridYbounds(int gridID1, int gridID2, long nlon, long nlat, long nlon2, long
 static int
 gengrid(int gridID1, const SelboxInfo &selboxInfo)
 {
-  const auto &lat1 = selboxInfo.lat1;
-  const auto &lat2 = selboxInfo.lat2;
-  const auto &lon11 = selboxInfo.lon11;
-  const auto &lon12 = selboxInfo.lon12;
-  const auto &lon21 = selboxInfo.lon21;
-  const auto &lon22 = selboxInfo.lon22;
+  auto const &lat1 = selboxInfo.lat1;
+  auto const &lat2 = selboxInfo.lat2;
+  auto const &lon11 = selboxInfo.lon11;
+  auto const &lon12 = selboxInfo.lon12;
+  auto const &lon21 = selboxInfo.lon21;
+  auto const &lon22 = selboxInfo.lon22;
   long nlon = gridInqXsize(gridID1);
   long nlat = gridInqYsize(gridID1);
 
@@ -383,10 +383,10 @@ gengridcell(int gridID1, size_t gridsize2, const std::vector<long> &cellidx)
 
   if (gridHasBounds(gridID1))
     {
-      auto nv = gridInqNvertex(gridID1);
       const auto *xbounds1 = gridInqXboundsPtr(gridID1);
       const auto *ybounds1 = gridInqYboundsPtr(gridID1);
 
+      auto nv = gridInqNvertex(gridID1);
       gridDefNvertex(gridID2, nv);
 
       Varray<double> xybounds2(nv * gridsize2);
@@ -459,20 +459,16 @@ gen_lonlat_selbox_reg2d(int gridID, double xlon1, double xlon2, double xlat1, do
     {
       // while ( nlon == 1 || (xvals[nlon-1] - xvals[0]) >= 360 ) nlon--;
 
-      for (lon21 = 0; lon21 < nlon && xvals[lon21] < xlon1; lon21++)
-        ;
-      for (lon22 = lon21; lon22 < nlon && xvals[lon22] < xlon2; lon22++)
-        ;
+      for (lon21 = 0; lon21 < nlon && xvals[lon21] < xlon1; lon21++);
+      for (lon22 = lon21; lon22 < nlon && xvals[lon22] < xlon2; lon22++);
 
       if (lon22 >= nlon || xvals[lon22] > xlon2) lon22--;
 
       xlon1 -= 360;
       xlon2 -= 360;
 
-      for (lon11 = 0; xvals[lon11] < xlon1; lon11++)
-        ;
-      for (lon12 = lon11; lon12 < nlon && xvals[lon12] < xlon2; lon12++)
-        ;
+      for (lon11 = 0; xvals[lon11] < xlon1; lon11++);
+      for (lon12 = lon11; lon12 < nlon && xvals[lon12] < xlon2; lon12++);
 
       // lon12--;
       if (lon12 >= nlon || xvals[lon12] > xlon2) lon12--;
@@ -487,34 +483,26 @@ gen_lonlat_selbox_reg2d(int gridID, double xlon1, double xlon2, double xlat1, do
         {
           if (xlat1 > xlat2)
             {
-              for (lat1 = 0; lat1 < nlat && yvals[lat1] > xlat1; lat1++)
-                ;
-              for (lat2 = nlat - 1; lat2 && yvals[lat2] < xlat2; lat2--)
-                ;
+              for (lat1 = 0; lat1 < nlat && yvals[lat1] > xlat1; lat1++);
+              for (lat2 = nlat - 1; lat2 && yvals[lat2] < xlat2; lat2--);
             }
           else
             {
-              for (lat1 = 0; lat1 < nlat && yvals[lat1] > xlat2; lat1++)
-                ;
-              for (lat2 = nlat - 1; lat2 && yvals[lat2] < xlat1; lat2--)
-                ;
+              for (lat1 = 0; lat1 < nlat && yvals[lat1] > xlat2; lat1++);
+              for (lat2 = nlat - 1; lat2 && yvals[lat2] < xlat1; lat2--);
             }
         }
       else
         {
           if (xlat1 < xlat2)
             {
-              for (lat1 = 0; lat1 < nlat && yvals[lat1] < xlat1; lat1++)
-                ;
-              for (lat2 = nlat - 1; lat2 && yvals[lat2] > xlat2; lat2--)
-                ;
+              for (lat1 = 0; lat1 < nlat && yvals[lat1] < xlat1; lat1++);
+              for (lat2 = nlat - 1; lat2 && yvals[lat2] > xlat2; lat2--);
             }
           else
             {
-              for (lat1 = 0; lat1 < nlat && yvals[lat1] < xlat2; lat1++)
-                ;
-              for (lat2 = nlat - 1; lat2 && yvals[lat2] > xlat1; lat2--)
-                ;
+              for (lat1 = 0; lat1 < nlat && yvals[lat1] < xlat2; lat1++);
+              for (lat2 = nlat - 1; lat2 && yvals[lat2] > xlat1; lat2--);
             }
         }
 
@@ -540,7 +528,8 @@ gen_lonlat_selbox_curv(int gridID, double xlon1, double xlon2, double xlat1, dou
 
   auto grid_is_circular = gridIsCircular(gridID);
 
-  Varray<double> xvals(gridsize), yvals(gridsize);
+  Varray<double> xvals(gridsize);
+  Varray<double> yvals(gridsize);
 
   gridInqXvals(gridID, xvals.data());
   gridInqYvals(gridID, yvals.data());
@@ -919,9 +908,9 @@ window_box(int nwpv, const T1 *array1, int gridID, T2 *array2, long lat1, long l
 }
 
 static void
-window_box(const Field &field1, Field &field2, long lat1, long lat2, long lon11, long lon12, long lon21, long lon22)
+window_box(Field const &field1, Field &field2, long lat1, long lat2, long lon11, long lon12, long lon21, long lon22)
 {
-  auto func = [&](const auto &v1, auto &v2) {
+  auto func = [&](auto const &v1, auto &v2) {
     window_box(field1.nwpv, v1.data(), field1.grid, v2.data(), lat1, lat2, lon11, lon12, lon21, lon22);
   };
   field_operation2(func, field1, field2);
@@ -949,7 +938,7 @@ window_cell(int nwpv, const Varray<T1> &v1, Varray<T2> &v2, long n, const std::v
 }
 
 void
-window_cell(const Field &field1, Field &field2, const std::vector<long> &cellidx)
+window_cell(Field const &field1, Field &field2, const std::vector<long> &cellidx)
 {
   auto func = [&](auto &v1, auto &v2) { window_cell(field1.nwpv, v1, v2, field2.gridsize, cellidx); };
   field_operation2(func, field1, field2);
@@ -960,8 +949,8 @@ get_selboxInfoList(int vlistID1, int vlistID2, bool operIndexBox)
 {
   std::vector<SelboxInfo> selboxInfoList;
 
-  auto ngrids = vlistNumGrids(vlistID1);
-  for (int index = 0; index < ngrids; ++index)
+  auto numGrids = vlistNumGrids(vlistID1);
+  for (int index = 0; index < numGrids; ++index)
     {
       auto gridID1 = vlistGrid(vlistID1, index);
       if (gridInqSize(gridID1) == 1) continue;
@@ -993,7 +982,7 @@ get_selboxInfoList(int vlistID1, int vlistID2, bool operIndexBox)
 
   if (Options::cdoVerbose)
     {
-      for (const auto &sb : selboxInfoList)
+      for (auto const &sb : selboxInfoList)
         if (sb.gridtype != GRID_UNSTRUCTURED && !is_healpix_grid(sb.gridID1))
           {
             cdo_print("box1 - idx1,idx2,idy1,idy2: %ld,%ld,%ld,%ld", sb.lon21 + 1, sb.lon22 + 1, sb.lat1 + 1, sb.lat2 + 1);
@@ -1012,7 +1001,7 @@ get_processVars(const VarList &varList, const std::vector<SelboxInfo> &selboxInf
   std::vector<bool> processVars(numVars, false);
 
   int varID;
-  for (const auto &sb : selboxInfoList)
+  for (auto const &sb : selboxInfoList)
     {
       for (varID = 0; varID < numVars; ++varID)
         if (sb.gridID1 == varList.vars[varID].gridID) processVars[varID] = true;
@@ -1029,7 +1018,7 @@ get_processVars(const VarList &varList, const std::vector<SelboxInfo> &selboxInf
 static const SelboxInfo &
 select_selboxInfo(int gridID, const std::vector<SelboxInfo> &selboxInfoList)
 {
-  for (const auto &selboxInfo : selboxInfoList)
+  for (auto const &selboxInfo : selboxInfoList)
     if (gridID == selboxInfo.gridID1) return selboxInfo;
 
   cdo_abort("Internal problem, grid not found!");
@@ -1056,13 +1045,13 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   VarList varList1;
   VarList varList2;
 
-  int vlistID2;
+  int vlistID2{ CDI_UNDEFID };
   int operatorID;
 
   std::vector<SelboxInfo> selboxInfoList;
@@ -1107,26 +1096,26 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
-            const auto &var1 = varList1.vars[varID];
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            auto const &var1 = varList1.vars[varID];
             field1.init(var1);
-            cdo_read_record(streamID1, field1);
+            cdo_read_field(streamID1, field1);
 
-            cdo_def_record(streamID2, varID, levelID);
+            cdo_def_field(streamID2, varID, levelID);
 
             if (processVars[varID])
               {
                 field2.init(varList2.vars[varID]);
 
-                const auto &sb = select_selboxInfo(var1.gridID, selboxInfoList);
+                auto const &sb = select_selboxInfo(var1.gridID, selboxInfoList);
 
                 if (operatorID == SELLONLATBOX && (sb.gridtype == GRID_UNSTRUCTURED || is_healpix_grid(sb.gridID1)))
                   window_cell(field1, field2, sb.cellidx);
@@ -1135,9 +1124,9 @@ public:
 
                 if (field1.numMissVals) field_num_mv(field2);
 
-                cdo_write_record(streamID2, field2);
+                cdo_write_field(streamID2, field2);
               }
-            else { cdo_write_record(streamID2, field1); }
+            else { cdo_write_field(streamID2, field1); }
           }
 
         tsID++;
