@@ -24,7 +24,7 @@
 #include "cdi_lockedIO.h"
 #include "cdo_cdi_wrapper.h"
 
-int cdo_read_timestepmask(const std::string &maskfile, std::vector<bool> &imask);
+int cdo_read_timestepmask(std::string const &maskfile, std::vector<bool> &imask);
 
 std::string
 dom_to_string(CdiDate date)
@@ -32,7 +32,7 @@ dom_to_string(CdiDate date)
   int year, month, day;
   cdiDate_decode(date, &year, &month, &day);
 
-  const char *cmons[] = { "", "JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC" };
+  char const *cmons[] = { "", "JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC" };
   if (month < 0 || month > 12) month = 0;
   char cstr[32];
   std::snprintf(cstr, sizeof(cstr), "%d%s", day, cmons[month]);
@@ -41,9 +41,9 @@ dom_to_string(CdiDate date)
 }
 
 static void
-write_const_vars(CdoStreamID streamID2, const VarList &varList, Varray2D<double> &varsData2)
+write_const_vars(CdoStreamID streamID2, VarList const &varList, Varray2D<double> &varsData2)
 {
-  for (const auto &var : varList.vars)
+  for (auto const &var : varList.vars)
     {
       if (varsData2[var.ID].size())
         {
@@ -51,8 +51,8 @@ write_const_vars(CdoStreamID streamID2, const VarList &varList, Varray2D<double>
             {
               auto pdata = &varsData2[var.ID][var.gridsize * levelID2];
               auto numMissVals = array_num_mv(var.gridsize, pdata, var.missval);
-              cdo_def_record(streamID2, var.ID, levelID2);
-              cdo_write_record(streamID2, pdata, numMissVals);
+              cdo_def_field(streamID2, var.ID, levelID2);
+              cdo_write_field(streamID2, pdata, numMissVals);
             }
           varsData2[var.ID].clear();
           varsData2[var.ID].shrink_to_fit();
@@ -61,7 +61,7 @@ write_const_vars(CdoStreamID streamID2, const VarList &varList, Varray2D<double>
 }
 
 static void
-eval_timestepmask(const std::string &maskfile, KVList &kvlist)
+eval_timestepmask(std::string const &maskfile, KVList &kvlist)
 {
   std::vector<bool> imask;
   auto n = cdo_read_timestepmask(maskfile, imask);
@@ -94,9 +94,9 @@ eval_timestepmask(const std::string &maskfile, KVList &kvlist)
 }
 
 static bool
-has_selected_params(const VarList &varList, int vlistID)
+has_selected_params(VarList const &varList, int vlistID)
 {
-  for (auto &var : varList.vars)
+  for (auto const &var : varList.vars)
     {
       for (int levelID = 0; levelID < var.nlevels; ++levelID)
         if (vlistInqFlag(vlistID, var.ID, levelID) == true) return true;
@@ -106,9 +106,9 @@ has_selected_params(const VarList &varList, int vlistID)
 }
 
 static bool
-has_const_vars(const VarList &varList, const std::vector<bool> &processVars)
+has_const_vars(VarList const &varList, std::vector<bool> const &processVars)
 {
-  for (auto &var : varList.vars)
+  for (auto const &var : varList.vars)
     {
       if (processVars[var.ID] && var.isConstant) return true;
     }
@@ -138,6 +138,15 @@ public:
     .constraints = { -1, 1, NoRestriction },
   };
   inline static RegisterEntry<Select> registration = RegisterEntry<Select>(module);
+  inline static CdoModule module1 = {
+    .name = "Select1",
+    .operators = { { "select1", 0, 0, "parameter list", SelectHelp }, { "delete1", 0, 0, "parameter list", SelectHelp } },
+    .aliases = {},
+    .mode = INTERNAL,    // Module mode: 0:intern 1:extern
+    .number = CDI_BOTH,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Select> registration1 = RegisterEntry<Select>(module1);
 
   int SELECT, DELETE;
   CdoStreamID streamID2 = CDO_STREAM_UNDEF;
@@ -162,7 +171,7 @@ public:
     operator_input_arg(cdo_operator_enter(operatorID));
 
     auto argc = cdo_operator_argc();
-    const auto &argnames = cdo_get_oper_argv();
+    auto const &argnames = cdo_get_oper_argv();
 
     if (argc == 0) cdo_abort("Parameter missing!");
 
@@ -274,7 +283,7 @@ public:
                 xresult = false;
                 for (int varID = 0; varID < numVars; ++varID)
                   {
-                    const auto &var = varList1.vars[varID];
+                    auto const &var = varList1.vars[varID];
                     for (int levelID = 0; levelID < var.nlevels; ++levelID) vlistDefFlag(vlistID1, varID, levelID, true);
                   }
               }
@@ -292,7 +301,7 @@ public:
 
             for (int varID = 0; varID < numVars; ++varID)
               {
-                const auto &var = varList1.vars[varID];
+                auto const &var = varList1.vars[varID];
 
                 code = var.code;
 
@@ -366,7 +375,7 @@ public:
               {
                 if (processVars[varID])
                   {
-                    const auto &var = varList1.vars[varID];
+                    auto const &var = varList1.vars[varID];
                     if (zaxisInqType(var.zaxisID) == ZAXIS_HYBRID)
                       {
                         auto psvarid = varList_get_psvarid(varList1, var.zaxisID);
@@ -379,7 +388,7 @@ public:
               {
                 if (processVars[varID])
                   {
-                    const auto &var = varList1.vars[varID];
+                    auto const &var = varList1.vars[varID];
                     for (int levelID = 0; levelID < var.nlevels; ++levelID)
                       {
                         levidx = levelID + 1;
@@ -436,7 +445,7 @@ public:
                     for (int varID = 0; varID < numVars; ++varID)
                       {
                         processVars[varID] = true;
-                        const auto &var = varList1.vars[varID];
+                        auto const &var = varList1.vars[varID];
                         for (int levelID = 0; levelID < var.nlevels; ++levelID) vlistDefFlag(vlistID1, varID, levelID, true);
                       }
                   }
@@ -448,7 +457,7 @@ public:
             vlistID0 = vlistDuplicate(vlistID1);
             for (int varID = 0; varID < numVars; ++varID)
               {
-                const auto &var = varList1.vars[varID];
+                auto const &var = varList1.vars[varID];
                 for (int levelID = 0; levelID < var.nlevels; ++levelID)
                   vlistDefFlag(vlistID0, varID, levelID, vlistInqFlag(vlistID1, varID, levelID));
               }
@@ -516,8 +525,8 @@ public:
 
             SELINFO_GET_VAL(startdate, 0, &startdate);
             SELINFO_GET_VAL(enddate, 0, &enddate);
-            if (SELINFO_NVAL(startdate)) fstartdate = date_str_to_double(startdate, 0);
-            if (SELINFO_NVAL(enddate)) fenddate = date_str_to_double(enddate, 1);
+            if (SELINFO_NVAL(startdate)) fstartdate = datestr_to_double(startdate, 0);
+            if (SELINFO_NVAL(enddate)) fenddate = datestr_to_double(enddate, 1);
           }
         else { varList_compare(VarList(vlistID0), varList1); }
 
@@ -533,8 +542,8 @@ public:
         int tsID1 = 0;
         while (true)
           {
-            auto nrecs = cdo_stream_inq_timestep(streamID1, tsID1);
-            if (nrecs == 0) break;
+            auto numFields = cdo_stream_inq_timestep(streamID1, tsID1);
+            if (numFields == 0) break;
 
             timestep++;
             auto copyTimestep = true;
@@ -638,12 +647,12 @@ public:
                   }
                 cdo_def_timestep(streamID2, tsID2);
 
-                for (int recID = 0; recID < nrecs; ++recID)
+                for (int fieldID = 0; fieldID < numFields; ++fieldID)
                   {
-                    auto [varID, levelID] = cdo_inq_record(streamID1);
+                    auto [varID, levelID] = cdo_inq_field(streamID1);
                     if (vlistInqFlag(vlistID0, varID, levelID) == true)
                       {
-                        const auto &var = varList1.vars[varID];
+                        auto const &var = varList1.vars[varID];
 
                         if (hasConstVars && tsID2 > 0 && tsID1 == 0)
                           if (var.isConstant) continue;
@@ -652,13 +661,13 @@ public:
                         auto levelID2 = vlistFindLevel(vlistID2, varID, levelID);
                         if (copyConstVars && tsID2 == 0) write_const_vars(streamID2, varList2, varsData2);
 
-                        cdo_def_record(streamID2, varID2, levelID2);
-                        if (dataIsUnchanged) { cdo_copy_record(streamID2, streamID1); }
+                        cdo_def_field(streamID2, varID2, levelID2);
+                        if (dataIsUnchanged) { cdo_copy_field(streamID2, streamID1); }
                         else
                           {
                             field.init(var);
-                            cdo_read_record(streamID1, field);
-                            cdo_write_record(streamID2, field);
+                            cdo_read_field(streamID1, field);
+                            cdo_write_field(streamID2, field);
                           }
                       }
                   }
@@ -669,20 +678,19 @@ public:
               }
             else if (copyConstVars && indf == 0 && tsID1 == 0)
               {
-                for (int recID = 0; recID < nrecs; ++recID)
+                for (int fieldID = 0; fieldID < numFields; ++fieldID)
                   {
-                    int varID, levelID;
-                    cdo_inq_record(streamID1, &varID, &levelID);
+                    auto [varID, levelID] = cdo_inq_field(streamID1);
                     if (vlistInqFlag(vlistID0, varID, levelID) == true)
                       {
                         auto varID2 = vlistFindVar(vlistID2, varID);
-                        const auto &var = varList2.vars[varID2];
+                        auto const &var = varList2.vars[varID2];
                         if (var.isConstant)
                           {
                             auto levelID2 = vlistFindLevel(vlistID2, varID, levelID);
                             if (levelID == 0) varsData2[varID2].resize(var.gridsize * var.nlevels);
                             size_t numMissVals;
-                            cdo_read_record(streamID1, &varsData2[varID2][var.gridsize * levelID2], &numMissVals);
+                            cdo_read_field(streamID1, &varsData2[varID2][var.gridsize * levelID2], &numMissVals);
                           }
                       }
                   }
