@@ -170,20 +170,18 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  int vlistID1;
-  int vlistID2;
+  int vlistID1{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
   int operatorID;
   int numVars;
-  size_t gridsizemax;
 
   bool lcopy = false;
 
   VarList varList1;
-  Varray<double> array;
 
 public:
   void
@@ -239,7 +237,7 @@ public:
 
     for (int varID = 0; varID < numVars; ++varID)
       {
-        const auto &var = varList1.vars[varID];
+        auto const &var = varList1.vars[varID];
         auto ltype = zaxis_to_ltype(var.zaxisID);
 
         for (int levelID = 0; levelID < var.nlevels; ++levelID)
@@ -360,8 +358,8 @@ public:
                     break;
                   }
               }  // end for ( .. NUMTUPLES
-          }      // end for ( levelID
-      }          // end for ( varID
+          }  // end for ( levelID
+      }  // end for ( varID
 
     Debug(cdoDebugExt, " Writing the selected fields ...");
 
@@ -383,29 +381,28 @@ public:
 
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
-
-    gridsizemax = vlistGridsizeMax(vlistID1);
-    if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
-    array = Varray<double>(gridsizemax);
   }
 
   void
   run() override
   {
+    auto gridsizeMax = varList1.gridsizeMax();
+    if (vlistNumber(vlistID1) != CDI_REAL) gridsizeMax *= 2;
+    Varray<double> array(gridsizeMax);
+
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            const auto &var = varList1.vars[varID];
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            auto const &var = varList1.vars[varID];
 
             if (vlistInqFlag(vlistID1, varID, levelID) == true)
               {
@@ -476,33 +473,33 @@ public:
                               code, ltype, (int) (level), varID, levelID, varID2, levelID2);
                     continue;
                   }
-                cdo_def_record(streamID2, varID2, levelID2);
+                cdo_def_field(streamID2, varID2, levelID2);
 
                 if (lcopy)
                   {
                     if (cdoDebugExt)
-                      cdo_print(" Copying record [%4d] with (code %3i, ltype %3i, level %3i)   [varID(%d),levelID(%d)]", recID,
+                      cdo_print(" Copying record [%4d] with (code %3i, ltype %3i, level %3i)   [varID(%d),levelID(%d)]", fieldID,
                                 code, ltype, (int) (level), varID, levelID);
-                    cdo_copy_record(streamID2, streamID1);
+                    cdo_copy_field(streamID2, streamID1);
                   }
                 else
                   {
                     size_t numMissVals;
-                    cdo_read_record(streamID1, array.data(), &numMissVals);
+                    cdo_read_field(streamID1, array.data(), &numMissVals);
 
                     if (!simpleMath)
                       {
                         if (cdoDebugExt)
-                          cdo_print(" Writing record [%4d] with (code %3i, ltype %3i, level %3i)   [varID(%d),levelID(%d)]", recID,
-                                    code, ltype, (int) (level), varID, levelID);
+                          cdo_print(" Writing record [%4d] with (code %3i, ltype %3i, level %3i)   [varID(%d),levelID(%d)]",
+                                    fieldID, code, ltype, (int) (level), varID, levelID);
                       }
                     else  // 1:  simple array arithmetics ( *,+)
                       {
                         if (cdoDebugExt)
                           cdo_print(" Writing record [%4d] with (code %3i, ltype %3i, level %3i)   [varID(%d),levelID(%d)]; "
                                     "SCALE=%f; OFFSET=%f",
-                                    recID, code, ltype, (int) (level), varID, levelID, scale, offset);
-                        for (size_t li = 0; li < gridsizemax; ++li)
+                                    fieldID, code, ltype, (int) (level), varID, levelID, scale, offset);
+                        for (size_t li = 0; li < gridsizeMax; ++li)
                           if (!fp_is_equal(array[li], var.missval))
                             {
                               array[li] = scale * (array[li] - offset);
@@ -511,10 +508,10 @@ public:
                             }
                       }
 
-                    cdo_write_record(streamID2, array.data(), numMissVals);
+                    cdo_write_field(streamID2, array.data(), numMissVals);
                   }  // end of else ( lcopy )
-              }      // end if ( vlistInqFlag(vlistID1, varID, levelID) == true )
-          }          // end for ( recID ..
+              }  // end if ( vlistInqFlag(vlistID1, varID, levelID) == true )
+          }  // end for ( fieldID ..
 
         tsID++;
       }  // end while
@@ -535,7 +532,7 @@ public:
 TUPLEREC *
 TUPLERECNew()
 {
-  TUPLEREC *tpl = (TUPLEREC *) malloc(sizeof(TUPLEREC));
+  TUPLEREC *tpl = (TUPLEREC *) std::malloc(sizeof(TUPLEREC));
 
   tpl->ncodes = 0;
 
@@ -655,8 +652,8 @@ strContains(char *str, const char *substr)
 
   str = removeSpaces(str);
 
-  size_t lensub = strlen(substr);
-  size_t lenstr = strlen(str);
+  size_t lensub = std::strlen(substr);
+  size_t lenstr = std::strlen(str);
 
   if (lensub > lenstr)
     {
@@ -694,7 +691,7 @@ findParamEnd(char *str)
       return ptrEnding;
     }
   Debug(cdoDebugExt >= 100, " ptrEnding=end-of-string");
-  size_t lenstr = strlen(str);
+  size_t lenstr = std::strlen(str);
   ptrEnding = str + lenstr;
   ptrEnding = removeSpaces(ptrEnding);
   return ptrEnding;
@@ -717,7 +714,7 @@ findTupleEnd(char *str)
       return ptrEnding;
     }
   Debug(cdoDebugExt >= 100, " findTupleEnd=end-of-string");
-  size_t lenstr = strlen(str);
+  size_t lenstr = std::strlen(str);
   ptrEnding = str + lenstr;
   ptrEnding = removeSpaces(ptrEnding);
   return ptrEnding;
@@ -740,7 +737,7 @@ readlineForParsing(std::ifstream &gfile, char *strToParsePtr, char *line)
     {
       if (cdoDebugExt >= 30) cdo_print("%s(): Parsing selection string:  %s", __func__, strToParsePtr);
       char *tpEnd = nullptr;
-      if (strlen(strToParsePtr) > 0) tpEnd = findTupleEnd(strToParsePtr);
+      if (std::strlen(strToParsePtr) > 0) tpEnd = findTupleEnd(strToParsePtr);
       if (tpEnd == nullptr)
         {
           Debug(cdoDebugExt >= 100, "%s(): End of selection string reached.", __func__);
@@ -749,7 +746,7 @@ readlineForParsing(std::ifstream &gfile, char *strToParsePtr, char *line)
       else
         {
           tpEnd[0] = 0;
-          if (strlen(strToParsePtr) <= MAX_LINE_LEN) std::strcpy(line, strToParsePtr);
+          if (std::strlen(strToParsePtr) <= MAX_LINE_LEN) std::strcpy(line, strToParsePtr);
           Debug(cdoDebugExt >= 100, "%s(): Current selection line=%s", __func__, line);
           strToParsePtr = tpEnd + 1;
           return strToParsePtr;
@@ -790,7 +787,7 @@ multiSelectionParser(const char *filenameOrString)
       std::strncpy(strToParse, filenameOrString, MAX_LINE_LEN - 1);
       strToParsePtr = &strToParse[0];
       if (strToParsePtr[0] == '{') strToParsePtr++;
-      int strLn = strlen(strToParsePtr);
+      int strLn = std::strlen(strToParsePtr);
       if (strToParsePtr[strLn - 1] == '}') strToParsePtr[strLn - 1] = 0;
       Debug(cdoDebugExt, " Parsing selection string:  %s", strToParsePtr);
     }
@@ -838,7 +835,7 @@ multiSelectionParser(const char *filenameOrString)
           tuplerec->sel_or_del_or_change = selectionRec;
           push_backSelTuple(tuplerec);
           // SELECT, PARAMETER=11/17, LEVTYPE=105, LEVEL=2
-          while ((pline != nullptr) && (strlen(pline) != 0))
+          while ((pline != nullptr) && (std::strlen(pline) != 0))
             {
               Debug(cdoDebugExt >= 100, "pline='%s'", pline);
               strpos = strContains(pline, "PARAMETER=");
@@ -849,7 +846,7 @@ multiSelectionParser(const char *filenameOrString)
                   pline = removeSpaces(pline);
                   parEnd = findParamEnd(pline);
                   if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
-                  while ((pline != parEnd) && (strlen(pline) > 0))
+                  while ((pline != parEnd) && (std::strlen(pline) > 0))
                     {
                       pline = removeSpaces(pline);
                       if (pline[0] == '*')
@@ -874,7 +871,7 @@ multiSelectionParser(const char *filenameOrString)
                   pline = removeSpaces(pline);
                   parEnd = findParamEnd(pline);
                   if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
-                  while ((pline != parEnd) && (strlen(pline) > 0))
+                  while ((pline != parEnd) && (std::strlen(pline) > 0))
                     {
                       pline = removeSpaces(pline);
                       if (pline[0] == '*')
@@ -908,7 +905,7 @@ multiSelectionParser(const char *filenameOrString)
                   pline = removeSpaces(pline);
                   parEnd = findParamEnd(pline);
                   if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
-                  while ((pline != parEnd) && (strlen(pline) > 0))
+                  while ((pline != parEnd) && (std::strlen(pline) > 0))
                     {
                       pline = removeSpaces(pline);
                       val = (pline[0] == '*') ? -1 : atoi(pline);
@@ -931,7 +928,7 @@ multiSelectionParser(const char *filenameOrString)
                   pline = removeSpaces(pline);
                   parEnd = findParamEnd(pline);
                   if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
-                  while ((pline != parEnd) && (strlen(pline) > 0))
+                  while ((pline != parEnd) && (std::strlen(pline) > 0))
                     {
                       pline = removeSpaces(pline);
                       floatval = atof(pline);
@@ -954,7 +951,7 @@ multiSelectionParser(const char *filenameOrString)
                   pline = removeSpaces(pline);
                   parEnd = findParamEnd(pline);
                   if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
-                  while ((pline != parEnd) && (strlen(pline) > 0))
+                  while ((pline != parEnd) && (std::strlen(pline) > 0))
                     {
                       pline = removeSpaces(pline);
                       floatval = atof(pline);
@@ -1006,7 +1003,7 @@ multiSelectionParser(const char *filenameOrString)
           tuplerec->sel_or_del_or_change = selectionRec;
           push_backSelTuple(tuplerec);
           // (33/34; 105; 10)
-          while ((pline != nullptr) && (strlen(pline) != 0) && (pline[0] != ')'))
+          while ((pline != nullptr) && (std::strlen(pline) != 0) && (pline[0] != ')'))
             {
               Debug(cdoDebugExt >= 100, "[1]: pline='%s'", pline);
               // 1st is code
@@ -1014,7 +1011,7 @@ multiSelectionParser(const char *filenameOrString)
                 pline = removeSpaces(pline);
                 parEnd = findParamEnd(pline);
                 if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
-                while ((pline != parEnd) && (strlen(pline) > 0))
+                while ((pline != parEnd) && (std::strlen(pline) > 0))
                   {
                     pline = removeSpaces(pline);
                     if (pline[0] == '*')
@@ -1041,7 +1038,7 @@ multiSelectionParser(const char *filenameOrString)
                 pline = removeSpaces(pline);
                 parEnd = findParamEnd(pline);
                 if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
-                while ((pline != parEnd) && (strlen(pline) > 0))
+                while ((pline != parEnd) && (std::strlen(pline) > 0))
                   {
                     pline = removeSpaces(pline);
                     val = (pline[0] == '*') ? -1 : atoi(pline);
@@ -1075,7 +1072,7 @@ multiSelectionParser(const char *filenameOrString)
                 pline = removeSpaces(pline);
                 parEnd = findParamEnd(pline);
                 if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
-                while ((pline != parEnd) && (strlen(pline) > 0))
+                while ((pline != parEnd) && (std::strlen(pline) > 0))
                   {
                     pline = removeSpaces(pline);
                     val = (pline[0] == '*') ? -1 : atoi(pline);
@@ -1148,7 +1145,7 @@ multiSelectionParser(const char *filenameOrString)
                   }  // changemulti specification
                 parEnd = findParamEnd(pline);
                 if ((!parEnd) && (!pline)) Debug(cdoDebugExt >= 100, "strpos=%s; parEnd=%s ... pline=%s", strpos, parEnd, pline);
-                while ((pline != parEnd) && (strlen(pline) > 0))
+                while ((pline != parEnd) && (std::strlen(pline) > 0))
                   {
                     pline = removeSpaces(pline);
                     val = (pline[0] == '*') ? -1 : atoi(pline);
@@ -1169,8 +1166,8 @@ multiSelectionParser(const char *filenameOrString)
               }
 
             }  // end while pline
-        }      // end if "("
-    }          // end while ( readlineForParsing() )
+        }  // end if "("
+    }  // end while ( readlineForParsing() )
 
   if (gfile.is_open()) gfile.close();
 
