@@ -29,11 +29,11 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int vlistID1;
-  int vlistID2;
+  int vlistID1{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   VarList varList1;
 
@@ -68,7 +68,7 @@ public:
         for (int levID = 0; levID < nlevels; levID++)
           {
             auto level = cdo_zaxis_inq_level(zaxisID, levID);
-            auto sellevel = (cdo_operator_argc() == 3) ? IS_EQUAL(level, slevel) : true;
+            auto sellevel = (cdo_operator_argc() == 3) ? is_equal(level, slevel) : true;
             auto selcode = (scode == -1 || scode == code);
             auto selltype = (sltype == -1 || sltype == ltype);
 
@@ -104,27 +104,27 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             if (vlistInqFlag(vlistID1, varID, levelID) == true)
               {
                 auto varID2 = vlistFindVar(vlistID2, varID);
                 auto levelID2 = vlistFindLevel(vlistID2, varID, levelID);
-                cdo_def_record(streamID2, varID2, levelID2);
+                cdo_def_field(streamID2, varID2, levelID2);
 
-                if (dataIsUnchanged) { cdo_copy_record(streamID2, streamID1); }
+                if (dataIsUnchanged) { cdo_copy_field(streamID2, streamID1); }
                 else
                   {
                     field.init(varList1.vars[varID]);
-                    cdo_read_record(streamID1, field);
-                    cdo_write_record(streamID2, field);
+                    cdo_read_field(streamID1, field);
+                    cdo_write_field(streamID2, field);
                   }
               }
           }
