@@ -10,28 +10,29 @@
 #include "process_int.h"
 #include "param_conversion.h"
 #include "pmlist.h"
-#include "grid_point_search.h"
+#include "grid_pointsearch.h"
 #include "mpim_grid.h"
 #include "region.h"
 
-int gengridcell(const int gridID1, const size_t gridsize2, const std::vector<long> &cellidx);
-void window_cell(const Field &field1, Field &field2, const std::vector<long> &cellidx);
+int gengridcell(int gridID1, size_t gridsize2, const std::vector<long> &cellidx);
+void window_cell(Field const &field1, Field &field2, const std::vector<long> &cellidx);
 double radiusDegToKm(double radiusInDeg);
 
 struct CirclePoint
 {
-  double radius = 1.0;
-  double lon = 0.0;
-  double lat = 0.0;
-  size_t maxpoints = SIZE_MAX;
+  double radius{ 1.0 };
+  double lon{ 0.0 };
+  double lat{ 0.0 };
+  size_t maxpoints{ SIZE_MAX };
 };
 
 struct RegionInfo
 {
   std::vector<long> cellidx;
-  long nvals = 0;
-  int gridtype = -1;
-  int gridID1 = -1, gridID2 = -1;
+  long nvals{ 0 };
+  int gridtype{ -1 };
+  int gridID1{ -1 };
+  int gridID2{ -1 };
 };
 
 static inline bool
@@ -71,8 +72,8 @@ point_is_inside(double xval, double yval, double xmin, double xmax, const double
 }
 
 static void
-sel_region_cell(std::vector<int8_t> &mask, size_t gridsize, const Varray<double> &xvals, const Varray<double> &yvals,
-                const double *xcoords, const double *ycoords, size_t segmentSize, std::vector<long> &cellidx)
+sel_region_cell(Vmask &mask, size_t gridsize, Varray<double> const &xvals, Varray<double> const &yvals, const double *xcoords,
+                const double *ycoords, size_t segmentSize, std::vector<long> &cellidx)
 {
   auto xmm = varray_min_max(segmentSize, xcoords);
   auto ymm = varray_min_max(segmentSize, ycoords);
@@ -115,7 +116,7 @@ generate_region_grid(int gridID1, long &gridsize2, std::vector<long> &cellidx, i
   cdo_grid_to_degree(gridID1, CDI_XAXIS, xvals, "grid center lon");
   cdo_grid_to_degree(gridID1, CDI_YAXIS, yvals, "grid center lat");
 
-  std::vector<int8_t> mask(gridsize, false);
+  Vmask mask(gridsize, false);
   for (int i = 0; i < numFiles; ++i)
     {
       Regions regions;
@@ -165,23 +166,20 @@ generate_circle_grid(int gridID1, long &gridsize2, std::vector<long> &cellidx, c
     cdo_grid_to_radian(gridID1, CDI_XAXIS, xvals, "grid center lon");
     cdo_grid_to_radian(gridID1, CDI_YAXIS, yvals, "grid center lat");
 
-    GridPointSearch gps;
-    grid_point_search_create(gps, xvals, yvals);
-
-    grid_point_search_set_arc_radius(gps, cpoint.radius);
+    GridPointsearch gps;
+    gps.set_radius(arc_to_chord_length(cpoint.radius));
+    grid_pointsearch_create_unstruct(gps, xvals, yvals);
 
     auto numNeighbors = cpoint.maxpoints;
     if (numNeighbors > gridsize1) numNeighbors = gridsize1;
 
-    knnWeightsType knnWeights(numNeighbors);
-    grid_search_point_smooth(gps, cpoint.lon, cpoint.lat, knnWeights);
+    KnnData knnData(numNeighbors);
+    grid_search_point_smooth(gps, PointLonLat{ cpoint.lon, cpoint.lat }, knnData);
 
-    auto nvals = knnWeights.m_numNeighbors;
+    auto nvals = knnData.m_numNeighbors;
     cellidx.resize(nvals);
 
-    for (size_t i = 0; i < nvals; ++i) cellidx[i] = knnWeights.m_indices[i];
-
-    grid_point_search_delete(gps);
+    for (size_t i = 0; i < nvals; ++i) cellidx[i] = knnData.m_indices[i];
 
     if (nvals == 0) cdo_abort("No grid points found!");
 
@@ -201,19 +199,19 @@ selcircle_get_parameter(CirclePoint &cpoint)
   auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      const auto &pargv = cdo_get_oper_argv();
+      auto const &pargv = cdo_get_oper_argv();
 
       KVList kvlist;
       kvlist.name = cdo_module_name();
       if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-          const auto &value = kv.values[0];
+          auto const &value = kv.values[0];
 
           // clang-format off
           if      (key == "maxpoints") cpoint.maxpoints = parameter_to_size_t(value);
@@ -245,8 +243,8 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   int numGrids;
 
@@ -280,7 +278,7 @@ public:
     auto numVars = varList1.numVars();
     varIDs = std::vector<bool>(numVars, false);
 
-    numGrids = vlistNumGrids(vlistID1);
+    numGrids = varList1.numGrids();
     regions = std::vector<RegionInfo>(numGrids);
 
     int numFiles = 0;
@@ -296,8 +294,7 @@ public:
         selcircle_get_parameter(cpoint);
         if (cpoint.radius < 0.0 || cpoint.radius > 180.0) cdo_abort("radius=%g out of bounds (0-180 deg)!", cpoint.radius);
 
-        auto gridsizemax = vlistGridsizeMax(vlistID1);
-        if (gridsizemax < cpoint.maxpoints) cpoint.maxpoints = gridsizemax;
+        if (varList1.gridsizeMax() < cpoint.maxpoints) cpoint.maxpoints = varList1.gridsizeMax();
         if (Options::cdoVerbose)
           cdo_print("lon = %g, lat = %g, radius = %gdeg(%gkm)", cpoint.lon, cpoint.lat, cpoint.radius,
                     radiusDegToKm(cpoint.radius));
@@ -335,7 +332,7 @@ public:
 
                 vlistChangeGridIndex(vlistID2, index, gridID2);
 
-                for (const auto &var : varList1.vars)
+                for (auto const &var : varList1.vars)
                   if (gridID1 == var.gridID) varIDs[var.ID] = true;
               }
           }
@@ -356,21 +353,20 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            const auto &var = varList1.vars[varID];
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            auto const &var = varList1.vars[varID];
             field1.init(var);
-            cdo_read_record(streamID1, field1);
+            cdo_read_field(streamID1, field1);
 
-            cdo_def_record(streamID2, varID, levelID);
+            cdo_def_field(streamID2, varID, levelID);
 
             if (varIDs[varID])
               {
@@ -385,9 +381,9 @@ public:
 
                 if (field1.numMissVals) field_num_mv(field2);
 
-                cdo_write_record(streamID2, field2);
+                cdo_write_field(streamID2, field2);
               }
-            else { cdo_write_record(streamID2, field1); }
+            else { cdo_write_field(streamID2, field1); }
           }
 
         tsID++;
