@@ -22,6 +22,7 @@
 
 #include "cdo_options.h"
 #include "process_int.h"
+#include "util_string.h"
 #include "param_conversion.h"
 #include "field_functions.h"
 
@@ -63,28 +64,24 @@ get_season_list(const std::vector<std::string> &seasonString)
  *      std::vector<std::string> => vector of length 1 or 2 containing string representing a range of dates or a single date to be
  * selected Output: std::vector<double>       => vector of length 1 or 2 containing the to double converted date or date range
  *
- * This function turns a date range string into a list of double values which
- * represent the start and the end of the range.
- * when only one argument is given and it contains no time string (marked with
- * T e.g 2006-01-01T23:00:30) we set the second value of the retrun vector to
- * the end of the day from the first argument.  if only one argument is given
- * AND it contains a time string we return a vector of length one which
- * contains only the converted value of the first argument. This function
- * accepts the string "-". This represents the possible maximum start or end
- * date depending on where the string is in the arguments. If the first
- * argument is "-" we set the start date to -999999999 and to 999999999 if the
- * string is in the second.
+ * This function turns a date range string into a list of double values which represent the start and the end of the range.
+ * When only one argument is given and it contains no time string (marked with T e.g 2006-01-01T23:00:30) we set the second value
+ * of the retrun vector to the end of the day from the first argument.
+ * If only one argument is given AND it contains a time string we return a vector of length one which contains only
+ * the converted value of the first argument.
+ * This function accepts the string "-".
+ * This represents the possible maximum start or end date depending on where the string is in the arguments.
+ * If the first argument is "-" we set the start date to -999999999 and to 999999999 if the string is in the second.
  */
 
 static std::vector<double>
 get_date_list(const std::vector<std::string> &argVec)
 {
-  std::vector<double> dateList;
-
   if (argVec.size() < 1) cdo_abort("Too few arguments!");
   if (argVec.size() > 2) cdo_abort("Too many arguments!");
 
-  bool containsTime = false;
+  std::vector<double> dateList;
+  short containsTime = -1;
   for (size_t i = 0, n = argVec.size(); i < n; ++i)
     {
       double fval = 0.0;
@@ -93,18 +90,13 @@ get_date_list(const std::vector<std::string> &argVec)
       // get the double value representing the date and check for time string
       else
         {
-          fval = date_str_to_double(argVec[i].c_str(), 0);
-          if (strchr(argVec[i].c_str(), 'T')) containsTime = true;
+          fval = datestr_to_double(argVec[i], i);
+          if (i == 0 && string_contains(argVec[i], '-')) containsTime = string_contains(argVec[i], 'T');
         }
       dateList.push_back(fval);
     }
-  // if two arguments and no time:
-  // set second argument to end of day
-  if (argVec.size() == 2 && !containsTime) dateList[1] += 0.999;
-
-  // if time and only one argument:
-  // set second date to first and set second to end of day
-  if (argVec.size() == 1 && !containsTime) dateList.push_back(dateList[0] + 0.999);
+  // if date and only one argument: set second date to first and set time to end of day
+  if (argVec.size() == 1) dateList.push_back(dateList[0] + ((containsTime != -1 && containsTime == 0) ? 0.235959 : 0.0));
 
   return dateList;
 }
@@ -120,7 +112,7 @@ get_argv_string(const std::vector<std::string> &argVec)
 static bool
 argv_has_negativ_values(const std::vector<std::string> &argVec)
 {
-  for (const auto &argument : argVec)
+  for (auto const &argument : argVec)
     {
       int first, last, inc;
       split_intstring(argument, first, last, inc);
@@ -135,7 +127,7 @@ cdo_argv_to_int_timestep(const std::vector<std::string> &argVec, int ntimesteps)
 {
   std::vector<int> v;
 
-  for (const auto &argument : argVec)
+  for (auto const &argument : argVec)
     {
       int first, last, inc;
       split_intstring(argument, first, last, inc);
@@ -171,15 +163,17 @@ public:
   using Process::Process;
   inline static CdoModule module = {
     .name = "Seltime",
+    // clang-format off
     .operators = { { "seltimestep", func_step, 0, "timesteps", SeltimeHelp },
-                   { "selyear", func_date, 0, "years", SeltimeHelp },
-                   { "selseason", func_date, 0, "seasons", SeltimeHelp },
-                   { "selmonth", func_date, 0, "months", SeltimeHelp },
-                   { "selday", func_date, 0, "days", SeltimeHelp },
-                   { "selhour", func_time, 0, "hours", SeltimeHelp },
-                   { "seldate", func_datetime, 0, "startdateandenddate(formatYYYY-MM-DDThh:mm:ss)", SeltimeHelp },
-                   { "seltime", func_time, 0, "times(formathh:mm:ss)", SeltimeHelp },
-                   { "selsmon", func_date, 0, "month[,nts1[,nts2]]", SeltimeHelp } },
+                   { "selyear",     func_date, 0, "years", SeltimeHelp },
+                   { "selseason",   func_date, 0, "seasons", SeltimeHelp },
+                   { "selmonth",    func_date, 0, "months", SeltimeHelp },
+                   { "selday",      func_date, 0, "days", SeltimeHelp },
+                   { "selhour",     func_time, 0, "hours", SeltimeHelp },
+                   { "seldate",     func_datetime, 0, "startdate and enddate (format: YYYY-MM-DDThh:mm:ss)", SeltimeHelp },
+                   { "seltime",     func_time, 0, "times (format: hh:mm:ss)", SeltimeHelp },
+                   { "selsmon",     func_date, 0, "month[,nts1[,nts2]]", SeltimeHelp } },
+    // clang-format on
     .aliases = { { "selseas", "selseason" }, { "selmon", "selmonth" } },
     .mode = EXPOSED,     // Module mode: 0:intern 1:extern
     .number = CDI_BOTH,  // Allowed number type
@@ -191,16 +185,15 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2 = CDO_STREAM_UNDEF;
 
-  int vlistID1;
-  int vlistID2;
+  int vlistID1{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
-  int taxisID2;
+  int taxisID2{ CDI_UNDEFID };
 
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
   int operfunc;
   int operatorID;
 
-  int varID, levelID;
   int numSel = 0;
   int numVars;
   int ncts = 0, nts;
@@ -300,7 +293,7 @@ public:
         for (int i = 0; i < numSel; ++i)
           {
             auto currentArgument = cdo_operator_argv(i).c_str();
-            if (strchr(currentArgument, ':'))
+            if (std::strchr(currentArgument, ':'))
               {
                 int hour = 0, minute = 0, second = 0;
                 std::sscanf(currentArgument, "%d:%d:%d", &hour, &minute, &second);
@@ -334,6 +327,8 @@ public:
     vlistID1 = cdo_stream_inq_vlist(streamID1);
     vlistID2 = vlistDuplicate(vlistID1);
 
+    // cdo::delete_chunks_dimT(vlistID2);
+
     varList1 = VarList(vlistID1);
 
     // add support for negative timestep values
@@ -366,7 +361,7 @@ public:
 
     if (operatorID == SELTIMESTEP)
       {
-        ranges::sort(intarr);
+        std::ranges::sort(intarr);
         auto ip = std::unique(intarr.begin(), intarr.end());
         intarr.resize(std::distance(intarr.begin(), ip));
         numSel = intarr.size();
@@ -411,12 +406,12 @@ public:
           {
             field2D_init(varsData[tsID], varList1);
 
-            for (varID = 0; varID < numVars; ++varID)
+            for (int varID = 0; varID < numVars; ++varID)
               {
-                const auto &var = varList1.vars[varID];
+                auto const &var = varList1.vars[varID];
                 if (lnts1 || var.isConstant)
                   {
-                    for (levelID = 0; levelID < var.nlevels; ++levelID) varsData[tsID][varID][levelID].resize(var.gridsize);
+                    for (int levelID = 0; levelID < var.nlevels; ++levelID) varsData[tsID][varID][levelID].resize(var.gridsize);
                   }
               }
           }
@@ -436,8 +431,8 @@ public:
     int tsID2 = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         auto vDateTime = taxisInqVdatetime(taxisID1);
         auto vdate = cdiDate_get(vDateTime.date);
@@ -544,16 +539,16 @@ public:
                     taxisDefVdatetime(taxisID2, vDateTimes[it]);
                     cdo_def_timestep(streamID2, tsID2++);
 
-                    for (varID = 0; varID < numVars; ++varID)
+                    for (int varID = 0; varID < numVars; ++varID)
                       {
-                        const auto &var = varList1.vars[varID];
+                        auto const &var = varList1.vars[varID];
                         if (var.isConstant && tsID2 > 1) continue;
-                        for (levelID = 0; levelID < var.nlevels; ++levelID)
+                        for (int levelID = 0; levelID < var.nlevels; ++levelID)
                           {
-                            cdo_def_record(streamID2, varID, levelID);
+                            cdo_def_field(streamID2, varID, levelID);
                             auto single = varsData[it][varID][levelID].vec_d.data();
                             auto numMissVals = varsData[it][varID][levelID].numMissVals;
-                            cdo_write_record(streamID2, single, numMissVals);
+                            cdo_write_field(streamID2, single, numMissVals);
                           }
                       }
                   }
@@ -574,33 +569,33 @@ public:
               {
                 isConstOut = false;
                 nts = nts1 - 1;
-                for (varID = 0; varID < numVars; ++varID)
+                for (int varID = 0; varID < numVars; ++varID)
                   {
-                    const auto &var = varList1.vars[varID];
+                    auto const &var = varList1.vars[varID];
                     if (var.isConstant)
                       {
-                        for (levelID = 0; levelID < var.nlevels; ++levelID)
+                        for (int levelID = 0; levelID < var.nlevels; ++levelID)
                           {
-                            cdo_def_record(streamID2, varID, levelID);
+                            cdo_def_field(streamID2, varID, levelID);
                             auto single = varsData[nts][varID][levelID].vec_d.data();
                             auto numMissVals = varsData[nts][varID][levelID].numMissVals;
-                            cdo_write_record(streamID2, single, numMissVals);
+                            cdo_write_field(streamID2, single, numMissVals);
                           }
                       }
                   }
               }
 
-            for (int recID = 0; recID < nrecs; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                cdo_inq_record(streamID1, &varID, &levelID);
-                cdo_def_record(streamID2, varID, levelID);
-                if (dataIsUnchanged) { cdo_copy_record(streamID2, streamID1); }
+                auto [varID, levelID] = cdo_inq_field(streamID1);
+                cdo_def_field(streamID2, varID, levelID);
+                if (dataIsUnchanged) { cdo_copy_field(streamID2, streamID1); }
                 else
                   {
-                    const auto &var = varList1.vars[varID];
+                    auto const &var = varList1.vars[varID];
                     field.init(var);
-                    cdo_read_record(streamID1, field);
-                    cdo_write_record(streamID2, field);
+                    cdo_read_field(streamID1, field);
+                    cdo_write_field(streamID2, field);
                   }
               }
           }
@@ -621,11 +616,11 @@ public:
                       for (int it = 0; it < nts; it++)
                         {
                           vDateTimes[it] = vDateTimes[it + 1];
-                          for (varID = 0; varID < numVars; ++varID)
+                          for (int varID = 0; varID < numVars; ++varID)
                             {
-                              const auto &var = varList1.vars[varID];
+                              auto const &var = varList1.vars[varID];
                               if (var.isConstant) continue;
-                              for (levelID = 0; levelID < var.nlevels; ++levelID)
+                              for (int levelID = 0; levelID < var.nlevels; ++levelID)
                                 {
                                   varsData[it][varID][levelID].vec_d = varsData[it + 1][varID][levelID].vec_d;
                                   varsData[it][varID][levelID].numMissVals = varsData[it + 1][varID][levelID].numMissVals;
@@ -638,14 +633,14 @@ public:
                     its1++;
                   }
 
-                for (int recID = 0; recID < nrecs; ++recID)
+                for (int fieldID = 0; fieldID < numFields; ++fieldID)
                   {
-                    cdo_inq_record(streamID1, &varID, &levelID);
-                    const auto &var = varList1.vars[varID];
+                    auto [varID, levelID] = cdo_inq_field(streamID1);
+                    auto const &var = varList1.vars[varID];
                     if (lnts1 || var.isConstant)
                       {
                         auto single = varsData[nts][varID][levelID].vec_d.data();
-                        cdo_read_record(streamID1, single, &varsData[nts][varID][levelID].numMissVals);
+                        cdo_read_field(streamID1, single, &varsData[nts][varID][levelID].numMissVals);
                       }
                   }
               }
