@@ -8,8 +8,8 @@
 /*
    This module contains the following operators:
 
-      Selvar     selparam        Select parameters by identifier (format: code.tabnum  or  pnum.cat.dis)
-      Selvar     delparam        Delete parameters by identifier (format: code.tabnum  or  pnum.cat.dis)
+      Selvar     selparam        Select parameters by identifier (format: code.tabnum or pnum.cat.dis)
+      Selvar     delparam        Delete parameters by identifier (format: code.tabnum or pnum.cat.dis)
       Selvar     selcode         Select parameters by code number
       Selvar     delcode         Delete parameters by code number
       Selvar     selname         Select parameters by name
@@ -38,20 +38,22 @@ public:
   using Process::Process;
   inline static CdoModule module = {
     .name = "Selvar",
-    .operators = { { "selparam", 0, 2, "parameters", SelvarHelp },
-                   { "selcode", 0, 4, "code numbers", SelvarHelp },
-                   { "selname", 0, 2, "variable names", SelvarHelp },
-                   { "selstdname", 0, 2, "standard names", SelvarHelp },
-                   { "sellevel", 0, 8, "levels", SelvarHelp },
-                   { "sellevidx", 0, 4, "index of levels", SelvarHelp },
-                   { "selgrid", 0, 4 | 2, "list of grid names or numbers", SelvarHelp },
-                   { "selzaxis", 0, 4 | 2, "list of zaxis types or numbers", SelvarHelp },
+    // clang-format off
+    .operators = { { "selparam",     0, 2, "parameters", SelvarHelp },
+                   { "selcode",      0, 4, "code numbers", SelvarHelp },
+                   { "selname",      0, 2, "variable names", SelvarHelp },
+                   { "selstdname",   0, 2, "standard names", SelvarHelp },
+                   { "sellevel",     0, 8, "levels", SelvarHelp },
+                   { "sellevidx",    0, 4, "index of levels", SelvarHelp },
+                   { "selgrid",      0, 4 | 2, "list of grid names or numbers", SelvarHelp },
+                   { "selzaxis",     0, 4 | 2, "list of zaxis types or numbers", SelvarHelp },
                    { "selzaxisname", 0, 2, "list of zaxis names", SelvarHelp },
-                   { "seltabnum", 0, 4, "table numbers", SelvarHelp },
-                   { "delparam", 1, 2 | 1, "parameter", SelvarHelp },
-                   { "delcode", 1, 1, "code numbers", SelvarHelp },
-                   { "delname", 1, 2 | 1, "variable names", SelvarHelp },
-                   { "selltype", 0, 4, "GRIB level types", SelvarHelp } },
+                   { "seltabnum",    0, 4, "table numbers", SelvarHelp },
+                   { "delparam",     1, 2 | 1, "parameter", SelvarHelp },
+                   { "delcode",      1, 1, "code numbers", SelvarHelp },
+                   { "delname",      1, 2 | 1, "variable names", SelvarHelp },
+                   { "selltype",     0, 4, "GRIB level types", SelvarHelp } },
+    // clang-format on
     .aliases = { { "selvar", "selname" }, { "delvar", "delname" }, { "selgridname", "selgrid" } },
     .mode = EXPOSED,     // Module mode: 0:intern 1:extern
     .number = CDI_BOTH,  // Allowed number type
@@ -62,11 +64,11 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int vlistID1;
-  int vlistID2;
+  int vlistID1{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   VarList varList1;
 
@@ -113,7 +115,7 @@ public:
 
     auto argsAreNumeric = (cdo_operator_argc() > 0 && std::isdigit(cdo_operator_argv(0)[0]));
 
-    const auto &argnames = cdo_get_oper_argv();
+    auto const &argnames = cdo_get_oper_argv();
     if (TAKES_STRINGS(operatorID) && !(TAKES_INTEGERS(operatorID) && argsAreNumeric))
       {
         nsel = cdo_operator_argc();
@@ -161,7 +163,7 @@ public:
 
         for (int varID = 0; varID < numVars; ++varID)
           {
-            const auto &var = varList1.vars[varID];
+            auto const &var = varList1.vars[varID];
             if (var.name == gridvarname)
               {
                 gridnum = 1 + vlistGridIndex(vlistID1, var.gridID);
@@ -179,7 +181,7 @@ public:
       {
         vars[varID] = doDelete;
 
-        const auto &var = varList1.vars[varID];
+        auto const &var = varList1.vars[varID];
         auto tabnum = tableInqNum(vlistInqVarTable(vlistID1, varID));
         auto grididx = vlistGridIndex(vlistID1, var.gridID);
         auto zaxisidx = vlistZaxisIndex(vlistID1, var.zaxisID);
@@ -238,7 +240,7 @@ public:
       {
         if (vars[varID])
           {
-            const auto &var = varList1.vars[varID];
+            auto const &var = varList1.vars[varID];
             if (zaxisInqType(var.zaxisID) == ZAXIS_HYBRID)
               {
                 auto psvarid = varList_get_psvarid(varList1, var.zaxisID);
@@ -301,31 +303,42 @@ public:
   {
     Field field;
 
+// #define USE_STREAM_PTR 1
+#ifdef USE_STREAM_PTR
+    auto stream1 = stream_get_pointer(streamID1->m_fileID);
+    auto vlist1 = stream_get_vlist_pointer(streamID1->m_fileID);
+#endif
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
+#ifdef USE_STREAM_PTR
+            int varID, levelID;
+            pstreamInqField(stream1, &varID, &levelID);
+            if (pvlistInqFlag(vlist1, varID, levelID) == true)
+#else
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             if (vlistInqFlag(vlistID1, varID, levelID) == true)
+#endif
               {
                 auto varID2 = vlistFindVar(vlistID2, varID);
                 auto levelID2 = vlistFindLevel(vlistID2, varID, levelID);
-                cdo_def_record(streamID2, varID2, levelID2);
+                cdo_def_field(streamID2, varID2, levelID2);
 
-                if (dataIsUnchanged) { cdo_copy_record(streamID2, streamID1); }
+                if (dataIsUnchanged) { cdo_copy_field(streamID2, streamID1); }
                 else
                   {
-                    const auto &var = varList1.vars[varID];
+                    auto const &var = varList1.vars[varID];
                     field.init(var);
-                    cdo_read_record(streamID1, field);
-                    cdo_write_record(streamID2, field);
+                    cdo_read_field(streamID1, field);
+                    cdo_write_field(streamID2, field);
                   }
               }
           }
