@@ -36,11 +36,11 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
   CdoStreamID streamID3;
-  int vlistID1;
-  int vlistID2;
+  int vlistID1{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
   int taxisID3;
 
   bool allSteps;
@@ -81,8 +81,8 @@ public:
   {
     Field field;
 
-    auto maxRecords = varList1.numRecords();
-    std::vector<RecordInfo> recordList(maxRecords);
+    auto maxFields = varList1.maxFields();
+    std::vector<FieldInfo> fieldInfoList(maxFields);
 
     FieldVector2D varsData1, varsData2;
     field2D_init(varsData1, varList1, FIELD_VEC);
@@ -91,7 +91,7 @@ public:
     auto numVars = varList1.numVars();
     for (int varID = 0; varID < numVars; ++varID)
       {
-        const auto &var = varList1.vars[varID];
+        auto const &var = varList1.vars[varID];
         auto missval = varList2.vars[varID].missval;
         for (int levelID = 0; levelID < var.nlevels; ++levelID)
           {
@@ -104,38 +104,38 @@ public:
     int tsID3 = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         auto year1 = taxisInqVdatetime(taxisID1).date.year;
 
-        auto useExactDate = (nrecs == 1 && varList1.vars[0].gridsize == 1);
+        auto useExactDate = (numFields == 1 && varList1.vars[0].gridsize == 1);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             auto &field1 = varsData1[varID][levelID];
-            cdo_read_record(streamID1, field1);
+            cdo_read_field(streamID1, field1);
 
-            if (tsID == 0) recordList[recID].set(varID, levelID);
+            if (tsID == 0) fieldInfoList[fieldID].set(varID, levelID);
           }
 
-        int nrecs2;
+        int numFields2;
         int numSets = 0;
-        while ((nrecs2 = cdo_stream_inq_timestep(streamID2, tsID2)))
+        while ((numFields2 = cdo_stream_inq_timestep(streamID2, tsID2)))
           {
             auto year = taxisInqVdatetime(taxisID2).date.year;
 
             if (allSteps == false && year1 != year) break;
 
-            for (int recID = 0; recID < nrecs2; ++recID)
+            for (int fieldID = 0; fieldID < numFields2; ++fieldID)
               {
-                auto [varID, levelID] = cdo_inq_record(streamID2);
-                const auto &var = varList2.vars[varID];
+                auto [varID, levelID] = cdo_inq_field(streamID2);
+                auto const &var = varList2.vars[varID];
                 field.init(var);
-                cdo_read_record(streamID2, field);
+                cdo_read_field(streamID2, field);
 
-                const auto &field1 = varsData1[varID][levelID];
+                auto const &field1 = varsData1[varID][levelID];
                 auto &field2 = varsData2[varID][levelID];
                 for (size_t i = 0; i < var.gridsize; ++i)
                   if (numSets == (int) std::lround(field1.vec_d[i]))
@@ -154,15 +154,15 @@ public:
             if (!useExactDate) cdo_taxis_copy_timestep(taxisID3, taxisID1);
             cdo_def_timestep(streamID3, tsID3);
 
-            for (int recID = 0; recID < maxRecords; ++recID)
+            for (int fieldID = 0; fieldID < maxFields; ++fieldID)
               {
-                auto [varID, levelID] = recordList[recID].get();
+                auto [varID, levelID] = fieldInfoList[fieldID].get();
                 if (tsID && varList1.vars[varID].isConstant) continue;
 
                 auto &field2 = varsData2[varID][levelID];
                 field_num_mv(field2);
-                cdo_def_record(streamID3, varID, levelID);
-                cdo_write_record(streamID3, field2);
+                cdo_def_field(streamID3, varID, levelID);
+                cdo_write_field(streamID3, field2);
               }
 
             tsID3++;
@@ -174,7 +174,7 @@ public:
             break;
           }
 
-        if (nrecs2 == 0) break;
+        if (numFields2 == 0) break;
 
         tsID++;
       }
