@@ -21,7 +21,7 @@ get_datatype(char *buffer)
 {
   int dtype = -1;
 
-  auto slen = strlen(buffer);
+  auto slen = std::strlen(buffer);
   if (slen >= 3 && buffer[slen - 2] == ':')
     {
       if (slen >= 4 && buffer[slen - 3] == '\\')
@@ -45,7 +45,7 @@ get_datatype(char *buffer)
 }
 
 static std::pair<std::string, std::string>
-split_var_attr(const std::string &key, int delimiter)
+split_var_attr(std::string const &key, int delimiter)
 {
   std::string varName, attName;
 
@@ -62,15 +62,15 @@ split_var_attr(const std::string &key, int delimiter)
 }
 
 static std::vector<int>
-find_variables(const std::string &varName, int vlistID, std::vector<std::string> &wnames, int &cdiID)
+find_variables(std::string const &varName, int vlistID, std::vector<std::string> &wnames, int &cdiID)
 {
   std::vector<int> varIDs;
 
   constexpr int Undefined = -99;
   cdiID = Undefined;
   auto numVars = vlistNvars(vlistID);
-  // auto ngrids = vlistNumGrids(vlistID);
-  auto nzaxis = vlistNumZaxis(vlistID);
+  // auto numGrids = vlistNumGrids(vlistID);
+  auto numZaxes = vlistNumZaxis(vlistID);
   if (varName.size())
     {
       for (int idx = 0; idx < numVars; idx++)
@@ -86,7 +86,7 @@ find_variables(const std::string &varName, int vlistID, std::vector<std::string>
       if (cdiID == Undefined)
         {
           /*
-          for ( int idx = 0; idx < ngrids; idx++ )
+          for ( int idx = 0; idx < numGrids; idx++ )
             {
               int gridID = vlistGrid(vlistID, idx);
               auto xname = cdo::inq_key_string(gridID, CDI_XAXIS, CDI_KEY_NAME);
@@ -103,7 +103,7 @@ find_variables(const std::string &varName, int vlistID, std::vector<std::string>
                 }
               }
           */
-          for (int idx = 0; idx < nzaxis; idx++)
+          for (int idx = 0; idx < numZaxes; idx++)
             {
               auto zaxisID = vlistZaxis(vlistID, idx);
               auto key = cdo::inq_key_string(zaxisID, CDI_GLOBAL, CDI_KEY_NAME);
@@ -120,16 +120,10 @@ find_variables(const std::string &varName, int vlistID, std::vector<std::string>
           auto printWarning = true;
           for (size_t i = 0, n = wnames.size(); i < n; ++i)
             {
-              if (wnames[i].empty())
-                {
-                  wnames[i] = varName;
-                  break;
-                }
-              if (wnames[i] == varName)
-                {
-                  printWarning = false;
-                  break;
-                }
+              // clang-format off
+              if (wnames[i].empty())    { wnames[i] = varName; break; }
+              if (wnames[i] == varName) { printWarning = false; break; }
+              // clang-format on
             }
           if (printWarning) cdo_warning("Variable >%s< not found!", varName);
         }
@@ -144,7 +138,7 @@ find_variables(const std::string &varName, int vlistID, std::vector<std::string>
 }
 
 static std::vector<std::string>
-find_attribute(int cdiID, int varID, const std::string &attrName, int &dtype)
+find_attribute(int cdiID, int varID, std::string const &attrName, int &dtype)
 {
   std::vector<std::string> attrValues;
 
@@ -186,7 +180,7 @@ find_attribute(int cdiID, int varID, const std::string &attrName, int &dtype)
   if (varID != CDI_GLOBAL && attrValues.empty())
     {
       VarList varList(cdiID);
-      const auto &var = varList.vars[varID];
+      auto const &var = varList.vars[varID];
       auto stdname = cdo::inq_key_string(cdiID, varID, CDI_KEY_STDNAME);
       auto table = vlistInqVarTable(cdiID, varID);
 
@@ -204,14 +198,14 @@ find_attribute(int cdiID, int varID, const std::string &attrName, int &dtype)
       else if (attrName == "missing_value") attrValues.push_back(std::to_string(var.missval));
       else if (attrName == "add_offset")    attrValues.push_back(std::to_string(addoffset));
       else if (attrName == "scale_factor")  attrValues.push_back(std::to_string(scalefactor));
-      // clang-format off
+      // clang-format on
     }
 
   return attrValues;
 }
 
 static std::vector<std::string>
-get_attribute(int vlistID, const std::string &varAttr, int &dtype)
+get_attribute(int vlistID, std::string const &varAttr, int &dtype)
 {
   auto [varName, attName] = split_var_attr(varAttr, Delimiter);
   if (attName.empty()) cdo_abort("Attribute name missing in >%s<!", varAttr);
@@ -246,39 +240,59 @@ get_attribute(int vlistID, const std::string &varAttr, int &dtype)
 }
 
 static void
-delete_attribute(int cdiID, int varID, const std::string &attName)
+delete_attribute(int cdiID, int varID, std::string const &attName, std::string const &varAtt)
 {
   auto status = cdiDelAtt(cdiID, varID, attName.c_str());
   if (status != CDI_NOERR)  // check CDI keys
     {
-      // clang-format off
-      if      (attName == "long_name") cdiDeleteKey(cdiID, varID, CDI_KEY_LONGNAME);
-      else if (attName == "units")     cdiDeleteKey(cdiID, varID, CDI_KEY_UNITS);
-      // clang-format on
+      if (attName == "long_name")
+        cdiDeleteKey(cdiID, varID, CDI_KEY_LONGNAME);
+      else if (attName == "units")
+        cdiDeleteKey(cdiID, varID, CDI_KEY_UNITS);
+      else
+        {
+          bool foundAtt = false;
+          int numAtts = 0;
+          cdiInqNatts(cdiID, varID, &numAtts);
+          for (int attnum = 0; attnum < numAtts; ++attnum)
+            {
+              int type;
+              int len;
+              char name[CDI_MAX_NAME];
+              cdiInqAtt(cdiID, varID, attnum, name, &type, &len);
+
+              if (wildcardmatch(attName, name) == 0)
+                {
+                  foundAtt = true;
+                  cdiDelAtt(cdiID, varID, name);
+                }
+            }
+          if (foundAtt == false) cdo_warning("Attribute %s not found!", varAtt);
+        }
     }
 }
 
 static void
-set_attribute(int cdiID, int varID, const std::string &attName, int dtype, const std::vector<std::string> &values)
+set_attribute(int cdiID, int varID, std::string const &attName, int dtype, std::vector<std::string> const &values)
 {
-  int nvalues = values.size();
+  int numValues = values.size();
 
   if (dtype == CDI_DATATYPE_INT8 || dtype == CDI_DATATYPE_INT16 || dtype == CDI_DATATYPE_INT32)
     {
-      std::vector<int> ivals(nvalues);
-      for (int i = 0; i < nvalues; ++i) ivals[i] = literal_to_int(values[i]);
-      cdiDefAttInt(cdiID, varID, attName.c_str(), dtype, nvalues, ivals.data());
+      std::vector<int> ivals(numValues);
+      for (int i = 0; i < numValues; ++i) ivals[i] = literal_to_int(values[i]);
+      cdiDefAttInt(cdiID, varID, attName.c_str(), dtype, numValues, ivals.data());
     }
   else if (dtype == CDI_DATATYPE_FLT32 || dtype == CDI_DATATYPE_FLT64)
     {
-      Varray<double> dvals(nvalues);
-      for (int i = 0; i < nvalues; ++i) dvals[i] = literal_to_double(values[i]);
-      cdiDefAttFlt(cdiID, varID, attName.c_str(), dtype, nvalues, dvals.data());
+      Varray<double> dvals(numValues);
+      for (int i = 0; i < numValues; ++i) dvals[i] = literal_to_double(values[i]);
+      cdiDefAttFlt(cdiID, varID, attName.c_str(), dtype, numValues, dvals.data());
     }
   else
     {
-      if (nvalues > 1) cdo_abort("Multidimensional string attributes not supported! %s=\"%s\"", attName, values[1]);
-      const auto &value = values[0];
+      if (numValues > 1) cdo_abort("Multidimensional string attributes not supported! %s=\"%s\"", attName, values[1]);
+      auto const &value = values[0];
       auto len = (int) value.size();
       int outlen = 0;
       std::vector<char> outvalue(len);
@@ -296,13 +310,13 @@ set_attribute(int cdiID, int varID, const std::string &attName, int dtype, const
 }
 
 static void
-set_attributes(const KVList &kvlist, int vlistID)
+set_attributes(KVList const &kvlist, int vlistID)
 {
   int kvn = kvlist.size();
   std::vector<std::string> wnames(kvn);
 
   char buffer[CDI_MAX_NAME];
-  for (const auto &kv : kvlist)
+  for (auto const &kv : kvlist)
     {
       std::strcpy(buffer, kv.key.c_str());
       auto dtype = get_datatype(buffer);
@@ -318,7 +332,7 @@ set_attributes(const KVList &kvlist, int vlistID)
         {
           if (kv.nvalues == 0 || (kv.nvalues == 1 && kv.values[0].empty()))
             {
-              for (int idx = 0; idx < numVars; ++idx) delete_attribute(cdiID, varIDs[idx], attName);
+              for (auto varID : varIDs) delete_attribute(cdiID, varID, attName, buffer);
             }
           else
             {
@@ -326,7 +340,7 @@ set_attributes(const KVList &kvlist, int vlistID)
               auto useAttrValues = false;
               if (kv.nvalues == 1)
                 {
-                  const auto &value = kv.values[0];
+                  auto const &value = kv.values[0];
                   if (value.size() > 2 && value[0] == '{' && value[value.size() - 1] == '}')
                     {
                       attrValues = get_attribute(vlistID, value.substr(1, value.size() - 2), dtype);
@@ -334,11 +348,11 @@ set_attributes(const KVList &kvlist, int vlistID)
                     }
                 }
 
-              const auto &values = useAttrValues ? attrValues : kv.values;
+              auto const &values = useAttrValues ? attrValues : kv.values;
 
               if (dtype == -1) dtype = literals_find_datatype(values.size(), values);
 
-              for (int idx = 0; idx < numVars; ++idx) set_attribute(cdiID, varIDs[idx], attName, dtype, values);
+              for (auto varID : varIDs) set_attribute(cdiID, varID, attName, dtype, values);
             }
         }
     }
@@ -350,7 +364,10 @@ public:
   using Process::Process;
   inline static CdoModule module = {
     .name = "Setattribute",
-    .operators = { { "setattribute", 0, 0, "attributes", SetattributeHelp } },
+    // clang-format off
+    .operators = { { "setattribute", 0, 0, "attributes", SetattributeHelp },
+                   { "delattribute", 0, 0, "attributes" } },
+    // clang-format on
     .aliases = {},
     .mode = EXPOSED,     // Module mode: 0:intern 1:extern
     .number = CDI_REAL,  // Allowed number type
@@ -361,8 +378,8 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   bool dataIsUnchanged;
 
@@ -374,6 +391,8 @@ public:
   {
     dataIsUnchanged = data_is_unchanged();
 
+    auto DELATTRIBUTE = module.get_id("delattribute");
+
     auto operatorID = cdo_operator_id();
 
     operator_input_arg(cdo_operator_enter(operatorID));
@@ -381,10 +400,17 @@ public:
     auto numAtts = cdo_operator_argc();
     if (numAtts == 0) cdo_abort("Parameter missing!");
 
+    auto attrList = cdo_get_oper_argv();
+    if (operatorID == DELATTRIBUTE)
+      {
+        for (auto &attr : attrList)
+          if (attr.back() != '=') attr.push_back('=');
+      }
+
     PMList pmlist;
     KVList kvlist;
     kvlist.name = cdo_module_name();
-    if (kvlist.parse_arguments(cdo_get_oper_argv()) != 0) cdo_abort("Parse error!");
+    if (kvlist.parse_arguments(attrList) != 0) cdo_abort("Parse error!");
     if (Options::cdoVerbose) kvlist.print();
 
     auto pkvlist = &kvlist;
@@ -429,24 +455,23 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            cdo_def_record(streamID2, varID, levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            cdo_def_field(streamID2, varID, levelID);
 
-            if (dataIsUnchanged) { cdo_copy_record(streamID2, streamID1); }
+            if (dataIsUnchanged) { cdo_copy_field(streamID2, streamID1); }
             else
               {
                 field.init(varList1.vars[varID]);
-                cdo_read_record(streamID1, field);
-                cdo_write_record(streamID2, field);
+                cdo_read_field(streamID1, field);
+                cdo_write_field(streamID2, field);
               }
           }
 
