@@ -22,14 +22,14 @@
 static void
 setcbox(double constant, double *array, int gridID, const SelboxInfo &selboxInfo)
 {
-  const auto &lat1 = selboxInfo.lat1;
-  const auto &lat2 = selboxInfo.lat2;
-  const auto &lon11 = selboxInfo.lon11;
-  const auto &lon12 = selboxInfo.lon12;
-  const auto &lon21 = selboxInfo.lon21;
-  const auto &lon22 = selboxInfo.lon22;
-  const long nlon = gridInqXsize(gridID);
-  const long nlat = gridInqYsize(gridID);
+  auto const &lat1 = selboxInfo.lat1;
+  auto const &lat2 = selboxInfo.lat2;
+  auto const &lon11 = selboxInfo.lon11;
+  auto const &lon12 = selboxInfo.lon12;
+  auto const &lon21 = selboxInfo.lon21;
+  auto const &lon22 = selboxInfo.lon22;
+  long nlon = gridInqXsize(gridID);
+  long nlat = gridInqYsize(gridID);
 
   for (long ilat = 0; ilat < nlat; ilat++)
     for (long ilon = 0; ilon < nlon; ilon++)
@@ -44,17 +44,17 @@ get_gridID(int vlistID1, bool operIndexBox)
 {
   std::vector<int> gridsFound;
 
-  const auto ngrids = vlistNgrids(vlistID1);
-  for (int index = 0; index < ngrids; ++index)
+  auto numGrids = vlistNumGrids(vlistID1);
+  for (int index = 0; index < numGrids; ++index)
     {
-      const auto gridID1 = vlistGrid(vlistID1, index);
+      auto gridID1 = vlistGrid(vlistID1, index);
       if (gridInqSize(gridID1) == 1) continue;
 
-      const auto gridtype = gridInqType(gridID1);
-      const auto projtype = gridInqProjType(gridID1);
+      auto gridtype = gridInqType(gridID1);
+      auto projtype = gridInqProjType(gridID1);
 
-      const auto isReg2dGeoGrid = (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_CURVILINEAR);
-      const auto projHasGeoCoords = (gridtype == GRID_PROJECTION && projtype == CDI_PROJ_RLL);
+      auto isReg2dGeoGrid = (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN || gridtype == GRID_CURVILINEAR);
+      auto projHasGeoCoords = (gridtype == GRID_PROJECTION && projtype == CDI_PROJ_RLL);
 
       if (isReg2dGeoGrid || projHasGeoCoords || (operIndexBox && (gridtype == GRID_GENERIC || gridtype == GRID_PROJECTION)))
         {
@@ -69,38 +69,54 @@ get_gridID(int vlistID1, bool operIndexBox)
   if (gridsFound.size() == 0) cdo_abort("No processable grid found!");
   if (gridsFound.size() > 1) cdo_abort("Too many different grids!");
 
-  const auto gridID = gridsFound[0];
+  auto gridID = gridsFound[0];
   return gridID;
 }
 
 static std::vector<bool>
-get_processVars(int vlistID1, int gridID)
+get_processVars(const VarList &varList, int gridID)
 {
-  const auto nvars = vlistNvars(vlistID1);
+  auto numVars = varList.numVars();
 
-  std::vector<bool> processVars(nvars, false);
+  std::vector<bool> processVars(numVars, false);
 
   int varID;
-  for (varID = 0; varID < nvars; ++varID)
-    if (gridID == vlistInqVarGrid(vlistID1, varID)) processVars[varID] = true;
+  for (varID = 0; varID < numVars; ++varID)
+    if (gridID == varList.vars[varID].gridID) processVars[varID] = true;
 
-  for (varID = 0; varID < nvars; ++varID)
+  for (varID = 0; varID < numVars; ++varID)
     if (processVars[varID]) break;
 
-  if (varID >= nvars) cdo_abort("No processable variable found!");
+  if (varID >= numVars) cdo_abort("No processable variable found!");
 
   return processVars;
 }
 
-class ModuleSetbox
+class Setbox : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Setbox",
+    .operators
+    = { { "setclonlatbox", 0, 0, "constant, western and eastern longitude and southern and northern latitude", SetboxHelp },
+        { "setcindexbox", 0, 0, "constant, index of first and last longitude and index of first and last latitude", SetboxHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Setbox> registration = RegisterEntry<Setbox>(module);
+
+  int SETCLONLATBOX, SETCINDEXBOX;
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  int vlistID1;
+  int vlistID1{ CDI_UNDEFID };
+  VarList varList1;
 
   int gridID;
 
@@ -109,24 +125,19 @@ class ModuleSetbox
 
   Varray<double> array;
   std::vector<bool> processVars;
-  int SETCLONLATBOX, SETCINDEXBOX;
   SelboxInfo selboxInfo;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    // clang-format off
-   SETCLONLATBOX = cdo_operator_add("setclonlatbox", 0, 0, "constant, western and eastern longitude and southern and northern latitude");
-   SETCINDEXBOX = cdo_operator_add("setcindexbox", 0, 0, "constant, index of first and last longitude and index of first and last latitude");
-    // clang-format on
+    SETCLONLATBOX = module.get_id("setclonlatbox");
+    SETCINDEXBOX = module.get_id("setcindexbox");
 
     (void) SETCLONLATBOX;
 
-    const auto operatorID = cdo_operator_id();
-    const auto operIndexBox = (operatorID == SETCINDEXBOX);
+    auto operatorID = cdo_operator_id();
+    auto operIndexBox = (operatorID == SETCINDEXBOX);
 
     operator_input_arg(cdo_operator_enter(operatorID));
 
@@ -135,15 +146,17 @@ public:
     streamID1 = cdo_open_read(0);
     vlistID1 = cdo_stream_inq_vlist(streamID1);
 
+    varList1 = VarList(vlistID1);
+
     gridID = get_gridID(vlistID1, operIndexBox);
 
-    processVars = get_processVars(vlistID1, gridID);
+    processVars = get_processVars(varList1, gridID);
 
     operator_input_arg(cdo_operator_enter(operatorID));
 
     selboxInfo = operIndexBox ? gen_index_selbox(1, gridID) : gen_lonlat_selbox(1, gridID);
 
-    const auto vlistID2 = vlistDuplicate(vlistID1);
+    auto vlistID2 = vlistDuplicate(vlistID1);
 
     taxisID1 = vlistInqTaxis(vlistID1);
     taxisID2 = taxisDuplicate(taxisID1);
@@ -156,57 +169,45 @@ public:
     gridsize = gridInqSize(gridID);
     array = Varray<double>(gridsize);
   }
+
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
       {
-        const auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
 
             if (processVars[varID])
               {
-                size_t nmiss;
-                cdo_read_record(streamID1, array.data(), &nmiss);
+                size_t numMissVals;
+                cdo_read_field(streamID1, array.data(), &numMissVals);
 
                 setcbox(constant, array.data(), gridID, selboxInfo);
 
-                const auto missval = vlistInqVarMissval(vlistID1, varID);
-                nmiss = varray_num_mv(gridsize, array, missval);
-                cdo_def_record(streamID2, varID, levelID);
-                cdo_write_record(streamID2, array.data(), nmiss);
+                auto missval = varList1.vars[varID].missval;
+                numMissVals = varray_num_mv(gridsize, array, missval);
+                cdo_def_field(streamID2, varID, levelID);
+                cdo_write_field(streamID2, array.data(), numMissVals);
               }
           }
 
         tsID++;
       }
   }
+
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Setbox(void *process)
-{
-  ModuleSetbox setbox;
-  setbox.init(process);
-  setbox.run();
-  setbox.close();
-
-  return nullptr;
-}
