@@ -43,7 +43,7 @@ change_grid(int vlistID1, int vlistID2, int gridID2)
 }
 
 static void
-grid_set_type(int vlistID1, int vlistID2, int gridtype, const std::string &gridname, bool lregular, bool lregularnn,
+grid_set_type(int vlistID1, int vlistID2, int gridtype, std::string const &gridname, bool lregular, bool lregularnn,
               bool ldereference, bool withBounds, std::vector<int> &grid2_vgpm)
 {
   auto needCorners = withBounds ? NeedCorners::Yes : NeedCorners::No;
@@ -158,7 +158,7 @@ grid_set_cellarea(int vlistID1, int vlistID2, Varray<double> &gridcellArea)
 }
 
 static void
-grid_set_mask(int vlistID1, int vlistID2, const Varray<double> &gridmask)
+grid_set_mask(int vlistID1, int vlistID2, Varray<double> const &gridmask)
 {
   auto masksize = gridmask.size();
   auto numGrids = vlistNumGrids(vlistID1);
@@ -194,7 +194,7 @@ grid_unset_mask(int vlistID1, int vlistID2)
 }
 
 static void
-grid_set_proj_params(int vlistID1, int vlistID2, const std::string &projparams)
+grid_set_proj_params(int vlistID1, int vlistID2, std::string const &projparams)
 {
   auto numGrids = vlistNumGrids(vlistID1);
   for (int index = 0; index < numGrids; ++index)
@@ -210,7 +210,7 @@ grid_set_proj_params(int vlistID1, int vlistID2, const std::string &projparams)
 }
 
 static void
-grid_read_cellarea(const std::string &areafile, Varray<double> &gridcellArea)
+grid_read_cellarea(std::string const &areafile, Varray<double> &gridcellArea)
 {
   auto searchName = false;
   std::string filename = areafile;
@@ -294,8 +294,8 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   bool lregular = false;
   bool lregularnn = false;
@@ -500,8 +500,8 @@ public:
             auto [varID, levelID] = cdo_inq_field(streamID1);
             cdo_def_field(streamID2, varID, levelID);
 
-            const auto &var1 = varList1.vars[varID];
-            const auto &var2 = varList2.vars[varID];
+            auto const &var1 = varList1.vars[varID];
+            auto const &var2 = varList2.vars[varID];
             field.init((lregular || lregularnn) ? var2 : var1);
             cdo_read_field(streamID1, field);
             auto numMissVals = field.numMissVals;
