@@ -364,12 +364,12 @@ get_parameter(void)
       if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-          const auto &value = kv.values[0];
+          auto const &value = kv.values[0];
 
           // clang-format off
           if      (key == "east")   haloParam.east = parameter_to_long(value);
@@ -429,7 +429,7 @@ gen_index_grid(int gridID1, HaloParam &haloParam)
 }
 
 static bool
-regular_halo(const Varray<double> &array1, int gridID1, Varray<double> &array2, const HaloParam &haloParam, double missval)
+regular_halo(Varray<double> const &array1, int gridID1, Varray<double> &array2, const HaloParam &haloParam, double missval)
 {
   auto recalcNumMiss = false;
 
@@ -484,7 +484,7 @@ regular_halo(const Varray<double> &array1, int gridID1, Varray<double> &array2,
 }
 
 static void
-tripolar_halo(const Varray<double> &array1, int gridID1, Varray<double> &array2)
+tripolar_halo(Varray<double> const &array1, int gridID1, Varray<double> &array2)
 {
   auto nlon = gridInqXsize(gridID1);
   auto nlat = gridInqYsize(gridID1);
@@ -505,8 +505,8 @@ get_haloInfoList(int vlistID1, int vlistID2, HaloParam &haloParam, bool operSetH
 {
   std::vector<HaloInfo> haloInfoList;
 
-  auto ngrids = vlistNumGrids(vlistID1);
-  for (int index = 0; index < ngrids; ++index)
+  auto numGrids = vlistNumGrids(vlistID1);
+  for (int index = 0; index < numGrids; ++index)
     {
       auto gridID1 = vlistGrid(vlistID1, index);
       if (gridInqSize(gridID1) == 1) continue;
@@ -528,7 +528,7 @@ get_haloInfoList(int vlistID1, int vlistID2, HaloParam &haloParam, bool operSetH
 
   if (Options::cdoVerbose)
     {
-      /*  for (const auto &sb : haloInfoList)
+      /*  for (auto const &sb : haloInfoList)
          if (sb.gridtype != GRID_UNSTRUCTURED && !is_healpix_grid(sb.gridID1))
            {
              cdo_print("box1 - idx1,idx2,idy1,idy2: %ld,%ld,%ld,%ld", sb.lon21 + 1, sb.lon22 + 1, sb.lat1 + 1, sb.lat2 + 1);
@@ -548,7 +548,7 @@ get_processVars(const VarList &varList, const std::vector<HaloInfo> &haloInfoLis
   std::vector<bool> processVars(numVars, false);
 
   int varID;
-  for (const auto &haloInfo : haloInfoList)
+  for (auto const &haloInfo : haloInfoList)
     {
       for (varID = 0; varID < numVars; ++varID)
         if (haloInfo.gridID1 == varList.vars[varID].gridID) processVars[varID] = true;
@@ -565,7 +565,7 @@ get_processVars(const VarList &varList, const std::vector<HaloInfo> &haloInfoLis
 static const HaloInfo &
 select_haloInfo(int gridID, const std::vector<HaloInfo> &haloInfoList)
 {
-  for (const auto &haloInfo : haloInfoList)
+  for (auto const &haloInfo : haloInfoList)
     if (gridID == haloInfo.gridID1) return haloInfo;
 
   cdo_abort("Internal problem, grid not found!");
@@ -596,8 +596,8 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   VarList varList1;
   VarList varList2;
@@ -643,20 +643,20 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
-            const auto &var1 = varList1.vars[varID];
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            auto const &var1 = varList1.vars[varID];
             field1.init(var1);
-            cdo_read_record(streamID1, field1);
+            cdo_read_field(streamID1, field1);
 
-            cdo_def_record(streamID2, varID, levelID);
+            cdo_def_field(streamID2, varID, levelID);
 
             if (processVars[varID])
               {
@@ -665,7 +665,7 @@ public:
 
                 field2.init(varList2.vars[varID]);
 
-                const auto &haloInfo = select_haloInfo(var1.gridID, haloInfoList);
+                auto const &haloInfo = select_haloInfo(var1.gridID, haloInfoList);
 
                 if (operatorID == SETHALO)
                   recalcNumMiss = regular_halo(field1.vec_d, haloInfo.gridID1, field2.vec_d, haloParam, missval);
@@ -674,9 +674,9 @@ public:
 
                 if (field1.numMissVals || recalcNumMiss) field_num_mv(field2);
 
-                cdo_write_record(streamID2, field2);
+                cdo_write_field(streamID2, field2);
               }
-            else { cdo_write_record(streamID2, field1); }
+            else { cdo_write_field(streamID2, field1); }
           }
 
         tsID++;
