@@ -15,6 +15,7 @@
 
 #include "cdo_options.h"
 #include "process_int.h"
+#include "util_string.h"
 #include "table.h"
 #include "param_conversion.h"
 #include "cdo_cmor.h"
@@ -32,12 +33,14 @@ enum pt_mode_t
   STANDARD_NAME
 };
 
+void cdo_define_var_units(CmorVar &cmorVar, int vlistID2, int varID, std::string const &units);
+
 void
 mapvar(int vlistID, int varID, const KeyValues &kv, CmorVar &cmorVar, bool &hasValidMin, bool &hasValidMax, int ptab,
        bool isnPtmodeName)
 {
   const auto key = string_to_lower(kv.key);
-  const auto &value = kv.values[0];
+  auto const &value = kv.values[0];
   auto lv1 = (kv.nvalues == 1);
 
   // printf("key=%s  value=>%s<\n", key.c_str(), value.c_str());
@@ -79,7 +82,7 @@ mapvar(int vlistID, int varID, const KeyValues &kv, CmorVar &cmorVar, bool &hasV
   else if (key == "units")
     {
       if (not lv1) cdo_abort("%s can only have one string value!", key);
-      cdiDefKeyString(vlistID, varID, CDI_KEY_UNITS, value.c_str());
+      cdo_define_var_units(cmorVar, vlistID, varID, value);
     }
   else if (key == "filterspec")
     {
@@ -172,12 +175,12 @@ mapvar(int vlistID, int varID, const KeyValues &kv, CmorVar &cmorVar, bool &hasV
   else if (lv1 && key == "dimensions") {}
   else
     {
-      const auto &values = kv.values;
-      const auto &rvalue = kv.values[0];
-      int nvalues = kv.nvalues;
+      auto const &values = kv.values;
+      auto const &rvalue = kv.values[0];
+      auto nvalues = kv.nvalues;
       if (nvalues == 1 && rvalue.empty()) nvalues = 0;
 
-      int dtype = literals_find_datatype(nvalues, values);
+      auto dtype = literals_find_datatype(nvalues, values);
 
       if (dtype == CDI_DATATYPE_INT8 || dtype == CDI_DATATYPE_INT16 || dtype == CDI_DATATYPE_INT32)
         {
@@ -197,9 +200,29 @@ mapvar(int vlistID, int varID, const KeyValues &kv, CmorVar &cmorVar, bool &hasV
 }
 
 static void
-apply_parameterList(pt_mode_t ptmode, PMList &pmlist, int vlistID2, std::vector<CmorVar> &cmorVars)
+search_global_missval(PMList &pmlist, bool &hasMissvals, double &missval)
 {
   const std::vector<std::string> hentry = { "Header" };
+  auto kvlist = pmlist.getKVListVentry(hentry);
+  if (kvlist)
+    {
+      auto kv = kvlist->search("missing_value");
+      if (kv && kv->nvalues > 0)
+        {
+          hasMissvals = true;
+          missval = parameter_to_double(kv->values[0]);
+        }
+    }
+}
+
+static void
+apply_parameterList(pt_mode_t ptmode, PMList &pmlist, int vlistID2, std::vector<CmorVar> &cmorVars)
+{
+  // search for global missing value
+  auto hasMissvals = false;
+  double missval = 0.0;
+  search_global_missval(pmlist, hasMissvals, missval);
+
   const std::vector<std::string> ventry = { "variable_entry", "parameter" };
   char valstr[CDI_MAX_NAME];
   char paramstr[32];
@@ -207,23 +230,6 @@ apply_parameterList(pt_mode_t ptmode, PMList &pmlist, int vlistID2, std::vector<
 
   VarList varList2(vlistID2);
 
-  // search for global missing value
-  auto hasMissvals = false;
-  double missval = 0.0;
-
-  {
-    auto kvlist = pmlist.getKVListVentry(hentry);
-    if (kvlist)
-      {
-        auto kv = kvlist->search("missing_value");
-        if (kv && kv->nvalues > 0)
-          {
-            hasMissvals = true;
-            missval = parameter_to_double(kv->values[0]);
-          }
-      }
-  }
-
   int numVarsFound = 0;
   int numVars = cmorVars.size();
   for (int varID = 0; varID < numVars; ++varID)
@@ -231,7 +237,7 @@ apply_parameterList(pt_mode_t ptmode, PMList &pmlist, int vlistID2, std::vector<
       auto &cmorVar = cmorVars[varID];
       cmorVar.name = cdo::inq_var_name(vlistID2, varID);
 
-      const auto &var2 = varList2.vars[varID];
+      auto const &var2 = varList2.vars[varID];
       if (hasMissvals)
         {
           if (fp_is_not_equal(missval, var2.missval))
@@ -292,7 +298,7 @@ apply_parameterList(pt_mode_t ptmode, PMList &pmlist, int vlistID2, std::vector<
 
           auto hasValidMin = false;
           auto hasValidMax = false;
-          for (const auto &kv : *kvlist)
+          for (auto const &kv : *kvlist)
             {
               mapvar(vlistID2, varID, kv, cmorVar, hasValidMin, hasValidMax, ptab, (ptmode != VARIABLE_NAME));
             }
@@ -318,6 +324,36 @@ apply_parameterList(pt_mode_t ptmode, PMList &pmlist, int vlistID2, std::vector<
     }
 }
 
+static void
+apply_codetable(int vlistID2, int tableID)
+{
+  char name[CDI_MAX_NAME], longname[CDI_MAX_NAME], units[CDI_MAX_NAME];
+  auto numVars = vlistNvars(vlistID2);
+  for (int varID = 0; varID < numVars; ++varID)
+    {
+      auto param = vlistInqVarParam(vlistID2, varID);
+      int pdis, pcat, pnum;
+      cdiDecodeParam(param, &pnum, &pcat, &pdis);
+      if (pdis == 255)
+        {
+          auto code = pnum;
+          int ltype = 0;
+          cdiInqKeyInt(vlistInqVarZaxis(vlistID2, varID), CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &ltype);
+          name[0] = 0;
+          longname[0] = 0;
+          units[0] = 0;
+          tableInqEntry(tableID, code, ltype, name, longname, units);
+          if (name[0])
+            {
+              cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, name);
+              if (longname[0]) cdiDefKeyString(vlistID2, varID, CDI_KEY_LONGNAME, longname);
+              if (units[0]) cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, units);
+            }
+        }
+      vlistDefVarTable(vlistID2, varID, tableID);
+    }
+}
+
 class Setpartab : public Process
 {
 public:
@@ -342,14 +378,13 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  int vlistID1;
-  int vlistID2;
+  int vlistID1{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
   std::vector<CmorVar> cmorVars;
-  int numVars;
 
   VarList varList2;
 
@@ -389,7 +424,7 @@ public:
 
     if (ptmode == CODE_NUMBER)
       {
-        const auto &partab = cdo_operator_argv(0);
+        auto const &partab = cdo_operator_argv(0);
         FILE *fp = FileUtils::file_exists(partab) ? std::fopen(partab.c_str(), "r") : nullptr;
         if (fp != nullptr)
           {
@@ -419,7 +454,7 @@ public:
     vlistID2 = vlistDuplicate(vlistID1);
     // vlistPrint(vlistID2);
 
-    numVars = vlistNvars(vlistID2);
+    auto numVars = vlistNvars(vlistID2);
     cmorVars.resize(numVars);
 
     if (convertData)
@@ -428,30 +463,7 @@ public:
     if (tableformat == 0)
       {
         // for (int varID = 0; varID < numVars; ++varID) vlistDefVarTable(vlistID2, varID, tableID);
-        char name[CDI_MAX_NAME], longname[CDI_MAX_NAME], units[CDI_MAX_NAME];
-        for (int varID = 0; varID < numVars; ++varID)
-          {
-            auto param = vlistInqVarParam(vlistID2, varID);
-            int pdis, pcat, pnum;
-            cdiDecodeParam(param, &pnum, &pcat, &pdis);
-            if (pdis == 255)
-              {
-                auto code = pnum;
-                int ltype = 0;
-                cdiInqKeyInt(vlistInqVarZaxis(vlistID2, varID), CDI_GLOBAL, CDI_KEY_TYPEOFFIRSTFIXEDSURFACE, &ltype);
-                name[0] = 0;
-                longname[0] = 0;
-                units[0] = 0;
-                tableInqEntry(tableID, code, ltype, name, longname, units);
-                if (name[0])
-                  {
-                    cdiDefKeyString(vlistID2, varID, CDI_KEY_NAME, name);
-                    if (longname[0]) cdiDefKeyString(vlistID2, varID, CDI_KEY_LONGNAME, longname);
-                    if (units[0]) cdiDefKeyString(vlistID2, varID, CDI_KEY_UNITS, units);
-                  }
-              }
-            vlistDefVarTable(vlistID2, varID, tableID);
-          }
+        apply_codetable(vlistID2, tableID);
       }
     else
       {
@@ -467,7 +479,7 @@ public:
           apply_parameterList(ptmode, pmlist, vlistID2, cmorVars);
         }
 
-        for (auto &var : cmorVars)
+        for (auto const &var : cmorVars)
           if (var.remove)
             {
               deleteVars = true;
@@ -508,8 +520,7 @@ public:
           {
             if (!var.convert) var.changeUnits = false;
             if (var.changeUnits)
-              cdo::convert_units(&var.ut_converter, &var.changeUnits, (char *) &var.units, (char *) &var.unitsOld,
-                                 var.name.c_str());
+              cdo::convert_units(&var.ut_converter, &var.changeUnits, (char *) &var.units, (char *) &var.unitsOld, var.name);
           }
       }
 
@@ -527,24 +538,24 @@ public:
   void
   run() override
   {
-    auto gridsizemax = vlistGridsizeMax(vlistID1);
-    if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
-    Varray<double> array(gridsizemax);
+    auto gridsizeMax = varList2.gridsizeMax();
+    if (vlistNumber(vlistID1) != CDI_REAL) gridsizeMax *= 2;
+    Varray<double> array(gridsizeMax);
 
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
         cmor_check_init(cmorVars);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
 
             auto &cmorVar = cmorVars[varID];
             auto varID2 = varID;
@@ -561,12 +572,12 @@ public:
                   }
               }
 
-            cdo_def_record(streamID2, varID2, levelID2);
+            cdo_def_field(streamID2, varID2, levelID2);
 
             size_t numMissVals;
-            cdo_read_record(streamID1, array.data(), &numMissVals);
+            cdo_read_field(streamID1, array.data(), &numMissVals);
 
-            const auto &var2 = varList2.vars[varID2];
+            auto const &var2 = varList2.vars[varID2];
             auto missval = var2.missval;
             auto gridsize = var2.nwpv * var2.gridsize;
 
@@ -607,7 +618,7 @@ public:
               }
 #endif
 
-            cdo_write_record(streamID2, array.data(), numMissVals);
+            cdo_write_field(streamID2, array.data(), numMissVals);
 
             cmor_check_prep(cmorVar, gridsize, missval, array.data());
           }
