@@ -56,7 +56,7 @@ shift_time(int calendar, int tunit, int64_t ijulinc, CdiDateTime &vDateTime)
 }
 
 static void
-time_gen_bounds(int calendar, int tunit, int incrPeriod, const CdiDateTime &vDateTime, CdiDateTime *vDateTimeBounds)
+time_gen_bounds(int calendar, int tunit, int incrPeriod, CdiDateTime const &vDateTime, CdiDateTime *vDateTimeBounds)
 {
   cdiDateTime_init(&vDateTimeBounds[0]);
   cdiDateTime_init(&vDateTimeBounds[1]);
@@ -116,7 +116,7 @@ time_gen_bounds(int calendar, int tunit, int incrPeriod, const CdiDateTime &vDat
 }
 
 int
-evaluate_calendar_string(int operatorID, const std::string &calendarName)
+evaluate_calendar_string(int operatorID, std::string const &calendarName)
 {
   int calendar = CALENDAR_STANDARD;
   auto calendarString = string_to_lower(calendarName);
@@ -168,17 +168,19 @@ public:
   using Process::Process;
   inline static CdoModule module = {
     .name = "Settime",
-    .operators = { { "setyear", 0, 1, "year", SettimeHelp },
-                   { "setmon", 0, 1, "month", SettimeHelp },
-                   { "setday", 0, 1, "day", SettimeHelp },
-                   { "setdate", 0, 1, "date(format:YYYY-MM-DD)", SettimeHelp },
-                   { "settime", 0, 1, "time(format:hh:mm:ss)", SettimeHelp },
-                   { "settunits", 0, 1, "timeunits(seconds,minutes,hours,days,months,years)", SettimeHelp },
-                   { "settaxis", 0, -2, "date<,time<,increment>>(formatYYYY-MM-DD,hh:mm:ss)", SettimeHelp },
-                   { "settbounds", 0, 1, "frequency(hour,day,month,year)", SettimeHelp },
-                   { "setreftime", 0, -2, "date<,time<,units>>(formatYYYY-MM-DD,hh:mm:ss)", SettimeHelp },
-                   { "setcalendar", 0, 1, "calendar(standard,proleptic_gregorian,360_day,365_day,366_day)", SettimeHelp },
-                   { "shifttime", 0, 1, "shiftvalue", SettimeHelp } },
+    // clang-format off
+    .operators = { { "setyear",     0,  1, "year", SettimeHelp },
+                   { "setmon",      0,  1, "month", SettimeHelp },
+                   { "setday",      0,  1, "day", SettimeHelp },
+                   { "setdate",     0,  1, "date (format: YYYY-MM-DD)", SettimeHelp },
+                   { "settime",     0,  1, "time (format: hh:mm:ss)", SettimeHelp },
+                   { "settunits",   0,  1, "timeunits (seconds|minutes|hours|days|months|years)", SettimeHelp },
+                   { "settaxis",    0, -2, "date<,time<,increment>> (format: YYYY-MM-DD,hh:mm:ss)", SettimeHelp },
+                   { "settbounds",  0,  1, "frequency (hour|day|month|year)", SettimeHelp },
+                   { "setreftime",  0, -2, "date<,time<,units>> (format: YYYY-MM-DD,hh:mm:ss)", SettimeHelp },
+                   { "setcalendar", 0,  1, "calendar (standard|proleptic_gregorian|360_day|365_day|366_day)", SettimeHelp },
+                   { "shifttime",   0,  1, "shiftvalue", SettimeHelp } },
+    // clang-format on
     .aliases = {},
     .mode = EXPOSED,     // Module mode: 0:intern 1:extern
     .number = CDI_BOTH,  // Allowed number type
@@ -204,11 +206,11 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2 = CDO_STREAM_UNDEF;
 
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
   int taxisID2 = CDI_UNDEFID;
 
-  int vlistID1;
-  int vlistID2;
+  int vlistID1{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
   int operatorID;
   int ntsteps;
@@ -375,8 +377,8 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         auto vDateTime = taxisInqVdatetime(taxisID1);
 
@@ -473,15 +475,14 @@ public:
 
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            cdo_def_record(streamID2, varID, levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            cdo_def_field(streamID2, varID, levelID);
 
             field.init(varList1.vars[varID]);
-            cdo_read_record(streamID1, field);
-            cdo_write_record(streamID2, field);
+            cdo_read_field(streamID1, field);
+            cdo_write_field(streamID2, field);
           }
 
         tsID++;
