@@ -25,6 +25,7 @@
 
 #include "process_int.h"
 #include "printinfo.h"
+#include "cdo_history.h"
 #include "cdo_zaxis.h"
 
 static void
@@ -54,11 +55,11 @@ show_year(CdoStreamID streamID)
   int tsID = 0;
   while (true)
     {
-      auto nrecs = cdo_stream_inq_timestep(streamID, tsID);
-      if (nrecs == 0) break;
+      auto numFields = cdo_stream_inq_timestep(streamID, tsID);
+      if (numFields == 0) break;
 
       auto vDateTime = taxisInqVdatetime(taxisID);
-      const int year = vDateTime.date.year;
+      int year = vDateTime.date.year;
 
       if (tsID == 0 || year0 != year)
         {
@@ -88,11 +89,11 @@ show_mon(CdoStreamID streamID)
   int tsID = 0;
   while (true)
     {
-      auto nrecs = cdo_stream_inq_timestep(streamID, tsID);
-      if (nrecs == 0) break;
+      auto numFields = cdo_stream_inq_timestep(streamID, tsID);
+      if (numFields == 0) break;
 
       auto vDateTime = taxisInqVdatetime(taxisID);
-      const int month = vDateTime.date.month;
+      int month = vDateTime.date.month;
 
       if (tsID == 0 || month0 != month)
         {
@@ -122,8 +123,8 @@ show_date(CdoStreamID streamID)
   int tsID = 0;
   while (true)
     {
-      auto nrecs = cdo_stream_inq_timestep(streamID, tsID);
-      if (nrecs == 0) break;
+      auto numFields = cdo_stream_inq_timestep(streamID, tsID);
+      if (numFields == 0) break;
 
       auto vDateTime = taxisInqVdatetime(taxisID);
       auto vdate = cdiDate_get(vDateTime.date);
@@ -155,8 +156,8 @@ show_time(CdoStreamID streamID)
   int tsID = 0;
   while (true)
     {
-      auto nrecs = cdo_stream_inq_timestep(streamID, tsID);
-      if (nrecs == 0) break;
+      auto numFields = cdo_stream_inq_timestep(streamID, tsID);
+      if (numFields == 0) break;
 
       auto vDateTime = taxisInqVdatetime(taxisID);
       nout++;
@@ -182,8 +183,8 @@ show_timestamp(CdoStreamID streamID)
   int tsID = 0;
   while (true)
     {
-      auto nrecs = cdo_stream_inq_timestep(streamID, tsID);
-      if (nrecs == 0) break;
+      auto numFields = cdo_stream_inq_timestep(streamID, tsID);
+      if (numFields == 0) break;
 
       nout++;
       fprintf(stdout, " %s", datetime_to_string(taxisInqVdatetime(taxisID)).c_str());
@@ -200,11 +201,11 @@ show_code(const VarList &varList)
   constexpr int maxOut = 25;
   int nout = 0;
 
-  int numVars = varList.numVars();
+  auto numVars = varList.numVars();
   for (int varID = 0; varID < numVars; ++varID)
     {
       nout++;
-      const auto &var = varList.vars[varID];
+      auto const &var = varList.vars[varID];
       fprintf(stdout, " %d", var.code);
       print_newline(nout, maxOut);
     }
@@ -216,7 +217,7 @@ show_grid(const VarList &varList)
 {
   fprintf(stdout, "# param nr | grid nr | z-axis nr:   /* Use in combination with operatores: griddes and zaxisdes */\n");
   auto vlistID = varList.vlistID;
-  for (const auto &var : varList.vars)
+  for (auto const &var : varList.vars)
     {
       fprintf(stdout, "      %3d     %3d      %3d\n", var.code, vlistGridIndex(vlistID, var.gridID) + 1,
               vlistZaxisIndex(vlistID, var.zaxisID) + 1);
@@ -229,7 +230,7 @@ show_unit(const VarList &varList)
   constexpr int maxOut = 10;
   int nout = 0;
 
-  for (const auto &var : varList.vars)
+  for (auto const &var : varList.vars)
     {
       nout++;
       if (var.units.size()) fprintf(stdout, " %s", var.units.c_str());
@@ -245,11 +246,9 @@ show_param(const VarList &varList)
   int nout = 0;
 
   char paramstr[32];
-  int numVars = varList.numVars();
-  for (int varID = 0; varID < numVars; ++varID)
+  for (auto const &var : varList.vars)
     {
       nout++;
-      const auto &var = varList.vars[varID];
       cdiParamToString(var.param, paramstr, sizeof(paramstr));
 
       fprintf(stdout, " %s", paramstr);
@@ -261,33 +260,26 @@ show_param(const VarList &varList)
 static void
 show_name(const VarList &varList)
 {
-  constexpr int maxOut = 10;
   int nout = 0;
-
-  int numVars = varList.numVars();
-  for (int varID = 0; varID < numVars; ++varID)
+  for (auto const &var : varList.vars)
     {
       nout++;
-      const auto &var = varList.vars[varID];
-      fprintf(stdout, " %s", var.name.c_str());
-      print_newline(nout, maxOut);
+      if (nout > 1) fprintf(stdout, " ");
+      fprintf(stdout, "%s", var.name.c_str());
     }
-  print_newline_if_missing(nout, maxOut);
+  fprintf(stdout, "\n");
 }
 
 static void
-show_stdname(int vlistID)
+show_stdname(const VarList &varList)
 {
-  auto numVars = vlistNvars(vlistID);
-
   constexpr int maxOut = 1;
   int nout = 0;
 
-  for (int varID = 0; varID < numVars; ++varID)
+  for (auto const &var : varList.vars)
     {
       nout++;
-      auto stdname = cdo::inq_key_string(vlistID, varID, CDI_KEY_STDNAME);
-      fprintf(stdout, " %s", stdname.size() ? stdname.c_str() : "unknown");
+      fprintf(stdout, " %s", var.stdname.size() ? var.stdname.c_str() : "unknown");
       print_newline(nout, maxOut);
     }
   print_newline_if_missing(nout, maxOut);
@@ -296,10 +288,8 @@ show_stdname(int vlistID)
 static void
 show_level(const VarList &varList)
 {
-  int numVars = varList.numVars();
-  for (int varID = 0; varID < numVars; ++varID)
+  for (auto const &var : varList.vars)
     {
-      const auto &var = varList.vars[varID];
       for (int levelID = 0; levelID < var.nlevels; ++levelID) fprintf(stdout, " %.9g", cdo_zaxis_inq_level(var.zaxisID, levelID));
       fprintf(stdout, "\n");
     }
@@ -308,8 +298,8 @@ show_level(const VarList &varList)
 static void
 show_ltype(int vlistID)
 {
-  auto nzaxis = vlistNumZaxis(vlistID);
-  for (int index = 0; index < nzaxis; ++index)
+  auto numZaxes = vlistNumZaxis(vlistID);
+  for (int index = 0; index < numZaxes; ++index)
     {
       auto zaxisID = vlistZaxis(vlistID, index);
       auto ltype = zaxis_to_ltype(zaxisID);
@@ -319,6 +309,43 @@ show_ltype(int vlistID)
   fprintf(stdout, "\n");
 }
 
+static void
+show_filter(int vlistID, const VarList &varList)
+{
+  char filterSpec[CDI_MAX_NAME];
+  auto numVars = varList.numVars();
+  for (int varID = 0; varID < numVars; ++varID)
+    {
+      auto comptype = vlistInqVarCompType(vlistID, varID);
+      if (comptype == CDI_COMPRESS_FILTER || comptype == CDI_COMPRESS_ZIP || comptype == CDI_COMPRESS_SZIP)
+        {
+          auto const &var = varList.vars[varID];
+          int length = CDI_MAX_NAME;
+          cdiInqKeyString(vlistID, varID, CDI_KEY_FILTERSPEC_IN, filterSpec, &length);
+          if (length > 0) fprintf(stdout, "%s=\"%s\"\n", var.name.c_str(), filterSpec);
+        }
+    }
+}
+
+static void
+show_chunkspec(int vlistID, const VarList &varList)
+{
+  auto numVars = varList.numVars();
+  for (int varID = 0; varID < numVars; ++varID)
+    {
+      auto const &var = varList.vars[varID];
+      auto chunkSpecString = cdo::get_chunkspec_string(vlistID, varID);
+      if (chunkSpecString.size() > 0) fprintf(stdout, "%s=\"%s\"\n", var.name.c_str(), chunkSpecString.c_str());
+    }
+}
+
+static void
+show_history(int vlistID)
+{
+  auto historyString = cdo_inq_history(vlistID);
+  fprintf(stdout, "%s\n", historyString.c_str());
+}
+
 class Showinfo : public Process
 {
 public:
@@ -339,8 +366,9 @@ public:
                    { "showltype", ShowinfoHelp },
                    { "showformat", ShowinfoHelp },
                    { "showgrid", ShowinfoHelp },
-                   { "showatts", ShowinfoHelp },
-                   { "showattsglob", ShowinfoHelp } },
+                   { "showchunkspec", ShowinfoHelp },
+                   { "showhistory" },
+                   { "showfilter", ShowinfoHelp } },
     .aliases = { { "showvar", "showname" } },
     .mode = EXPOSED,     // Module mode: 0:intern 1:extern
     .number = CDI_BOTH,  // Allowed number type
@@ -348,44 +376,42 @@ public:
   };
   inline static RegisterEntry<Showinfo> registration = RegisterEntry<Showinfo>(module);
 
-  int SHOWYEAR, SHOWMON, SHOWDATE, SHOWTIME, SHOWTIMESTAMP, SHOWCODE, SHOWUNIT, SHOWPARAM, SHOWNAME, SHOWSTDNAME, SHOWLEVEL,
-      SHOWLTYPE, SHOWFORMAT, SHOWGRID, SHOWATTS, SHOWATTSGLOB;
-  int operatorID;
-  CdoStreamID streamID;
-  int vlistID;
-  VarList varList;
-
 public:
   void
   init() override
   {
-    SHOWYEAR = module.get_id("showyear");
-    SHOWMON = module.get_id("showmon");
-    SHOWDATE = module.get_id("showdate");
-    SHOWTIME = module.get_id("showtime");
-    SHOWTIMESTAMP = module.get_id("showtimestamp");
-    SHOWCODE = module.get_id("showcode");
-    SHOWUNIT = module.get_id("showunit");
-    SHOWPARAM = module.get_id("showparam");
-    SHOWNAME = module.get_id("showname");
-    SHOWSTDNAME = module.get_id("showstdname");
-    SHOWLEVEL = module.get_id("showlevel");
-    SHOWLTYPE = module.get_id("showltype");
-    SHOWFORMAT = module.get_id("showformat");
-    SHOWGRID = module.get_id("showgrid");
+    cdiDefGlobal("COPY_CHUNKSPEC", true);
 
-    operatorID = cdo_operator_id();
+    if (Options::lazyGridLoad && this_is_the_only_process()) { cdiDefGlobal("NETCDF_LAZY_GRID_LOAD", true); }
+    if (this_is_the_only_process()) { cdiDefGlobal("READ_CELL_CORNERS", false); }
+    if (this_is_the_only_process()) { cdiDefGlobal("READ_CELL_CENTER", false); }
+
+    auto SHOWYEAR = module.get_id("showyear");
+    auto SHOWMON = module.get_id("showmon");
+    auto SHOWDATE = module.get_id("showdate");
+    auto SHOWTIME = module.get_id("showtime");
+    auto SHOWTIMESTAMP = module.get_id("showtimestamp");
+    auto SHOWCODE = module.get_id("showcode");
+    auto SHOWUNIT = module.get_id("showunit");
+    auto SHOWPARAM = module.get_id("showparam");
+    auto SHOWNAME = module.get_id("showname");
+    auto SHOWSTDNAME = module.get_id("showstdname");
+    auto SHOWLEVEL = module.get_id("showlevel");
+    auto SHOWLTYPE = module.get_id("showltype");
+    auto SHOWFORMAT = module.get_id("showformat");
+    auto SHOWGRID = module.get_id("showgrid");
+    auto SHOWCHUNKSPEC = module.get_id("showchunkspec");
+    auto SHOWFILTER = module.get_id("showfilter");
+    auto SHOWHISTORY = module.get_id("showhistory");
+
+    auto operatorID = cdo_operator_id();
 
     operator_check_argc(0);
 
-    streamID = cdo_open_read(0);
-    vlistID = cdo_stream_inq_vlist(streamID);
-    varList = VarList(vlistID);
-  }
+    auto streamID = cdo_open_read(0);
+    auto vlistID = cdo_stream_inq_vlist(streamID);
+    VarList varList(vlistID);
 
-  void
-  run() override
-  {
     // clang-format off
     if      (operatorID == SHOWYEAR)      show_year(streamID);
     else if (operatorID == SHOWMON)       show_mon(streamID);
@@ -397,16 +423,25 @@ public:
     else if (operatorID == SHOWUNIT)      show_unit(varList);
     else if (operatorID == SHOWPARAM)     show_param(varList);
     else if (operatorID == SHOWNAME)      show_name(varList);
-    else if (operatorID == SHOWSTDNAME)   show_stdname(vlistID);
+    else if (operatorID == SHOWSTDNAME)   show_stdname(varList);
     else if (operatorID == SHOWLEVEL)     show_level(varList);
     else if (operatorID == SHOWLTYPE)     show_ltype(vlistID);
     else if (operatorID == SHOWFORMAT)    print_filetype(streamID, vlistID);
+    else if (operatorID == SHOWCHUNKSPEC) show_chunkspec(vlistID, varList);
+    else if (operatorID == SHOWFILTER)    show_filter(vlistID, varList);
+    else if (operatorID == SHOWHISTORY)   show_history(vlistID);
     // clang-format on
+
+    cdo_stream_close(streamID);
+  }
+
+  void
+  run() override
+  {
   }
 
   void
   close() override
   {
-    cdo_stream_close(streamID);
   }
 };
