@@ -8,13 +8,14 @@
 /*
    This module contains the following operators:
 
-      Sinfo      sinfo           Short dataset information
+      Sinfo      sinfo           Short dataset informationl
 */
 
 #include "cdi.h"
 #include "julian_date.h"
 
 #include <cstring>
+#include <string>
 #include "cdo_options.h"
 #include "printinfo.h"
 #include "mpmo_color.h"
@@ -46,7 +47,7 @@ num_values_to_byte_cstr(size_t numValues)
 
   size_t muindex = 0;
   const char *mu[] = { "Bytes", "Kbytes", "Mbytes", "Gbytes", "Tbytes", "Pbytes" };
-  const size_t nmu = sizeof(mu) / sizeof(char *);
+  size_t nmu = sizeof(mu) / sizeof(char *);
   while (numValues > 9999 && muindex < nmu - 1)
     {
       numValues /= 1024;
@@ -94,7 +95,7 @@ static void
 limit_string_length(char *string, size_t maxlen)
 {
   string[maxlen - 1] = 0;
-  auto len = strlen(string);
+  auto len = std::strlen(string);
   if (len > 10)
     {
       for (size_t i = 3; i < len; ++i)
@@ -117,7 +118,7 @@ print_vars_info(int operfunc, bool ensembleInfo, const VarList &varList, int vli
 
   for (int varID = 0; varID < numVars; ++varID)
     {
-      const auto &var = varList.vars[varID];
+      auto const &var = varList.vars[varID];
 
       auto tabnum = tableInqNum(vlistInqVarTable(vlistID, varID));
 
@@ -199,7 +200,7 @@ print_vars_info(int operfunc, bool ensembleInfo, const VarList &varList, int vli
       // set_text_color(stdout, GREEN);
       // clang-format off
       if      (operfunc == func_name) fprintf(stdout, "%-14s", var.name.c_str());
-      else if (operfunc == func_code) fprintf(stdout, "%4d %4d", tabnum, var.code);
+      else if (operfunc == func_code) fprintf(stdout, " %4d %4d", tabnum, var.code);
       else                            fprintf(stdout, "%-14s", paramstr);
       // clang-format on
       if (xsInfo && Options::cdoVerbose && operfunc == func_name && var.units.size()) fprintf(stdout, " [%s]", var.units.c_str());
@@ -207,8 +208,8 @@ print_vars_info(int operfunc, bool ensembleInfo, const VarList &varList, int vli
 
       if (Options::cdoVerbose)
         {
-          auto chunks = cdo::inq_key_string(vlistID, varID, CDI_KEY_CHUNKS);
-          fprintf(stdout, " : %s", chunks.c_str());
+          auto chunkSpecString = cdo::get_chunkspec_string(vlistID, varID);
+          fprintf(stdout, " : %s", chunkSpecString.c_str());
         }
 
       fprintf(stdout, "\n");
@@ -236,9 +237,9 @@ print_time_info(int ntsteps, int taxisID)
 
   if (Options::cdoVerbose)
     {
-      // int datatype;
-      // cdiInqKeyInt(taxisID, CDI_GLOBAL, CDI_KEY_DATATYPE, &datatype);
-      // fprintf(stdout, "%33s : %s\n", "datatype", cdo::datatype_to_cstr(datatype));
+      int datatype;
+      cdiInqKeyInt(taxisID, CDI_GLOBAL, CDI_KEY_DATATYPE, &datatype);
+      fprintf(stdout, "%33s : %s\n", "datatype", cdo::datatype_to_cstr(datatype));
       fprintf(stdout, "%33s : %d\n", "taxisID", taxisID);
     }
 
@@ -288,8 +289,8 @@ print_time_info_xs(int ntsteps, int taxisID, CdoStreamID streamID)
       int tsID = 0;
       while (true)
         {
-          auto nrecs = cdo_stream_inq_timestep(streamID, tsID);
-          if (nrecs == 0) break;
+          auto numFields = cdo_stream_inq_timestep(streamID, tsID);
+          if (numFields == 0) break;
 
           auto vDateTime = taxisInqVdatetime(taxisID);
 
@@ -361,6 +362,14 @@ print_time_info_xs(int ntsteps, int taxisID, CdoStreamID streamID)
         }
 
       if (taxisHasBounds(taxisID)) fprintf(stdout, "%33s : %s\n", "available", "bounds");
+
+      if (Options::cdoVerbose)
+        {
+          int datatype;
+          cdiInqKeyInt(taxisID, CDI_GLOBAL, CDI_KEY_DATATYPE, &datatype);
+          fprintf(stdout, "%33s : %s\n", "datatype", cdo::datatype_to_cstr(datatype));
+          fprintf(stdout, "%33s : %d\n", "taxisID", taxisID);
+        }
     }
 
   return numTimesteps;
@@ -392,14 +401,16 @@ public:
   inline static RegisterEntry<Sinfo> registration = RegisterEntry<Sinfo>(module);
 
 private:
-  int operfunc;
-  int ensembleInfo;
-  bool xsInfo;
+  int operfunc{ 0 };
+  int ensembleInfo{ 0 };
+  bool xsInfo{ false };
 
 public:
   void
   init() override
   {
+    cdiDefGlobal("COPY_CHUNKSPEC", true);
+
     auto operatorID = cdo_operator_id();
 
     operfunc = cdo_operator_f1(operatorID);
@@ -436,13 +447,13 @@ public:
         fprintf(stdout, " : %s",
                 (operfunc == func_name) ? "Parameter name" : ((operfunc == func_code) ? "Table Code" : "Parameter ID"));
 
-        if (Options::cdoVerbose) fprintf(stdout, " : Extra");
+        if (Options::cdoVerbose) fprintf(stdout, " : Chunkspec");
         reset_text_color(stdout);
         fprintf(stdout, "\n");
 
         auto numVars = varList.numVars();
-        int numRecordsConst = 0;
-        int numRecordsVar = 0;
+        int numFieldsConst = 0;
+        int numFieldsVar = 0;
         size_t numValues = 0;
         // size_t memorySize = 0;
         size_t inputSize = 0;
@@ -451,12 +462,12 @@ public:
           {
             for (int varID = 0; varID < numVars; ++varID)
               {
-                const auto &var = varList.vars[varID];
+                auto const &var = varList.vars[varID];
 
                 if (var.isConstant)
-                  numRecordsConst += var.nlevels;
+                  numFieldsConst += var.nlevels;
                 else
-                  numRecordsVar += var.nlevels;
+                  numFieldsVar += var.nlevels;
 
                 auto size = var.nlevels * var.gridsize * var.nwpv;
                 numValues += size;
@@ -524,7 +535,7 @@ public:
             reset_text_color(stdout);
             fprintf(stdout, " :\n");
 
-            int numFields = numRecordsVar * numTimesteps + numRecordsConst;
+            int numFields = numFieldsVar * numTimesteps + numFieldsConst;
             fprintf(stdout, "%33s : %d\n", "number of fields", numFields);
             fprintf(stdout, "%33s : %d\n", "number of variables", numVars);
             if (numTimesteps) fprintf(stdout, "%33s : %d\n", "number of timesteps", numTimesteps);
