@@ -26,24 +26,22 @@
 #include "cdo_options.h"
 #include "progress.h"
 #include "cdo_omp.h"
-#include "grid_point_search.h"
-
-enum class CurveForm
-{
-  AVG = 0,
-  LINEAR,
-};
-
-static const char *Form[] = { "avg", "linear" };
+#include "grid_pointsearch.h"
+#include "interpol.h"
 
 struct SmoothPoint
 {
-  double arc_radius = 0.0;
-  double radius = 1.0;
-  double weight0 = 0.25;
-  double weightR = 0.25;
-  size_t maxpoints = SIZE_MAX;
-  CurveForm form = CurveForm::LINEAR;
+  double arc_radius{ 0.0 };
+  double radius{ 1.0 };
+  size_t maxpoints{ SIZE_MAX };
+  KnnParams knnParams;
+
+  SmoothPoint()
+  {
+    knnParams.weighted = WeightingMethod::linear;
+    knnParams.weight0 = 0.25;
+    knnParams.weightR = 0.25;
+  }
 };
 
 template <typename T1, typename T2>
@@ -56,7 +54,7 @@ smooth(int gridID, double mv, const Varray<T1> &array1, Varray<T2> &array2, cons
   auto numNeighbors = spoint.maxpoints;
   if (numNeighbors > gridsize) numNeighbors = gridsize;
 
-  Varray<int8_t> mask(gridsize);
+  Vmask mask(gridsize);
   for (size_t i = 0; i < gridsize; ++i) mask[i] = fp_is_not_equal(array1[i], missval);
 
   gridID = generate_full_point_grid(gridID);
@@ -70,18 +68,19 @@ smooth(int gridID, double mv, const Varray<T1> &array1, Varray<T2> &array2, cons
   cdo_grid_to_radian(gridID, CDI_XAXIS, xvals, "grid center lon");
   cdo_grid_to_radian(gridID, CDI_YAXIS, yvals, "grid center lat");
 
-  std::vector<knnWeightsType> knnWeights;
-  for (int i = 0; i < Threading::ompNumThreads; ++i) knnWeights.push_back(knnWeightsType(numNeighbors));
+  auto knnParams = spoint.knnParams;
+  knnParams.k = numNeighbors;
+  knnParams.kMin = 1;
+  knnParams.searchRadius = spoint.radius;
+
+  std::vector<KnnData> knnDataList;
+  for (int i = 0; i < Threading::ompNumMaxThreads; ++i) knnDataList.emplace_back(knnParams);
 
   cdo::timer timer;
 
-  GridPointSearch gps;
-  grid_point_search_create(gps, xvals, yvals, true);
-
-  if (spoint.arc_radius > 0.0)
-    grid_point_search_set_arc_radius(gps, spoint.arc_radius);
-  else
-    grid_point_search_set_chord_radius(gps, spoint.radius);
+  GridPointsearch gps;
+  gps.set_radius((spoint.arc_radius > 0.0) ? arc_to_chord_length(spoint.arc_radius) : spoint.radius);
+  grid_pointsearch_create_unstruct(gps, xvals, yvals, true);
 
   if (Options::cdoVerbose) cdo_print("Point search created: %.2f seconds (%zu points)", timer.elapsed(), gridsize);
 
@@ -89,38 +88,39 @@ smooth(int gridID, double mv, const Varray<T1> &array1, Varray<T2> &array2, cons
 
   timer.reset();
 
-  size_t naddsMin = gridsize, naddsMax = 0;
+  size_t numWeightsMin = gridsize, numWeightsMax = 0;
   std::atomic<size_t> atomicCount{ 0 }, atomicSum{ 0 }, atomicNumMiss{ 0 };
 
 #ifdef HAVE_OPENMP4
-#pragma omp parallel for default(shared) schedule(dynamic) reduction(min : naddsMin) reduction(max : naddsMax)
+#pragma omp parallel for default(shared) schedule(dynamic) reduction(min : numWeightsMin) reduction(max : numWeightsMax)
 #endif
   for (size_t i = 0; i < gridsize; ++i)
     {
-      auto ompthID = cdo_omp_get_thread_num();
-
       atomicCount++;
-      if (cdo_omp_get_thread_num() == 0) progress.update((double) atomicCount / gridsize);
+      auto ompthID = cdo_omp_get_thread_num();
+      if (ompthID == 0 && gridsize > progressMinSize) progress.update((double) atomicCount / gridsize);
 
-      grid_search_point_smooth(gps, xvals[i], yvals[i], knnWeights[ompthID]);
+      grid_search_point_smooth(gps, PointLonLat{ xvals[i], yvals[i] }, knnDataList[ompthID]);
 
-      // Compute weights based on inverse distance if mask is false, eliminate those points
-      auto nadds = knnWeights[ompthID].computeWeights(mask, spoint.radius, spoint.weight0, spoint.weightR);
-      naddsMin = std::min(naddsMin, nadds);
-      naddsMax = std::max(naddsMax, nadds);
+      // Compute weights if mask is false, eliminate those points
+      auto numWeights = knnDataList[ompthID].compute_weights(mask);
 
-      array2[i] = nadds ? knnWeights[ompthID].arrayWeightsSum(array1) : missval;
-      atomicSum += nadds;
-      if (nadds == 0) atomicNumMiss++;
+      array2[i] = numWeights ? knnDataList[ompthID].array_weights_sum(array1) : missval;
+      atomicSum += numWeights;
+      if (numWeights == 0) atomicNumMiss++;
+
+      if (Options::cdoVerbose)
+        {
+          numWeightsMin = std::min(numWeightsMin, numWeights);
+          numWeightsMax = std::max(numWeightsMax, numWeights);
+        }
     }
 
   size_t numMissValsx = atomicNumMiss;
   size_t numPoints = atomicSum;
 
   if (Options::cdoVerbose) cdo_print("Point search nearest: %.2f seconds (%zu points)", timer.elapsed(), numPoints);
-  if (Options::cdoVerbose) cdo_print("Min/Max points found: %zu/%zu", naddsMin, naddsMax);
-
-  grid_point_search_delete(gps);
+  if (Options::cdoVerbose) cdo_print("Min/Max points found: %zu/%zu", numWeightsMin, numWeightsMax);
 
   if (gridID0 != gridID) gridDestroy(gridID);
 
@@ -128,23 +128,12 @@ smooth(int gridID, double mv, const Varray<T1> &array1, Varray<T2> &array2, cons
 }
 
 static void
-smooth(const Field &field1, Field &field2, const SmoothPoint &spoint)
+smooth(Field const &field1, Field &field2, const SmoothPoint &spoint)
 {
-  auto func = [&](const auto &v1, auto &v2) { field2.numMissVals = smooth(field1.grid, field1.missval, v1, v2, spoint); };
+  auto func = [&](auto const &v1, auto &v2) { field2.numMissVals = smooth(field1.grid, field1.missval, v1, v2, spoint); };
   field_operation2(func, field1, field2);
 }
 
-template <typename T>
-static inline void
-smooth9_sum(size_t ij, const std::vector<int8_t> &mask, double sfac, const Varray<T> &array, double &avg, double &divavg)
-{
-  if (mask[ij])
-    {
-      avg += sfac * array[ij];
-      divavg += sfac;
-    }
-}
-
 template <typename T1, typename T2>
 static size_t
 smooth9(int gridID, double mv, const Varray<T1> &array1, Varray<T2> &array2)
@@ -155,80 +144,74 @@ smooth9(int gridID, double mv, const Varray<T1> &array1, Varray<T2> &array2)
   auto nlat = gridInqYsize(gridID);
   auto gridIsCyclic = gridIsCircular(gridID);
 
-  std::vector<int8_t> mask(gridsize);
+  Vmask mask(gridsize);
 
   for (size_t i = 0; i < gridsize; ++i) mask[i] = fp_is_not_equal(missval, array1[i]);
 
+  double avg = 0;
+  double divavg = 0;
+  auto smooth9_sum = [&](double sfac, size_t ij) {
+    if (mask[ij])
+      {
+        avg += sfac * array1[ij];
+        divavg += sfac;
+      }
+  };
+
   size_t numMissVals = 0;
   for (size_t i = 0; i < nlat; ++i)
     {
       for (size_t j = 0; j < nlon; ++j)
         {
-          double avg = 0, divavg = 0;
+          avg = 0;
+          divavg = 0;
 
           if ((i == 0) || (j == 0) || (i == (nlat - 1)) || (j == (nlon - 1)))
             {
               auto ij = j + nlon * i;
+              // clang-format off
               if (mask[ij])
                 {
                   avg += array1[ij];
                   divavg += 1;
                   // upper left corner
-                  if ((i != 0) && (j != 0))
-                    smooth9_sum(((i - 1) * nlon) + j - 1, mask, 0.3, array1, avg, divavg);
-                  else if (i != 0 && gridIsCyclic)
-                    smooth9_sum((i - 1) * nlon + j - 1 + nlon, mask, 0.3, array1, avg, divavg);
-
+                  if      (i != 0 && j != 0)                   smooth9_sum(0.3, (i - 1) * nlon + j - 1);
+                  else if (i != 0 && gridIsCyclic)             smooth9_sum(0.3, (i - 1) * nlon + j - 1 + nlon);
                   // upper cell
-                  if (i != 0) smooth9_sum(((i - 1) * nlon) + j, mask, 0.5, array1, avg, divavg);
-
+                  if      (i != 0)                             smooth9_sum(0.5, (i - 1) * nlon + j);
                   // upper right corner
-                  if ((i != 0) && (j != (nlon - 1)))
-                    smooth9_sum(((i - 1) * nlon) + j + 1, mask, 0.3, array1, avg, divavg);
-                  else if ((i != 0) && gridIsCyclic)
-                    smooth9_sum((i - 1) * nlon + j + 1 - nlon, mask, 0.3, array1, avg, divavg);
-
+                  if      (i != 0 && j != (nlon - 1))          smooth9_sum(0.3, (i - 1) * nlon + j + 1);
+                  else if (i != 0 && gridIsCyclic)             smooth9_sum(0.3, (i - 1) * nlon + j + 1 - nlon);
                   // left cell
-                  if (j != 0)
-                    smooth9_sum(i * nlon + j - 1, mask, 0.5, array1, avg, divavg);
-                  else if (gridIsCyclic)
-                    smooth9_sum(i * nlon - 1 + nlon, mask, 0.5, array1, avg, divavg);
-
+                  if      (j != 0)                             smooth9_sum(0.5, i * nlon + j - 1);
+                  else if (gridIsCyclic)                       smooth9_sum(0.5, i * nlon - 1 + nlon);
                   // right cell
-                  if (j != (nlon - 1))
-                    smooth9_sum((i * nlon) + j + 1, mask, 0.5, array1, avg, divavg);
-                  else if (gridIsCyclic)
-                    smooth9_sum(i * nlon + j + 1 - nlon, mask, 0.5, array1, avg, divavg);
-
+                  if      (j != (nlon - 1))                    smooth9_sum(0.5, i * nlon + j + 1);
+                  else if (gridIsCyclic)                       smooth9_sum(0.5, i * nlon + j + 1 - nlon);
                   // lower left corner
-                  if ((i != (nlat - 1)) && (j != 0))
-                    smooth9_sum(((i + 1) * nlon + j - 1), mask, 0.3, array1, avg, divavg);
-                  else if ((i != (nlat - 1)) && gridIsCyclic)
-                    smooth9_sum((i + 1) * nlon - 1 + nlon, mask, 0.3, array1, avg, divavg);
-
+                  if      (i != (nlat - 1) && j != 0)          smooth9_sum(0.3, (i + 1) * nlon + j - 1);
+                  else if (i != (nlat - 1) && gridIsCyclic)    smooth9_sum(0.3, (i + 1) * nlon - 1 + nlon);
                   // lower cell
-                  if (i != (nlat - 1)) smooth9_sum(((i + 1) * nlon) + j, mask, 0.5, array1, avg, divavg);
-
+                  if      (i != (nlat - 1))                    smooth9_sum(0.5, (i + 1) * nlon + j);
                   // lower right corner
-                  if ((i != (nlat - 1)) && (j != (nlon - 1)))
-                    smooth9_sum(((i + 1) * nlon) + j + 1, mask, 0.3, array1, avg, divavg);
-                  else if ((i != (nlat - 1)) && gridIsCyclic)
-                    smooth9_sum(((i + 1) * nlon) + j + 1 - nlon, mask, 0.3, array1, avg, divavg);
+                  if      (i != (nlat - 1) && j != (nlon - 1)) smooth9_sum(0.3, (i + 1) * nlon + j + 1);
+                  else if (i != (nlat - 1) && gridIsCyclic)    smooth9_sum(0.3, (i + 1) * nlon + j + 1 - nlon);
                 }
+              // clang-format on
             }
           else if (mask[j + nlon * i])
             {
               avg += array1[j + nlon * i];
               divavg += 1;
 
-              smooth9_sum(((i - 1) * nlon) + j - 1, mask, 0.3, array1, avg, divavg);
-              smooth9_sum(((i - 1) * nlon) + j, mask, 0.5, array1, avg, divavg);
-              smooth9_sum(((i - 1) * nlon) + j + 1, mask, 0.3, array1, avg, divavg);
-              smooth9_sum(((i) *nlon) + j - 1, mask, 0.5, array1, avg, divavg);
-              smooth9_sum((i * nlon) + j + 1, mask, 0.5, array1, avg, divavg);
-              smooth9_sum(((i + 1) * nlon + j - 1), mask, 0.3, array1, avg, divavg);
-              smooth9_sum(((i + 1) * nlon) + j, mask, 0.5, array1, avg, divavg);
-              smooth9_sum(((i + 1) * nlon) + j + 1, mask, 0.3, array1, avg, divavg);
+              smooth9_sum(0.3, (i - 1) * nlon + j - 1);
+              smooth9_sum(0.5, (i - 1) * nlon + j);
+              smooth9_sum(0.3, (i - 1) * nlon + j + 1);
+              smooth9_sum(0.5, i * nlon + j - 1);
+              smooth9_sum(0.5, i * nlon + j + 1);
+              smooth9_sum(0.3, (i + 1) * nlon + j - 1);
+              smooth9_sum(0.5, (i + 1) * nlon + j);
+              smooth9_sum(0.3, (i + 1) * nlon + j + 1);
             }
 
           if (std::fabs(divavg) > 0) { array2[i * nlon + j] = avg / divavg; }
@@ -244,9 +227,9 @@ smooth9(int gridID, double mv, const Varray<T1> &array1, Varray<T2> &array2)
 }
 
 static void
-smooth9(const Field &field1, Field &field2)
+smooth9(Field const &field1, Field &field2)
 {
-  auto func = [&](const auto &v1, auto &v2) { field2.numMissVals = smooth9(field1.grid, field1.missval, v1, v2); };
+  auto func = [&](auto const &v1, auto &v2) { field2.numMissVals = smooth9(field1.grid, field1.missval, v1, v2); };
   field_operation2(func, field1, field2);
 }
 
@@ -256,47 +239,36 @@ radiusDegToKm(double radiusInDeg)
   return radiusInDeg * (2.0 * PlanetRadiusDefault * M_PI) / (360.0 * 1000.0);
 }
 
-static CurveForm
-convert_curveform(const std::string &formstr)
-{
-  CurveForm form = CurveForm::LINEAR;
-
-  if (formstr == "linear")
-    form = CurveForm::LINEAR;
-  else
-    cdo_abort("form=%s unsupported!", formstr);
-
-  return form;
-}
-
 static void
 get_parameter(int &xnsmooth, SmoothPoint &spoint)
 {
+  auto &knnParams = spoint.knnParams;
   auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      const auto &pargv = cdo_get_oper_argv();
+      auto const &pargv = cdo_get_oper_argv();
 
       KVList kvlist;
       kvlist.name = cdo_module_name();
       if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-          const auto &value = kv.values[0];
+          auto const &value = kv.values[0];
 
           // clang-format off
-          if      (key == "nsmooth")    xnsmooth = parameter_to_int(value);
-          else if (key == "maxpoints")  spoint.maxpoints = parameter_to_size_t(value);
-          else if (key == "weight0")    spoint.weight0 = parameter_to_double(value);
-          else if (key == "weightR")    spoint.weightR = parameter_to_double(value);
-          else if (key == "radius")     spoint.radius = radius_str_to_deg(value);
-          else if (key == "arc_radius") spoint.arc_radius = radius_str_to_deg(value);
-          else if (key == "form")       spoint.form = convert_curveform(value);
+          if      (key == "nsmooth")     xnsmooth = parameter_to_int(value);
+          else if (key == "maxpoints")   spoint.maxpoints = parameter_to_size_t(value);
+          else if (key == "radius")      spoint.radius = radius_str_to_deg(value);
+          else if (key == "arc_radius")  spoint.arc_radius = radius_str_to_deg(value);
+          else if (key == "weighted")    knnParams.weighted = string_to_weightingMethod(parameter_to_word(value));
+          else if (key == "gauss_scale") knnParams.gaussScale = parameter_to_double(value);
+          else if (key == "weight0")     knnParams.weight0 = parameter_to_double(value);
+          else if (key == "weightR")     knnParams.weightR = parameter_to_double(value);
           else cdo_abort("Invalid parameter key >%s<!", key);
           // clang-format on
         }
@@ -304,17 +276,20 @@ get_parameter(int &xnsmooth, SmoothPoint &spoint)
 }
 
 static void
-print_parameter(const SmoothPoint &sp)
+print_parameter(SmoothPoint const &sp)
 {
+  auto const &kp = sp.knnParams;
   std::stringstream outbuffer;
 
   if (sp.arc_radius > 0.0)
-    outbuffer << ", arc_radius=" << sp.arc_radius << "deg(" << radiusDegToKm(sp.arc_radius) << "km)";
+    outbuffer << "arc_radius=" << sp.arc_radius << "deg(" << radiusDegToKm(sp.arc_radius) << "km)";
   else
-    outbuffer << ", radius=" << sp.radius << "deg(" << radiusDegToKm(sp.radius) << "km)";
+    outbuffer << "radius=" << sp.radius << "deg(" << radiusDegToKm(sp.radius) << "km)";
 
-  outbuffer << ", form=" << Form[(int) sp.form];
-  if (sp.form == CurveForm::LINEAR) outbuffer << ", weight0=" << sp.weight0 << ", weightR=" << sp.weightR;
+  outbuffer << ", maxpoints=" << sp.maxpoints;
+  outbuffer << ", weighted=" << weightingMethod_to_string(kp.weighted);
+  if (kp.weighted == WeightingMethod::linear) outbuffer << ", weight0=" << kp.weight0 << ", weightR=" << kp.weightR;
+  if (kp.weighted == WeightingMethod::gaussWeighted) outbuffer << ", gauss_scale=" << kp.gaussScale;
 
   cdo_print("%s", outbuffer.str());
 }
@@ -348,10 +323,10 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  int vlistID2;
+  int vlistID2{ CDI_UNDEFID };
 
   int xnsmooth = 1;
   int operatorID;
@@ -388,7 +363,7 @@ public:
 
     for (int varID = 0; varID < numVars; ++varID)
       {
-        const auto &var = varList1.vars[varID];
+        auto const &var = varList1.vars[varID];
         auto gridID = var.gridID;
         auto gridtype = gridInqType(gridID);
         if (gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT || gridtype == GRID_CURVILINEAR || gridtype == GRID_PROJECTION
@@ -400,8 +375,7 @@ public:
         else { cdo_warning("Unsupported grid for variable %s", var.name); }
       }
 
-    auto gridsizemax = vlistGridsizeMax(vlistID1);
-    if (gridsizemax < spoint.maxpoints) spoint.maxpoints = gridsizemax;
+    if (varList1.gridsizeMax() < spoint.maxpoints) spoint.maxpoints = varList1.gridsizeMax();
     if (Options::cdoVerbose && operatorID == SMOOTH) print_parameter(spoint);
 
     spoint.radius *= DEG2RAD;
@@ -418,20 +392,19 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            const auto &var = varList1.vars[varID];
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            auto const &var = varList1.vars[varID];
             field1.init(var);
             field2.init(var);
-            cdo_read_record(streamID1, field1);
+            cdo_read_field(streamID1, field1);
 
             if (varIDs[varID])
               {
@@ -446,8 +419,8 @@ public:
                   }
               }
 
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, field1);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, field1);
           }
 
         tsID++;
