@@ -40,7 +40,6 @@ public:
   };
   inline static RegisterEntry<Sorttimestamp> registration = RegisterEntry<Sorttimestamp>(module);
 
-  int varID, levelID;
   int lasttsID = -1;
   int nalloc = 0;
   int vlistID2 = -1, taxisID2 = -1;
@@ -101,8 +100,8 @@ public:
         int tsID = 0;
         while (true)
           {
-            auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-            if (nrecs == 0) break;
+            auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+            if (numFields == 0) break;
 
             if (xtsID >= nalloc)
               {
@@ -116,12 +115,12 @@ public:
 
             field2D_init(varsData[xtsID], varList1);
 
-            for (int recID = 0; recID < nrecs; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                cdo_inq_record(streamID1, &varID, &levelID);
+                auto [varID, levelID] = cdo_inq_field(streamID1);
                 auto &field = varsData[xtsID][varID][levelID];
                 field.init(varList1.vars[varID]);
-                cdo_read_record(streamID1, field);
+                cdo_read_field(streamID1, field);
               }
 
             tsID++;
@@ -145,7 +144,7 @@ public:
         timeinfo[tsID].datetime = jdatetime;
       }
 
-    ranges::stable_sort(timeinfo, {}, &TimeInfo::datetime);
+    std::ranges::stable_sort(timeinfo, {}, &TimeInfo::datetime);
 
     vlistDefTaxis(vlistID2, taxisID2);
 
@@ -157,7 +156,7 @@ public:
       {
         xtsID = timeinfo[tsID].index;
 
-        if (tsID > 0 && IS_EQUAL(timeinfo[tsID].datetime, timeinfo[lasttsID].datetime))
+        if (tsID > 0 && is_equal(timeinfo[tsID].datetime, timeinfo[lasttsID].datetime))
           {
             if (skipSameTime)
               {
@@ -170,8 +169,8 @@ public:
               {
                 auto lskip = false;
                 auto xtsID2 = timeinfo[lasttsID].index;
-                const auto &field1 = varsData[xtsID][0][0];
-                const auto &field2 = varsData[xtsID2][0][0];
+                auto const &field1 = varsData[xtsID][0][0];
+                auto const &field2 = varsData[xtsID2][0][0];
                 if (field1.memType == MemType::Float)
                   {
                     if (field1.vec_f == field2.vec_f) lskip = true;
@@ -198,15 +197,15 @@ public:
 
         VarList varList2(vlistID2);
 
-        for (varID = 0; varID < numVars; ++varID)
+        for (int varID = 0; varID < numVars; ++varID)
           {
-            for (levelID = 0; levelID < varList2.vars[varID].nlevels; ++levelID)
+            for (int levelID = 0; levelID < varList2.vars[varID].nlevels; ++levelID)
               {
                 auto &field = varsData[xtsID][varID][levelID];
                 if (field.hasData())
                   {
-                    cdo_def_record(streamID2, varID, levelID);
-                    cdo_write_record(streamID2, field);
+                    cdo_def_field(streamID2, varID, levelID);
+                    cdo_write_field(streamID2, field);
                   }
               }
           }
