@@ -95,8 +95,25 @@ sp2gp_init(SP_Transformation &spTrans, int gridID1, int gridIDsp, int gridIDgp,
   return gridID2;
 }
 
-class ModuleSpectral
+class Spectral : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Spectral",
+    .operators = { { "gp2sp", SpectralHelp },
+                   { "gp2spl", SpectralHelp },
+                   { "sp2gp", SpectralHelp },
+                   { "sp2gpl", SpectralHelp },
+                   { "sp2sp", SpecconvHelp },
+                   { "spcut", SpecconvHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Spectral> registration = RegisterEntry<Spectral>(module);
+  int GP2SP, GP2SPL, SP2GP, SP2GPL, SP2SP, SPCUT;
 
   int gridID1 = -1, gridID2 = -1;
   int defaultTrunc = 0;
@@ -109,42 +126,30 @@ class ModuleSpectral
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  int vlistID1;
+  int vlistID1{ CDI_UNDEFID };
+  VarList varList1;
 
   std::vector<bool> processVars;
-  Varray<double> array1;
-  Varray<double> array2;
 
   bool lgp2sp;
   bool lsp2gp;
   bool linear;
 
-  int GP2SP;
-  int GP2SPL;
-  int SP2GP;
-  int SP2GPL;
-  int SP2SP;
-  int SPCUT;
-
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
     dataIsUnchanged = data_is_unchanged();
 
-    // clang-format off
-    GP2SP  = cdo_operator_add("gp2sp",  0, 0, nullptr);
-    GP2SPL = cdo_operator_add("gp2spl", 0, 0, nullptr);
-    SP2GP  = cdo_operator_add("sp2gp",  0, 0, nullptr);
-    SP2GPL = cdo_operator_add("sp2gpl", 0, 0, nullptr);
-    SP2SP  = cdo_operator_add("sp2sp",  0, 0, nullptr);
-    SPCUT  = cdo_operator_add("spcut",  0, 0, nullptr);
-    // clang-format on
+    GP2SP = module.get_id("gp2sp");
+    GP2SPL = module.get_id("gp2spl");
+    SP2GP = module.get_id("sp2gp");
+    SP2GPL = module.get_id("sp2gpl");
+    SP2SP = module.get_id("sp2sp");
+    SPCUT = module.get_id("spcut");
 
     operatorID = cdo_operator_id();
 
@@ -158,8 +163,8 @@ public:
     auto paramArgc = cdo_operator_argc();
     if ((lgp2sp || lsp2gp) && paramArgc == 1)
       {
-        const auto &parg = cdo_operator_argv(0);
-        auto pos = parg.find("=");
+        auto const &parg = cdo_operator_argv(0);
+        auto pos = parg.find('=');
         if (pos > 0 && parg.substr(0, pos) == "trunc") { defaultTrunc = parameter_to_int(parg.substr(pos + 1)); }
         else
           {
@@ -186,6 +191,8 @@ public:
     vlistID1 = cdo_stream_inq_vlist(streamID1);
     auto vlistID2 = vlistDuplicate(vlistID1);
 
+    varList1 = VarList(vlistID1);
+
     taxisID1 = vlistInqTaxis(vlistID1);
     taxisID2 = taxisDuplicate(taxisID1);
     vlistDefTaxis(vlistID2, taxisID2);
@@ -215,7 +222,7 @@ public:
         operator_input_arg("truncation");
         if (gridID1 != -1)
           {
-            if (!isdigit(cdo_operator_argv(0)[0])) cdo_abort("parameter truncation must comprise only digits [0-9]!");
+            if (!std::isdigit(cdo_operator_argv(0)[0])) cdo_abort("parameter truncation must comprise only digits [0-9]!");
             long ntr = parameter_to_int(cdo_operator_argv(0));
             long nsp = (ntr + 1) * (ntr + 2);
             gridIDsp = gridCreate(GRID_SPECTRAL, nsp);
@@ -234,15 +241,16 @@ public:
         operator_input_arg("wave numbers");
         if (gridID1 != -1)
           {
-            long maxntr = 1 + gridInqTrunc(gridID1);
-            auto wnums = cdo_argv_to_int(cdo_get_oper_argv());
-            long ncut = wnums.size();
-            waves.resize(maxntr);
-            for (long i = 0; i < maxntr; ++i) waves[i] = 1;
+            long maxWaveNumbers = 1 + gridInqTrunc(gridID1);
+            auto waveNumber = cdo_argv_to_intarr(cdo_get_oper_argv());
+            long ncut = waveNumber.size();
+            waves.resize(maxWaveNumbers);
+            for (long i = 0; i < maxWaveNumbers; ++i) waves[i] = 1;
             for (long i = 0; i < ncut; ++i)
               {
-                long j = wnums[i] - 1;
-                if (j < 0 || j >= maxntr) cdo_abort("wave number %ld out of range (min=1, max=%l qd)!", wnums[i], maxntr);
+                long j = waveNumber[i] - 1;
+                if (j < 0 || j >= maxWaveNumbers)
+                  cdo_abort("wave number %ld out of range (min=1, max=%l qd)!", waveNumber[i], maxWaveNumbers);
                 waves[j] = 0;
               }
           }
@@ -252,62 +260,62 @@ public:
         gridID2 = gridIDsp;
       }
 
-    auto nvars = vlistNvars(vlistID2);
-    processVars = std::vector<bool>(nvars);
-    for (int varID = 0; varID < nvars; ++varID) processVars[varID] = (gridID1 == vlistInqVarGrid(vlistID1, varID));
+    auto numVars = varList1.numVars();
+    processVars = std::vector<bool>(numVars);
+    for (auto const &var : varList1.vars) processVars[var.ID] = (gridID1 == var.gridID);
 
     if (gridID1 != -1) vlistChangeGrid(vlistID2, gridID1, gridID2);
 
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
-
-    auto gridsizemax = vlistGridsizeMax(vlistID1);
-    array1 = Varray<double>(gridsizemax);
-    if (gridID2 != -1) array2.resize(gridInqSize(gridID2));
   }
+
   void
-  run()
+  run() override
   {
+    Varray<double> array1(varList1.gridsizeMax());
+    Varray<double> array2;
+    if (gridID2 != -1) array2.resize(gridInqSize(gridID2));
+
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
 
             if (processVars[varID])
               {
-                size_t nmiss;
-                cdo_read_record(streamID1, array1.data(), &nmiss);
-                if (nmiss) cdo_abort("Missing values unsupported for spectral data!");
+                size_t numMissVals;
+                cdo_read_field(streamID1, array1.data(), &numMissVals);
+                if (numMissVals) cdo_abort("Missing values unsupported for spectral data!");
 
-                gridID1 = vlistInqVarGrid(vlistID1, varID);
+                gridID1 = varList1.vars[varID].gridID;
                 // clang-format off
-              if      (lgp2sp) grid2spec(spTrans, gridID1, array1, gridID2, array2);
-              else if (lsp2gp) spec2grid(spTrans, gridID1, array1, gridID2, array2);
-              else if (operatorID == SP2SP) spec2spec(gridID1, array1, gridID2, array2);
-              else if (operatorID == SPCUT) speccut(gridID1, array1, array2, waves);
+                if      (lgp2sp) grid2spec(spTrans, gridID1, array1, gridID2, array2);
+                else if (lsp2gp) spec2grid(spTrans, gridID1, array1, gridID2, array2);
+                else if (operatorID == SP2SP) spec2spec(gridID1, array1, gridID2, array2);
+                else if (operatorID == SPCUT) speccut(gridID1, array1, array2, waves);
                 // clang-format on
 
-                cdo_def_record(streamID2, varID, levelID);
-                cdo_write_record(streamID2, array2.data(), nmiss);
+                cdo_def_field(streamID2, varID, levelID);
+                cdo_write_field(streamID2, array2.data(), numMissVals);
               }
             else
               {
-                cdo_def_record(streamID2, varID, levelID);
-                if (dataIsUnchanged) { cdo_copy_record(streamID2, streamID1); }
+                cdo_def_field(streamID2, varID, levelID);
+                if (dataIsUnchanged) { cdo_copy_field(streamID2, streamID1); }
                 else
                   {
-                    size_t nmiss;
-                    cdo_read_record(streamID1, array1.data(), &nmiss);
-                    cdo_write_record(streamID2, array1.data(), nmiss);
+                    size_t numMissVals;
+                    cdo_read_field(streamID1, array1.data(), &numMissVals);
+                    cdo_write_field(streamID2, array1.data(), numMissVals);
                   }
               }
           }
@@ -315,23 +323,11 @@ public:
         tsID++;
       }
   }
-  void
-  close()
-  {
 
+  void
+  close() override
+  {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Spectral(void *process)
-{
-  ModuleSpectral spectral;
-  spectral.init(process);
-  spectral.run();
-  spectral.close();
-  return nullptr;
-}
