@@ -34,7 +34,7 @@ public:
   inline static RegisterEntry<Splitrec> registration = RegisterEntry<Splitrec>(module);
 
   CdoStreamID streamID1;
-  int vlistID1;
+  int vlistID1{ CDI_UNDEFID };
   std::string fileSuffix;
   Field field;
   VarList varList1;
@@ -63,13 +63,12 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
 
             vlistClearFlag(vlistID1);
             vlistDefFlag(vlistID1, varID, levelID, true);
@@ -83,21 +82,20 @@ public:
 
             if (Options::cdoVerbose) cdo_print("create file %s", fileName);
 
-            auto streamID2 = cdo_open_write(fileName.c_str());
-
+            auto streamID2 = open_write(fileName);
             cdo_def_vlist(streamID2, vlistID2);
 
             auto varID2 = vlistFindVar(vlistID2, varID);
             auto levelID2 = vlistFindLevel(vlistID2, varID, levelID);
 
             cdo_def_timestep(streamID2, 0);
-            cdo_def_record(streamID2, varID2, levelID2);
-            if (dataIsUnchanged) { cdo_copy_record(streamID2, streamID1); }
+            cdo_def_field(streamID2, varID2, levelID2);
+            if (dataIsUnchanged) { cdo_copy_field(streamID2, streamID1); }
             else
               {
                 field.init(varList1.vars[varID]);
-                cdo_read_record(streamID1, field);
-                cdo_write_record(streamID2, field);
+                cdo_read_field(streamID1, field);
+                cdo_write_field(streamID2, field);
               }
 
             cdo_stream_close(streamID2);
