@@ -45,18 +45,39 @@ datetime_to_tm(CdiDateTime vDateTime)
   return stime;
 }
 
-class ModuleSplitTime
+class Splittime : public Process
 {
+  enum
+  {
+    func_time,
+    func_date
+  };
+
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Splittime",
+    .operators = { { "splithour", func_time, 10000, SplittimeHelp },
+                   { "splitday", func_date, 1, SplittimeHelp },
+                   { "splitmon", func_date, 100, SplittimeHelp },
+                   { "splitseas", func_date, 100, SplittimeHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_BOTH,  // Allowed number type
+    .constraints = { 1, OBASE, OnlyFirst },
+  };
+  inline static RegisterEntry<Splittime> registration = RegisterEntry<Splittime>(module);
+
+  int SPLITMON, SPLITSEAS;
   CdoStreamID streamID1;
-  CdoStreamID streamID2;
   CdoStreamID streamIDs[MaxStreams];
   int tsIDs[MaxStreams];
   int index = 0;
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  int vlistID1;
-  int vlistID2;
+  int vlistID1{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
   int operfunc;
   int operintval;
@@ -65,38 +86,20 @@ class ModuleSplitTime
   std::string fileSuffix;
   const char *format = nullptr;
 
-  int SPLITMON;
-  int SPLITSEAS;
-
   VarList varList1;
   FieldVector2D vars;
-  Varray<double> array;
 
   bool haveConstVars;
   bool dataIsUnchanged;
 
-  enum
-  {
-    func_time,
-    func_date
-  };
-
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    if (process_self().m_ID != 0) cdo_abort("This operator can't be combined with other operators!");
-
     dataIsUnchanged = data_is_unchanged();
 
-    // clang-format off
-                cdo_operator_add("splithour", func_time, 10000, nullptr);
-                cdo_operator_add("splitday",  func_date,     1, nullptr);
-    SPLITMON  = cdo_operator_add("splitmon",  func_date,   100, nullptr);
-    SPLITSEAS = cdo_operator_add("splitseas", func_date,   100, nullptr);
-    // clang-format on
+    SPLITMON = module.get_id("splitmon");
+    SPLITSEAS = module.get_id("splitseas");
 
     operatorID = cdo_operator_id();
     operfunc = cdo_operator_f1(operatorID);
@@ -107,8 +110,8 @@ public:
     else
       operator_check_argc(0);
 
-    for (int i = 0; i < MaxStreams; ++i) streamIDs[i] = CDO_STREAM_UNDEF;
-    for (int i = 0; i < MaxStreams; ++i) tsIDs[i] = 0;
+    for (auto &streamID : streamIDs) { streamID = CDO_STREAM_UNDEF; }
+    for (auto &tsID : tsIDs) { tsID = 0; }
 
     streamID1 = cdo_open_read(0);
 
@@ -121,29 +124,19 @@ public:
 
     fileSuffix = FileUtils::gen_suffix(cdo_inq_filetype(streamID1), vlistID1, cdo_get_stream_name(0));
 
-    varList_init(varList1, vlistID1);
-
-    //  if (! dataIsUnchanged)
-    {
-      auto gridsizemax = vlistGridsizeMax(vlistID1);
-      if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
-      array.resize(gridsizemax);
-    }
-
-    haveConstVars = (varList_numConstVars(varList1) > 0);
+    varList1 = VarList(vlistID1);
+    haveConstVars = (varList1.numConstVars() > 0);
 
     if (haveConstVars)
       {
-        int numVars = varList1.size();
+        auto numVars = varList1.numVars();
         vars.resize(numVars);
-
         for (int varID = 0; varID < numVars; ++varID)
           {
-            const auto &var = varList1[varID];
+            auto const &var = varList1.vars[varID];
             if (var.isConstant)
               {
                 vars[varID].resize(var.nlevels);
-
                 for (int levelID = 0; levelID < var.nlevels; ++levelID)
                   {
                     vars[varID][levelID].grid = var.gridID;
@@ -155,15 +148,23 @@ public:
   }
 
   void
-  run()
+  run() override
   {
+    Varray<double> array;
+    //  if (! dataIsUnchanged)
+    {
+      auto gridsizeMax = varList1.gridsizeMax();
+      if (vlistNumber(vlistID1) != CDI_REAL) gridsizeMax *= 2;
+      array.resize(gridsizeMax);
+    }
+
     auto seasonNames = get_season_name();
 
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         auto vDateTime = taxisInqVdatetime(taxisID1);
@@ -175,14 +176,11 @@ public:
 
             if (operatorID == SPLITSEAS) index = month_to_season(index);
           }
-        else if (operfunc == func_time)
-          {
-            index = (cdiTime_get(vDateTime.time) / operintval) % 100;
-          }
+        else if (operfunc == func_time) { index = (cdiTime_get(vDateTime.time) / operintval) % 100; }
 
         if (index < 0 || index >= MaxStreams) cdo_abort("Index out of range!");
 
-        streamID2 = streamIDs[index];
+        auto &streamID2 = streamIDs[index];
         if (streamID2 == CDO_STREAM_UNDEF)
           {
             auto fileName = cdo_get_obase();
@@ -208,53 +206,47 @@ public:
                 if (fileSuffix.size() > 0) fileName += fileSuffix;
               }
 
-            if (Options::cdoVerbose) cdo_print("create file %s", fileName);
-
-            streamID2 = cdo_open_write(fileName.c_str());
+            streamID2 = open_write(fileName);
             cdo_def_vlist(streamID2, vlistID2);
-            streamIDs[index] = streamID2;
           }
 
         cdo_def_timestep(streamID2, tsIDs[index]);
 
         if (tsID > 0 && tsIDs[index] == 0 && haveConstVars)
           {
-            int numVars = varList1.size();
-            for (int varID = 0; varID < numVars; ++varID)
+            for (int varID = 0; varID < varList1.numVars(); ++varID)
               {
-                const auto &var = varList1[varID];
+                auto const &var = varList1.vars[varID];
                 if (var.isConstant)
                   {
                     for (int levelID = 0; levelID < var.nlevels; ++levelID)
                       {
-                        cdo_def_record(streamID2, varID, levelID);
-                        auto nmiss = vars[varID][levelID].nmiss;
-                        cdo_write_record(streamID2, vars[varID][levelID].vec_d.data(), nmiss);
+                        cdo_def_field(streamID2, varID, levelID);
+                        cdo_write_field(streamID2, vars[varID][levelID]);
                       }
                   }
               }
           }
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            cdo_def_record(streamID2, varID, levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            cdo_def_field(streamID2, varID, levelID);
 
-            if (dataIsUnchanged && !(tsID == 0 && haveConstVars)) { cdo_copy_record(streamID2, streamID1); }
+            if (dataIsUnchanged && !(tsID == 0 && haveConstVars)) { cdo_copy_field(streamID2, streamID1); }
             else
               {
-                size_t nmiss;
-                cdo_read_record(streamID1, array.data(), &nmiss);
-                cdo_write_record(streamID2, array.data(), nmiss);
+                size_t numMissVals;
+                cdo_read_field(streamID1, array.data(), &numMissVals);
+                cdo_write_field(streamID2, array.data(), numMissVals);
 
                 if (tsID == 0 && haveConstVars)
                   {
-                    const auto &var = varList1[varID];
+                    auto const &var = varList1.vars[varID];
                     if (var.isConstant)
                       {
                         varray_copy(var.gridsize, array, vars[varID][levelID].vec_d);
-                        vars[varID][levelID].nmiss = nmiss;
+                        vars[varID][levelID].numMissVals = numMissVals;
                       }
                   }
               }
@@ -266,28 +258,15 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID1);
 
-    for (index = 0; index < MaxStreams; ++index)
+    for (auto &streamID : streamIDs)
       {
-        if (streamIDs[index] != CDO_STREAM_UNDEF) cdo_stream_close(streamIDs[index]);
+        if (streamID != CDO_STREAM_UNDEF) cdo_stream_close(streamID);
       }
 
     vlistDestroy(vlistID2);
-
-    cdo_finish();
   }
 };
-
-void *
-Splittime(void *process)
-{
-  ModuleSplitTime splittime;
-  splittime.init(process);
-  splittime.run();
-  splittime.close();
-
-  return nullptr;
-}
