@@ -41,9 +41,9 @@ public:
   CdoStreamID streamID1;
   Varray<int> cyear = Varray<int>(MaxYears, 0);
   std::string fileSuffix;
-  int taxisID1;
-  int taxisID2;
-  int vlistID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
   CdoStreamID streamID2 = CDO_STREAM_UNDEF;
   Varray<double> array;
   VarList varList1;
@@ -75,25 +75,25 @@ public:
 
     fileSuffix = FileUtils::gen_suffix(cdo_inq_filetype(streamID1), vlistID1, cdo_get_stream_name(0));
 
+    varList1 = VarList(vlistID1);
+
     // if (! dataIsUnchanged)
     {
-      auto gridsizemax = vlistGridsizeMax(vlistID1);
-      if (vlistNumber(vlistID1) != CDI_REAL) gridsizemax *= 2;
-      array.resize(gridsizemax);
+      auto gridsizeMax = varList1.gridsizeMax();
+      if (vlistNumber(vlistID1) != CDI_REAL) gridsizeMax *= 2;
+      array.resize(gridsizeMax);
     }
 
-    varList1 = VarList(vlistID1);
-
     haveConstVars = (varList1.numConstVars() > 0);
 
     if (haveConstVars)
       {
-        int numVars = varList1.numVars();
+        auto numVars = varList1.numVars();
         vars.resize(numVars);
 
         for (int varID = 0; varID < numVars; ++varID)
           {
-            const auto &var = varList1.vars[varID];
+            auto const &var = varList1.vars[varID];
             if (var.isConstant)
               {
                 vars[varID].resize(var.nlevels);
@@ -120,8 +120,8 @@ public:
     int tsID2 = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
 
@@ -152,7 +152,7 @@ public:
 
                 if (Options::cdoVerbose) cdo_print("create file %s", fileName);
 
-                streamID2 = cdo_open_write(fileName.c_str());
+                streamID2 = open_write(fileName);
                 cdo_def_vlist(streamID2, vlistID2);
               }
             mon1 = mon2;
@@ -174,7 +174,7 @@ public:
 
                 if (Options::cdoVerbose) cdo_print("create file %s", fileName);
 
-                streamID2 = cdo_open_write(fileName.c_str());
+                streamID2 = open_write(fileName);
                 cdo_def_vlist(streamID2, vlistID2);
               }
           }
@@ -183,37 +183,36 @@ public:
 
         if (tsID > 0 && tsID2 == 0 && haveConstVars)
           {
-            int numVars = varList1.numVars();
+            auto numVars = varList1.numVars();
             for (int varID = 0; varID < numVars; ++varID)
               {
-                const auto &var = varList1.vars[varID];
+                auto const &var = varList1.vars[varID];
                 if (var.isConstant)
                   {
                     for (int levelID = 0; levelID < var.nlevels; ++levelID)
                       {
-                        auto numMissVals = vars[varID][levelID].numMissVals;
-                        cdo_def_record(streamID2, varID, levelID);
-                        cdo_write_record(streamID2, vars[varID][levelID].vec_d.data(), numMissVals);
+                        cdo_def_field(streamID2, varID, levelID);
+                        cdo_write_field(streamID2, vars[varID][levelID]);
                       }
                   }
               }
           }
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
-            cdo_def_record(streamID2, varID, levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            cdo_def_field(streamID2, varID, levelID);
 
-            if (dataIsUnchanged && !(tsID == 0 && haveConstVars)) { cdo_copy_record(streamID2, streamID1); }
+            if (dataIsUnchanged && !(tsID == 0 && haveConstVars)) { cdo_copy_field(streamID2, streamID1); }
             else
               {
                 size_t numMissVals;
-                cdo_read_record(streamID1, array.data(), &numMissVals);
-                cdo_write_record(streamID2, array.data(), numMissVals);
+                cdo_read_field(streamID1, array.data(), &numMissVals);
+                cdo_write_field(streamID2, array.data(), numMissVals);
 
                 if (tsID == 0 && haveConstVars)
                   {
-                    const auto &var = varList1.vars[varID];
+                    auto const &var = varList1.vars[varID];
                     if (var.isConstant)
                       {
                         varray_copy(var.gridsize, array, vars[varID][levelID].vec_d);
