@@ -9,22 +9,33 @@
 
 #include "process_int.h"
 
-class ModuleTee
+class Tee : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Tee",
+    .operators = { { "tee", TeeHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Tee> registration = RegisterEntry<Tee>(module);
+
   Field field;
   CdoStreamID streamID1;
   CdoStreamID streamID2;
   CdiStreamID streamID3;
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
   int taxisID3;
   VarList varList1;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
     operator_check_argc(1);
 
     streamID1 = cdo_open_read(0);
@@ -48,17 +59,17 @@ public:
     cdo_def_vlist(streamID2, vlistID2);
     streamDefVlist(streamID3, vlistID3);
 
-    varList_init(varList1, vlistID1);
+    varList1 = VarList(vlistID1);
   }
 
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_taxis_copy_timestep(taxisID3, taxisID1);
@@ -66,21 +77,20 @@ public:
         cdo_def_timestep(streamID2, tsID);
         streamDefTimestep(streamID3, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            field.init(varList1[varID]);
-            cdo_read_record(streamID1, field);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            field.init(varList1.vars[varID]);
+            cdo_read_field(streamID1, field);
 
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, field);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, field);
 
-            streamDefRecord(streamID3, varID, levelID);
+            streamDefField(streamID3, varID, levelID);
             if (field.memType == MemType::Float)
-              streamWriteRecordF(streamID3, field.vec_f.data(), field.nmiss);
+              streamWriteFieldF(streamID3, field.vec_f.data(), field.numMissVals);
             else
-              streamWriteRecord(streamID3, field.vec_d.data(), field.nmiss);
+              streamWriteField(streamID3, field.vec_d.data(), field.numMissVals);
           }
 
         tsID++;
@@ -88,23 +98,10 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID1);
     cdo_stream_close(streamID2);
     streamClose(streamID3);
-
-    cdo_finish();
   }
 };
-
-void *
-Tee(void *process)
-{
-  ModuleTee tee;
-  tee.init(process);
-  tee.run();
-  tee.close();
-
-  return nullptr;
-}
