@@ -31,17 +31,16 @@ public:
 
   int NORMAL, STUDENTT, CHISQUARE, BETA, FISHER;
   CdoStreamID streamID1;
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
 
   CdoStreamID streamID2;
-  int taxisID2;
-  int vlistID2;
+  int taxisID2{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
   int operatorID;
 
   double degreeOfFreedom = 0, p = 0, q = 0, n = 0, d = 0;
 
-  Varray<double> array1, array2;
   VarList varList1;
 
 public:
@@ -99,30 +98,31 @@ public:
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
 
-    auto gridsizemax = vlistGridsizeMax(vlistID1);
-    array1 = Varray<double>(gridsizemax);
-    array2 = Varray<double>(gridsizemax);
-
     varList1 = VarList(vlistID1);
   }
 
   void
   run() override
   {
+
+    auto gridsizeMax = varList1.gridsizeMax();
+    Varray<double> array1(gridsizeMax);
+    Varray<double> array2(gridsizeMax);
+
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             size_t numMissVals;
-            cdo_read_record(streamID1, array1.data(), &numMissVals);
+            cdo_read_field(streamID1, array1.data(), &numMissVals);
 
             auto gridsize = varList1.vars[varID].gridsize;
             auto missval = varList1.vars[varID].missval;
@@ -130,17 +130,17 @@ public:
             if (operatorID == NORMAL)
               {
                 for (size_t i = 0; i < gridsize; ++i)
-                  array2[i] = dbl_is_equal(array1[i], missval) ? missval : cdo::normal(array1[i]);
+                  array2[i] = fp_is_equal(array1[i], missval) ? missval : cdo::normal(array1[i]);
               }
             else if (operatorID == STUDENTT)
               {
                 for (size_t i = 0; i < gridsize; ++i)
-                  array2[i] = dbl_is_equal(array1[i], missval) ? missval : cdo::student_t(degreeOfFreedom, array1[i]);
+                  array2[i] = fp_is_equal(array1[i], missval) ? missval : cdo::student_t(degreeOfFreedom, array1[i]);
               }
             else if (operatorID == CHISQUARE)
               {
                 for (size_t i = 0; i < gridsize; ++i)
-                  array2[i] = dbl_is_equal(array1[i], missval) ? missval : cdo::chi_square(degreeOfFreedom, array1[i]);
+                  array2[i] = fp_is_equal(array1[i], missval) ? missval : cdo::chi_square(degreeOfFreedom, array1[i]);
               }
             else if (operatorID == BETA)
               {
@@ -148,18 +148,18 @@ public:
                   {
                     if (array1[i] < 0 || array1[i] > 1) cdo_abort("Value out of range (0-1)!");
 
-                    array2[i] = dbl_is_equal(array1[i], missval) ? missval : cdo::beta_distr(p, q, array1[i]);
+                    array2[i] = fp_is_equal(array1[i], missval) ? missval : cdo::beta_distr(p, q, array1[i]);
                   }
               }
             else if (operatorID == FISHER)
               {
                 for (size_t i = 0; i < gridsize; ++i)
-                  array2[i] = dbl_is_equal(array1[i], missval) ? missval : cdo::fisher(n, d, array1[i]);
+                  array2[i] = fp_is_equal(array1[i], missval) ? missval : cdo::fisher(n, d, array1[i]);
               }
             else { cdo_abort("Internal problem, operator not implemented!"); }
 
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, array2.data(), numMissVals);
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, array2.data(), numMissVals);
           }
 
         tsID++;
