@@ -21,7 +21,7 @@
 #include "fill_1d.h"
 
 static double
-julianDate_to_double(int calendar, const CdiDateTime &dateTime1, const CdiDateTime &datetime0)
+julianDate_to_double(int calendar, CdiDateTime const &dateTime1, CdiDateTime const &datetime0)
 {
   return julianDate_to_seconds(julianDate_sub(julianDate_encode(calendar, dateTime1), julianDate_encode(calendar, datetime0)))
          / 86400.0;
@@ -45,11 +45,11 @@ private:
   DateTimeList dtlist;
   CdoStreamID streamID1;
   CdoStreamID streamID2;
-  int vlistID1;
-  int vlistID2;
+  int vlistID1{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   VarList varList1;
   FieldVector3D varsData;
@@ -72,19 +72,19 @@ private:
     auto pargc = cdo_operator_argc();
     if (pargc)
       {
-        const auto &pargv = cdo_get_oper_argv();
+        auto const &pargv = cdo_get_oper_argv();
 
         KVList kvlist;
         kvlist.name = cdo_module_name();
         if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
         if (Options::cdoVerbose) kvlist.print();
 
-        for (const auto &kv : kvlist)
+        for (auto const &kv : kvlist)
           {
-            const auto &key = kv.key;
+            auto const &key = kv.key;
             if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
             if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-            const auto &value = kv.values[0];
+            auto const &value = kv.values[0];
 
             // clang-format off
             if      (key == "method")   method = convert<FillMethod>(value);
@@ -125,7 +125,7 @@ public:
   void
   step(int varID)
   {
-    const auto &var = varList1.vars[varID];
+    auto const &var = varList1.vars[varID];
     auto fieldMemType = var.memType;
     auto gridsize = var.gridsize;
     auto missval = var.missval;
@@ -195,7 +195,7 @@ public:
         timeValues[tsID] = julianDate_to_double(calendar, dtlist.get_vDateTime(tsID), dtlist.get_vDateTime(0));
       }
 
-    dataValues2D = Varray2D<double>(Threading::ompNumThreads);
+    dataValues2D = Varray2D<double>(Threading::ompNumMaxThreads);
     for (auto &dataValues : dataValues2D) dataValues.resize(numSteps);
 
     for (int varID = 0; varID < numVars; ++varID) { step(varID); }
