@@ -57,18 +57,18 @@ public:
 private:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   int noffset;
   int ndates;
   int nskip;
 
-  int maxRecords;
+  int maxFields;
 
   cdo::StepStat2D stepStat;
 
-  std::vector<RecordInfo> recordList;
+  std::vector<FieldInfo> fieldInfoList;
   DateTimeList dtlist;
   VarList varList1;
 
@@ -110,8 +110,8 @@ public:
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
 
-    maxRecords = varList1.numRecords();
-    recordList = std::vector<RecordInfo>(maxRecords);
+    maxFields = varList1.maxFields();
+    fieldInfoList = std::vector<FieldInfo>(maxFields);
 
     dtlist.set_stat(TimeStat::MEAN);
     dtlist.set_calendar(taxisInqCalendar(taxisID1));
@@ -125,14 +125,14 @@ public:
   {
     for (tsID = 0; tsID < noffset; ++tsID)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
 
-            if (tsID == 0) recordList[recID].set(varID, levelID);
+            if (tsID == 0) fieldInfoList[fieldID].set(varID, levelID);
           }
       }
 
@@ -140,12 +140,12 @@ public:
   }
 
   void
-  skip_nskip_steps(int &tsID, int &nrecs)
+  skip_nskip_steps(int &tsID, int &numFields)
   {
     for (int i = 0; i < nskip; ++i)
       {
-        nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
         tsID++;
       }
   }
@@ -184,23 +184,23 @@ public:
               }
           }
 
-        int nrecs{ 0 };
+        int numFields{ 0 };
         int numSets{ 0 };
         for (numSets = numSetsStart; numSets < ndates; numSets++)
           {
-            nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-            if (nrecs == 0) break;
+            numFields = cdo_stream_inq_timestep(streamID1, tsID);
+            if (numFields == 0) break;
 
             if (numSteps > 1) progress.update((tsID + 1.0) / numSteps);
 
             dtlist.taxis_inq_timestep(taxisID1, numSets);
 
-            for (int recID = 0; recID < nrecs; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                auto [varID, levelID] = cdo_inq_record(streamID1);
-                if (tsID == 0) recordList[recID].set(varID, levelID);
+                auto [varID, levelID] = cdo_inq_field(streamID1);
+                if (tsID == 0) fieldInfoList[fieldID].set(varID, levelID);
                 field.init(varList1.vars[varID]);
-                cdo_read_record(streamID1, field);
+                cdo_read_field(streamID1, field);
                 stepStat.add_field(field, varID, levelID, numSets);
                 if (nskip == -1 && numSets == (ndates - 1)) { field_copy(field, lastData[varID][levelID]); }
               }
@@ -208,13 +208,13 @@ public:
             tsID++;
           }
 
-        if (nrecs == 0 && numSets == 0) break;
-        if (nrecs == 0 && numSets == 1 && nskip == -1) break;
+        if (numFields == 0 && numSets == 0) break;
+        if (numFields == 0 && numSets == 1 && nskip == -1) break;
 
-        cdo::records_process(recordList, varList1, stepStat, numSets);
+        cdo::fields_process(fieldInfoList, varList1, stepStat, numSets);
 
         dtlist.stat_taxis_def_timestep(taxisID2, numSets);
-        cdo::write_out_stream(streamID2, recordList, varList1, stepStat, otsID);
+        cdo::write_out_stream(streamID2, fieldInfoList, varList1, stepStat, otsID);
 
         if (numSets < ndates)
           {
@@ -223,12 +223,12 @@ public:
             break;
           }
 
-        if (nrecs == 0) break;
+        if (numFields == 0) break;
         otsID++;
 
-        skip_nskip_steps(tsID, nrecs);
+        skip_nskip_steps(tsID, numFields);
 
-        if (nrecs == 0) break;
+        if (numFields == 0) break;
       }
 
     if (otsID == 0) cdo_abort("No output step found!");
