@@ -15,7 +15,7 @@
 
 #include "process_int.h"
 #include "cdo_options.h"
-#include "cimdOmp.h"
+#include "cdo_omp.h"
 #include "field_functions.h"
 
 class Timsort : public Process
@@ -37,10 +37,10 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
-  int vlistID1;
-  int vlistID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
+  int vlistID1{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
   int numVars;
 
@@ -76,8 +76,8 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         if (tsID >= nalloc)
           {
@@ -91,12 +91,12 @@ public:
 
         field2D_init(varsData[tsID], varList1);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             auto &field = varsData[tsID][varID][levelID];
             field.init(varList1.vars[varID]);
-            cdo_read_record(streamID1, field);
+            cdo_read_field(streamID1, field);
           }
 
         tsID++;
@@ -104,11 +104,11 @@ public:
 
     int nts = tsID;
 
-    std::vector<Field> fields(Threading::ompNumThreads);
+    std::vector<Field> fields(Threading::ompNumMaxThreads);
 
     for (int varID = 0; varID < numVars; ++varID)
       {
-        const auto &var = varList1.vars[varID];
+        auto const &var = varList1.vars[varID];
 
         if (var.isConstant) continue;
 
@@ -129,7 +129,7 @@ public:
                     v.resize(nts);
                     for (int t = 0; t < nts; ++t) v[t] = varsData[t][varID][levelID].vec_f[i];
 
-                    ranges::sort(v);
+                    std::ranges::sort(v);
 
                     for (int t = 0; t < nts; ++t) varsData[t][varID][levelID].vec_f[i] = v[t];
                   }
@@ -139,7 +139,7 @@ public:
                     v.resize(nts);
                     for (int t = 0; t < nts; ++t) v[t] = varsData[t][varID][levelID].vec_d[i];
 
-                    ranges::sort(v);
+                    std::ranges::sort(v);
 
                     for (int t = 0; t < nts; ++t) varsData[t][varID][levelID].vec_d[i] = v[t];
                   }
@@ -154,14 +154,14 @@ public:
 
         for (int varID = 0; varID < numVars; ++varID)
           {
-            const auto &var = varList1.vars[varID];
+            auto const &var = varList1.vars[varID];
             for (int levelID = 0; levelID < var.nlevels; ++levelID)
               {
                 auto &field = varsData[tsID][varID][levelID];
                 if (field.hasData())
                   {
-                    cdo_def_record(streamID2, varID, levelID);
-                    cdo_write_record(streamID2, field);
+                    cdo_def_field(streamID2, varID, levelID);
+                    cdo_write_field(streamID2, field);
                   }
               }
           }
