@@ -66,7 +66,7 @@
 
 #include "cdo_options.h"
 #include "cdo_stepstat.h"
-#include "cdo_task.h"
+#include "workerthread.h"
 #include "process_int.h"
 #include "datetime.h"
 #include "printinfo.h"
@@ -77,7 +77,7 @@
 #include "pmlist.h"
 
 static void
-vlist_set_frequency(int vlistID, const std::string &frequency)
+vlist_set_frequency(int vlistID, std::string const &frequency)
 {
   if (frequency.size()) cdiDefAttTxt(vlistID, CDI_GLOBAL, "frequency", frequency.size(), frequency.c_str());
 }
@@ -98,19 +98,19 @@ get_parameter(double &vfraction, bool &completeOnly)
   auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      const auto &pargv = cdo_get_oper_argv();
+      auto const &pargv = cdo_get_oper_argv();
 
       KVList kvlist;
       kvlist.name = cdo_module_name();
       if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-          const auto &value = kv.values[0];
+          auto const &value = kv.values[0];
 
           // clang-format off
           if      (key == "vfraction")       vfraction = parameter_to_double(value);
@@ -196,14 +196,13 @@ private:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
   CdoStreamID streamID3;
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   int vlistID3;
   int taxisID3{ -1 };
 
   int compareDate;
-  int maxRecords;
 
   bool completeOnly{ false };
   bool handleVfraction{ false };
@@ -211,18 +210,18 @@ private:
 
   cdo::StepStat2D stepStat;
 
-  std::vector<RecordInfo> recordList;
+  std::vector<FieldInfo> fieldInfoList;
   DateTimeList dtlist;
   VarList varList1;
 
   void
   create_diag_stream(int operatorID, int vlistID1, int numVars)
   {
-    char filename[8192];
-    std::strcpy(filename, cdo_operator_name(operatorID));
-    std::strcat(filename, "_");
-    std::strcat(filename, cdo_get_stream_name(1));
-    streamID3 = cdo_open_write(filename);
+    char fileName[8192];
+    std::strcpy(fileName, cdo_operator_name(operatorID));
+    std::strcat(fileName, "_");
+    std::strcat(fileName, cdo_get_stream_name(1));
+    streamID3 = open_write(fileName);
 
     vlistID3 = vlistDuplicate(vlistID1);
 
@@ -304,8 +303,7 @@ public:
 
     if (Options::cdoDiag) create_diag_stream(operatorID, vlistID1, varList1.numVars());
 
-    maxRecords = varList1.numRecords();
-    recordList = std::vector<RecordInfo>(maxRecords);
+    fieldInfoList.resize(varList1.maxFields());
 
     dtlist.set_stat(timestatDate);
     dtlist.set_calendar(taxisInqCalendar(taxisID1));
@@ -336,11 +334,11 @@ public:
     while (true)
       {
         int numSets = 0;
-        int nrecs = 0;
+        int numFields = 0;
         while (true)
           {
-            nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-            if (nrecs == 0) break;
+            numFields = cdo_stream_inq_timestep(streamID1, tsID);
+            if (numFields == 0) break;
 
             if (numSteps1 > 1) progress.update((tsID + 1.0) / numSteps1);
 
@@ -355,12 +353,12 @@ public:
                 break;
               }
 
-            for (int recID = 0; recID < nrecs; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                auto [varID, levelID] = cdo_inq_record(streamID1);
-                if (tsID == 0) recordList[recID].set(varID, levelID);
+                auto [varID, levelID] = cdo_inq_field(streamID1);
+                if (tsID == 0) fieldInfoList[fieldID].set(varID, levelID);
                 field.init(varList1.vars[varID]);
-                cdo_read_record(streamID1, field);
+                cdo_read_field(streamID1, field);
                 stepStat.add_field(field, varID, levelID, numSets);
               }
 
@@ -369,41 +367,41 @@ public:
             tsID++;
           }
 
-        if (nrecs == 0 && numSets == 0) break;
+        if (numFields == 0 && numSets == 0) break;
 
         if (compareDate == CMP_DAY || compareDate == CMP_MONTH || compareDate == CMP_YEAR)
           {
             numSetsList.push_back(numSets);
-            if (nrecs == 0 && check_numSets(numSetsList) && completeOnly) break;
+            if (numFields == 0 && check_numSets(numSetsList) && completeOnly) break;
           }
 
-        cdo::records_process(recordList, varList1, stepStat, numSets);
+        cdo::fields_process(fieldInfoList, varList1, stepStat, numSets);
 
         if (Options::cdoVerbose) cdo_print("%s  numSteps = %d", datetime_to_string(vDateTimeN), numSets);
 
-        if (handleVfraction && stepStat.lmean) cdo::records_set_missval(recordList, varList1, stepStat, numSets, vfraction);
+        if (handleVfraction && stepStat.lmean) cdo::fields_set_missval(fieldInfoList, varList1, stepStat, numSets, vfraction);
 
         dtlist.stat_taxis_def_timestep(taxisID2, numSets);
-        cdo::write_out_stream(streamID2, recordList, varList1, stepStat, otsID);
+        cdo::write_out_stream(streamID2, fieldInfoList, varList1, stepStat, otsID);
 
         if (Options::cdoDiag)
           {
             dtlist.stat_taxis_def_timestep(taxisID3, numSets);
-            cdo::write_diag_stream(streamID3, recordList, varList1, stepStat, otsID, numSets);
+            cdo::write_diag_stream(streamID3, fieldInfoList, varList1, stepStat, otsID, numSets);
           }
 
-        if (nrecs == 0) break;
+        if (numFields == 0) break;
         otsID++;
       }
   }
 
   static void
-  records_add_fields2D(const FieldVector2D &fields2D, const std::vector<RecordInfo> &recordList, const VarList &varList1,
-                       cdo::StepStat2D &stepStat, int numSets) noexcept
+  add_fields2D(const FieldVector2D &fields2D, const std::vector<FieldInfo> &fieldInfoList, const VarList &varList1,
+               cdo::StepStat2D &stepStat, int numSets) noexcept
   {
-    for (const auto &record : recordList)
+    for (auto const &fieldInfo : fieldInfoList)
       {
-        auto [varID, levelID] = record.get();
+        auto [varID, levelID] = fieldInfo.get();
         if (varList1.vars[varID].isConstant) continue;
 
         stepStat.add_field(fields2D[varID][levelID], varID, levelID, numSets);
@@ -424,18 +422,18 @@ public:
     field2D_init(fields3D[1], varList1, FIELD_VEC | FIELD_NAT);
 
     auto useTask = true;
-    auto task = useTask ? std::make_unique<cdo::Task>() : nullptr;
+    auto workerThread = useTask ? std::make_unique<WorkerThread>() : nullptr;
 
     int tsID = 0;
     int otsID = 0;
     while (true)
       {
         int numSets = 0;
-        int nrecs = 0;
+        int numFields = 0;
         while (true)
           {
-            nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-            if (nrecs == 0) break;
+            numFields = cdo_stream_inq_timestep(streamID1, tsID);
+            if (numFields == 0) break;
 
             if (numSteps > 1) progress.update((tsID + 1.0) / numSteps);
 
@@ -450,52 +448,52 @@ public:
                 break;
               }
 
-            for (int recID = 0; recID < nrecs; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                auto [varID, levelID] = cdo_inq_record(streamID1);
-                if (tsID == 0) recordList[recID].set(varID, levelID);
-                cdo_read_record(streamID1, fields3D[numSets % 2][varID][levelID]);
+                auto [varID, levelID] = cdo_inq_field(streamID1);
+                if (tsID == 0) fieldInfoList[fieldID].set(varID, levelID);
+                cdo_read_field(streamID1, fields3D[numSets % 2][varID][levelID]);
               }
 
-            if (useTask && numSets > 0) task->wait();
+            if (useTask && numSets > 0) workerThread->wait();
 
-            std::function<void()> records_add_fields2D_func
-                = std::bind(records_add_fields2D, std::cref(fields3D[numSets % 2]), std::cref(recordList), std::cref(varList1),
+            std::function<void()> add_fields2D_task
+                = std::bind(add_fields2D, std::cref(fields3D[numSets % 2]), std::cref(fieldInfoList), std::cref(varList1),
                             std::ref(stepStat), numSets);
 
-            useTask ? task->doAsync(records_add_fields2D_func) : records_add_fields2D_func();
+            useTask ? workerThread->doAsync(add_fields2D_task) : add_fields2D_task();
 
             vDateTimeN = vDateTime;
             numSets++;
             tsID++;
           }
 
-        if (nrecs == 0 && numSets == 0) break;
+        if (numFields == 0 && numSets == 0) break;
 
-        if (useTask) task->wait();
+        if (useTask) workerThread->wait();
 
         if (compareDate == CMP_DAY || compareDate == CMP_MONTH || compareDate == CMP_YEAR)
           {
             numSetsList.push_back(numSets);
-            if (nrecs == 0 && check_numSets(numSetsList) && completeOnly) break;
+            if (numFields == 0 && check_numSets(numSetsList) && completeOnly) break;
           }
 
-        cdo::records_process(recordList, varList1, stepStat, numSets);
+        cdo::fields_process(fieldInfoList, varList1, stepStat, numSets);
 
         if (Options::cdoVerbose) cdo_print("%s  numSteps = %d", datetime_to_string(vDateTimeN), numSets);
 
-        if (handleVfraction && stepStat.lmean) cdo::records_set_missval(recordList, varList1, stepStat, numSets, vfraction);
+        if (handleVfraction && stepStat.lmean) cdo::fields_set_missval(fieldInfoList, varList1, stepStat, numSets, vfraction);
 
         dtlist.stat_taxis_def_timestep(taxisID2, numSets);
-        cdo::write_out_stream(streamID2, recordList, varList1, stepStat, otsID);
+        cdo::write_out_stream(streamID2, fieldInfoList, varList1, stepStat, otsID);
 
         if (Options::cdoDiag)
           {
             dtlist.stat_taxis_def_timestep(taxisID3, numSets);
-            cdo::write_diag_stream(streamID3, recordList, varList1, stepStat, otsID, numSets);
+            cdo::write_diag_stream(streamID3, fieldInfoList, varList1, stepStat, otsID, numSets);
           }
 
-        if (nrecs == 0) break;
+        if (numFields == 0) break;
         otsID++;
       }
   }
@@ -503,8 +501,7 @@ public:
   void
   run() override
   {
-    auto runAsync = (Options::CDO_Parallel_Read > 0);
-    runAsync ? run_async() : run_sync();
+    Options::CDO_Async_Read ? run_async() : run_sync();
   }
 
   void
