@@ -20,7 +20,6 @@
 #include "arithmetic.h"
 
 constexpr int NIN = 2;
-constexpr int NOUT = 1;
 constexpr int NWORK = 6;
 
 static void
@@ -30,8 +29,8 @@ varquot2test(double rconst, double risk, size_t gridsize, double missval, Varray
   auto missval2 = missval;
 
   auto varquot2test_kernel = [&](auto i, auto is_EQ) {
-    auto temp0 = DIVM(MULM(work[0][i], work[0][i]), work[2][i]);
-    auto temp1 = DIVM(MULM(work[3][i], work[3][i]), work[5][i]);
+    auto temp0 = DIVMX(MULM(work[0][i], work[0][i]), work[2][i]);
+    auto temp1 = DIVMX(MULM(work[3][i], work[3][i]), work[5][i]);
     auto temp2 = SUBM(work[1][i], temp0);
     auto temp3 = SUBM(work[4][i], temp1);
     auto statistic = DIVM(temp2, ADDM(temp2, MULM(rconst, temp3)));
@@ -45,7 +44,7 @@ varquot2test(double rconst, double risk, size_t gridsize, double missval, Varray
   };
 
   if (std::isnan(missval))
-    for (size_t i = 0; i < gridsize; ++i) out[i] = varquot2test_kernel(i, dbl_is_equal);
+    for (size_t i = 0; i < gridsize; ++i) out[i] = varquot2test_kernel(i, fp_is_equal);
   else
     for (size_t i = 0; i < gridsize; ++i) out[i] = varquot2test_kernel(i, is_equal);
 }
@@ -65,23 +64,23 @@ meandiff2test(double rconst, double risk, size_t gridsize, double missval, Varra
   auto meandiff2test_kernel = [&](auto i, auto is_EQ) {
     double temp0 = 0.0;
     double degOfFreedom = -NIN;
-    auto tmp = DIVM(MULM(work[0][i], work[0][i]), work[2][i]);
-    temp0 = ADDM(temp0, DIVM(SUBM(work[1][i], tmp), varFactor[0]));
+    auto tmp = DIVMX(MULM(work[0][i], work[0][i]), work[2][i]);
+    temp0 = ADDM(temp0, DIVMX(SUBM(work[1][i], tmp), varFactor[0]));
     degOfFreedom = ADDM(degOfFreedom, work[2][i]);
-    tmp = DIVM(MULM(work[3][i], work[3][i]), work[5][i]);
-    temp0 = ADDM(temp0, DIVM(SUBM(work[4][i], tmp), varFactor[1]));
+    tmp = DIVMX(MULM(work[3][i], work[3][i]), work[5][i]);
+    temp0 = ADDM(temp0, DIVMX(SUBM(work[4][i], tmp), varFactor[1]));
     degOfFreedom = ADDM(degOfFreedom, work[5][i]);
 
-    if (dbl_is_not_equal(temp0, missval1) && temp0 < 0) temp0 = 0;  // This is possible because of rounding errors
+    if (fp_is_not_equal(temp0, missval1) && temp0 < 0) temp0 = 0;  // This is possible because of rounding errors
 
-    auto stddevEstimator = SQRTM(DIVM(temp0, degOfFreedom));
+    auto stddevEstimator = SQRTM(DIVMX(temp0, degOfFreedom));
     auto meanEstimator = -rconst;
-    meanEstimator = ADDM(meanEstimator, MULM(meanFactor[0], DIVM(work[0][i], work[2][i])));
-    meanEstimator = ADDM(meanEstimator, MULM(meanFactor[1], DIVM(work[3][i], work[5][i])));
+    meanEstimator = ADDM(meanEstimator, MULM(meanFactor[0], DIVMX(work[0][i], work[2][i])));
+    meanEstimator = ADDM(meanEstimator, MULM(meanFactor[1], DIVMX(work[3][i], work[5][i])));
 
     double temp1 = 0.0;
-    temp1 = ADDM(temp1, DIVM(factor1, work[2][i]));
-    temp1 = ADDM(temp1, DIVM(factor2, work[5][i]));
+    temp1 = ADDM(temp1, DIVMX(factor1, work[2][i]));
+    temp1 = ADDM(temp1, DIVMX(factor2, work[5][i]));
     auto norm = SQRTM(temp1);
 
     auto temp2 = DIVM(DIVM(meanEstimator, norm), stddevEstimator);
@@ -92,7 +91,7 @@ meandiff2test(double rconst, double risk, size_t gridsize, double missval, Varra
   };
 
   if (std::isnan(missval))
-    for (size_t i = 0; i < gridsize; ++i) out[i] = meandiff2test_kernel(i, dbl_is_equal);
+    for (size_t i = 0; i < gridsize; ++i) out[i] = meandiff2test_kernel(i, fp_is_equal);
   else
     for (size_t i = 0; i < gridsize; ++i) out[i] = meandiff2test_kernel(i, is_equal);
 }
@@ -116,14 +115,12 @@ public:
 
   CdoStreamID streamID[NIN];
   CdoStreamID streamID3;
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
   int taxisID3;
 
   double rconst;
   double risk;
 
-  size_t gridsizemax;
-
   int operatorID;
 
   VarList varList1;
@@ -149,13 +146,12 @@ public:
     for (int is = 0; is < NIN; ++is) { vlistID[is] = cdo_stream_inq_vlist(streamID[is]); }
 
     varList1 = VarList(vlistID[0]);
+    for (auto &var : varList1.vars) var.memType = MemType::Double;
 
     for (int is = 1; is < NIN; ++is) { varList_compare(varList1, VarList(vlistID[is])); }
 
     auto vlistID3 = vlistDuplicate(vlistID[0]);
 
-    gridsizemax = vlistGridsizeMax(vlistID[0]);
-
     taxisID1 = vlistInqTaxis(vlistID[0]);
     taxisID3 = taxisDuplicate(taxisID1);
 
@@ -171,12 +167,10 @@ public:
     int reachedEOF[NIN]{ 0 };
 
     auto numVars = varList1.numVars();
-    auto maxRecords = varList1.numRecords();
-    auto recordList = std::vector<RecordInfo>(maxRecords);
+    auto maxFields = varList1.maxFields();
+    auto fieldInfoList = std::vector<FieldInfo>(maxFields);
 
-    Field in[NIN], out[NOUT];
-    for (int i = 0; i < NIN; ++i) in[i].resize(gridsizemax);
-    for (int i = 0; i < NOUT; ++i) out[i].resize(gridsizemax);
+    Field inField, outField;
 
     Varray4D<double> work(numVars);
 
@@ -184,11 +178,11 @@ public:
       {
         auto const &var = varList1.vars[varID];
         auto gridsize = var.gridsize;
-        auto nlevels = var.nlevels;
+        auto numLevels = var.nlevels;
 
-        work[varID].resize(nlevels);
+        work[varID].resize(numLevels);
 
-        for (int levelID = 0; levelID < nlevels; ++levelID)
+        for (int levelID = 0; levelID < numLevels; ++levelID)
           {
             work[varID][levelID].resize(NWORK);
             for (int iw = 0; iw < NWORK; ++iw) work[varID][levelID][iw].resize(gridsize, 0);
@@ -203,8 +197,8 @@ public:
           {
             if (reachedEOF[is]) continue;
 
-            auto nrecs = cdo_stream_inq_timestep(streamID[is], tsID);
-            if (nrecs == 0)
+            auto numFields = cdo_stream_inq_timestep(streamID[is], tsID);
+            if (numFields == 0)
               {
                 reachedEOF[is] = 1;
                 continue;
@@ -212,25 +206,21 @@ public:
 
             vDateTime = taxisInqVdatetime(taxisID1);
 
-            auto &inField = in[is];
-
-            for (int recID = 0; recID < nrecs; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                auto [varID, levelID] = cdo_inq_record(streamID[is]);
+                auto [varID, levelID] = cdo_inq_field(streamID[is]);
+
+                if (tsID == 0 && is == 0) fieldInfoList[fieldID].set(varID, levelID);
 
                 auto const &var = varList1.vars[varID];
-
-                auto gridsize = var.gridsize;
-                inField.missval = var.missval;
-
-                if (tsID == 0 && is == 0) recordList[recID].set(varID, levelID);
-
-                cdo_read_record(streamID[is], inField.vec_d.data(), &inField.numMissVals);
+                inField.init(var);
+                cdo_read_field(streamID[is], inField);
 
                 auto const &inArray = inField.vec_d;
                 auto &rwork1 = work[varID][levelID][3 * is + 0];
                 auto &rwork2 = work[varID][levelID][3 * is + 1];
                 auto &rwork3 = work[varID][levelID][3 * is + 2];
+                auto gridsize = var.gridsize;
                 for (size_t i = 0; i < gridsize; ++i)
                   {
                     rwork1[i] += inArray[i];
@@ -241,7 +231,7 @@ public:
           }
 
         for (is = 0; is < NIN; ++is)
-          if (!reachedEOF[is]) break;
+          if (not reachedEOF[is]) break;
 
         if (is == NIN) break;
 
@@ -251,20 +241,20 @@ public:
     taxisDefVdatetime(taxisID3, vDateTime);
     cdo_def_timestep(streamID3, 0);
 
-    for (int recID = 0; recID < maxRecords; ++recID)
+    for (int fieldID = 0; fieldID < maxFields; ++fieldID)
       {
-        auto [varID, levelID] = recordList[recID].get();
+        auto [varID, levelID] = fieldInfoList[fieldID].get();
 
         auto const &var = varList1.vars[varID];
-
+        outField.init(var);
         auto const &rwork = work[varID][levelID];
 
-        if (operatorID == VARQUOT2TEST) { varquot2test(rconst, risk, var.gridsize, var.missval, rwork, out[0].vec_d); }
-        else if (operatorID == MEANDIFF2TEST) { meandiff2test(rconst, risk, var.gridsize, var.missval, rwork, out[0].vec_d); }
+        if (operatorID == VARQUOT2TEST) { varquot2test(rconst, risk, var.gridsize, var.missval, rwork, outField.vec_d); }
+        else if (operatorID == MEANDIFF2TEST) { meandiff2test(rconst, risk, var.gridsize, var.missval, rwork, outField.vec_d); }
 
-        out[0].missval = var.missval;
-        cdo_def_record(streamID3, varID, levelID);
-        cdo_write_record(streamID3, out[0].vec_d.data(), field_num_miss(out[0]));
+        field_num_mv(outField);
+        cdo_def_field(streamID3, varID, levelID);
+        cdo_write_field(streamID3, outField);
       }
   }
 
