@@ -17,7 +17,7 @@
 #include "process_int.h"
 #include "cdo_vlist.h"
 #include "cdo_options.h"
-#include "cdo_task.h"
+#include "workerthread.h"
 #include "field_trend.h"
 #include "cdo_omp.h"
 #include "datetime.h"
@@ -33,19 +33,19 @@ get_parameter(bool &tstepIsEqual)
   auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      const auto &pargv = cdo_get_oper_argv();
+      auto const &pargv = cdo_get_oper_argv();
 
       KVList kvlist;
       kvlist.name = cdo_module_name();
       if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-          const auto &value = kv.values[0];
+          auto const &value = kv.values[0];
 
           // clang-format off
           if (key == "equal") tstepIsEqual = parameter_to_bool(value);
@@ -75,8 +75,8 @@ public:
   CdoStreamID streamID2;
   CdoStreamID streamID3;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   int maxFields;
 
@@ -123,7 +123,7 @@ public:
   }
 
   void
-  write_output(const FieldVector3D &work)
+  write_output(FieldVector3D const &work)
   {
     Field field2, field3;
 
@@ -134,7 +134,7 @@ public:
       {
         auto [varID, levelID] = fieldInfoList[fieldID].get();
 
-        const auto &var = varList2.vars[varID];
+        auto const &var = varList2.vars[varID];
         field2.init(var);
         field3.init(var);
 
@@ -198,10 +198,10 @@ public:
   }
 
   static void
-  fields_calc_trend_sum(FieldVector3D &work, const FieldVector2D &fields2D, const std::vector<FieldInfo> &fieldInfoList,
+  fields_calc_trend_sum(FieldVector3D &work, FieldVector2D const &fields2D, std::vector<FieldInfo> const &fieldInfoList,
                         double zj) noexcept
   {
-    for (const auto &fieldInfo : fieldInfoList)
+    for (auto const &fieldInfo : fieldInfoList)
       {
         auto [varID, levelID] = fieldInfo.get();
         calc_trend_sum(work, fields2D[varID][levelID], zj, varID, levelID);
@@ -227,7 +227,7 @@ public:
     field2D_init(fields3D[1], varList1, FIELD_VEC | FIELD_NAT);
 
     bool useTask = true;
-    auto task = useTask ? std::make_unique<cdo::Task>() : nullptr;
+    auto workerThread = useTask ? std::make_unique<WorkerThread>() : nullptr;
 
     int tsID = 0;
     while (true)
@@ -250,18 +250,18 @@ public:
             cdo_read_field(streamID1, fields3D[tsID % 2][varID][levelID]);
           }
 
-        if (useTask && tsID > 0) task->wait();
+        if (useTask && tsID > 0) workerThread->wait();
 
-        std::function<void()> fields_calc_trend_sum_func
+        std::function<void()> fields_calc_trend_sum_task
             = std::bind(fields_calc_trend_sum, std::ref(work), std::ref(fields3D[tsID % 2]), std::cref(fieldInfoList), zj);
 
-        if (useTask) { task->doAsync(fields_calc_trend_sum_func); }
-        else { fields_calc_trend_sum_func(); }
+        if (useTask) { workerThread->doAsync(fields_calc_trend_sum_task); }
+        else { fields_calc_trend_sum_task(); }
 
         tsID++;
       }
 
-    if (useTask) task->wait();
+    if (useTask) workerThread->wait();
 
     taxisDefVdatetime(taxisID2, vDateTime);
     write_output(work);
@@ -270,7 +270,7 @@ public:
   void
   run() override
   {
-    auto runAsync = (Options::CDO_Parallel_Read > 0);
+    auto runAsync = (Options::CDO_Async_Read > 0);
     runAsync ? run_async() : run_sync();
   }
 
