@@ -16,6 +16,7 @@
 
 #include "process_int.h"
 #include "cdo_vlist.h"
+#include "cdo_omp.h"
 #include "datetime.h"
 #include "pmlist.h"
 #include "param_conversion.h"
@@ -25,71 +26,79 @@
 
 template <typename T>
 static void
-add_trend(double zj, Varray<T> &v1, const Varray<double> &v2, const Varray<double> &v3, size_t n, double mv)
+add_trend(double zj, Varray<T> &v1, Varray<double> const &v2, Varray<double> const &v3, size_t n, double mv)
 {
   auto missval1 = mv;
   auto missval2 = mv;
 
-  auto add_kernel = [&](auto i, auto is_EQ) { return ADDM(v1[i], ADDM(v2[i], MULM(v3[i], zj))); };
+  auto add_kernel = [&](auto is_EQ) {
+#ifdef _OPENMP
+#pragma omp parallel for if (n > cdoMinLoopSize) default(shared) schedule(static)
+#endif
+    for (size_t i = 0; i < n; ++i)
+      {
+        auto tmp = (is_EQ(v2[i], mv) || is_EQ(v3[i], mv)) ? mv : (v2[i] + v3[i] * zj);
+        v1[i] = ADDM(v1[i], tmp);
+      }
+  };
 
-  if (std::isnan(missval1))
-    for (size_t i = 0; i < n; ++i) v1[i] = add_kernel(i, dbl_is_equal);
-  else
-    for (size_t i = 0; i < n; ++i) v1[i] = add_kernel(i, is_equal);
+  std::isnan(mv) ? add_kernel(fp_is_equal) : add_kernel(is_equal);
 }
 
 static void
-add_trend(double zj, Field &field1, const Field &field2, const Field &field3)
+add_trend(double zj, Field &field1, Field const &field2, Field const &field3)
 {
-  if (field1.memType == MemType::Float)
-    add_trend(zj, field1.vec_f, field2.vec_d, field3.vec_d, field1.size, field1.missval);
-  else
-    add_trend(zj, field1.vec_d, field2.vec_d, field3.vec_d, field1.size, field1.missval);
+  auto func = [&](auto &v1) { add_trend(zj, v1, field2.vec_d, field3.vec_d, field1.size, field1.missval); };
+  field_operation(func, field1);
 }
 
 template <typename T>
 static void
-sub_trend(double zj, Varray<T> &v1, const Varray<double> &v2, const Varray<double> &v3, size_t n, double mv)
+sub_trend(double zj, Varray<T> &v1, Varray<double> const &v2, Varray<double> const &v3, size_t n, double mv)
 {
   auto missval1 = mv;
   auto missval2 = mv;
 
-  auto sub_kernel = [&](auto i, auto is_EQ) { return SUBM(v1[i], ADDM(v2[i], MULM(v3[i], zj))); };
+  auto sub_kernel = [&](auto is_EQ) {
+#ifdef _OPENMP
+#pragma omp parallel for if (n > cdoMinLoopSize) default(shared) schedule(static)
+#endif
+    for (size_t i = 0; i < n; ++i)
+      {
+        auto tmp = (is_EQ(v2[i], mv) || is_EQ(v3[i], mv)) ? mv : (v2[i] + v3[i] * zj);
+        v1[i] = SUBM(v1[i], tmp);
+      }
+  };
 
-  if (std::isnan(missval1))
-    for (size_t i = 0; i < n; ++i) v1[i] = sub_kernel(i, dbl_is_equal);
-  else
-    for (size_t i = 0; i < n; ++i) v1[i] = sub_kernel(i, is_equal);
+  std::isnan(mv) ? sub_kernel(fp_is_equal) : sub_kernel(is_equal);
 }
 
 static void
-sub_trend(double zj, Field &field1, const Field &field2, const Field &field3)
+sub_trend(double zj, Field &field1, Field const &field2, Field const &field3)
 {
-  if (field1.memType == MemType::Float)
-    sub_trend(zj, field1.vec_f, field2.vec_d, field3.vec_d, field1.size, field1.missval);
-  else
-    sub_trend(zj, field1.vec_d, field2.vec_d, field3.vec_d, field1.size, field1.missval);
+  auto func = [&](auto &v1) { sub_trend(zj, v1, field2.vec_d, field3.vec_d, field1.size, field1.missval); };
+  field_operation(func, field1);
 }
 
 static void
-trendarithGetParameter(bool &tstepIsEqual)
+get_parameter(bool &tstepIsEqual)
 {
   auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      const auto &pargv = cdo_get_oper_argv();
+      auto const &pargv = cdo_get_oper_argv();
 
       KVList kvlist;
       kvlist.name = cdo_module_name();
       if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-          const auto &value = kv.values[0];
+          auto const &value = kv.values[0];
 
           // clang-format off
           if (key == "equal") tstepIsEqual = parameter_to_bool(value);
@@ -121,7 +130,7 @@ public:
   CdoStreamID streamID3;
   CdoStreamID streamID4;
 
-  int taxisID1;
+  int taxisID1{ CDI_UNDEFID };
   int taxisID4;
 
   int operfunc;
@@ -129,7 +138,6 @@ public:
   bool tstepIsEqual;
 
   VarList varList1;
-  FieldVector2D vars2, vars3;
 
 public:
   void
@@ -139,7 +147,7 @@ public:
     operfunc = cdo_operator_f1(operatorID);
 
     tstepIsEqual = true;
-    trendarithGetParameter(tstepIsEqual);
+    get_parameter(tstepIsEqual);
 
     streamID1 = cdo_open_read(0);
     streamID2 = cdo_open_read(1);
@@ -165,45 +173,46 @@ public:
 
     streamID4 = cdo_open_write(3);
     cdo_def_vlist(streamID4, vlistID4);
-
-    field2D_init(vars2, varList1, FIELD_VEC);
-    field2D_init(vars3, varList1, FIELD_VEC);
-
-    {
-      auto nrecs = cdo_stream_inq_timestep(streamID2, 0);
-      for (int recID = 0; recID < nrecs; ++recID)
-        {
-          auto [varID, levelID] = cdo_inq_record(streamID2);
-          cdo_read_record(streamID2, vars2[varID][levelID]);
-        }
-    }
-
-    {
-      auto nrecs = cdo_stream_inq_timestep(streamID3, 0);
-      for (int recID = 0; recID < nrecs; ++recID)
-        {
-          auto [varID, levelID] = cdo_inq_record(streamID3);
-          cdo_read_record(streamID3, vars3[varID][levelID]);
-        }
-    }
   }
 
   void
   run() override
   {
+    FieldVector2D vars2, vars3;
+    field2D_init(vars2, varList1, FIELD_VEC);
+    field2D_init(vars3, varList1, FIELD_VEC);
+
+    {
+      auto numFields = cdo_stream_inq_timestep(streamID2, 0);
+      while (numFields--)
+        {
+          auto [varID, levelID] = cdo_inq_field(streamID2);
+          cdo_read_field(streamID2, vars2[varID][levelID]);
+        }
+    }
+
+    {
+      auto numFields = cdo_stream_inq_timestep(streamID3, 0);
+      while (numFields--)
+        {
+          auto [varID, levelID] = cdo_inq_field(streamID3);
+          cdo_read_field(streamID3, vars3[varID][levelID]);
+        }
+    }
+
     auto calendar = taxisInqCalendar(taxisID1);
     CheckTimeIncr checkTimeIncr;
     JulianDate julianDate0;
     double deltat1 = 0.0;
     auto numSteps = varList1.numSteps();
-    cdo::Progress progress;
+    cdo::Progress progress(get_id());
     Field field;
 
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         auto vDateTime = taxisInqVdatetime(taxisID1);
 
@@ -213,23 +222,23 @@ public:
         cdo_taxis_copy_timestep(taxisID4, taxisID1);
         cdo_def_timestep(streamID4, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto fstatus = (tsID + (recID + 1.0) / nrecs) / numSteps;
+            auto fstatus = (tsID + (fieldID + 1.0) / numFields) / numSteps;
             if (numSteps > 0) progress.update(fstatus);
 
-            auto [varID, levelID] = cdo_inq_record(streamID1);
-            const auto &var1 = varList1.vars[varID];
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            auto const &var1 = varList1.vars[varID];
             field.init(var1);
-            cdo_read_record(streamID1, field);
+            cdo_read_field(streamID1, field);
 
             if (operfunc == FieldFunc_Add)
               add_trend(zj, field, vars2[varID][levelID], vars3[varID][levelID]);
             else
               sub_trend(zj, field, vars2[varID][levelID], vars3[varID][levelID]);
 
-            cdo_def_record(streamID4, varID, levelID);
-            cdo_write_record(streamID4, field);
+            cdo_def_field(streamID4, varID, levelID);
+            cdo_write_field(streamID4, field);
           }
 
         tsID++;
