@@ -16,21 +16,21 @@
 #include "process_int.h"
 #include "param_conversion.h"
 #include "cdo_options.h"
-#include "cimdOmp.h"
+#include "cdo_omp.h"
 #include "field_functions.h"
 
 template <typename T>
 static T
-tstepcount(long nts, T missval, const Varray<T> &v, T refval)
+tstepcount(long nts, T missval, Varray<T> const &v, T refval)
 {
-  if (dbl_is_equal(refval, missval)) return missval;
+  if (fp_is_equal(refval, missval)) return missval;
 
   long j;
   long n = 0;
   for (j = 0; j < nts; ++j)
     {
       n++;
-      if (dbl_is_equal(v[j], refval)) break;
+      if (fp_is_equal(v[j], refval)) break;
     }
 
   return (j == nts) ? missval : (T) n;
@@ -55,9 +55,9 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int vlistID1;
-  int taxisID1;
-  int taxisID2;
+  int vlistID1{ CDI_UNDEFID };
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   int numVars;
   double refval;
@@ -97,8 +97,8 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         constexpr size_t NALLOC_INC = 1024;
         if ((size_t) tsID >= varsData.size()) varsData.resize(varsData.size() + NALLOC_INC);
@@ -107,12 +107,12 @@ public:
 
         field2D_init(varsData[tsID], varList1);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             auto &field = varsData[tsID][varID][levelID];
             field.init(varList1.vars[varID]);
-            cdo_read_record(streamID1, field);
+            cdo_read_field(streamID1, field);
           }
 
         tsID++;
@@ -120,11 +120,11 @@ public:
 
     int nts = tsID;
 
-    std::vector<Field> fields(Threading::ompNumThreads);
+    std::vector<Field> fields(Threading::ompNumMaxThreads);
 
     for (int varID = 0; varID < numVars; ++varID)
       {
-        const auto &var1 = varList1.vars[varID];
+        auto const &var1 = varList1.vars[varID];
         auto memType = var1.memType;
         auto missval = var1.missval;
         auto gridsize = var1.gridsize;
@@ -168,10 +168,10 @@ public:
       {
         for (int levelID = 0; levelID < varList1.vars[varID].nlevels; ++levelID)
           {
-            cdo_def_record(streamID2, varID, levelID);
+            cdo_def_field(streamID2, varID, levelID);
             auto &field1 = varsData[0][varID][levelID];
             field_num_mv(field1);
-            cdo_write_record(streamID2, field1);
+            cdo_write_field(streamID2, field1);
           }
       }
   }
