@@ -104,7 +104,7 @@ calc_index_jj(size_t ny, double yval)
 }
 
 static void
-remap_nn_reg2d_to_reg2d(size_t nx, size_t ny, const Varray<float> &data, int gridID, Varray<float> &array)
+remap_nn_reg2d_to_reg2d(size_t nx, size_t ny, Varray<float> const &data, int gridID, Varray<float> &array)
 {
   auto gridtype = gridInqType(gridID);
   if (gridtype != GRID_LONLAT && gridtype != GRID_GAUSSIAN) cdo_abort("Internal error, wrong grid type!");
@@ -135,7 +135,7 @@ remap_nn_reg2d_to_reg2d(size_t nx, size_t ny, const Varray<float> &data, int gri
 }
 
 static void
-remap_nn_reg2d_to_nonreg2d(size_t nx, size_t ny, const Varray<float> &data, int gridID, Varray<float> &array)
+remap_nn_reg2d_to_nonreg2d(size_t nx, size_t ny, Varray<float> const &data, int gridID, Varray<float> &array)
 {
   auto gridsize = gridInqSize(gridID);
   Varray<double> xvals(gridsize), yvals(gridsize);
@@ -164,13 +164,11 @@ remap_nn_reg2d_to_nonreg2d(size_t nx, size_t ny, const Varray<float> &data, int
 }
 
 static void
-remap_nn_reg2d_to_healpix(size_t nx, size_t ny, const Varray<float> &data, int gridID, Varray<float> &array)
+remap_nn_reg2d_to_healpix(size_t nx, size_t ny, Varray<float> const &data, int gridID, Varray<float> &array)
 {
   auto gridID2 = gridID;
   auto gridsize = gridInqSize(gridID2);
-  auto [r_nside, r_order] = cdo::get_healpix_params(gridID2);
-  int nside = r_nside;
-  HpOrder order = r_order;
+  auto hpParams = cdo::get_healpix_params(gridID2);
 
 #ifdef _OPENMP
 #pragma omp parallel for default(shared) schedule(static)
@@ -178,7 +176,7 @@ remap_nn_reg2d_to_healpix(size_t nx, size_t ny, const Varray<float> &data, int g
   for (size_t i = 0; i < gridsize; ++i)
     {
       double xval, yval;
-      hp_index_to_lonlat(order, nside, i, &xval, &yval);
+      hp_index_to_lonlat(hpParams, i, &xval, &yval);
       auto jj = calc_index_jj(ny, yval * RAD2DEG);
       auto ii = calc_index_ii(nx, xval * RAD2DEG);
       array[i] = data[jj * nx + ii];
@@ -186,7 +184,7 @@ remap_nn_reg2d_to_healpix(size_t nx, size_t ny, const Varray<float> &data, int g
 }
 
 static void
-remap_nn_reg2d(size_t nx, size_t ny, const Varray<float> &data, int gridID, Varray<float> &array)
+remap_nn_reg2d(size_t nx, size_t ny, Varray<float> const &data, int gridID, Varray<float> &array)
 {
   auto gridtype = gridInqType(gridID);
   if (gridtype == GRID_LONLAT || gridtype == GRID_GAUSSIAN)
@@ -282,7 +280,7 @@ private:
   double rstart = 0.0, rstop = 0.0, rinc = 0.0;
   double rconst = 0.0;
   std::vector<double> levels;
-  int ntimesteps;
+  int numSteps;
 
   int taxisID;
   CdoStreamID streamID;
@@ -299,7 +297,7 @@ public:
     RANDOM = module.get_id("random");
     SINCOS = module.get_id("sincos");
     COSHILL = module.get_id("coshill");
-    // not usesd todo: make unavailable for non developers
+    // not used todo: make unavailable for non developers
     TESTFIELD = module.get_id("testfield");
     CONST = module.get_id("const");
     SEQ = module.get_id("seq");
@@ -350,14 +348,14 @@ public:
         rstart = parameter_to_double(cdo_operator_argv(0));
         rstop = parameter_to_double(cdo_operator_argv(1));
         rinc = (cdo_operator_argc() == 3) ? parameter_to_double(cdo_operator_argv(2)) : 1;
-        if (DBL_IS_EQUAL(rinc, 0.0)) cdo_abort("Increment is zero!");
+        if (fp_is_equal(rinc, 0.0)) cdo_abort("Increment is zero!");
 
         gridID = define_point_grid();
       }
     else if (operatorID == STDATM)
       {
         operator_input_arg(cdo_operator_enter(operatorID));
-        levels = cdo_argv_to_flt(cdo_get_oper_argv());
+        levels = cdo_argv_to_fltarr(cdo_get_oper_argv());
         nlevels = levels.size();
 
         if (Options::cdoVerbose)
@@ -402,7 +400,7 @@ public:
 
     cdo_def_vlist(streamID, vlistID);
 
-    ntimesteps = (operatorID == SEQ) ? 1.001 + ((rstop - rstart) / rinc) : 1;
+    numSteps = (operatorID == SEQ) ? 1.001 + ((rstop - rstart) / rinc) : 1;
     if (operatorID != SEQ) vlistDefNtsteps(vlistID, 0);
   }
 
@@ -416,7 +414,7 @@ public:
     size_t gridsize = gridInqSize(gridID);
     Varray<float> array(gridsize);
 
-    for (int tsID = 0; tsID < ntimesteps; ++tsID)
+    for (int tsID = 0; tsID < numSteps; ++tsID)
       {
         auto rval = rstart + rinc * tsID;
         CdiDateTime vDateTime{};
@@ -429,7 +427,7 @@ public:
           {
             for (int levelID = 0; levelID < varList.vars[varID].nlevels; ++levelID)
               {
-                cdo_def_record(streamID, varID, levelID);
+                cdo_def_field(streamID, varID, levelID);
 
                 if (operatorID == RANDOM) { cdo::fill_random(array); }
                 else if (operatorID == SINCOS || operatorID == COSHILL || operatorID == TESTFIELD)
@@ -445,7 +443,7 @@ public:
                     else if (operatorID == TESTFIELD) cdo::fill_testfield(array, xvals, yvals);
                     // clang-format on
                   }
-                else if (operatorID == CONST) { ranges::fill(array, rconst); }
+                else if (operatorID == CONST) { std::ranges::fill(array, rconst); }
                 else if (operatorID == TOPO || operatorID == TEMP || operatorID == MASK)
                   {
                     Varray<float> data;
@@ -466,7 +464,7 @@ public:
                         = (varID == varID2) ? cdo::std_atm_temperatur(levels[levelID]) : cdo::std_atm_pressure(levels[levelID]);
                   }
 
-                cdo_write_record_f(streamID, array.data(), 0);
+                cdo_write_field_f(streamID, array.data(), 0);
               }
           }
       }
