@@ -279,7 +279,7 @@ are_polygon_vertices_arranged_in_clockwise_order(const Varray<Point> &cellCorner
 }
 
 static void
-create_sorted_point_array(size_t gridsize, const Varray<double> &grid_center_lon, const Varray<double> &grid_center_lat,
+create_sorted_point_array(size_t gridsize, Varray<double> const &grid_center_lon, Varray<double> const &grid_center_lat,
                           Varray<double> &center_point_array)
 {
   for (size_t cell_no = 0; cell_no < gridsize; cell_no++)
@@ -317,9 +317,9 @@ create_sorted_point_array(size_t gridsize, const Varray<double> &grid_center_lon
 }
 
 static void
-print_header(int gridtype, size_t gridsize, size_t nx, int gridno, int ngrids)
+print_header(int gridtype, size_t gridsize, size_t nx, int gridno, int numGrids)
 {
-  if (ngrids == 1)
+  if (numGrids == 1)
     {
       if (nx)
         cdo_print(Blue("Grid consists of %zu (%zux%zu) cells (type: %s), of which"), gridsize, nx, gridsize / nx,
@@ -330,17 +330,17 @@ print_header(int gridtype, size_t gridsize, size_t nx, int gridno, int ngrids)
   else
     {
       if (nx)
-        cdo_print(Blue("Grid no %u (of %u) consists of %zu (%zux%zu) cells (type: %s), of which"), gridno + 1, ngrids, gridsize, nx,
-                  gridsize / nx, gridNamePtr(gridtype));
+        cdo_print(Blue("Grid no %u (of %u) consists of %zu (%zux%zu) cells (type: %s), of which"), gridno + 1, numGrids, gridsize,
+                  nx, gridsize / nx, gridNamePtr(gridtype));
       else
-        cdo_print(Blue("Grid no %u (of %u) consists of %zu cells (type: %s), of which"), gridno + 1, ngrids, gridsize,
+        cdo_print(Blue("Grid no %u (of %u) consists of %zu cells (type: %s), of which"), gridno + 1, numGrids, gridsize,
                   gridNamePtr(gridtype));
     }
 }
 
 static size_t
-get_no_unique_center_points(size_t gridsize, size_t nx, const Varray<double> &grid_center_lon,
-                            const Varray<double> &grid_center_lat)
+get_no_unique_center_points(size_t gridsize, size_t nx, Varray<double> const &grid_center_lon,
+                            Varray<double> const &grid_center_lat)
 {
   (void) nx;
   // For performing the first test, an array of all center point coordinates is built.
@@ -503,8 +503,8 @@ compute_cell_barycenter(size_t numCorners, const Varray<Point3D> &cellCorners3D,
 }
 
 static void
-verify_grid(size_t gridsize, size_t nx, int ncorner, const Varray<double> &grid_center_lon, const Varray<double> &grid_center_lat,
-            const Varray<double> &grid_corner_lon, const Varray<double> &grid_corner_lat)
+verify_grid(size_t gridsize, size_t nx, int ncorner, Varray<double> const &grid_center_lon, Varray<double> const &grid_center_lat,
+            Varray<double> const &grid_corner_lon, Varray<double> const &grid_corner_lat)
 {
   /*
      First, this function performs the following test:
@@ -552,7 +552,7 @@ verify_grid(size_t gridsize, size_t nx, int ncorner, const Varray<double> &grid_
     {
       // Conversion of center point spherical coordinates to Cartesian coordinates.
       double centerCoordinates[3];
-      gcLLtoXYZdeg(grid_center_lon[cell_no], grid_center_lat[cell_no], centerCoordinates);
+      gcLLtoXYZ(deg_to_rad(grid_center_lon[cell_no]), deg_to_rad(grid_center_lat[cell_no]), centerCoordinates);
       Point3D centerPoint3D;
       centerPoint3D.X = centerCoordinates[0];
       centerPoint3D.Y = centerCoordinates[1];
@@ -562,7 +562,8 @@ verify_grid(size_t gridsize, size_t nx, int ncorner, const Varray<double> &grid_
       for (int k = 0; k < ncorner; ++k)
         {
           // Conversion of corner spherical coordinates to Cartesian coordinates.
-          gcLLtoXYZdeg(grid_corner_lon[cell_no * ncorner + k], grid_corner_lat[cell_no * ncorner + k], cornerCoordinates);
+          auto index = cell_no * ncorner + k;
+          gcLLtoXYZ(deg_to_rad(grid_corner_lon[index]), deg_to_rad(grid_corner_lat[index]), cornerCoordinates);
 
           // The components of the result vector are appended to the list of cell corner coordinates.
           cellCorners3D_openCell[k].X = cornerCoordinates[0];
@@ -599,7 +600,7 @@ verify_grid(size_t gridsize, size_t nx, int ncorner, const Varray<double> &grid_
       // Checks if there are any duplicate vertices in the list of corners. Note that the last (additional) corner has not been set
       // yet.
 
-      auto no_duplicates = get_no_duplicates(actualNumberOfCorners, cellCorners3D_openCell, markedDuplicateIndices);
+      auto noDuplicates = get_no_duplicates(actualNumberOfCorners, cellCorners3D_openCell, markedDuplicateIndices);
 
       if (Options::cdoVerbose)
         {
@@ -619,13 +620,13 @@ verify_grid(size_t gridsize, size_t nx, int ncorner, const Varray<double> &grid_
 
       copy_unique_corners(actualNumberOfCorners, cellCorners3D_openCell, markedDuplicateIndices, cellCorners3D);
 
-      actualNumberOfCorners -= no_duplicates;
+      actualNumberOfCorners -= noDuplicates;
 
       // We are creating a closed polygon/cell by setting the additional last corner to be the same as the first one.
 
       cellCorners3D[actualNumberOfCorners] = cellCorners3D[0];
 
-      if (no_duplicates != 0)
+      if (noDuplicates != 0)
         {
           no_of_cells_with_duplicates++;
           cells_with_duplicates.push_back(cell_no);
@@ -748,7 +749,7 @@ verify_grid(size_t gridsize, size_t nx, int ncorner, const Varray<double> &grid_
 }
 
 static void
-print_lonlat(int dig, int gridID, size_t gridsize, const Varray<double> &lon, const Varray<double> &lat)
+print_lonlat(int dig, int gridID, size_t gridsize, Varray<double> const &lon, Varray<double> const &lat)
 {
   size_t num_lons_nan = 0;
   for (size_t i = 0; i < gridsize; ++i)
@@ -771,33 +772,41 @@ print_lonlat(int dig, int gridID, size_t gridsize, const Varray<double> &lon, co
   if (ymm.min < -90.00001 || ymm.max > 90.00001) cdo_warning("Grid cell center latitudes out of range!");
 }
 
-class ModuleVerifygrid
+class Verifygrid : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Verifygrid",
+    .operators = { { "verifygrid", VerifygridHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 0, NoRestriction },
+  };
+  inline static RegisterEntry<Verifygrid> registration = RegisterEntry<Verifygrid>(module);
 
   CdoStreamID streamID;
   int vlistID;
 
-  int ngrids;
+  int numGrids;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    cdo_operator_add("verifygrid", 0, 0, nullptr);
 
     operator_check_argc(0);
 
     streamID = cdo_open_read(0);
     vlistID = cdo_stream_inq_vlist(streamID);
 
-    ngrids = vlistNgrids(vlistID);
+    numGrids = vlistNumGrids(vlistID);
   }
   void
-  run()
+  run() override
   {
-    for (int gridno = 0; gridno < ngrids; ++gridno)
+    for (int gridno = 0; gridno < numGrids; ++gridno)
       {
         auto lgeo = true;
 
@@ -844,7 +853,7 @@ public:
             cdo_grid_to_degree(gridID, CDI_XAXIS, grid_center_lon, "grid center lon");
             cdo_grid_to_degree(gridID, CDI_YAXIS, grid_center_lat, "grid center lat");
 
-            print_header(gridtype, gridsize, nx, gridno, ngrids);
+            print_header(gridtype, gridsize, nx, gridno, numGrids);
 
             if (gridHasBounds(gridID))
               {
@@ -867,32 +876,19 @@ public:
           }
         else
           {
-            if (ngrids == 1)
+            if (numGrids == 1)
               cdo_print(Blue("Grid consists of %zu points (type: %s)"), gridsize,
                         isHealpixGrid ? "healpix" : gridNamePtr(gridtype));
             else
-              cdo_print(Blue("Grid no %u (of %u) consists of %zu points (type: %s)"), gridno + 1, ngrids, gridsize,
+              cdo_print(Blue("Grid no %u (of %u) consists of %zu points (type: %s)"), gridno + 1, numGrids, gridsize,
                         gridNamePtr(gridtype));
             // cdo_print("");
           }
       }
   }
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID);
-
-    cdo_finish();
   }
 };
-
-void *
-Verifygrid(void *process)
-{
-
-  ModuleVerifygrid verifygrid;
-  verifygrid.init(process);
-  verifygrid.run();
-  verifygrid.close();
-  return nullptr;
-}
