@@ -10,7 +10,7 @@
 #endif
 
 #ifdef HAVE_LIBNETCDF
-#include "netcdf.h"
+#include <netcdf.h>
 #endif
 
 #include "cdo_options.h"
@@ -18,6 +18,7 @@
 #include "griddes.h"
 #include <mpim_grid.h>
 #include "remap_vars.h"
+#include "knndata.h"
 
 #ifdef HAVE_LIBNETCDF
 static void
@@ -155,12 +156,11 @@ struct RemapGridW
 static void
 remapGridWAlloc(RemapMethod mapType, RemapGridW &grid)
 {
-  if (mapType == RemapMethod::CONSERV_SCRIP || mapType == RemapMethod::CONSERV) { grid.cell_area.resize(grid.ncells, 0.0); }
+  if (mapType == RemapMethod::CONSERV) { grid.cell_area.resize(grid.ncells, 0.0); }
 }
 
 struct RemapVarsW
 {
-  bool sort_add = false;
   RemapMethod mapType{ RemapMethod::UNDEF };  // identifier for remapping method
   NormOpt normOpt{ NormOpt::NONE };           // option for normalization (conserv only)
   size_t numLinks = 0;                        // number of links for remapping
@@ -174,15 +174,13 @@ struct RemapVarsW
 static void
 remapVarsWInit(RemapMethod mapType, int remapOrder, RemapVarsW &rv)
 {
-  rv.sort_add = (mapType == RemapMethod::CONSERV_SCRIP);
-
   // Determine the number of weights
-  rv.numWeights = (mapType == RemapMethod::CONSERV_SCRIP) ? 3 : ((mapType == RemapMethod::BICUBIC) ? 4 : 1);
+  rv.numWeights = (mapType == RemapMethod::BICUBIC) ? 4 : 1;
   if (mapType == RemapMethod::CONSERV && remapOrder == 2) rv.numWeights = 3;
 }
 
 static NormOpt
-remapGetNormOpt(const std::string &normOptStr)
+remapGetNormOpt(std::string const &normOptStr)
 {
   NormOpt normOpt(NormOpt::NONE);
   // clang-format off
@@ -202,9 +200,10 @@ remapGetNormOpt(const std::string &normOptStr)
 }
 
 RemapSwitches get_maptype(int ncfileid);
+KnnParams read_knn_params(int ncfileid);
 
 static void
-read_remapgrid_scrip(int ncfileid, const std::string &prefix, bool lgridarea, RemapGridW &grid)
+read_remapgrid_scrip(int ncfileid, std::string const &prefix, bool lgridarea, RemapGridW &grid)
 {
   // Read all variables of the grid
   cdf_read_var_size(ncfileid, (prefix + "_dims").c_str(), 2, grid.dims);
@@ -236,7 +235,7 @@ read_remapweights_scrip(int ncfileid, RemapVarsW &rv)
 }
 
 static RemapSwitches
-readRemapFileScrip(const std::string &remapFile, RemapAttributes &remapAtts, RemapGridW &srcGrid, RemapGridW &tgtGrid,
+readRemapFileScrip(std::string const &remapFile, RemapAttributes &remapAtts, RemapGridW &srcGrid, RemapGridW &tgtGrid,
                    RemapVarsW &rv)
 {
   // The routine reads a NetCDF file to extract remapping info in SCRIP format
@@ -257,12 +256,12 @@ readRemapFileScrip(const std::string &remapFile, RemapAttributes &remapAtts, Rem
 
   // Map Type
   auto remapSwitches = get_maptype(ncfileid);
-  auto lgridarea = (remapSwitches.mapType == RemapMethod::CONSERV || remapSwitches.mapType == RemapMethod::CONSERV_SCRIP);
+  if (remapSwitches.mapType == RemapMethod::KNN && remapSwitches.numNeighbors == -1) auto knnParams = read_knn_params(ncfileid);
+  auto lgridarea = (remapSwitches.mapType == RemapMethod::CONSERV);
 
   remapVarsWInit(remapSwitches.mapType, remapSwitches.remapOrder, rv);
 
   rv.mapType = remapSwitches.mapType;
-  rv.sort_add = false;
 
   // Normalization option
   std::string normalizeOptStr;  // character string for normalization option
@@ -342,7 +341,7 @@ cdf_def_var(int ncfileid, const char *name, nc_type xtype, int ndims, const int
 }
 
 static void
-cdfWriteAttText(int ncfileid, int ncvarid, const char *name, const std::string &text)
+cdfWriteAttText(int ncfileid, int ncvarid, const char *name, std::string const &text)
 {
   if (text.size()) nce(nc_put_att_text(ncfileid, ncvarid, name, text.size(), text.c_str()));
 }
@@ -431,7 +430,7 @@ struct CDFgrid
 };
 
 static CDFgrid
-define_remapgrid_scrip(int ncfileid, const std::string &prefix, nc_type sizetype, bool lgridarea, const RemapGridW &grid)
+define_remapgrid_scrip(int ncfileid, std::string const &prefix, nc_type sizetype, bool lgridarea, const RemapGridW &grid)
 {
   CDFgrid cdfGrid;
 
@@ -514,16 +513,12 @@ remap_set_mapmethod(const RemapSwitches &remapSwitches)
   std::string mapMethod;
   switch (remapSwitches.mapType)
     {
-    case RemapMethod::CONSERV_SCRIP:
-      mapMethod = submapLAF ? "Largest area fraction" : "Conservative remapping";
-      break;
     case RemapMethod::CONSERV:
-      // mapMethod = submapLAF ? "Largest area fraction" : "Conservative remapping using clipping on sphere";
-      mapMethod = "Conservative remapping using clipping on sphere";
+      mapMethod = submapLAF ? "Largest area fraction" : "Conservative remapping";
       break;
     case RemapMethod::BILINEAR: mapMethod = "Bilinear remapping"; break;
     case RemapMethod::BICUBIC:  mapMethod = "Bicubic remapping"; break;
-    case RemapMethod::DISTWGT:
+    case RemapMethod::KNN:
       mapMethod = (remapSwitches.numNeighbors == 1) ? "Nearest neighbor" : "Distance weighted avg of nearest neighbors";
       break;
     default:
@@ -535,7 +530,7 @@ remap_set_mapmethod(const RemapSwitches &remapSwitches)
 }
 
 static void
-write_remapfile_scrip(const std::string &remapFile, const RemapAttributes &remapAtts, const RemapSwitches &remapSwitches,
+write_remapfile_scrip(std::string const &remapFile, const RemapAttributes &remapAtts, const RemapSwitches &remapSwitches,
                       RemapGridW &srcGrid, RemapGridW &tgtGrid, RemapVarsW &rv)
 {
   // Writes remap data to a NetCDF file using SCRIP conventions
@@ -544,7 +539,7 @@ write_remapfile_scrip(const std::string &remapFile, const RemapAttributes &remap
 
   int nc_dims2_id[2];  // NetCDF ids for 2d array dims
 
-  auto lgridarea = (remapSwitches.mapType == RemapMethod::CONSERV_SCRIP || remapSwitches.mapType == RemapMethod::CONSERV);
+  auto lgridarea = (remapSwitches.mapType == RemapMethod::CONSERV);
 
   // if ( rv.numLinks == 0 ) cdo_abort("Number of remap links is 0, no remap weights found!");
 
@@ -577,7 +572,7 @@ write_remapfile_scrip(const std::string &remapFile, const RemapAttributes &remap
   cdfWriteAttText(ncfileid, NC_GLOBAL, "map_method", mapMethod);
 
   // Remap order
-  if (remapSwitches.mapType == RemapMethod::CONSERV_SCRIP && remapSwitches.submapType == SubmapType::NONE)
+  if (remapSwitches.mapType == RemapMethod::CONSERV && remapSwitches.submapType == SubmapType::NONE)
     nce(nc_put_att_int(ncfileid, NC_GLOBAL, "remap_order", NC_INT, 1L, &remapSwitches.remapOrder));
 
   // File convention
@@ -632,8 +627,8 @@ write_remapfile_scrip(const std::string &remapFile, const RemapAttributes &remap
 }
 
 static void
-check_areas(size_t n_a, const Varray<double> &area_a, const Varray<double> &area_b, size_t n_s, const Varray<size_t> &col,
-            const Varray<size_t> &row, const Varray<double> &S)
+check_areas(size_t n_a, Varray<double> const &area_a, Varray<double> const &area_b, size_t n_s, const Varray<size_t> &col,
+            const Varray<size_t> &row, Varray<double> const &S)
 {
   if (area_a.size() == 0 || area_b.size() == 0) return;
 
@@ -656,7 +651,7 @@ check_areas(size_t n_a, const Varray<double> &area_a, const Varray<double> &area
 }
 
 static void
-verify_weights(const std::string &remapFile)
+verify_weights(std::string const &remapFile)
 {
   RemapGridW srcGrid, tgtGrid;
   RemapVarsW remapVars;
@@ -669,7 +664,7 @@ verify_weights(const std::string &remapFile)
 }
 
 static void
-write_remap_scrip(const std::string &remapFileIn, const std::string &remapFileOut)
+write_remap_scrip(std::string const &remapFileIn, std::string const &remapFileOut)
 {
   RemapGridW srcGrid, tgtGrid;
   RemapVarsW remapVars;
@@ -683,8 +678,8 @@ write_remap_scrip(const std::string &remapFileIn, const std::string &remapFileOu
   if (date_and_time_in_sec != -1)
     {
       struct tm *date_and_time = localtime(&date_and_time_in_sec);
-      (void) strftime(history, 1024, "%d %b %Y : ", date_and_time);
-      strcat(history, command_line());
+      (void) str::strftime(history, 1024, "%d %b %Y : ", date_and_time);
+      std::strcat(history, cdo::command_line());
     }
     */
   if (Options::VersionInfo) remapAtts.cdo_version = cdo_comment();
@@ -706,10 +701,6 @@ public:
   };
   inline static RegisterEntry<Verifyweights> registration = RegisterEntry<Verifyweights>(module);
 
-  int VERIFYWEIGHTS, WRITEREMAPSCRIP;
-  int operatorID;
-  std::string remapFileIn;
-
 public:
   void
   init() override
@@ -717,23 +708,17 @@ public:
 #ifndef HAVE_LIBNETCDF
     cdo_abort("NetCDF support not compiled in!");
 #else
-    VERIFYWEIGHTS = module.get_id("verifyweights");
-    WRITEREMAPSCRIP = module.get_id("writeremapscrip");
+    auto VERIFYWEIGHTS = module.get_id("verifyweights");
+    auto WRITEREMAPSCRIP = module.get_id("writeremapscrip");
 
-    operatorID = cdo_operator_id();
+    auto operatorID = cdo_operator_id();
 
     operator_input_arg(cdo_operator_enter(operatorID));
     auto nfiles = cdo_operator_f2(operatorID);
     operator_check_argc(nfiles);
 
-    remapFileIn = cdo_operator_argv(0);
-#endif
-  }
+    auto remapFileIn = cdo_operator_argv(0);
 
-  void
-  run() override
-  {
-#ifdef HAVE_LIBNETCDF
     if (operatorID == VERIFYWEIGHTS)
       verify_weights(remapFileIn);
     else if (operatorID == WRITEREMAPSCRIP)
@@ -741,6 +726,11 @@ public:
 #endif
   }
 
+  void
+  run() override
+  {
+  }
+
   void
   close() override
   {
