@@ -23,7 +23,7 @@
 #include "vertint_util.h"
 
 static void
-check_range_ps(int stepNum, const Field &psProg)
+check_range_ps(int stepNum, Field const &psProg)
 {
   auto mm = field_min_max(psProg);
   if (mm.min < MIN_PS || mm.max > MAX_PS)
@@ -46,7 +46,7 @@ is_height_axis(int zaxisID)
 
 template <typename T>
 static void
-calc_half_press(size_t gridsize, int numFullLevels, const Varray<T> &fullPress, int numHalfLevels, Varray<T> &halfPress)
+calc_half_press(size_t gridsize, int numFullLevels, Varray<T> const &fullPress, int numHalfLevels, Varray<T> &halfPress)
 {
   for (size_t i = 0; i < gridsize; ++i) halfPress[i] = 0;
 #ifdef _OPENMP
@@ -104,8 +104,8 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   size_t gridsize;
 
@@ -226,8 +226,8 @@ public:
     numFullLevels = (-1 == zaxisID_FL) ? 0 : varList1.vars[airPressID_FL].nlevels;
     numHalfLevels = (-1 == zaxisID_HL) ? numFullLevels + 1 : varList1.vars[airPressID_HL].nlevels;
 
-    auto nzaxis = vlistNumZaxis(vlistID1);
-    for (int index = 0; index < nzaxis; ++index)
+    auto numZaxes = vlistNumZaxis(vlistID1);
+    for (int index = 0; index < numZaxes; ++index)
       {
         auto zaxisID = vlistZaxis(vlistID1, index);
         auto nlevels = zaxisInqSize(zaxisID);
@@ -275,7 +275,7 @@ public:
 
     for (int varID = 0; varID < numVars; ++varID)
       {
-        const auto &var = varList1.vars[varID];
+        auto const &var = varList1.vars[varID];
         auto isHeightAxis = is_height_axis(var.zaxisID);
 
         if (gridInqType(var.gridID) == GRID_SPECTRAL) cdo_abort("Spectral data unsupported!");
@@ -325,23 +325,23 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         for (int varID = 0; varID < numVars; ++varID)
           {
             processVars[varID] = false;
-            const auto &var = varList1.vars[varID];
+            auto const &var = varList1.vars[varID];
             for (int levelID = 0; levelID < var.nlevels; ++levelID) varnumMissVals[varID][levelID] = 0;
           }
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
-            cdo_read_record(streamID1, vardata1[varID], levelID, &varnumMissVals[varID][levelID]);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            cdo_read_field(streamID1, vardata1[varID], levelID, &varnumMissVals[varID][levelID]);
             processVars[varID] = true;
           }
 
@@ -391,7 +391,7 @@ public:
           {
             if (processVars[varID])
               {
-                const auto &var = varList1.vars[varID];
+                auto const &var = varList1.vars[varID];
 
                 if (tsID > 0 && !interpVars[varID] && var.isConstant) continue;
 
@@ -405,21 +405,21 @@ public:
                         if (varnumMissVals[varID][levelID]) cdo_abort("Missing values unsupported for this operator!");
                       }
 
-                    const auto &levels3D = (var.nlevels == numFullLevels) ? fullPress : halfPress;
-                    const auto &vertIndex3D = (var.nlevels == numFullLevels) ? vertIndex_FL : vertIndex_HL;
+                    auto const &levels3D = (var.nlevels == numFullLevels) ? fullPress : halfPress;
+                    auto const &vertIndex3D = (var.nlevels == numFullLevels) ? vertIndex_FL : vertIndex_HL;
                     vertical_interp_X(levels3D, vardata1[varID], vardata2[varID], vertIndex3D, levels, gridsize);
 
                     if (!extrapolate)
                       {
-                        const auto &numMiss = (var.nlevels == numFullLevels) ? numMiss_FL : numMiss_HL;
+                        auto const &numMiss = (var.nlevels == numFullLevels) ? numMiss_FL : numMiss_HL;
                         varray_copy(numPL, numMiss, varnumMissVals[varID]);
                       }
                   }
 
                 for (int levelID = 0; levelID < varList2.vars[varID].nlevels; ++levelID)
                   {
-                    cdo_def_record(streamID2, varID, levelID);
-                    cdo_write_record(streamID2, interpVars[varID] ? vardata2[varID] : vardata1[varID], levelID,
+                    cdo_def_field(streamID2, varID, levelID);
+                    cdo_write_field(streamID2, interpVars[varID] ? vardata2[varID] : vardata1[varID], levelID,
                                      varnumMissVals[varID][levelID]);
                   }
               }
