@@ -26,7 +26,7 @@
 
 /*
 static void
-field_copy_div_2d_to_3d(MemType memType, size_t gridsize, int nlevels, const Field &field2d, Field3D &field3d)
+field_copy_div_2d_to_3d(MemType memType, size_t gridsize, int nlevels, Field const &field2d, Field3D &field3d)
 {
   if (memType == MemType::Float)
     for (size_t i = 0; i < gridsize; ++i) field3d.vec_f[gridsize * nlevels + i] = field2d.vec_f[i] / PlanetGrav;
@@ -36,7 +36,7 @@ field_copy_div_2d_to_3d(MemType memType, size_t gridsize, int nlevels, const Fie
 */
 template <typename T>
 static void
-field_check_sgeopot(const Varray<T> &sgeopot, const T *gheightAtSurface)
+field_check_sgeopot(Varray<T> const &sgeopot, const T *gheightAtSurface)
 {
   auto len = sgeopot.size();
   double sumDiff = 0;
@@ -48,7 +48,7 @@ field_check_sgeopot(const Varray<T> &sgeopot, const T *gheightAtSurface)
 }
 
 static void
-field_check_sgeopot(MemType memType, size_t gridsize, int nlevels, const Field &field2d, const Field3D &field3d)
+field_check_sgeopot(MemType memType, size_t gridsize, int nlevels, Field const &field2d, const Field3D &field3d)
 {
   if (memType == MemType::Float)
     field_check_sgeopot(field2d.vec_f, &field3d.vec_f[gridsize * (nlevels - 1)]);
@@ -57,7 +57,7 @@ field_check_sgeopot(MemType memType, size_t gridsize, int nlevels, const Field &
 }
 
 static void
-vct_to_hybrid_pressure(MemType memType, Field3D &pressure_FL, Field3D &pressure_HL, const Varray<double> &vct, const Field &ps,
+vct_to_hybrid_pressure(MemType memType, Field3D &pressure_FL, Field3D &pressure_HL, Varray<double> const &vct, Field const &ps,
                        long numHybridLevels, long ngp)
 {
   if (memType == MemType::Float)
@@ -80,22 +80,22 @@ invert_vct(Varray<double> &vct)
 
 template <class ForwardIt>
 static void
-check_range(double rmin, double rmax, ForwardIt first, ForwardIt last, const std::string &name)
+check_range(double rmin, double rmax, ForwardIt first, ForwardIt last, std::string const &name)
 {
   const auto [min, max] = std::minmax_element(first, last);
-  if (fabs(*max - *min) <= 1.0e-9 || (*min < rmin) || (*max > rmax))
+  if (std::fabs(*max - *min) <= 1.0e-9 || (*min < rmin) || (*max > rmax))
     cdo_warning("%s out of range (%g - %g) min=%g max=%g", name, rmin, rmax, *min, *max);
 }
 
 static void
-check_vct(const Varray<double> &vct, int numHalfLevels)
+check_vct(Varray<double> const &vct, int numHalfLevels)
 {
   check_range(0.0, 50000.0, vct.begin(), vct.begin() + numHalfLevels, "vct A");
   check_range(0.0, 1.0, vct.begin() + numHalfLevels, vct.end(), "vct B");
 }
 
 static void
-check_range_ps(int stepNum, const Field &psProg)
+check_range_ps(int stepNum, Field const &psProg)
 {
   auto mm = field_min_max(psProg);
   if (mm.min < MIN_PS || mm.max > MAX_PS)
@@ -103,7 +103,7 @@ check_range_ps(int stepNum, const Field &psProg)
 }
 
 static void
-check_range_sgeopot(int stepNum, const Field &sgeopot)
+check_range_sgeopot(int stepNum, Field const &sgeopot)
 {
   auto mm = field_min_max(sgeopot);
   if (mm.min < MIN_FIS || mm.max > MAX_FIS)
@@ -296,7 +296,7 @@ pressure_level_interpolation(Varray<double> &pressureLevels, bool useHeightLevel
     {
       if (interpVars[varID])
         {
-          const auto &var1 = varList1.vars[varID];
+          auto const &var1 = varList1.vars[varID];
           if (var1.nlevels == numHalfLevels && varID != varIDs.gheightID) needVertIndexHalf = true;
         }
     }
@@ -370,7 +370,7 @@ pressure_level_interpolation(Varray<double> &pressureLevels, bool useHeightLevel
       for (int fieldID = 0; fieldID < numFields; ++fieldID)
         {
           auto [varID, levelID] = cdo_inq_field(streamID1);
-          const auto &var = varList1.vars[varID];
+          auto const &var = varList1.vars[varID];
 
           if (vctIsInverted && zaxisID_ML != -1 && var.zaxisID == zaxisID_ML) levelID = var.nlevels - 1 - levelID;
 
@@ -416,7 +416,7 @@ pressure_level_interpolation(Varray<double> &pressureLevels, bool useHeightLevel
         {
           if (processVars[varID])
             {
-              const auto &var = varList1.vars[varID];
+              auto const &var = varList1.vars[varID];
 
               if (tsID > 0 && var.isConstant) continue;
 
@@ -447,8 +447,8 @@ pressure_level_interpolation(Varray<double> &pressureLevels, bool useHeightLevel
                     }
                   else
                     {
-                      const auto &levels3D = (var.nlevels == numFullLevels) ? pressure_FL : pressure_HL;
-                      const auto &vertIndex3D = (var.nlevels == numFullLevels) ? vertIndex_FL : vertIndex_HL;
+                      auto const &levels3D = (var.nlevels == numFullLevels) ? pressure_FL : pressure_HL;
+                      auto const &vertIndex3D = (var.nlevels == numFullLevels) ? vertIndex_FL : vertIndex_HL;
                       vertical_interp_X(levels3D, vardata1[varID], vardata2[varID], vertIndex3D, pressureLevels, gridsize);
                     }
 
@@ -459,7 +459,7 @@ pressure_level_interpolation(Varray<double> &pressureLevels, bool useHeightLevel
                 {
                   cdo_def_field(streamID2, varID, levelID);
                   cdo_write_field(streamID2, interpVars[varID] ? vardata2[varID] : vardata1[varID], levelID,
-                                   varnumMissVals[varID][levelID]);
+                                  varnumMissVals[varID][levelID]);
                 }
             }
         }
@@ -555,7 +555,7 @@ height_level_interpolation(Varray<double> &heightLevels, bool extrapolate)
 
   for (int varID = 0; varID < numVars; ++varID)
     {
-      const auto &var1 = varList1.vars[varID];
+      auto const &var1 = varList1.vars[varID];
       auto nlevels = var1.nlevels;
 
       if (var1.gridType == GRID_SPECTRAL && zaxis_is_hybrid(var1.zaxisType)) cdo_abort("Spectral data on model level unsupported!");
@@ -644,7 +644,7 @@ height_level_interpolation(Varray<double> &heightLevels, bool extrapolate)
         {
           if (processVars[varID])
             {
-              const auto &var = varList1.vars[varID];
+              auto const &var = varList1.vars[varID];
               if (tsID > 0 && var.isConstant) continue;
 
               if (interpVars[varID])
@@ -657,7 +657,7 @@ height_level_interpolation(Varray<double> &heightLevels, bool extrapolate)
                       if (varnumMissVals[varID][levelID]) cdo_abort("Missing values unsupported for this operator!");
                     }
 
-                  const auto &levels3D = vardata1[varIDs.gheightID];
+                  auto const &levels3D = vardata1[varIDs.gheightID];
                   vertical_interp_X(levels3D, vardata1[varID], vardata2[varID], vertIndex, heightLevels, gridsize);
 
                   if (!extrapolate) varray_copy(numHeightLevels, pnumMissVals, varnumMissVals[varID]);
@@ -667,7 +667,7 @@ height_level_interpolation(Varray<double> &heightLevels, bool extrapolate)
                 {
                   cdo_def_field(streamID2, varID, levelID);
                   cdo_write_field(streamID2, interpVars[varID] ? vardata2[varID] : vardata1[varID], levelID,
-                                   varnumMissVals[varID][levelID]);
+                                  varnumMissVals[varID][levelID]);
                 }
             }
         }
