@@ -17,9 +17,7 @@
 #include "process_int.h"
 #include "cdo_vlist.h"
 #include "field_vinterp.h"
-#include "stdnametable.h"
 #include "util_string.h"
-#include "const.h"
 #include "cdo_zaxis.h"
 #include "param_conversion.h"
 
@@ -52,10 +50,7 @@ create_zaxis_depth(Varray<double> &depthLevels)
       else
         cdo_abort("Open failed on %s", zfilename);
     }
-  else
-    {
-      depthLevels = cdo_argv_to_flt(cdo_get_oper_argv());
-    }
+  else { depthLevels = cdo_argv_to_fltarr(cdo_get_oper_argv()); }
 
   if (zaxisID == CDI_UNDEFID)
     {
@@ -66,8 +61,20 @@ create_zaxis_depth(Varray<double> &depthLevels)
   return zaxisID;
 }
 
-class ModuleVertintzs
+class Vertintzs : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Vertintzs",
+    .operators = { { "zs2zl", 0, 0, "depth levels in meter" }, { "zs2zlx" } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Vertintzs> registration = RegisterEntry<Vertintzs>(module);
+
 private:
   int nvars;
   std::vector<bool> processVars;
@@ -76,15 +83,15 @@ private:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
   VarList varList1;
   VarList varList2;
   Field3DVector vardata1;
   Field3DVector vardata2;
 
-  Varray2D<size_t> varnmiss;
+  Varray2D<size_t> varnumMissVals;
 
   int depthID = -1;
 
@@ -99,15 +106,12 @@ private:
   Field depthBottom;
   int numFullLevels;
 
-  Varray<size_t> pnmiss;
+  Varray<size_t> pnumMissVals;
 
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-
-    cdo_operator_add("zs2zl", 0, 0, "depth levels in meter");
 
     auto operatorID = cdo_operator_id();
 
@@ -126,32 +130,31 @@ public:
 
     gridsize = vlist_check_gridsize(vlistID1);
 
-    varList_init(varList1, vlistID1);
-    varListSetUniqueMemtype(varList1);
-    auto memType = varList1[0].memType;
-
-    nvars = vlistNvars(vlistID1);
+    varList1 = VarList(vlistID1);
+    varList_set_unique_memtype(varList1);
+    auto memType = varList1.vars[0].memType;
 
+    nvars = varList1.numVars();
     for (int varID = 0; varID < nvars; ++varID)
       {
-        if (string_to_lower(varList1[varID].name) == "depth_c") depthID = varID;
+        if (string_to_lower(varList1.vars[varID].name) == "depth_c") depthID = varID;
       }
 
     if (Options::cdoVerbose)
       {
         cdo_print("Found:");
         // clang-format off
-        if (-1 != depthID)   cdo_print("  %s -> %s", "zstar depth at cell center", varList1[depthID].name);
+        if (-1 != depthID)   cdo_print("  %s -> %s", "zstar depth at cell center", varList1.vars[depthID].name);
         // clang-format on
       }
 
     if (-1 == depthID) cdo_abort("depth_c not found!");
 
-    auto zaxisIDfull = (-1 == depthID) ? -1 : varList1[depthID].zaxisID;
+    auto zaxisIDfull = (-1 == depthID) ? -1 : varList1.vars[depthID].zaxisID;
     numFullLevels = (-1 == zaxisIDfull) ? 0 : zaxisInqSize(zaxisIDfull);
 
-    auto nzaxis = vlistNzaxis(vlistID1);
-    for (int index = 0; index < nzaxis; ++index)
+    auto numZaxes = vlistNumZaxis(vlistID1);
+    for (int index = 0; index < numZaxes; ++index)
       {
         auto zaxisID = vlistZaxis(vlistID1, index);
         auto nlevels = zaxisInqSize(zaxisID);
@@ -159,16 +162,16 @@ public:
           vlistChangeZaxis(vlistID2, zaxisID, zaxisID2);
       }
 
-    varList_init(varList2, vlistID2);
-    varListSetMemtype(varList2, memType);
+    varList2 = VarList(vlistID2);
+    varList_set_memtype(varList2, memType);
 
-    if (!extrapolate) pnmiss.resize(depthLevels.size());
+    if (!extrapolate) pnumMissVals.resize(depthLevels.size());
 
     vertIndexFull.resize(gridsize * depthLevels.size());
 
     processVars = std::vector<bool>(nvars);
     interpVars = std::vector<bool>(nvars);
-    varnmiss = Varray2D<size_t>(nvars);
+    varnumMissVals = Varray2D<size_t>(nvars);
     vardata1 = Field3DVector(nvars);
     vardata2 = Field3DVector(nvars);
 
@@ -176,16 +179,16 @@ public:
 
     for (int varID = 0; varID < nvars; ++varID)
       {
-        const auto &var = varList1[varID];
+        auto const &var = varList1.vars[varID];
 
         if (gridInqType(var.gridID) == GRID_SPECTRAL) cdo_abort("Spectral data unsupported!");
 
         vardata1[varID].init(var);
-        varnmiss[varID].resize(maxlev, 0);
+        varnumMissVals[varID].resize(maxlev, 0);
 
         interpVars[varID] = (var.zaxisID == zaxisIDfull || (is_depth_axis(var.zaxisID) && (var.nlevels == numFullLevels)));
 
-        if (interpVars[varID]) { vardata2[varID].init(varList2[varID]); }
+        if (interpVars[varID]) { vardata2[varID].init(varList2.vars[varID]); }
         else if (is_depth_axis(var.zaxisID) && var.nlevels > 1)
           {
 
@@ -193,11 +196,11 @@ public:
           }
       }
 
-    fullDepth.init(varList1[depthID]);
+    fullDepth.init(varList1.vars[depthID]);
 
     for (int varID = 0; varID < nvars; ++varID)
       {
-        if (interpVars[varID] && varList1[varID].isConstant) vlistDefVarTimetype(vlistID2, varID, TIME_VARYING);
+        if (interpVars[varID] && varList1.vars[varID].isConstant) vlistDefVarTimetype(vlistID2, varID, TIME_VARYING);
       }
 
     streamID2 = cdo_open_write(1);
@@ -206,45 +209,44 @@ public:
   }
 
   void
-  run()
+  run() override
   {
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         for (int varID = 0; varID < nvars; ++varID)
           {
             processVars[varID] = false;
-            const auto &var = varList1[varID];
-            for (int levelID = 0; levelID < var.nlevels; ++levelID) varnmiss[varID][levelID] = 0;
+            auto const &var = varList1.vars[varID];
+            for (int levelID = 0; levelID < var.nlevels; ++levelID) varnumMissVals[varID][levelID] = 0;
           }
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            cdo_read_record(streamID1, vardata1[varID], levelID, &varnmiss[varID][levelID]);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            cdo_read_field(streamID1, vardata1[varID], levelID, &varnumMissVals[varID][levelID]);
             processVars[varID] = true;
           }
 
         for (int varID = 0; varID < nvars; ++varID)
           if (interpVars[varID]) processVars[varID] = true;
 
-        if (tsID == 0 || !varList1[depthID].isConstant)
+        if (tsID == 0 || !varList1.vars[depthID].isConstant)
           {
             constexpr auto lreverse = false;
             field_copy(vardata1[depthID], fullDepth);
             gen_vert_index(vertIndexFull, depthLevels, fullDepth, gridsize, lreverse);
             if (!extrapolate)
               {
-                depthBottom.init(varList1[depthID]);
+                depthBottom.init(varList1.vars[depthID]);
                 field_copy(fullDepth, numFullLevels - 1, depthBottom);
-                gen_vert_index_mv(vertIndexFull, depthLevels, gridsize, depthBottom, pnmiss, lreverse);
+                gen_vert_index_mv(vertIndexFull, depthLevels, gridsize, depthBottom, pnumMissVals, lreverse);
               }
           }
 
@@ -252,7 +254,7 @@ public:
           {
             if (processVars[varID])
               {
-                const auto &var = varList1[varID];
+                auto const &var = varList1.vars[varID];
 
                 if (tsID > 0 && !interpVars[varID] && var.isConstant) continue;
 
@@ -263,19 +265,19 @@ public:
 
                     for (int levelID = 0; levelID < var.nlevels; ++levelID)
                       {
-                        if (varnmiss[varID][levelID]) cdo_abort("Missing values unsupported for this operator!");
+                        if (varnumMissVals[varID][levelID]) cdo_abort("Missing values unsupported for this operator!");
                       }
 
                     vertical_interp_X(fullDepth, vardata1[varID], vardata2[varID], vertIndexFull, depthLevels, gridsize);
 
-                    if (!extrapolate) varray_copy(depthLevels.size(), pnmiss, varnmiss[varID]);
+                    if (!extrapolate) varray_copy(depthLevels.size(), pnumMissVals, varnumMissVals[varID]);
                   }
 
-                for (int levelID = 0; levelID < varList2[varID].nlevels; ++levelID)
+                for (int levelID = 0; levelID < varList2.vars[varID].nlevels; ++levelID)
                   {
-                    cdo_def_record(streamID2, varID, levelID);
+                    cdo_def_field(streamID2, varID, levelID);
                     auto varout = (interpVars[varID] ? vardata2[varID] : vardata1[varID]);
-                    cdo_write_record(streamID2, varout, levelID, varnmiss[varID][levelID]);
+                    cdo_write_field(streamID2, varout, levelID, varnumMissVals[varID][levelID]);
                   }
               }
           }
@@ -285,21 +287,9 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Vertintzs(void *process)
-{
-  ModuleVertintzs vertintzs;
-  vertintzs.init(process);
-  vertintzs.run();
-  vertintzs.close();
-  return nullptr;
-}
