@@ -37,32 +37,32 @@
 int
 get_surface_ID(int vlistID)
 {
-  int surfID = -1;
+  int surfaceID = -1;
 
-  auto nzaxis = vlistNumZaxis(vlistID);
-  for (int index = 0; index < nzaxis; ++index)
+  auto numZaxes = vlistNumZaxis(vlistID);
+  for (int index = 0; index < numZaxes; ++index)
     {
       auto zaxisID = vlistZaxis(vlistID, index);
       if (IS_SURFACE_LEVEL(zaxisID))
         {
-          surfID = vlistZaxis(vlistID, index);
+          surfaceID = vlistZaxis(vlistID, index);
           break;
         }
     }
 
-  if (surfID == -1) surfID = zaxis_from_name("surface");
+  if (surfaceID == -1) surfaceID = zaxis_from_name("surface");
 
-  return surfID;
+  return surfaceID;
 }
 
 static void
-set_surface_ID(const int vlistID, const int surfID)
+set_surface_ID(int vlistID, int surfaceID)
 {
-  auto nzaxis = vlistNumZaxis(vlistID);
-  for (int index = 0; index < nzaxis; ++index)
+  auto numZaxes = vlistNumZaxis(vlistID);
+  for (int index = 0; index < numZaxes; ++index)
     {
       auto zaxisID = vlistZaxis(vlistID, index);
-      if (zaxisID != surfID || !IS_SURFACE_LEVEL(zaxisID)) vlistChangeZaxisIndex(vlistID, index, surfID);
+      if (zaxisID != surfaceID || !IS_SURFACE_LEVEL(zaxisID)) vlistChangeZaxisIndex(vlistID, index, surfaceID);
     }
 }
 
@@ -72,19 +72,19 @@ vertstat_get_parameter(bool &weights, bool &genbounds)
   auto pargc = cdo_operator_argc();
   if (pargc)
     {
-      const auto &pargv = cdo_get_oper_argv();
+      auto const &pargv = cdo_get_oper_argv();
 
       KVList kvlist;
       kvlist.name = cdo_module_name();
       if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-          const auto &value = kv.values[0];
+          auto const &value = kv.values[0];
 
           // clang-format off
           if      (key == "weights")   weights = parameter_to_bool(value);
@@ -134,12 +134,11 @@ private:
 
   CdoStreamID streamID1;
   CdoStreamID streamID2;
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  int vlistID2;
+  int vlistID2{ CDI_UNDEFID };
 
-  int nzaxis;
   int numVars;
 
   VarList varList1;
@@ -165,9 +164,11 @@ public:
 
     streamID1 = cdo_open_read(0);
     auto vlistID1 = cdo_stream_inq_vlist(streamID1);
-
     vlistClearFlag(vlistID1);
-    numVars = vlistNvars(vlistID1);
+
+    varList1 = VarList(vlistID1);
+
+    numVars = varList1.numVars();
     for (int varID = 0; varID < numVars; ++varID) vlistDefFlag(vlistID1, varID, 0, true);
 
     vlistID2 = vlistCreate();
@@ -181,8 +182,8 @@ public:
     auto surfID = get_surface_ID(vlistID1);
     set_surface_ID(vlistID2, surfID);
 
-    nzaxis = vlistNumZaxis(vlistID1);
-    vert = std::vector<VertInfo>(nzaxis);
+    auto numZaxes = varList1.numZaxes();
+    vert = std::vector<VertInfo>(numZaxes);
     if (needWeights)
       {
         auto useweights = true;
@@ -195,7 +196,7 @@ public:
             cdo_print("Using constant vertical weights!");
           }
 
-        for (int index = 0; index < nzaxis; ++index)
+        for (int index = 0; index < numZaxes; ++index)
           {
             auto zaxisID = vlistZaxis(vlistID1, index);
             auto nlev = zaxisInqSize(zaxisID);
@@ -217,8 +218,6 @@ public:
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
 
-    varList1 = VarList(vlistID1);
-
     int VARS_MEMTYPE = stepStat.lminmax ? FIELD_NAT : 0;
     stepStat.alloc(varList1, VARS_MEMTYPE);
   }
@@ -231,19 +230,19 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
         std::vector<bool> varsLevelInit(numVars, false);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
 
-            const auto &var = varList1.vars[varID];
+            auto const &var = varList1.vars[varID];
 
             auto &rsamp1 = stepStat.samp(varID);
             auto &rvar1 = stepStat.var1(varID);
@@ -258,7 +257,7 @@ public:
             auto layerThickness = 1.0;
             if (needWeights)
               {
-                for (int index = 0; index < nzaxis; ++index)
+                for (int index = 0, n = vert.size(); index < n; ++index)
                   if (vert[index].zaxisID == var.zaxisID)
                     {
                       if (vert[index].status == 0 && tsID == 0 && levelID == 0 && var.nlevels > 1)
@@ -276,7 +275,7 @@ public:
               }
 
             field.init(var);
-            cdo_read_record(streamID1, field);
+            cdo_read_field(streamID1, field);
 
             if (varsLevelInit[varID] == false)
               {
@@ -348,8 +347,8 @@ public:
               {
                 stepStat.process(varID, numSets);
 
-                cdo_def_record(streamID2, varID, 0);
-                cdo_write_record(streamID2, stepStat.var1(varID));
+                cdo_def_field(streamID2, varID, 0);
+                cdo_write_field(streamID2, stepStat.var1(varID));
                 stepStat.var1(varID).nsamp = 0;
               }
           }
