@@ -29,7 +29,7 @@ public:
   using Process::Process;
   inline static CdoModule module = {
     .name = "Vertwind",
-    .operators = { { "vertwind"} },
+    .operators = { { "vertwind" } },
     .aliases = {},
     .mode = EXPOSED,     // Module mode: 0:intern 1:extern
     .number = CDI_REAL,  // Allowed number type
@@ -40,7 +40,7 @@ public:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int vlistID2;
+  int vlistID2{ CDI_UNDEFID };
 
   int taxisID1 = -1;
   int taxisID2 = -1;
@@ -116,7 +116,7 @@ public:
     missval_t = varList1.vars[tempID].missval;
     missval_sq = varList1.vars[sqID].missval;
 
-    const auto &varOmega = varList1.vars[omegaID];
+    auto const &varOmega = varList1.vars[omegaID];
     missval_wap = varOmega.missval;
     missval_out = missval_wap;
 
@@ -188,31 +188,31 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
             size_t numMissVals;
-            auto[varID, levelID] = cdo_inq_record(streamID1);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
 
             auto offset = (size_t) levelID * gridsize;
 
             if (varID == tempID)
-              cdo_read_record(streamID1, &temp[offset], &numMissVals);
+              cdo_read_field(streamID1, &temp[offset], &numMissVals);
             else if (varID == sqID)
-              cdo_read_record(streamID1, &sq[offset], &numMissVals);
+              cdo_read_field(streamID1, &sq[offset], &numMissVals);
             else if (varID == omegaID)
-              cdo_read_record(streamID1, &omega[offset], &numMissVals);
+              cdo_read_field(streamID1, &omega[offset], &numMissVals);
             else if (varID == psID && zaxisInqType(zaxisID) == ZAXIS_HYBRID)
-              cdo_read_record(streamID1, psProg.data(), &numMissVals);
+              cdo_read_field(streamID1, psProg.data(), &numMissVals);
           }
 
         if (zaxisInqType(zaxisID) == ZAXIS_HYBRID)
-          vct_to_hybrid_pressure(fpress.data(), hpress.data(), vct.data(), psProg.data(), nlevels, gridsize);
+          vct_to_hybrid_pressure(fpress.data(), hpress.data(), vct, psProg.data(), nlevels, gridsize);
 
         for (int levelID = 0; levelID < nlevels; ++levelID)
           {
@@ -220,8 +220,8 @@ public:
 
             for (size_t i = 0; i < gridsize; ++i)
               {
-                if (dbl_is_equal(temp[offset + i], missval_t) || dbl_is_equal(omega[offset + i], missval_wap)
-                    || dbl_is_equal(sq[offset + i], missval_sq))
+                if (fp_is_equal(temp[offset + i], missval_t) || fp_is_equal(omega[offset + i], missval_wap)
+                    || fp_is_equal(sq[offset + i], missval_sq))
                   {
                     wms[offset + i] = missval_out;
                   }
@@ -247,10 +247,10 @@ public:
 
             size_t numMissVals_out = 0;
             for (size_t i = 0; i < gridsize; ++i)
-              if (dbl_is_equal(wms[offset + i], missval_out)) numMissVals_out++;
+              if (fp_is_equal(wms[offset + i], missval_out)) numMissVals_out++;
 
-            cdo_def_record(streamID2, 0, levelID);
-            cdo_write_record(streamID2, &wms[offset], numMissVals_out);
+            cdo_def_field(streamID2, 0, levelID);
+            cdo_write_field(streamID2, &wms[offset], numMissVals_out);
           }
 
         tsID++;
