@@ -288,7 +288,7 @@ DestaggerUV()
   int nvars = vlistNvars(vlistID1);
   for (int varID = 0; varID < nvars; ++varID)
     {
-      const auto &var = varList1.vars[varID];
+      auto const &var = varList1.vars[varID];
 
       int pnum, pcat, pdis;
       cdiDecodeParam(var.param, &pnum, &pcat, &pdis);
@@ -359,7 +359,7 @@ DestaggerUV()
       gridID0 = gridID1;
     }
 
-  int ngrids = vlistNumGrids(vlistID1);
+  auto numGrids = vlistNumGrids(vlistID1);
 
   double xincU = 0.0, yincU = 0.0, xincV = 0.0, yincV = 0.0;
 
@@ -434,7 +434,7 @@ DestaggerUV()
           cdo_print("About to perform destaggering (V-wind): (%3.2f; %3.2f) - %s order ", dxU, dyV, order);
         }
 
-      for (int index = 0; index < ngrids; ++index)
+      for (int index = 0; index < numGrids; ++index)
         {
           gridID = vlistGrid(vlistID1, index);
           if (cdoDebugExt >= 10)
@@ -446,7 +446,7 @@ DestaggerUV()
           Debug(cdoDebugExt, "Last trial to find a reference grid for destaggered wind.");
           for (int varID = 0; varID < nvars; ++varID)
             {
-              const auto &var = varList1.vars[varID];
+              auto const &var = varList1.vars[varID];
 
               gridID = var.gridID;
               Debug(cdoDebugExt, "Var.id %d has grid nr:%d", varID, gridID);
@@ -459,15 +459,13 @@ DestaggerUV()
         }
       if (gridID0 == -1) cdo_abort("Referencial horizontal grid not found!");
 
-      gridsize = vlistGridsizeMax(vlistID1);
-
       if (gridInqSize(gridID2) != gridInqSize(gridID1))
         cdo_abort("gridSize of U-wind != gridSize of V-wind!  This should not happen!");
 
       if (cdoDebugExt)
-        cdo_print("Allocating memory for maximum gridsize (for input) = %ld [%4.3f MB]", gridsize,
-                  gridsize * sizeof(double) / (1024.0 * 1024));
-      ivar.resize(gridsize);  // storage for other fields than
+        cdo_print("Allocating memory for maximum gridsize (for input) = %ld [%4.3f MB]", varList1.gridsizeMax(),
+                  varList1.gridsizeMax() * sizeof(double) / (1024.0 * 1024));
+      ivar.resize(varList1.gridsizeMax());  // storage for other fields than
 
       gridsize = gridInqSize(gridID1);  // actual size of U-wind should be same as V-wind
       if (cdoDebugExt)
@@ -490,8 +488,8 @@ DestaggerUV()
   int tsID = 0;
   while (true)
     {
-      auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-      if (nrecs == 0) break;
+      auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+      if (numFields == 0) break;
 
       cdo_taxis_copy_timestep(taxisID2, taxisID1);
       cdo_def_timestep(streamID2, tsID);
@@ -499,14 +497,14 @@ DestaggerUV()
       if (!lcopy && cdoDebugExt)
         {
           cdo_print("Processing timestep: %d", tsID);
-          cdo_print("Starting destaggering. Total records to be processed: %05d", nrecs);
+          cdo_print("Starting destaggering. Total fields to be processed: %05d", numFields);
         }
 
-      for (int recID = 0; recID < nrecs; ++recID)
+      for (int fieldID = 0; fieldID < numFields; ++fieldID)
         {
-          auto [varID, levelID] = cdo_inq_record(streamID1);
+          auto [varID, levelID] = cdo_inq_field(streamID1);
 
-          const auto &var = varList1.vars[varID];
+          auto const &var = varList1.vars[varID];
 
           int pnum, pcat, pdis;
           cdiDecodeParam(var.param, &pnum, &pcat, &pdis);
@@ -540,7 +538,7 @@ DestaggerUV()
                           if (cdoDebugExt >= 10)
                             cdo_print("Destaggering U-wind record: %05d (timestep:%d); Var.id [%4d]; (code=%3d; "
                                       "ltype=%3d; level=%4d; levelID=%3d); GridID %d => %d  *** <===",
-                                      recID, tsID, varID, code, ltype, level, levelID, var.gridID, varList2.vars[varID].gridID);
+                                      fieldID, tsID, varID, code, ltype, level, levelID, var.gridID, varList2.vars[varID].gridID);
                         }
                       else
                         UorV = -1;  // this U is not staggered, just copy the record..
@@ -552,7 +550,7 @@ DestaggerUV()
                           if (cdoDebugExt >= 10)
                             cdo_print("Destaggering V-wind record: %05d (timestep:%d); Var.id [%4d]; (code=%3d; "
                                       "ltype=%3d; level=%4d; levelID=%3d); GridID %d => %d  *** <===",
-                                      recID, tsID, varID, code, ltype, level, levelID, var.gridID, varList2.vars[varID].gridID);
+                                      fieldID, tsID, varID, code, ltype, level, levelID, var.gridID, varList2.vars[varID].gridID);
                         }
                       else
                         UorV = -1;  // this V is not staggered, just copy the record..
@@ -562,7 +560,7 @@ DestaggerUV()
               if (UorV >= 0)  // re-check again since it could mean that current record with U or V is not staggered
                 {
                   size_t numMissVals;
-                  cdo_read_record(streamID1, ivar.data(), &numMissVals);
+                  cdo_read_field(streamID1, ivar.data(), &numMissVals);
 
                   // void destaggerUorV(double *fu, double *fuOut, int klev, int nlat, int nlon, int UorV, long int offset);
                   // We handle one level at the time; klev=1;offset=0.
@@ -590,33 +588,33 @@ DestaggerUV()
                   // cdo uvDestag: Grid info: (dxU; dyU) = (0.00; 0.05); (dxV; dyV) = (0.05; 0.00)
                   if (cdoDebugExt >= 20) cdo_print("Setting GRID id from: %d => to: %d", var.gridID, varList2.vars[varID].gridID);
 
-                  cdo_def_record(streamID2, varID, levelID);
-                  cdo_write_record(streamID2, ovar.data(), numMissVals);
+                  cdo_def_field(streamID2, varID, levelID);
+                  cdo_write_field(streamID2, ovar.data(), numMissVals);
                 }
               else
                 {  // copy the record to the output unchanged...
-                  cdo_def_record(streamID2, varID, levelID);
+                  cdo_def_field(streamID2, varID, levelID);
                   if (cdoDebugExt >= 20)
                     cdo_print("Stream-copy data record:    %05d (timestep:%d); "
                               "Var.id [%4d]; (code=%3d; ltype=%3d; level=%4d; levelID=%3d)",
-                              recID, tsID, varID, code, ltype, level, levelID);
-                  cdo_copy_record(streamID2, streamID1);
+                              fieldID, tsID, varID, code, ltype, level, levelID);
+                  cdo_copy_field(streamID2, streamID1);
                 }
             }  // end of: if (!lcopy)
           else
             {  // copy the record to the output unchanged...
-              cdo_def_record(streamID2, varID, levelID);
+              cdo_def_field(streamID2, varID, levelID);
               if (cdoDebugExt >= 20)
                 cdo_print("Stream-copy data record:    %05d (timestep:%d); "
                           "Var.id [%4d]; (code=%3d; ltype=%3d; level=%4d; levelID=%3d)",
-                          recID, tsID, varID, code, ltype, level, levelID);
-              cdo_copy_record(streamID2, streamID1);
+                          fieldID, tsID, varID, code, ltype, level, levelID);
+              cdo_copy_field(streamID2, streamID1);
             }
 
-        }  // end of for ( recID = ...
+        }  // end of for ( fieldID = ...
 
       tsID++;
-    }  // end of while ( (nrecs ...
+    }  // end of while ( (numFields ...
 
   cdo_stream_close(streamID2);
   cdo_stream_close(streamID1);
@@ -708,7 +706,7 @@ rot_uv_north(int gridID, double *us, double *vs)
     cdo_print("%s(gridname=%s) .. processing grid with UV [nx*ny] (%zu * %zu)", __func__, gridNamePtr(gridInqType(gridID)), nx, ny);
 
   if (gridInqSize(gridID) != (nx * ny)) cdo_abort("Incorrect gridsize (%zu) != nx*ny (%zu * %zu)", gridInqSize(gridID), nx, ny);
-    // this should never happen
+  // this should never happen
 
 #define OPTrotuvNorth 1  // ACTIVATE SPEED - OPTIMIZATION
 
@@ -899,7 +897,7 @@ rot_uv_north(int gridID, double *us, double *vs)
                 }
 
           }  // end of for ( i = 0; i < nx; i++ )
-    }        // end of if (rotationMatrixArray== nullptr)
+    }  // end of if (rotationMatrixArray== nullptr)
 
   // Take the rotation matrix from the cache
   for (j = 0; j < ny; ++j)
@@ -1168,9 +1166,9 @@ public:
     VarList varList2(vlistID2);
 
     auto numVars = varList1.numVars();
-    auto numRecords = varList1.numRecords();
+    auto maxFields = varList1.maxFields();
 
-    std::vector<RecordInfo> recordList(numRecords);
+    std::vector<FieldInfo> fieldInfoList(maxFields);
 
     std::vector<std::vector<size_t>> varnumMissVals(numVars);
     Varray3D<double> vardata(numVars);
@@ -1208,7 +1206,7 @@ public:
 
     for (int varID = 0; varID < numVars; ++varID)
       {
-        const auto &var = varList1.vars[varID];
+        auto const &var = varList1.vars[varID];
 
         cdiDecodeParam(var.param, &pnum, &pcat, &pdis);
         code = pnum;
@@ -1254,47 +1252,47 @@ public:
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         cdo_taxis_copy_timestep(taxisID2, taxisID1);
         cdo_def_timestep(streamID2, tsID);
 
         if (cdoDebugExt) cdo_print("About to read U & V data to memory. Other data will be stream-copied to the output file.");
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
             code = varList1.vars[varID].code;
             zaxisID = varList1.vars[varID].zaxisID;
             ltype = zaxis_to_ltype(zaxisID);
             level = zaxisInqLevel(zaxisID, levelID);
 
             if (vardata[varID].empty())
-              {                                // This means that it is not eighter U neither V.
-                recordList[recID].varID = -1;  // We will NOT record/store this field in memory
-                recordList[recID].levelID = -1;
+              {                                     // This means that it is not eighter U neither V.
+                fieldInfoList[fieldID].varID = -1;  // We will NOT record/store this field in memory
+                fieldInfoList[fieldID].levelID = -1;
                 // We will stream-copy this data
-                cdo_def_record(streamID2, varID, levelID);
-                // if ( cdoDebugExt>10 ) cdo_print("Copying data record.. %05d (timestep:%05d)", recID, tsID);
+                cdo_def_field(streamID2, varID, levelID);
+                // if ( cdoDebugExt>10 ) cdo_print("Copying data record.. %05d (timestep:%05d)", fieldID, tsID);
                 if (cdoDebugExt >= 20)
                   cdo_print("Stream-copy data record:    %05d (timestep:%d); Var.id [%4d]; (code=%3d; ltype=%3d; level=%4d; "
                             "levelID=%3d)",
-                            recID, tsID, varID, code, ltype, level, levelID);
-                cdo_copy_record(streamID2, streamID1);  // cannot do this ! We have to set the flag uvGridRelative = 0
+                            fieldID, tsID, varID, code, ltype, level, levelID);
+                cdo_copy_field(streamID2, streamID1);  // cannot do this ! We have to set the flag uvGridRelative = 0
               }
             else
               {
-                recordList[recID].varID = varID;
-                recordList[recID].levelID = levelID;
+                fieldInfoList[fieldID].varID = varID;
+                fieldInfoList[fieldID].levelID = levelID;
                 if (cdoDebugExt >= 10)
                   cdo_print("Memmory-read data record:   %05d (timestep:%d); Var.id [%4d]; (code=%3d; ltype=%3d; level=%4d; "
                             "levelID=%3d)",
-                            recID, tsID, varID, code, ltype, level, levelID);
-                cdo_read_record(streamID1, vardata[varID][levelID].data(), &varnumMissVals[varID][levelID]);
+                            fieldID, tsID, varID, code, ltype, level, levelID);
+                cdo_read_field(streamID1, vardata[varID][levelID].data(), &varnumMissVals[varID][levelID]);
                 if (varnumMissVals[varID][levelID]) cdo_abort("Missing values unsupported for this operator!");
               }
-          }  // end of for ( recID = 0; recID < nrecs; ..
+          }  // end of for ( fieldID = 0; fieldID < numFields; ..
 
         Debug(cdoDebugExt, "All neccessary U & V data are in memory. About to transform the windvectors...");
 
@@ -1335,7 +1333,7 @@ public:
                   }
                 else  // This means that it is U or V.
                   {
-                    const auto &var2 = varList2.vars[varID2];
+                    auto const &var2 = varList2.vars[varID2];
                     code2 = var2.code;
                     zaxisID2 = var2.zaxisID;
                     ltype2 = zaxis_to_ltype(zaxisID2);
@@ -1424,7 +1422,7 @@ public:
                               }  // end of if (cdoDebugExt)
                             Debug(cdoDebugExt, "LAT-LON grid created.");
                           }  // end of if (gridIDcurvl==-1)
-                      }      // end of if (operatorID != ROTUVN)
+                      }  // end of if (operatorID != ROTUVN)
 
                     int uRelativeToGrid = -1, vRelativeToGrid = -1;
                     cdiInqKeyInt(vlistID1, varID1, CDI_KEY_UVRELATIVETOGRID, &uRelativeToGrid);
@@ -1463,12 +1461,12 @@ public:
                     Debug(cdoDebugExt, "Finished processing level type: %d", ltype1);
                     break;
                   }  // end  for varID2
-            }        // end  for varID1
+            }  // end  for varID1
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto varID = recordList[recID].varID;
-            auto levelID = recordList[recID].levelID;
+            auto varID = fieldInfoList[fieldID].varID;
+            auto levelID = fieldInfoList[fieldID].levelID;
             if (varID != -1)
               {
                 code = varList1.vars[varID].code;
@@ -1478,10 +1476,10 @@ public:
                 if (cdoDebugExt >= 10)
                   cdo_print("Write modified data record: %05d (timestep:%d); Var.id [%4d]; (code=%3d; ltype=%3d; level=%4d; "
                             "levelID=%3d)",
-                            recID, tsID, varID, code, ltype, level, levelID);
+                            fieldID, tsID, varID, code, ltype, level, levelID);
 
-                cdo_def_record(streamID2, varID, levelID);
-                cdo_write_record(streamID2, vardata[varID][levelID].data(), varnumMissVals[varID][levelID]);
+                cdo_def_field(streamID2, varID, levelID);
+                cdo_write_field(streamID2, vardata[varID][levelID].data(), varnumMissVals[varID][levelID]);
               }
           }
 
