@@ -44,9 +44,9 @@ public:
   CdoStreamID streamID3;
   CdoStreamID streamID4;
 
-  int vlistID1;
-  int taxisID1;
-  int taxisID2;
+  int vlistID1{ CDI_UNDEFID };
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
   int taxisID3;
   int taxisID4;
 
@@ -137,13 +137,13 @@ public:
             vars1[dayOfYear] = true;
             hsets[dayOfYear].create(numVars, numSteps);
 
-            for (const auto &var : varList1.vars) hsets[dayOfYear].createVarLevels(var.ID, var.nlevels, var.gridsize);
+            for (auto const &var : varList1.vars) hsets[dayOfYear].createVarLevels(var.ID, var.nlevels, var.gridsize);
           }
 
         for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
             auto [varID, levelID] = cdo_inq_field(streamID2);
-            const auto &var = varList1.vars[varID];
+            auto const &var = varList1.vars[varID];
             field1.init(var);
             cdo_read_field(streamID2, field1);
 
@@ -178,7 +178,7 @@ public:
         for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
             auto [varID, levelID] = cdo_inq_field(streamID1);
-            const auto &var = varList1.vars[varID];
+            auto const &var = varList1.vars[varID];
 
             if (tsID == 0) fieldInfoList[fieldID].set(varID, levelID);
 
@@ -214,7 +214,7 @@ public:
           for (int fieldID = 0; fieldID < maxFields; ++fieldID)
             {
               auto [varID, levelID] = fieldInfoList[fieldID].get();
-              const auto &var = varList1.vars[varID];
+              auto const &var = varList1.vars[varID];
               if (otsID && var.isConstant) continue;
 
               cdo_def_field(streamID4, varID, levelID);
