@@ -68,7 +68,7 @@ ydstat_update(YdayStats &stats, CdiDateTime vDateTime, const FieldVector2D &vars
   auto numVars = stats.varList.numVars();
   for (int varID = 0; varID < numVars; ++varID)
     {
-      const auto &var = stats.varList.vars[varID];
+      auto const &var = stats.varList.vars[varID];
       if (var.timeType == TIME_CONSTANT) continue;
 
       for (int levelID = 0; levelID < var.nlevels; ++levelID)
@@ -109,7 +109,7 @@ ydstat_finalize(YdayStats &stats, int operfunc)
         auto numVars = stats.varList.numVars();
         for (int varID = 0; varID < numVars; ++varID)
           {
-            const auto &var = stats.varList.vars[varID];
+            auto const &var = stats.varList.vars[varID];
             if (var.timeType == TIME_CONSTANT) continue;
 
             for (int levelID = 0; levelID < var.nlevels; ++levelID)
@@ -120,7 +120,7 @@ ydstat_finalize(YdayStats &stats, int operfunc)
                 if (lmean) { fieldc_div(rvars1, (double) numSets); }
                 else if (lvarstd)
                   {
-                    const auto &rvars2 = stats.varsData2[dayOfYear][varID][levelID];
+                    auto const &rvars2 = stats.varsData2[dayOfYear][varID][levelID];
                     fieldc_stdvar_func(rvars1, rvars2, numSets, divisor);
                   }
               }
@@ -161,12 +161,12 @@ get_parameter()
       if (kvlist.parse_arguments(pargv) != 0) cdo_abort("Parse error!");
       if (Options::cdoVerbose) kvlist.print();
 
-      for (const auto &kv : kvlist)
+      for (auto const &kv : kvlist)
         {
-          const auto &key = kv.key;
+          auto const &key = kv.key;
           if (kv.nvalues > 1) cdo_abort("Too many values for parameter key >%s<!", key);
           if (kv.nvalues < 1) cdo_abort("Missing value for parameter key >%s<!", key);
-          const auto &value = kv.values[0];
+          auto const &value = kv.values[0];
 
           // clang-format off
           if      (key == "nts") params.nts = parameter_to_int(value);
@@ -215,9 +215,9 @@ private:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
 
-  int taxisID1;
-  int taxisID2;
-  int vlistID1;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
+  int vlistID1{ CDI_UNDEFID };
 
   char readMethod{ 0 };
   bool lvarstd;
