@@ -16,56 +16,48 @@
 #include "calendar.h"
 
 #include "cdo_options.h"
-#include "cdo_vlist.h"
 #include "process_int.h"
 #include "datetime.h"
 #include "printinfo.h"
 #include "field_functions.h"
 
-class ModuleYearMonStat
+class Yearmonstat : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Yearmonstat",
+    .operators = { { "yearmonmean", FieldFunc_Mean, 0, YearmonstatHelp }, { "yearmonavg", FieldFunc_Avg, 0, YearmonstatHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 1, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Yearmonstat> registration = RegisterEntry<Yearmonstat>(module);
 
 private:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  int vlistID1;
-  int vlistID2;
-
-  int taxisID1;
-  int taxisID2;
+  int vlistID1{ CDI_UNDEFID };
+  int vlistID2{ CDI_UNDEFID };
 
   int year0 = 0, month0 = 0;
   int year, month, day;
 
   DateTimeList dtlist;
 
-  VarList varList;
-  Field field;
-
-  FieldVector2D samp1, vars1;
   CdiDateTime vDateTime0{};
 
   int calendar;
-
-  std::vector<RecordInfo> recList;
   int operfunc;
 
-  int nvars;
-  int maxrecs;
-
 public:
   void
-  init(void *process)
+  init() override
   {
-
-    cdo_initialize(process);
-
-    // clang-format off
-    cdo_operator_add("yearmonmean",  FieldFunc_Mean, 0, nullptr);
-    cdo_operator_add("yearmonavg",   FieldFunc_Avg,  0, nullptr);
-    // clang-format on
-
     auto operatorID = cdo_operator_id();
 
     operfunc = cdo_operator_f1(operatorID);
@@ -87,36 +79,36 @@ public:
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
 
-    nvars = vlistNvars(vlistID1);
-
-    maxrecs = vlistNrecs(vlistID1);
-    recList = std::vector<RecordInfo>(maxrecs);
-
     calendar = taxisInqCalendar(taxisID1);
     constexpr auto timestatDate{ TimeStat::MEAN };
     dtlist.set_stat(timestatDate);
     dtlist.set_calendar(calendar);
-
-    varList_init(varList, vlistID1);
-
-    fields_from_vlist(vlistID1, samp1);
-    fields_from_vlist(vlistID1, vars1, FIELD_VEC);
   }
 
   void
-  run()
+  run() override
   {
+    Field field;
+    VarList varList1(vlistID1);
+
+    auto maxFields = varList1.maxFields();
+    std::vector<FieldInfo> fieldInfoList(maxFields);
+
+    FieldVector2D samp1, varsData1;
+    field2D_init(samp1, varList1);
+    field2D_init(varsData1, varList1, FIELD_VEC);
+
     int tsID = 0;
     int otsID = 0;
     while (true)
       {
-        int nrecs = 0;
-        long numSets = 0;
+        int numFields = 0;
+        int numSets = 0;
         double dsets = 0.0;
         while (true)
           {
-            nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-            if (nrecs == 0) break;
+            numFields = cdo_stream_inq_timestep(streamID1, tsID);
+            if (numFields == 0) break;
 
             dtlist.taxis_inq_timestep(taxisID1, numSets);
             auto vDateTime = dtlist.get_vDateTime(numSets);
@@ -139,24 +131,23 @@ public:
 
             auto dpm = days_per_month(calendar, year, month);
 
-            for (int recID = 0; recID < nrecs; ++recID)
+            for (int fieldID = 0; fieldID < numFields; ++fieldID)
               {
-                int varID, levelID;
-                cdo_inq_record(streamID1, &varID, &levelID);
-                const auto &var = varList[varID];
+                auto [varID, levelID] = cdo_inq_field(streamID1);
+                auto const &var = varList1.vars[varID];
 
-                if (tsID == 0) recList[recID].set(varID, levelID);
+                if (tsID == 0) fieldInfoList[fieldID].set(varID, levelID);
 
                 auto &rsamp1 = samp1[varID][levelID];
-                auto &rvars1 = vars1[varID][levelID];
+                auto &rvars1 = varsData1[varID][levelID];
 
                 if (numSets == 0)
                   {
-                    cdo_read_record(streamID1, rvars1);
+                    cdo_read_field(streamID1, rvars1);
 
                     fieldc_mul(rvars1, dpm);
 
-                    if (rvars1.nmiss || !rsamp1.empty())
+                    if (rvars1.numMissVals || !rsamp1.empty())
                       {
                         if (rsamp1.empty()) rsamp1.resize(rvars1.size);
                         field2_vinit(rsamp1, rvars1, dpm);
@@ -165,11 +156,11 @@ public:
                 else
                   {
                     field.init(var);
-                    cdo_read_record(streamID1, field);
+                    cdo_read_field(streamID1, field);
 
                     fieldc_mul(field, dpm);
 
-                    if (field.nmiss || !rsamp1.empty())
+                    if (field.numMissVals || !rsamp1.empty())
                       {
                         if (rsamp1.empty()) rsamp1.resize(rvars1.size, dsets);
                         field2_vincr(rsamp1, field, dpm);
@@ -186,16 +177,17 @@ public:
             tsID++;
           }
 
-        if (nrecs == 0 && numSets == 0) break;
+        if (numFields == 0 && numSets == 0) break;
 
-        for (int varID = 0; varID < nvars; ++varID)
+        auto numVars = varList1.numVars();
+        for (int varID = 0; varID < numVars; ++varID)
           {
-            const auto &var = varList[varID];
+            auto const &var = varList1.vars[varID];
             if (var.isConstant) continue;
             for (int levelID = 0; levelID < var.nlevels; ++levelID)
               {
-                const auto &rsamp1 = samp1[varID][levelID];
-                auto &rvars1 = vars1[varID][levelID];
+                auto const &rsamp1 = samp1[varID][levelID];
+                auto &rvars1 = varsData1[varID][levelID];
                 if (!rsamp1.empty())
                   field2_div(rvars1, rsamp1);
                 else
@@ -208,38 +200,25 @@ public:
         dtlist.stat_taxis_def_timestep(taxisID2, numSets);
         cdo_def_timestep(streamID2, otsID);
 
-        for (int recID = 0; recID < maxrecs; ++recID)
+        for (int fieldID = 0; fieldID < maxFields; ++fieldID)
           {
-            auto [varID, levelID] = recList[recID].get();
-            if (otsID && varList[varID].isConstant) continue;
+            auto [varID, levelID] = fieldInfoList[fieldID].get();
+            if (otsID && varList1.vars[varID].isConstant) continue;
 
-            auto &rvars1 = vars1[varID][levelID];
-            cdo_def_record(streamID2, varID, levelID);
-            cdo_write_record(streamID2, rvars1);
+            auto &rvars1 = varsData1[varID][levelID];
+            cdo_def_field(streamID2, varID, levelID);
+            cdo_write_field(streamID2, rvars1);
           }
 
-        if (nrecs == 0) break;
+        if (numFields == 0) break;
         otsID++;
       }
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Yearmonstat(void *process)
-{
-  ModuleYearMonStat yearmonstat;
-  yearmonstat.init(process);
-  yearmonstat.run();
-  yearmonstat.close();
-
-  return nullptr;
-}
