@@ -16,17 +16,26 @@
 
 #include <cdi.h>
 
-#include "cdo_options.h"
 #include "cdo_vlist.h"
-#include "field.h"
 #include "datetime.h"
 #include "process_int.h"
 #include "param_conversion.h"
 #include "percentiles_hist.h"
 #include "field_functions.h"
 
-class ModuleYmonpctl
+class Ymonpctl : public Process
 {
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Ymonpctl",
+    .operators = { { "ymonpctl", FieldFunc_Pctl, 0, YmonpctlHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 3, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Ymonpctl> registration = RegisterEntry<Ymonpctl>(module);
   constexpr static int MaxMonths = 17;
 
   CdoStreamID streamID1;
@@ -34,35 +43,20 @@ class ModuleYmonpctl
   CdoStreamID streamID3;
   CdoStreamID streamID4;
 
-  int taxisID1;
-  int taxisID2;
+  int vlistID1{ CDI_UNDEFID };
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
   int taxisID3;
   int taxisID4;
 
-  CdiDateTime vDateTimes1[MaxMonths]{};
-  CdiDateTime vDateTimes2[MaxMonths]{};
-
-  std::vector<bool> vars1 = std::vector<bool>(MaxMonths, false);
-  HistogramSet hsets[MaxMonths];
-  long numSets[MaxMonths] = { 0 };
-  VarList varList1;
-  Field field1, field2;
-  FieldVector constFields;
-  std::vector<RecordInfo> recList;
-
-  int maxrecs;
-  int nvars;
-  int ntsteps;
-
   double pn;
 
+  VarList varList1;
+
 public:
   void
-  init(void *process)
+  init() override
   {
-    cdo_initialize(process);
-    cdo_operator_add("ymonpctl", FieldFunc_Pctl, 0, nullptr);
-
     operator_input_arg("percentile number");
     pn = parameter_to_double(cdo_operator_argv(0));
 
@@ -70,15 +64,19 @@ public:
     streamID2 = cdo_open_read(1);
     streamID3 = cdo_open_read(2);
 
-    auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+    vlistID1 = cdo_stream_inq_vlist(streamID1);
     auto vlistID2 = cdo_stream_inq_vlist(streamID2);
     auto vlistID3 = cdo_stream_inq_vlist(streamID3);
     auto vlistID4 = vlistDuplicate(vlistID1);
 
     vlist_unpack(vlistID4);
 
-    vlist_compare(vlistID1, vlistID2, CmpVlist::All);
-    vlist_compare(vlistID1, vlistID3, CmpVlist::All);
+    varList1 = VarList(vlistID1);
+    VarList varList2(vlistID2);
+    VarList varList3(vlistID3);
+
+    varList_compare(varList1, varList2);
+    varList_compare(varList1, varList3);
 
     taxisID1 = vlistInqTaxis(vlistID1);
     taxisID2 = vlistInqTaxis(vlistID2);
@@ -91,29 +89,33 @@ public:
 
     streamID4 = cdo_open_write(3);
     cdo_def_vlist(streamID4, vlistID4);
-
-    ntsteps = vlistNtsteps(vlistID1);
-    nvars = vlistNvars(vlistID1);
-
-    maxrecs = vlistNrecs(vlistID1);
-    recList = std::vector<RecordInfo>(maxrecs);
-
-    constFields = FieldVector(maxrecs);
-
-    varList_init(varList1, vlistID1);
   }
 
   void
-  run()
+  run() override
   {
+    Field field1, field2;
+    std::vector<bool> varsData1(MaxMonths, false);
+    CdiDateTime vDateTimes1[MaxMonths]{};
+    CdiDateTime vDateTimes2[MaxMonths]{};
+    HistogramSet hsets[MaxMonths];
+    long numSets[MaxMonths] = { 0 };
+
+    auto numVars = varList1.numVars();
+    auto numSteps = varList1.numSteps();
+    auto maxFields = varList1.maxFields();
+    std::vector<FieldInfo> fieldInfoList(maxFields);
+
+    FieldVector constFields(maxFields);
+
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID2, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID2, tsID);
+        if (numFields == 0) break;
 
-        if (nrecs != cdo_stream_inq_timestep(streamID3, tsID))
-          cdo_abort("Number of records at time step %d of %s and %s differ!", tsID + 1, cdo_get_stream_name(1),
+        if (numFields != cdo_stream_inq_timestep(streamID3, tsID))
+          cdo_abort("Number of fields at time step %d of %s and %s differ!", tsID + 1, cdo_get_stream_name(1),
                     cdo_get_stream_name(2));
 
         auto vDateTime2 = taxisInqVdatetime(taxisID2);
@@ -130,28 +132,23 @@ public:
 
         vDateTimes2[month] = vDateTime2;
 
-        if (!vars1[month])
+        if (!varsData1[month])
           {
-            vars1[month] = true;
-            hsets[month].create(nvars, ntsteps);
-            for (int varID = 0; varID < nvars; ++varID)
-              {
-                const auto &var = varList1[varID];
-                hsets[month].createVarLevels(varID, var.nlevels, var.gridsize);
-              }
+            varsData1[month] = true;
+            hsets[month].create(numVars, numSteps);
+            for (auto const &var : varList1.vars) hsets[month].createVarLevels(var.ID, var.nlevels, var.gridsize);
           }
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID2, &varID, &levelID);
-            const auto &var = varList1[varID];
+            auto [varID, levelID] = cdo_inq_field(streamID2);
+            auto const &var = varList1.vars[varID];
             field1.init(var);
-            cdo_read_record(streamID2, field1);
+            cdo_read_field(streamID2, field1);
 
-            cdo_inq_record(streamID3, &varID, &levelID);
+            (void) cdo_inq_field(streamID3);
             field2.init(var);
-            cdo_read_record(streamID3, field2);
+            cdo_read_field(streamID3, field2);
 
             hsets[month].defVarLevelBounds(varID, levelID, field1, field2);
           }
@@ -162,8 +159,8 @@ public:
     tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         auto vDateTime = taxisInqVdatetime(taxisID1);
 
@@ -174,25 +171,25 @@ public:
 
         vDateTimes1[month] = vDateTime;
 
-        if (!vars1[month]) cdo_abort("No data for month %d in %s and %s", month, cdo_get_stream_name(1), cdo_get_stream_name(2));
+        if (!varsData1[month])
+          cdo_abort("No data for month %d in %s and %s", month, cdo_get_stream_name(1), cdo_get_stream_name(2));
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            const auto &var = varList1[varID];
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            auto const &var = varList1.vars[varID];
 
-            if (tsID == 0) recList[recID].set(varID, levelID);
+            if (tsID == 0) fieldInfoList[fieldID].set(varID, levelID);
 
             if (tsID == 0 && var.isConstant)
               {
-                constFields[recID].init(var);
-                cdo_read_record(streamID1, constFields[recID]);
+                constFields[fieldID].init(var);
+                cdo_read_field(streamID1, constFields[fieldID]);
               }
             else
               {
                 field1.init(var);
-                cdo_read_record(streamID1, field1);
+                cdo_read_field(streamID1, field1);
 
                 hsets[month].addVarLevelValues(varID, levelID, field1);
               }
@@ -213,19 +210,20 @@ public:
           taxisDefVdatetime(taxisID4, vDateTimes1[month]);
           cdo_def_timestep(streamID4, otsID);
 
-          for (int recID = 0; recID < maxrecs; ++recID)
+          for (int fieldID = 0; fieldID < maxFields; ++fieldID)
             {
-              auto [varID, levelID] = recList[recID].get();
-              if (otsID && varList1[varID].isConstant) continue;
+              auto [varID, levelID] = fieldInfoList[fieldID].get();
+              auto const &var = varList1.vars[varID];
+              if (otsID && var.isConstant) continue;
 
-              cdo_def_record(streamID4, varID, levelID);
+              cdo_def_field(streamID4, varID, levelID);
 
-              if (varList1[varID].isConstant) { cdo_write_record(streamID4, constFields[recID]); }
+              if (var.isConstant) { cdo_write_field(streamID4, constFields[fieldID]); }
               else
                 {
-                  field1.init(varList1[varID]);
+                  field1.init(var);
                   hsets[month].getVarLevelPercentiles(field1, varID, levelID, pn);
-                  cdo_write_record(streamID4, field1);
+                  cdo_write_field(streamID4, field1);
                 }
             }
 
@@ -234,24 +232,11 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID4);
     cdo_stream_close(streamID3);
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Ymonpctl(void *process)
-{
-  ModuleYmonpctl ymonpctl;
-  ymonpctl.init(process);
-  ymonpctl.run();
-  ymonpctl.close();
-
-  return nullptr;
-}
