@@ -56,14 +56,14 @@ public:
 private:
   CdoStreamID streamID1;
   CdoStreamID streamID2;
-  int taxisID1;
-  int taxisID2;
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
 
-  int vlistID1;
+  int vlistID1{ CDI_UNDEFID };
   VarList varList1;
 
-  int maxRecords;
-  std::vector<RecordInfo> recordList;
+  int maxFields;
+  std::vector<FieldInfo> fieldInfoList;
 
   cdo::StepStat3D stepStat;
 
@@ -96,8 +96,8 @@ public:
     streamID2 = cdo_open_write(1);
     cdo_def_vlist(streamID2, vlistID2);
 
-    maxRecords = varList1.numRecords();
-    recordList = std::vector<RecordInfo>(maxRecords);
+    maxFields = varList1.maxFields();
+    fieldInfoList = std::vector<FieldInfo>(maxFields);
   }
 
   void
@@ -127,8 +127,8 @@ public:
     int otsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         if (numSteps > 1) progress.update((tsID + 1.0) / numSteps);
 
@@ -143,12 +143,12 @@ public:
         if (!stepStat.var1(stepIndex).size()) { stepStat.alloc(stepIndex, varList1, VARS_MEMTYPE); }
 
         auto numSets = rangeNumSets[stepIndex];
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            auto [varID, levelID] = cdo_inq_record(streamID1);
-            if (tsID == 0) recordList[recID].set(varID, levelID);
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            if (tsID == 0) fieldInfoList[fieldID].set(varID, levelID);
             field.init(varList1.vars[varID]);
-            cdo_read_record(streamID1, field);
+            cdo_read_field(streamID1, field);
             stepStat.add_field(field, stepIndex, varID, levelID, numSets);
           }
 
@@ -161,18 +161,18 @@ public:
         auto numSets = rangeNumSets[stepIndex];
         if (numSets)
           {
-            cdo::records_process_3D(stepIndex, recordList, varList1, stepStat, numSets);
+            cdo::fields_process_3D(stepIndex, fieldInfoList, varList1, stepStat, numSets);
 
             dtLists[stepIndex].stat_taxis_def_timestep(taxisID2);
             cdo_def_timestep(streamID2, otsID);
 
-            for (int recID = 0; recID < maxRecords; ++recID)
+            for (int fieldID = 0; fieldID < maxFields; ++fieldID)
               {
-                auto [varID, levelID] = recordList[recID].get();
+                auto [varID, levelID] = fieldInfoList[fieldID].get();
                 if (otsID && varList1.vars[varID].isConstant) continue;
 
-                cdo_def_record(streamID2, varID, levelID);
-                cdo_write_record(streamID2, stepStat.var1(stepIndex, varID, levelID));
+                cdo_def_field(streamID2, varID, levelID);
+                cdo_write_field(streamID2, stepStat.var1(stepIndex, varID, levelID));
               }
 
             otsID++;
