@@ -26,46 +26,45 @@
 #include "printinfo.h"
 #include "field_functions.h"
 
-class ModuleYseaspctl
+class Yseaspctl : public Process
 {
+  enum
+  {
+    MaxSeasons = 4
+  };
+  long numSets[MaxSeasons] = { 0 };
+
+public:
+  using Process::Process;
+  inline static CdoModule module = {
+    .name = "Yseaspctl",
+    .operators = { { "yseaspctl", FieldFunc_Pctl, 0, YseaspctlHelp } },
+    .aliases = {},
+    .mode = EXPOSED,     // Module mode: 0:intern 1:extern
+    .number = CDI_REAL,  // Allowed number type
+    .constraints = { 3, 1, NoRestriction },
+  };
+  inline static RegisterEntry<Yseaspctl> registration = RegisterEntry<Yseaspctl>(module);
+
   CdoStreamID streamID1;
   CdoStreamID streamID2;
   CdoStreamID streamID3;
   CdoStreamID streamID4;
-  FieldVector constFields;
 
-  int taxisID1;
-  int taxisID2;
+  int vlistID1{ CDI_UNDEFID };
+  int taxisID1{ CDI_UNDEFID };
+  int taxisID2{ CDI_UNDEFID };
   int taxisID3;
   int taxisID4;
 
-  VarList varList1;
-  Field field1, field2;
-
-  enum { MaxSeasons = 4 };
-  long numSets[MaxSeasons] = { 0 };
-  CdiDateTime vDateTimes1[MaxSeasons]{};
-  CdiDateTime vDateTimes2[MaxSeasons]{};
-
-  std::vector<bool> vars1;
-  HistogramSet hsets[MaxSeasons];
-  std::vector<RecordInfo> recList;
-
-  int maxrecs;
-  int ntsteps;
-  int nvars;
   int pn;
 
+  VarList varList1;
+
 public:
   void
-  init(void *process)
+  init() override
   {
-    vars1 = std::vector<bool>(MaxSeasons, false);
-
-    cdo_initialize(process);
-
-    cdo_operator_add("yseaspctl", FieldFunc_Pctl, 0, nullptr);
-
     operator_input_arg("percentile number");
     pn = parameter_to_double(cdo_operator_argv(0));
 
@@ -73,15 +72,19 @@ public:
     streamID2 = cdo_open_read(1);
     streamID3 = cdo_open_read(2);
 
-    auto vlistID1 = cdo_stream_inq_vlist(streamID1);
+    vlistID1 = cdo_stream_inq_vlist(streamID1);
     auto vlistID2 = cdo_stream_inq_vlist(streamID2);
     auto vlistID3 = cdo_stream_inq_vlist(streamID3);
     auto vlistID4 = vlistDuplicate(vlistID1);
 
     vlist_unpack(vlistID4);
 
-    vlist_compare(vlistID1, vlistID2, CmpVlist::All);
-    vlist_compare(vlistID1, vlistID3, CmpVlist::All);
+    varList1 = VarList(vlistID1);
+    VarList varList2(vlistID2);
+    VarList varList3(vlistID3);
+
+    varList_compare(varList1, varList2);
+    varList_compare(varList1, varList3);
 
     taxisID1 = vlistInqTaxis(vlistID1);
     taxisID2 = vlistInqTaxis(vlistID2);
@@ -94,29 +97,32 @@ public:
 
     streamID4 = cdo_open_write(3);
     cdo_def_vlist(streamID4, vlistID4);
-
-    ntsteps = vlistNtsteps(vlistID1);
-    nvars = vlistNvars(vlistID1);
-
-    maxrecs = vlistNrecs(vlistID1);
-    recList = std::vector<RecordInfo>(maxrecs);
-
-    constFields = FieldVector(maxrecs);
-
-    varList_init(varList1, vlistID1);
   }
 
   void
-  run()
+  run() override
   {
+    Field field1, field2;
+    std::vector<bool> seasVars(MaxSeasons, false);
+    CdiDateTime vDateTimes1[MaxSeasons]{};
+    CdiDateTime vDateTimes2[MaxSeasons]{};
+    HistogramSet hsets[MaxSeasons];
+
+    auto numVars = varList1.numVars();
+    auto numSteps = varList1.numSteps();
+    auto maxFields = varList1.maxFields();
+    std::vector<FieldInfo> fieldInfoList(maxFields);
+
+    FieldVector constFields(maxFields);
+
     int tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID2, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID2, tsID);
+        if (numFields == 0) break;
 
-        if (nrecs != cdo_stream_inq_timestep(streamID3, tsID))
-          cdo_abort("Number of records at time step %d of %s and %s differ!", tsID + 1, cdo_get_stream_name(1),
+        if (numFields != cdo_stream_inq_timestep(streamID3, tsID))
+          cdo_abort("Number of fields at time step %d of %s and %s differ!", tsID + 1, cdo_get_stream_name(1),
                     cdo_get_stream_name(2));
 
         auto vDateTime = taxisInqVdatetime(taxisID2);
@@ -131,28 +137,23 @@ public:
 
         set_date_time(vDateTimes2[seas], vDateTime);
 
-        if (!vars1[seas])
+        if (!seasVars[seas])
           {
-            vars1[seas] = true;
-            hsets[seas].create(nvars, ntsteps);
-            for (int varID = 0; varID < nvars; ++varID)
-              {
-                const auto &var = varList1[varID];
-                hsets[seas].createVarLevels(varID, var.nlevels, var.gridsize);
-              }
+            seasVars[seas] = true;
+            hsets[seas].create(numVars, numSteps);
+            for (auto const &var : varList1.vars) hsets[seas].createVarLevels(var.ID, var.nlevels, var.gridsize);
           }
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID2, &varID, &levelID);
-            const auto &var = varList1[varID];
+            auto [varID, levelID] = cdo_inq_field(streamID2);
+            auto const &var = varList1.vars[varID];
             field1.init(var);
-            cdo_read_record(streamID2, field1);
+            cdo_read_field(streamID2, field1);
 
-            cdo_inq_record(streamID3, &varID, &levelID);
+            (void) cdo_inq_field(streamID3);
             field2.init(var);
-            cdo_read_record(streamID3, field2);
+            cdo_read_field(streamID3, field2);
 
             hsets[seas].defVarLevelBounds(varID, levelID, field1, field2);
           }
@@ -163,8 +164,8 @@ public:
     tsID = 0;
     while (true)
       {
-        auto nrecs = cdo_stream_inq_timestep(streamID1, tsID);
-        if (nrecs == 0) break;
+        auto numFields = cdo_stream_inq_timestep(streamID1, tsID);
+        if (numFields == 0) break;
 
         auto vDateTime = taxisInqVdatetime(taxisID1);
 
@@ -175,25 +176,24 @@ public:
 
         set_date_time(vDateTimes1[seas], vDateTime);
 
-        if (!vars1[seas]) cdo_abort("No data for season %d in %s and %s", seas, cdo_get_stream_name(1), cdo_get_stream_name(2));
+        if (!seasVars[seas]) cdo_abort("No data for season %d in %s and %s", seas, cdo_get_stream_name(1), cdo_get_stream_name(2));
 
-        for (int recID = 0; recID < nrecs; ++recID)
+        for (int fieldID = 0; fieldID < numFields; ++fieldID)
           {
-            int varID, levelID;
-            cdo_inq_record(streamID1, &varID, &levelID);
-            const auto &var = varList1[varID];
+            auto [varID, levelID] = cdo_inq_field(streamID1);
+            auto const &var = varList1.vars[varID];
 
-            if (tsID == 0) recList[recID].set(varID, levelID);
+            if (tsID == 0) fieldInfoList[fieldID].set(varID, levelID);
 
             if (tsID == 0 && var.isConstant)
               {
-                constFields[recID].init(var);
-                cdo_read_record(streamID1, constFields[recID]);
+                constFields[fieldID].init(var);
+                cdo_read_field(streamID1, constFields[fieldID]);
               }
             else
               {
                 field1.init(var);
-                cdo_read_record(streamID1, field1);
+                cdo_read_field(streamID1, field1);
 
                 hsets[seas].addVarLevelValues(varID, levelID, field1);
               }
@@ -214,19 +214,20 @@ public:
           taxisDefVdatetime(taxisID4, vDateTimes1[seas]);
           cdo_def_timestep(streamID4, otsID);
 
-          for (int recID = 0; recID < maxrecs; ++recID)
+          for (int fieldID = 0; fieldID < maxFields; ++fieldID)
             {
-              auto [varID, levelID] = recList[recID].get();
-              if (otsID && varList1[varID].isConstant) continue;
+              auto [varID, levelID] = fieldInfoList[fieldID].get();
+              auto const &var = varList1.vars[varID];
+              if (otsID && var.isConstant) continue;
 
-              cdo_def_record(streamID4, varID, levelID);
+              cdo_def_field(streamID4, varID, levelID);
 
-              if (varList1[varID].isConstant) { cdo_write_record(streamID4, constFields[recID]); }
+              if (var.isConstant) { cdo_write_field(streamID4, constFields[fieldID]); }
               else
                 {
-                  field1.init(varList1[varID]);
+                  field1.init(var);
                   hsets[seas].getVarLevelPercentiles(field1, varID, levelID, pn);
-                  cdo_write_record(streamID4, field1);
+                  cdo_write_field(streamID4, field1);
                 }
             }
 
@@ -235,24 +236,11 @@ public:
   }
 
   void
-  close()
+  close() override
   {
     cdo_stream_close(streamID4);
     cdo_stream_close(streamID3);
     cdo_stream_close(streamID2);
     cdo_stream_close(streamID1);
-
-    cdo_finish();
   }
 };
-
-void *
-Yseaspctl(void *process)
-{
-  ModuleYseaspctl yseaspctl;
-  yseaspctl.init(process);
-  yseaspctl.run();
-  yseaspctl.close();
-
-  return nullptr;
-}
