@@ -1,7 +1,7 @@
 #include "cdo_fftw3.h"
 
 #include "cdo_options.h"
-#include "cimdOmp.h"
+#include "cdo_omp.h"
 #include <cdi.h>
 
 #ifdef HAVE_LIBFFTW3
@@ -11,7 +11,7 @@ static std::mutex fftwMutex;
 
 #ifdef HAVE_LIBFFTW3
 void
-fourier2grid(int gridID1, const Varray<double> &array1, Varray<double> &array2)
+fourier2grid(int gridID1, Varray<double> const &array1, Varray<double> &array2)
 {
   auto nlon = gridInqXsize(gridID1);
   auto nlat = gridInqYsize(gridID1);
@@ -23,9 +23,9 @@ fourier2grid(int gridID1, const Varray<double> &array1, Varray<double> &array2)
     fftw_plan plan;
   };
 
-  std::vector<FourierMemory> ompmem(Threading::ompNumThreads);
+  std::vector<FourierMemory> ompmem(Threading::ompNumMaxThreads);
 
-  for (int i = 0; i < Threading::ompNumThreads; ++i)
+  for (int i = 0; i < Threading::ompNumMaxThreads; ++i)
     {
       ompmem[i].in_fft = fftw_alloc_complex(nlon);
       ompmem[i].out_fft = (double *) fftw_malloc(nlon * sizeof(double));
@@ -42,7 +42,7 @@ fourier2grid(int gridID1, const Varray<double> &array1, Varray<double> &array2)
     {
       auto ompthID = cdo_omp_get_thread_num();
       auto in_fft = ompmem[ompthID].in_fft;
-      auto out_fft = ompmem[ompthID].out_fft;
+      const auto out_fft = ompmem[ompthID].out_fft;
 
       for (size_t ifc = 0; ifc < nlon; ++ifc)
         {
@@ -55,16 +55,17 @@ fourier2grid(int gridID1, const Varray<double> &array1, Varray<double> &array2)
       for (size_t ilon = 0; ilon < nlon; ++ilon) array2[ilat * nlon + ilon] = out_fft[ilon];
     }
 
-  for (int i = 0; i < Threading::ompNumThreads; ++i)
+  for (int i = 0; i < Threading::ompNumMaxThreads; ++i)
     {
       fftw_free(ompmem[i].in_fft);
       fftw_free(ompmem[i].out_fft);
+      std::scoped_lock lock(fftwMutex);
       fftw_destroy_plan(ompmem[i].plan);
     }
 }
 
 void
-grid2fourier(int gridID1, const Varray<double> &array1, int gridID2, Varray<double> &array2)
+grid2fourier(int gridID1, Varray<double> const &array1, int gridID2, Varray<double> &array2)
 {
   (void) gridID2;
   auto nlon = gridInqXsize(gridID1);
@@ -78,9 +79,9 @@ grid2fourier(int gridID1, const Varray<double> &array1, int gridID2, Varray<doub
     fftw_plan plan;
   };
 
-  std::vector<FourierMemory> ompmem(Threading::ompNumThreads);
+  std::vector<FourierMemory> ompmem(Threading::ompNumMaxThreads);
 
-  for (int i = 0; i < Threading::ompNumThreads; ++i)
+  for (int i = 0; i < Threading::ompNumMaxThreads; ++i)
     {
       ompmem[i].in_fft = (double *) fftw_malloc(nlon * sizeof(double));
       ompmem[i].out_fft = fftw_alloc_complex(nlon);
@@ -110,13 +111,15 @@ grid2fourier(int gridID1, const Varray<double> &array1, int gridID2, Varray<doub
         }
     }
 
-  for (int i = 0; i < Threading::ompNumThreads; ++i)
+  for (int i = 0; i < Threading::ompNumMaxThreads; ++i)
     {
       fftw_free(ompmem[i].in_fft);
       fftw_free(ompmem[i].out_fft);
+      std::scoped_lock lock(fftwMutex);
       fftw_destroy_plan(ompmem[i].plan);
     }
 }
+
 void
 filter_fftw(int nts, const std::vector<int> &fmasc, fftw_complex *fft_out, fftw_plan *p_T2S, fftw_plan *p_S2T)
 {
@@ -133,10 +136,12 @@ filter_fftw(int nts, const std::vector<int> &fmasc, fftw_complex *fft_out, fftw_
 
   return;
 }
+
 #else
+
 #include "cdo_output.h"
 void
-fourier2grid(int gridID1, const Varray<double> &array1, Varray<double> &array2)
+fourier2grid(int gridID1, Varray<double> const &array1, Varray<double> &array2)
 {
   (void) gridID1;
   (void) array1;
@@ -145,7 +150,7 @@ fourier2grid(int gridID1, const Varray<double> &array1, Varray<double> &array2)
 }
 
 void
-grid2fourier(int gridID1, const Varray<double> &array1, int gridID2, Varray<double> &array2)
+grid2fourier(int gridID1, Varray<double> const &array1, int gridID2, Varray<double> &array2)
 {
   (void) gridID1;
   (void) gridID2;
