@@ -55,7 +55,7 @@ get_width()
 }
 
 std::string
-CLIOptions::pad_size_terminal(char padWith, const std::string &sourround)
+CLIOptions::pad_size_terminal(char padWith, std::string const &sourround)
 {
   int terminal_width = get_width();
 
@@ -96,6 +96,8 @@ CLIOptions::parse(std::vector<std::string> p_argv)
     {
       Debug(CLIOP_DBG, "Checking: %s", p_argv[i]);
       const std::map<std::string, std::shared_ptr<cdo_option_2>>::iterator &it = optionMap.find(p_argv[i]);
+
+
       if (it == optionMap.end())
         {
           std::string arg = p_argv[i];
@@ -111,18 +113,7 @@ CLIOptions::parse(std::vector<std::string> p_argv)
           i++;
 
           bool out_of_bounds = i >= p_argv.size();
-
-          auto is_new_option = [&]() {
-            /*
-             * extracted into labmda for a balance of readability and preventing the seqfault that is caused
-             * by checking for this when we are out of bounds. By delaying execution we prevent the seqfault
-             */
-            bool is_not_negative_number = !std::get<0>(tokenize_comma_seperated_int_list(p_argv[i]));
-            bool has_minus_in_first_pos = (p_argv[i][0] == '-');
-            return has_minus_in_first_pos && is_not_negative_number;
-          };
-
-          if (out_of_bounds || is_new_option())
+          if (out_of_bounds || optionMap.find(p_argv[i]) != optionMap.end())
             {
               if (argument.default_value.size() > 0) { argument.value = argument.default_value; }
               else if (!argument.is_optional)
@@ -177,19 +168,19 @@ CLIOptions::get_env_vars()
 void
 CLIOptions::print_registry(const std::vector<std::shared_ptr<cdo_option_2>> &p_registry)
 {
-  for (const auto &it : p_registry)
+  for (auto const &it : p_registry)
     {
       if (it->argument.value.size() > 0) fprintf(stderr, "%s = %s\n", it->name.c_str(), it->argument.value.c_str());
     }
 }
 
 std::shared_ptr<cdo_option_2> &
-CLIOptions::envvar(const std::string &p_name)
+CLIOptions::envvar(std::string const &p_name)
 {
   if (envvarMap.find(p_name) == envvarMap.end())
     {
       envvarRegistry.push_back(std::make_shared<cdo_option_2>());
-      const auto &newEnvVar = envvarRegistry.back();
+      auto const &newEnvVar = envvarRegistry.back();
       envvarMap[p_name] = newEnvVar;
       newEnvVar->name = p_name;
     }
@@ -198,7 +189,7 @@ CLIOptions::envvar(const std::string &p_name)
 }
 
 std::shared_ptr<cdo_option_2> &
-CLIOptions::option(const std::string &p_name, const std::string &p_shortForm)
+CLIOptions::option(std::string const &p_name, std::string const &p_shortForm)
 {
   std::string name = "--" + p_name;
   std::string shortForm = p_shortForm;
@@ -226,7 +217,7 @@ std::string
 CLIOptions::print_envvar_help()
 {
   std::stringstream helpString;
-  for (const auto &it : envvarMap)
+  for (auto const &it : envvarMap)
     {
       if (!it.second->isInternal)
         {
@@ -236,14 +227,14 @@ CLIOptions::print_envvar_help()
           if (it.second->hasArgument) helpString << " <" + it.second->argument.description + "> ";
           int spaceLeft = padding - helpString.str().size() + len0;
           if (spaceLeft < 0) spaceLeft = 0;
-          for (const auto &line : it.second->description) { helpString << std::string(spaceLeft, ' ') + line + "\n"; }
+          for (auto const &line : it.second->description) { helpString << std::string(spaceLeft, ' ') + line + "\n"; }
         }
     }
   return helpString.str();
 }
 
 std::string
-CLIOptions::print_options_help(const std::string &p_category)
+CLIOptions::print_options_help(std::string const &p_category)
 {
   std::stringstream help;
   for (auto &iter : optionMap)
@@ -267,7 +258,7 @@ CLIOptions::print_options_help(const std::string &p_category)
               helpString += "\n";
               spaceLeft = padding;
             }
-          for (const auto &line : option->description)
+          for (auto const &line : option->description)
             {
               helpString += std::string(spaceLeft, ' ') + line + "\n";
               spaceLeft = padding;
@@ -282,7 +273,7 @@ CLIOptions::print_options_help(const std::string &p_category)
 #include <algorithm>
 
 cdo_option_2 *
-CLIOptions::option_from_envvar(const std::string &p_envvarName)
+CLIOptions::option_from_envvar(std::string const &p_envvarName)
 {
   if (envvarMap.find(p_envvarName) == envvarMap.end()) { cdo_abort("Error envvar %s does not exist", p_envvarName); }
   std::string ENVVAR_SUFFIX = "CDO_";
@@ -315,7 +306,7 @@ CLIOptions::option_from_envvar(const std::string &p_envvarName)
 void
 CLIOptions::print_available_options()
 {
-  for (const auto &iter : optionMap) { std::cerr << iter.first << std::endl; }
+  for (auto const &iter : optionMap) { std::cerr << iter.first << std::endl; }
   std::cerr << "_---------------------------------_" << std::endl;
   for (auto &iter : optionMap)
     {
