@@ -22,11 +22,12 @@ extern unsigned CDO_DEBUG;
 extern unsigned FACTORY;
 extern unsigned KVLIST;
 extern unsigned MODULE_INFO;
+extern unsigned ARGUMENTS;
 
 extern std::string debug_option_string;
 
 void print_debug_options();
-void query_user_exit(const char *argument);
+void query_user_exit(std::string const &argument);
 
 namespace cdo
 {
@@ -40,12 +41,12 @@ void set_exit_function(void (*func)(std::string msg));
 void set_context_function(const char *(*func)(void) );
 }  // namespace cdo
 
-void cdi_open_error(int cdiErrno, const std::string &format, const char *path);
+void cdi_open_error(int cdiErrno, std::string const &format, const char *path);
 std::string cdo_argv_to_string(const std::vector<std::string> &argv);
 
 template <typename... Args>
 void
-cdo_abort(const std::string &format, Args const &...args)
+cdo_abort(std::string const &format, Args const &...args)
 {
   fflush(stdout);
   std::string errmsg = MpMO::PrintCerr(Red("\n%s (Abort): ") + format, cdo::getContext(), args...);
@@ -54,7 +55,7 @@ cdo_abort(const std::string &format, Args const &...args)
 
 template <typename... Args>
 void
-cdo_error(const std::string &format, Args const &...args) noexcept
+cdo_error(std::string const &format, Args const &...args) noexcept
 {
   fflush(stdout);
   MpMO::PrintCerr(Red("\n%s (Abort): ") + format, cdo::getContext(), args...);
@@ -63,7 +64,7 @@ cdo_error(const std::string &format, Args const &...args) noexcept
 
 template <typename... Args>
 void
-cdo_warning(const std::string &format, Args const &...args) noexcept
+cdo_warning(std::string const &format, Args const &...args) noexcept
 {
   if (MpMO::warningsEnabled)
     {
@@ -78,14 +79,14 @@ cdo_warning(const std::string &format, Args const &...args) noexcept
 
 template <typename... Args>
 void
-cdo_verbose(const std::string &format, Args const &...args) noexcept
+cdo_verbose(std::string const &format, Args const &...args) noexcept
 {
   MpMO::PrintCerr(Green("%s: ") + format, cdo::getContext(), args...);
 }
 
 template <typename... Args>
 void
-cdo_print(const std::string &format, Args const &...args) noexcept
+cdo_print(std::string const &format, Args const &...args) noexcept
 {
   if (!MpMO::silentMode) MpMO::Print(Green("%s: ") + format, cdo::getContext(), args...);
 }
