@@ -15,12 +15,12 @@
 #include "varray.h"
 
 int
-cdo_read_timestepmask(const std::string &maskfile, std::vector<bool> &imask)
+cdo_read_timestepmask(std::string const &maskfile, std::vector<bool> &imask)
 {
   auto streamID = stream_open_read_locked(maskfile.c_str());
   auto vlistID = streamInqVlist(streamID);
   VarList varList(vlistID);
-  const auto &var0 = varList.vars[0];
+  auto const &var0 = varList.vars[0];
 
   if (varList.numVars() > 1) cdo_abort("timestepmask %s contains more than one variable!", maskfile);
   if (var0.nlevels > 1) cdo_abort("timestepmask %s has more than one level!", maskfile);
@@ -45,16 +45,16 @@ cdo_read_timestepmask(const std::string &maskfile, std::vector<bool> &imask)
   int tsID = 0;
   while (true)
     {
-      auto nrecs = streamInqTimestep(streamID, tsID);
-      if (nrecs == 0) break;
+      auto numFields = streamInqTimestep(streamID, tsID);
+      if (numFields == 0) break;
 
-      if (nrecs != 1) cdo_abort("Internal error; unexprected number of records!");
+      if (numFields != 1) cdo_abort("Internal error; unexprected number of fields!");
 
       int varID, levelID;
       size_t numMissVals;
       double value;
-      streamInqRecord(streamID, &varID, &levelID);
-      streamReadRecord(streamID, &value, &numMissVals);
+      streamInqField(streamID, &varID, &levelID);
+      streamReadField(streamID, &value, &numMissVals);
 
       imask[tsID] = !(numMissVals || is_equal(value, 0));
 
@@ -72,18 +72,18 @@ read_one_field(const char *text, const char *filename, Varray<double> &array)
   auto streamID = stream_open_read_locked(filename);
   auto vlistID = streamInqVlist(streamID);
   VarList varList(vlistID);
-  const auto &var0 = varList.vars[0];
+  auto const &var0 = varList.vars[0];
 
   if (varList.numVars() > 1) cdo_abort("%s file %s contains more than one variable!", text, filename);
   if (var0.nlevels > 1) cdo_abort("%s file %s has more than one level!", text, filename);
-  if (varList.numRecords() != 1) cdo_abort("%s file %s contains more than one field!", text, filename);
+  if (varList.maxFields() != 1) cdo_abort("%s file %s contains more than one field!", text, filename);
 
   array.resize(var0.gridsize);
 
   int varID, levelID;
   size_t numMissVals;
-  streamInqRecord(streamID, &varID, &levelID);
-  streamReadRecord(streamID, array.data(), &numMissVals);
+  streamInqField(streamID, &varID, &levelID);
+  streamReadField(streamID, array.data(), &numMissVals);
   streamClose(streamID);
 }
 
