@@ -112,6 +112,8 @@ compare_lon_reg2d(size_t xsize, int gridID1, int gridID2)
 static void
 compare_grid_unstructured(int gridID1, int gridID2)
 {
+  if (gridID1 == gridID2) return;
+
   if (gridInqXvals(gridID1, nullptr) && gridInqXvals(gridID1, nullptr) == gridInqXvals(gridID2, nullptr)
       && gridInqYvals(gridID1, nullptr) && gridInqYvals(gridID1, nullptr) == gridInqYvals(gridID2, nullptr))
     {
@@ -236,12 +238,7 @@ vlist_compare_x(int vlistID1, int vlistID2, int cmpFlag)
         }
     }
 
-  if (cmpFlag & CmpVarList::Grid)
-    {
-      auto gridID1 = varList1.vars[0].gridID;
-      auto gridID2 = varList2.vars[0].gridID;
-      if (gridID1 != gridID2) cdo_compare_grids(gridID1, gridID2);
-    }
+  if (cmpFlag & CmpVarList::Grid) { cdo_compare_grids(varList1.vars[0].gridID, varList2.vars[0].gridID); }
 
   return nlevels2;
 }
@@ -266,8 +263,8 @@ vlist_check_gridsize(int vlistID)
   auto ngp = gridInqSize(vlistGrid(vlistID, 0));
 
   // check gridsize
-  auto ngrids = vlistNumGrids(vlistID);
-  for (int index = 0; index < ngrids; ++index)
+  auto numGrids = vlistNumGrids(vlistID);
+  for (int index = 0; index < numGrids; ++index)
     {
       auto gridID = vlistGrid(vlistID, index);
       if (ngp != gridInqSize(gridID))
@@ -281,7 +278,7 @@ vlist_check_gridsize(int vlistID)
     {
       cdo_print("This operator requires all variables on the same horizontal grid.");
       cdo_print("Horizontal grids found:");
-      for (int index = 0; index < ngrids; ++index)
+      for (int index = 0; index < numGrids; ++index)
         {
           auto gridID = vlistGrid(vlistID, index);
           cdo_print("  grid=%d  type=%s  points=%zu", index + 1, gridNamePtr(gridInqType(gridID)), gridInqSize(gridID));
@@ -303,8 +300,8 @@ vlist_read_vct(int vlistID, int &zaxisID_ML, int &numHybridLevels, int &numFullL
   numHalfLevels = 0;
 
   auto haveVCT = false;
-  auto nzaxis = vlistNumZaxis(vlistID);
-  for (int iz = 0; iz < nzaxis; ++iz)
+  auto numZaxes = vlistNumZaxis(vlistID);
+  for (int iz = 0; iz < numZaxes; ++iz)
     {
       // auto monoLevel = false;
       auto monoLevel = true;
@@ -422,8 +419,8 @@ vlist_change_hybrid_zaxis(int vlistID1, int vlistID2, int zaxisID1, int zaxisID2
   int vctSize0 = 0;
   Varray<double> vct;
 
-  auto nzaxis = vlistNumZaxis(vlistID1);
-  for (int i = 0; i < nzaxis; ++i)
+  auto numZaxes = vlistNumZaxis(vlistID1);
+  for (int i = 0; i < numZaxes; ++i)
     {
       auto zaxisID = vlistZaxis(vlistID1, i);
       auto nlevels = zaxisInqSize(zaxisID);
@@ -441,7 +438,7 @@ vlist_change_hybrid_zaxis(int vlistID1, int vlistID2, int zaxisID1, int zaxisID2
             }
           else
             {
-              if (vctSize0 == vctSize && memcmp(vct.data(), zaxisInqVctPtr(zaxisID), vctSize * sizeof(double)) == 0)
+              if (vctSize0 == vctSize && std::memcmp(vct.data(), zaxisInqVctPtr(zaxisID), vctSize * sizeof(double)) == 0)
                 vlistChangeZaxisIndex(vlistID2, i, zaxisID2);
             }
         }
@@ -452,8 +449,8 @@ int
 vlist_get_first_spectral_grid(int vlistID)
 {
   // find first spectral grid
-  auto ngrids = vlistNumGrids(vlistID);
-  for (int index = 0; index < ngrids; ++index)
+  auto numGrids = vlistNumGrids(vlistID);
+  for (int index = 0; index < numGrids; ++index)
     {
       auto gridID = vlistGrid(vlistID, index);
       if (gridInqType(gridID) == GRID_SPECTRAL) return gridID;
@@ -466,8 +463,8 @@ int
 vlist_get_first_gaussian_grid(int vlistID)
 {
   // find first gaussian grid
-  auto ngrids = vlistNumGrids(vlistID);
-  for (int index = 0; index < ngrids; ++index)
+  auto numGrids = vlistNumGrids(vlistID);
+  for (int index = 0; index < numGrids; ++index)
     {
       auto gridID = vlistGrid(vlistID, index);
       if (gridInqType(gridID) == GRID_GAUSSIAN) return gridID;
@@ -480,8 +477,8 @@ int
 vlist_get_first_fourier_grid(int vlistID)
 {
   // find first fourier grid
-  auto ngrids = vlistNumGrids(vlistID);
-  for (int index = 0; index < ngrids; ++index)
+  auto numGrids = vlistNumGrids(vlistID);
+  for (int index = 0; index < numGrids; ++index)
     {
       auto gridID = vlistGrid(vlistID, index);
       if (gridInqType(gridID) == GRID_FOURIER) return gridID;
@@ -491,9 +488,9 @@ vlist_get_first_fourier_grid(int vlistID)
 }
 
 void
-cdo_check_missval(double missval, const std::string &varname)
+cdo_check_missval(double missval, std::string const &varname)
 {
-  if (dbl_is_equal(0.0, missval) || dbl_is_equal(1.0, missval))
+  if (fp_is_equal(0.0, missval) || fp_is_equal(1.0, missval))
     {
       static auto printWarning = true;
       if (printWarning)
