@@ -7,7 +7,7 @@
 #endif
 
 #ifdef HAVE_LIBNETCDF
-#include "netcdf.h"
+#include <netcdf.h>
 #endif
 
 #include "dcw_reader.h"
@@ -45,7 +45,7 @@ dcw_get_path(const char *name, const char *suffix, char *path)
 
   if (DCW_Dir)
     {
-      sprintf(path, "%s/%s%s", DCW_Dir, name, suffix);
+      std::snprintf(path, PATH_MAX, "%s/%s%s", DCW_Dir, name, suffix);
       if (access(path, R_OK) == 0) found = true;
     }
   else
@@ -73,7 +73,7 @@ dcw_load_lists(DCW_Lists &dcw_lists)
   size_t n_alloc = 300;
   unsigned int k, n;
   char line[BUFSIZ] = { 0 };
-  struct DCW_Country_State *Country_State = NULL;
+  // DCW_Country_State *Country_State = NULL;
 
   char path[PATH_MAX] = { 0 };
   if (!dcw_get_path("dcw-countries", ".txt", path)) return -1;
@@ -125,7 +125,7 @@ dcw_load_lists(DCW_Lists &dcw_lists)
     {
       if (line[0] == '#') continue;  // Skip comments
       std::sscanf(line, "%s %s %[^\n]", states[k].country, states[k].code, states[k].name);
-      if (k && strcmp(states[k].country, states[k - 1].country)) n++;  // New country with states
+      if (k && std::strcmp(states[k].country, states[k - 1].country)) n++;  // New country with states
       k++;
       if (k == n_alloc)
         {
@@ -141,17 +141,19 @@ dcw_load_lists(DCW_Lists &dcw_lists)
 
   // Get list of countries with states
 
-  dim[2] = n;    // Number of countries with states
-  if (0 /*CS*/)  // Wants list returned
-    {
-      Country_State = (DCW_Country_State *) malloc(n * sizeof(struct DCW_Country_State));
-      memcpy(Country_State[0].country, states[0].country, 4);
-      for (k = n = 1; k < dim[1]; ++k)
-        {
-          if (strcmp(states[k].country, states[k - 1].country)) memcpy(Country_State[n++].country, states[k].country, 4);
-        }
-      // *CS = Country_State;
-    }
+  dim[2] = n;  // Number of countries with states
+  /*
+  if (0)  // Wants list returned
+  {
+    Country_State = (DCW_Country_State *) malloc(n * sizeof(struct DCW_Country_State));
+    std::memcpy(Country_State[0].country, states[0].country, 4);
+    for (k = n = 1; k < dim[1]; ++k)
+      {
+        if (std::strcmp(states[k].country, states[k - 1].country)) std::memcpy(Country_State[n++].country, states[k].country, 4);
+      }
+    // *CS = Country_State;
+  }
+  */
 
   // fprintf(stderr, "# DCW: Found %u countries, %u countries with states, and %u states\n", dim[0], dim[2], dim[1]);
 
@@ -159,7 +161,7 @@ dcw_load_lists(DCW_Lists &dcw_lists)
 }
 
 static int
-dcw_find_country(const std::string &code, const std::vector<DCW_Country> &list)
+dcw_find_country(std::string const &code, const std::vector<DCW_Country> &list)
 {
   int low = 0, high = (int) list.size() - 1;
 
@@ -168,7 +170,7 @@ dcw_find_country(const std::string &code, const std::vector<DCW_Country> &list)
       auto midpoint = low + (high - low) / 2;
 
       // check to see if value is equal to item in array
-      auto way = strcmp(code.c_str(), list[midpoint].code);
+      auto way = std::strcmp(code.c_str(), list[midpoint].code);
       if (way == 0) return midpoint;
 
       if (way < 0)
@@ -182,9 +184,9 @@ dcw_find_country(const std::string &code, const std::vector<DCW_Country> &list)
 }
 
 static int
-dcw_find_state(const std::string &code, const std::vector<DCW_State> &list)
+dcw_find_state(std::string const &code, const std::vector<DCW_State> &list)
 {
-  auto country = code.substr(0,2);
+  auto country = code.substr(0, 2);
   auto state = code.substr(2);
   for (size_t i = 0, n = list.size(); i < n; ++i)
     {
@@ -197,7 +199,7 @@ dcw_find_state(const std::string &code, const std::vector<DCW_State> &list)
 
 #ifdef HAVE_LIBNETCDF
 static int
-nc_get_minmax(int ncid, const std::string &name, double &minval, double &maxval)
+nc_get_minmax(int ncid, std::string const &name, double &minval, double &maxval)
 {
   int varid;
   if (nc_inq_varid(ncid, name.c_str(), &varid)) return 1;
@@ -330,7 +332,7 @@ nc_get_data(int ncid, const char *name, std::vector<double> &data)
 */
 #ifdef HAVE_LIBNETCDF
 static int
-nc_get_lonlat(int ncid, const std::string &xname, const std::string &yname, std::vector<double> &x, std::vector<double> &y)
+nc_get_lonlat(int ncid, std::string const &xname, std::string const &yname, std::vector<double> &x, std::vector<double> &y)
 {
   int xvarid;
   double xscale, xmin;
@@ -478,7 +480,7 @@ dcw_print_lonlat(const DCW_Lists &dcw_lists, const std::vector<std::string> &cod
 static bool
 compare_code(const DCW_Country &a, const DCW_Country &b)
 {
-  return (strcmp(a.code, b.code) < 0);
+  return (std::strcmp(a.code, b.code) < 0);
 }
 
 std::vector<std::string>
@@ -543,7 +545,7 @@ dcw_print_polygons(const DCW_Lists &dcw_lists, const std::vector<std::string> &c
 void
 dcw_sort_countries(DCW_Lists &dcw_lists)
 {
-  std::sort(dcw_lists.countries.begin(), dcw_lists.countries.end(), compare_code);
+  std::ranges::sort(dcw_lists.countries, compare_code);
 }
 
 /*
