@@ -64,7 +64,7 @@ eca1(const ECA_REQUEST_1 &request)
   auto ovlistID = vlistCreate();
 
   VarList varList1(ivlistID);
-  const auto varFirst = varList1.vars[FIRST_VAR_ID];
+  auto const &varFirst = varList1.vars[FIRST_VAR_ID];
   auto gridID = varFirst.gridID;
   auto zaxisID = varFirst.zaxisID;
   auto missval = varFirst.missval;
@@ -96,8 +96,7 @@ eca1(const ECA_REQUEST_1 &request)
   //  taxisDefTunit(otaxisID, TUNIT_MINUTE);
   //  taxisDefCalendar(otaxisID, CALENDAR_PROLEPTIC);
   taxisDefCalendar(otaxisID, taxisInqCalendar(itaxisID));
-  taxisDefRdate(otaxisID, request.var1.refdate);
-  taxisDefRtime(otaxisID, 0);
+  taxisDefRdatetime(otaxisID, cdiDateTime_set(request.var1.refdate, 0));
   vlistDefTaxis(ovlistID, otaxisID);
 
   auto ostreamID = cdo_open_write(1);
@@ -133,12 +132,12 @@ eca1(const ECA_REQUEST_1 &request)
   otsID = 0;
   while (true)
     {
-      int nrecs = 0;
+      int numFields = 0;
       long numSets = 0;
       while (true)
         {
-          nrecs = cdo_stream_inq_timestep(istreamID, itsID);
-          if (nrecs == 0) break;
+          numFields = cdo_stream_inq_timestep(istreamID, itsID);
+          if (numFields == 0) break;
 
           auto ivDateTime = taxisInqVdatetime(itaxisID);
 
@@ -150,10 +149,10 @@ eca1(const ECA_REQUEST_1 &request)
               break;
             }
 
-          for (int recID = 0; recID < nrecs; ++recID)
+          for (int fieldID = 0; fieldID < numFields; ++fieldID)
             {
               int levelID;
-              cdo_inq_record(istreamID, &varID, &levelID);
+              cdo_inq_field(istreamID, &varID, &levelID);
 
               if (varID != FIRST_VAR_ID) continue;
 
@@ -176,7 +175,7 @@ eca1(const ECA_REQUEST_1 &request)
                   if (IS_SET(request.var2.h3)) var23[levelID].numMissVals = gridsize;
                 }
 
-              cdo_read_record(istreamID, field1);
+              cdo_read_field(istreamID, field1);
               field1.grid = var12[levelID].grid;
               field1.missval = var12[levelID].missval;
 
@@ -246,7 +245,7 @@ eca1(const ECA_REQUEST_1 &request)
                   if (request.var1.f2 == &vfarnum2)
                     {
                       auto &array1 = field3.vec_d;
-                      const auto &array2 = var12[levelID].vec_d;
+                      auto const &array2 = var12[levelID].vec_d;
                       auto missval2 = field2.missval;
 
                       auto len = field1.size;
@@ -282,11 +281,11 @@ eca1(const ECA_REQUEST_1 &request)
         {
           for (int levelID = 0; levelID < nlevels; ++levelID)
             {
-              if (nrecs == 0) { request.var1.f3(var13[levelID], var12[levelID]); }
+              if (numFields == 0) { request.var1.f3(var13[levelID], var12[levelID]); }
               else
                 {
                   auto &array1 = var13[levelID].vec_d;
-                  const auto &array2 = var12[levelID].vec_d;
+                  auto const &array2 = var12[levelID].vec_d;
                   auto missval2 = field2.missval;
                   auto len = field1.size;
 
@@ -298,7 +297,7 @@ eca1(const ECA_REQUEST_1 &request)
             }
         }
 
-      if (nrecs == 0 && numSets == 0) break;
+      if (numFields == 0 && numSets == 0) break;
 
       if (request.var1.epilog == MEAN || request.var1.epilog == PERCENT_OF_TIME)
         for (int levelID = 0; levelID < nlevels; ++levelID)
@@ -335,8 +334,8 @@ eca1(const ECA_REQUEST_1 &request)
 
           vfarsel(rvar, samp1[levelID]);
 
-          cdo_def_record(ostreamID, varID, levelID);
-          cdo_write_record(ostreamID, rvar.vec_d.data(), rvar.numMissVals);
+          cdo_def_field(ostreamID, varID, levelID);
+          cdo_write_field(ostreamID, rvar);
         }
 
       if (IS_SET(request.var2.h2) || IS_SET(request.var2.h3))
@@ -348,12 +347,12 @@ eca1(const ECA_REQUEST_1 &request)
 
               vfarsel(rvar, samp1[levelID]);
 
-              cdo_def_record(ostreamID, varID, levelID);
-              cdo_write_record(ostreamID, rvar.vec_d.data(), rvar.numMissVals);
+              cdo_def_field(ostreamID, varID, levelID);
+              cdo_write_field(ostreamID, rvar);
             }
         }
 
-      if (nrecs == 0) break;
+      if (numFields == 0) break;
       otsID++;
     }
 
@@ -381,8 +380,8 @@ eca2(const ECA_REQUEST_2 &request)
 
   VarList varList1(ivlistID1);
   VarList varList2(ivlistID2);
-  const auto var1First = varList1.vars[FIRST_VAR_ID];
-  const auto var2First = varList2.vars[FIRST_VAR_ID];
+  auto const &var1First = varList1.vars[FIRST_VAR_ID];
+  auto const &var2First = varList2.vars[FIRST_VAR_ID];
 
   varList_compare(varList1, varList2);
 
@@ -418,8 +417,7 @@ eca2(const ECA_REQUEST_2 &request)
   //  taxisDefTunit(otaxisID, TUNIT_MINUTE);
   //  taxisDefCalendar(otaxisID, CALENDAR_PROLEPTIC);
   taxisDefCalendar(otaxisID, taxisInqCalendar(itaxisID1));
-  taxisDefRdate(otaxisID, request.var1.refdate);
-  taxisDefRtime(otaxisID, 0);
+  taxisDefRdatetime(otaxisID, cdiDateTime_set(request.var1.refdate, 0));
   vlistDefTaxis(ovlistID, otaxisID);
 
   auto ostreamID = cdo_open_write(2);
@@ -457,8 +455,8 @@ eca2(const ECA_REQUEST_2 &request)
   itsID = 0;
   while (true)
     {
-      auto nrecs = cdo_stream_inq_timestep(istreamID2, itsID);
-      if (nrecs == 0) break;
+      auto numFields = cdo_stream_inq_timestep(istreamID2, itsID);
+      if (numFields == 0) break;
 
       auto ivDateTime = taxisInqVdatetime(vlistInqTaxis(ivlistID2));
 
@@ -467,13 +465,13 @@ eca2(const ECA_REQUEST_2 &request)
 
       if (!varsData2[dayOfYear].size()) field2D_init(varsData2[dayOfYear], varList2, FIELD_VEC);
 
-      for (int recID = 0; recID < nrecs; ++recID)
+      for (int fieldID = 0; fieldID < numFields; ++fieldID)
         {
           int levelID;
-          cdo_inq_record(istreamID2, &varID, &levelID);
+          cdo_inq_field(istreamID2, &varID, &levelID);
           if (varID != FIRST_VAR_ID) continue;
           auto &rvar = varsData2[dayOfYear][0][levelID];
-          cdo_read_record(istreamID2, rvar);
+          cdo_read_field(istreamID2, rvar);
         }
 
       itsID++;
@@ -484,12 +482,12 @@ eca2(const ECA_REQUEST_2 &request)
 
   while (true)
     {
-      int nrecs = 0;
+      int numFields = 0;
       long numSets = 0;
       while (true)
         {
-          nrecs = cdo_stream_inq_timestep(istreamID1, itsID);
-          if (nrecs == 0) break;
+          numFields = cdo_stream_inq_timestep(istreamID1, itsID);
+          if (numFields == 0) break;
 
           auto ivDateTime = taxisInqVdatetime(itaxisID1);
 
@@ -504,10 +502,10 @@ eca2(const ECA_REQUEST_2 &request)
               break;
             }
 
-          for (int recID = 0; recID < nrecs; ++recID)
+          for (int fieldID = 0; fieldID < numFields; ++fieldID)
             {
               int levelID;
-              cdo_inq_record(istreamID1, &varID, &levelID);
+              cdo_inq_field(istreamID1, &varID, &levelID);
 
               if (varID != FIRST_VAR_ID) continue;
 
@@ -532,7 +530,7 @@ eca2(const ECA_REQUEST_2 &request)
                   if (IS_SET(request.var2.h2)) var22[levelID].numMissVals = gridsize;
                 }
 
-              cdo_read_record(istreamID1, field1);
+              cdo_read_field(istreamID1, field1);
               field1.grid = gridID;
               field1.missval = missval1;
               field2.grid = varsData2[dayOfYear][0][levelID].grid;
@@ -590,7 +588,7 @@ eca2(const ECA_REQUEST_2 &request)
                   if (request.var1.f4 == &vfarnum2)
                     {
                       auto &array1 = field3.vec_d;
-                      const auto &array2 = var14[levelID].vec_d;
+                      auto const &array2 = var14[levelID].vec_d;
                       auto missvaltemp = field1.missval;
 
                       auto len = field1.size;
@@ -626,10 +624,10 @@ eca2(const ECA_REQUEST_2 &request)
         {
           for (int levelID = 0; levelID < nlevels; ++levelID)
             {
-              if (nrecs == 0) { request.var1.f5(var15[levelID], var14[levelID], request.var1.f5arg); }
+              if (numFields == 0) { request.var1.f5(var15[levelID], var14[levelID], request.var1.f5arg); }
               else
                 {
-                  const auto &array2 = var14[levelID].vec_d;
+                  auto const &array2 = var14[levelID].vec_d;
                   auto &array1 = var15[levelID].vec_d;
                   auto len = field1.size;
                   auto missvaltemp = field1.missval;
@@ -641,7 +639,7 @@ eca2(const ECA_REQUEST_2 &request)
             }
         }
 
-      if (nrecs == 0 && numSets == 0) break;
+      if (numFields == 0 && numSets == 0) break;
 
       if (request.var1.epilog == MEAN || request.var1.epilog == PERCENT_OF_TIME)
         for (int levelID = 0; levelID < nlevels; ++levelID)
@@ -687,8 +685,8 @@ eca2(const ECA_REQUEST_2 &request)
           vfarsel(rvar, samp1[levelID]);
           vfarsel(rvar, samp2[levelID]);
 
-          cdo_def_record(ostreamID, varID, levelID);
-          cdo_write_record(ostreamID, rvar.vec_d.data(), rvar.numMissVals);
+          cdo_def_field(ostreamID, varID, levelID);
+          cdo_write_field(ostreamID, rvar);
         }
 
       if (IS_SET(request.var2.h2))
@@ -701,12 +699,12 @@ eca2(const ECA_REQUEST_2 &request)
               vfarsel(rvar, samp1[levelID]);
               vfarsel(rvar, samp2[levelID]);
 
-              cdo_def_record(ostreamID, varID, levelID);
-              cdo_write_record(ostreamID, rvar.vec_d.data(), rvar.numMissVals);
+              cdo_def_field(ostreamID, varID, levelID);
+              cdo_write_field(ostreamID, rvar);
             }
         }
 
-      if (nrecs == 0) break;
+      if (numFields == 0) break;
 
       otsID++;
     }
@@ -736,7 +734,7 @@ eca3(const ECA_REQUEST_3 &request)
 
   VarList varList1(ivlistID1);
   VarList varList2(ivlistID2);
-  const auto var1First = varList1.vars[FIRST_VAR_ID];
+  auto const &var1First = varList1.vars[FIRST_VAR_ID];
 
   varList_compare(varList1, varList2);
 
@@ -761,8 +759,7 @@ eca3(const ECA_REQUEST_3 &request)
   //  taxisDefTunit(otaxisID, TUNIT_MINUTE);
   //  taxisDefCalendar(otaxisID, CALENDAR_PROLEPTIC);
   taxisDefCalendar(otaxisID, taxisInqCalendar(itaxisID1));
-  taxisDefRdate(otaxisID, request.refdate);
-  taxisDefRtime(otaxisID, 0);
+  taxisDefRdatetime(otaxisID, cdiDateTime_set(request.refdate, 0));
   vlistDefTaxis(ovlistID, otaxisID);
 
   auto ostreamID = cdo_open_write(2);
@@ -774,11 +771,11 @@ eca3(const ECA_REQUEST_3 &request)
   field1.resize(gridsize);
   field2.resize(gridsize);
 
-  const int nlevels = zaxisInqSize(zaxisID);
+  auto numLevels = zaxisInqSize(zaxisID);
 
-  FieldVector var1(nlevels), var2(nlevels);
+  FieldVector var1(numLevels), var2(numLevels);
 
-  for (int levelID = 0; levelID < nlevels; ++levelID)
+  for (int levelID = 0; levelID < numLevels; ++levelID)
     {
       init_field(var1[levelID], gridID, missval, gridsize);
       init_field(var2[levelID], gridID, missval, gridsize);
@@ -788,12 +785,12 @@ eca3(const ECA_REQUEST_3 &request)
   otsID = 0;
   while (true)
     {
-      int nrecs = 0;
+      int numFields = 0;
       long numSets = 0;
       while (true)
         {
-          nrecs = cdo_stream_inq_timestep(istreamID1, itsID);
-          if (nrecs == 0) break;
+          numFields = cdo_stream_inq_timestep(istreamID1, itsID);
+          if (numFields == 0) break;
 
           if (!cdo_stream_inq_timestep(istreamID2, itsID)) cdo_abort("Input streams have different number of time steps!");
 
@@ -816,11 +813,11 @@ eca3(const ECA_REQUEST_3 &request)
               break;
             }
 
-          for (int recID = 0; recID < nrecs; ++recID)
+          for (int fieldID = 0; fieldID < numFields; ++fieldID)
             {
               int levelID;
-              cdo_inq_record(istreamID1, &varID, &levelID);
-              cdo_inq_record(istreamID2, &varID, &levelID);
+              cdo_inq_field(istreamID1, &varID, &levelID);
+              cdo_inq_field(istreamID2, &varID, &levelID);
 
               if (varID != FIRST_VAR_ID) continue;
 
@@ -835,11 +832,11 @@ eca3(const ECA_REQUEST_3 &request)
                   var2[levelID].numMissVals = gridsize;
                 }
 
-              cdo_read_record(istreamID1, field1);
+              cdo_read_field(istreamID1, field1);
               field1.grid = var1[levelID].grid;
               field1.missval = var1[levelID].missval;
 
-              cdo_read_record(istreamID2, field2);
+              cdo_read_field(istreamID2, field2);
               field2.grid = var1[levelID].grid;
               field2.missval = var1[levelID].missval;
 
@@ -852,9 +849,9 @@ eca3(const ECA_REQUEST_3 &request)
           itsID++;
         }
 
-      if (nrecs == 0 && numSets == 0) break;
+      if (numFields == 0 && numSets == 0) break;
 
-      for (int levelID = 0; levelID < nlevels; ++levelID) request.f3(var1[levelID], var2[levelID]);
+      for (int levelID = 0; levelID < numLevels; ++levelID) request.f3(var1[levelID], var2[levelID]);
 
       if (request.refdate == 19550101)
         {
@@ -872,13 +869,13 @@ eca3(const ECA_REQUEST_3 &request)
       if (otsID && var1First.timeType == TIME_CONSTANT) continue;
 
       varID = 0;
-      for (int levelID = 0; levelID < nlevels; ++levelID)
+      for (int levelID = 0; levelID < numLevels; ++levelID)
         {
-          cdo_def_record(ostreamID, varID, levelID);
-          cdo_write_record(ostreamID, var1[levelID].vec_d.data(), var1[levelID].numMissVals);
+          cdo_def_field(ostreamID, varID, levelID);
+          cdo_write_field(ostreamID, var1[levelID]);
         }
 
-      if (nrecs == 0) break;
+      if (numFields == 0) break;
       otsID++;
     }
 
@@ -889,7 +886,7 @@ eca3(const ECA_REQUEST_3 &request)
 
 // check for non missval values
 static bool
-fldhvs(const FieldVector &fieldVector, const size_t nlevels)
+fldhvs(FieldVector const &fieldVector, size_t nlevels)
 {
   for (size_t level = 0; level < nlevels; level++)
     {
@@ -921,8 +918,8 @@ eca4(const ECA_REQUEST_4 &request)
 
   VarList varList1(ivlistID1);
   VarList varList2(ivlistID2);
-  const auto var1First = varList1.vars[FIRST_VAR_ID];
-  const auto var2First = varList2.vars[FIRST_VAR_ID];
+  auto const &var1First = varList1.vars[FIRST_VAR_ID];
+  auto const &var2First = varList2.vars[FIRST_VAR_ID];
 
   auto gridID = var1First.gridID;
   if (var1First.gridsize != var2First.gridsize) cdo_abort("Grid sizes of the input fields do not match!");
@@ -954,8 +951,7 @@ eca4(const ECA_REQUEST_4 &request)
   //  taxisDefTunit(otaxisID, TUNIT_MINUTE);
   //  taxisDefCalendar(otaxisID, CALENDAR_PROLEPTIC);
   taxisDefCalendar(otaxisID, taxisInqCalendar(itaxisID));
-  taxisDefRdate(otaxisID, 19550101);
-  taxisDefRtime(otaxisID, 0);
+  taxisDefRdatetime(otaxisID, cdiDateTime_set(19550101, 0));
   vlistDefTaxis(ovlistID, otaxisID);
 
   auto ostreamID = cdo_open_write(2);
@@ -1019,8 +1015,8 @@ eca4(const ECA_REQUEST_4 &request)
   if (cdo_stream_inq_timestep(istreamID2, itsID))
     {
       int levelID;
-      cdo_inq_record(istreamID2, &varID, &levelID);
-      cdo_read_record(istreamID2, mask);
+      cdo_inq_field(istreamID2, &varID, &levelID);
+      cdo_read_field(istreamID2, mask);
       mask.grid = gridID;
       mask.missval = var2First.missval;
 
@@ -1031,12 +1027,12 @@ eca4(const ECA_REQUEST_4 &request)
 
   while (true)
     {
-      int nrecs = 0;
+      int numFields = 0;
       long numSets = 0;
       while (true)
         {
-          nrecs = cdo_stream_inq_timestep(istreamID1, itsID);
-          if (nrecs == 0) break;
+          numFields = cdo_stream_inq_timestep(istreamID1, itsID);
+          if (numFields == 0) break;
 
           auto ivDateTime = taxisInqVdatetime(itaxisID);
           ivdate = cdiDate_get(ivDateTime.date);
@@ -1054,10 +1050,10 @@ eca4(const ECA_REQUEST_4 &request)
               break;
             }
 
-          for (int recID = 0; recID < nrecs; ++recID)
+          for (int fieldID = 0; fieldID < numFields; ++fieldID)
             {
               int levelID;
-              cdo_inq_record(istreamID1, &varID, &levelID);
+              cdo_inq_field(istreamID1, &varID, &levelID);
 
               if (varID != FIRST_VAR_ID) continue;
 
@@ -1085,7 +1081,7 @@ eca4(const ECA_REQUEST_4 &request)
                   endDateWithHist[1][levelID].numMissVals = gridsize;
                 }
 
-              cdo_read_record(istreamID1, fieldGt);
+              cdo_read_field(istreamID1, fieldGt);
               fieldLt.vec_d = fieldGt.vec_d;
               fieldLt.numMissVals = fieldGt.numMissVals;
               fieldGt.grid = startCount[levelID].grid;
@@ -1263,7 +1259,7 @@ eca4(const ECA_REQUEST_4 &request)
           itsID++;
         }
 
-      if (nrecs == 0 && numSets == 0) break;
+      if (numFields == 0 && numSets == 0) break;
 
       adjust_end_date(nlevels, gridsize, yvals, missval, ovdate, startDateWithHist, endDateWithHist);
 
@@ -1325,7 +1321,7 @@ eca4(const ECA_REQUEST_4 &request)
         }
       yearcnt++;
 
-      if (nrecs == 0) break;
+      if (numFields == 0) break;
     }
 
   cdo_stream_close(ostreamID);
