@@ -1,24 +1,16 @@
 /*
-  This file is part of CDO. CDO is a collection of Operators to
-  manipulate and analyse Climate model Data.
+  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.
 
   Copyright (C) 2006 Brockmann Consult
-  See COPYING file for copying and redistribution conditions.
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
+  Author: Ralf Quast
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
 */
-
 #ifndef ECAUTIL_H_
 #define ECAUTIL_H_
 
 #include "field.h"
+#include "cdoStream.h"
 
 /**
  * Computes the day-of-year correspnding a given Gregorian date.
@@ -30,7 +22,7 @@
 unsigned long day_of_year(int date);
 
 /**
- * Counts the number of nonmissing values in a field. The result
+ * Counts the number of nonumMissValsing values in a field. The result
  * of the operation is computed according to the following rules:
  *
  * field1  field2  result
@@ -42,10 +34,10 @@ unsigned long day_of_year(int date);
  * @param field1 the 1st input field, also holds the result
  * @param field2 the 2nd input field
  */
-void farnum(Field *field1, Field field2);
+void vfarnum(Field &field1, Field const &field2);
 
 /**
- * Counts the number of consecutive nonmissing values in a field.
+ * Counts the number of consecutive nonumMissValsing values in a field.
  * The result of the operation is computed according to the following
  * rules:
  *
@@ -58,11 +50,11 @@ void farnum(Field *field1, Field field2);
  * @param field1 the 1st input field, also holds the result
  * @param field2 the 2nd input field
  */
-void farnum2(Field *field1, Field field2);
+void vfarnum2(Field &field1, Field const &field2);
 
 /**
  * Counts the number of values in series of at least n consecutive
- * nonmissing values. The result of the operation is computed according
+ * nonumMissValsing values. The result of the operation is computed according
  * to the following rules:
  *
  * field1  field2  result
@@ -76,7 +68,7 @@ void farnum2(Field *field1, Field field2);
  * @param n      the number of consecutive values, must be an exact
  *               mathematical integer
  */
-void farnum3(Field *field1, Field field2, double n);
+void vfarnum3(Field &field1, Field const &field2, double n);
 
 /**
  * Selects field elements according to a given mask. The result of
@@ -91,7 +83,7 @@ void farnum3(Field *field1, Field field2, double n);
  * @param field1  the input field, also holds the result
  * @param field2  the mask
  */
-void farsel(Field *field1, Field field2);
+void vfarsel(Field &field1, Field const &field2);
 
 /**
  * Selects all field elements that are less than or equal to the
@@ -107,7 +99,7 @@ void farsel(Field *field1, Field field2);
  * @param field1 the input field, also holds the result
  * @param field2 the reference field
  */
-void farselle(Field *field1, Field field2);
+void vfarselle(Field &field1, Field const &field2);
 
 /**
  * Selects all field elements that are less than the
@@ -123,7 +115,7 @@ void farselle(Field *field1, Field field2);
  * @param field1 the input field, also holds the result
  * @param field2 the reference field
  */
-void farsellt(Field *field1, Field field2);
+void vfarsellt(Field &field1, Field const &field2);
 
 /**
  * Selects all field elements that are greater than or equal to
@@ -139,7 +131,7 @@ void farsellt(Field *field1, Field field2);
  * @param field1 the input field, also holds the result
  * @param field2 the reference field
  */
-void farselge(Field *field1, Field field2);
+void vfarselge(Field &field1, Field const &field2);
 
 /**
  * Selects all field elements that are greater than the
@@ -155,7 +147,7 @@ void farselge(Field *field1, Field field2);
  * @param field1 the input field, also holds the result
  * @param field2 the reference field
  */
-void farselgt(Field *field1, Field field2);
+void vfarselgt(Field &field1, Field const &field2);
 
 /**
  * Selects all field elements that are equal to the
@@ -171,7 +163,7 @@ void farselgt(Field *field1, Field field2);
  * @param field1 the input field, also holds the result
  * @param field2 the reference field
  */
-void farseleq(Field *field1, Field field2);
+void vfarseleq(Field &field1, Field const &field2);
 
 /**
  * Selects all field elements that are not equal to the
@@ -187,7 +179,7 @@ void farseleq(Field *field1, Field field2);
  * @param field1 the input field, also holds the result
  * @param field2 the reference field
  */
-void farselne(Field *field1, Field field2);
+void vfarselne(Field &field1, Field const &field2);
 
 /**
  * Selects all field elements that are less than or equal to a
@@ -203,7 +195,7 @@ void farselne(Field *field1, Field field2);
  * @param field the input field, also holds the result
  * @param c     the reference value
  */
-void farsellec(Field *field, double c);
+void vfarsellec(Field &field, double c);
 
 /**
  * Selects all field elements that are less a
@@ -219,7 +211,7 @@ void farsellec(Field *field, double c);
  * @param field the input field, also holds the result
  * @param c     the reference value
  */
-void farselltc(Field *field, double c);
+void vfarselltc(Field &field, double c);
 
 /**
  * Selects all field elements that are greater than or equal to a
@@ -235,7 +227,7 @@ void farselltc(Field *field, double c);
  * @param field the input field, also holds the result
  * @param c     the reference value
  */
-void farselgec(Field *field, double c);
+void vfarselgec(Field &field, double c);
 
 /**
  * Selects all field elements that are greater than a
@@ -251,7 +243,7 @@ void farselgec(Field *field, double c);
  * @param field the input field, also holds the result
  * @param c     the reference value
  */
-void farselgtc(Field *field, double c);
+void vfarselgtc(Field &field, double c);
 
 /**
  * Selects all field elements that are equal to a
@@ -267,7 +259,7 @@ void farselgtc(Field *field, double c);
  * @param field the input field, also holds the result
  * @param c     the reference value
  */
-void farseleqc(Field *field, double c);
+void vfarseleqc(Field &field, double c);
 
 /**
  * Selects all field elements that are not equal to a
@@ -283,7 +275,7 @@ void farseleqc(Field *field, double c);
  * @param field the input field, also holds the result
  * @param c     the reference value
  */
-void farselnec(Field *field, double c);
+void vfarselnec(Field &field, double c);
 
 /**
  * reset the fields real values to the missval for all levels
@@ -295,7 +287,7 @@ void farselnec(Field *field, double c);
  * @param yvals     list of latitudes
  * @param onlyNorth boolean for processing only the norther hemisphere
  */
-void updateHist(Field *field[2], int nlevels, size_t gridsize, double *yvals, int onlyNorth);
+void update_hist(FieldVector2D &field, int nlevels, size_t gridsize, const std::vector<double> &yvals, bool onlyNorth);
 
 /*
  * Compute the Gsl and its starting day
@@ -312,13 +304,13 @@ void updateHist(Field *field[2], int nlevels, size_t gridsize, double *yvals, in
  * @param Field *endDateWithHist[2]
  * @param Field *gslDuration
  * @param Field *gslFirstDay
- * @param int useCurrentYear = if TRUE, only measurements of the current year
+ * @param bool useCurrentYear = if true, only measurements of the current year
  *                             (index 0) are used for computation, i.e. that
  *                             gsl can only be computed for the northern
  *                             hemisphere (see definition of GSL: EcaGsl()
  */
-void computeGsl(int nlevels, size_t gridsize, double *yvals, double missval, Field *startDateWithHist[2],
-                Field *endDateWithHist[2], Field *gslDuration, Field *gslFirstDay, int useCurrentYear);
+void compute_gsl(int nlevels, size_t gridsize, std::vector<double> &yvals, double missval, FieldVector2D &startDateWithHist,
+                 FieldVector2D &endDateWithHist, FieldVector &gslDuration, FieldVector &gslFirstDay, bool useCurrentYear);
 
 /*
  * Adjust the endDates found in the current year:
@@ -326,11 +318,15 @@ void computeGsl(int nlevels, size_t gridsize, double *yvals, double missval, Fie
  * should be the last day of the corresponding year for norther and June, 30th
  * for southern hemisphere
  */
-void adjustEndDate(int nlevels, size_t gridsize, double *yvals, double missval, int64_t ovdate, Field *startDateWithHist[2],
-                   Field *endDateWithHist[2]);
+void adjust_end_date(int nlevels, size_t gridsize, const std::vector<double> &yvals, double missval, int64_t ovdate,
+                     const FieldVector2D &startDateWithHist, FieldVector2D &endDateWithHist);
+/*
+ * Calculates the mid of the year or month
+ */
+void define_mid_of_time(int frequency, int taxisID, int year, int month, int MaxMonths);
 /*
  * Write GSL related fields to an output stream
  */
-void writeGslStream(int ostreamID, int otaxisID, int otsID, int ovarID1, int ovarID2, int ivlistID1, int first_var_id,
-                    Field *gslDuration, Field *gslFirstDay, int64_t vdate, int vtime, int nlevels);
+void write_gsl_stream(CdoStreamID ostreamID, int otaxisID, int otsID, int ovarID1, int ovarID2, int ivlistID1, int first_var_id,
+                      FieldVector &gslDuration, FieldVector &gslFirstDay, CdiDateTime const &vDateTime, int nlevels);
 #endif /*ECAUTIL_H_*/
