@@ -10,9 +10,9 @@
 #include <cdi.h>
 
 #include "cdo_options.h"
-#include "process_int.h"
 #include "field_functions.h"
-#include "cimdOmp.h"
+#include "cdo_output.h"
+#include "cdo_omp.h"
 #include "interpol.h"
 #include "expr.h"
 #include "expr_fun.h"
@@ -173,8 +173,8 @@ static const ExprFuncEntry funcSymbolTable[] = {
   { FT_VERT, 1, "vertkurt",   reinterpret_cast<void (*)(void)>(&field_kurt) },
   { FT_VERT, 1, "vertmedian", reinterpret_cast<void (*)(void)>(&field_median) },
   // varOut=intgrid<METHOD>(varIn, gridOut)s
-  { FT_REMAP, 0, "intgridbil", reinterpret_cast<void (*)(void)>(&intgridbil) },
-  { FT_REMAP, 0, "intgridnn",  reinterpret_cast<void (*)(void)>(&intgridnn) },
+  { FT_REMAP, 0, "intgridbil", reinterpret_cast<void (*)(void)>(&intgrid_bil) },
+  { FT_REMAP, 0, "intgridnn",  reinterpret_cast<void (*)(void)>(&intgrid_1nn) },
   // coord=func(varName)
   { FT_COORD, 0, "clon",       nullptr },
   { FT_COORD, 0, "clat",       nullptr },
@@ -231,7 +231,7 @@ node_delete(nodeType *p)
 }
 
 static int
-get_funcID(const std::string &fun)
+get_funcID(std::string const &fun)
 {
   constexpr int funcTableSize = sizeof(funcSymbolTable) / sizeof(funcSymbolTable[0]);
   for (int funcID = 0; funcID < funcTableSize; funcID++)
@@ -278,8 +278,6 @@ expr_con_var(int init, int oper, const nodeType *p1, const nodeType *p2)
   auto datatype = p2->param.datatype;
   auto missval2 = p2->param.missval;
 
-  auto n = ngp * nlev;
-
   auto p = new nodeType;
 
   p->type = NodeEnum::typeVar;
@@ -290,6 +288,7 @@ expr_con_var(int init, int oper, const nodeType *p1, const nodeType *p2)
 
   if (!init)
     {
+      auto n = ngp * nlev;
       p->param.data = new double[n];
       auto odat = p->param.data;
       const auto idat = p2->param.data;
@@ -313,8 +312,6 @@ expr_var_con(int init, int oper, const nodeType *p1, const nodeType *p2)
   auto datatype = p1->param.datatype;
   auto missval1 = p1->param.missval;
 
-  auto n = ngp * nlev;
-
   auto p = new nodeType;
 
   p->type = NodeEnum::typeVar;
@@ -325,6 +322,7 @@ expr_var_con(int init, int oper, const nodeType *p1, const nodeType *p2)
 
   if (!init)
     {
+      auto n = ngp * nlev;
       p->param.data = new double[n];
       auto odat = p->param.data;
       const auto idat = p1->param.data;
@@ -480,7 +478,7 @@ ex_copy_var(int init, nodeType *p2, const nodeType *p1)
 
   if (!init)
     {
-      if (copyConstValue1) { ranges::fill_n(p2->param.data, ngp2 * nlev2, p1->param.data[0]); }
+      if (copyConstValue1) { std::ranges::fill_n(p2->param.data, ngp2 * nlev2, p1->param.data[0]); }
       else
         {
           array_copy(ngp2 * nlev2, p1->param.data, p2->param.data);
@@ -518,7 +516,7 @@ ex_copy_con(int init, nodeType *p2, const nodeType *p1)
   if (!init)
     {
       assert(p2->param.data != nullptr);
-      ranges::fill_n(p2->param.data, ngp * nlev, cval);
+      std::ranges::fill_n(p2->param.data, ngp * nlev, cval);
     }
 }
 
@@ -646,14 +644,14 @@ ex_fun_std(int funcID, size_t n, bool hasMV, double mv, const double *p1data, do
     {
       if (funcEntry.name == "isMissval")
         {
-          for (size_t i = 0; i < n; ++i) { pdata[i] = dbl_is_equal(p1data[i], mv); }
+          for (size_t i = 0; i < n; ++i) { pdata[i] = fp_is_equal(p1data[i], mv); }
         }
       else
         {
           for (size_t i = 0; i < n; ++i)
             {
               errno = -1;
-              pdata[i] = dbl_is_equal(p1data[i], mv) ? mv : exprfunc(p1data[i]);
+              pdata[i] = fp_is_equal(p1data[i], mv) ? mv : exprfunc(p1data[i]);
               if (errno == EDOM || errno == ERANGE || std::isnan(pdata[i])) pdata[i] = mv;
             }
         }
@@ -680,7 +678,7 @@ func_expr_con_var(int funcID, bool hasMV, size_t n, double mv, double *odat, dou
       for (size_t i = 0; i < n; ++i)
         {
           errno = -1;
-          odat[i] = (dbl_is_equal(idat[i], mv)) ? mv : exprfunc(cval, idat[i]);
+          odat[i] = (fp_is_equal(idat[i], mv)) ? mv : exprfunc(cval, idat[i]);
           if (errno == EDOM || errno == ERANGE || std::isnan(odat[i])) odat[i] = mv;
         }
     }
@@ -706,7 +704,7 @@ func_expr_var_con(int funcID, bool hasMV, size_t n, double mv, double *odat, con
       for (size_t i = 0; i < n; ++i)
         {
           errno = -1;
-          odat[i] = (dbl_is_equal(idat[i], mv)) ? mv : exprfunc(idat[i], cval);
+          odat[i] = (fp_is_equal(idat[i], mv)) ? mv : exprfunc(idat[i], cval);
           if (errno == EDOM || errno == ERANGE || std::isnan(odat[i])) odat[i] = mv;
         }
     }
@@ -732,7 +730,7 @@ func_expr_var_var(int funcID, bool hasMV, size_t n, double mv1, double mv2, doub
       for (size_t i = 0; i < n; ++i)
         {
           errno = -1;
-          odat[i] = (dbl_is_equal(idat1[i], mv1) || dbl_is_equal(idat2[i], mv2)) ? mv1 : exprfunc(idat1[i], idat2[i]);
+          odat[i] = (fp_is_equal(idat1[i], mv1) || fp_is_equal(idat2[i], mv2)) ? mv1 : exprfunc(idat1[i], idat2[i]);
           if (errno == EDOM || errno == ERANGE || std::isnan(odat[i])) odat[i] = mv1;
         }
     }
@@ -814,7 +812,7 @@ ex_fun_var(int init, int funcID, nodeType *p1)
               field.weightv.resize(ngp);
             }
 
-          auto exprfunc = (double (*)(const Field &)) funcEntry.func;
+          auto exprfunc = (double (*)(Field const &)) funcEntry.func;
           for (size_t k = 0; k < nlev; ++k)
             {
               fld_field_init(field, numMissVals, missval, ngp, p1data + k * ngp, p1->param.weight);
@@ -826,7 +824,7 @@ ex_fun_var(int init, int funcID, nodeType *p1)
           Field field1, field2;
           field1.resize(ngp);
           field2.resize(nlat);
-          auto exprfunc = (void (*)(const Field &, Field &)) funcEntry.func;
+          auto exprfunc = (void (*)(Field const &, Field &)) funcEntry.func;
           for (size_t k = 0; k < nlev; ++k)
             {
               fld_field_init(field1, numMissVals, missval, ngp, &p1data[k * ngp], nullptr);
@@ -841,7 +839,7 @@ ex_fun_var(int init, int funcID, nodeType *p1)
           Field field;
           field.resize(nlev);
           if (funcflag == 1) vert_weights(p1->param.zaxisID, nlev, field.weightv);
-          auto exprfunc = (double (*)(const Field &)) funcEntry.func;
+          auto exprfunc = (double (*)(Field const &)) funcEntry.func;
           for (size_t i = 0; i < ngp; ++i)
             {
               for (size_t k = 0; k < nlev; ++k) field.vec_d[k] = p1data[k * ngp + i];
@@ -894,8 +892,6 @@ func_con_var(int init, int funcID, const nodeType *p1, const nodeType *p2)
   auto hasMV = (p2->param.numMissVals > 0);
   auto missval2 = p2->param.missval;
 
-  auto n = ngp * nlev;
-
   auto p = new nodeType;
 
   p->type = NodeEnum::typeVar;
@@ -906,6 +902,7 @@ func_con_var(int init, int funcID, const nodeType *p1, const nodeType *p2)
 
   if (!init)
     {
+      auto n = ngp * nlev;
       p->param.data = new double[n];
       auto odat = p->param.data;
       const auto idat = p2->param.data;
@@ -931,8 +928,6 @@ func_var_con(int init, int funcID, const nodeType *p1, const nodeType *p2)
   auto hasMV = (p1->param.numMissVals > 0);
   auto missval1 = p1->param.missval;
 
-  auto n = ngp * nlev;
-
   auto p = new nodeType;
 
   p->type = NodeEnum::typeVar;
@@ -943,6 +938,7 @@ func_var_con(int init, int funcID, const nodeType *p1, const nodeType *p2)
 
   if (!init)
     {
+      auto n = ngp * nlev;
       p->param.data = new double[n];
       auto odat = p->param.data;
       const auto idat = p1->param.data;
@@ -1167,7 +1163,7 @@ ex_remap(int init, int funcID, nodeType *p1, nodeType *p2)
           Field field1, field2;
           field1.resize(ngp);
           field2.resize(ngp2);
-          auto exprfunc = (void (*)(const Field &, Field &)) funcEntry.func;
+          auto exprfunc = (void (*)(Field const &, Field &)) funcEntry.func;
           for (size_t k = 0; k < nlev; ++k)
             {
               fld_field_init(field1, numMissVals, missval, ngp, &p1data[k * ngp], nullptr);
@@ -1187,7 +1183,7 @@ ex_remap(int init, int funcID, nodeType *p1, nodeType *p2)
 }
 
 static size_t
-get_levidx(size_t nlev, const double *data, double value, const std::string &funcname)
+get_levidx(size_t nlev, const double *data, double value, std::string const &funcname)
 {
   size_t levidx;
 
@@ -1199,7 +1195,7 @@ get_levidx(size_t nlev, const double *data, double value, const std::string &fun
 }
 
 static void
-get_levidxrange(size_t nlev, const double *data, double value1, double value2, const std::string &funcname, size_t &levidx1,
+get_levidxrange(size_t nlev, const double *data, double value1, double value2, std::string const &funcname, size_t &levidx1,
                 size_t &levidx2)
 {
   long n = nlev;
@@ -1492,7 +1488,7 @@ ex_uminus_var(int init, nodeType *p1)
 
       if (numMissVals)
         {
-          for (size_t i = 0; i < ngp * nlev; ++i) pdata[i] = dbl_is_equal(p1data[i], missval) ? missval : -(p1data[i]);
+          for (size_t i = 0; i < ngp * nlev; ++i) pdata[i] = fp_is_equal(p1data[i], missval) ? missval : -(p1data[i]);
         }
       else
         {
@@ -1565,7 +1561,7 @@ ex_not_var(int init, nodeType *p1)
 
       if (numMissVals)
         {
-          for (size_t i = 0; i < ngp * nlev; ++i) pdata[i] = dbl_is_equal(p1data[i], missval) ? missval : unary_op_not(p1data[i]);
+          for (size_t i = 0; i < ngp * nlev; ++i) pdata[i] = fp_is_equal(p1data[i], missval) ? missval : unary_op_not(p1data[i]);
         }
       else
         {
@@ -1617,7 +1613,7 @@ ex_not(int init, nodeType *p1)
 static void
 str_add_node_info(char *string, size_t stringlen, nodeType *p, const char *ext)
 {
-  auto len = strlen(string);
+  auto len = std::strlen(string);
   if (p->type == NodeEnum::typeCon) { snprintf(string + len, stringlen - len, "%g%s", p->con().value, ext); }
   else { snprintf(string + len, stringlen - len, "%s[N%zu][L%zu]%s", p->var().name.c_str(), p->param.ngp, p->param.nlev, ext); }
 }
@@ -1778,12 +1774,12 @@ ex_ifelse(int init, nodeType *p1, nodeType *p2, nodeType *p3)
               auto ival2 = idat2[(ngp2 > 1) ? i : 0];
               auto ival3 = idat3[(ngp3 > 1) ? i : 0];
 
-              if (numMissVals1 && dbl_is_equal(ival1, missval1))
+              if (numMissVals1 && fp_is_equal(ival1, missval1))
                 odat[i] = missval1;
               else if (is_not_equal(ival1, 0.0))
-                odat[i] = dbl_is_equal(ival2, missval2) ? missval1 : ival2;
+                odat[i] = fp_is_equal(ival2, missval2) ? missval1 : ival2;
               else
-                odat[i] = dbl_is_equal(ival3, missval3) ? missval1 : ival3;
+                odat[i] = fp_is_equal(ival3, missval3) ? missval1 : ival3;
             }
 
           numMissVals += array_num_mv(ngp, odat, missval1);
@@ -1815,7 +1811,7 @@ exNode(nodeType *p, ParseParamType &parseArg)
 */
 
 static int
-param_search_name(int nparam, const std::vector<ParamEntry> &params, const std::string &name)
+param_search_name(int nparam, const std::vector<ParamEntry> &params, std::string const &name)
 {
   for (int varID = 0; varID < nparam; ++varID)
     {
@@ -1826,7 +1822,7 @@ param_search_name(int nparam, const std::vector<ParamEntry> &params, const std::
 }
 
 static int
-param_search_name_size(int nparam, std::vector<ParamEntry> &params, const std::string &name, size_t ngp, size_t nlev)
+param_search_name_size(int nparam, std::vector<ParamEntry> &params, std::string const &name, size_t ngp, size_t nlev)
 {
   for (int varID = 0; varID < nparam; ++varID)
     {
@@ -1845,7 +1841,7 @@ param_search_name_size(int nparam, std::vector<ParamEntry> &params, const std::s
 }
 
 static void
-param_print(const std::string &vname, const ParamEntry &param, long tsID)
+param_print(std::string const &vname, const ParamEntry &param, long tsID)
 {
   constexpr size_t maxout = 100;
   const auto data = param.data;
@@ -1865,10 +1861,10 @@ param_print(const std::string &vname, const ParamEntry &param, long tsID)
 }
 
 static void
-add_new_constant(const std::string &varname, ParseParamType &parseArg, std::vector<ParamEntry> &params, const ParamEntry &param)
+add_new_constant(std::string const &varname, ParseParamType &parseArg, std::vector<ParamEntry> &params, const ParamEntry &param)
 {
-  auto varID = parseArg.nparams;
-  if (varID >= parseArg.maxparams) cdo_abort("Too many parameter (limit=%d)", parseArg.maxparams);
+  auto varID = parseArg.numParams;
+  if (varID >= parseArg.maxParams) cdo_abort("Too many parameter (limit=%d)", parseArg.maxParams);
 
   param_meta_copy(params[varID], param);
   params[varID].type = ParamType::CONST;
@@ -1879,15 +1875,14 @@ add_new_constant(const std::string &varname, ParseParamType &parseArg, std::vect
   params[varID].missval = -9.e33;
   params[varID].numMissVals = 0;
   params[varID].name = varname;
-  parseArg.nparams++;
-  parseArg.cnparams++;
+  parseArg.numParams++;
 }
 
 static void
-add_new_param(const std::string &varname, ParseParamType &parseArg, std::vector<ParamEntry> &params, const ParamEntry &param)
+add_new_param(std::string const &varname, ParseParamType &parseArg, std::vector<ParamEntry> &params, const ParamEntry &param)
 {
-  auto varID = parseArg.nparams;
-  if (varID >= parseArg.maxparams) cdo_abort("Too many parameter (limit=%d)", parseArg.maxparams);
+  auto varID = parseArg.numParams;
+  if (varID >= parseArg.maxParams) cdo_abort("Too many parameter (limit=%d)", parseArg.maxParams);
 
   param_meta_copy(params[varID], param);
   params[varID].isValid = true;
@@ -1896,8 +1891,7 @@ add_new_param(const std::string &varname, ParseParamType &parseArg, std::vector<
   params[varID].numMissVals = param.numMissVals;
   if (param.units.size()) params[varID].units = param.units;
   if (param.longname.size()) params[varID].longname = param.longname;
-  parseArg.nparams++;
-  parseArg.cnparams++;
+  parseArg.numParams++;
 }
 
 static nodeType *
@@ -1906,11 +1900,11 @@ expr_run_cmd(nodeType *p, ParseParamType &parseArg)
   auto init = parseArg.init;
   auto &params = parseArg.params;
 
-  const auto &cmdName = p->cmd().cmdName;
-  const auto &varName = p->cmd().varName;
+  auto const &cmdName = p->cmd().cmdName;
+  auto const &varName = p->cmd().varName;
   if (parseArg.debug) cdo_print("\tstatement\t\t%s(%s)", cmdName, varName);
 
-  auto varID = param_search_name(parseArg.nparams, params, varName);
+  auto varID = param_search_name(parseArg.numParams, params, varName);
   if (varID == -1) cdo_abort("Variable %s not found, needed for statement %s(%s)!", varName, cmdName, varName);
 
   if (init)
@@ -1933,21 +1927,21 @@ expr_run_con(nodeType *p, const ParseParamType &parseArg)
 }
 
 static int
-expr_run_var_grid(const std::string &vnm, int coord, ParseParamType &parseArg)
+expr_run_var_grid(std::string const &vnm, int coord, ParseParamType &parseArg)
 {
   auto &params = parseArg.params;
 
   auto varname = vnm.substr(0, vnm.size() - 2);
-  auto varID = param_search_name(parseArg.nparams, params, varname);
+  auto varID = param_search_name(parseArg.numParams, params, varname);
   if (varID == -1) cdo_abort("Coordinate %c: variable >%s< not found!", coord, varname);
 
-  auto nvarID = parseArg.nparams;
-  if (nvarID >= parseArg.maxparams) cdo_abort("Too many parameter (limit=%d)", parseArg.maxparams);
+  auto nvarID = parseArg.numParams;
+  if (nvarID >= parseArg.maxParams) cdo_abort("Too many parameter (limit=%d)", parseArg.maxParams);
 
   auto coordID = params_get_coord_ID(parseArg, coord, params[varID].gridID);
   parseArg.coords[coordID].needed = true;
-  const auto &units = parseArg.coords[coordID].units;
-  const auto &longname = parseArg.coords[coordID].longname;
+  auto const &units = parseArg.coords[coordID].units;
+  auto const &longname = parseArg.coords[coordID].longname;
 
   params[nvarID].isValid = true;
   params[nvarID].coord = coord;
@@ -1962,28 +1956,27 @@ expr_run_var_grid(const std::string &vnm, int coord, ParseParamType &parseArg)
   params[nvarID].nlev = 1;
   if (units.size()) params[nvarID].units = units;
   if (longname.size()) params[nvarID].longname = longname;
-  parseArg.nparams++;
-  parseArg.cnparams++;
+  parseArg.numParams++;
 
   return nvarID;
 }
 
 static int
-expr_run_var_zaxis(const std::string &vnm, int coord, ParseParamType &parseArg)
+expr_run_var_zaxis(std::string const &vnm, int coord, ParseParamType &parseArg)
 {
   auto &params = parseArg.params;
 
   auto varname = vnm.substr(0, vnm.size() - 2);
-  auto varID = param_search_name(parseArg.nparams, params, varname);
+  auto varID = param_search_name(parseArg.numParams, params, varname);
   if (varID == -1) cdo_abort("Coordinate %c: variable >%s< not found!", coord, varname);
 
-  auto nvarID = parseArg.nparams;
-  if (nvarID >= parseArg.maxparams) cdo_abort("Too many parameter (limit=%d)", parseArg.maxparams);
+  auto nvarID = parseArg.numParams;
+  if (nvarID >= parseArg.maxParams) cdo_abort("Too many parameter (limit=%d)", parseArg.maxParams);
 
   auto coordID = params_get_coord_ID(parseArg, coord, params[varID].zaxisID);
   parseArg.coords[coordID].needed = true;
-  const auto &units = parseArg.coords[coordID].units;
-  const auto &longname = parseArg.coords[coordID].longname;
+  auto const &units = parseArg.coords[coordID].units;
+  auto const &longname = parseArg.coords[coordID].longname;
 
   params[nvarID].isValid = true;
   params[nvarID].coord = coord;
@@ -1997,8 +1990,7 @@ expr_run_var_zaxis(const std::string &vnm, int coord, ParseParamType &parseArg)
   params[nvarID].nlev = params[varID].nlev;
   if (units.size()) params[nvarID].units = units;
   if (longname.size()) params[nvarID].longname = longname;
-  parseArg.nparams++;
-  parseArg.cnparams++;
+  parseArg.numParams++;
 
   return nvarID;
 }
@@ -2007,10 +1999,10 @@ static nodeType *
 expr_run_var(nodeType *p, ParseParamType &parseArg)
 {
   auto init = parseArg.init;
-  const auto &params = parseArg.params;
+  auto const &params = parseArg.params;
 
-  const auto &vnm = p->var().name;
-  auto varID = param_search_name(parseArg.nparams, params, vnm);
+  auto const &vnm = p->var().name;
+  auto varID = param_search_name(parseArg.numParams, params, vnm);
   if (varID == -1 && init)
     {
       int len = vnm.size();
@@ -2062,7 +2054,7 @@ expr_run_fun2(nodeType *p, ParseParamType &parseArg)
   auto init = parseArg.init;
 
   auto funcID = get_funcID(p->fun().name);
-  const auto &funcEntry = funcSymbolTable[funcID];
+  auto const &funcEntry = funcSymbolTable[funcID];
 
   auto fnode1 = expr_run(p->fun().op[0], parseArg);
   auto fnode2 = expr_run(p->fun().op[1], parseArg);
@@ -2081,7 +2073,7 @@ expr_run_fun3(nodeType *p, ParseParamType &parseArg)
   auto init = parseArg.init;
 
   auto funcID = get_funcID(p->fun().name);
-  const auto &funcEntry = funcSymbolTable[funcID];
+  auto const &funcEntry = funcSymbolTable[funcID];
 
   auto fnode1 = expr_run(p->fun().op[0], parseArg);
   auto fnode2 = expr_run(p->fun().op[1], parseArg);
@@ -2097,7 +2089,7 @@ static nodeType *
 expr_run_fun(nodeType *p, ParseParamType &parseArg)
 {
   auto init = parseArg.init;
-  const auto &params = parseArg.params;
+  auto const &params = parseArg.params;
 
   auto funcID = get_funcID(p->fun().name);
   auto &funcEntry = funcSymbolTable[funcID];
@@ -2153,7 +2145,7 @@ expr_run_opr(nodeType *p, ParseParamType &parseArg)
       {
         auto rnode = expr_run(p->opr().op[1], parseArg);
 
-        const auto &varname2 = p->opr().op[0]->var().name;
+        auto const &varname2 = p->opr().op[0]->var().name;
 
         auto isVar = (rnode && rnode->type == NodeEnum::typeVar && rnode->param.ngp > 0 && rnode->param.nlev > 0);
         if (parseArg.debug)
@@ -2167,7 +2159,7 @@ expr_run_opr(nodeType *p, ParseParamType &parseArg)
         // auto varID = param_search_name(parseArg.nparams, params, varname2);
         auto ngp = isVar ? rnode->param.ngp : 0;
         auto nlev = isVar ? rnode->param.nlev : 0;
-        auto varID = param_search_name_size(parseArg.nparams, params, varname2, ngp, nlev);
+        auto varID = param_search_name_size(parseArg.numParams, params, varname2, ngp, nlev);
         if (init)
           {
             if (varID >= 0)
