@@ -4,6 +4,9 @@
   Author: Uwe Schulzweida
 
 */
+#ifndef CDO_EXPR_H
+#define CDO_EXPR_H
+
 #include <cstdio>
 #include <cstdarg>
 #include <cassert>
@@ -26,12 +29,12 @@ enum CoordIndex
   MINUTE,
   HOUR,
   CALENDAR,
-  DOY,      // day of year
-  DPY,      // days per year
+  DOY,  // day of year
+  DPY,  // days per year
   LEN
 };
 
-enum class NodeEnum
+enum struct NodeEnum
 {
   typeUndef,
   typeCon,
@@ -46,7 +49,8 @@ struct conNodeType
 {
   double value;  // value of constant
 
-  conNodeType(double _value) : value(_value) {}
+  conNodeType() : value(0.0) {}  // Default ctor is needed to initialize variant object
+  explicit conNodeType(double _value) : value(_value) {}
 };
 
 // variables
@@ -54,7 +58,7 @@ struct varNodeType
 {
   std::string name;  // variable name
 
-  varNodeType(const char *_name) : name(_name) {}
+  explicit varNodeType(const char *_name) : name(_name) {}
 };
 
 // commands
@@ -73,7 +77,7 @@ struct funNodeType
   int nops;                           // number of operands
   std::vector<struct nodeType *> op;  // operands
 
-  funNodeType(const std::string &fname, int _nops, va_list args) : name(fname), nops(_nops)
+  funNodeType(std::string const &fname, int _nops, va_list args) : name(fname), nops(_nops)
   {
     op.resize(nops);
     for (int i = 0; i < nops; i++) op[i] = va_arg(args, nodeType *);
@@ -94,7 +98,7 @@ struct oprNodeType
   }
 };
 
-enum class ParamType
+enum struct ParamType
 {
   UNDEFINED,
   VAR,
@@ -117,7 +121,7 @@ struct ParamEntry
   size_t ngp = 0;
   size_t nlat = 0;
   size_t nlev = 0;
-  size_t nmiss = 0;
+  size_t numMissVals = 0;
   std::string name;
   std::string longname;
   std::string stdname;
@@ -133,8 +137,7 @@ struct nodeType
   ParamEntry param;
   NodeEnum type{ NodeEnum::typeUndef };  // type of node
   bool isTmpObj = false;
-  // int variant is a dummy for default constructor
-  std::variant<int, conNodeType, varNodeType, cmdNodeType, funNodeType, oprNodeType> v;
+  std::variant<conNodeType, varNodeType, cmdNodeType, funNodeType, oprNodeType> v;
 
   auto &con() const { assert(std::holds_alternative<conNodeType>(v)); return std::get<conNodeType>(v); }
   auto &var()       { assert(std::holds_alternative<varNodeType>(v)); return std::get<varNodeType>(v); }
@@ -161,11 +164,10 @@ struct ParseParamType
   std::vector<ParamEntry> params;
   std::vector<CoordType> coords;
   std::vector<bool> needed;
-  int maxparams;
-  int nparams;
-  int cnparams;  // current number of valid params
-  int nvars1;
-  int ncoords;
+  int maxParams;
+  int numParams;
+  int numVars1;
+  int numCoords;
   int maxCoords;
   int tsID;
   int pointID;
@@ -198,3 +200,5 @@ void yyset_extra(YY_EXTRA_TYPE, void *);
 
 nodeType *expr_run(nodeType *p, ParseParamType &parseArg);
 int params_get_coord_ID(const ParseParamType &parseArg, int coord, int cdiID);
+
+#endif
