@@ -21,7 +21,7 @@ void
 Field::init(const CdoVar &var)
 {
   fpeRaised = 0;
-  nwpv = 1;
+  nwpv = var.nwpv;
   grid = var.gridID;
   gridsize = var.gridsize;
   numMissVals = 0;
@@ -36,7 +36,7 @@ Field::init(const CdoVar &var)
 }
 
 void
-Field::resize(const size_t count)
+Field::resize(size_t count)
 {
   memType = MemType::Double;
   m_count = count;
@@ -45,7 +45,7 @@ Field::resize(const size_t count)
 }
 
 void
-Field::resize(const size_t count, double value)
+Field::resize(size_t count, double value)
 {
   memType = MemType::Double;
   m_count = count;
@@ -54,7 +54,7 @@ Field::resize(const size_t count, double value)
 }
 
 void
-Field::resizef(const size_t count)
+Field::resizef(size_t count)
 {
   memType = MemType::Float;
   m_count = count;
@@ -63,7 +63,7 @@ Field::resizef(const size_t count)
 }
 
 void
-Field::resizef(const size_t count, const float value)
+Field::resizef(size_t count, const float value)
 {
   memType = MemType::Float;
   m_count = count;
@@ -88,6 +88,7 @@ void
 Field3D::init(const CdoVar &var)
 {
   nlevels = var.nlevels;
+  nwpv = var.nwpv;
   grid = var.gridID;
   gridsize = var.gridsize;
   missval = var.missval;
@@ -109,19 +110,19 @@ field_fill(Field &field, double value)
 }
 
 void
-field_ncopy(size_t n, const Field &fieldIn, Field &fieldOut)
+field_ncopy(size_t n, Field const &fieldIn, Field &fieldOut)
 {
   if (n > fieldIn.size) cdo_abort("Source field to small (%s)", __func__);
   if (n > fieldOut.size) cdo_abort("Target field to small (%s)", __func__);
 
   fieldOut.numMissVals = fieldIn.numMissVals;
 
-  auto func = [&](const auto &v1, auto &v2) { varray_copy(n, v1, v2); };
+  auto func = [&](auto const &v1, auto &v2) { varray_copy(n, v1, v2); };
   field_operation2(func, fieldIn, fieldOut);
 }
 
 void
-field_copy(const Field &fieldIn, Field &fieldOut)
+field_copy(Field const &fieldIn, Field &fieldOut)
 {
   field_ncopy(fieldIn.size, fieldIn, fieldOut);
 }
@@ -151,7 +152,7 @@ field_add(Field &field1, const Field3D &field2, int levelID)
 {
   auto size = field1.gridsize * field1.nwpv;
   auto offset = levelID * size;
-  auto func = [&](auto &v1, const auto &v2) {
+  auto func = [&](auto &v1, auto const &v2) {
     for (size_t i = 0; i < size; ++i) v1[i] += v2[offset + i];
   };
   field_operation2(func, field1, field2);
@@ -186,9 +187,9 @@ public:
 };
 
 size_t
-field_num_NANs(const Field &field)
+field_num_NANs(Field const &field)
 {
-  auto func = [&](const auto &v, auto n) {
+  auto func = [&](auto const &v, auto n) {
     size_t numNANs = 0;
 #ifdef HAVE_OPENMP4
 #pragma omp parallel for simd if (n > cdoMinLoopSize) default(shared) schedule(static) reduction(+ : numNANs)
@@ -202,140 +203,140 @@ field_num_NANs(const Field &field)
 size_t
 field_num_mv(Field &field)
 {
-  auto func = [](const auto &v, auto n, auto mv) { return varray_num_mv(n, v, mv); };
+  auto func = [](auto const &v, auto n, auto mv) { return varray_num_mv(n, v, mv); };
   field.numMissVals = field_operation(func, field, field.size, field.missval);
   return field.numMissVals;
 }
 
 MinMax
-field_min_max(const Field &field)
+field_min_max(Field const &field)
 {
-  auto func = [](const auto &v) { return varray_min_max(v); };
-  auto func_mv = [](const auto &v, auto n, auto mv) { return varray_min_max_mv(n, v, mv); };
+  auto func = [](auto const &v) { return varray_min_max(v); };
+  auto func_mv = [](auto const &v, auto n, auto mv) { return varray_min_max_mv(n, v, mv); };
   return field.numMissVals ? field_operation(func_mv, field, field.size, field.missval) : field_operation(func, field);
 }
 
 double
-field_min(const Field &field)
+field_min(Field const &field)
 {
-  auto func = [](const auto &v, auto n) { return varray_min(n, v); };
-  auto func_mv = [](const auto &v, auto n, auto mv) { return varray_min_mv(n, v, mv); };
+  auto func = [](auto const &v, auto n) { return varray_min(n, v); };
+  auto func_mv = [](auto const &v, auto n, auto mv) { return varray_min_mv(n, v, mv); };
   return field.numMissVals ? field_operation(func_mv, field, field.size, field.missval) : field_operation(func, field, field.size);
 }
 
 double
-field_max(const Field &field)
+field_max(Field const &field)
 {
-  auto func = [](const auto &v, auto n) { return varray_max(n, v); };
-  auto func_mv = [](const auto &v, auto n, auto mv) { return varray_max_mv(n, v, mv); };
+  auto func = [](auto const &v, auto n) { return varray_max(n, v); };
+  auto func_mv = [](auto const &v, auto n, auto mv) { return varray_max_mv(n, v, mv); };
   return field.numMissVals ? field_operation(func_mv, field, field.size, field.missval) : field_operation(func, field, field.size);
 }
 
 double
-field_range(const Field &field)
+field_range(Field const &field)
 {
-  auto func = [](const auto &v, auto n) { return varray_range(n, v); };
-  auto func_mv = [](const auto &v, auto n, auto mv) { return varray_range_mv(n, v, mv); };
+  auto func = [](auto const &v, auto n) { return varray_range(n, v); };
+  auto func_mv = [](auto const &v, auto n, auto mv) { return varray_range_mv(n, v, mv); };
   return field.numMissVals ? field_operation(func_mv, field, field.size, field.missval) : field_operation(func, field, field.size);
 }
 
 double
-field_sum(const Field &field)
+field_sum(Field const &field)
 {
-  auto func = [](const auto &v, auto n) { return varray_sum(n, v); };
-  auto func_mv = [](const auto &v, auto n, auto mv) { return varray_sum_mv(n, v, mv); };
+  auto func = [](auto const &v, auto n) { return varray_sum(n, v); };
+  auto func_mv = [](auto const &v, auto n, auto mv) { return varray_sum_mv(n, v, mv); };
   return field.numMissVals ? field_operation(func_mv, field, field.size, field.missval) : field_operation(func, field, field.size);
 }
 
 double
-field_mean(const Field &field)
+field_mean(Field const &field)
 {
-  auto func = [](const auto &v, auto n) { return varray_mean(n, v); };
-  auto func_mv = [](const auto &v, auto n, auto mv) { return varray_mean_mv(n, v, mv); };
+  auto func = [](auto const &v, auto n) { return varray_mean(n, v); };
+  auto func_mv = [](auto const &v, auto n, auto mv) { return varray_mean_mv(n, v, mv); };
   return field.numMissVals ? field_operation(func_mv, field, field.size, field.missval) : field_operation(func, field, field.size);
 }
 
 double
-field_meanw(const Field &field)
+field_meanw(Field const &field)
 {
-  auto func = [](const auto &v, auto n, auto mv, const auto &w) { return varray_weighted_mean(n, v, w, mv); };
-  auto func_mv = [](const auto &v, auto n, auto mv, const auto &w) { return varray_weighted_mean_mv(n, v, w, mv); };
+  auto func = [](auto const &v, auto n, auto mv, auto const &w) { return varray_weighted_mean(n, v, w, mv); };
+  auto func_mv = [](auto const &v, auto n, auto mv, auto const &w) { return varray_weighted_mean_mv(n, v, w, mv); };
   return field.numMissVals ? field_operation(func_mv, field, field.size, field.missval, field.weightv)
                            : field_operation(func, field, field.size, field.missval, field.weightv);
 }
 
 double
-field_avg(const Field &field)
+field_avg(Field const &field)
 {
-  auto func = [](const auto &v, auto n) { return varray_mean(n, v); };
-  auto func_mv = [](const auto &v, auto n, auto mv) { return varray_avg_mv(n, v, mv); };
+  auto func = [](auto const &v, auto n) { return varray_mean(n, v); };
+  auto func_mv = [](auto const &v, auto n, auto mv) { return varray_avg_mv(n, v, mv); };
   return field.numMissVals ? field_operation(func_mv, field, field.size, field.missval) : field_operation(func, field, field.size);
 }
 
 double
-field_avgw(const Field &field)
+field_avgw(Field const &field)
 {
-  auto func = [](const auto &v, auto n, auto mv, const auto &w) { return varray_weighted_mean(n, v, w, mv); };
-  auto func_mv = [](const auto &v, auto n, auto mv, const auto &w) { return varray_weighted_avg_mv(n, v, w, mv); };
+  auto func = [](auto const &v, auto n, auto mv, auto const &w) { return varray_weighted_mean(n, v, w, mv); };
+  auto func_mv = [](auto const &v, auto n, auto mv, auto const &w) { return varray_weighted_avg_mv(n, v, w, mv); };
   return field.numMissVals ? field_operation(func_mv, field, field.size, field.missval, field.weightv)
                            : field_operation(func, field, field.size, field.missval, field.weightv);
 }
 
 double
-field_var(const Field &field)
+field_var(Field const &field)
 {
-  auto func = [](const auto &v, auto n, auto numMissVals, auto mv) { return varray_var(n, v, numMissVals, mv); };
+  auto func = [](auto const &v, auto n, auto numMissVals, auto mv) { return varray_var(n, v, numMissVals, mv); };
   return field_operation(func, field, field.size, field.numMissVals, field.missval);
 }
 
 double
-field_var1(const Field &field)
+field_var1(Field const &field)
 {
-  auto func = [](const auto &v, auto n, auto numMissVals, auto mv) { return varray_var_1(n, v, numMissVals, mv); };
+  auto func = [](auto const &v, auto n, auto numMissVals, auto mv) { return varray_var_1(n, v, numMissVals, mv); };
   return field_operation(func, field, field.size, field.numMissVals, field.missval);
 }
 
 double
-field_skew(const Field &field)
+field_skew(Field const &field)
 {
-  auto func = [](const auto &v, auto n, auto numMissVals, auto mv) { return varray_skew(n, v, numMissVals, mv); };
+  auto func = [](auto const &v, auto n, auto numMissVals, auto mv) { return varray_skew(n, v, numMissVals, mv); };
   return field_operation(func, field, field.size, field.numMissVals, field.missval);
 }
 
 double
-field_kurt(const Field &field)
+field_kurt(Field const &field)
 {
-  auto func = [](const auto &v, auto n, auto numMissVals, auto mv) { return varray_kurt(n, v, numMissVals, mv); };
+  auto func = [](auto const &v, auto n, auto numMissVals, auto mv) { return varray_kurt(n, v, numMissVals, mv); };
   return field_operation(func, field, field.size, field.numMissVals, field.missval);
 }
 
 double
-field_median(const Field &field)
+field_median(Field const &field)
 {
-  auto func = [](const auto &v, auto n, auto numMissVals, auto mv) { return varray_median(n, v, numMissVals, mv); };
+  auto func = [](auto const &v, auto n, auto numMissVals, auto mv) { return varray_median(n, v, numMissVals, mv); };
   return field_operation(func, field, field.size, field.numMissVals, field.missval);
 }
 
 double
-field_count(const Field &field)
+field_count(Field const &field)
 {
-  auto func = [](const auto &v, auto n, auto numMissVals, auto mv) { return varray_count(n, v, numMissVals, mv); };
+  auto func = [](auto const &v, auto n, auto numMissVals, auto mv) { return varray_count(n, v, numMissVals, mv); };
   return field_operation(func, field, field.size, field.numMissVals, field.missval);
 }
 
 double
-field_varw(const Field &field)
+field_varw(Field const &field)
 {
-  auto func = [](const auto &v, auto n, auto numMissVals, auto mv, const auto &w) {
+  auto func = [](auto const &v, auto n, auto numMissVals, auto mv, auto const &w) {
     return varray_weighted_var(n, v, w, numMissVals, mv);
   };
   return field_operation(func, field, field.size, field.numMissVals, field.missval, field.weightv);
 }
 
 double
-field_var1w(const Field &field)
+field_var1w(Field const &field)
 {
-  auto func = [](const auto &v, auto n, auto numMissVals, auto mv, const auto &w) {
+  auto func = [](auto const &v, auto n, auto numMissVals, auto mv, auto const &w) {
     return varray_weighted_var_1(n, v, w, numMissVals, mv);
   };
   return field_operation(func, field, field.size, field.numMissVals, field.missval, field.weightv);
@@ -350,31 +351,31 @@ var_to_std(double rvar, double missval)
 }
 
 double
-field_std(const Field &field)
+field_std(Field const &field)
 {
   return var_to_std(field_var(field), field.missval);
 }
 
 double
-field_std1(const Field &field)
+field_std1(Field const &field)
 {
   return var_to_std(field_var1(field), field.missval);
 }
 
 double
-field_stdw(const Field &field)
+field_stdw(Field const &field)
 {
   return var_to_std(field_varw(field), field.missval);
 }
 
 double
-field_std1w(const Field &field)
+field_std1w(Field const &field)
 {
   return var_to_std(field_var1w(field), field.missval);
 }
 
 void
-field_rms(const Field &field, const Field &field2, Field &field3)
+field_rms(Field const &field, Field const &field2, Field &field3)
 {
   size_t rnumMissVals = 0;
   auto grid1 = field.grid;
@@ -385,7 +386,7 @@ field_rms(const Field &field, const Field &field2, Field &field3)
   const auto array2 = field2.vec_d.data();
   auto missval1 = field.missval;
   auto missval2 = field2.missval;
-  const auto &w = field.weightv;
+  auto const &w = field.weightv;
   auto rsum = 0.0, rsumw = 0.0;
   auto is_NE = fp_is_not_equal;
   auto is_EQ = fp_is_equal;
@@ -460,9 +461,9 @@ field_pctl(Field &field, double pn)
 static int
 compare_double(const void *const a, const void *const b)
 {
-  const double *const x = (const double *) a;
-  const double *const y = (const double *) b;
-  return ((*x > *y) - (*x < *y)) * 2 + (*x > *y) - (*x < *y);
+  const auto *const x = static_cast<const double *>(a);
+  const auto *const y = static_cast<const double *>(b);
+  return (*x < *y) ? -1 : (*x > *y);
 }
 
 double
@@ -492,7 +493,7 @@ field_rank(Field &field)
 }
 
 double
-field_function(const Field &field, int function)
+field_function(Field const &field, int function)
 {
   // clang-format off
   switch (function)
