@@ -12,13 +12,13 @@
 #include "cdo_output.h"
 
 static void
-field2_add_complex(Field &field1, const Field &field2)
+field2_add_complex(Field &field1, Field const &field2)
 {
-  auto is_EQ = dbl_is_equal;
+  auto is_EQ = fp_is_equal;
   auto missval1 = field1.missval;
   auto missval2 = field2.missval;
   auto &array1 = field1.vec_d;
-  const auto &array2 = field2.vec_d;
+  auto const &array2 = field2.vec_d;
 
   if (field1.nwpv != 2) cdo_abort("Field1 is not complex!");
   if (field2.nwpv != 2) cdo_abort("Field2 is not complex!");
@@ -34,13 +34,13 @@ field2_add_complex(Field &field1, const Field &field2)
 }
 
 static void
-field2_sub_complex(Field &field1, const Field &field2)
+field2_sub_complex(Field &field1, Field const &field2)
 {
-  auto is_EQ = dbl_is_equal;
+  auto is_EQ = fp_is_equal;
   auto missval1 = field1.missval;
   auto missval2 = field2.missval;
   auto &array1 = field1.vec_d;
-  const auto &array2 = field2.vec_d;
+  auto const &array2 = field2.vec_d;
 
   if (field1.nwpv != 2) cdo_abort("Field1 is not complex!");
   if (field2.nwpv != 2) cdo_abort("Field2 is not complex!");
@@ -56,13 +56,13 @@ field2_sub_complex(Field &field1, const Field &field2)
 }
 
 static void
-field2_mul_complex(Field &field1, const Field &field2)
+field2_mul_complex(Field &field1, Field const &field2)
 {
-  auto is_EQ = dbl_is_equal;
+  auto is_EQ = fp_is_equal;
   auto missval1 = field1.missval;
   auto missval2 = field2.missval;
   auto &array1 = field1.vec_d;
-  const auto &array2 = field2.vec_d;
+  auto const &array2 = field2.vec_d;
 
   if (field1.nwpv != 2) cdo_abort("Field1 is not complex!");
   if (field2.nwpv != 2) cdo_abort("Field2 is not complex!");
@@ -81,13 +81,13 @@ field2_mul_complex(Field &field1, const Field &field2)
 }
 
 static void
-field2_div_complex(Field &field1, const Field &field2)
+field2_div_complex(Field &field1, Field const &field2)
 {
-  auto is_EQ = dbl_is_equal;
+  auto is_EQ = fp_is_equal;
   auto missval1 = field1.missval;
   auto missval2 = field2.missval;
   auto &array1 = field1.vec_d;
-  const auto &array2 = field2.vec_d;
+  auto const &array2 = field2.vec_d;
 
   if (field1.nwpv != 2) cdo_abort("Field1 is not complex!");
   if (field2.nwpv != 2) cdo_abort("Field2 is not complex!");
@@ -107,7 +107,7 @@ field2_div_complex(Field &field1, const Field &field2)
 }
 
 void
-field2_function_complex(Field &field1, const Field &field2, int function)
+field2_function_complex(Field &field1, Field const &field2, int function)
 {
   // clang-format off
   switch (function)
