@@ -16,18 +16,16 @@
 
 template <typename T>
 static void
-varray_copy_zonal(size_t offset, size_t nx, const Varray<T> &v1, Varray<double> &v2)
+varray_copy_zonal(size_t offset, size_t nx, Varray<T> const &v1, Varray<double> &v2)
 {
   std::copy(v1.begin() + offset, v1.begin() + offset + nx, v2.begin());
 }
 
 static void
-copy_latitude_row(size_t offset, size_t nx, const Field &field1, Varray<double> &v)
+copy_latitude_row(size_t offset, size_t nx, Field const &field1, Varray<double> &v)
 {
-  if (field1.memType == MemType::Float)
-    varray_copy_zonal(offset, nx, field1.vec_f, v);
-  else
-    varray_copy_zonal(offset, nx, field1.vec_d, v);
+  auto func = [&](auto const &v1) { varray_copy_zonal(offset, nx, v1, v); };
+  field_operation(func, field1);
 }
 
 static size_t
@@ -44,12 +42,12 @@ fill_reduced_points(int gridID, size_t ny, std::vector<int> &reducedPoints, std:
   return nx;
 }
 
-using funcType = double(size_t, const Varray<double> &);
-using funcTypeMV = double(size_t, const Varray<double> &, double);
-using funcTypeNmissMV = double(size_t, const Varray<double> &, size_t, double);
+using funcType = double(size_t, Varray<double> const &);
+using funcTypeMV = double(size_t, Varray<double> const &, double);
+using funcTypeNmissMV = double(size_t, Varray<double> const &, size_t, double);
 
 static void
-zonal_kernel_1(const Field &field1, Field &field2, funcTypeNmissMV funcNmissMV)
+zonal_kernel_1(Field const &field1, Field &field2, funcTypeNmissMV funcNmissMV)
 {
   size_t rnumMissVals = 0;
   auto numMissVals = field1.numMissVals;
@@ -78,7 +76,7 @@ zonal_kernel_1(const Field &field1, Field &field2, funcTypeNmissMV funcNmissMV)
 }
 
 static void
-zonal_kernel_2(const Field &field1, Field &field2, funcType func, funcTypeMV funcMV)
+zonal_kernel_2(Field const &field1, Field &field2, funcType func, funcTypeMV funcMV)
 {
   size_t rnumMissVals = 0;
   auto numMissVals = field1.numMissVals;
@@ -106,55 +104,55 @@ zonal_kernel_2(const Field &field1, Field &field2, funcType func, funcTypeMV fun
 }
 
 void
-zonal_min(const Field &field1, Field &field2)
+zonal_min(Field const &field1, Field &field2)
 {
   zonal_kernel_2(field1, field2, varray_min, varray_min_mv);
 }
 
 void
-zonal_max(const Field &field1, Field &field2)
+zonal_max(Field const &field1, Field &field2)
 {
   zonal_kernel_2(field1, field2, varray_max, varray_max_mv);
 }
 
 void
-zonal_range(const Field &field1, Field &field2)
+zonal_range(Field const &field1, Field &field2)
 {
   zonal_kernel_2(field1, field2, varray_range, varray_range_mv);
 }
 
 void
-zonal_sum(const Field &field1, Field &field2)
+zonal_sum(Field const &field1, Field &field2)
 {
   zonal_kernel_2(field1, field2, varray_sum, varray_sum_mv);
 }
 
 void
-zonal_mean(const Field &field1, Field &field2)
+zonal_mean(Field const &field1, Field &field2)
 {
   zonal_kernel_2(field1, field2, varray_mean, varray_mean_mv);
 }
 
 void
-zonal_avg(const Field &field1, Field &field2)
+zonal_avg(Field const &field1, Field &field2)
 {
   zonal_kernel_2(field1, field2, varray_mean, varray_avg_mv);
 }
 
 void
-zonal_var(const Field &field1, Field &field2)
+zonal_var(Field const &field1, Field &field2)
 {
   zonal_kernel_1(field1, field2, varray_var);
 }
 
 void
-zonal_var1(const Field &field1, Field &field2)
+zonal_var1(Field const &field1, Field &field2)
 {
   zonal_kernel_1(field1, field2, varray_var_1);
 }
 
 void
-zonal_std(const Field &field1, Field &field2)
+zonal_std(Field const &field1, Field &field2)
 {
   size_t rnumMissVals = 0;
   auto missval = field2.missval;
@@ -173,7 +171,7 @@ zonal_std(const Field &field1, Field &field2)
 }
 
 void
-zonal_std1(const Field &field1, Field &field2)
+zonal_std1(Field const &field1, Field &field2)
 {
   size_t rnumMissVals = 0;
   auto missval = field2.missval;
@@ -192,25 +190,25 @@ zonal_std1(const Field &field1, Field &field2)
 }
 
 void
-zonal_skew(const Field &field1, Field &field2)
+zonal_skew(Field const &field1, Field &field2)
 {
   zonal_kernel_1(field1, field2, varray_skew);
 }
 
 void
-zonal_kurt(const Field &field1, Field &field2)
+zonal_kurt(Field const &field1, Field &field2)
 {
   zonal_kernel_1(field1, field2, varray_kurt);
 }
 
 void
-zonal_median(const Field &field1, Field &field2)
+zonal_median(Field const &field1, Field &field2)
 {
   zonal_kernel_1(field1, field2, varray_median);
 }
 
 void
-zonal_pctl(const Field &field1, Field &field2, double pn)
+zonal_pctl(Field const &field1, Field &field2, double pn)
 {
   size_t rnumMissVals = 0;
   auto missval = field2.missval;
@@ -263,7 +261,7 @@ zonal_pctl(const Field &field1, Field &field2, double pn)
 }
 
 void
-zonal_function(const Field &field1, Field &field2, int function)
+zonal_function(Field const &field1, Field &field2, int function)
 {
   // clang-format off
   switch (function)
