@@ -7,7 +7,7 @@
 
 #include <cdi.h>
 
-#include "process_int.h"
+#include "cdo_output.h"
 #include "mpim_grid.h"
 #include "grid_healpix.h"
 #include "gaussian_latitudes.h"
@@ -15,11 +15,11 @@
 #include "util_string.h"
 #include "dcw_reader.h"
 
-size_t genIcosphereCoords(int subdivisions, bool withBounds, std::vector<double> &xvals, std::vector<double> &yvals,
-                          std::vector<double> &xbounds, std::vector<double> &ybounds);
+size_t gen_icosphere_coords(int subdivisions, bool withBounds, std::vector<double> &xvals, std::vector<double> &yvals,
+                            std::vector<double> &xbounds, std::vector<double> &ybounds);
 
 static void
-generate_grid_icosphere(GridDesciption &grid, const std::string &gridname)
+generate_grid_icosphere(GridDesciption &grid, std::string const &gridname)
 {
   int gridtype = GRID_UNSTRUCTURED;
 
@@ -37,7 +37,7 @@ generate_grid_icosphere(GridDesciption &grid, const std::string &gridname)
   grid.type = gridtype;
   if (withBounds) grid.nvertex = 3;
 
-  auto ncells = genIcosphereCoords(bVal + 1, withBounds, grid.xvals, grid.yvals, grid.xbounds, grid.ybounds);
+  auto ncells = gen_icosphere_coords(bVal + 1, withBounds, grid.xvals, grid.yvals, grid.xbounds, grid.ybounds);
   grid.xsize = ncells;
   grid.ysize = ncells;
   grid.xname = "clon";
@@ -47,7 +47,7 @@ generate_grid_icosphere(GridDesciption &grid, const std::string &gridname)
 }
 
 static void
-generate_grid_zonal(GridDesciption &grid, const std::string &gridname, double inc, double lon1, double lon2, double lat1,
+generate_grid_zonal(GridDesciption &grid, std::string const &gridname, double inc, double lon1, double lon2, double lat1,
                     double lat2)
 {
   int gridtype = GRID_LONLAT;
@@ -93,7 +93,8 @@ generate_grid_zonal(GridDesciption &grid, const std::string &gridname, double in
 }
 
 static void
-generate_grid_lonlat(GridDesciption &grid, const std::string &gridname, double inc, double lon1, double lon2, double lat1, double lat2)
+generate_grid_lonlat(GridDesciption &grid, std::string const &gridname, double inc, double lon1, double lon2, double lat1,
+                     double lat2)
 {
   int gridtype = GRID_LONLAT;
   bool withBounds = true;
@@ -188,7 +189,7 @@ generate_grid_lonlat(GridDesciption &grid, const std::string &gridname, double i
 
           grid_gen_bounds(nlon, grid.xvals, xbounds);
           grid_gen_bounds(nlat, yvals, ybounds);
-          grid_check_lat_borders(2 * nlat, ybounds.data());
+          grid_check_lat_borders(2 * nlat, ybounds);
 
           grid.xbounds.resize(4 * gridsize);
           grid.ybounds.resize(4 * gridsize);
@@ -199,10 +200,10 @@ generate_grid_lonlat(GridDesciption &grid, const std::string &gridname, double i
 }
 
 static void
-generate_grid_dcw(GridDesciption &grid, const std::string &gridname, double inc)
+generate_grid_dcw(GridDesciption &grid, std::string const &gridname, double inc)
 {
   auto param1 = gridname.c_str();
-  auto param2 = strstr(param1, "_");
+  auto param2 = std::strstr(param1, "_");
   auto param1len = param2 ? param2 - param1 : gridname.size();
 
   if (param2)
@@ -241,7 +242,7 @@ generate_grid_dcw(GridDesciption &grid, const std::string &gridname, double inc)
 }
 
 static void
-generate_grid_gme(GridDesciption &grid, const std::string &gridname)
+generate_grid_gme(GridDesciption &grid, std::string const &gridname)
 {
   if (gridname.size())
     {
@@ -263,7 +264,7 @@ generate_grid_gme(GridDesciption &grid, const std::string &gridname)
 }
 
 static void
-generate_grid_healpix(GridDesciption &grid, const std::string &gridname, bool isZoom = false)
+generate_grid_healpix(GridDesciption &grid, std::string const &gridname, bool isZoom = false)
 {
   if (gridname.size())
     {
@@ -318,7 +319,7 @@ gaussian_latitudes_in_degrees(std::vector<double> &lats, std::vector<double> &la
 }
 
 static void
-generate_grid_gea(GridDesciption &grid, const std::string &gridname)
+generate_grid_gea(GridDesciption &grid, std::string const &gridname)
 {
   char endChar = '?';
   double dx = 0.0;
@@ -389,7 +390,7 @@ generate_grid_gea(GridDesciption &grid, const std::string &gridname)
 }
 
 static void
-generate_grid_zonal(GridDesciption &grid, const std::string &gridname)
+generate_grid_zonal(GridDesciption &grid, std::string const &gridname)
 {
   char endChar = '?';
   int intVal = 0;
@@ -454,7 +455,7 @@ generate_grid_zonal(GridDesciption &grid, const std::string &gridname)
 }
 
 static void
-generate_grid_reg2d(GridDesciption &grid, const std::string &gridname)
+generate_grid_reg2d(GridDesciption &grid, std::string const &gridname)
 {
   char endChar = '?';
   char sepChar = '?';
@@ -472,7 +473,7 @@ generate_grid_reg2d(GridDesciption &grid, const std::string &gridname)
 }
 
 static void
-generate_grid_point(GridDesciption &grid, const std::string &gridname)
+generate_grid_point(GridDesciption &grid, std::string const &gridname)
 {
   char endChar = '?';
   char sepChar = '?';
@@ -492,7 +493,7 @@ generate_grid_point(GridDesciption &grid, const std::string &gridname)
 }
 
 static void
-generate_grid_generic(GridDesciption &grid, const std::string &gridname)
+generate_grid_generic(GridDesciption &grid, std::string const &gridname)
 {
   char endChar = '?';
   char sepChar = '?';
@@ -532,15 +533,15 @@ generate_grid_gaussian(GridDesciption &grid, std::string gridname)
 
       if (gridname.size())
         {
-          int intVal = 0;
+          int intValue = 0;
           std::vector<char> typeString(gridname.size(), 0);
-          auto numVals = sscanf(gridname.c_str(), "%d%s", &intVal, typeString.data());
+          auto numValues = sscanf(gridname.c_str(), "%d%s", &intValue, typeString.data());
           // printf("%s: %d %d%s\n", gridname.c_str(), numVals, intVal, typeString.data());
 
-          if (numVals <= 0 || numVals > 2) return;
-          if (intVal < 0) return;
+          if (numValues <= 0 || numValues > 2) return;
+          if (intValue < 0) return;
 
-          grid.ntr = intVal;
+          grid.ntr = intValue;
           // clang-format off
           if      (cdo_cmpstr(typeString.data(), "grid")) grid.type = GRID_GAUSSIAN;
           else if (cdo_cmpstr(typeString.data(), "zon"))  grid.type = GRID_GAUSSIAN;
@@ -580,7 +581,7 @@ generate_grid_gaussian(GridDesciption &grid, std::string gridname)
 }
 
 static void
-generate_grid_gaussian_full(GridDesciption &grid, const std::string &gridname)
+generate_grid_gaussian_full(GridDesciption &grid, std::string const &gridname)
 {
   if (gridname.size())
     {
@@ -636,7 +637,7 @@ generate_grid_gaussian_full(GridDesciption &grid, const std::string &gridname)
 }
 
 static void
-generate_grid_gaussian_o(GridDesciption &grid, const std::string &gridname)
+generate_grid_gaussian_o(GridDesciption &grid, std::string const &gridname)
 {
   // FXXX - full (regular) Gaussian grid with XXX latitude lines between the pole and equator
   int intVal = 0;
@@ -677,7 +678,7 @@ generate_grid_gaussian_o(GridDesciption &grid, const std::string &gridname)
 }
 
 int
-grid_from_name(const std::string &gridname)
+grid_from_name(std::string const &gridname)
 {
   int gridID = CDI_UNDEFID;
   if (gridname.size() < 2) return gridID;
@@ -694,27 +695,27 @@ grid_from_name(const std::string &gridname)
     {
       generate_grid_reg2d(grid, gridNameLC);
     }
-  else if (string_starts_with(gridNameLC, "lon="))  // lon=<LON>_lat=<LAT>; one gridpoint
+  else if (gridNameLC.starts_with("lon="))  // lon=<LON>_lat=<LAT>; one gridpoint
     {
       generate_grid_point(grid, gridNameLC);
     }
-  else if (string_starts_with(gridNameLC, "gme"))  // gme<NI>
+  else if (gridNameLC.starts_with("gme"))  // gme<NI>
     {
       generate_grid_gme(grid, gridNameLC.substr(3));
     }
-  else if (string_starts_with(gridNameLC, "ni"))  // ni<NI>
+  else if (gridNameLC.starts_with("ni"))  // ni<NI>
     {
       generate_grid_gme(grid, gridNameLC.substr(2));
     }
-  else if (string_starts_with(gridNameLC, "hpz"))  // healpix  hpz<zoom>[_order] (order=[nested|ring|xy])
+  else if (gridNameLC.starts_with("hpz"))  // healpix  hpz<zoom>[_order] (order=[nested|ring|xy])
     {
       generate_grid_healpix(grid, gridNameLC.substr(3), true);
     }
-  else if (string_starts_with(gridNameLC, "hp"))  // healpix  hp<nside>[_order] (order=[nested|ring|xy])
+  else if (gridNameLC.starts_with("hp"))  // healpix  hp<nside>[_order] (order=[nested|ring|xy])
     {
       generate_grid_healpix(grid, gridNameLC.substr(2));
     }
-  else if (string_starts_with(gridNameLC, "gea"))
+  else if (gridNameLC.starts_with("gea"))
     {
       generate_grid_gea(grid, gridNameLC);  // gea<DX>: gaussian reduced equal area; DX in km
     }
@@ -731,27 +732,27 @@ grid_from_name(const std::string &gridname)
     {
       generate_grid_generic(grid, gridNameLC);
     }
-  else if (string_starts_with(gridNameLC, "dcw:"))  // dcw:code_Xdeg
+  else if (gridNameLC.starts_with("dcw:"))  // dcw:code_Xdeg
     {
       generate_grid_dcw(grid, gridNameLC.substr(4), 0.1);
     }
-  else if (string_starts_with(gridNameLC, "germany"))  // germany_Xdeg
+  else if (gridNameLC.starts_with("germany"))  // germany_Xdeg
     {
       generate_grid_lonlat(grid, gridNameLC.substr(7), 0.1, 5.6, 15.2, 47.1, 55.1);
     }
-  else if (string_starts_with(gridNameLC, "europe"))  // europe_Xdeg
+  else if (gridNameLC.starts_with("europe"))  // europe_Xdeg
     {
       generate_grid_lonlat(grid, gridNameLC.substr(6), 1, -30, 60, 30, 80);
     }
-  else if (string_starts_with(gridNameLC, "africa"))  // africa_Xdeg
+  else if (gridNameLC.starts_with("africa"))  // africa_Xdeg
     {
       generate_grid_lonlat(grid, gridNameLC.substr(6), 1, -20, 60, -40, 40);
     }
-  else if (string_starts_with(gridNameLC, "global"))  // global_Xdeg
+  else if (gridNameLC.starts_with("global"))  // global_Xdeg
     {
       generate_grid_lonlat(grid, gridNameLC.substr(6), 1, -180, 180, -90, 90);
     }
-  else if (string_starts_with(gridNameLC, "zonal"))  // zonal_Xdeg
+  else if (gridNameLC.starts_with("zonal"))  // zonal_Xdeg
     {
       generate_grid_zonal(grid, gridNameLC, 1, -180, 180, -90, 90);
     }
@@ -759,7 +760,7 @@ grid_from_name(const std::string &gridname)
     {
       generate_grid_zonal(grid, gridNameLC);
     }
-  else if (string_starts_with(gridNameLC, "ico"))  // icoR02BXX
+  else if (gridNameLC.starts_with("ico"))  // icoR02BXX
     {
       generate_grid_icosphere(grid, gridNameLC);
     }
