@@ -13,11 +13,13 @@
 
 constexpr double undefGridValue = 9.e20;
 
-struct // GridDesciption
+// clang-format off
+struct  // GridDesciption
 #ifdef WARN_UNUSED
 [[gnu::warn_unused]]
 #endif
 GridDesciption
+// clang-format on
 {
   std::vector<int> mask;
   std::vector<double> xvals;
@@ -26,6 +28,8 @@ GridDesciption
   std::vector<double> ybounds;
   std::vector<double> area;
   std::vector<int> reducedPoints;
+  char **xcvals = nullptr;
+  char **ycvals = nullptr;
   double xfirst = undefGridValue, yfirst = undefGridValue;
   double xlast = undefGridValue, ylast = undefGridValue;
   double xinc = undefGridValue, yinc = undefGridValue;
@@ -53,33 +57,36 @@ GridDesciption
   int nd = 0, ni = 0, ni2 = 0, ni3 = 0;
   int number = 0, position = 0;
   unsigned char uuid[CDI_UUID_SIZE] = { 0 };
-  char path[16384] = { 0 };
-  char xname[CDI_MAX_NAME] = { 0 };
-  char xlongname[CDI_MAX_NAME] = { 0 };
-  char xunits[CDI_MAX_NAME] = { 0 };
-  char xdimname[CDI_MAX_NAME] = { 0 };
-  char yname[CDI_MAX_NAME] = { 0 };
-  char ylongname[CDI_MAX_NAME] = { 0 };
-  char yunits[CDI_MAX_NAME] = { 0 };
-  char ydimname[CDI_MAX_NAME] = { 0 };
-  char vdimname[CDI_MAX_NAME] = { 0 };
+  std::string path;
+  std::string xname;
+  std::string xlongname;
+  std::string xunits;
+  std::string xdimname;
+  std::string yname;
+  std::string ylongname;
+  std::string yunits;
+  std::string ydimname;
+  std::string vdimname;
+  std::string projection;
+  std::string healpixOrder;
+  int healpixNside = 0;
 };
 
 int grid_define(GridDesciption &grid);
 
-int gird_from_nc_file(const char *gridfile);
+int grid_from_nc_file(const char *gridfile);
 int grid_from_h5_file(const char *gridfile);
-int grid_from_name(const char *gridname);
+int grid_from_name(std::string const &gridname);
 
-void write_n_cgrid(const char *gridfile, int gridID, int *imask);
+void write_nc_grid(const char *gridfile, int gridID, int *imask);
 
-int cdo_define_grid(const std::string &gridfile);
+int cdo_define_grid(std::string const &gridfile);
 
 int grid_read(FILE *gfp, const char *dname);  // TODO: Find better place for this
 
 int cdo_cdf_openread(const char *filename);
 void cdo_cdf_close(int nc_file_id);
-void cdo_set_grids(const char *gridarg);
+void cdo_set_grids(std::string const &gridarg);
 
 void gaussian_latitudes_in_degrees(std::vector<double> &lats, std::vector<double> &lat_bounds, size_t nlat);
 
