@@ -5,17 +5,16 @@
 
 */
 
-//#define  OUTPUT 1
+// #define  OUTPUT 1
 
 #ifdef _OPENMP
 #include <omp.h>
 #endif
 
 #include "varray.h"
-#include "process_int.h"
 #include "hetaeta.h"
 #include "cdo_options.h"
-#include "cimdOmp.h"
+#include "cdo_omp.h"
 
 static constexpr double apr = 101325.0;  // reference pressure
 static constexpr double aipr = 1.0 / 101325.0;
@@ -32,22 +31,8 @@ static constexpr double eta_pbl = 0.8;  // upper limit of BPL eta coordiante
 static FILE *fpold, *fpnew;
 #endif
 
-#ifdef TEST_HETAETA
-// g++ -DTEST_HETAETA -I../libcdi/src -g -Wall -fopenmp hetaeta.cc
-int Threading::ompNumThreads = 2;
-int
-cdo_omp_get_thread_num()
-{
-#ifdef _OPENMP
-  return omp_get_thread_num();
-#else
-  return 0;
-#endif
-}
-#endif
-
 static long
-int_index(long n, const Varray<double> &x1, const double x2)
+int_index(long n, Varray<double> const &x1, const double x2)
 {
   long k;
   long klo = 0;
@@ -74,24 +59,22 @@ int_index(long n, const Varray<double> &x1, const double x2)
 static double
 esat(double temperature)
 {
-  const auto tc = temperature - 273.16;
-  const auto zes = (tc < 0.0) ? 21.8745584 * tc / (temperature - 7.66) : 17.2693882 * tc / (temperature - 35.86);
-  const auto es = 610.78 * std::exp(zes);
+  auto tc = temperature - 273.16;
+  auto zes = (tc < 0.0) ? 21.8745584 * tc / (temperature - 7.66) : 17.2693882 * tc / (temperature - 35.86);
+  auto es = 610.78 * std::exp(zes);
 
   return es;
 }
 
-#define MAX_VARS 512
-
 // Source from INTERA
 
 template <typename T>
 static void
-hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long nvars, const Varray<double> &af1,
-           const Varray<double> &bf1, const Varray<double> &etah2, const Varray<double> &af2, const Varray<double> &bf2,
-           const Varray<double> &w1, const Varray<double> &w2, const Varray<long> &jl1, const Varray<long> &jl2, const double *ah1,
-           const double *bh1, const Varray<double> &ps1, double epsm1i, const Varray<T> &q1, const Varray<T> &t1,
-           const Varray<double> &fis1, const Varray<double> &fis2, Varray<double> &ps2, const double *ah2, const double *bh2,
+hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long nvars, Varray<double> const &af1,
+           Varray<double> const &bf1, Varray<double> const &etah2, Varray<double> const &af2, Varray<double> const &bf2,
+           Varray<double> const &w1, Varray<double> const &w2, const Varray<long> &jl1, const Varray<long> &jl2, const double *ah1,
+           const double *bh1, Varray<double> const &ps1, double epsm1i, Varray<T> const &q1, Varray<T> const &t1,
+           Varray<double> const &fis1, Varray<double> const &fis2, Varray<double> &ps2, const double *ah2, const double *bh2,
            const Varray2D<T> &vars1, Varray2D<T> &vars2, Varray<T> &t2, Varray<T> &q2, Varray<double> &tscor, Varray<double> &pscor,
            Varray<double> &secor, long jblt, Varray<double> &ph1, Varray<double> &lnph1, Varray<double> &fi1, Varray<double> &pf1,
            Varray<double> &lnpf1, Varray<double> &tv1, Varray<double> &theta1, Varray<double> &rh1, Varray<double> &zvar,
@@ -104,8 +87,8 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
 
   constexpr double rair_d_cpair = rair / cpair;
 
-  const auto nlev1p1 = nlev1 + 1;
-  const auto nlev2p1 = nlev2 + 1;
+  auto nlev1p1 = nlev1 + 1;
+  auto nlev2p1 = nlev2 + 1;
 
   // ****** initialise atmospheric fields in old system
 
@@ -128,9 +111,9 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
   if (ltq)
     for (int k = 0; k < nlev1; ++k)
       {
-        const auto ijk = k * ngp + ij;
-        const auto zq1 = q1[ijk];
-        const auto zt1 = t1[ijk];
+        auto ijk = k * ngp + ij;
+        auto zq1 = q1[ijk];
+        auto zt1 = t1[ijk];
         tv1[k] = (1.0 + epsm1i * zq1) * zt1;
         rh1[k] = zq1 * pf1[k] / (epsilon * esat(zt1));
         theta1[k] = zt1 * std::pow(apr / pf1[k], rair_d_cpair);
@@ -147,7 +130,7 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
   if (ij == OPOINT)
     for (int k = nlev1 - 1; k >= 0; --k)
       {
-        const auto ijk = k * ngp + ij;
+        auto ijk = k * ngp + ij;
         double t = ltq ? t1[ijk] : 0;
         double q = ltq ? q1[ijk] : 0;
         double fi = ltq ? fi1[k] : 0;
@@ -277,8 +260,8 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
   for (int iv = 0; iv < nvars; ++iv)
     for (int k = jjblt; k < nlev2; ++k)
       {
-        const auto ijk1 = jl1[k] * ngp + ij;
-        const auto ijk2 = jl2[k] * ngp + ij;
+        auto ijk1 = jl1[k] * ngp + ij;
+        auto ijk2 = jl2[k] * ngp + ij;
         vars_pbl[iv][k] = w1[k] * vars1[iv][ijk1] + w2[k] * vars1[iv][ijk2];
       }
 
@@ -293,13 +276,13 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
     {
       for (int k = 0; k < nlev1; ++k)
         {
-          const auto ijk = k * ngp + ij;
+          auto ijk = k * ngp + ij;
           zvar[k] = t1[ijk];
         }
 
       for (int k = 0; k <= jjblt; ++k)
         {
-          const auto klo = idx[k];
+          auto klo = idx[k];
           zt2[k] = wgt[k] * zvar[klo] + (1 - wgt[k]) * zvar[klo + 1];
           rh2[k] = wgt[k] * rh1[klo] + (1 - wgt[k]) * rh1[klo + 1];
         }
@@ -309,13 +292,13 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
     {
       for (int k = 0; k < nlev1; ++k)
         {
-          const auto ijk = k * ngp + ij;
+          auto ijk = k * ngp + ij;
           zvar[k] = vars1[iv][ijk];
         }
       for (int k = 0; k <= jjblt; ++k)
         {
-          const auto ijk = k * ngp + ij;
-          const auto klo = idx[k];
+          auto ijk = k * ngp + ij;
+          auto klo = idx[k];
           vars2[iv][ijk] = wgt[k] * zvar[klo] + (1 - wgt[k]) * zvar[klo + 1];
         }
     }
@@ -332,7 +315,7 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
     }
 
   {
-    const auto ijk = jjblt * ngp + ij;
+    auto ijk = jjblt * ngp + ij;
     for (int iv = 0; iv < nvars; ++iv) vars2[iv][ijk] = 0.5 * (vars2[iv][ijk] + vars_pbl[iv][jjblt]);
   }
 
@@ -347,7 +330,7 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
   for (int iv = 0; iv < nvars; ++iv)
     for (int k = jjblt + 1; k < nlev2; ++k)
       {
-        const auto ijk = k * ngp + ij;
+        auto ijk = k * ngp + ij;
         vars2[iv][ijk] = vars_pbl[iv][k];
       }
 
@@ -393,7 +376,7 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
   if (ij == OPOINT)
     for (int k = nlev2 - 1; k >= 0; --k)
       {
-        const auto ijk = k * ngp + ij;
+        auto ijk = k * ngp + ij;
         double t = ltq ? t1[ijk] : 0;
         double q = ltq ? q1[ijk] : 0;
         double fi = ltq ? fi1[k] : 0;
@@ -417,7 +400,7 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
   if (ltq)
     for (int k = 0; k < nlev2; ++k)
       {
-        const auto ijk = k * ngp + ij;
+        auto ijk = k * ngp + ij;
         t2[ijk] = zt2[k];
         q2[ijk] = zq2[k];
       }
@@ -425,12 +408,11 @@ hetaeta_sc(bool ltq, int lpsmod, long ij, long ngp, long nlev1, long nlev2, long
 
 template <typename T>
 void
-hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const double *bh1, const Varray<double> &fis1,
-        const Varray<double> &ps1, const Varray<T> &t1, const Varray<T> &q1, int nlev2, const double *ah2, const double *bh2,
-        const Varray<double> &fis2, Varray<double> &ps2, Varray<T> &t2, Varray<T> &q2, int nvars, const Varray2D<T> &vars1,
+hetaeta(bool ltq, int ngp, Vmask const &imiss, int nlev1, const double *ah1, const double *bh1, Varray<double> const &fis1,
+        Varray<double> const &ps1, Varray<T> const &t1, Varray<T> const &q1, int nlev2, const double *ah2, const double *bh2,
+        Varray<double> const &fis2, Varray<double> &ps2, Varray<T> &t2, Varray<T> &q2, int nvars, const Varray2D<T> &vars1,
         Varray2D<T> &vars2, Varray<double> &tscor, Varray<double> &pscor, Varray<double> &secor)
 {
-  long jblt;
   long jlev = 0;
   int lpsmod = 1;
 #ifdef OUTPUT
@@ -441,7 +423,7 @@ hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const
   long nlev1p1 = nlev1 + 1;
   long nlev2p1 = nlev2 + 1;
 
-  const auto nthreads = Threading::ompNumThreads;
+  auto nthreads = Threading::ompNumMaxThreads;
   Varray2D<double> ph1(nthreads, Varray<double>(nlev1p1));
   Varray2D<double> lnph1(nthreads, Varray<double>(nlev1p1));
   Varray2D<double> fi1(nthreads, Varray<double>(nlev1p1));
@@ -467,12 +449,6 @@ hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const
   Varray2D<double> rh_pbl(nthreads, Varray<double>(ltq ? nlev2 : 0));
   Varray2D<double> theta_pbl(nthreads, Varray<double>(ltq ? nlev2 : 0));
 
-  if (nvars > MAX_VARS)
-    {
-      fprintf(stderr, "Too many vars (max = %d)!\n", MAX_VARS);
-      exit(-1);
-    }
-
   Varray3D<double> vars_pbl(nthreads);
   if (nvars > 0)
     {
@@ -483,8 +459,8 @@ hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const
         }
     }
 
-  Varray<double> af1(nlev1), bf1(nlev1), etaf1(nlev1);
   Varray<double> etah2(nlev2p1);
+  Varray<double> af1(nlev1), bf1(nlev1), etaf1(nlev1);
   Varray<double> af2(nlev2), bf2(nlev2), etaf2(nlev2);
   Varray<double> w1(nlev2), w2(nlev2);
   Varray<long> jl1(nlev2), jl2(nlev2);
@@ -504,7 +480,7 @@ hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const
   // etah1[nlev1] = ah1[nlev1]*aipr+bh1[nlev1];
   for (int k = 0; k < nlev1; ++k)
     {
-      // etah1[k] = ah1[k]*aipr+bh1[k];
+      // etah1[k] = ah1[k] * aipr + bh1[k];
       etaf1[k] = af1[k] * aipr + bf1[k];
     }
 
@@ -518,7 +494,7 @@ hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const
     }
 
   etah2[nlev2] = ah2[nlev2] * aipr + bh2[nlev2];
-  jblt = nlev2;
+  auto jblt = nlev2;
   for (int k = nlev2 - 1; k >= 0; --k)
     {
       etah2[k] = ah2[k] * aipr + bh2[k];
@@ -567,9 +543,9 @@ hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const
 #endif
   for (int ij = 0; ij < ngp; ++ij)
     {
-      if (imiss && imiss[ij]) continue;
+      if ((imiss.size() > 0) && imiss[ij]) continue;
 
-      const auto ompthID = cdo_omp_get_thread_num();
+      auto ompthID = cdo_omp_get_thread_num();
       hetaeta_sc(ltq, lpsmod, ij, ngp, nlev1, nlev2, nvars, af1, bf1, etah2, af2, bf2, w1, w2, jl1, jl2, ah1, bh1, ps1, epsm1i, q1,
                  t1, fis1, fis2, ps2, ah2, bh2, vars1, vars2, t2, q2, tscor, pscor, secor, jblt, ph1[ompthID], lnph1[ompthID],
                  fi1[ompthID], pf1[ompthID], lnpf1[ompthID], tv1[ompthID], theta1[ompthID], rh1[ompthID], zvar[ompthID],
@@ -586,25 +562,39 @@ hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const
 }
 
 // Explicit instantiation
-template void hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const double *bh1,
-                      const Varray<double> &fis1, const Varray<double> &ps1, const Varray<float> &t1, const Varray<float> &q1,
-                      int nlev2, const double *ah2, const double *bh2, const Varray<double> &fis2, Varray<double> &ps2,
+template void hetaeta(bool ltq, int ngp, Vmask const &imiss, int nlev1, const double *ah1, const double *bh1,
+                      Varray<double> const &fis1, Varray<double> const &ps1, Varray<float> const &t1, Varray<float> const &q1,
+                      int nlev2, const double *ah2, const double *bh2, Varray<double> const &fis2, Varray<double> &ps2,
                       Varray<float> &t2, Varray<float> &q2, int nvars, const Varray2D<float> &vars1, Varray2D<float> &vars2,
                       Varray<double> &scor, Varray<double> &pscor, Varray<double> &secor);
-template void hetaeta(bool ltq, int ngp, const int *imiss, int nlev1, const double *ah1, const double *bh1,
-                      const Varray<double> &fis1, const Varray<double> &ps1, const Varray<double> &t1, const Varray<double> &q1,
-                      int nlev2, const double *ah2, const double *bh2, const Varray<double> &fis2, Varray<double> &ps2,
+template void hetaeta(bool ltq, int ngp, Vmask const &imiss, int nlev1, const double *ah1, const double *bh1,
+                      Varray<double> const &fis1, Varray<double> const &ps1, Varray<double> const &t1, Varray<double> const &q1,
+                      int nlev2, const double *ah2, const double *bh2, Varray<double> const &fis2, Varray<double> &ps2,
                       Varray<double> &t2, Varray<double> &q2, int nvars, const Varray2D<double> &vars1, Varray2D<double> &vars2,
                       Varray<double> &scor, Varray<double> &pscor, Varray<double> &secor);
 
 #ifdef TEST_HETAETA
-#define NGP (512)
+
+// g++ -DTEST_HETAETA -I../libcdi/src -std=c++20 -g -Wall -fopenmp hetaeta.cc
+int Threading::ompNumMaxThreads = 2;
+
+int
+cdo_omp_get_thread_num()
+{
+#ifdef _OPENMP
+  return omp_get_thread_num();
+#else
+  return 0;
+#endif
+}
 
 int
 main(int argc, char *argv[])
 {
+  constexpr int NGP = 512;
+
 #ifdef _OPENMP
-  omp_set_num_threads(Threading::ompNumThreads);
+  omp_set_num_threads(Threading::ompNumMaxThreads);
 #endif
 
   printf("NGP = %d\n", NGP);
@@ -698,7 +688,8 @@ main(int argc, char *argv[])
 
   for (int ij = 0; ij < NGP; ++ij) fis2[ij] = fis1[ij];
 
-  hetaeta(ltq, NGP, NULL, 19, a1, b1, fis1, ps1, t1, q1, 40, a2, b2, fis2, ps2, t2, q2, 5, vars1, vars2, tscor, pscor, secor);
+  Vmask imiss;
+  hetaeta(ltq, NGP, imiss, 19, a1, b1, fis1, ps1, t1, q1, 40, a2, b2, fis2, ps2, t2, q2, 5, vars1, vars2, tscor, pscor, secor);
 
   double ot2[40] = { 224.226, 212.75,  209.616, 208.895, 210.468, 213.454, 218.091, 220.49,  220.977, 221.114,
                      220.731, 220.803, 223.595, 226.547, 230.566, 234.99,  239.411, 243.722, 248.125, 252.164,
