@@ -7,19 +7,15 @@
 
 #include <cdi.h>
 
+#include <unistd.h>
+
 #include "process_int.h"
-#include "param_conversion.h"
-#include "util_files.h"
-#include "cdo_options.h"
 #include "varray.h"
 
-#include <cassert>
-#include <unistd.h>
 #include <mpim_grid.h>
 
 #include "pmlist.h"
 #include "merge_axis.h"
-#include "listbuffer.h"
 
 /** All Vars in the input file will be merged.
 *** Therefore, they need to have the same structure i.e. axis sizes
@@ -207,7 +203,7 @@ MergeVarsOnAxis::define_new_axes(std::vector<int> axissize)
 **/
 
 void
-MergeVarsOnAxis::define_var_structure(int vlistID, int ntsteps, std::vector<int> axissize)
+MergeVarsOnAxis::define_var_structure(int vlistID, int ntsteps, const std::vector<int> &axissize)
 {
   this->output.vlistID = vlistID;
   this->output.varID = vlistDefVar(this->output.vlistID, this->output.gridID, this->output.zaxisID, TIME_VARYING);
@@ -230,7 +226,7 @@ MergeVarsOnAxis::define_var_structure(int vlistID, int ntsteps, std::vector<int>
 **/
 
 void
-MergeVarsOnAxis::read_cmor_charvar(std::vector<int> axissize, int streamID, int oldgridsize)
+MergeVarsOnAxis::read_cmor_charvar(const std::vector<int> &axissize, int streamID, int oldgridsize)
 {
   Varray<double> buffer_old(oldgridsize);
 
@@ -239,18 +235,18 @@ MergeVarsOnAxis::read_cmor_charvar(std::vector<int> axissize, int streamID, int
   int tsID = 0;
   while (true)
     {
-      auto nrecs = streamInqTimestep(streamID, tsID);
-      if (nrecs == 0) break;
+      auto numFields = streamInqTimestep(streamID, tsID);
+      if (numFields == 0) break;
 
-      while (nrecs--)
+      while (numFields--)
         {
           int varIDrw, levelIDrw;
-          size_t nmiss;
-          streamInqRecord(streamID, &varIDrw, &levelIDrw);
+          size_t numMissVals;
+          streamInqField(streamID, &varIDrw, &levelIDrw);
           for (int i = 0; i < this->inputNames.nvalues; ++i)
             if (varIDrw == this->inputKeys[i].varID)
               {
-                streamReadRecord(streamID, buffer_old.data(), &nmiss);
+                streamReadField(streamID, buffer_old.data(), &numMissVals);
                 int newIndex;
                 for (int j = 0; j < oldgridsize; ++j)
                   {
