@@ -12,7 +12,7 @@
 typedef std::function<bool(const CdoModule &mod)> ModuleQuery;
 
 bool
-ModListOptions::requested(const std::string &name)
+ModListOptions::requested(std::string const &name)
 {
   return opt[name];
 }
@@ -25,10 +25,10 @@ ModListOptions::mod_info_requested()
 }
 
 bool
-ModListOptions::parse_request(const std::string &requestString)
+ModListOptions::parse_request(std::string const &requestString)
 {
   auto all = true;
-  const auto splitString = split_with_seperator(requestString, ',');
+  const auto splitString = split_string(requestString, ",");
 
   if (requestString.size() > 0)
     {
@@ -58,21 +58,108 @@ ModListOptions::parse_request(const std::string &requestString)
   return true;
 }
 
+std::pair<std::map<std::string, std::vector<std::string>>, std::map<std::string, std::pair<std::string, std::vector<std::string>>>>
+create_help_sections(const CdoHelp &p_help)
+{
+  std::map<std::string, std::vector<std::string>> sections = {};
+  std::map<std::string, std::vector<std::string>> operators = {};
+  std::map<std::string, std::string> oper_synopsis = {};
+
+  std::string key;
+  std::string operator_name;
+  bool operator_section_active = true;
+  bool synopsis_active = true;
+
+  for (auto &line : p_help)
+    {
+      bool is_all_caps = std::all_of(begin(line), end(line), [](char c) { return isupper(c); });
+      if (line.compare("OPERATORS") == 0)
+        {
+          operator_section_active = true;
+          synopsis_active = false;
+        }
+      else if (line.compare("SYNOPSIS") == 0)
+        {
+          key = line;
+          operator_section_active = false;
+          synopsis_active = true;
+        }
+
+      else if (is_all_caps == true)
+        {
+          key = line;
+          sections[key] = std::vector<std::string>();
+          operator_section_active = false;
+          synopsis_active = false;
+        }
+      else if (operator_section_active == true)
+        {
+          constexpr size_t op_name_padding = 4;
+          if (line.find_first_not_of(' ') == op_name_padding)
+            {
+              int first_space_after_name = line.find_first_of(' ', op_name_padding);
+              int name_length = first_space_after_name - op_name_padding;
+              operator_name = line.substr(op_name_padding, name_length);
+
+              operators[operator_name] = std::vector<std::string>{ std::string(line) };
+            }
+          else { operators[operator_name].push_back(std::string(line)); }
+        }
+      else if (synopsis_active == true)
+        {
+          constexpr size_t op_name_padding = 4;
+          std::size_t general_op_desc_pos = line.find("<operator>");
+          if (general_op_desc_pos != std::string::npos)
+            {
+              std::size_t len_generic = std::string("<operator>").size();
+              auto cleaned_line = std::string(line).erase(0, general_op_desc_pos + len_generic);
+              sections["SYNOPSIS"] = { cleaned_line };
+            }
+          else if (line.find_first_not_of(' ') == op_name_padding)
+            {
+              int first_space_after_name = line.find_first_of("[, ", op_name_padding);
+              int name_length = first_space_after_name - op_name_padding;
+              operator_name = line.substr(op_name_padding, name_length);
+              oper_synopsis[operator_name] = std::string(line);
+            }
+        }
+
+      else { sections[key].push_back(std::string(line)); }
+    }
+
+  std::map<std::string, std::pair<std::string, std::vector<std::string>>> oper_syn_map;
+  for (auto const &op : operators)
+    {
+      std::string syn = "";
+      auto it = oper_synopsis.find(op.first);
+      if (it != oper_synopsis.end())
+        {
+          syn = oper_synopsis[op.first];
+          oper_synopsis.erase(it);
+        }
+      else { syn = "    " + op.first + " " + sections["SYNOPSIS"][0]; }
+      oper_syn_map[op.first] = std::make_pair(syn, op.second);
+    }
+  for (auto &syn : oper_synopsis) { oper_syn_map[syn.first].first = syn.second; }
+
+  return std::make_pair(sections, oper_syn_map);
+}
+
 std::string
-get_operator_description(const std::string &p_current_op_name, const CdoHelp &p_help)
+get_operator_description(std::string const &p_current_op_name, const CdoHelp &p_help)
 {
   std::string description = "";
   if (p_help.empty()) return description;
 
   // search for operator section
 
-  auto it = std::find_if(begin(p_help), end(p_help), [&](const auto &l) { return l.find("OPERATORS") != std::string::npos; });
+  auto it = std::find_if(begin(p_help), end(p_help), [&](auto const &l) { return l.find("OPERATORS") != std::string::npos; });
   // if no operator section is found
   if (it == end(p_help))
     {
       std::string name_section = std::string(p_help[0]);
       it = std::find_if(begin(p_help), end(p_help),
-                        [&](const auto &l) { return l.find("    " + p_current_op_name) != std::string::npos; });
+                        [&](auto const &l) { return l.find("    " + p_current_op_name) != std::string::npos; });
 
       if (it != end(p_help))
         {
@@ -83,7 +170,7 @@ get_operator_description(const std::string &p_current_op_name, const CdoHelp &p_
   else
     {
       it = std::find_if(++it, end(p_help),
-                        [&](const auto &l) { return l.find("    " + p_current_op_name + " ") != std::string::npos; });
+                        [&](auto const &l) { return l.find("    " + p_current_op_name + " ") != std::string::npos; });
       if (it != p_help.end())
         {
           std::string line = std::string(*it);
@@ -102,7 +189,7 @@ get_operator_description(const std::string &p_current_op_name, const CdoHelp &p_
 
 // helper function for setting the spacing in operator_print_list
 static std::string
-get_spacing_for(int p_space, const std::string &str)
+get_spacing_for(int p_space, std::string const &str)
 {
   std::string spacing = "";
   for (int i = str.size(); i <= p_space; ++i) spacing += " ";
@@ -141,7 +228,7 @@ operator_print_list(std::function<bool(const CdoModule &)> selectionCriteria)
       if (selectionCriteria(current_module)) { output_list.push_back(operatorGetShortInfoString(current_op_name, current_module)); }
     }
   // print generated output list
-  for (const std::string &str : output_list) { std::cout << str << std::endl; }
+  for (std::string const &str : output_list) { std::cout << str << std::endl; }
 }
 
 void
@@ -202,7 +289,7 @@ operatorPrintAll(void)
   std::vector<std::string> sorted_operator_names = Factory::get_sorted_operator_name_list();
 
   std::cout << tab;
-  for (const auto &operatorName : sorted_operator_names)
+  for (auto const &operatorName : sorted_operator_names)
     {
       if (number_of_chars > 85)
         {
@@ -245,13 +332,12 @@ operator_print_list(bool print_no_output)
       output_list[out_list_idx] += get_spacing_for(90, output_list[out_list_idx]) + in_out_info;
     }
   // print generated output list
-  for (const std::string &str : output_list) { std::cout << str << std::endl; }
+  for (std::string const &str : output_list) { std::cout << str << std::endl; }
 }
 
 void
-cdo_print_help(const std::string &p_operator_name)
+cdo_print_help(std::string const &p_operator_name)
 {
-
   auto it
       = Factory::find(p_operator_name, [&p_operator_name]() { cdo_abort("%s", Factory::err_msg_oper_not_found(p_operator_name)); });
   const CdoHelp &help = Factory::get_help(it);
@@ -264,30 +350,14 @@ cdo_print_help(const std::string &p_operator_name)
           auto doPrint = !(help[i][0] == '\0' && help[i + 1][0] == ' ');
           if (doPrint)
             {
-
-              //===========================================================================
-              // This is necessary as long as we use fprintf.
-              // Without the string we cant use c_str and as such fprintf throws a seqfault
-              // As (14.11.2023) CdoHelp is a vector<string_view> and string_view does not
-              // provide a .c_str()
-              std::string line = std::string(help[i]);
-              //===========================================================================
-
-              if (color_enabled())
-                {
-                  if (cdo_cmpstr(line, "NAME") || cdo_cmpstr(line, "SYNOPSIS") || cdo_cmpstr(line, "DESCRIPTION")
-                      || cdo_cmpstr(line, "OPERATORS") || cdo_cmpstr(line, "NAMELIST") || cdo_cmpstr(line, "PARAMETER")
-                      || cdo_cmpstr(line, "ENVIRONMENT") || cdo_cmpstr(line, "NOTE") || cdo_cmpstr(line, "EXAMPLES"))
-                    {
-                      set_text_color(stdout, BRIGHT);
-                      fprintf(stdout, "%s", line.c_str());
-                      reset_text_color(stdout);
-                      fprintf(stdout, "\n");
-                    }
-                  else
-                    fprintf(stdout, "%s\n", line.c_str());
-                }
-              else { fprintf(stdout, "%s\n", line.c_str()); }
+              auto line = help[i];
+              auto useBold
+                  = (color_enabled()
+                     && (line == "NAME" || line == "SYNOPSIS" || line == "DESCRIPTION" || line == "OPERATORS" || line == "NAMELIST"
+                         || line == "PARAMETER" || line == "ENVIRONMENT" || line == "NOTE" || line == "EXAMPLES"));
+              if (useBold) set_text_color(stdout, BRIGHT);
+              std::cout << line << "\n";
+              if (useBold) reset_text_color(stdout);
             }
         }
     }
