@@ -25,7 +25,7 @@
 const char *
 parameter_to_word(const char *cstring)
 {
-  auto len = strlen(cstring);
+  auto len = std::strlen(cstring);
 
   for (size_t i = 0; i < len; ++i)
     {
@@ -45,7 +45,7 @@ parameter_error(const char *name, const char *cstring, const char *endptr)
 }
 
 long
-parameter_to_bytes(const std::string &string)
+parameter_to_bytes(std::string const &string)
 {
   char *endptr = nullptr;
   auto numBytes = strtoimax(string.c_str(), &endptr, 10);
@@ -83,7 +83,7 @@ parameter_to_imax(const char *const cstring)
   char *endptr = nullptr;
   auto ival = strtoimax(cstring, &endptr, 10);
   if (*endptr) parameter_error("Integer", cstring, endptr);
-  int len = strlen(cstring);
+  int len = std::strlen(cstring);
   if ((cstring[0] != '-' && len > 19) || len > 20)
     cdo_abort("Integer parameter %s too long (min=%ld max=%ld)!", cstring, LONG_MIN, LONG_MAX);
   return ival;
@@ -125,8 +125,8 @@ parameter_to_intlist(const char *const cstring)
   return ival;
 }
 
-const std::string &
-parameter_to_word(const std::string &string)
+std::string const &
+parameter_to_word(std::string const &string)
 {
   auto len = string.size();
 
@@ -142,7 +142,7 @@ parameter_to_word(const std::string &string)
 }
 
 bool
-parameter_to_bool(const std::string &string)
+parameter_to_bool(std::string const &string)
 {
   auto lstring = string_to_lower(string);
 
@@ -155,37 +155,37 @@ parameter_to_bool(const std::string &string)
 }
 
 double
-parameter_to_double(const std::string &string)
+parameter_to_double(std::string const &string)
 {
   return parameter_to_double(string.c_str());
 }
 
 int
-parameter_to_int(const std::string &string)
+parameter_to_int(std::string const &string)
 {
   return parameter_to_int(string.c_str());
 }
 
 long
-parameter_to_long(const std::string &string)
+parameter_to_long(std::string const &string)
 {
   return parameter_to_long(string.c_str());
 }
 
 size_t
-parameter_to_size_t(const std::string &string)
+parameter_to_size_t(std::string const &string)
 {
   return parameter_to_size_t(string.c_str());
 }
 
 int
-parameter_to_intlist(const std::string &string)
+parameter_to_intlist(std::string const &string)
 {
   return parameter_to_intlist(string.c_str());
 }
 
 double
-radius_str_to_meter(const std::string &string)
+radius_str_to_meter(std::string const &string)
 {
   char *endptr = nullptr;
   auto radius = strtod(string.c_str(), &endptr);
@@ -204,7 +204,7 @@ radius_str_to_meter(const std::string &string)
 }
 
 double
-radius_str_to_deg(const std::string &string)
+radius_str_to_deg(std::string const &string)
 {
   char *endptr = nullptr;
   auto radius = strtod(string.c_str(), &endptr);
@@ -238,7 +238,7 @@ string_to_param(const char *const paramstr)
 }
 
 int
-string_to_param(const std::string &paramstr)
+string_to_param(std::string const &paramstr)
 {
   return string_to_param(paramstr.c_str());
 }
@@ -267,11 +267,11 @@ param_to_string(int param)
 /* time/date/season converisons */
 /* =================================================================================== */
 void
-season_to_months(const std::string &season, int *imonths)
+season_to_months(std::string const &season, int *imonths)
 {
   const char *const smons = "JFMAMJJASONDJFMAMJJASOND";
   const int imons[] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 };
-  assert(strlen(smons) == (sizeof(imons) / sizeof(int)));
+  assert(std::strlen(smons) == (sizeof(imons) / sizeof(int)));
 
   if (season == "ANN")
     {
@@ -283,8 +283,8 @@ season_to_months(const std::string &season, int *imonths)
       if (len > 12) cdo_abort("Too many months %d (limit=12)!", (int) len);
       char *const season_u = strdup(season.c_str());
       cstr_to_upper(season_u);
-      const char *const sstr = strstr(smons, season_u);
-      free(season_u);
+      const char *const sstr = std::strstr(smons, season_u);
+      std::free(season_u);
       if (sstr != nullptr)
         {
           size_t ks = (size_t) (sstr - smons);
@@ -296,20 +296,20 @@ season_to_months(const std::string &season, int *imonths)
 }
 
 double
-date_str_to_double(const char *datestr, int opt)
+datestr_to_double(std::string const &datestr, int opt)
 {
-  int year = 1, month = 1, day = 1, hour = 0, minute = 0, second = 0;
-  double fval = 0.0;
-
-  auto len = strlen(datestr);
-
-  for (size_t i = 0; i < len; ++i)
+  for (size_t i = 0, n = datestr.size(); i < n; ++i)
     {
       int c = datestr[i];
       if (!(std::isdigit(c) || c == '-' || c == ':' || c == '.' || c == 'T'))
         cdo_abort("Date string >%s< contains invalid character at position %zu!", datestr, i + 1);
     }
 
+  if (not string_contains(datestr, '-')) { return parameter_to_double(datestr); }
+  if (datestr[0] == '-' && not string_contains(datestr.c_str() + 1, '-')) { return parameter_to_double(datestr); }
+
+  int year = 1;
+  unsigned month = 1, day = 1, hour = 0, minute = 0, second = 0;
   if (opt)
     {
       hour = 23;
@@ -317,24 +317,25 @@ date_str_to_double(const char *datestr, int opt)
       second = 59;
     }
 
-  if (strchr(datestr, '-') == nullptr) { fval = parameter_to_double(datestr); }
-  else if (strchr(datestr, 'T'))
+  if (string_contains(datestr, 'T'))
     {
-      auto status = std::sscanf(datestr, "%d-%d-%dT%d:%d:%d", &year, &month, &day, &hour, &minute, &second);
+      auto status = std::sscanf(datestr.c_str(), "%d-%u-%uT%u:%u:%u", &year, &month, &day, &hour, &minute, &second);
       if (status != 6) cdo_abort("Invalid date string >%s<!", datestr);
-      fval = cdiEncodeTime(hour, minute, second);
-      if (std::fabs(fval) > 0.0) fval /= 1000000;
-      fval += cdiEncodeDate(year, month, day);
     }
   else
     {
-      auto status = std::sscanf(datestr, "%d-%d-%d", &year, &month, &day);
+      auto status = std::sscanf(datestr.c_str(), "%d-%u-%u", &year, &month, &day);
       if (status != 3) cdo_abort("Invalid date string >%s<!", datestr);
-      fval = cdiEncodeTime(hour, minute, second);
-      if (std::fabs(fval) > 0.0) fval /= 1000000;
-      fval += cdiEncodeDate(year, month, day);
     }
 
+  if (month > 12 || day > 31) cdo_abort("Date components out of bounds >%s<!", datestr);
+  if (hour > 24 || minute > 60 || second > 60) cdo_abort("Time components out of bounds >%s<!", datestr);
+
+  double fval = cdiEncodeTime(hour, minute, second);
+  if (std::fabs(fval) > 0.0) fval /= 1000000;
+  fval += std::abs(cdiEncodeDate(year, month, day));
+  if (year < 0) fval *= -1;
+
   return fval;
 }
 
@@ -344,7 +345,7 @@ cdo_argv_to_intarr(const std::vector<std::string> &argv)
 {
   std::vector<int> v;
 
-  for (const auto &argument : argv)
+  for (auto const &argument : argv)
     {
       int first, last, inc;
       split_intstring(argument, first, last, inc);
@@ -367,7 +368,7 @@ cdo_argv_to_fltarr(const std::vector<std::string> &argv)
 {
   std::vector<double> v;
 
-  for (const auto &argument : argv)
+  for (auto const &argument : argv)
     {
       auto len = (int) argument.size();
       int i;
@@ -426,7 +427,7 @@ split_intstring(const char *const intstr, int &first, int &last, int &inc)
 }
 
 void
-split_intstring(const std::string &intstr, int &first, int &last, int &inc)
+split_intstring(std::string const &intstr, int &first, int &last, int &inc)
 {
   split_intstring(intstr.c_str(), first, last, inc);
 }
@@ -463,56 +464,56 @@ split_fltstring(const char *const intstr, double &first, double &last, double &i
 }
 
 void
-split_fltstring(const std::string &fltstr, double &first, double &last, double &inc)
+split_fltstring(std::string const &fltstr, double &first, double &last, double &inc)
 {
   split_fltstring(fltstr.c_str(), first, last, inc);
 }
 
 template <>
 int
-convert(const std::string &str_value)
+convert(std::string const &str_value)
 {
   return parameter_to_int(str_value.c_str());
 }
 
 template <>
 double
-convert(const std::string &str_value)
+convert(std::string const &str_value)
 {
   return parameter_to_double(str_value.c_str());
 }
 
 template <>
 bool
-convert(const std::string &str_value)
+convert(std::string const &str_value)
 {
   return parameter_to_bool(str_value);
 }
 
 template <>
 long
-convert(const std::string &str_value)
+convert(std::string const &str_value)
 {
   return parameter_to_long(str_value.c_str());
 }
 
 template <>
 size_t
-convert(const std::string &str_value)
+convert(std::string const &str_value)
 {
   return parameter_to_size_t(str_value.c_str());
 }
 
 template <>
 std::string
-convert(const std::string &str_value)
+convert(std::string const &str_value)
 {
   return parameter_to_word(str_value.c_str());
 }
 
 template <>
 FillMethod
-convert(const std::string &str_value)
+convert(std::string const &str_value)
 {
   auto fillMethod = string_to_fillmethod(parameter_to_word(str_value));
 
