@@ -14,13 +14,13 @@
 #include "parse_literals.h"
 
 int
-literal_get_datatype(const std::string &literal)
+literal_get_datatype(std::string const &literal)
 {
   if (!literal.empty())
     {
       char *endptr;
       errno = 0;
-      const long lval = strtol(literal.c_str(), &endptr, 10);
+      long lval = strtol(literal.c_str(), &endptr, 10);
       if (errno == 0 && *endptr == 0)
         return CDI_DATATYPE_INT32;
       else if (errno == 0 && *(endptr + 1) == 0 && (*endptr == 's' || *endptr == 'b'))
@@ -33,7 +33,7 @@ literal_get_datatype(const std::string &literal)
       else
         {
           errno = 0;
-          const double dval = strtod(literal.c_str(), &endptr);
+          double dval = strtod(literal.c_str(), &endptr);
           if (errno == 0 && *endptr == 0)
             return CDI_DATATYPE_FLT64;
           else if (errno == 0 && *(endptr + 1) == 0)
@@ -47,7 +47,7 @@ literal_get_datatype(const std::string &literal)
 }
 
 int
-literals_find_datatype(const int n, const std::vector<std::string> &literals)
+literals_find_datatype(int n, const std::vector<std::string> &literals)
 {
   int dtype = -1;
 
@@ -57,27 +57,14 @@ literals_find_datatype(const int n, const std::vector<std::string> &literals)
       if (dtype != -1)
         for (int i = 1; i < n; ++i)
           {
-            const int xtype = literal_get_datatype(literals[i]);
+            auto xtype = literal_get_datatype(literals[i]);
             if (dtype != xtype)
               {
                 if (xtype == CDI_DATATYPE_FLT32 || xtype == CDI_DATATYPE_FLT64)
                   {
-                    if (dtype == CDI_DATATYPE_FLT32 || dtype == CDI_DATATYPE_FLT64)
-                      {
-                        if (xtype > dtype) dtype = xtype;
-                      }
-                    else
-                      dtype = xtype;
-                  }
-                else
-                  {
-                    if (!(dtype == CDI_DATATYPE_FLT32 || dtype == CDI_DATATYPE_FLT64))
-                      {
-                        if (xtype > dtype) dtype = xtype;
-                      }
-                    else
-                      dtype = xtype;
+                    dtype = (dtype == CDI_DATATYPE_FLT32 || dtype == CDI_DATATYPE_FLT64) ? std::max(dtype, xtype) : xtype;
                   }
+                else { dtype = (!(dtype == CDI_DATATYPE_FLT32 || dtype == CDI_DATATYPE_FLT64)) ? std::max(dtype, xtype) : xtype; }
               }
           }
     }
@@ -86,7 +73,7 @@ literals_find_datatype(const int n, const std::vector<std::string> &literals)
 }
 
 int
-literal_to_int(const std::string &literal)
+literal_to_int(std::string const &literal)
 {
   int ival = INT_MAX;
 
@@ -100,7 +87,7 @@ literal_to_int(const std::string &literal)
 }
 
 double
-literal_to_double(const std::string &literal)
+literal_to_double(std::string const &literal)
 {
   double dval = DBL_MAX;
 
@@ -119,26 +106,21 @@ main(void)
 {
   std::vector<std::string> literals
       = { "127b", "-32768s", "-2147483647", "-1.e+36f", "1.e+308", "temperature", "surface pressure", "1000." };
-  const int nliterals = literals.size();
+  int nliterals = literals.size();
 
   for (int i = 0; i < nliterals; ++i)
     {
-      const int dtype = literal_get_datatype(literals[i]);
+      int dtype = literal_get_datatype(literals[i]);
       printf("%d %s type = %d", i + 1, literals[i].c_str(), dtype);
       if (dtype == CDI_DATATYPE_INT8 || dtype == CDI_DATATYPE_INT16 || dtype == CDI_DATATYPE_INT32)
         {
-          const auto ival = literal_to_int(literals[i]);
-          printf("  ival = %d", ival);
+          printf("  ival = %d", literal_to_int(literals[i]));
         }
       else if (dtype == CDI_DATATYPE_FLT32 || dtype == CDI_DATATYPE_FLT64)
         {
-          const auto dval = literal_to_double(literals[i]);
-          printf("  dval = %g", dval);
-        }
-      else
-        {
-          printf("  sval = '%s'", literals[i].c_str());
+          printf("  dval = %g", literal_to_double(literals[i]));
         }
+      else { printf("  sval = '%s'", literals[i].c_str()); }
 
       printf("\n");
     }
