@@ -28,7 +28,7 @@ namespace Util
 {
 
 void
-extract_name_and_argument(const std::string &command, std::string &operatorName, std::string &operatorArgument)
+extract_name_and_argument(std::string const &command, std::string &operatorName, std::string &operatorArgument)
 {
   constexpr char delimiter = ',';
 
@@ -52,7 +52,7 @@ tab()
 }
 
 std::vector<std::string>
-generate_tokens(const std::string &p_oper)
+generate_tokens(std::string const &p_oper)
 {
   std::vector<std::string> result = {};
 
@@ -96,7 +96,6 @@ append(std::shared_ptr<T> &parent, std::shared_ptr<T> &child)
 
 // Factory Funcs!
 
-
 // Factory Funcs!
 static std::shared_ptr<Node>
 create_operator_node(ARGV_ITERATOR &p_curentArgument, bool is_first = false)
@@ -109,9 +108,7 @@ create_operator_node(ARGV_ITERATOR &p_curentArgument, bool is_first = false)
   Util::extract_name_and_argument(*p_curentArgument, operatorName, operatorArguments);
 
   auto moduleIterator = Factory::find(
-      operatorName, [&]() {
-      THROW(InternalCdoSyntaxError, p_curentArgument, Factory::err_msg_oper_not_found(operatorName));
-      });
+      operatorName, [&]() { THROW(InternalCdoSyntaxError, p_curentArgument, Factory::err_msg_oper_not_found(operatorName)); });
 
   auto mod = Factory::get_module(moduleIterator);
   auto newNode = std::make_shared<Node>(p_curentArgument, operatorName, operatorArguments, mod.constraints);
@@ -127,7 +124,7 @@ static std::shared_ptr<Node>
 create_file_node(ARGV_ITERATOR &p_curentArgument)
 {
   debug_parser("Creating new file node, %s", *p_curentArgument);
-  auto newNode = std::make_shared<Node>(p_curentArgument, false);
+  auto newNode = std::make_shared<Node>(p_curentArgument, Node::NodeType::INFILE);
   return newNode;
 }
 
@@ -187,7 +184,7 @@ public:
   void
   pop()
   {
-    const auto &node = stack.back();
+    auto const &node = stack.back();
     debug_parser("poping node: %s", node->oper);
     if (node->constraints.streamInCnt == -1) { cntVariableInputs--; }
     debug_parser("poping node: %s", node->oper);
@@ -374,16 +371,6 @@ handle_sub_group(ARGV_ITERATOR &p_cur_arg, PARSER_STACK &stack)
   stack.push(Subgroup(p_cur_arg));
   staticParserCounter = stack.size();
 }
-bool
-has_required_inputs(std::shared_ptr<Node> &node)
-{
-  debug_parser("Checking required inputs");
-  bool variable_done = (node->constraints.streamInCnt == -1 && node->children.size() >= 1);
-  bool node_done = node->is_done() || variable_done;
-
-  bool has_required_inputs = !node->isFile && !node_done;
-  return has_required_inputs;
-}
 
 static void
 handle_sub_group_end(ARGV_ITERATOR &p_cur_arg, PARSER_STACK &p_parser_stack)
@@ -410,7 +397,7 @@ handle_sub_group_end(ARGV_ITERATOR &p_cur_arg, PARSER_STACK &p_parser_stack)
     {
       // passing on all root to roots of next parser in case of variable input operators, apply and too many brackets
       // This also allows to ignore double brackets e.g -merge [ [ -topo -topo ] ]
-      for (const auto &node : finished_group.get_roots()) { cur_parser.add_root(node); }
+      for (auto const &node : finished_group.get_roots()) { cur_parser.add_root(node); }
     }
   else
     {
@@ -419,7 +406,7 @@ handle_sub_group_end(ARGV_ITERATOR &p_cur_arg, PARSER_STACK &p_parser_stack)
       if (cur_parser.top()->children.size() != 0) { THROW(InternalCdoSyntaxError, cur_parser.top()->iter++, errmsg_mixed_input); }
       for (auto &node : finished_group.get_roots())
         {
-          if (has_required_inputs(node)) { THROW(InternalCdoSyntaxError, node->iter, errmsg_missing_inputs); }
+          if (node->has_required_inputs()) { THROW(InternalCdoSyntaxError, node->iter, errmsg_missing_inputs); }
           else { Util::append(cur_parser.top(), node); }
         }
 
@@ -454,7 +441,7 @@ run(std::vector<std::string> &p_argv)
   ARGV_ITERATOR end = p_argv.end();
 
   std::string first_argv = *cur_arg;
-  if (first_argv.find("-apply,") == 0) { THROW(InternalCdoSyntaxError, cur_arg, errmsg_apply_in_first_pos); }
+  if (first_argv.starts_with("-apply,")) { THROW(InternalCdoSyntaxError, cur_arg, errmsg_apply_in_first_pos); }
 
   first_operator = create_first_operator_node(cur_arg);
 
@@ -498,7 +485,7 @@ run(std::vector<std::string> &p_argv)
             {
               bool isOutFile = true;
               debug_parser("creating new out file: %s", *cur_arg);
-              auto outFileNode = std::make_shared<Node>(cur_arg, isOutFile);
+              auto outFileNode = std::make_shared<Node>(cur_arg, Node::NodeType::OUTFILE);
               node_structure.push_back(outFileNode);
               Util::append(outFileNode, first_operator);
               cur_arg++;
