@@ -17,7 +17,7 @@
 #include "cdo_output.h"
 #include "cdo_options.h"
 
-enum class PercentileMethod
+enum struct PercentileMethod
 {
   NRANK = 1,
   NIST,
@@ -25,7 +25,7 @@ enum class PercentileMethod
   NR8
 };
 
-enum class NumpyMethod
+enum struct NumpyMethod
 {
   // NumPy
   linear = 1,
@@ -61,7 +61,7 @@ static double
 percentile_nrank(T *array, size_t n, double quantile)
 {
   auto irank = (size_t) std::ceil(n * quantile);
-  irank = std::min(std::max(irank, static_cast<size_t>(1)), n);
+  irank = std::clamp(irank, static_cast<size_t>(1), n);
   return get_nth_element(array, n, irank - 1);
 }
 
@@ -114,17 +114,17 @@ percentile_numpy(T *array, size_t n, double quantile)
       else if (numpyMethod == NumpyMethod::lower)
         {
           size_t lo = std::floor(rank);
-          percentil = get_nth_element(array, n, std::min(std::max(lo, static_cast<size_t>(1)), n) - 1);
+          percentil = get_nth_element(array, n, std::clamp(lo, static_cast<size_t>(1), n) - 1);
         }
       else if (numpyMethod == NumpyMethod::higher)
         {
           size_t hi = std::ceil(rank);
-          percentil = get_nth_element(array, n, std::min(std::max(hi, static_cast<size_t>(1)), n) - 1);
+          percentil = get_nth_element(array, n, std::clamp(hi, static_cast<size_t>(1), n) - 1);
         }
       else if (numpyMethod == NumpyMethod::nearest)  // numpy is using around(), with rounds to the nearest even value
         {
           size_t j = std::lround(rank);
-          percentil = get_nth_element(array, n, std::min(std::max(j, static_cast<size_t>(1)), n) - 1);
+          percentil = get_nth_element(array, n, std::clamp(j, static_cast<size_t>(1), n) - 1);
         }
       else if (numpyMethod == NumpyMethod::midpoint)
         {
@@ -152,7 +152,7 @@ percentile_numpy(T *array, size_t n, double quantile)
               constexpr double fuzz = 4.0 * std::numeric_limits<double>::epsilon();
               size_t j = std::floor(nppn + fuzz);  // m = a + probs*(1 - a - b)
               double h = nppn - j;
-              if (fabs(h) < fuzz) h = 0.0;
+              if (std::fabs(h) < fuzz) h = 0.0;
               if (h > 0.0 && h < 1.0)
                 percentil = (1.0 - h) * get_nth_element(array, n, j - 1) + h * get_nth_element(array, n, j);
               else
@@ -165,7 +165,7 @@ percentile_numpy(T *array, size_t n, double quantile)
               size_t j = std::floor(nppm);
               double h = (numpyMethod == NumpyMethod::inverted_cdf)            ? (nppm > j)
                          : (numpyMethod == NumpyMethod::averaged_inverted_cdf) ? ((nppm > j) + 1.0) / 2.0
-                                                                               : ((fabs(nppm - j) > 0.0) | ((j % 2) == 1));
+                                                                               : ((std::fabs(nppm - j) > 0.0) | ((j % 2) == 1));
               if (h > 0.0 && h < 1.0)
                 percentil = (1.0 - h) * get_nth_element(array, n, j - 1) + h * get_nth_element(array, n, j);
               else
@@ -274,7 +274,7 @@ set_numpy_method(NumpyMethod npMethod)
 }
 
 void
-percentile_set_method(const std::string &methodStr)
+percentile_set_method(std::string const &methodStr)
 {
   auto methodName = string_to_lower(methodStr);
 
@@ -314,7 +314,7 @@ EOF
 #
 cat > per_cdo.sh << EOR
 #/bin/sh
-CDO=/Users/m214003/cdt/work/cdo/build/clang/src/cdo
+CDO=cdo
 PERS="30 40 50 75 100"
 METS="nrank nist rtype8 numpy numpy_lower numpy_higher numpy_nearest"
 METS="linear lower higher nearest midpoint inverted_cdf averaged_inverted_cdf closest_observation interpolated_inverted_cdf hazen
