@@ -26,7 +26,7 @@ keyValuesPrint(FILE *fp, const KeyValues &keyValues)
 void
 KVList::print(FILE *fp) const
 {
-  for (const auto &keyValues : *this) keyValuesPrint(fp, keyValues);
+  for (auto const &keyValues : *this) keyValuesPrint(fp, keyValues);
 }
 
 int
@@ -79,9 +79,9 @@ KVList::parse_arguments(const std::vector<std::string> &argv)
 }
 
 const KeyValues *
-KVList::search(const std::string &key) const
+KVList::search(std::string const &key) const
 {
-  for (const auto &kv : *this)
+  for (auto const &kv : *this)
     {
       if (kv.key == key) return &kv;
     }
@@ -90,7 +90,7 @@ KVList::search(const std::string &key) const
 }
 
 const std::string
-KVList::get_first_value(const std::string key, const std::string &replacer)
+KVList::get_first_value(std::string const &key, std::string const &replacer)
 {
   auto kv = this->search(key);
   if (kv && kv->nvalues > 0) return kv->values[0];
@@ -98,7 +98,7 @@ KVList::get_first_value(const std::string key, const std::string &replacer)
 }
 
 void
-KVList::append(const std::string &key, const std::vector<std::string> &values, int nvalues)
+KVList::append(std::string const &key, const std::vector<std::string> &values, int nvalues)
 {
   KeyValues kv;
   kv.key = key;
@@ -110,7 +110,7 @@ KVList::append(const std::string &key, const std::vector<std::string> &values, i
 
 // Remove only one list item
 void
-KVList::remove(const std::string &inkey)
+KVList::remove(std::string const &inkey)
 {
   std::list<KeyValues>::iterator i;
   for (i = this->begin(); i != this->end(); ++i)
@@ -119,11 +119,11 @@ KVList::remove(const std::string &inkey)
 }
 
 const KVList *
-PMList::searchKVListVentry(const std::string &key, const std::string &value, const std::vector<std::string> &entry)
+PMList::searchKVListVentry(std::string const &key, std::string const &value, const std::vector<std::string> &entry)
 {
-  for (const auto &kvlist : *this)
+  for (auto const &kvlist : *this)
     {
-      for (const auto &s : entry)
+      for (auto const &s : entry)
         if (kvlist.name == s)
           {
             auto kv = kvlist.search(key);
@@ -137,9 +137,9 @@ PMList::searchKVListVentry(const std::string &key, const std::string &value, con
 const KVList *
 PMList::getKVListVentry(const std::vector<std::string> &entry)
 {
-  for (const auto &kvlist : *this)
+  for (auto const &kvlist : *this)
     {
-      for (const auto &s : entry)
+      for (auto const &s : entry)
         if (kvlist.name == s) return &kvlist;
     }
 
@@ -203,7 +203,7 @@ get_number_of_values(const unsigned long ntok, const NamelistToken *tokens)
 static void
 replace_name(char *name)
 {
-  for (size_t pos = 0; pos < strlen(name); pos++) name[pos] = std::tolower(name[pos]);
+  for (size_t pos = 0; pos < std::strlen(name); pos++) name[pos] = std::tolower(name[pos]);
 
   if (cdo_cmpstr(name, "conventions")) std::strcpy(name, "Conventions");
   if (cdo_cmpstr(name, "cn")) std::strcpy(name, "cmor_name");
@@ -226,7 +226,7 @@ parse_namelist(PMList &pmlist, NamelistParser &parser, char *buf, bool cdocmor)
 
   for (unsigned long it = 0; it < ntok; ++it)
     {
-      const auto &t = tokens[it];
+      auto const &t = tokens[it];
       // printf("Token %u", it+1);
       if (t.type == NamelistType::OBJECT)
         {
@@ -234,7 +234,7 @@ parse_namelist(PMList &pmlist, NamelistParser &parser, char *buf, bool cdocmor)
           if (it + 1 < ntok && tokens[it + 1].type == NamelistType::WORD)
             {
               it++;
-              const auto &t2 = tokens[it];
+              auto const &t2 = tokens[it];
               std::snprintf(name, sizeof(name), "%.*s", (int) (t2.end - t2.start), buf + t2.start);
               name[sizeof(name) - 1] = 0;
             }
@@ -330,7 +330,7 @@ PMList::read_namelist(FILE *fp, const char *name)
 void
 PMList::print(FILE *fp)
 {
-  for (const auto &kvlist : *this)
+  for (auto const &kvlist : *this)
     {
       fprintf(fp, "\nFound %s list with %zu key/values: \n", kvlist.name.c_str(), kvlist.size());
       kvlist.print(fp);
