@@ -35,10 +35,10 @@ public:
 
   void print(FILE *fp = stderr) const;
   int parse_arguments(const std::vector<std::string> &argv);
-  const KeyValues *search(const std::string &key) const;
-  void remove(const std::string &inkey);
-  void append(const std::string &key, const std::vector<std::string> &values, int nvalues);
-  const std::string get_first_value(const std::string key, const std::string &replacer);
+  const KeyValues *search(std::string const &key) const;
+  void remove(std::string const &inkey);
+  void append(std::string const &key, const std::vector<std::string> &values, int nvalues);
+  const std::string get_first_value(std::string const &key, std::string const &replacer);
 };
 
 // clang-format off
@@ -50,7 +50,7 @@ PMList : public std::list<KVList>
 // clang-format on
 {
 public:
-  const KVList *searchKVListVentry(const std::string &key, const std::string &value, const std::vector<std::string> &entry);
+  const KVList *searchKVListVentry(std::string const &key, std::string const &value, const std::vector<std::string> &entry);
   const KVList *getKVListVentry(const std::vector<std::string> &entry);
   void print(FILE *fp = stderr);
   void read_namelist(FILE *fp, const char *name);
@@ -58,7 +58,7 @@ public:
 };
 
 void mapvar(int vlistID, int varID, const KeyValues &kv, CmorVar &cmorVar, bool &hasValidMin, bool &hasValidMax, int ptab,
-       bool isnPtmodeName);
+            bool isnPtmodeName);
 int parse_namelist(PMList &pmlist, NamelistParser &parser, char *buf, bool cdocmor);
 int parse_list_buffer(NamelistParser &p, ListBuffer &listBuffer);
 
